// CCheckDataDlg.cpp - Checks POI Data Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR CCheckDataDlg::m_szSection[] = _T("Check POI");

////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts a new item

int CCheckDataDlg::CLvCtrl::InsertItem(PLVITEM pLVI)
{
	TRACE(_T("Entering CCheckDataDlg::CLvCtrl::InsertItem\n"));

	if(e_dwWinVer > WV_XP)
	{
		pLVI->mask  |= LVIF_IMAGE;
		pLVI->iImage = 0;
	}
	return CListView::InsertItem(pLVI);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets or clears the window redraw flag

AFXINLINE void CCheckDataDlg::CLvCtrl::SetWindowRedraw(BOOL bRedraw /* = TRUE */)
{
	TRACE(_T("Entering CCheckDataDlg::CLvCtrl::SetWindowRedraw\n"));

	m_bRedraw = bRedraw;
	CListView::SetWindowRedraw(bRedraw);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has pressed or double-clicked

LRESULT CCheckDataDlg::CLvHeader::LBtnDblClkDown(BOOL bDblClk, LPARAM lParam)
{
	TRACE(_T("Entering CCheckDataDlg::CLvHeader::LBtnDblClkDown\n"));

	CLvCtrl*      pListView;
	HDHITTESTINFO hhti;
	pListView = &e_pTPE->m_jCheckDataDlg.m_jListView;
	hhti.pt   = MAKEPOINT(lParam);
	HitTest(&hhti);
	if(hhti.flags & (HHT_ONDIVIDER | HHT_ONDIVOPEN))
	{
		if((hhti.iItem == LVC_NAME) && bDblClk &&
			(hhti.flags & HHT_ONDIVIDER) && (e_dwWinVer > WV_XP))
		{
			pListView->SetWindowRedraw(FALSE);
			pListView->SetColumnWidth(LVC_NAME, LVSCW_AUTOSIZE_USEHEADER);
			pListView->SetColumnWidth(LVC_NAME,
				pListView->GetColumnWidth(LVC_NAME) + CTomPoiEdr::m_exLvNameAdj);
			pListView->SetWindowRedraw();
			return WMR_PROCESSED;
		}

		if((hhti.iItem == LVC_LNG) || (hhti.iItem == LVC_LAT))
			return ((e_pTPE->m_fst == CTomPoiEdr::FST_CPB) ? WMR_PROCESSED : WMR_DEFAULT);

		return (((hhti.iItem == LVC_DIST) &&
			(e_pTPE->m_jCheckDataDlg.m_iCheckType != m_eNearDist)) ?
			WMR_PROCESSED : WMR_DEFAULT);
	}

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return ((pListView->GetItemCount() < 2) ? WMR_PROCESSED : WMR_DEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has double-clicked

AFXINLINE LRESULT CCheckDataDlg::CLvHeader::OnLButtonDblClk(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CCheckDataDlg::CLvHeader::OnLButtonDblClk\n"));

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return LBtnDblClkDown(TRUE, lParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has pressed

AFXINLINE LRESULT CCheckDataDlg::CLvHeader::OnLButtonDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CCheckDataDlg::CLvHeader::OnLButtonDown\n"));

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return LBtnDblClkDown(FALSE, lParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse cursor shape should be set

AFXINLINE BOOL CCheckDataDlg::CLvHeader::OnSetCursor(HWND hCursorWnd, int iHitTest, UINT nMsg)
{
	TRACE(_T("Entering CCheckDataDlg::CLvHeader::OnSetCursor\n"));

	HDHITTESTINFO hhti;
	GetCursorPos(&hhti.pt);
	ScreenToClient(&hhti.pt);
	HitTest(&hhti);
	if((hhti.iItem == LVC_LNG) || (hhti.iItem == LVC_LAT))
	{
		if(e_pTPE->m_fst != CTomPoiEdr::FST_CPB)
			return FALSE;
	}
	else if((hhti.iItem != LVC_DIST) || (e_pTPE->m_jCheckDataDlg.m_iCheckType == m_eNearDist))
		return FALSE;

	SetCursor(LoadCursor(IDC_ARROW));

	// Cursor shape is set (TRUE) or not (FALSE)
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CCheckDataDlg::CLvHeader

BEGIN_MESSAGE_MAP(CCheckDataDlg::CLvHeader)
	ON_ANY_MAP(WM_LBUTTONDBLCLK, OnLButtonDblClk)
	ON_ANY_MAP(WM_LBUTTONDOWN, OnLButtonDown)
	ON_WM_SETCURSOR(OnSetCursor)
END_MESSAGE_MAP()

////////////////////////////////////////////////////////////////////////////////////////////////
// Automatic sizes column widths of list view control

void CCheckDataDlg::AutoSizeLvCol()
{
	TRACE(_T("Entering CCheckDataDlg::AutoSizeLvCol\n"));

	int      i;
	LVCOLUMN lvc;
	lvc.mask = LVCF_WIDTH;
	lvc.cx   = 0;
	m_jListView.InsertColumn(LVC_COUNT, &lvc);
	i = LVC_FIRST;
	do
	{
		m_jListView.SetColumnWidth(i, (((e_pTPE->m_fst != CTomPoiEdr::FST_CPB) ||
			((i != LVC_LNG) && (i != LVC_LAT))) &&
			((i != LVC_DIST) || (m_iCheckType == m_eNearDist))) ? LVSCW_AUTOSIZE_USEHEADER : 0);

		if((i == LVC_NAME) && (e_dwWinVer > WV_XP))
			m_jListView.SetColumnWidth(LVC_NAME,
				m_jListView.GetColumnWidth(LVC_NAME) + CTomPoiEdr::m_exLvNameAdj);

		i++;
	}
	while(i < LVC_COUNT);
	m_jListView.DeleteColumn(LVC_COUNT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Checks the empty coordinates of POIs

void CCheckDataDlg::CheckEmptyCoord()
{
	TRACE(_T("Entering CCheckDataDlg::CheckEmptyCoord\n"));

	if(e_pTPE->m_jCheckPoiDlg.m_nButton[m_eEmptyCoord] == BST_CHECKED)
	{
		PLVITEMDATA pLVID;
		TCHAR       szText[e_eccNameCol];
		int         iLvItem;
		LVITEM      lvi;
		ShowProgress();
		m_jListView.SetWindowRedraw(FALSE);
		iLvItem        = INVALID_VALUE;
		lvi.pszText    = szText;
		lvi.cchTextMax = 2;
		for(int i = 0, j; i < m_ceItemId; i++)
		{
			lvi.iSubItem = LVC_LNG;
			if(e_pTPE->m_jListView.GetItemText(i, &lvi))
			{
				lvi.iSubItem = LVC_LAT;
				if(e_pTPE->m_jListView.GetItemText(i, &lvi))
					continue;
			}
			lvi.iSubItem   = LVC_NAME;
			lvi.cchTextMax = countof(szText);
			e_pTPE->m_jListView.GetItemText(i, &lvi);
			pLVID          = (PLVITEMDATA)malloc(sizeof(LVITEMDATA));
			pLVID->nItemId = e_pTPE->m_jListView.MapIndexToId(i);
			lvi.mask       = LVIF_PARAM | LVIF_TEXT;
			lvi.iItem      = MAX_INT;
			lvi.lParam     = (LPARAM)pLVID;
			iLvItem        = m_jListView.InsertItem(&lvi);
			j              = LVC_LNG;
			do
			{
				lvi.iSubItem = j;
				e_pTPE->m_jListView.GetItemText(i, &lvi);
				m_jListView.SetItemText(iLvItem, &lvi);
				j++;
			}
			while(j <= LVC_MEMO);
			lvi.cchTextMax = 2;
		}
		m_jListView.SetWindowRedraw();
		if(iLvItem != INVALID_VALUE)
		{
			ShowDialog();
			return;
		}
	}
	m_iCheckType = m_eNearDist;
	m_iItemId    = 0;
	CheckNearDist();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Checks the near distances of POIs

void CCheckDataDlg::CheckNearDist()
{
	TRACE(_T("Entering CCheckDataDlg::CheckNearDist\n"));

	if(e_pTPE->m_jCheckPoiDlg.m_nButton[m_eNearDist] == BST_CHECKED)
	{
		PLVITEMDATA pLVID;
		TCHAR       szText[e_eccNameCol];
		BOOL        bInsFirst;
		int         iLvItemL, iLvItemR, iInsItem;
		LPARAM      lParam, lDistance;
		LVITEM      lvi;
		ShowProgress();
		SetCursor(LoadCursor(IDC_WAIT));
		ShowCursor();
		m_jListView.SetWindowRedraw(FALSE);
		bInsFirst      = FALSE;
		iInsItem       = INVALID_VALUE;
		lvi.pszText    = szText;
		lvi.cchTextMax = 2;
		lDistance      = e_pTPE->m_jCheckPoiDlg.m_iCheckDist;
		for(int i, j; m_iItemId < m_ceItemId; m_iItemId++)
		{
			iLvItemL     = e_pTPE->m_jListView.MapIdToIndex(m_pnItemId[m_iItemId]);
			lvi.iSubItem = LVC_LNG;
			if(!e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi))
				continue;
			lvi.iSubItem = LVC_LAT;
			if(!e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi))
				continue;
			for(i = m_iItemId + 1; i < m_ceItemId; i++)
			{
				iLvItemR = e_pTPE->m_jListView.MapIdToIndex(m_pnItemId[i]);
				lParam   = e_pTPE->CalcPoiDistance(iLvItemL, iLvItemR);
				if(lParam > lDistance)
					continue;
				lvi.iSubItem   = LVC_NAME;
				lvi.cchTextMax = countof(szText);
				if(!bInsFirst)
				{
					e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi);
					pLVID          = (PLVITEMDATA)malloc(sizeof(LVITEMDATA));
					pLVID->nItemId = m_pnItemId[m_iItemId];
					pLVID->lParam  = MIN_LPARAM;
					lvi.mask       = LVIF_PARAM | LVIF_TEXT;
					lvi.iItem      = MAX_INT;
					lvi.lParam     = (LPARAM)pLVID;
					iInsItem       = m_jListView.InsertItem(&lvi);
					j              = LVC_LNG;
					do
					{
						lvi.iSubItem = j;
						if(j == LVC_DIST)
							lvi.pszText = NULL;
						else
							e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi);
						m_jListView.SetItemText(iInsItem, &lvi);
						j++;
					}
					while(j < LVC_COUNT);
					lvi.iSubItem = LVC_NAME;
					lvi.pszText  = szText;
					bInsFirst    = TRUE;
				}
				e_pTPE->m_jListView.GetItemText(iLvItemR, &lvi);
				pLVID          = (PLVITEMDATA)malloc(sizeof(LVITEMDATA));
				pLVID->nItemId = m_pnItemId[i];
				pLVID->lParam  = lParam;
				lvi.mask       = LVIF_PARAM | LVIF_TEXT;
				lvi.iItem      = MAX_INT;
				lvi.lParam     = (LPARAM)pLVID;
				iInsItem       = m_jListView.InsertItem(&lvi);
				j              = LVC_LNG;
				do
				{
					lvi.iSubItem = j;
					if(j == LVC_DIST)
						GetDistanceString(lParam, szText);
					else
						e_pTPE->m_jListView.GetItemText(iLvItemR, &lvi);
					m_jListView.SetItemText(iInsItem, &lvi);
					j++;
				}
				while(j < LVC_COUNT);
			}
			if(iInsItem != INVALID_VALUE)
				break;
			if(e_pTPE->PeekUpdateMessage(TRUE))
			{
				iInsItem = MIN_INT;
				break;
			}
		}
		m_jListView.SetWindowRedraw();
		HideCursor();
		SetCursor(LoadCursor(IDC_ARROW));
		if(iInsItem >= 0)
		{
			ShowDialog();
			return;
		}
		if(iInsItem != INVALID_VALUE)
		{
			FinishCheck();
			return;
		}
	}
	m_iCheckType = m_eSimilarName;
	m_iItemId    = 0;
	CheckSimilarName();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Checks the similar names of POIs

void CCheckDataDlg::CheckSimilarName()
{
	TRACE(_T("Entering CCheckDataDlg::CheckSimilarName\n"));

	int iInsItem;
	iInsItem = INVALID_VALUE;
	if(e_pTPE->m_jCheckPoiDlg.m_nButton[m_eSimilarName] == BST_CHECKED)
	{
		PCTSTR      pszText, pszSubText;
		PLVITEMDATA pLVID;
		TCHAR       szText[e_eccNameCol], szNameL[countof(szText)], szNameR[countof(szText)];
		BOOL        bInsFirst;
		int         iLvItemL, iLvItemR, ccNameL, ccNameR;
		LVITEM      lvi;
		ShowProgress();
		SetCursor(LoadCursor(IDC_WAIT));
		ShowCursor();
		m_jListView.SetWindowRedraw(FALSE);
		bInsFirst      = FALSE;
		lvi.iSubItem   = LVC_NAME;
		lvi.cchTextMax = countof(szNameL);
		for(int i, j; m_iItemId < m_ceItemId; m_iItemId++)
		{
			iLvItemL    = e_pTPE->m_jListView.MapIdToIndex(m_pnItemId[m_iItemId]);
			lvi.pszText = szNameL;
			ccNameL     = e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi);
			lvi.pszText = szNameR;
			for(i = m_iItemId + 1; i < m_ceItemId; i++)
			{
				iLvItemR = e_pTPE->m_jListView.MapIdToIndex(m_pnItemId[i]);
				ccNameR  = e_pTPE->m_jListView.GetItemText(iLvItemR, &lvi);
				if(ccNameL >= ccNameR)
				{
					pszText    = szNameL;
					pszSubText = szNameR;
				}
				else
				{
					pszText    = szNameR;
					pszSubText = szNameL;
				}
				if(!StrStrI(pszText, pszSubText) && (CompareString(LOCALE_USER_DEFAULT,
					NORM_IGNORECASE | NORM_IGNOREKANATYPE | NORM_IGNORESYMBOLS | NORM_IGNOREWIDTH,
					szNameL, ccNameL, szNameR, ccNameR) != CSTR_EQUAL))
					continue;
				if(!bInsFirst)
				{
					pLVID          = (PLVITEMDATA)malloc(sizeof(LVITEMDATA));
					pLVID->nItemId = m_pnItemId[m_iItemId];
					lvi.mask       = LVIF_PARAM | LVIF_TEXT;
					lvi.iItem      = MAX_INT;
					lvi.pszText    = szNameL;
					lvi.lParam     = (LPARAM)pLVID;
					iInsItem       = m_jListView.InsertItem(&lvi);
					lvi.pszText    = szText;
					j              = LVC_LNG;
					do
					{
						lvi.iSubItem = j;
						e_pTPE->m_jListView.GetItemText(iLvItemL, &lvi);
						m_jListView.SetItemText(iInsItem, &lvi);
						j++;
					}
					while(j <= LVC_MEMO);
					lvi.iSubItem = LVC_NAME;
					lvi.pszText  = szNameR;
					bInsFirst    = TRUE;
				}
				pLVID          = (PLVITEMDATA)malloc(sizeof(LVITEMDATA));
				pLVID->nItemId = m_pnItemId[i];
				lvi.mask       = LVIF_PARAM | LVIF_TEXT;
				lvi.iItem      = MAX_INT;
				lvi.lParam     = (LPARAM)pLVID;
				iInsItem       = m_jListView.InsertItem(&lvi);
				lvi.pszText    = szText;
				j              = LVC_LNG;
				do
				{
					lvi.iSubItem = j;
					e_pTPE->m_jListView.GetItemText(iLvItemR, &lvi);
					m_jListView.SetItemText(iInsItem, &lvi);
					j++;
				}
				while(j <= LVC_MEMO);
				lvi.iSubItem = LVC_NAME;
				lvi.pszText  = szNameR;
			}
			if(iInsItem != INVALID_VALUE)
				break;
			if(e_pTPE->PeekUpdateMessage(TRUE))
			{
				iInsItem = MIN_INT;
				break;
			}
		}
		m_jListView.SetWindowRedraw();
		HideCursor();
		SetCursor(LoadCursor(IDC_ARROW));
		if(iInsItem >= 0)
		{
			ShowDialog();
			return;
		}
	}
	FinishCheck();
	if(iInsItem == INVALID_VALUE)
		e_pTPE->DisplayMessage(IDS_CHECK_POI_COMPLETED, MB_ICONINFORMATION | MB_OK, UNUSED);
	e_pTPE->m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables buttons

void CCheckDataDlg::EnableButtons()
{
	TRACE(_T("Entering CCheckDataDlg::EnableButtons\n"));

	int ceSelItem;
	ceSelItem = m_jListView.GetSelCount();
	m_jButton[m_eDelete].EnableWindow(ceSelItem);
	m_jButton[m_eDeleteAll].EnableWindow(m_jListView.GetItemCount());
	m_jButton[m_eGoTo].EnableWindow(ceSelItem == 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finishes checking the data

void CCheckDataDlg::FinishCheck()
{
	TRACE(_T("Entering CCheckDataDlg::FinishCheck\n"));

	m_jProgress.DestroyWindow();
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	DestroyWindow();
	m_hWnd              = NULL;
	e_pTPE->m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Shows the dialog box

void CCheckDataDlg::ShowDialog()
{
	TRACE(_T("Entering CCheckDataDlg::ShowDialog\n"));

	TCHAR  szText[MAX_STR];
	LVITEM lvi;
	m_jProgress.HideWindow();
	e_pTPE->m_bSetReady = TRUE;
	LoadString(e_hInstance, IDS_CHKCAP_SIMILAR_NAME + m_iCheckType, szText, countof(szText));
	SetWindowText(szText);
	AutoSizeLvCol();
	lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(0, &lvi);
	if(e_pTPE->m_jOptionDlg.m_nFadeWnd == BST_CHECKED)
	{
		SetWindowExStyle(GetWindowExStyle() | WS_EX_LAYERED);
		SetLayeredWindowAttributes(UNUSED, 0, LWA_ALPHA);
	}
	ShowWindow();
	SetActiveWindow();
	FadeInDialog(*this);
	m_jListView.SetFocus();
	e_pTPE->m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Shows the progress control

void CCheckDataDlg::ShowProgress()
{
	TRACE(_T("Entering CCheckDataDlg::ShowProgress\n"));

	TCHAR szText[MAX_STR];
	int   iRightPos[CTomPoiEdr::m_eceSbPart];
	SIZE  size;
	GetTextExtent(e_pTPE->m_hSbDC, szText,
		LoadString(e_hInstance, IDP_CHECKING_SIMILAR_NAME + m_iCheckType,
		szText, countof(szText)), &size);
	e_pTPE->m_jStatusBar.SetWindowText(szText);
	size.cx += CTomPoiEdr::m_ecxPbPadL;
	e_pTPE->m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
	m_jProgress.MoveWindow(size.cx, CTomPoiEdr::m_ecyPbPadT,
		iRightPos[CTomPoiEdr::m_eInfoPart] - size.cx - CTomPoiEdr::m_ecxPbPadR,
		e_pTPE->m_cyStatusBar - CTomPoiEdr::m_ecyPbPadB);
	m_jProgress.ShowWindow();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Swaps the identifiers of list view items

void CCheckDataDlg::SwapItemId(int iItem1, int iItem2)
{
	TRACE(_T("Entering CCheckDataDlg::SwapItemId\n"));

	if(!*this)
		return;

	PUINT pnItemId;
	int   i, j;
	UINT  nItemId1, nItemId2;
	nItemId1 = e_pTPE->m_jListView.MapIndexToId(iItem1);
	nItemId2 = e_pTPE->m_jListView.MapIndexToId(iItem2);
	i        = 0;
	j        = 0;
	pnItemId = m_pnItemId;
	do
	{
		if(nItemId1 == *pnItemId)
		{
			*pnItemId = nItemId2;
			j++;
		}
		else if(nItemId2 == *pnItemId)
		{
			*pnItemId = nItemId1;
			j++;
		}

		if(j == 2)
			break;

		pnItemId++;
		i++;
	}
	while(i < m_ceItemId);

	if(!j)
		return;

	PLVITEMDATA pLVID;
	int         ceLvItem;
	LVITEM      lvi;
	ceLvItem     = m_jListView.GetItemCount();
	lvi.mask     = LVIF_PARAM;
	lvi.iSubItem = 0;
	for(i = 0; i < ceLvItem; i++)
	{
		lvi.iItem = i;
		m_jListView.GetItem(&lvi);
		pLVID = (PLVITEMDATA)lvi.lParam;
		if(pLVID->nItemId == nItemId1)
		{
			pLVID->nItemId = nItemId2;
			j--;
		}
		else if(pLVID->nItemId == nItemId2)
		{
			pLVID->nItemId = nItemId1;
			j--;
		}

		if(!j)
			return;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Updates the data of list view item

void CCheckDataDlg::UpdateItemData(int iItem, int iColumn)
{
	TRACE(_T("Entering CCheckDataDlg::UpdateItemData\n"));

	ASSERT(iColumn != LVC_DIST);
	if(!*this)
		return;

	UINT nItemId;
	nItemId = e_pTPE->m_jListView.MapIndexToId(iItem);
	for(int i = 0; (i < m_ceItemId) && (nItemId != m_pnItemId[i]); i++);
	if(i == m_ceItemId)
		return;

	BOOL bDelBase;
	bDelBase = FALSE;
	if(iColumn == LVC_COUNT)
	{
		PUINT pnItemId;
		pnItemId = m_pnItemId + i;
		MoveMemory(pnItemId, pnItemId + 1, (m_ceItemId - i - 1) * sizeof(int));
		m_ceItemId--;
		if(m_iItemId == i)
		{
			m_iItemId--;
			bDelBase = TRUE;
		}
	}

	PLVITEMDATA pLVID;
	int         ceChkItem;
	LVITEM      lvi;
	ceChkItem    = m_jListView.GetItemCount();
	lvi.mask     = LVIF_PARAM;
	lvi.iSubItem = 0;
	for(lvi.iItem = 0; lvi.iItem < ceChkItem; lvi.iItem++)
	{
		m_jListView.GetItem(&lvi);
		pLVID = (PLVITEMDATA)lvi.lParam;
		if(pLVID->nItemId == nItemId)
			break;
	}
	if(lvi.iItem == ceChkItem)
		return;

	TCHAR szText[e_eccNameCol];
	int   iChkItem;
	iChkItem = lvi.iItem;
	if(iColumn == LVC_COUNT)
	{
		lvi.state     = UNUSED;
		lvi.stateMask = LVIS_FOCUSED;
		m_jListView.SetItemState(&lvi);
		m_jListView.DeleteItem(iChkItem);
		ceChkItem--;
	}
	else
	{
		lvi.iSubItem   = iColumn;
		lvi.pszText    = szText;
		lvi.cchTextMax = countof(szText);
		e_pTPE->m_jListView.GetItemText(iItem, &lvi);
		m_jListView.SetItemText(iChkItem, &lvi);
	}

	if(m_iCheckType != m_eNearDist)
		return;

	if((iColumn == LVC_LNG) || (iColumn == LVC_LAT))
	{
		lvi.mask     = LVIF_PARAM;
		lvi.iSubItem = 0;
		for(lvi.iItem = 0; lvi.iItem < ceChkItem; lvi.iItem++)
		{
			m_jListView.GetItem(&lvi);
			pLVID = (PLVITEMDATA)lvi.lParam;
			if(pLVID->lParam == MIN_LPARAM)
				break;
		}
		if(lvi.iItem == ceChkItem)
			return;

		int iChkBase, iBaseItem, iFirstItem, iLastItem;
		iChkBase = lvi.iItem;
		lvi.mask = LVIF_PARAM;
		m_jListView.GetItem(&lvi);
		pLVID     = (PLVITEMDATA)lvi.lParam;
		iBaseItem = e_pTPE->m_jListView.MapIdToIndex(pLVID->nItemId);
		if(iChkItem == iChkBase)
		{
			iFirstItem = 0;
			iLastItem  = ceChkItem - 1;
		}
		else
		{
			iFirstItem = iChkItem;
			iLastItem  = iChkItem;
		}
		for(lvi.iItem = iFirstItem; lvi.iItem <= iLastItem; lvi.iItem++)
		{
			if(lvi.iItem == iChkBase)
				continue;

			lvi.mask = LVIF_PARAM;
			m_jListView.GetItem(&lvi);
			pLVID         = (PLVITEMDATA)lvi.lParam;
			pLVID->lParam = e_pTPE->CalcPoiDistance(iBaseItem,
				e_pTPE->m_jListView.MapIdToIndex(pLVID->nItemId));
			if(pLVID->lParam == MAX_LPARAM)
				lvi.pszText = NULL;
			else
			{
				GetDistanceString(pLVID->lParam, szText);
				lvi.pszText = szText;
			}
			lvi.iSubItem = LVC_DIST;
			m_jListView.SetItemText(lvi.iItem, &lvi);
		}
		return;
	}

	if(bDelBase)
	{
		lvi.mask    = LVIF_PARAM;
		lvi.pszText = NULL;
		for(lvi.iItem = 0; lvi.iItem < ceChkItem; lvi.iItem++)
		{
			lvi.iSubItem = LVC_DIST;
			m_jListView.SetItemText(lvi.iItem, &lvi);
			lvi.iSubItem = 0;
			m_jListView.GetItem(&lvi);
			pLVID         = (PLVITEMDATA)lvi.lParam;
			pLVID->lParam = MAX_LPARAM;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes the position and size of dialog box into profile

void CCheckDataDlg::WriteSection()
{
	TRACE(_T("Entering CCheckDataDlg::WriteSection\n"));

	TCHAR szEntry[32];
	RECT  rcDlg;
	GetWindowRect(&rcDlg);
	_ltot(rcDlg.left, szEntry);
	WritePrivateProfileString(m_szSection, e_szLeft, szEntry, e_pszProfile);
	_ltot(rcDlg.top, szEntry);
	WritePrivateProfileString(m_szSection, e_szTop, szEntry, e_pszProfile);
	_ltot(GetRectWidth(&rcDlg), szEntry);
	WritePrivateProfileString(m_szSection, e_szWidth, szEntry, e_pszProfile);
	_ltot(GetRectHeight(&rcDlg), szEntry);
	WritePrivateProfileString(m_szSection, e_szHeight, szEntry, e_pszProfile);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item strings

int CALLBACK CCheckDataDlg::CompareLvItem(int iItem1, int iItem2, LPARAM lSortParam)
{
	TRACE(_T("Entering CCheckDataDlg::CompareLvItem\n"));

	TCHAR  szText1[e_eccNameCol], szText2[countof(szText1)];
	int    iLvCol, iResult;
	LVITEM lvi;
	iLvCol         = e_pTPE->m_jCheckDataDlg.m_iLvCol;
	lvi.iSubItem   = iLvCol;
	lvi.pszText    = szText1;
	lvi.cchTextMax = countof(szText1);
	e_pTPE->m_jCheckDataDlg.m_jListView.GetItemText(iItem1, &lvi);
	lvi.pszText = szText2;
	e_pTPE->m_jCheckDataDlg.m_jListView.GetItemText(iItem2, &lvi);
	if((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT))
	{
		if(!*szText1)
			iResult = -1;
		else if(!*szText2)
			iResult = 1;
		else
		{
			double dTemp;
			dTemp   = GetCoordValueD(szText1) - GetCoordValueD(szText2);
			iResult = (dTemp < 0) ? -1 : ((dTemp > 0) ? 1 : 0);
		}
	}
	else
	{
#ifdef UNICODE
		iResult = StrCmpLogicalW(szText1, szText2);
		if(!iResult)
			iResult = lstrcmp(szText1, szText2);
#else
		iResult = lstrcmp(szText1, szText2);
#endif
	}
	return (e_pTPE->m_jCheckDataDlg.m_bAscend ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item parameters

int CALLBACK CCheckDataDlg::CompareLvParam(LPARAM lParam1, LPARAM lParam2, LPARAM lSortParam)
{
	TRACE(_T("Entering CCheckDataDlg::CompareLvParam\n"));

	lParam1 = ((PLVITEMDATA)lParam1)->lParam;
	lParam2 = ((PLVITEMDATA)lParam2)->lParam;
	if(lParam1 == lParam2)
		return 0;

	int iResult;
	iResult = (lParam1 > lParam2) ? 1 : -1;
	return (e_pTPE->m_jCheckDataDlg.m_bAscend ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Top-level window is being activated or deactivated

AFXINLINE LRESULT CCheckDataDlg::OnActivate(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CCheckDataDlg::OnActivate\n"));

	if(LOWORD(wParam) == WA_INACTIVE)
		return WMR_DEFAULT;

	if(e_pTPE->m_jLvEdit.IsWindowVisible())
		e_pTPE->m_jLvEdit.SendMessage(WM_CHAR, CHR_ESC);

	HWND hDialog, hPopup;
	hDialog = *this;
	do
	{
		hDialog = ::GetWindow(hDialog, GW_HWNDNEXT);
		if(!hDialog)
			return WMR_DEFAULT;
	}
	while((::GetWindow(hDialog, GW_OWNER) != *e_pTPE) || !::IsWindowVisible(hDialog));
	hPopup = ::GetWindow(hDialog, GW_ENABLEDPOPUP);
	::SetActiveWindow(hPopup ? hPopup : hDialog);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHKDATA_DELETE control sends a notification message

AFXINLINE void CCheckDataDlg::OnDelete(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnDelete\n"));

	int    iTemp, iLvItem;
	LVITEM lvi;
	m_jListView.SetWindowRedraw(FALSE);
	e_pTPE->m_jListView.SetWindowRedraw(FALSE);
	e_pTPE->m_jListView.SetAction(AT_NEW);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED;
	m_jListView.SetItemState(&lvi);
	e_pTPE->m_jListView.SetItemState(&lvi);
	iLvItem      = m_jListView.GetNextItem(LVNI_SELECTED);
	lvi.mask     = LVIF_PARAM;
	lvi.iSubItem = 0;
	do
	{
		lvi.iItem = iLvItem;
		m_jListView.GetItem(&lvi);
		iTemp = e_pTPE->m_jListView.MapIdToIndex(((PLVITEMDATA)lvi.lParam)->nItemId);
		e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, iTemp);
		e_pTPE->m_jListView.DeleteItem(iTemp);
		if(!iTemp && e_pTPE->m_bMyBP)
			e_pTPE->m_bMyBP = FALSE;
		else
			e_pTPE->m_bFileDirty = TRUE;
		iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem - 1);
	}
	while(iLvItem != RV_ERR);
	if(m_jListView.GetItemCount() < 2)
	{
		LVCOLUMN lvc;
		lvc.mask = LVCF_FMT;
		iTemp    = LVC_FIRST;
		do
		{
			lvc.fmt = (iTemp == LVC_DIST) ? LVCFMT_RIGHT : LVCFMT_LEFT;
			m_jListView.SetColumn(iTemp, &lvc);
			iTemp++;
		}
		while(iTemp < LVC_COUNT);
	}
	AutoSizeLvCol();
	m_jListView.SetWindowRedraw();
	m_jListView.SetFocus();
	EnableButtons();
	e_pTPE->SetBasePoint(LVI_FIND, TRUE);
	e_pTPE->m_jListView.SetWindowRedraw();
	e_pTPE->m_bSetCaption = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHKDATA_DELETE_ALL control sends a notification message

AFXINLINE void CCheckDataDlg::OnDeleteAll(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnDeleteAll\n"));

	LVITEM lvi;
	lvi.state     = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	PostMessage(WM_COMMAND, MAKEWPARAM(IDC_CHKDATA_DELETE, BN_CLICKED));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHKDATA_GO_TO control sends a notification message

AFXINLINE void CCheckDataDlg::OnGoTo(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnGoTo\n"));

	e_pTPE->SetActiveWindow();
	e_pTPE->m_jListView.SetFocus();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHKDATA_CONTINUE control sends a notification message

AFXINLINE void CCheckDataDlg::OnContinue(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnContinue\n"));

	int      i;
	LVCOLUMN lvc;
	e_pTPE->m_ceDragDrop++;
	FadeOutDialog(*this);
	e_pTPE->SetActiveWindow();
	WriteSection();
	HideWindow();
	lvc.mask = LVCF_FMT;
	i        = LVC_FIRST;
	do
	{
		lvc.fmt = (i == LVC_DIST) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		m_jListView.SetColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	if(m_jListView.GetItemCount())
		m_jListView.DeleteAllItems();
	if(m_iCheckType == m_eEmptyCoord)
	{
		m_iCheckType = m_eNearDist;
		m_iItemId    = 0;
	}
	else
		m_iItemId++;
	if(m_iCheckType == m_eSimilarName)
		CheckSimilarName();
	else
		CheckNearDist();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CCheckDataDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnCancel\n"));

	FadeOutDialog(*this);
	FinishCheck();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window is being destroyed

AFXINLINE void CCheckDataDlg::OnDestroy()
{
	TRACE(_T("Entering CCheckDataDlg::OnDestroy\n"));

	WriteSection();
	free(m_pnItemId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size or position is about to change

AFXINLINE void CCheckDataDlg::OnGetMinMaxInfo(PMINMAXINFO pMMI)
{
	TRACE(_T("Entering CCheckDataDlg::OnGetMinMaxInfo\n"));

	pMMI->ptMinTrackSize.x = max(pMMI->ptMinTrackSize.x, m_ecxMinDlg);
	pMMI->ptMinTrackSize.y = max(pMMI->ptMinTrackSize.y, m_ecyMinDlg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CCheckDataDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CCheckDataDlg::OnInitDialog\n"));

	SCUINT snButtonId[] =
	{
		IDC_CHKDATA_DELETE,
		IDC_CHKDATA_DELETE_ALL,
		IDC_CHKDATA_GO_TO,
		IDC_CHKDATA_CONTINUE,
		ID_CANCEL,
	};
	int i;
	ASSERT(countof(m_jButton) == countof(snButtonId));
	SetDialogIcon(*this, ID_POI_CHECK);
	m_jListView.m_hWnd = GetChild(IDC_CHKDATA_LV);
	m_jListView.SetTextColor(e_pTPE->m_jListView.GetTextColor());
	m_jListView.SetTextBkColor(GetSysColor(COLOR_WINDOW));
	m_jLvHeader.m_hWnd = m_jListView.GetHeader();
	m_jLvHeader.SubclassWindow();
	i = 0;
	do
	{
		m_jButton[i] = GetChild(snButtonId[i]);
		i++;
	}
	while(i < countof(m_jButton));
	m_jProgress.CreateWindow(m_ePbMqStyle, 0, 0, 0, 0, e_pTPE->m_jStatusBar, IDW_PROGRESS);
	m_jProgress.SetMarquee(CTomPoiEdr::m_ePbMqIntl);

	TCHAR    szLvCol[64];
	UINT     nResId;
	DWORD    dwLvStyle;
	LVCOLUMN lvc;
	lvc.mask    = LVCF_FMT | LVCF_TEXT;
	lvc.pszText = szLvCol;
	i           = LVC_FIRST;
	do
	{
		lvc.fmt = (i == LVC_DIST) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		if((e_pTPE->m_fst == CTomPoiEdr::FST_SC) && ((i == LVC_SPEED) || (i == LVC_CAMERA)))
			nResId = IDS_SPEED_COL - LVC_SPEED;
		else
		{
			if((e_pTPE->m_fst == CTomPoiEdr::FST_CPB) && (i == LVC_CPBT))
				nResId = IDS_CPBT_COL - LVC_CPBT;
			else
				nResId = IDS_FIRST_COL;
		}
		LoadString(e_hInstance, nResId + i, szLvCol, countof(szLvCol));
		m_jListView.InsertColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	m_jListView.SetColumnOrder(e_iHdOrder, countof(e_iHdOrder));
	if(e_dwWinVer > WV_XP)
	{
		m_jListView.SetWindowStyle(m_jListView.GetWindowStyle() | LVS_SHOWSELALWAYS);
		dwLvStyle = m_eLvExStyle;
	}
	else
		dwLvStyle = m_eLvExStyleXP;
	m_jListView.SetExStyle(dwLvStyle, dwLvStyle);
	m_jListView.SetWindowTheme(e_szAppTheme);

	int  xDlg, yDlg, cxDlg, cyDlg;
	RECT rcWork;
	SystemParametersInfo(SPI_GETWORKAREA, &rcWork);
	xDlg  = clamp(GetPrivateProfileInt(m_szSection, e_szLeft,
		(rcWork.left + rcWork.right - m_ecxMinDlg) / 2, e_pszProfile),
		rcWork.left, rcWork.right - m_ecxMinDlg);
	yDlg  = clamp(GetPrivateProfileInt(m_szSection, e_szTop,
		(rcWork.top + rcWork.bottom - m_ecyMinDlg) / 2, e_pszProfile),
		rcWork.top, rcWork.bottom - m_ecyMinDlg);
	cxDlg = clamp(GetPrivateProfileInt(m_szSection, e_szWidth, m_ecxMinDlg, e_pszProfile),
		m_ecxMinDlg, rcWork.right - xDlg);
	cyDlg = clamp(GetPrivateProfileInt(m_szSection, e_szHeight, m_ecyMinDlg, e_pszProfile),
		m_ecyMinDlg, rcWork.bottom - yDlg);
	MoveWindow(xDlg, yDlg, cxDlg, cyDlg, FALSE);

	m_iCheckType = m_eEmptyCoord;
	m_ceItemId   = e_pTPE->m_jListView.GetItemCount();
	m_pnItemId   = (PUINT)malloc(m_ceItemId * sizeof(UINT));
	for(i = 0; i < m_ceItemId; i++)
		m_pnItemId[i] = e_pTPE->m_jListView.MapIndexToId(i);
	m_iItemId    = MAX_INT;
	m_iLvCol     = LVC_NAME;
	m_bAscend    = FALSE;
	m_bLRKeyDown = FALSE;

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A column in a list view was clicked

AFXINLINE void CCheckDataDlg::OnLvnColumnClick(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CCheckDataDlg::OnLvnColumnClick\n"));

	int      i;
	LVCOLUMN lvc;
	if(m_iLvCol == pNMLV->iSubItem)
		m_bAscend = !m_bAscend;
	else
		m_iLvCol = pNMLV->iSubItem;
	lvc.mask = LVCF_FMT;
	i        = LVC_FIRST;
	do
	{
		lvc.fmt = (i == LVC_DIST) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		if(i == m_iLvCol)
			lvc.fmt |= m_bAscend ? HDF_SORTUP : HDF_SORTDOWN;
		m_jListView.SetColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	if(m_iLvCol == LVC_DIST)
		m_jListView.SortItems(CompareLvParam);
	else
		m_jListView.SortItems(CompareLvItem);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// An item in a list view is about to be deleted

AFXINLINE void CCheckDataDlg::OnLvnDeleteItem(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CCheckDataDlg::OnLvnDeleteItem\n"));

	LVITEM lvi;
	lvi.mask     = LVIF_PARAM;
	lvi.iItem    = pNMLV->iItem;
	lvi.iSubItem = 0;
	m_jListView.GetItem(&lvi);
	free((PLVITEMDATA)lvi.lParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes of an item in a list view have changed

AFXINLINE void CCheckDataDlg::OnLvnItemChanged(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CCheckDataDlg::OnLvnItemChanged\n"));

	EnableButtons();
	if(!(pNMLV->uNewState & LVIS_SELECTED))
		return;

	if(m_bLRKeyDown)
	{
		m_bLRKeyDown = FALSE;
		return;
	}

	if(m_jListView.GetSelCount() != 1)
		return;

	LVITEM lvi;
	lvi.mask     = LVIF_PARAM;
	lvi.iItem    = pNMLV->iItem;
	lvi.iSubItem = 0;
	m_jListView.GetItem(&lvi);
	lvi.iItem     = e_pTPE->m_jListView.MapIdToIndex(((PLVITEMDATA)lvi.lParam)->nItemId);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	e_pTPE->m_jListView.SetItemState(&lvi);
	lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
	e_pTPE->m_jListView.SetItemState(lvi.iItem, &lvi);
	e_pTPE->EnsureLvItemVisible(lvi.iItem);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view has the keyboard focus and the user presses a virtual key

AFXINLINE void CCheckDataDlg::OnLvnKeyDown(PCNMLVKEYDOWN pNMLVKD)
{
	TRACE(_T("Entering CCheckDataDlg::OnLvnKeyDown\n"));

	if(e_dwWinVer > WV_XP)
		return;

	if((pNMLVKD->wVKey == VK_LEFT) || (pNMLVKD->wVKey == VK_RIGHT))
		m_bLRKeyDown = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view is about to be drawn

AFXINLINE UINT CCheckDataDlg::OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering CCheckDataDlg::OnNmCustomDraw\n"));

	// Result value of the message processed
	return ((pNMCD->hdr.idFrom == IDC_CHKDATA_LV) ?
		CTomPoiEdr::CustomDrawListView((PNMLVCUSTOMDRAW)pNMCD) : CDRF_DODEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has double-clicked (an item with) the left mouse button within a control

AFXINLINE BOOL CCheckDataDlg::OnNmDblClk(PCNMHDR pNMH)
{
	TRACE(_T("Entering CCheckDataDlg::OnNmDblClk\n"));

	if(((PCNMITEMACTIVATE)pNMH)->iItem != -1)
	{
		e_pTPE->m_bGmSelItem = FALSE;
		PostMessage(WM_COMMAND, MAKEWPARAM(IDC_CHKDATA_GO_TO, BN_CLICKED));
	}

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size has changed

AFXINLINE void CCheckDataDlg::OnSize(UINT nType, int cxClient, int cyClient)
{
	TRACE(_T("Entering CCheckDataDlg::OnSize\n"));

	int  i, cxGap, cyGap, xButton, yButton, cxButton, cyButton, cyBtnGap;
	RECT rcCtrl;
	m_jListView.GetWindowRect(&rcCtrl);
	ScreenToClient((PPOINT)&rcCtrl);
	cxGap    = rcCtrl.left;
	cyGap    = rcCtrl.top;
	yButton  = cyGap;
	cyBtnGap = cyGap / 3;
	m_jButton[m_eDelete].GetWindowRect(&rcCtrl);
	cxButton = GetRectWidth(&rcCtrl);
	cyButton = GetRectHeight(&rcCtrl) + cyBtnGap;
	xButton  = cxClient - cxButton - cxGap;
	i        = 0;
	do
	{
		m_jButton[i].SetWindowPos(NULL, xButton, yButton, UNUSED, UNUSED,
			SWP_NOSIZE | SWP_NOZORDER);
		if(i == m_eGoTo)
			yButton += cyBtnGap;
		yButton += cyButton;
		i++;
	}
	while(i < countof(m_jButton));
	m_jListView.SetWindowPos(NULL, UNUSED, UNUSED,
		cxClient - cxButton - cxGap * 3, cyClient - cyGap * 2, SWP_NOMOVE | SWP_NOZORDER);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CCheckDataDlg

BEGIN_MESSAGE_MAP(CCheckDataDlg)
	ON_ANY_MAP(WM_ACTIVATE, OnActivate)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_CHKDATA_DELETE, OnDelete)
		ON_CONTROL(IDC_CHKDATA_DELETE_ALL, OnDeleteAll)
		ON_CONTROL(IDC_CHKDATA_GO_TO, OnGoTo)
		ON_CONTROL(IDC_CHKDATA_CONTINUE, OnContinue)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_DESTROY(OnDestroy)
	ON_WM_GETMINMAXINFO(OnGetMinMaxInfo)
	ON_WM_INITDIALOG(OnInitDialog)
	BEGIN_NOTIFY_MAP()
		ON_LVN_COLUMNCLICK(OnLvnColumnClick)
		ON_LVN_DELETEITEM(OnLvnDeleteItem)
		ON_LVN_ITEMCHANGED(OnLvnItemChanged)
		ON_LVN_KEYDOWN(OnLvnKeyDown)
		ON_NM_CUSTOMDRAW(OnNmCustomDraw)
		ON_NM_DBLCLK(OnNmDblClk)
	END_NOTIFY_MAP()
	ON_WM_SIZE(OnSize)
END_MESSAGE_MAP()
