// CCheckDataDlg.hpp - Checks POI Data Dialog Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CCHECKDATADLG_INCLUDED_
#define _CCHECKDATADLG_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CCheckDataDlg - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

#include "CCheckPoiDlg.hpp"

class CCheckDataDlg : public CDialog
{
public:
	enum
	{
		// Minimum size of dialog box
		m_ecxMinDlg = 320,
		m_ecyMinDlg = 240,

		// List view extended styles
		m_eLvExStyle   = LVS_EX_DOUBLEBUFFER | LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP,
		m_eLvExStyleXP = m_eLvExStyle | LVS_EX_GRIDLINES,

		// Progress styles
		m_ePbMqStyle = WS_CHILD | PBS_MARQUEE,

		// Checking data type
		m_eSimilarName = CCheckPoiDlg::m_eSimilarName,
		m_eNearDist    = CCheckPoiDlg::m_eNearDist,
		m_eEmptyCoord  = CCheckPoiDlg::m_eEmptyCoord,

		// Button indices
		m_eDelete = 0,
		m_eDeleteAll,
		m_eGoTo,
		m_eContinue,
		m_eCancel,
		m_eceButton,
	};

	// List view item data (lvid)
	typedef struct LVITEMDATA
	{
		UINT   nItemId;
		LPARAM lParam;
	} * PLVITEMDATA;

	class CLvCtrl : public CListView
	{
	public:
		BOOL m_bRedraw;

	public:
		int InsertItem(PLVITEM pLVI);
		void SetWindowRedraw(BOOL bRedraw = TRUE);
	};

	class CLvHeader : public CSubclassWnd<CHeader>
	{
	public:
		LRESULT LBtnDblClkDown(BOOL bDblClk, LPARAM lParam);

	protected:
		LRESULT OnLButtonDblClk(WPARAM wParam, LPARAM lParam);
		LRESULT OnLButtonDown(WPARAM wParam, LPARAM lParam);
		BOOL OnSetCursor(HWND hCursorWnd, int iHitTest, UINT nMsg);

		DECLARE_MESSAGE_MAP()
	};

public:
	BOOL      m_bAscend, m_bLRKeyDown;
	int       m_iCheckType, m_ceItemId, m_iItemId, m_iLvCol;
	PUINT     m_pnItemId;
	CButton   m_jButton[m_eceButton];
	CLvCtrl   m_jListView;
	CLvHeader m_jLvHeader;
	CProgress m_jProgress;

public:
	static CTCHAR m_szSection[];

public:
	CCheckDataDlg();
	void AutoSizeLvCol();
	void CheckEmptyCoord();
	void CheckNearDist();
	void CheckSimilarName();
	void EnableButtons();
	void FinishCheck();
	void ShowDialog();
	void ShowProgress();
	void SwapItemId(int iItem1, int iItem2);
	void UpdateItemData(int iItem, int iColumn);
	void WriteSection();

public:
	static int CALLBACK CompareLvItem(int iItem1, int iItem2, LPARAM lSortParam);
	static int CALLBACK CompareLvParam(LPARAM lParam1, LPARAM lParam2, LPARAM lSortParam);

protected:
	LRESULT OnActivate(WPARAM wParam, LPARAM lParam);
	void OnDelete(UINT nNotify, HWND hCtrlWnd);
	void OnDeleteAll(UINT nNotify, HWND hCtrlWnd);
	void OnGoTo(UINT nNotify, HWND hCtrlWnd);
	void OnContinue(UINT nNotify, HWND hCtrlWnd);
	void OnCancel(UINT nNotify, HWND hCtrlWnd);
	void OnDestroy();
	void OnGetMinMaxInfo(PMINMAXINFO pMMI);
	BOOL OnInitDialog(HWND hFocusWnd);
	void OnLvnColumnClick(PCNMLISTVIEW pNMLV);
	void OnLvnDeleteItem(PCNMLISTVIEW pNMLV);
	void OnLvnItemChanged(PCNMLISTVIEW pNMLV);
	void OnLvnKeyDown(PCNMLVKEYDOWN pNMLVKD);
	UINT OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD);
	BOOL OnNmDblClk(PCNMHDR pNMH);
	void OnSize(UINT nType, int cxClient, int cyClient);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CCheckDataDlg::CCheckDataDlg()
{
	TRACE(_T("Entering CCheckDataDlg::CCheckDataDlg\n"));

	m_hWnd                = NULL;
	m_jListView.m_bRedraw = TRUE;
}

// End of _CCHECKDATADLG_INCLUDED_
#endif
