// CCheckPoiDlg.cpp - Checks POI Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes the dialog

void CCheckPoiDlg::CloseDialog(int iResult)
{
	TRACE(_T("Entering CCheckPoiDlg::CloseDialog\n"));

	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables buttons

void CCheckPoiDlg::EnableButtons()
{
	TRACE(_T("Entering CCheckPoiDlg::EnableButtons\n"));

	BOOL bNearDist;
	int  i;
	i = 0;
	do
	{
		if(m_jButton[i].GetCheck() == BST_CHECKED)
			break;
		i++;
	}
	while(i < countof(m_jButton));
	bNearDist = m_jButton[m_eNearDist].GetCheck() == BST_CHECKED;
	m_jCkeckDist.EnableWindow(bNearDist);
	m_jLargeBtn.EnableWindow(bNearDist);
	m_jSmallBtn.EnableWindow(bNearDist);
	m_jOKBtn.EnableWindow(i < countof(m_jButton));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHECK_SIMILAR_NAME control sends a notification message

AFXINLINE void CCheckPoiDlg::OnSimilarName(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnSimilarName\n"));

	EnableButtons();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHECK_NEAR_DIST control sends a notification message

AFXINLINE void CCheckPoiDlg::OnNearDist(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnNearDist\n"));

	EnableButtons();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_CHECK_EMPTY_COORD control sends a notification message

AFXINLINE void CCheckPoiDlg::OnEmptyCoord(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnEmptyCoord\n"));

	EnableButtons();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_OK control sends a notification message

AFXINLINE void CCheckPoiDlg::OnOK(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnOK\n"));

	int i;
	i = 0;
	do
	{
		m_nButton[i] = m_jButton[i].GetCheck();
		i++;
	}
	while(i < countof(m_nButton));
	if(m_nButton[m_eNearDist] == BST_CHECKED)
	{
		TCHAR szText[MAX_STR];
		int   ccText;
		m_jCkeckDist.GetWindowText(szText, countof(szText));
		ccText = TrimString(szText);
		for(i = 0; (i < ccText) && IsValidNumber(szText[i]); i++);
		if(i < ccText)
		{
			LoadString(e_hInstance, IDS_CHECKDIST_ERROR, szText, countof(szText));
			MessageBox(szText, e_pszTpeAppName, MB_ICONINFORMATION | MB_OK);
			m_jCkeckDist.SetSel(0);
			m_jCkeckDist.SetFocus();
			return;
		}

		m_iInitDist = _ttoi(szText);
		m_nDistUnit = m_jLargeBtn.GetCheck() ? IDC_CHECK_LARGE_UNIT : IDC_CHECK_SMALL_UNIT;
		if(e_pTPE->m_jOptionDlg.m_nMetricUnit == BST_CHECKED)
		{
			m_iCheckDist = m_iInitDist;
			if(m_nDistUnit == IDC_CHECK_LARGE_UNIT)
				m_iCheckDist *= 1000;
		}
		else
			m_iCheckDist = (int)(m_iInitDist * ((m_nDistUnit == IDC_CHECK_LARGE_UNIT) ?
				1609.344 : 0.3048));
	}
	CloseDialog(ID_OK);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CCheckPoiDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnCancel\n"));

	CloseDialog(ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CCheckPoiDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CCheckPoiDlg::OnInitDialog\n"));

	SCUINT snButtonId[] =
	{
		IDC_CHECK_SIMILAR_NAME,
		IDC_CHECK_NEAR_DIST,
		IDC_CHECK_EMPTY_COORD,
	};
	PCTSTR pszLargeUnit, pszSmallUnit;
	BOOL   bSimilarName, bNearDist, bEmptyCoord;
	int    i;
	UINT   nSimilarName, nNearDist, nEmptyCoord;
	ASSERT(countof(m_jButton) == countof(snButtonId));
	SetDialogIcon(*this, ID_POI_CHECK);
	i = 0;
	do
	{
		m_jButton[i] = GetChild(snButtonId[i]);
		i++;
	}
	while(i < countof(m_jButton));
	m_jCkeckDist = GetChild(IDC_CHECK_DIST);
	m_jCkeckDist.SetLimitText(m_eccCheckDist);
	SetChildInt(IDC_CHECK_DIST, m_iInitDist);
	m_jLargeBtn = GetChild(IDC_CHECK_LARGE_UNIT);
	m_jSmallBtn = GetChild(IDC_CHECK_SMALL_UNIT);
	if(e_pTPE->m_jOptionDlg.m_nMetricUnit == BST_CHECKED)
	{
		pszLargeUnit = e_pszDistKm;
		pszSmallUnit = e_pszDistM;
	}
	else
	{
		pszLargeUnit = e_pszDistMi;
		pszSmallUnit = e_pszDistFt;
	}
	m_jLargeBtn.SetWindowText(pszLargeUnit);
	m_jSmallBtn.SetWindowText(pszSmallUnit);
	CheckButton(IDC_CHECK_LARGE_UNIT, IDC_CHECK_SMALL_UNIT, m_nDistUnit);
	m_jOKBtn     = GetChild(ID_OK);
	bSimilarName = TRUE;
	nSimilarName = m_nButton[m_eSimilarName];
	bNearDist    = TRUE;
	nNearDist    = m_nButton[m_eNearDist];
	bEmptyCoord  = TRUE;
	nEmptyCoord  = m_nButton[m_eEmptyCoord];
	if(e_pTPE->m_fst == CTomPoiEdr::FST_SC)
	{
		bSimilarName = FALSE;
		nSimilarName = BST_UNCHECKED;
	}
	else if(e_pTPE->m_fst == CTomPoiEdr::FST_CPB)
	{
		bNearDist   = FALSE;
		nNearDist   = BST_UNCHECKED;
		bEmptyCoord = FALSE;
		nEmptyCoord = BST_UNCHECKED;
	}
	if(e_pTPE->m_jListView.GetItemCount() < 2)
	{
		bSimilarName = FALSE;
		nSimilarName = BST_UNCHECKED;
		bNearDist    = FALSE;
		nNearDist    = BST_UNCHECKED;
	}
	m_jButton[m_eSimilarName].EnableWindow(bSimilarName);
	m_nButton[m_eSimilarName] = nSimilarName;
	m_jButton[m_eNearDist].EnableWindow(bNearDist);
	m_nButton[m_eNearDist] = nNearDist;
	m_jButton[m_eEmptyCoord].EnableWindow(bEmptyCoord);
	m_nButton[m_eEmptyCoord] = nEmptyCoord;
	i                        = 0;
	do
	{
		m_jButton[i].SetCheck(m_nButton[i]);
		i++;
	}
	while(i < countof(m_jButton));
	EnableButtons();
	MoveDialogCenter();
	FadeInDialog(*this);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CCheckPoiDlg

BEGIN_MESSAGE_MAP(CCheckPoiDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_CHECK_SIMILAR_NAME, OnSimilarName)
		ON_CONTROL(IDC_CHECK_NEAR_DIST, OnNearDist)
		ON_CONTROL(IDC_CHECK_EMPTY_COORD, OnEmptyCoord)
		ON_CONTROL(ID_OK, OnOK)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_INITDIALOG(OnInitDialog)
END_MESSAGE_MAP()
