// CCustTbDlg.cpp - Customize Toolbar Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the Close button

void CCustTbDlg::EnableCloseButton()
{
	TRACE(_T("Entering CCustTbDlg::EnableCloseButton\n"));

	BOOL     bEnable;
	int      iButton;
	TBBUTTON tbb;
	iButton = e_pTPE->m_jToolbar.ButtonCount();
	do
	{
		bEnable = iButton;
		if(!bEnable)
			break;
		iButton--;
		e_pTPE->m_jToolbar.GetButton(iButton, &tbb);
	}
	while(!tbb.idCommand);
	m_jCloseBtn.EnableWindow(bEnable);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Initializes the dialog box

void CCustTbDlg::InitDialog(HWND hDlg)
{
	TRACE(_T("Entering CCustTbDlg::InitDialog\n"));

	TCHAR      szText[32];
	int        yStatic, yEdit, xCtrl, cyCtrl;
	SIZE       size;
	RECT       rc;
	TEXTMETRIC tm;
	HFONT      hFont, hOldFont;
	HDC        hDC;
	CListBox   jListBox;
	CStatic    jStatic;
	CUpDown    jUpDown;
	m_hWnd  = hDlg;
	jStatic = GetChild(m_eAvailStcId);
	jStatic.GetWindowRect(&rc);
	::MapWindowPoints(*this, (PPOINT)&rc);
	yStatic  = rc.top;
	jListBox = GetChild(m_eAvailLstId);
	jListBox.GetWindowRect(&rc);
	::MapWindowPoints(*this, (PPOINT)&rc);
	yEdit = rc.top;
	::GetWindowRect(GetChild(m_eAddBtnId), &rc);
	::MapWindowPoints(*this, (PPOINT)&rc);
	xCtrl = rc.left + m_exDelta;
	hDC   = jStatic.GetDC();
	hFont = jStatic.GetWindowFont();
	if(!hFont)
		hFont = GetStockFont(SYSTEM_FONT);
	SelectFont(hDC, hFont);
	GetTextMetrics(hDC, &tm);
	GetTextExtent(hDC, szText, LoadString(e_hInstance, IDS_CUST_TB_SPACING,
		szText, countof(szText)), &size);
	jStatic.ReleaseDC(hDC);
	cyCtrl = tm.tmHeight + tm.tmExternalLeading;
	jStatic.CreateWindow(szText, m_eStcStyle, xCtrl, yStatic,
		size.cx, cyCtrl, *this, IDC_STATIC);
	hOldFont = jStatic.GetWindowFont();
	jStatic.SetWindowFont(hFont);
	if(hOldFont)
		DeleteFont(hOldFont);
	jStatic.SetWindowPos(jListBox, UNUSED, UNUSED, UNUSED, UNUSED, SWP_NOMOVE | SWP_NOSIZE);
	szText[0] = m_cxSpacing + _T('0');
	szText[1] = CHR_NULL;
	m_jSpacingEdt.CreateWindow(m_eEdtExStyle, szText, m_eEdtStyle, xCtrl, yEdit,
		m_ecxSpacingEdt, cyCtrl + m_ecyBorder, *this, m_eSpacingEdtId);
	hOldFont = m_jSpacingEdt.GetWindowFont();
	m_jSpacingEdt.SetWindowFont(hFont);
	if(hOldFont)
		DeleteFont(hOldFont);
	m_jSpacingEdt.SetWindowPos(jStatic, UNUSED, UNUSED, UNUSED, UNUSED,
		SWP_NOMOVE | SWP_NOSIZE);
	jUpDown.CreateWindow(m_eUpdStyle, 0, 0, 0, 0, *this, m_eSpacingUpdId,
		m_jSpacingEdt, m_eMaxSpacing, m_eMinSpacing, m_cxSpacing);
	jUpDown.SetWindowPos(m_jSpacingEdt, UNUSED, UNUSED, UNUSED, UNUSED,
		SWP_NOMOVE | SWP_NOSIZE);
	m_jCloseBtn = GetChild(m_eCloseBtnId);
	EnableCloseButton();
	m_jResetBtn = GetChild(m_eResetBtnId);
	m_jResetBtn.EnableWindow(GetPrivateProfileString(CTomPoiEdr::m_szOptionSection,
		CTomPoiEdr::m_szTbButtonEntry, e_szNull, szText, 2, e_pszProfile));
	MoveDialogCenter();
	FadeInDialog(*this);
	m_wpPrev = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WndProc);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window procedure for Customize Toolbar dialog box

LRESULT CALLBACK CCustTbDlg::WndProc(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CCustTbDlg::WndProc\n"));

	CCustTbDlg* pCTD;
	pCTD = &e_pTPE->m_jCustTbDlg;
	if(nMsg == WM_COMMAND)
	{
		PINT      piSpacing;
		TCHAR     szText[2];
		int       cxSpacing;
		TBMETRICS tbm;
		CEdit*    pEdit;
		switch(LOWORD(wParam))
		{
		case m_eSpacingEdtId:
			pEdit     = &pCTD->m_jSpacingEdt;
			piSpacing = &pCTD->m_cxSpacing;
			switch(HIWORD(wParam))
			{
			case EN_CHANGE:
				if(!pEdit->GetWindowText(szText, countof(szText)))
					break;
				if(!IsValidNumber(*szText))
				{
					pCTD->SetChildInt(m_eSpacingEdtId, *piSpacing);
					pEdit->SetSel(0);
					break;
				}
				cxSpacing = *szText - _T('0');
				if(cxSpacing == *piSpacing)
					break;
				*piSpacing          = cxSpacing;
				tbm.cbSize          = sizeof(tbm);
				tbm.dwMask          = TBMF_BUTTONSPACING;
				tbm.cxButtonSpacing = cxSpacing;
				tbm.cyButtonSpacing = 0;
				e_pTPE->m_jToolbar.SetMetrics(&tbm);
				e_pTPE->m_jToolbar.MoveWindow(0, 0, 0, 0, FALSE);
				e_pTPE->ResizeWindow(TRUE);
				break;
			case EN_KILLFOCUS:
				if(!pEdit->GetWindowTextLength())
					pCTD->SetChildInt(m_eSpacingEdtId, *piSpacing);
				break;
			}
			break;
		case m_eCloseBtnId:
			FadeOutDialog(hWnd);
		}
	}
	return CallWindowProc(pCTD->m_wpPrev, hWnd, nMsg, wParam, lParam);
}
