// CDumpDeviceDlg.cpp - Dump Device Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CDumpDeviceDlg.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR CDumpDeviceDlg::m_szSection[]        = _T("Dump Device");
CTCHAR CDumpDeviceDlg::m_szTimestampEntry[] = _T("Append Timestamp");
CTCHAR CDumpDeviceDlg::m_szFolderEntry[]    = _T("Dest Folder");
CTCHAR CDumpDeviceDlg::m_szSizeEntry[]      = _T("Specify Size");
CTCHAR CDumpDeviceDlg::m_szWidthEntry[]     = _T("Image Width");
CTCHAR CDumpDeviceDlg::m_szHeightEntry[]    = _T("Image Height");
CTCHAR CDumpDeviceDlg::m_szPixelEntry[]     = _T("Image Pixel");
CTCHAR CDumpDeviceDlg::m_szRatioEntry[]     = _T("Keep Ratio");
CTCHAR CDumpDeviceDlg::m_szRotateEntry[]    = _T("Rotate Image");
CTCHAR CDumpDeviceDlg::m_szFormatEntry[]    = _T("Image Format");

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes the dialog

void CDumpDeviceDlg::CloseDialog(int iResult)
{
	TRACE(_T("Entering CDumpDeviceDlg::CloseDialog\n"));

	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Updates transformed information

void CDumpDeviceDlg::UpdateTransform()
{
	TRACE(_T("Entering CDumpDeviceDlg::UpdateTransform\n"));

	TCHAR szText[32], szTransform[countof(szText) * 2];
	int   iWidth, iHeight;
	iWidth  = m_jWidthEdt.GetWindowText(szText, countof(szText)) ? _ttoi(szText) : 0;
	iHeight = m_jHeightEdt.GetWindowText(szText, countof(szText)) ? _ttoi(szText) : 0;
	if(m_jPercentBtn.GetCheck() == BST_CHECKED)
	{
		iWidth  = (iWidth * m_iWidth + 50) / 100;
		iHeight = (iHeight * m_iHeight + 50) / 100;
	}
	LoadString(e_hInstance, IDS_GPS_TRANSFORM, szText, countof(szText));
	wsprintf(szTransform, szText, iWidth, iHeight);
	m_jTransformStc.SetWindowText(szTransform);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_SELECT_ALL control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnSelectAll(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnSelectAll\n"));

	LVITEM lvi;
	lvi.state     = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_DELETE_ALL control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnDeleteAll(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnDeleteAll\n"));

	TCHAR szMessage[MAX_STR];
	LoadString(e_hInstance, IDS_GPS_DELETE_ALL, szMessage, countof(szMessage));
	if(MessageBox(szMessage, e_pszTpeAppName, MB_ICONQUESTION | MB_OKCANCEL) == ID_OK)
		CloseDialog(IDC_GPS_DELETE_ALL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_REFRESH control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnRefresh(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnRefresh\n"));

	CloseDialog(IDC_GPS_REFRESH);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_FOLDER control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnFolder(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnFolder\n"));

	if((nNotify != EN_CHANGE) && (nNotify != EN_KILLFOCUS))
		return;

	TCHAR szFolder[MAX_PATH];
	m_ccFolder = m_jFolderEdt.GetWindowText(szFolder, countof(szFolder));
	if(m_ccFolder)
		m_ccFolder = TrimString(szFolder);
	if(nNotify == EN_CHANGE)
		SetTimer(m_eTimerId, m_eTimerIntl);
	else
		m_jFolderEdt.SetWindowText(szFolder);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_BROWSE control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnBrowse(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnBrowse\n"));

	TCHAR szFolder[MAX_PATH], szDispName[countof(szFolder)], szTitle[MAX_STR];
	if(m_jFolderEdt.GetWindowText(szFolder, MAX_PATH) && !IsValidDirectory(szFolder))
		*szFolder = CHR_NULL;
	LoadString(e_hInstance, IDS_DFD_TITLE, szTitle, countof(szTitle));
	m_jDumpFolderDlg.hwndOwner      = *this;
	m_jDumpFolderDlg.pszDisplayName = szDispName;
	m_jDumpFolderDlg.lpszTitle      = szTitle;
	if(m_jDumpFolderDlg.Browse(szFolder))
		m_jFolderEdt.SetWindowText(szFolder);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_SPECIFY_SIZE control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnSpecifySize(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnSpecifySize\n"));

	BOOL bCheck;
	bCheck = m_jSizeBtn.GetCheck() == BST_CHECKED;
	m_jWidthStc.EnableWindow(bCheck);
	m_jWidthEdt.EnableWindow(bCheck);
	m_jWidthSpin.EnableWindow(bCheck);
	m_jHeightStc.EnableWindow(bCheck);
	m_jHeightEdt.EnableWindow(bCheck);
	m_jHeightSpin.EnableWindow(bCheck);
	m_jPixelBtn.EnableWindow(bCheck);
	m_jPercentBtn.EnableWindow(bCheck);
	m_jRatioBtn.EnableWindow(bCheck);
	if(bCheck)
		return;

	m_jWidthSpin.SetRange(m_ecxMaxPixel, m_ecxMinPixel);
	m_jWidthSpin.SetPos(m_iWidth);
	m_jHeightSpin.SetRange(m_ecyMaxPixel, m_ecyMinPixel);
	m_jHeightSpin.SetPos(m_iHeight);
	CheckButton(IDC_GPS_PIXEL, IDC_GPS_PERCENT, IDC_GPS_PIXEL);
	m_bPixel = TRUE;
	m_jRatioBtn.SetCheck(BST_UNCHECKED);
	UpdateTransform();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_WIDTH_BUDDY or IDC_GPS_HEIGHT_BUDDY control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnBuddy(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnBuddy\n"));

	if((nNotify != EN_CHANGE) || (GetFocus() != hCtrlWnd))
		return;

	TCHAR szText[32];
	int   iAlpha, iDelta;
	CEdit jBuddyEdt, jPartnerEdt;
	if(hCtrlWnd == m_jWidthEdt)
	{
		jBuddyEdt   = m_jWidthEdt;
		jPartnerEdt = m_jHeightEdt;
		iAlpha      = m_iWidth;
		iDelta      = m_iHeight;
	}
	else
	{
		jBuddyEdt   = m_jHeightEdt;
		jPartnerEdt = m_jWidthEdt;
		iAlpha      = m_iHeight;
		iDelta      = m_iWidth;
	}
	if(jBuddyEdt.GetWindowText(szText, countof(szText)))
	{
		PTCHAR pcSrc, pcDest;
		pcSrc  = szText;
		pcDest = pcSrc;
		do
		{
			if(IsValidNumber(*pcSrc))
			{
				*pcDest = *pcSrc;
				pcDest++;
			}
			pcSrc++;
		}
		while(*pcSrc);
		if(pcSrc != pcDest)
		{
			*pcDest = CHR_NULL;
			jBuddyEdt.SetWindowText(szText);
			return;
		}
	}
	if(m_jRatioBtn.GetCheck() == BST_CHECKED)
	{
		if(*szText && m_bPixel)
			_ltot((_ttoi(szText) * iDelta * 10 / iAlpha + 5) / 10, szText);
		jPartnerEdt.SetWindowText(szText);
	}
	UpdateTransform();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_PIXEL control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnPixel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnPixel\n"));

	if(m_bPixel)
		return;

	TCHAR szText[32];
	int   ccText, iPercent;
	m_bPixel = TRUE;
	m_jWidthSpin.SetRange(m_ecxMaxPixel, m_ecxMinPixel);
	m_jHeightSpin.SetRange(m_ecyMaxPixel, m_ecyMinPixel);
	ccText = m_jWidthEdt.GetWindowText(szText, countof(szText));
	if(ccText)
	{
		iPercent = _ttoi(szText);
		SetChildInt(IDC_GPS_WIDTH_BUDDY, (iPercent * m_iWidth + 50) / 100);
	}
	if(m_jRatioBtn.GetCheck() == BST_CHECKED)
	{
		if(!ccText)
			return;
	}
	else
	{
		if(!m_jHeightEdt.GetWindowText(szText, countof(szText)))
			return;
		iPercent = _ttoi(szText);
	}
	SetChildInt(IDC_GPS_HEIGHT_BUDDY, (iPercent * m_iHeight + 50) / 100);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_PERCENT control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnPercent(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnPercent\n"));

	if(!m_bPixel)
		return;

	TCHAR szText[32];
	int   ccText, iPercent;
	m_bPixel = FALSE;
	m_jWidthSpin.SetRange(m_ecxMaxPercent, m_ecxMinPercent);
	m_jHeightSpin.SetRange(m_ecyMaxPercent, m_ecyMinPercent);
	ccText = m_jWidthEdt.GetWindowText(szText, countof(szText));
	if(ccText)
	{
		iPercent = (_ttoi(szText) * 1000 / m_iWidth + 5) / 10;
		SetChildInt(IDC_GPS_WIDTH_BUDDY, iPercent);
	}
	if(m_jRatioBtn.GetCheck() == BST_CHECKED)
	{
		if(!ccText)
			iPercent = MIN_INT;
	}
	else
		iPercent = m_jHeightEdt.GetWindowText(szText, countof(szText)) ?
			((_ttoi(szText) * 1000 / m_iHeight + 5) / 10) : MIN_INT;
	if(iPercent != MIN_INT)
		SetChildInt(IDC_GPS_HEIGHT_BUDDY, iPercent);
	UpdateTransform();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GPS_KEEP_RATIO control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnKeepRatio(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnKeepRatio\n"));

	if(m_jRatioBtn.GetCheck() == BST_UNCHECKED)
		return;

	TCHAR szText[32];
	int   ccText, iAlpha, iDelta, iValue;
	UINT  nBuddyId;
	ccText = m_jWidthEdt.GetWindowText(szText, countof(szText));
	if(ccText)
	{
		nBuddyId = IDC_GPS_HEIGHT_BUDDY;
		iAlpha   = m_iWidth;
		iDelta   = m_iHeight;
	}
	else
	{
		ccText = m_jHeightEdt.GetWindowText(szText, countof(szText));
		if(!ccText)
			return;
		nBuddyId = IDC_GPS_WIDTH_BUDDY;
		iAlpha   = m_iHeight;
		iDelta   = m_iWidth;
	}
	iValue = _ttoi(szText);
	SetChildInt(nBuddyId, m_bPixel ? ((iValue * iDelta * 10 / iAlpha + 5) / 10) : iValue);
	UpdateTransform();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_OK control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnOK(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnOK\n"));

	TCHAR szMessage[MAX_STR];
	m_jFolderEdt.GetWindowText(m_pszFolder, MAX_PATH);
	if(!IsValidDirectory(m_pszFolder))
	{
		int iResult;
		LoadString(e_hInstance, IDS_GPS_FOLDER_ERROR, szMessage, countof(szMessage));
		iResult = MessageBox(szMessage, e_pszTpeAppName, MB_ICONINFORMATION | MB_OKCANCEL);
		if(iResult == ID_OK)
		{
			if(SHCreateDirectory(NULL, m_pszFolder) != ERROR_SUCCESS)
			{
				LoadString(e_hInstance, IDS_GPS_CREATE_FOLDER_ERROR,
					szMessage, countof(szMessage));
				MessageBox(szMessage, e_pszTpeAppName, MB_ICONINFORMATION | MB_OK);
				iResult = ID_CANCEL;
			}
		}
		if(iResult == ID_CANCEL)
		{
			m_jFolderEdt.SetSel(0);
			m_jFolderEdt.SetFocus();
			return;
		}
	}

	TCHAR szText[MAX_STR];
	BOOL  bError;
	int   iWidth, iHeight, iMaxPixel, iMinPixel, iMaxPercent, iMinPercent;
	CEdit jBuddyEdt;
	iWidth = m_jWidthSpin.GetPos(&bError);
	if(bError)
	{
		jBuddyEdt   = m_jWidthEdt;
		iMaxPixel   = m_ecxMaxPixel;
		iMinPixel   = m_ecxMinPixel;
		iMaxPercent = m_ecxMaxPercent;
		iMinPercent = m_ecxMinPercent;
	}
	else
	{
		iHeight = m_jHeightSpin.GetPos(&bError);
		if(bError)
		{
			jBuddyEdt   = m_jHeightEdt;
			iMaxPixel   = m_ecyMaxPixel;
			iMinPixel   = m_ecyMinPixel;
			iMaxPercent = m_ecyMaxPercent;
			iMinPercent = m_ecyMinPercent;
		}
	}
	if(bError)
	{
		int  iMaximum, iMinimum;
		UINT nResId;
		if(m_bPixel)
		{
			nResId   = IDS_GPS_PIXEL_ERROR;
			iMaximum = iMaxPixel;
			iMinimum = iMinPixel;
		}
		else
		{
			nResId   = IDS_GPS_PERCENT_ERROR;
			iMaximum = iMaxPercent;
			iMinimum = iMinPercent;
		}
		LoadString(e_hInstance, nResId, szText, countof(szText));
		wsprintf(szMessage, szText, iMinimum, iMaximum);
		MessageBox(szMessage, e_pszTpeAppName, MB_ICONINFORMATION | MB_OK);
		jBuddyEdt.SetSel(0);
		jBuddyEdt.SetFocus();
		return;
	}

	if(m_jListView.GetItemCount() != m_jListView.GetSelCount())
	{
		PGPSFILE   pGPSF, pPrevGPSF, pNextGPSF;
		int        iHitItem;
		LVFINDINFO lvfi;
		pGPSF      = *m_ppGPSF;
		pPrevGPSF  = NULL;
		lvfi.flags = LVFI_STRING;
		ASSERT(pGPSF);
		do
		{
			lvfi.psz  = pGPSF->pszName;
			pNextGPSF = pGPSF->pNext;
			iHitItem  = m_jListView.FindItem(&lvfi);
			ASSERT(iHitItem != RV_ERR);
			if(m_jListView.GetItemState(iHitItem, LVIS_SELECTED))
				pPrevGPSF = pGPSF;
			else
			{
				free(pGPSF);
				if(pPrevGPSF)
					pPrevGPSF->pNext = pNextGPSF;
				else
					*m_ppGPSF = pNextGPSF;
			}
			pGPSF = pNextGPSF;
		}
		while(pGPSF);
	}
	WritePrivateProfileString(m_szSection, m_szFolderEntry, m_pszFolder, e_pszProfile);
	m_jWidthEdt.GetWindowText(szText, countof(szText));
	WritePrivateProfileString(m_szSection, m_szWidthEntry, szText, e_pszProfile);
	m_jHeightEdt.GetWindowText(szText, countof(szText));
	WritePrivateProfileString(m_szSection, m_szHeightEntry, szText, e_pszProfile);
	*m_pbTimestamp = m_jTimestamp.GetCheck() == BST_CHECKED;
	szText[0]      = *m_pbTimestamp + _T('0');
	szText[1]      = CHR_NULL;
	WritePrivateProfileString(m_szSection, m_szTimestampEntry, szText, e_pszProfile);
	szText[0] = (m_jSizeBtn.GetCheck() == BST_CHECKED) + _T('0');
	WritePrivateProfileString(m_szSection, m_szSizeEntry, szText, e_pszProfile);
	szText[0] = m_bPixel + _T('0');
	WritePrivateProfileString(m_szSection, m_szPixelEntry, szText, e_pszProfile);
	szText[0] = (m_jRatioBtn.GetCheck() == BST_CHECKED) + _T('0');
	WritePrivateProfileString(m_szSection, m_szRatioEntry, szText, e_pszProfile);
	if(!m_bPixel)
	{
		iWidth  = (iWidth * m_iWidth + 50) / 100;
		iHeight = (iHeight * m_iHeight + 50) / 100;
	}
	if((iWidth == m_iWidth) && (iHeight == m_iHeight))
		m_pSize->cx = 0;
	else
	{
		m_pSize->cx = iWidth;
		m_pSize->cy = iHeight;
	}
	*m_piRotate = m_jRotateCmb.GetCurSel();
	szText[0]   = *m_piRotate + _T('0');
	WritePrivateProfileString(m_szSection, m_szRotateEntry, szText, e_pszProfile);
	*m_piFormat = m_jFormatCmb.GetCurSel();
	szText[0]   = *m_piFormat + _T('0');
	WritePrivateProfileString(m_szSection, m_szFormatEntry, szText, e_pszProfile);
	CloseDialog(ID_OK);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CDumpDeviceDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnCancel\n"));

	CloseDialog(ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CDumpDeviceDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnInitDialog\n"));

	PTCHAR   pcStart, pcEnd;
	PGPSFILE pGPSF;
	TCHAR    szFolder[MAX_PATH], szText[m_eccDateCol];
	TCHAR    szToken3[32], szToken4[countof(szToken3)];
	BOOL     bTimestamp, bDeleteAll;
	int      i, ccToken3, ccToken4, ceLvItem, iLvItem;
	DWORD    dwLvStyle;
	LVCOLUMN lvc;
	LVITEM   lvi;
	ASSERT((e_pTPE->m_gpsd == GPSD_V3) || (e_pTPE->m_gpsd == GPSD_V4));
	SetDialogIcon(*this, ID_FILE_DUMP_DEVICE);
	m_jListView     = GetChild(IDC_GPS_DUMP_LV);
	m_jFolderEdt    = GetChild(IDC_GPS_FOLDER);
	m_jTimestamp    = GetChild(IDC_GPS_TIMESTAMP);
	m_jSizeBtn      = GetChild(IDC_GPS_SPECIFY_SIZE);
	m_jWidthStc     = GetChild(IDC_GPS_WIDTH);
	m_jWidthEdt     = GetChild(IDC_GPS_WIDTH_BUDDY);
	m_jWidthSpin    = GetChild(IDC_GPS_WIDTH_SPIN);
	m_jHeightStc    = GetChild(IDC_GPS_HEIGHT);
	m_jHeightEdt    = GetChild(IDC_GPS_HEIGHT_BUDDY);
	m_jHeightSpin   = GetChild(IDC_GPS_HEIGHT_SPIN);
	m_jPixelBtn     = GetChild(IDC_GPS_PIXEL);
	m_jPercentBtn   = GetChild(IDC_GPS_PERCENT);
	m_jRatioBtn     = GetChild(IDC_GPS_KEEP_RATIO);
	m_jTransformStc = GetChild(IDC_GPS_TRANSFORM);
	m_jRotateCmb    = GetChild(IDC_GPS_ROTATE);
	m_jFormatCmb    = GetChild(IDC_GPS_FORMAT);
	m_jOKBtn        = GetChild(ID_OK);
	if(e_dwWinVer > WV_XP)
	{
		m_jListView.SetWindowStyle(m_jListView.GetWindowStyle() | LVS_SHOWSELALWAYS);
		dwLvStyle = m_eLvExStyle;
	}
	else
		dwLvStyle = m_eLvExStyleXP;
	m_jListView.SetExStyle(dwLvStyle, dwLvStyle);
	m_jListView.SetWindowTheme(e_szAppTheme);
	m_jListView.SetTextColor(e_pTPE->m_jListView.GetTextColor());
	m_jListView.SetTextBkColor(GetSysColor(COLOR_WINDOW));
	ccToken3    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN3, szToken3, countof(szToken3));
	ccToken4    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN4, szToken4, countof(szToken4));
	lvc.mask    = LVCF_TEXT;
	lvc.pszText = szText;
	i           = 0;
	do
	{
		LoadString(e_hInstance, IDS_GPS_FIRST_COL + i, szText, countof(szText));
		m_jListView.InsertColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	lvi.mask = LVIF_TEXT;
	pGPSF    = *m_ppGPSF;
	while(pGPSF)
	{
		lvi.iItem    = 0;
		lvi.iSubItem = 0;
		lvi.pszText  = pGPSF->pszName;
		iLvItem      = m_jListView.InsertItem(&lvi);
		pcStart      = pGPSF->pszAttr;
		while(pcStart = StrStrI(pcStart, szToken3))
		{
			pcStart += ccToken3;
			if(*pcStart != CHR_LT)
				break;
		}
		if(pcStart)
		{
			pcEnd = StrStrI(pcStart, szToken4);
			if(pcEnd)
			{
				i = pcEnd - pcStart;
				if(i < m_eccDateCol)
				{
					CopyMemory(szText, pcStart, i * sizeof(TCHAR));
					szText[i] = CHR_NULL;
					e_pTPE->DecodeEntity(szText);
					CopyMemory(pGPSF->pszAttr, szText,
						(TrimString(szText) + 1) * sizeof(TCHAR));
					lvi.iSubItem = LVC_DATE;
					lvi.pszText  = szText;
					m_jListView.SetItemText(iLvItem, &lvi);
				}
			}
		}
		pGPSF = pGPSF->pNext;
	}
	lvc.mask = LVCF_WIDTH;
	lvc.cx   = 0;
	m_jListView.InsertColumn(LVC_COUNT, &lvc);
	i = LVC_FIRST;
	do
	{
		m_jListView.SetColumnWidth(i, LVSCW_AUTOSIZE_USEHEADER);
		i++;
	}
	while(i < LVC_COUNT);
	m_jListView.DeleteColumn(LVC_COUNT);
	ceLvItem = m_jListView.GetItemCount();
	if(e_pTPE->m_gpsd == GPSD_V3)
	{
		bTimestamp = FALSE;
		bDeleteAll = ceLvItem;
	}
	else
	{
		bTimestamp = TRUE;
		bDeleteAll = FALSE;
		if(GetPrivateProfileInt(m_szSection, m_szTimestampEntry, FALSE, e_pszProfile))
			m_jTimestamp.SetCheck();
	}
	m_jTimestamp.EnableWindow(bTimestamp);
	::EnableWindow(GetChild(IDC_GPS_SELECT_ALL), ceLvItem);
	::EnableWindow(GetChild(IDC_GPS_DELETE_ALL), bDeleteAll);
	m_ccFolder = GetPrivateProfileString(m_szSection, m_szFolderEntry, e_szNull,
		szFolder, countof(szFolder), e_pszProfile);
	if(m_ccFolder)
	{
		m_ccFolder = TrimString(szFolder);
		if(m_ccFolder)
		{
			if(IsValidDirectory(szFolder))
				m_jFolderEdt.SetWindowText(szFolder);
			else
				m_ccFolder = 0;
		}
	}
	m_jFolderEdt.SetLimitText(MAX_PATH);
	m_iWidth  = m_ecxDefPixel;
	m_iHeight = m_ecyDefPixel;
	if(ceLvItem)
	{
		TCHAR szTempFile[MAX_PATH];
		m_jListView.EnsureVisible(ceLvItem - 1);
		if(!e_pTPE->GpsGetFile(m_hInternet, FALSE, (*m_ppGPSF)->pszName, FALSE, szTempFile))
		{
			if(e_pTPE->m_gpsd == GPSD_V3)
			{
				HFILE hFile;
				hFile = CreateFile(szTempFile, GENERIC_READ, OPEN_EXISTING);
				if(hFile != INVALID_HANDLE)
				{
					char  cHeader[sizeof(BITMAPFILEHEADER) + sizeof(BITMAPCOREHEADER)];
					DWORD cbRead;
					if(ReadFile(hFile, cHeader, sizeof(cHeader), &cbRead) &&
						(cbRead == sizeof(cHeader)))
					{
						PBITMAPINFOHEADER pBMIH;
						pBMIH = (PBITMAPINFOHEADER)(cHeader + sizeof(BITMAPFILEHEADER));
						if(pBMIH->biSize == sizeof(BITMAPCOREHEADER))
						{
							PBITMAPCOREHEADER pBMCH;
							pBMCH     = (PBITMAPCOREHEADER)pBMIH;
							m_iWidth  = pBMCH->bcWidth;
							m_iHeight = pBMCH->bcHeight;
						}
						else
						{
							ASSERT((pBMIH->biSize == sizeof(BITMAPINFOHEADER)) ||
								(pBMIH->biSize == sizeof(BITMAPV4HEADER)) ||
								(pBMIH->biSize == sizeof(BITMAPV5HEADER)));
							m_iWidth  = pBMIH->biWidth;
							m_iHeight = pBMIH->biHeight;
						}
					}
					CloseHandle(hFile);
				}
			}
			else
			{
				PIMAGE              pImage;
				ULONG_PTR           gdipt;
				GdiplusStartupInput gdipsi;
				GdiplusStartup(&gdipt, &gdipsi, NULL);
				pImage    = Image::FromFile(szTempFile);
				m_iWidth  = pImage->GetWidth();
				m_iHeight = pImage->GetHeight();
				delete pImage;
				GdiplusShutdown(gdipt);
			}
			DeleteFile(szTempFile);
		}
	}
	m_jWidthEdt.SetLimitText(m_eccMaxWidth);
	m_jHeightEdt.SetLimitText(m_eccMaxHeight);
	if(GetPrivateProfileInt(m_szSection, m_szSizeEntry, FALSE, e_pszProfile))
	{
		int     cxMaximum, cxMinimum, iWidth, cyMaximum, cyMinimum, iHeight;
		CButton jRadioBtn;
		m_jSizeBtn.SetCheck();
		m_bPixel = GetPrivateProfileInt(m_szSection, m_szPixelEntry, TRUE, e_pszProfile);
		if(m_bPixel)
		{
			jRadioBtn = m_jPixelBtn;
			cxMaximum = m_ecxMaxPixel;
			cxMinimum = m_ecxMinPixel;
			iWidth    = m_ecxDefPixel;
			cyMaximum = m_ecyMaxPixel;
			cyMinimum = m_ecyMinPixel;
			iHeight   = m_ecyDefPixel;
		}
		else
		{
			jRadioBtn = m_jPercentBtn;
			cxMaximum = m_ecxMaxPercent;
			cxMinimum = m_ecxMinPercent;
			iWidth    = m_ecxDefPercent;
			cyMaximum = m_ecyMaxPercent;
			cyMinimum = m_ecyMinPercent;
			iHeight   = m_ecyDefPercent;
		}
		m_jWidthSpin.SetRange(cxMaximum, cxMinimum);
		m_jHeightSpin.SetRange(cyMaximum, cyMinimum);
		iWidth = clamp(GetPrivateProfileInt(m_szSection, m_szWidthEntry,
			iWidth, e_pszProfile), cxMinimum, cxMaximum);
		if(GetPrivateProfileInt(m_szSection, m_szRatioEntry, FALSE, e_pszProfile))
		{
			m_jRatioBtn.SetCheck();
			iHeight = m_bPixel ? ((iWidth * m_iHeight * 10 / m_iWidth + 5) / 10) : iWidth;
		}
		else
			iHeight = clamp(GetPrivateProfileInt(m_szSection, m_szHeightEntry,
				iHeight, e_pszProfile), cyMinimum, cyMaximum);
		m_jWidthSpin.SetPos(iWidth);
		m_jHeightSpin.SetPos(iHeight);
		jRadioBtn.SetCheck();
		UpdateTransform();
	}
	else
		SendMessage(WM_COMMAND, MAKEWPARAM(IDC_GPS_SPECIFY_SIZE, BN_CLICKED));
	i = 0;
	do
	{
		_ltot(i, szText);
		m_jRotateCmb.AddItem(szText);
		i += 90;
	}
	while(i < 360);
	m_jRotateCmb.SetCurSel(clamp(GetPrivateProfileInt(m_szSection, m_szRotateEntry,
		0, e_pszProfile), 0, 3));
	i = IDS_GPS_IMAGE_FIRST;
	do
	{
		LoadString(e_hInstance, i, szText, countof(szText));
		m_jFormatCmb.AddItem(szText);
		i++;
	}
	while(i <= IDS_GPS_IMAGE_LAST);
	m_jFormatCmb.SetCurSel(clamp(GetPrivateProfileInt(m_szSection, m_szFormatEntry,
		IF_JPEG, e_pszProfile), IF_FIRST, IF_COUNT));
	MoveDialogCenter();
	FadeInDialog(*this);
	SetTimer(m_eTimerId, m_eTimerIntl);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes of an item in a list view have changed

AFXINLINE void CDumpDeviceDlg::OnLvnItemChanged(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnLvnItemChanged\n"));

	ASSERT(pNMLV->uChanged & LVIF_STATE);
	SetTimer(m_eTimerId, m_eTimerIntl);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view is about to be drawn

AFXINLINE UINT CDumpDeviceDlg::OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnNmCustomDraw\n"));

	// Result value of the message processed
	return ((pNMCD->hdr.idFrom == IDC_GPS_DUMP_LV) ?
		CTomPoiEdr::CustomDrawListView((PNMLVCUSTOMDRAW)pNMCD) : CDRF_DODEFAULT);

	return CDRF_DODEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A timer interval expires

AFXINLINE void CDumpDeviceDlg::OnTimer(UINT nTimerId)
{
	TRACE(_T("Entering CDumpDeviceDlg::OnTimer\n"));

	BOOL bEnable;
	bEnable = m_ccFolder ? m_jListView.GetSelCount() : FALSE;
	if(bEnable != m_jOKBtn.IsWindowEnabled())
		m_jOKBtn.EnableWindow(bEnable);
	KillTimer(m_eTimerId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CDumpDeviceDlg

BEGIN_MESSAGE_MAP(CDumpDeviceDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_GPS_SELECT_ALL, OnSelectAll)
		ON_CONTROL(IDC_GPS_DELETE_ALL, OnDeleteAll)
		ON_CONTROL(IDC_GPS_REFRESH, OnRefresh)
		ON_CONTROL(IDC_GPS_FOLDER, OnFolder)
		ON_CONTROL(IDC_GPS_BROWSE, OnBrowse)
		ON_CONTROL(IDC_GPS_SPECIFY_SIZE, OnSpecifySize)
		ON_SAME_MAP(IDC_GPS_WIDTH_BUDDY)
		ON_CONTROL(IDC_GPS_HEIGHT_BUDDY, OnBuddy)
		ON_CONTROL(IDC_GPS_PIXEL, OnPixel)
		ON_CONTROL(IDC_GPS_PERCENT, OnPercent)
		ON_CONTROL(IDC_GPS_KEEP_RATIO, OnKeepRatio)
		ON_CONTROL(ID_OK, OnOK)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_INITDIALOG(OnInitDialog)
	BEGIN_NOTIFY_MAP()
		ON_LVN_ITEMCHANGED(OnLvnItemChanged)
		ON_NM_CUSTOMDRAW(OnNmCustomDraw)
	END_NOTIFY_MAP()
	ON_WM_TIMER(OnTimer)
END_MESSAGE_MAP()
