// CDumpDeviceDlg.hpp - Dump Device Dialog Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CDUMPDEVICEDLG_INCLUDED_
#define _CDUMPDEVICEDLG_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CDumpDeviceDlg - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

#include "CDumpFolderDlg.hpp"
#include "CTomPoiEdr.hpp"

class CDumpDeviceDlg : public CDialog
{
public:
	enum
	{
		// List view extended styles
		m_eLvExStyle   = LVS_EX_DOUBLEBUFFER | LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP,
		m_eLvExStyleXP = m_eLvExStyle | LVS_EX_GRIDLINES,

		// List view columns
		LVC_FIRST = 0,
		LVC_NAME  = LVC_FIRST,
		LVC_DATE,
		LVC_COUNT,

		// Image formats
		IF_FIRST = 0,
		IF_JPEG  = IF_FIRST,
		IF_GIF,
		IF_PNG,
		IF_TIFF,
		IF_BMP,
		IF_COUNT,

		// Maximum number of characters, including null character
		m_eccNameCol = MAX_FNAME,
		m_eccDateCol = 64,

		// Maximum number of characters can be type in
		m_eccMaxWidth  = 4,
		m_eccMaxHeight = m_eccMaxWidth,

		// Maximum and minimum pixels and percents
		m_ecxMaxPixel   = 4096,
		m_ecxMinPixel   = 16,
		m_ecxDefPixel   = 480,
		m_ecyMaxPixel   = m_ecxMaxPixel,
		m_ecyMinPixel   = m_ecxMinPixel,
		m_ecyDefPixel   = 272,
		m_ecxMaxPercent = 1000,
		m_ecxMinPercent = 10,
		m_ecxDefPercent = 100,
		m_ecyMaxPercent = m_ecxMaxPercent,
		m_ecyMinPercent = m_ecxMinPercent,
		m_ecyDefPercent = 100,

		// Timer identifier and its interval
		m_eTimerId   = 7,
		m_eTimerIntl = 100,
	};

public:
	PTSTR          m_pszFolder;
	PBOOL          m_pbTimestamp;
	PINT           m_piRotate, m_piFormat;
	PSIZE          m_pSize;
	PPGPSFILE      m_ppGPSF;
	BOOL           m_bPixel;
	int            m_ccFolder, m_iWidth, m_iHeight;
	HINTERNET      m_hInternet;
	CButton        m_jTimestamp, m_jSizeBtn, m_jPixelBtn, m_jPercentBtn, m_jRatioBtn, m_jOKBtn;
	CComboBox      m_jRotateCmb, m_jFormatCmb;
	CEdit          m_jFolderEdt, m_jWidthEdt, m_jHeightEdt;
	CListView      m_jListView;
	CStatic        m_jWidthStc, m_jHeightStc, m_jTransformStc;
	CUpDown        m_jWidthSpin, m_jHeightSpin;
	CDumpFolderDlg m_jDumpFolderDlg;

public:
	static CTCHAR m_szSection[];
	static CTCHAR m_szTimestampEntry[];
	static CTCHAR m_szFolderEntry[];
	static CTCHAR m_szSizeEntry[];
	static CTCHAR m_szWidthEntry[];
	static CTCHAR m_szHeightEntry[];
	static CTCHAR m_szPixelEntry[];
	static CTCHAR m_szRatioEntry[];
	static CTCHAR m_szRotateEntry[];
	static CTCHAR m_szFormatEntry[];

public:
	CDumpDeviceDlg(HINTERNET hInternet, PPGPSFILE ppGPSF, PTSTR pszFolder,
		PBOOL pbTimestamp, PSIZE pSize, PINT piRotate, PINT piFormat);
	void CloseDialog(int iResult);
	void UpdateTransform();

protected:
	void OnSelectAll(UINT nNotify, HWND hCtrlWnd);
	void OnDeleteAll(UINT nNotify, HWND hCtrlWnd);
	void OnRefresh(UINT nNotify, HWND hCtrlWnd);
	void OnFolder(UINT nNotify, HWND hCtrlWnd);
	void OnBrowse(UINT nNotify, HWND hCtrlWnd);
	void OnSpecifySize(UINT nNotify, HWND hCtrlWnd);
	void OnBuddy(UINT nNotify, HWND hCtrlWnd);
	void OnPixel(UINT nNotify, HWND hCtrlWnd);
	void OnPercent(UINT nNotify, HWND hCtrlWnd);
	void OnKeepRatio(UINT nNotify, HWND hCtrlWnd);
	void OnOK(UINT nNotify, HWND hCtrlWnd);
	void OnCancel(UINT nNotify, HWND hCtrlWnd);
	BOOL OnInitDialog(HWND hFocusWnd);
	void OnLvnItemChanged(PCNMLISTVIEW pNMLV);
	UINT OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD);
	void OnTimer(UINT nTimerId);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CDumpDeviceDlg::CDumpDeviceDlg(HINTERNET hInternet, PPGPSFILE ppGPSF, PTSTR pszFolder,
	PBOOL pbTimestamp, PSIZE pSize, PINT piRotate, PINT piFormat)
{
	TRACE(_T("Entering CDumpDeviceDlg::CDumpDeviceDlg\n"));

	m_hInternet   = hInternet;
	m_ppGPSF      = ppGPSF;
	m_pszFolder   = pszFolder;
	m_pbTimestamp = pbTimestamp;
	m_pSize       = pSize;
	m_piRotate    = piRotate;
	m_piFormat    = piFormat;
}

// End of _CDUMPDEVICEDLG_INCLUDED_
#endif
