// CDumpFolderDlg.cpp - Dump Folder Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CDumpFolderDlg.hpp"
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

WNDPROC CDumpFolderDlg::m_wpPrev;

////////////////////////////////////////////////////////////////////////////////////////////////
// Initializes Browse Folder dialog box

AFXINLINE void CDumpFolderDlg::Initialized()
{
	TRACE(_T("Entering CDumpFolderDlg::Initialized\n"));

	TCHAR      szText[MAX_PATH + 64];
	int        xButton, yButton, cxButton, cyButton, yTvCtrl;
	RECT       rc;
	TEXTMETRIC tm;
	HFONT      hFont;
	HDC        hDC;
	CButton    jButton;
	CTreeView  jTreeView;
	LoadString(e_hInstance, IDS_DFD_NAME, szText, countof(szText));
	SetWindowText(szText);
	m_jTitleStc = GetChild(IDC_BFD_STATIC1);
	m_jTitleStc.SetWindowStyle(m_jTitleStc.GetWindowStyle() | SS_PATHELLIPSIS);
	hDC   = m_jTitleStc.GetDC();
	hFont = m_jTitleStc.GetWindowFont();
	if(!hFont)
		hFont = GetStockFont(SYSTEM_FONT);
	SelectFont(hDC, hFont);
	GetTextMetrics(hDC, &tm);
	m_jTitleStc.ReleaseDC(hDC);
	m_jTitleStc.MoveWindow(m_ecxBorder, m_ecyBorder,
		m_ecxTvCtrl, tm.tmHeight + tm.tmExternalLeading);
	jButton = GetChild(ID_CANCEL);
	jButton.SetWindowStyle(jButton.GetWindowStyle() & ~WS_GROUP);
	jButton.GetWindowRect(&rc);
	cxButton = GetRectWidth(&rc);
	cyButton = GetRectHeight(&rc);
	xButton  = m_ecxDialog - cxButton - m_ecxBorder + 1;
	yButton  = m_ecyDialog - cyButton - m_ecyBorder;
	jButton.SetWindowPos(NULL, xButton, yButton, UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	jButton = GetChild(ID_OK);
	jButton.SetWindowPos(NULL, xButton - cxButton - m_ecxGap, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	yTvCtrl   = tm.tmHeight + m_ecyBorder + m_ecyGap;
	jTreeView = GetChild(IDC_BFD_TREE_VIEW);
	jTreeView.SetWindowStyle(jTreeView.GetWindowStyle() |
		((e_dwWinVer > WV_XP) ? m_eTvStyle : m_eTvStyleXP));
	jTreeView.MoveWindow(m_ecxBorder, yTvCtrl, m_ecxTvCtrl, yButton - yTvCtrl - m_ecyGap);
	SetRect(&rc, 0, 0, m_ecxDialog, m_ecyDialog);
	AdjustWindowRect(&rc, GetWindowStyle(), FALSE);
	MoveWindow(0, 0, GetRectWidth(&rc), GetRectHeight(&rc));
	MoveDialogCenter();
	if(*m_pszPathName)
		SendMessage(BFFM_SETSELECTION, TRUE, (LPARAM)m_pszPathName);
	else
		m_jTitleStc.SetWindowText(lpszTitle);
	jTreeView.EnsureVisible(jTreeView.GetNextItem(TVGN_CARET));
	FadeInDialog(*this);
	m_wpPrev = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WndProc);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Selected item of Browse Folder dialog box has changed

AFXINLINE void CDumpFolderDlg::SelChanged(PITEMIDLIST pIDL)
{
	TRACE(_T("Entering CDumpFolderDlg::SelChanged\n"));

	if(!m_jTitleStc)
		return;

	TCHAR szText[MAX_PATH + 64];
	int   ccText;
	SHGetPathFromIDList(pIDL, m_pszPathName);
	ccText = lstrlen(lpszTitle);
	CopyMemory(szText, lpszTitle, ccText * sizeof(TCHAR));
	szText[ccText]     = CHR_SP;
	szText[ccText + 1] = CHR_SP;
	e_pTPE->GetPurePathName(m_pszPathName, szText + ccText + 2);
	m_jTitleStc.SetWindowText(szText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Callback procedure for Browse Folder dialog box

int CALLBACK CDumpFolderDlg::CbProc(HWND hWnd, UINT nMsg, LPARAM lData, LPARAM lParam)
{
	TRACE(_T("Entering CDumpFolderDlg::CbProc\n"));

	CDumpFolderDlg* pDFD;
	pDFD = (CDumpFolderDlg*)lParam;
	if(nMsg == BFFM_SELCHANGED)
	{
		pDFD->SelChanged((PITEMIDLIST)lData);
		return 0;
	}

	if(nMsg != BFFM_INITIALIZED)
		return 0;

	pDFD->m_hWnd = hWnd;
	pDFD->Initialized();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window procedure for Browse Folder dialog box

LRESULT CALLBACK CDumpFolderDlg::WndProc(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CDumpFolderDlg::WndProc\n"));

	if(nMsg == WM_COMMAND)
	{
		UINT nBtnId;
		nBtnId = LOWORD(wParam);
		if((nBtnId == ID_OK) || (nBtnId == ID_CANCEL))
			FadeOutDialog(hWnd);
	}
	return CallWindowProc(m_wpPrev, hWnd, nMsg, wParam, lParam);
}
