// CFindReplDlg.cpp - Finds and Replaces POI Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "Resource.hpp"
#include "WinMain.hpp"
#include "CFindReplDlg.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes the dialog

void CFindReplDlg::CloseDialog(int iResult)
{
	TRACE(_T("Entering CFindReplDlg::CloseDialog\n"));

	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_FIND_WHAT control sends a notification message

AFXINLINE void CFindReplDlg::OnFindWhat(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CFindReplDlg::OnFindWhat\n"));

	if(nNotify == EN_CHANGE)
		m_jOKBtn.EnableWindow(m_jFindWhat.GetWindowTextLength());
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_OK control sends a notification message

AFXINLINE void CFindReplDlg::OnOK(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CFindReplDlg::OnOK\n"));

	m_jFindWhat.GetWindowText(m_pszFindText, e_eccNameCol);
	m_nMatchCase = IsButtonChecked(IDC_MATCH_CASE);
	m_nOnlyName  = IsButtonChecked(IDC_ONLY_NAME);
	m_nAllPoi    = IsButtonChecked(IDC_ALL_POI);
	if(m_bReplaceDlg)
		m_jReplaceWith.GetWindowText(m_pszReplaceText, e_eccNameCol);
	CloseDialog(ID_OK);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CFindReplDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CFindReplDlg::OnCancel\n"));

	CloseDialog(ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CFindReplDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CFindReplDlg::OnInitDialog\n"));

	UINT nItemId;
	m_jFindWhat = GetChild(IDC_FIND_WHAT);
	m_jOKBtn    = GetChild(ID_OK);
	m_jFindWhat.SetLimitText(e_eccNameCol - 1);
	CheckButton(IDC_MATCH_CASE, m_nMatchCase);
	CheckButton(IDC_ONLY_NAME, m_nOnlyName);
	CheckButton(IDC_ALL_POI, m_nAllPoi);
	if(*m_pszFindText)
	{
		m_jFindWhat.SetWindowText(m_pszFindText);
		m_jFindWhat.SetSel(0);
	}
	else
		m_jOKBtn.DisableWindow();
	if(m_bReplaceDlg)
	{
		m_jReplaceWith = GetChild(IDC_REPLACE_WITH);
		m_jReplaceWith.SetLimitText(e_eccNameCol - 1);
		if(*m_pszReplaceText)
			m_jReplaceWith.SetWindowText(m_pszReplaceText);
		nItemId = ID_POI_REPLACE;
	}
	else
		nItemId = ID_POI_FIND;
	SetDialogIcon(*this, nItemId);
	MoveDialogCenter();
	FadeInDialog(*this);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CFindReplDlg

BEGIN_MESSAGE_MAP(CFindReplDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_FIND_WHAT, OnFindWhat)
		ON_CONTROL(ID_OK, OnOK)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_INITDIALOG(OnInitDialog)
END_MESSAGE_MAP()
