// CIEPane.cpp - IE Pane Container object Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR CIEPane::m_szClassName[] = _T("IEPane");

////////////////////////////////////////////////////////////////////////////////////////////////
// Manages interfaces for the CIEPane object

STDMETHODIMP CIEPane::QueryInterface(REFIID rIId, PPVOID ppvItf)
{
	TRACE(_T("Entering CIEPane::QueryInterface\n"));

	if((rIId == IID_IUnknown) || (rIId == IID_IOleClientSite))
		*ppvItf = (IOleClientSite*)this;
	else if(rIId == IID_IOleInPlaceFrame)
		*ppvItf = (IOleInPlaceFrame*)this;
	else if(rIId == IID_IOleInPlaceSite)
		*ppvItf = (IOleInPlaceSite*)this;
	else if(rIId == IID_IDocHostUIHandler)
		*ppvItf = (IDocHostUIHandler*)this;
	else
	{
		*ppvItf = NULL;
		return E_NOINTERFACE;
	}

	AddRef();
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Increments the reference count on the object

STDMETHODIMP_(ULONG) CIEPane::AddRef()
{
	TRACE(_T("Entering CIEPane::AddRef\n"));

	m_ceRef++;
	ASSERT(m_ceRef);
	return m_ceRef;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Decrements the reference count

STDMETHODIMP_(ULONG) CIEPane::Release()
{
	TRACE(_T("Entering CIEPane::Release\n"));

	ASSERT(m_ceRef);
	m_ceRef--;
	return m_ceRef;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Saves the object associated with the client site

STDMETHODIMP CIEPane::SaveObject()
{
	TRACE(_T("Entering CIEPane::SaveObject\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets a moniker to an object's client site

STDMETHODIMP CIEPane::GetMoniker(DWORD ogm, DWORD owm, IMoniker** ppMnk)
{
	TRACE(_T("Entering CIEPane::GetMoniker\n"));

	*ppMnk = NULL;
	return E_NOTIMPL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets a pointer to the container's IOleContainer interface

STDMETHODIMP CIEPane::GetContainer(IOleContainer** ppOC)
{
	TRACE(_T("Entering CIEPane::GetContainer\n"));

	*ppOC = NULL;
	return E_NOINTERFACE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Tells the container to position the object so it is visible to the user

STDMETHODIMP CIEPane::ShowObject()
{
	TRACE(_T("Entering CIEPane::ShowObject\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies a container when an embedded object's window is about to become visible or not

STDMETHODIMP CIEPane::OnShowWindow(BOOL bShow)
{
	TRACE(_T("Entering CIEPane::OnShowWindow\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Asks container to allocate more or less space for displaying an embedded object

STDMETHODIMP CIEPane::RequestNewObjectLayout()
{
	TRACE(_T("Entering CIEPane::RequestNewObjectLayout\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the window handle to one of the windows participating in in-place activation

STDMETHODIMP CIEPane::GetWindow(PHWND phWnd)
{
	TRACE(_T("Entering CIEPane::GetWindow\n"));

	*phWnd = *this;
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether context-sensitive help should be entered during an in-place activation

STDMETHODIMP CIEPane::ContextSensitiveHelp(BOOL bEnterMode)
{
	TRACE(_T("Entering CIEPane::ContextSensitiveHelp\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets a rectangle in which the object can put controls while active in place

STDMETHODIMP CIEPane::GetBorder(PRECT pRect)
{
	TRACE(_T("Entering CIEPane::GetBorder\n"));

	return INPLACE_E_NOTOOLSPACE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Requests available space for tools to be installed around the object's window frame

STDMETHODIMP CIEPane::RequestBorderSpace(PCBORDERWIDTHS pBW)
{
	TRACE(_T("Entering CIEPane::RequestBorderSpace\n"));

	return INPLACE_E_NOTOOLSPACE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Allocates space for the border

STDMETHODIMP CIEPane::SetBorderSpace(PCBORDERWIDTHS pBW)
{
	TRACE(_T("Entering CIEPane::SetBorderSpace\n"));

	return OLE_E_INVALIDRECT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Provides a direct channel of communication between the object and windows

STDMETHODIMP CIEPane::SetActiveObject(IOleInPlaceActiveObject* pOIPAO, PCOSTR pszName)
{
	TRACE(_T("Entering CIEPane::SetActiveObject\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Allows the container to insert its menu groups into the composite menu

STDMETHODIMP CIEPane::InsertMenus(HMENU hShareMenu, POLEMENUGROUPWIDTHS pOMGW)
{
	TRACE(_T("Entering CIEPane::InsertMenus\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the composite menu in the window containing the object being activated in place

STDMETHODIMP CIEPane::SetMenu(HMENU hShareMenu, HOLEMENU hOleMenu, HWND hWnd)
{
	TRACE(_T("Entering CIEPane::SetMenu\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Removes menu elements from the in-place composite menu

STDMETHODIMP CIEPane::RemoveMenus(HMENU hShareMenu)
{
	TRACE(_T("Entering CIEPane::RemoveMenus\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets and displays status text about the in-place object in the container's frame window

STDMETHODIMP CIEPane::SetStatusText(PCOSTR pszText)
{
	TRACE(_T("Entering CIEPane::SetStatusText\n"));

	if(e_pTPE->m_jLvEdit.IsWindowVisible() || e_pTPE->m_bAdjIEPane)
		return S_OK;

	e_pTPE->m_bSetReady = TRUE;
	if(!pszText || !*pszText)
		return S_OK;

	PWSTR pszRes;
	WCHAR szText[MAX_STR];
	UINT  nResId;
	LoadStringW(e_hInstance, IDS_IE_PANE_READY, szText, countof(szText));
	if(lstrcmpW(pszText, szText))
	{
		PWCHAR pcJSFun, pcTemp;
		int    i;
		pcJSFun  = szText + LoadStringW(e_hInstance, IDS_GM_JS, szText, countof(szText));
		*pcJSFun = L':';
		pcJSFun++;
		i      = IDS_GM_JS_FIRST;
		nResId = 0;
		do
		{
			pcTemp  = pcJSFun + LoadStringW(e_hInstance, i, pcJSFun, countof(szText));
			*pcTemp = L'(';
			if(EqualMemory(pszText, szText, (pcTemp - szText + 1) * sizeof(WCHAR)))
			{
				switch(i)
				{
				case IDS_GM_JS_LSADD:
				case IDS_GM_JS_IQRADD:
					i = IDS_GM_JS_ADD;
					break;
				case IDS_GM_JS_LSINSERT:
					i = IDS_GM_JS_INSERT;
					break;
				case IDS_GM_JS_LSIMAGE:
					i = IDS_GM_JS_IMAGE;
					break;
				case IDS_GM_JS_FWREMOVE:
					i = IDS_GM_JS_REMOVE;
					break;
				case IDS_GM_JS_VOID:
					return S_OK;
				}
				nResId = i - IDS_GM_JS_FIRST + IDP_GM_JS_FIRST;
				pszRes = szText;
				break;
			}
			i++;
		}
		while(i <= IDS_GM_JS_LAST);
	}
	else
	{
		SUINT snGmInfoIntl;
		snGmInfoIntl++;
		switch(snGmInfoIntl % m_eGmInfoIntl)
		{
		case 0:
			nResId = IDP_GM_INFO_PART1;
			break;
		case (m_eGmInfoIntl / 2):
			nResId = IDP_GM_INFO_PART2;
			break;
		default:
			return S_OK;
		}
		pszRes  = szText + LoadStringW(e_hInstance, IDP_INFO_PART, szText, countof(szText));
		*pszRes = L' ';
		pszRes++;
		*pszRes = L' ';
		pszRes++;
	}
	if(nResId)
	{
		LoadStringW(e_hInstance, nResId, pszRes, countof(szText));
		pszText = szText;
	}
	e_pTPE->m_jStatusBar.SetWindowTextW(pszText);
	e_pTPE->m_bSetReady = FALSE;
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables or disables a frame's modeless dialog boxes

STDMETHODIMP CIEPane::EnableModeless(BOOL bEnable)
{
	TRACE(_T("Entering CIEPane::EnableModeless\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Translates accelerator keystrokes intended for the container's frame window

STDMETHODIMP CIEPane::TranslateAccelerator(PMSG pMsg, WORD wCmdId)
{
	TRACE(_T("Entering CIEPane::TranslateAccelerator\n"));

	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether the container can activate the object in place

STDMETHODIMP CIEPane::CanInPlaceActivate()
{
	TRACE(_T("Entering CIEPane::CanInPlaceActivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the container that one of its objects is being activated in place

STDMETHODIMP CIEPane::OnInPlaceActivate()
{
	TRACE(_T("Entering CIEPane::OnInPlaceActivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the container that the object is about to be activated in place

STDMETHODIMP CIEPane::OnUIActivate()
{
	TRACE(_T("Entering CIEPane::OnUIActivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the in-place object to get the window interfaces

STDMETHODIMP CIEPane::GetWindowContext(IOleInPlaceFrame** ppOIPF,
	IOleInPlaceUIWindow** ppOIPUIW, PRECT pPosRect, PRECT pClipRect,
	POLEINPLACEFRAMEINFO pOIPFI)
{
	TRACE(_T("Entering CIEPane::GetWindowContext\n"));

	*ppOIPF   = this;
	*ppOIPUIW = NULL;
	GetClientRect(pPosRect);
	CopyRect(pClipRect, pPosRect);
	pOIPFI->fMDIApp       = FALSE;
	pOIPFI->hwndFrame     = *this;
	pOIPFI->haccel        = NULL;
	pOIPFI->cAccelEntries = 0;
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Scrolls the view of the object by a specified number of pixels

STDMETHODIMP CIEPane::Scroll(SIZE size)
{
	TRACE(_T("Entering CIEPane::Scroll\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the container on deactivation

STDMETHODIMP CIEPane::OnUIDeactivate(BOOL bUndoable)
{
	TRACE(_T("Entering CIEPane::OnUIDeactivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the container that the object is no longer active in place

STDMETHODIMP CIEPane::OnInPlaceDeactivate()
{
	TRACE(_T("Entering CIEPane::OnInPlaceDeactivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The object no longer has any undo state

STDMETHODIMP CIEPane::DiscardUndoState()
{
	TRACE(_T("Entering CIEPane::DiscardUndoState\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Ends the in-place session, deactivate the object, and revert to its own saved undo state

STDMETHODIMP CIEPane::DeactivateAndUndo()
{
	TRACE(_T("Entering CIEPane::DeactivateAndUndo\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Indicates the object's extents have changed

STDMETHODIMP CIEPane::OnPosRectChange(PCRECT pPosRect)
{
	TRACE(_T("Entering CIEPane::OnPosRectChange\n"));

	IOleInPlaceObject* pOIPO;
	e_pTPE->m_pOO->QueryInterface(IID_IOleInPlaceObject, (PPVOID)&pOIPO);
	if(pOIPO)
	{
		pOIPO->SetObjectRects(pPosRect, pPosRect);
		pOIPO->Release();
	}
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables MSHTML to display a shortcut menu

STDMETHODIMP CIEPane::ShowContextMenu(DWORD dwMenuId, PPOINT pptScreen,
	IUnknown* pUnk, IDispatch* pDsp)
{
	TRACE(_T("Entering CIEPane::ShowContextMenu\n"));

	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the user interface capabilities of the application that is hosting MSHTML

STDMETHODIMP CIEPane::GetHostInfo(PDOCHOSTUIINFO pDHUII)
{
	TRACE(_T("Entering CIEPane::GetHostInfo\n"));

	pDHUII->cbSize        = sizeof(DOCHOSTUIINFO);
	pDHUII->dwFlags       = DOCHOSTUIFLAG_NO3DBORDER | DOCHOSTUIFLAG_THEME;
	pDHUII->dwDoubleClick = DOCHOSTUIDBLCLK_DEFAULT;
	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the host to replace MSHTML menus and toolbars

STDMETHODIMP CIEPane::ShowUI(DWORD dhuit, IOleInPlaceActiveObject* pOIPAO,
	IOleCommandTarget* pOCT, IOleInPlaceFrame* pOIPF, IOleInPlaceUIWindow* pOIPUIW)
{
	TRACE(_T("Entering CIEPane::ShowUI\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the host to remove its menus and toolbars

STDMETHODIMP CIEPane::HideUI()
{
	TRACE(_T("Entering CIEPane::HideUI\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the host that the command state has changed

STDMETHODIMP CIEPane::UpdateUI()
{
	TRACE(_T("Entering CIEPane::UpdateUI\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the document window is activated or deactivated

STDMETHODIMP CIEPane::OnDocWindowActivate(BOOL bActive)
{
	TRACE(_T("Entering CIEPane::OnDocWindowActivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the top-level frame window is activated or deactivated

STDMETHODIMP CIEPane::OnFrameWindowActivate(BOOL bActive)
{
	TRACE(_T("Entering CIEPane::OnFrameWindowActivate\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies the object that it needs to resize its border space

STDMETHODIMP CIEPane::ResizeBorder(PCRECT pRect, IOleInPlaceUIWindow* pOIPUIW, BOOL bFrameWnd)
{
	TRACE(_T("Entering CIEPane::ResizeBorder\n"));

	return S_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Translates accelerator keystrokes intended for the container's frame window

STDMETHODIMP CIEPane::TranslateAccelerator(PMSG pMsg, PCGUID pGuid, DWORD dwCmdId)
{
	TRACE(_T("Entering CIEPane::TranslateAccelerator\n"));

	return (e_pTPE->TranslateAccelerator(m_hAccel, pMsg) ? S_OK : S_FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets a registry subkey path that overrides the default registry settings

STDMETHODIMP CIEPane::GetOptionKeyPath(PPOSTR ppszSubkey, DWORD dwReserved)
{
	TRACE(_T("Entering CIEPane::GetOptionKeyPath\n"));

	*ppszSubkey = NULL;
	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the host to supply an alternative IDropTarget interface

STDMETHODIMP CIEPane::GetDropTarget(IDropTarget* pCurDT, IDropTarget** ppNewDT)
{
	TRACE(_T("Entering CIEPane::GetDropTarget\n"));

	return E_NOTIMPL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the host's IDispatch interface

STDMETHODIMP CIEPane::GetExternal(IDispatch** ppDsp)
{
	TRACE(_T("Entering CIEPane::GetExternal\n"));

	*ppDsp = NULL;
	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the host to modify the URL to be loaded

STDMETHODIMP CIEPane::TranslateUrl(DWORD dwReserved, POSTR pszCurUrl, PPOSTR ppszNewUrl)
{
	TRACE(_T("Entering CIEPane::TranslateUrl\n"));

	*ppszNewUrl = NULL;
	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables the host to replace the MSHTML data object

STDMETHODIMP CIEPane::FilterDataObject(IDataObject* pCurDO, IDataObject** ppNewDO)
{
	TRACE(_T("Entering CIEPane::FilterDataObject\n"));

	*ppNewDO = NULL;
	return S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Paints the client area of a window

AFXINLINE void CIEPane::OnPaint()
{
	TRACE(_T("Entering CIEPane::OnPaint\n"));

	PAINTSTRUCT ps;
	HPEN        hPen;
	HDC         hDC;
	hDC  = BeginPaint(&ps);
	hPen = CreatePen(PS_SOLID, 1, m_ecrBorder);
	SelectPen(hDC, hPen);
	MoveTo(hDC, 0, m_cyClient);
	LineTo(hDC, 0, 0);
	LineTo(hDC, m_cxClient, 0);
	EndPaint(&ps);
	DeletePen(hPen);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size has changed

AFXINLINE void CIEPane::OnSize(UINT nType, int cxClient, int cyClient)
{
	TRACE(_T("Entering CIEPane::OnSize\n"));

	m_cxClient = cxClient;
	m_cyClient = cyClient;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CIEPane

BEGIN_MESSAGE_MAP(CIEPane)
	ON_WM_PAINT(OnPaint)
	ON_WM_SIZE(OnSize)
END_MESSAGE_MAP()
