// CIEPane.hpp - IE Pane Container object Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CIEPANE_INCLUDED_
#define _CIEPANE_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CIEPane - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

class CIEPane :
	public CChildWnd,
	public IOleClientSite,
	public IOleInPlaceFrame,
	public IOleInPlaceSite,
	public IDocHostUIHandler
{
public:
	enum
	{
		// Pane window styles
		m_ePaneStyle = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,

		// Pane windiw border
		m_ecxBorder = 1,
		m_ecyBorder = m_ecxBorder,
		m_ecrBorder = RGB(113, 111, 100),

		// Interval for showing Google Maps prompt
		m_eGmInfoIntl = 8,
	};

public:
	int    m_cxClient, m_cyClient;
	ULONG  m_ceRef;
	HACCEL m_hAccel;

public:
	static CTCHAR m_szClassName[];

public:
	CIEPane();
#ifdef DEBUG
	~CIEPane();
#endif

public:
	// IUnknown
	STDMETHOD(QueryInterface)(REFIID rIId, PPVOID ppvItf);
	STDMETHOD_(ULONG, AddRef)();
	STDMETHOD_(ULONG, Release)();

	// IOleClientSite
	STDMETHOD(SaveObject)();
	STDMETHOD(GetMoniker)(DWORD ogm, DWORD owm, IMoniker** ppMnk);
	STDMETHOD(GetContainer)(IOleContainer** ppOC);
	STDMETHOD(ShowObject)();
	STDMETHOD(OnShowWindow)(BOOL bShow);
	STDMETHOD(RequestNewObjectLayout)();

	// IOleWindow
	STDMETHOD(GetWindow)(PHWND phWnd);
	STDMETHOD(ContextSensitiveHelp)(BOOL bEnterMode);

	// IOleInPlaceFrame
	STDMETHOD(GetBorder)(PRECT pRect);
	STDMETHOD(RequestBorderSpace)(PCBORDERWIDTHS pBW);
	STDMETHOD(SetBorderSpace)(PCBORDERWIDTHS pBW);
	STDMETHOD(SetActiveObject)(IOleInPlaceActiveObject* pOIPAO, PCOSTR pszName);
	STDMETHOD(InsertMenus)(HMENU hShareMenu, POLEMENUGROUPWIDTHS pOMGW);
	STDMETHOD(SetMenu)(HMENU hShareMenu, HOLEMENU hOleMenu, HWND hWnd);
	STDMETHOD(RemoveMenus)(HMENU hShareMenu);
	STDMETHOD(SetStatusText)(PCOSTR pszText);
	STDMETHOD(EnableModeless)(BOOL bEnable);
	STDMETHOD(TranslateAccelerator)(PMSG pMsg, WORD wCmdId);

	// IOleInPlaceSite
	STDMETHOD(CanInPlaceActivate)();
	STDMETHOD(OnInPlaceActivate)();
	STDMETHOD(OnUIActivate)();
	STDMETHOD(GetWindowContext)(IOleInPlaceFrame** ppOIPF, IOleInPlaceUIWindow** ppOIPUIW,
		PRECT pPosRect, PRECT pClipRect, POLEINPLACEFRAMEINFO pOIPFI);
	STDMETHOD(Scroll)(SIZE size);
	STDMETHOD(OnUIDeactivate)(BOOL bUndoable);
	STDMETHOD(OnInPlaceDeactivate)();
	STDMETHOD(DiscardUndoState)();
	STDMETHOD(DeactivateAndUndo)();
	STDMETHOD(OnPosRectChange)(PCRECT pPosRect);

	// IDocHostUIHandler
	STDMETHOD(ShowContextMenu)(DWORD dwMenuId, PPOINT pptScreen,
		IUnknown* pUnk, IDispatch* pDsp);
	STDMETHOD(GetHostInfo)(PDOCHOSTUIINFO pDHUII);
	STDMETHOD(ShowUI)(DWORD dhuit, IOleInPlaceActiveObject* pOIPAO, IOleCommandTarget* pOCT,
		IOleInPlaceFrame* pOIPF, IOleInPlaceUIWindow* pOIPUIW);
	STDMETHOD(HideUI)();
	STDMETHOD(UpdateUI)();
	// STDMETHOD(EnableModeless)(BOOL bEnable);
	STDMETHOD(OnDocWindowActivate)(BOOL bActive);
	STDMETHOD(OnFrameWindowActivate)(BOOL bActive);
	STDMETHOD(ResizeBorder)(PCRECT pRect, IOleInPlaceUIWindow* pOIPUIW, BOOL bFrameWnd);
	STDMETHOD(TranslateAccelerator)(PMSG pMsg, PCGUID pGuid, DWORD dwCmdId);
	STDMETHOD(GetOptionKeyPath)(PPOSTR ppszSubkey, DWORD dwReserved);
	STDMETHOD(GetDropTarget)(IDropTarget* pCurDT, IDropTarget** ppNewDT);
	STDMETHOD(GetExternal)(IDispatch** ppDsp);
	STDMETHOD(TranslateUrl)(DWORD dwReserved, POSTR pszCurUrl, PPOSTR ppszNewUrl);
	STDMETHOD(FilterDataObject)(IDataObject* pCurDO, IDataObject** ppNewDO);

protected:
	void OnPaint();
	void OnSize(UINT nType, int cxClient, int cyClient);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CIEPane::CIEPane()
{
	TRACE(_T("Entering CIEPane::CIEPane\n"));

	m_ceRef  = 0;
	m_hAccel = LoadAccelerators(e_hInstance, IDA_IE_PANE);
}

#ifdef DEBUG
AFXINLINE CIEPane::~CIEPane()
{
	TRACE(_T("Entering CIEPane::~CIEPane\n"));

	ASSERT(!m_ceRef);
}
#endif

// End of _CIEPANE_INCLUDED_
#endif
