// CLvCtrl.cpp - List View Control Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Frees the storage for actions

void CLvCtrl::FreeStorage(int iLayer)
{
	TRACE(_T("Entering CLvCtrl::FreeStorage\n"));

	ASSERT((iLayer >= 0) && (iLayer <= countof(m_pAD)));
	if((iLayer == countof(m_pAD)) || !m_pAD[iLayer])
		return;

	PACTIONDATA pAD, pNextAD;
	do
	{
		pAD = m_pAD[iLayer];
		do
		{
			if(pAD->ac == AC_SETTEXT)
				free(((PLVCOLUMNDATA)pAD)->pszText);
			else if((pAD->ac == AC_INSERT) || (pAD->ac == AC_DELETE))
			{
				int iLvCol;
				iLvCol = LVC_NAME;
				do
				{
					free(((PLVDELETEDATA)pAD)->pszText[iLvCol]);
					iLvCol++;
				}
				while(iLvCol <= LVC_MEMO);
			}
			pNextAD = pAD->pNextAD;
			free(pAD);
			pAD = pNextAD;
		}
		while(pAD);
		m_pAD[iLayer] = NULL;
		iLayer++;
	}
	while((iLayer < countof(m_pAD)) && m_pAD[iLayer]);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the column text of an item

PTSTR CLvCtrl::GetColumnText(int iItem, int iColumn)
{
	TRACE(_T("Entering CLvCtrl::GetColumnText\n"));

	TCHAR  szText[e_eccNameCol];
	int    ccText;
	LVITEM lvi;
	lvi.iSubItem   = iColumn;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	ccText         = GetItemText(iItem, &lvi);
	if(!ccText)
		return NULL;

	PTSTR pszText;
	int   cbText;
	cbText  = (ccText + 1) * sizeof(TCHAR);
	pszText = (PTSTR)malloc(cbText);
	CopyMemory(pszText, szText, cbText);
	return pszText;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the storage for an item

CLvCtrl::PACTIONDATA CLvCtrl::GetStorage(ACTIONCMD ac, int iItem)
{
	TRACE(_T("Entering CLvCtrl::GetStorage\n"));

	PACTIONDATA pAD;
	ASSERT((m_at == AT_NEW) || (m_at == AT_LINK) || (m_at == AT_TEMP));
	if(m_at == AT_NEW)
	{
		m_iLayer++;
		FreeStorage(m_iLayer);
		if(m_iLayer == countof(m_pAD))
		{
			pAD      = m_pAD[1];
			m_pAD[1] = NULL;
			FreeStorage(0);
			m_pAD[0] = pAD;
			MoveMemory(m_pAD + 1, m_pAD + 2, (countof(m_pAD) - 2) * sizeof(PACTIONDATA));
			m_pAD[countof(m_pAD) - 1] = NULL;
			m_iLayer                  = countof(m_pAD) - 1;
		}
		if(m_pTempAD)
		{
			m_pAD[m_iLayer] = m_pTempAD;
			m_pTempAD       = NULL;
			return NULL;
		}
		m_at = AT_LINK;
	}

	int cbAlloc;
	switch(ac)
	{
	case AC_SETTEXT:
		cbAlloc = sizeof(LVCOLUMNDATA);
		break;
	case AC_INSERT:
		cbAlloc = sizeof(LVINSERTDATA);
		break;
	case AC_DELETE:
		cbAlloc = sizeof(LVDELETEDATA);
		break;
	case AC_SORT:
		cbAlloc = sizeof(LVSORTDATA) + iItem * sizeof(int);
		break;
	case AC_MOVEUP:
	case AC_MOVEDOWN:
	case AC_BASEPT:
	case AC_SHOWDIST:
		cbAlloc = sizeof(ACTIONDATA);
		break;
	default:
		ASSERT_UNREACHABLE();
	}
	pAD             = (PACTIONDATA)malloc(cbAlloc);
	pAD->ac         = ac;
	pAD->iItem      = iItem;
	pAD->bFileDirty = e_pTPE->m_bFileDirty;
	switch(ac)
	{
	case AC_SETTEXT:
		pAD->dcd = (iItem == e_pTPE->m_iLvBaseItem) ?
			(e_pTPE->m_bMyBP ? DCD_MY_BP : DCD_BP) : DCD_NORMAL;
		break;
	case AC_INSERT:
		pAD->dcd = (!iItem && e_pTPE->m_bMyBP) ? DCD_MY_BP : DCD_NORMAL;
		break;
	case AC_DELETE:
		if(iItem == e_pTPE->m_iLvBaseItem)
		{
			pAD->dcd              = e_pTPE->m_bMyBP ? DCD_MY_BP : DCD_BP;
			e_pTPE->m_iLvBaseItem = LVI_ERR;
		}
		else
		{
			pAD->dcd = DCD_NORMAL;
			if(iItem < e_pTPE->m_iLvBaseItem)
				e_pTPE->m_iLvBaseItem--;
		}
	}

	PACTIONDATA pNextAD;
	if(m_at == AT_TEMP)
	{
		ASSERT(!m_pTempAD);
		m_pTempAD = pAD;
		pNextAD   = NULL;
	}
	else
	{
		pNextAD         = m_pAD[m_iLayer];
		m_pAD[m_iLayer] = pAD;
	}
	pAD->pNextAD = pNextAD;
	return pAD;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Paints the drop mark

void CLvCtrl::PaintDropMark(int iItem)
{
	TRACE(_T("Entering CLvCtrl::PaintDropMark\n"));

	if(iItem < 0)
		return;

	int  iRight, iHeight, ceLvItem;
	RECT rc;
	GetClientRect(&rc);
	iRight   = rc.right;
	ceLvItem = GetItemCount();
	GetItemRect(min(iItem, ceLvItem - 1), LVIR_BOUNDS, &rc);
	iHeight  = (GetRectHeight(&rc) - m_ecxDropMark) / 2;
	rc.left  = 0;
	iRight   = min(rc.right, iRight);
	rc.right = iRight;
	if(iItem >= ceLvItem)
		rc.top = rc.bottom;
	rc.top   -= m_ecxDropMark / 2;
	rc.bottom = rc.top + m_ecxDropMark;
	InvertRect(m_hDC, &rc);
	rc.right   = rc.left + m_ecxDropMark;
	rc.top    -= iHeight;
	rc.bottom += iHeight;
	InvertRect(m_hDC, &rc);
	rc.left  = iRight - m_ecxDropMark;
	rc.right = iRight;
	InvertRect(m_hDC, &rc);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Redoes actions of layer

void CLvCtrl::RedoAction()
{
	TRACE(_T("Entering CLvCtrl::RedoAction\n"));

	PACTIONDATA pAD;
	m_iLayer++;
	pAD = m_pAD[m_iLayer];
	if((pAD->ac == AC_INSERT) || (pAD->ac == AC_DELETE))
	{
		UpdateWindow();
		SetWindowRedraw(FALSE);
	}

	PACTIONDATA pNewAD, pNextAD;
	PLVSORTDATA pLVSD;
	LVITEM      lvi;
	pNewAD = NULL;
	do
	{
		switch(pAD->ac)
		{
		case AC_SETTEXT:
			e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, pAD->iItem);
			UndoSetTextAction(pAD);
			if(pAD->iItem || !e_pTPE->m_bMyBP)
				e_pTPE->m_bFileDirty = TRUE;
			break;
		case AC_INSERT:
			UndoDeleteAction(pAD);
			if(pAD->iItem || !e_pTPE->m_bMyBP)
				e_pTPE->m_bFileDirty = TRUE;
			break;
		case AC_DELETE:
			e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, pAD->iItem);
			if(!pAD->iItem && e_pTPE->m_bMyBP)
				e_pTPE->m_bMyBP = FALSE;
			else
				e_pTPE->m_bFileDirty = TRUE;
			e_pTPE->m_jCheckDataDlg.UpdateItemData(pAD->iItem, LVC_COUNT);
			CListView::DeleteItem(pAD->iItem);
			break;
		case AC_MOVEUP:
		case AC_MOVEDOWN:
			UndoMoveAction(pAD, pAD->ac == AC_MOVEUP);
			break;
		case AC_SORT:
			pLVSD = (PLVSORTDATA)pAD;
			e_pTPE->SortLvItems(pLVSD->iSortCol, pLVSD->bAscend, AT_REDO);
			break;
		case AC_BASEPT:
			lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			SetItemState(pAD->iNewBase, &lvi);
			break;
		case AC_SHOWDIST:
			ShowDistance(pAD->iNewBase);
			break;
		default:
			ASSERT_UNREACHABLE();
		}
		pNextAD      = pAD->pNextAD;
		pAD->pNextAD = pNewAD;
		pNewAD       = pAD;
		pAD          = pNextAD;
	}
	while(pAD);
	pAD             = m_pAD[m_iLayer];
	m_pAD[m_iLayer] = pNewAD;

	int iBaseItem;
	switch(pNewAD->ac)
	{
	case AC_SETTEXT:
	case AC_INSERT:
		e_pTPE->EnsureLvItemVisible(pAD->iItem);
		iBaseItem = e_pTPE->m_iLvBaseItem;
		break;
	case AC_DELETE:
		iBaseItem = LVI_FIND;
		break;
	case AC_BASEPT:
		iBaseItem = pNewAD->iNewBase;
		e_pTPE->EnsureLvItemVisible(iBaseItem);
		break;
	default:
		return;
	}
	e_pTPE->SetBasePoint(iBaseItem, FALSE);
	e_pTPE->AutoSizeLvCol(LVC_ALL, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets an action

void CLvCtrl::SetAction(ACTIONCMD ac, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::SetAction\n"));

	PACTIONDATA pAD;
	PLVSORTDATA pLVSD;
	int         i, ceItem;
	switch(ac)
	{
	case AC_ADD:
		ASSERT(m_pTempAD);
	case AC_MOVEUP:
	case AC_MOVEDOWN:
		GetStorage(ac, lParam);
		return;
	case AC_CANCEL:
		ASSERT(m_pTempAD);
		free(m_pTempAD);
		m_pTempAD = NULL;
		return;
	case AC_SORT:
		if(lParam)
		{
			m_at            = AT_NEW;
			ceItem          = GetItemCount();
			pLVSD           = (PLVSORTDATA)GetStorage(AC_SORT, ceItem);
			pLVSD->iSortCol = e_pTPE->m_iLvCol;
			pLVSD->bAscend  = e_pTPE->m_bSortAscend;
			for(i = 0; i < ceItem; i++)
				pLVSD->nDataId[i] = MapIndexToId(i);
		}
		else
		{
			pLVSD = (PLVSORTDATA)m_pAD[m_iLayer];
			for(i = 0; i < pLVSD->ceItem; i++)
				pLVSD->iData[i] = MapIdToIndex(pLVSD->nDataId[i]);
		}
		return;
	case AC_BASEPT:
		pAD           = GetStorage(AC_BASEPT, e_pTPE->m_iLvBaseItem);
		pAD->iNewBase = lParam;
		return;
	case AC_SHOWDIST:
		pAD           = GetStorage(AC_SHOWDIST, lParam);
		pAD->iNewBase = e_pTPE->m_iLvBaseItem;
		return;
	default:
		ASSERT_UNREACHABLE();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Shows the Distance column of list view

void CLvCtrl::ShowDistance(int iBaseItem)
{
	TRACE(_T("Entering CLvCtrl::ShowDistance\n"));

	if(e_pTPE->m_jOptionDlg.m_nShowDist == BST_CHECKED)
	{
		e_pTPE->m_jOptionDlg.m_nShowDist = BST_UNCHECKED;
		e_pTPE->m_iLvBaseItem            = LVI_ERR;
		if(!e_pTPE->m_bLvBkgnd)
			InvalidateRect();
	}
	else
	{
		e_pTPE->m_jOptionDlg.m_nShowDist = BST_CHECKED;
		e_pTPE->SetBasePoint(iBaseItem, FALSE);
	}
	e_pTPE->AutoSizeLvCol(LVC_DIST + LVCA_FIRST, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes actions of layer

void CLvCtrl::UndoAction()
{
	TRACE(_T("Entering CLvCtrl::UndoAction\n"));

	PACTIONDATA pAD;
	pAD = m_pAD[m_iLayer];
	if((pAD->ac == AC_INSERT) || (pAD->ac == AC_DELETE))
	{
		UpdateWindow();
		SetWindowRedraw(FALSE);
	}

	PACTIONDATA pNewAD, pNextAD;
	PLVSORTDATA pLVSD;
	LVITEM      lvi;
	pNewAD = NULL;
	do
	{
		switch(pAD->ac)
		{
		case AC_SETTEXT:
			e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, pAD->iItem);
			UndoSetTextAction(pAD);
			break;
		case AC_INSERT:
			e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, pAD->iItem);
			UndoInsertAction(pAD);
			break;
		case AC_DELETE:
			UndoDeleteAction(pAD);
			break;
		case AC_MOVEUP:
		case AC_MOVEDOWN:
			UndoMoveAction(pAD, pAD->ac == AC_MOVEDOWN);
			break;
		case AC_SORT:
			pLVSD = (PLVSORTDATA)pAD;
			e_pTPE->SortLvItems(pLVSD->iSortCol, pLVSD->bAscend, AT_UNDO);
			break;
		case AC_BASEPT:
			e_pTPE->m_iLvBaseItem = pAD->iItem;
			if(pAD->iItem == INVALID_VALUE)
			{
				e_pTPE->SetDistanceText(LVI_ALL, FALSE);
				break;
			}
			lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			SetItemState(pAD->iItem, &lvi);
			break;
		case AC_SHOWDIST:
			ShowDistance(pAD->iItem);
			break;
		default:
			ASSERT_UNREACHABLE();
		}
		pNextAD      = pAD->pNextAD;
		pAD->pNextAD = pNewAD;
		pNewAD       = pAD;
		pAD          = pNextAD;
	}
	while(pAD);
	m_pAD[m_iLayer] = pNewAD;
	m_iLayer--;

	switch(pNewAD->ac)
	{
	case AC_INSERT:
		e_pTPE->m_iLvBaseItem = ((PLVINSERTDATA)pNewAD)->iOldBase;
	case AC_SETTEXT:
	case AC_DELETE:
		e_pTPE->m_bFileDirty = pNewAD->bFileDirty;
	case AC_BASEPT:
		e_pTPE->EnsureLvItemVisible(pNewAD->iItem);
		break;
	case AC_MOVEUP:
	case AC_MOVEDOWN:
	case AC_SORT:
		e_pTPE->m_bFileDirty = pNewAD->bFileDirty;
	default:
		return;
	}
	e_pTPE->SetBasePoint(e_pTPE->m_iLvBaseItem, FALSE);
	e_pTPE->AutoSizeLvCol(LVC_ALL, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes the DeleteItem action

void CLvCtrl::UndoDeleteAction(PACTIONDATA pAD)
{
	TRACE(_T("Entering CLvCtrl::UndoDeleteAction\n"));

	PTSTR         pszText;
	PLVDELETEDATA pLVDD;
	TCHAR         szCoord[32];
	int           iLvItem, iLvCol;
	LVITEM        lvi;
	pLVDD   = (PLVDELETEDATA)pAD;
	iLvItem = pLVDD->iItem;
	if(e_dwWinVer > WV_XP)
	{
		lvi.mask   = LVIF_IMAGE | LVIF_STATE | LVIF_TEXT;
		lvi.iImage = 0;
	}
	else
		lvi.mask = LVIF_STATE | LVIF_TEXT;
	lvi.iItem     = iLvItem;
	lvi.iSubItem  = 0;
	lvi.pszText   = pLVDD->pszText[LVC_NAME];
	lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	CListView::InsertItem(&lvi);
	iLvCol = LVC_LNG;
	do
	{
		lvi.iSubItem = iLvCol;
		pszText      = pLVDD->pszText[iLvCol];
		if(((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT)) && pszText)
		{
			GetCoordString(iLvCol, pszText, szCoord);
			pszText = szCoord;
		}
		lvi.pszText = pszText;
		CListView::SetItemText(iLvItem, &lvi);
		iLvCol++;
	}
	while(iLvCol <= LVC_MEMO);

	if(pLVDD->dcd == DCD_NORMAL)
	{
		if(iLvItem <= e_pTPE->m_iLvBaseItem)
			e_pTPE->m_iLvBaseItem++;
		return;
	}

	if(pLVDD->dcd == DCD_MY_BP)
		e_pTPE->m_bMyBP = TRUE;
	e_pTPE->m_iLvBaseItem = iLvItem;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes the InsertItem action

AFXINLINE void CLvCtrl::UndoInsertAction(PACTIONDATA pAD)
{
	TRACE(_T("Entering CLvCtrl::UndoInsertAction\n"));

	PLVINSERTDATA pLVID;
	int           i;
	pLVID = (PLVINSERTDATA)pAD;
	if(!pLVID->pszText[LVC_NAME])
	{
		i = LVC_NAME;
		do
		{
			pLVID->pszText[i] = GetColumnText(pLVID->iItem, i);
			i++;
		}
		while(i <= LVC_MEMO);
	}
	e_pTPE->m_jCheckDataDlg.UpdateItemData(pLVID->iItem, LVC_COUNT);
	CListView::DeleteItem(pLVID->iItem);
	if(pLVID->dcd == DCD_MY_BP)
		e_pTPE->m_bMyBP = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes the Move Up/Down action

void CLvCtrl::UndoMoveAction(PACTIONDATA pAD, BOOL bMoveUp)
{
	TRACE(_T("Entering CLvCtrl::UndoMoveAction\n"));

	LVITEM lvi;
	lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	SetItemState(pAD->iItem, &lvi);
	e_pTPE->MoveLvItem(bMoveUp, AT_CUT);
	pAD->iItem += bMoveUp ? -1 : 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes the SetItemText action

void CLvCtrl::UndoSetTextAction(PACTIONDATA pAD)
{
	TRACE(_T("Entering CLvCtrl::UndoSetTextAction\n"));

	PTSTR         pszAlloc, pszText;
	PLVCOLUMNDATA pLVCD;
	TCHAR         szCoord[32];
	int           iLvItem, iLvCol;
	LVITEM        lvi;
	pLVCD          = (PLVCOLUMNDATA)pAD;
	iLvItem        = pLVCD->iItem;
	iLvCol         = pLVCD->iColumn;
	pszAlloc       = pLVCD->pszText;
	pLVCD->pszText = GetColumnText(iLvItem, iLvCol);
	lvi.iSubItem   = iLvCol;
	pszText        = pszAlloc;
	if(((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT)) && pszText)
	{
		GetCoordString(iLvCol, pszText, szCoord);
		pszText = szCoord;
	}
	lvi.pszText = pszText;
	CListView::SetItemText(iLvItem, &lvi);
	e_pTPE->m_jCheckDataDlg.UpdateItemData(iLvItem, iLvCol);
	free(pszAlloc);
	lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	SetItemState(iLvItem, &lvi);
	if(pLVCD->dcd != DCD_NORMAL)
		e_pTPE->m_iLvBaseItem = iLvItem;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Deletes an item

BOOL CLvCtrl::DeleteItem(int iItem)
{
	TRACE(_T("Entering CLvCtrl::DeleteItem\n"));

	PLVDELETEDATA pLVDD;
	int           i;
	pLVDD = (PLVDELETEDATA)GetStorage(AC_DELETE, iItem);
	i     = LVC_NAME;
	do
	{
		pLVDD->pszText[i] = GetColumnText(iItem, i);
		i++;
	}
	while(i <= LVC_MEMO);
	e_pTPE->m_jCheckDataDlg.UpdateItemData(iItem, LVC_COUNT);
	return CListView::DeleteItem(iItem);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts a new item

int CLvCtrl::InsertItem(PLVITEM pLVI)
{
	TRACE(_T("Entering CLvCtrl::InsertItem\n"));

	int iItem;
	if(e_dwWinVer > WV_XP)
	{
		pLVI->mask  |= LVIF_IMAGE;
		pLVI->iImage = 0;
	}
	iItem = CListView::InsertItem(pLVI);
	if(m_at != AT_CUT)
	{
		PLVINSERTDATA pLVID;
		pLVID = (PLVINSERTDATA)GetStorage(AC_INSERT, iItem);
		ZeroMemory(pLVID->pszText, sizeof(pLVID->pszText));
		pLVID->iOldBase = e_pTPE->m_iLvBaseItem;
	}
	return iItem;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the text of an item

BOOL CLvCtrl::SetItemText(int iItem, PCLVITEM pLVI)
{
	TRACE(_T("Entering CLvCtrl::SetItemText\n"));

	BOOL          bResult;
	PLVCOLUMNDATA pLVCD;
	int           iLvCol;
	iLvCol = pLVI->iSubItem;
	ASSERT(iLvCol != LVC_DIST);
	if((m_at == AT_NEW) || ((m_at == AT_LINK) && (m_pAD[m_iLayer]->ac == AC_SETTEXT)))
	{
		pLVCD          = (PLVCOLUMNDATA)GetStorage(AC_SETTEXT, iItem);
		pLVCD->iColumn = iLvCol;
		pLVCD->pszText = GetColumnText(iItem, iLvCol);
	}
	else
		pLVCD = NULL;
	bResult = CListView::SetItemText(iItem, pLVI);
	e_pTPE->m_jCheckDataDlg.UpdateItemData(iItem, iLvCol);
	if(((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT) ||
		((e_pTPE->m_fst == CTomPoiEdr::FST_CPB) && (iLvCol == LVC_TEL))) && pLVCD)
	{
		if((pLVI->pszText && *pLVI->pszText) ^ (pLVCD->pszText != NULL))
			RedrawItems(iItem);
	}
	return bResult;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets or clears the window redraw flag

AFXINLINE void CLvCtrl::SetWindowRedraw(BOOL bRedraw /* = TRUE */)
{
	TRACE(_T("Entering CLvCtrl::SetWindowRedraw\n"));

	m_bRedraw = bRedraw;
	CListView::SetWindowRedraw(bRedraw);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item indices

int CALLBACK CLvCtrl::CompareLvIndex(int iItem1, int iItem2, LPARAM lSortParam)
{
	TRACE(_T("Entering CLvCtrl::CompareLvIndex\n"));

	PLVSORTDATA pLVSD;
	int         i, j;
	pLVSD = (PLVSORTDATA)e_pTPE->m_jListView.m_pAD[e_pTPE->m_jListView.m_iLayer];
	i     = -1;
	do
		i++;
	while(iItem1 != pLVSD->iData[i]);
	j = -1;
	do
		j++;
	while(iItem2 != pLVSD->iData[j]);
	return ((i < j) ? -1 : 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A character is pressed

AFXINLINE void CLvCtrl::OnChar(TCHAR cChar, LPARAM lKeyData)
{
	TRACE(_T("Entering CLvCtrl::OnChar\n"));

	if(cChar == VK_RETURN)
	{
		e_pTPE->PostMessage(WM_COMMAND, ID_POI_ADD);
		return;
	}

	if(cChar != CHR_ESC)
		return;

	if(m_bDragOper)
	{
		m_do = DO_NO;
		PostMessage(WM_LBUTTONUP);
		return;
	}

	LVITEM lvi;
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	SetItemState(&lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// An event occurs in a horizontal scroll bar

AFXINLINE LRESULT CLvCtrl::OnHScroll(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnHScroll\n"));

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return (m_bDragOper ? WMR_PROCESSED : WMR_DEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A nonsystem key is pressed

AFXINLINE LRESULT CLvCtrl::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnKeyDown\n"));

	TCHAR cVirtKey;
	cVirtKey = (TCHAR)wParam;
	if(m_bDragOper)
	{
		if(lParam & 0x40000000)
			return WMR_PROCESSED;

		WPARAM wParam;
		if(cVirtKey == VK_CONTROL)
			wParam = (GetKeyState(VK_SHIFT) & KS_DOWN) ?
				(MK_CONTROL | MK_LBUTTON | MK_SHIFT) : (MK_CONTROL | MK_LBUTTON);
		else if(cVirtKey == VK_SHIFT)
			wParam = (GetKeyState(VK_CONTROL) & KS_DOWN) ?
				(MK_CONTROL | MK_LBUTTON | MK_SHIFT) : (MK_LBUTTON | MK_SHIFT);
		else
			return WMR_PROCESSED;

		POINT pt;
		GetCursorPos(&pt);
		ScreenToClient(&pt);
		PostMessage(WM_MOUSEMOVE, wParam, PACKPOINT(pt));
		return WMR_PROCESSED;
	}

	if(e_dwWinVer > WV_XP)
		return WMR_DEFAULT;

	if((cVirtKey != VK_LEFT) && (cVirtKey != VK_RIGHT))
		return WMR_DEFAULT;

	e_pTPE->m_bLRKeyDown = TRUE;

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_DEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A nonsystem key is released

AFXINLINE void CLvCtrl::OnKeyUp(TCHAR cVirtKey, LPARAM lKeyData)
{
	TRACE(_T("Entering CLvCtrl::OnKeyUp\n"));

	if(!m_bDragOper)
		return;

	WPARAM wParam;
	if(cVirtKey == VK_CONTROL)
		wParam = (GetKeyState(VK_SHIFT) & KS_DOWN) ? (MK_LBUTTON | MK_SHIFT) : MK_LBUTTON;
	else if(cVirtKey == VK_SHIFT)
		wParam = (GetKeyState(VK_CONTROL) & KS_DOWN) ? (MK_CONTROL | MK_LBUTTON) : MK_LBUTTON;
	else
		return;

	POINT pt;
	GetCursorPos(&pt);
	ScreenToClient(&pt);
	PostMessage(WM_MOUSEMOVE, wParam, PACKPOINT(pt));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has released

AFXINLINE LRESULT CLvCtrl::OnLButtonUp(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnLButtonUp\n"));

	if(!m_bDragOper)
		return WMR_DEFAULT;

	m_bDragOper = FALSE;
	PaintDropMark(m_iDropItem);
	if(m_bDragTimer)
	{
		KillTimer(m_eTimerId);
		m_bDragTimer = FALSE;
	}
	if(::GetCapture() == *this)
		ReleaseCapture();
	ReleaseDC(m_hDC);
	if(m_do != DO_NO)
	{
		TCHAR  szText[e_eccNameCol];
		int    i, ceSelItem;
		LVITEM lvi;
		SetCursor(LoadCursor(IDC_WAIT));
		ceSelItem      = GetSelCount();
		lvi.cchTextMax = countof(szText);
		if(m_jDropWnd)
		{
			PWSTR pszPoiData;
			HFILE hFile;
			LoadString(e_hInstance, IDP_SENDING_POI, szText, countof(szText));
			e_pTPE->m_jStatusBar.SetWindowText(szText);
			pszPoiData = NULL;
			hFile      = CreateFileMapping(PAGE_READWRITE,
				ceSelItem * (e_eccAllCol * sizeof(WCHAR)), e_pszTpeAppName);
			if(hFile)
			{
				pszPoiData = (PWSTR)MapViewOfFile(hFile, FILE_MAP_WRITE);
				if(pszPoiData)
				{
					PWSTR pszDest;
					int   iLvItem, ccText;
					pszDest = pszPoiData;
					iLvItem = -1;
					while(iLvItem = GetNextItem(LVNI_SELECTED, iLvItem), iLvItem != RV_ERR)
					{
						i = LVC_NAME;
						do
						{
							if((i == LVC_MEMO) && e_pTPE->m_bRPF)
							{
								*pszDest = CHR_NULL;
								pszDest++;
								break;
							}
							lvi.iSubItem = i;
#ifdef UNICODE
							lvi.pszText  = pszDest;
#else
							lvi.pszText  = szText;
#endif
							ccText       = GetCoordDdString(i, lvi.pszText,
								GetItemText(iLvItem, &lvi));
#ifdef UNICODE
							pszDest     += ccText + 1;
#else
							pszDest     += MBStoWCS(szText, ccText + 1,
								pszDest, sizeof(szText));
#endif
							i++;
						}
						while(i <= LVC_MEMO);
					}
					*pszDest = CHR_NULL;
					UnmapViewOfFile(pszPoiData);
					m_jDropWnd.SendMessage(CTomPoiEdr::WM_DRAG_DROP, DS_DROP, TRUE);
					if(m_do == DO_CUT)
						e_pTPE->SendMessage(WM_COMMAND, ID_POI_DELETE);
				}
				CloseHandle(hFile);
			}
			if(!pszPoiData)
				e_pTPE->DisplayMessage(IDS_MEMORY_ERROR_SEND_POI,
					MB_ICONINFORMATION | MB_OK, UNUSED);
		}
		else
		{
			PUINT pnItemId;
			BOOL  bSetBpAct;
			int   j, iSrcItem, iDestItem;
			UINT  nBaseId, nItemId;
			LoadString(e_hInstance, (m_do == DO_MOVE) ? IDP_MOVING_POI : IDP_COPYING_POI,
				szText, countof(szText));
			e_pTPE->m_jStatusBar.SetWindowText(szText);
			SetAction(AT_NEW);
			pnItemId = (PUINT)malloc(ceSelItem * sizeof(UINT));
			iSrcItem = -1;
			i        = 0;
			do
			{
				iSrcItem    = GetNextItem(LVNI_SELECTED, iSrcItem);
				pnItemId[i] = MapIndexToId(iSrcItem);
				i++;
			}
			while(i < ceSelItem);
			lvi.pszText   = szText;
			lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			iDestItem     = m_iDropItem;
			i             = 0;
			bSetBpAct     = FALSE;
			nBaseId       = (e_pTPE->m_iLvBaseItem != LVI_ERR) ?
				MapIndexToId(e_pTPE->m_iLvBaseItem) : MAX_UINT;
			do
			{
				nItemId      = pnItemId[i];
				iSrcItem     = MapIdToIndex(nItemId);
				lvi.iSubItem = LVC_NAME;
				GetItemText(iSrcItem, &lvi);
				lvi.mask  = LVIF_STATE | LVIF_TEXT;
				lvi.iItem = iDestItem;
				InsertItem(&lvi);
				if(iSrcItem >= iDestItem)
					iSrcItem++;
				j = LVC_LNG;
				do
				{
					lvi.iSubItem = j;
					if(GetItemText(iSrcItem, &lvi))
						CListView::SetItemText(iDestItem, &lvi);
					j++;
				}
				while(j <= LVC_MEMO);
				if(m_do == DO_MOVE)
				{
					if(nItemId == nBaseId)
					{
						nBaseId   = MapIndexToId(iDestItem);
						bSetBpAct = TRUE;
					}
					e_pTPE->m_jCheckDataDlg.SwapItemId(iSrcItem, iDestItem);
				}
				else
				{
					lvi.state = UNUSED;
					SetItemState(iSrcItem, &lvi);
					lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
				}
				iDestItem++;
				i++;
			}
			while(i < ceSelItem);
			if(m_do == DO_MOVE)
			{
				i = 0;
				do
				{
					DeleteItem(MapIdToIndex(pnItemId[i]));
					i++;
				}
				while(i < ceSelItem);
			}
			if(nBaseId != MAX_UINT)
			{
				i = MapIdToIndex(nBaseId);
				if(bSetBpAct)
					SetAction(AC_BASEPT, i);
				e_pTPE->SetBasePoint(i, FALSE);
			}
			free(pnItemId);
			e_pTPE->m_bFileDirty  = TRUE;
			e_pTPE->m_bSetCaption = TRUE;
		}
	}
	if(m_jDropWnd)
	{
		m_jDropWnd.PostMessage(CTomPoiEdr::WM_DRAG_DROP, DS_OUT);
		m_jDropWnd = NULL;
	}
	SetWindowRedraw();
	SetCursor(LoadCursor(IDC_ARROW));
	e_pTPE->m_bSetReady = TRUE;

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left or right mouse button has pressed/double clicked

AFXINLINE LRESULT CLvCtrl::OnLRBtnDblClkDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnLRBtnDblClkDown\n"));

	if(m_bDragOper)
	{
		if(wParam & MK_RBUTTON)
			PostMessage(WM_MOUSEMOVE, wParam, lParam);
		return WMR_PROCESSED;
	}

	if(!e_pTPE->m_jLvEdit.IsWindowVisible())
		return WMR_DEFAULT;

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	e_pTPE->m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
	return (e_pTPE->m_jLvEdit.m_bTextOK ? WMR_DEFAULT : WMR_PROCESSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Middle mouse button has pressed

AFXINLINE LRESULT CLvCtrl::OnMButtonDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnMButtonDown\n"));

	if(m_bDragOper)
		return WMR_PROCESSED;

	if(e_pTPE->m_jLvEdit.IsWindowVisible())
	{
		e_pTPE->m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
		if(!e_pTPE->m_jLvEdit.m_bTextOK)
			return WMR_PROCESSED;
	}

	if(wParam & (MK_CONTROL | MK_SHIFT))
		return WMR_PROCESSED;

	e_pTPE->m_bMBtnDown = TRUE;
	PostMessage(WM_LBUTTONDOWN, wParam, lParam);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Middle mouse button has released

AFXINLINE LRESULT CLvCtrl::OnMButtonUp(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnMButtonUp\n"));

	if(m_bDragOper)
		return WMR_PROCESSED;

	e_pTPE->m_bMBtnDown = FALSE;
	PostMessage(WM_LBUTTONUP, wParam, lParam);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse cursor moves in the client area of a window

AFXINLINE LRESULT CLvCtrl::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnMouseMove\n"));

	if(!m_bDragOper)
		return WMR_DEFAULT;

	BOOL  bCapWnd, bSetTimer;
	int   iDropItem;
	POINT pt;
	RECT  rc;
	m_do      = DO_NO;
	iDropItem = -1;
	bCapWnd   = (BOOL)::GetCapture();
	bSetTimer = FALSE;
	GetClientRect(&rc);
	SetPoint(&pt, LOSHORT(lParam), HISHORT(lParam));
	if(PtInRect(&rc, pt))
	{
		int iTemp, ceLvItem;
		ceLvItem = GetItemCount();
		if(ceLvItem)
		{
			LVHITTESTINFO lvhti;
			iTemp = rc.bottom - 1;
			GetItemRect(0, LVIR_BOUNDS, &rc);
			SetPoint(&lvhti.pt, 7, min(pt.y + GetRectHeight(&rc) / 2, iTemp));
			iDropItem = HitTest(&lvhti);
			if(lvhti.flags & LVHT_NOWHERE)
				iDropItem = ceLvItem;
			else if(!iDropItem && e_pTPE->m_bMyBP)
				iDropItem = 1;
		}
		if(bCapWnd)
		{
			if(wParam & (MK_CONTROL | MK_RBUTTON))
				m_do = DO_COPY;
			else
			{
				int iFirstItem;
				iFirstItem = GetNextItem(LVNI_SELECTED);
				if(!iFirstItem && e_pTPE->m_bMyBP)
					iDropItem = -1;
				else
				{
					int ceSelItem, iLastItem;
					ceSelItem = GetSelCount();
					iLastItem = iFirstItem;
					for(iTemp = ceSelItem; iTemp > 1; iTemp--)
						iLastItem = GetNextItem(LVNI_SELECTED, iLastItem);
					iTemp = iLastItem + 1;
					if((ceSelItem != (iTemp - iFirstItem)) ||
						(iDropItem < iFirstItem) || (iDropItem > iTemp))
						m_do = DO_MOVE;
				}
			}
			if(m_jDropWnd)
			{
				m_jDropWnd.PostMessage(CTomPoiEdr::WM_DRAG_DROP, DS_OUT);
				m_jDropWnd = NULL;
			}
		}
		else
			m_do = DO_COPY;
	}
	else
	{
		POINT ptMouse;
		RECT  rcWnd;
		e_pTPE->GetWindowRect(&rcWnd);
		GetCursorPos(&ptMouse);
		if(PtInRect(&rcWnd, ptMouse))
		{
			bSetTimer = TRUE;
			if(pt.x < 0)
				m_nHitTest = LVHT_TOLEFT;
			else if(pt.x >= rc.right)
				m_nHitTest = LVHT_TORIGHT;
			else
				m_nHitTest = 0;
			if(pt.y < 0)
				m_nHitTest |= LVHT_ABOVE;
			else if(pt.y >= rc.bottom)
				m_nHitTest |= LVHT_BELOW;
			if(m_jDropWnd)
			{
				m_jDropWnd.PostMessage(CTomPoiEdr::WM_DRAG_DROP, DS_OUT);
				m_jDropWnd = NULL;
			}
		}
		else if(e_pTPE->m_fst == CTomPoiEdr::FST_GENERAL)
		{
			CWnd jDropWnd;
			jDropWnd.WindowFromPoint(ptMouse);
			if(jDropWnd)
			{
				CWnd jTempWnd;
				jTempWnd = jDropWnd;
				do
				{
					jDropWnd = jTempWnd;
					jTempWnd = jTempWnd.GetParent();
				}
				while(jTempWnd);
				if(jDropWnd == *e_pTPE)
					jDropWnd = NULL;
				else
				{
					TCHAR szClassName[64];
					jDropWnd.GetClassName(szClassName, countof(szClassName));
					if(lstrcmpi(szClassName, e_pszTpeClassName) &&
						lstrcmpi(szClassName, e_pszTieClassName))
						jDropWnd = NULL;
					else
					{
						DWORD dwResult;
						if(jDropWnd.SendMessageTimeout(CTomPoiEdr::WM_DRAG_DROP, DS_OVER,
							UNUSED, SMTO_BLOCK, 500, &dwResult) && (dwResult == DE_ACCEPT))
							m_do = (wParam & (MK_SHIFT | MK_RBUTTON)) ? DO_CUT : DO_COPY;
					}
				}
			}
			if(m_jDropWnd && (m_jDropWnd != jDropWnd))
				m_jDropWnd.PostMessage(CTomPoiEdr::WM_DRAG_DROP, DS_OUT);
			m_jDropWnd = jDropWnd;
		}
	}
	if(iDropItem != m_iDropItem)
	{
		PaintDropMark(m_iDropItem);
		PaintDropMark(iDropItem);
		m_iDropItem = iDropItem;
	}
	if(bSetTimer)
	{
		if(!m_bDragTimer)
		{
			SetTimer(m_eTimerId, m_eTimerIntl);
			m_bDragTimer = TRUE;
		}
	}
	else
	{
		if(m_bDragTimer)
		{
			KillTimer(m_eTimerId);
			m_bDragTimer = FALSE;
		}
	}
	if(!bCapWnd)
		return WMR_DEFAULT;

	SCUINT snCursorId[] = { IDC_DRAG_NO, IDC_DRAG_MOVE, IDC_DRAG_COPY, IDC_DRAG_CUT };
	SetCursor(LoadCursor(e_hInstance, snCursorId[m_do]));

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse wheel is rotated

AFXINLINE LRESULT CLvCtrl::OnMouseWheel(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnMouseWheel\n"));

	UINT nFlags;
	nFlags = LOWORD(wParam);
	if(nFlags & (MK_LBUTTON | MK_MBUTTON | MK_RBUTTON))
		return WMR_PROCESSED;

	if(m_bDragOper)
		return WMR_PROCESSED;

	POINT pt;
	RECT  rc;
	SetPoint(&pt, LOSHORT(lParam), HISHORT(lParam));
	if(e_pTPE->m_jCheckDataDlg)
	{
		e_pTPE->m_jCheckDataDlg.GetWindowRect(&rc);
		if(PtInRect(&rc, pt))
		{
			e_pTPE->m_jCheckDataDlg.m_jListView.SendMessage(WM_MOUSEWHEEL, wParam, lParam);
			return WMR_PROCESSED;
		}
	}

	if(e_pTPE->m_jIEPane.IsWindowVisible())
	{
		e_pTPE->m_jIEPane.GetWindowRect(&rc);
		if(PtInRect(&rc, pt))
		{
			HWND hChildWnd;
			CWnd jIEServerWnd;
			hChildWnd = e_pTPE->m_jIEPane;
			do
			{
				jIEServerWnd = hChildWnd;
				hChildWnd    = jIEServerWnd.GetFirstChild();
			}
			while(hChildWnd);
			jIEServerWnd.SendMessage(WM_MOUSEWHEEL, wParam, lParam);
			return WMR_PROCESSED;
		}
	}

	e_pTPE->GetWindowRect(&rc);
	if(!PtInRect(&rc, pt))
		return WMR_PROCESSED;

	if(e_pTPE->m_jLvCombo.GetDroppedState())
		e_pTPE->m_jLvCombo.HideDropDown();

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_DEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Middle or right mouse button has double-clicked

AFXINLINE LRESULT CLvCtrl::OnMRBtnDblClk(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnMRBtnDblClk\n"));

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return (m_bDragOper ? WMR_PROCESSED : WMR_DEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Right mouse button has released

AFXINLINE LRESULT CLvCtrl::OnRButtonUp(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnRButtonUp\n"));

	if(!m_bDragOper)
		return WMR_DEFAULT;

	PostMessage(WM_MOUSEMOVE, wParam, lParam);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A timer interval expires

AFXINLINE LRESULT CLvCtrl::OnTimer(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnTimer\n"));

	if(wParam != m_eTimerId)
		return WMR_DEFAULT;

	SetWindowRedraw();

	int iTemp;
	if(m_nHitTest & (LVHT_TOLEFT | LVHT_TORIGHT))
	{
		SCROLLINFO si;
		si.cbSize = sizeof(si);
		si.fMask  = SIF_PAGE | SIF_POS | SIF_RANGE;
		GetScrollInfo(SB_HORZ, &si);
		if(si.nPage < (UINT)(si.nMax - si.nMin + 1))
		{
			int iNewPos;
			if(m_nHitTest & LVHT_TOLEFT)
			{
				iNewPos = max(si.nMin, si.nPos - 1);
				iTemp   = -m_ecxScroll;
			}
			else
			{
				iNewPos = min(si.nPos, si.nMax - si.nPage) + 1;
				iTemp   = m_ecxScroll;
			}
			if(iNewPos != si.nPos)
				Scroll(iTemp, 0);
		}
	}

	if(m_nHitTest & (LVHT_ABOVE | LVHT_BELOW))
	{
		iTemp = GetTopIndex();
		if(m_nHitTest & LVHT_ABOVE)
			iTemp--;
		else
		{
			iTemp += GetCountPerPage();
			if(iTemp >= GetItemCount())
				iTemp = -1;
		}
		if(iTemp != -1)
			EnsureVisible(iTemp);
	}

	UpdateWindow();
	SetWindowRedraw(FALSE);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// An event occurs in a vertical scroll bar

AFXINLINE LRESULT CLvCtrl::OnVScroll(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CLvCtrl::OnVScroll\n"));

	if(m_bDragOper)
		return WMR_PROCESSED;

	if(e_dwWinVer > WV_XP)
		return WMR_DEFAULT;

	UINT nCode;
	nCode = LOWORD(wParam);
	if((nCode != SB_PAGEDOWN) && (nCode != SB_PAGEUP) &&
		(nCode != SB_LINEDOWN) && (nCode != SB_LINEUP))
		return WMR_DEFAULT;

	int  cyAmount;
	RECT rc;
	GetItemRect(0, LVIR_LABEL, &rc);
	cyAmount = GetRectHeight(&rc);
	if((nCode == SB_PAGEUP) || (nCode == SB_LINEUP))
		cyAmount = -cyAmount;
	if((nCode == SB_PAGEUP) || (nCode == SB_PAGEDOWN))
		cyAmount *= GetCountPerPage();
	Scroll(0, cyAmount);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CLvCtrl

BEGIN_MESSAGE_MAP(CLvCtrl)
	ON_WM_CHAR(OnChar)
	ON_ANY_MAP(WM_HSCROLL, OnHScroll)
	ON_ANY_MAP(WM_KEYDOWN, OnKeyDown)
	ON_WM_KEYUP(OnKeyUp)
	ON_ANY_MAP(WM_LBUTTONUP, OnLButtonUp)
	ON_SAME_MAP(WM_LBUTTONDBLCLK)
	ON_SAME_MAP(WM_LBUTTONDOWN)
	ON_ANY_MAP(WM_RBUTTONDOWN, OnLRBtnDblClkDown)
	ON_ANY_MAP(WM_MBUTTONDOWN, OnMButtonDown)
	ON_ANY_MAP(WM_MBUTTONUP, OnMButtonUp)
	ON_ANY_MAP(WM_MOUSEMOVE, OnMouseMove)
	ON_ANY_MAP(WM_MOUSEWHEEL, OnMouseWheel)
	ON_SAME_MAP(WM_MBUTTONDBLCLK)
	ON_ANY_MAP(WM_RBUTTONDBLCLK, OnMRBtnDblClk)
	ON_ANY_MAP(WM_RBUTTONUP, OnRButtonUp)
	ON_ANY_MAP(WM_TIMER, OnTimer)
	ON_ANY_MAP(WM_VSCROLL, OnVScroll)
END_MESSAGE_MAP()
