// CLvCtrl.hpp - List View Control Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CLVCTRL_INCLUDED_
#define _CLVCTRL_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CLvCtrl - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

// Number of layers for Undo/Redo actions
enum
{
	e_eceURLayer = 64,
};

// Action types (at)
enum ACTIONTYPE
{
	AT_NEW = 0,
	AT_LINK,
	AT_UNDO,
	AT_REDO,
	AT_TEMP,
	AT_CUT,
};

// Undo/Redo action commands (ac)
enum ACTIONCMD
{
	AC_ADD = 0,
	AC_CANCEL,
	AC_SETTEXT,
	AC_INSERT,
	AC_DELETE,
	AC_MOVEUP,
	AC_MOVEDOWN,
	AC_SORT,
	AC_BASEPT,
	AC_SHOWDIST,
};

class CLvCtrl : public CSubclassWnd<CListView>
{
public:
	// Distance column data (dcd)
	enum DISTCOLDATA
	{
		DCD_NORMAL = 0,
		DCD_BP,
		DCD_MY_BP,
	};

	// Drag operations (do)
	enum DRAGOPER
	{
		DO_NO = 0,
		DO_MOVE,
		DO_COPY,
		DO_CUT,
	};

	// Drag states (ds)
	enum DRAGSTATE
	{
		DS_OVER = 0,
		DS_DROP,
		DS_OUT,
	};

	// Drag effects (de)
	enum DRAGEFFECT
	{
		DE_UNUSED = 0,
		DE_REJECT = DE_UNUSED,
		DE_ACCEPT = 7,
	};

	enum
	{
		// Size of drop mark
		m_ecxDropMark = 2,

		// Timer identifier and interval for scrolling items
		m_eTimerId   = 7,
		m_eTimerIntl = 100,

		// Amount of pixels for scrolling items
		m_ecxScroll = 16,
	};

	// Action data (ad)
	typedef struct ACTIONDATA
	{
		ACTIONCMD ac;
		union
		{
			int iItem;
			int ceItem;
		};
		union
		{
			BOOL bFileDirty;
			int  iNewBase;
		};
		union
		{
			DISTCOLDATA dcd;
			int         iSortCol;
		};
		ACTIONDATA* pNextAD;
	} * PACTIONDATA;

	// Column data (lvcd)
	typedef struct LVCOLUMNDATA : ACTIONDATA
	{
		int   iColumn;
		PTSTR pszText;
	} * PLVCOLUMNDATA;

	// Delete data (lvdd)
	typedef struct LVDELETEDATA : ACTIONDATA
	{
		PTSTR pszText[LVC_COUNT - 1];
	} * PLVDELETEDATA;

	// Insert data (lvid)
	typedef struct LVINSERTDATA : LVDELETEDATA
	{
		int iOldBase;
	} * PLVINSERTDATA;

	// Sort data (lvsd)
	typedef struct LVSORTDATA : ACTIONDATA
	{
		BOOL bAscend;
		union
		{
			int  iData[1];
			UINT nDataId[1];
		};
	} * PLVSORTDATA;

public:
	PACTIONDATA m_pAD[e_eceURLayer], m_pTempAD;
	ACTIONTYPE  m_at;
	DRAGOPER    m_do;
	BOOL        m_bRedraw, m_bDragOper, m_bDragTimer;
	int         m_iLayer, m_iDropItem;
	UINT        m_nHitTest;
	HDC         m_hDC;
	CWnd        m_jDropWnd;

public:
	CLvCtrl();
	~CLvCtrl();
	void FreeStorage(int iLayer);
	void FreeStorage();
	PTSTR GetColumnText(int iItem, int iColumn);
	PACTIONDATA GetStorage(ACTIONCMD ac, int iItem);
	PACTIONDATA GetStorage(ACTIONCMD ac);
	void PaintDropMark(int iItem);
	void RedoAction();
	void SetAction(ACTIONTYPE at);
	void SetAction(ACTIONCMD ac, LPARAM lParam);
	void SetAction(ACTIONCMD ac);
	void ShowDistance(int iBaseItem);
	void UndoAction();
	void UndoDeleteAction(PACTIONDATA pAD);
	void UndoInsertAction(PACTIONDATA pAD);
	void UndoMoveAction(PACTIONDATA pAD, BOOL bMoveUp);
	void UndoSetTextAction(PACTIONDATA pAD);

public:
	BOOL DeleteItem(int iItem);
	int InsertItem(PLVITEM pLVI);
	BOOL SetItemText(int iItem, PCLVITEM pLVI);
	void SetWindowRedraw(BOOL bRedraw = TRUE);

public:
	static int CALLBACK CompareLvIndex(int iItem1, int iItem2, LPARAM lSortParam);

protected:
	void OnChar(TCHAR cChar, LPARAM lKeyData);
	LRESULT OnHScroll(WPARAM wParam, LPARAM lParam);
	LRESULT OnKeyDown(WPARAM wParam, LPARAM lParam);
	void OnKeyUp(TCHAR cVirtKey, LPARAM lKeyData);
	LRESULT OnLButtonUp(WPARAM wParam, LPARAM lParam);
	LRESULT OnLRBtnDblClkDown(WPARAM wParam, LPARAM lParam);
	LRESULT OnMButtonDown(WPARAM wParam, LPARAM lParam);
	LRESULT OnMButtonUp(WPARAM wParam, LPARAM lParam);
	LRESULT OnMouseMove(WPARAM wParam, LPARAM lParam);
	LRESULT OnMouseWheel(WPARAM wParam, LPARAM lParam);
	LRESULT OnMRBtnDblClk(WPARAM wParam, LPARAM lParam);
	LRESULT OnRButtonUp(WPARAM wParam, LPARAM lParam);
	LRESULT OnTimer(WPARAM wParam, LPARAM lParam);
	LRESULT OnVScroll(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CLvCtrl::CLvCtrl()
{
	TRACE(_T("Entering CLvCtrl::CLvCtrl\n"));

	ZeroMemory(m_pAD, sizeof(m_pAD));
	m_pTempAD    = NULL;
	m_iLayer     = INVALID_VALUE;
	m_bRedraw    = TRUE;
	m_jDropWnd   = NULL;
	m_bDragTimer = FALSE;
	m_bDragOper  = FALSE;
}

AFXINLINE CLvCtrl::~CLvCtrl()
{
	TRACE(_T("Entering CLvCtrl::~CLvCtrl\n"));

	ASSERT(!m_pTempAD);
	FreeStorage(0);
}

AFXINLINE void CLvCtrl::FreeStorage()
{
	TRACE(_T("Entering CLvCtrl::FreeStorage\n"));

	ASSERT(!m_pTempAD);
	FreeStorage(0);
	m_iLayer = INVALID_VALUE;
}

AFXINLINE CLvCtrl::PACTIONDATA CLvCtrl::GetStorage(ACTIONCMD ac)
{
	TRACE(_T("Entering CLvCtrl::GetStorage\n"));

	return GetStorage(ac, UNUSED);
}

AFXINLINE void CLvCtrl::SetAction(ACTIONTYPE at)
{
	TRACE(_T("Entering CLvCtrl::SetAction\n"));

	m_at = at;
}

AFXINLINE void CLvCtrl::SetAction(ACTIONCMD ac)
{
	TRACE(_T("Entering CLvCtrl::SetAction\n"));

	SetAction(ac, UNUSED);
}

// End of _CLVCTRL_INCLUDED_
#endif
