// CMyBPDlg.cpp - My Base Point Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CMyBPDlg.hpp"
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR CMyBPDlg::m_szSection[]     = _T("Base Points");
CTCHAR CMyBPDlg::m_szListEntry[]   = _T("BP List %d");
CTCHAR CMyBPDlg::m_szInsertEntry[] = _T("Insert List");

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets or clears the window redraw flag

AFXINLINE void CMyBPDlg::CLvCtrl::SetWindowRedraw(BOOL bRedraw /* = TRUE */)
{
	TRACE(_T("Entering CMyBPDlg::CLvCtrl::SetWindowRedraw\n"));

	m_bRedraw = bRedraw;
	CListView::SetWindowRedraw(bRedraw);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Automatic sizes column widths of list view control

void CMyBPDlg::AutoSizeLvCol()
{
	TRACE(_T("Entering CMyBPDlg::AutoSizeLvCol\n"));

	int      i;
	LVCOLUMN lvc;
	lvc.mask = LVCF_WIDTH;
	lvc.cx   = 0;
	m_jListView.InsertColumn(LVC_MEMO + 1, &lvc);
	i = LVC_FIRST;
	do
	{
		m_jListView.SetColumnWidth(i, LVSCW_AUTOSIZE_USEHEADER);
		i++;
	}
	while(i <= LVC_MEMO);
	m_jListView.DeleteColumn(LVC_MEMO + 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables buttons

void CMyBPDlg::EnableButtons(BOOL bSetFocus)
{
	TRACE(_T("Entering CMyBPDlg::EnableButtons\n"));

	BOOL bSelOne;
	int  ceLvItem, ceSelItem, iLvItem;
	if(bSetFocus)
		m_jListView.SetFocus();
	ceLvItem  = m_jListView.GetItemCount();
	ceSelItem = m_jListView.GetSelCount();
	iLvItem   = m_jListView.GetNextItem(LVNI_SELECTED);
	bSelOne   = ceSelItem == 1;
	m_jButton[m_eSet].EnableWindow(bSelOne);
	m_jButton[m_eMoveU].EnableWindow((iLvItem > 0) && bSelOne);
	m_jButton[m_eMoveD].EnableWindow((iLvItem < (ceLvItem - 1)) && bSelOne);
	m_jButton[m_eAdd].EnableWindow(e_pTPE->m_jListView.GetSelCount() &&
		(ceLvItem < m_eceMaxLvItem));
	m_jButton[m_eDel].EnableWindow(ceSelItem);
	m_jButton[m_eDelAll].EnableWindow(ceLvItem);
	m_jButton[m_eExport].EnableWindow(ceLvItem && !m_bExport);
	m_jButton[m_eOK].EnableWindow(m_bDirty);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Moves an item

void CMyBPDlg::MoveLvItem(BOOL bMoveUp)
{
	TRACE(_T("Entering CMyBPDlg::MoveLvItem\n"));

	TCHAR  szSelText[e_eccNameCol], szSwapText[countof(szSelText)];
	int    i, iSelItem, iSwapItem, iVisibleItem;
	LVITEM lvi;
	iSelItem = m_jListView.GetNextItem(LVNI_SELECTED);
	if(bMoveUp)
	{
		iSwapItem    = iSelItem - 1;
		iVisibleItem = max(0, iSelItem - 3);
	}
	else
	{
		iSwapItem    = iSelItem + 1;
		iVisibleItem = min(iSelItem + 3, m_jListView.GetItemCount() - 1);
	}
	lvi.pszText    = szSelText;
	lvi.cchTextMax = countof(szSelText);
	i              = LVC_NAME;
	do
	{
		lvi.iSubItem = i;
		m_jListView.GetItemText(iSelItem, &lvi);
		lvi.pszText = szSwapText;
		m_jListView.GetItemText(iSwapItem, &lvi);
		m_jListView.SetItemText(iSelItem, &lvi);
		lvi.pszText = szSelText;
		m_jListView.SetItemText(iSwapItem, &lvi);
		i++;
	}
	while(i <= LVC_MEMO);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(iSelItem, &lvi);
	lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(iSwapItem, &lvi);
	if(m_iInsertBP == iSelItem)
		m_iInsertBP = iSwapItem;
	else if(m_iInsertBP == iSwapItem)
		m_iInsertBP = iSelItem;
	m_jListView.EnsureVisible(iVisibleItem);
	m_bDirty = TRUE;
	EnableButtons(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes my base points and dialog size into profile

void CMyBPDlg::WriteSection(BOOL bWriteBP, int iResult)
{
	TRACE(_T("Entering CMyBPDlg::WriteSection\n"));

	TCHAR szEntry[32];
	RECT  rcDlg;
	if(m_bDirty && bWriteBP)
	{
		PTSTR  pszText;
		TCHAR  szText[e_eccAllCol];
		int    ceLvItem;
		LVITEM lvi;
		WritePrivateProfileSection(m_szSection, e_szNull, e_pszProfile);
		lvi.cchTextMax = e_eccNameCol;
		ceLvItem       = m_jListView.GetItemCount();
		for(int i = 0, j; i < ceLvItem; i++)
		{
			j       = LVC_NAME;
			pszText = szText;
			do
			{
				lvi.iSubItem = j;
				lvi.pszText  = pszText;
				pszText     += GetCoordDdString(j, pszText, m_jListView.GetItemText(i, &lvi));
				*pszText     = m_eSepChar;
				pszText++;
				j++;
			}
			while(j <= LVC_MEMO);
			*(pszText - 1) = CHR_NULL;
			wsprintf(szEntry, m_szListEntry, i + 1);
			WritePrivateProfileString(m_szSection, szEntry, szText, e_pszProfile);
		}
	}
	GetWindowRect(&rcDlg);
	_ltot(GetRectWidth(&rcDlg), szEntry);
	WritePrivateProfileString(m_szSection, e_szWidth, szEntry, e_pszProfile);
	_ltot(GetRectHeight(&rcDlg), szEntry);
	WritePrivateProfileString(m_szSection, e_szHeight, szEntry, e_pszProfile);
	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_SET control sends a notification message

AFXINLINE void CMyBPDlg::OnSet(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnSet\n"));

	TCHAR  szText[e_eccNameCol];
	int    i, iSelItem, iLastCol;
	LVITEM lvi;
	e_pTPE->m_jListView.SetAction(AT_NEW);
	iSelItem       = m_jListView.GetNextItem(LVNI_SELECTED);
	lvi.iSubItem   = LVC_NAME;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	m_jListView.GetItemText(iSelItem, &lvi);
	if(e_pTPE->m_bMyBP)
	{
		e_pTPE->PutGoogleMaps(CTomPoiEdr::m_eGmPutDelete, 0);
		e_pTPE->m_jListView.SetItemText(0, &lvi);
	}
	else
	{
		e_pTPE->m_bMyBP = TRUE;
		lvi.mask        = LVIF_TEXT;
		lvi.iItem       = 0;
		e_pTPE->m_jListView.InsertItem(&lvi);
	}
	iLastCol = (e_pTPE->m_fst == CTomPoiEdr::FST_GENERAL) ? LVC_MEMO : LVC_LAT;
	i        = LVC_LNG;
	do
	{
		lvi.iSubItem = i;
		m_jListView.GetItemText(iSelItem, &lvi);
		e_pTPE->m_jListView.SetItemText(0, &lvi);
		i++;
	}
	while(i <= iLastCol);
	e_pTPE->m_jListView.EnsureVisible(0);
	e_pTPE->m_iInsertBP = iSelItem;
	_ltot(iSelItem + 1, szText);
	WritePrivateProfileString(m_szSection, m_szInsertEntry, szText, e_pszProfile);
	WriteSection(TRUE, ID_OK);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_MOVE_UP control sends a notification message

AFXINLINE void CMyBPDlg::OnMoveUp(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnMoveUp\n"));

	MoveLvItem(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_MOVE_DOWN control sends a notification message

AFXINLINE void CMyBPDlg::OnMoveDown(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnMoveDown\n"));

	MoveLvItem(FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_ADD control sends a notification message

AFXINLINE void CMyBPDlg::OnAdd(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnAdd\n"));

	TCHAR      szText[e_eccNameCol], szOldText[countof(szText)], szSC[countof(szText)];
	BOOL       bDiffName;
	int        ceLvItem, iSrcItem, iDestItem, iFirstItem;
	LVFINDINFO lvfi;
	LVITEM     lvi;
	m_jListView.SetWindowRedraw(FALSE);
	ceLvItem    = min(e_pTPE->m_jListView.GetSelCount(),
		m_eceMaxLvItem - m_jListView.GetItemCount());
	iSrcItem    = -1;
	iFirstItem  = INVALID_VALUE;
	lvi.mask    = LVIF_TEXT;
	lvi.pszText = szText;
	lvfi.flags  = LVFI_STRING;
	lvfi.psz    = szText;
	for(int i = 0, j; i < ceLvItem; i++)
	{
		iSrcItem       = e_pTPE->m_jListView.GetNextItem(LVNI_SELECTED, iSrcItem);
		lvi.iSubItem   = LVC_LNG;
		lvi.cchTextMax = 2;
		if(!e_pTPE->m_jListView.GetItemText(iSrcItem, &lvi))
			continue;
		lvi.iSubItem = LVC_LAT;
		if(!e_pTPE->m_jListView.GetItemText(iSrcItem, &lvi))
			continue;
		lvi.cchTextMax = countof(szText);
		j              = LVC_NAME;
		do
		{
			lvi.iSubItem = j;
			e_pTPE->m_jListView.GetItemText(iSrcItem, &lvi);
			if(j == LVC_NAME)
			{
				iDestItem   = -1;
				lvi.pszText = szOldText;
				bDiffName   = TRUE;
				do
				{
					iDestItem = m_jListView.FindItem(&lvfi, iDestItem);
					if(iDestItem == RV_ERR)
						break;

					m_jListView.GetItemText(iDestItem, &lvi);
					bDiffName = lstrcmp(szText, szOldText);
				}
				while(bDiffName);
				lvi.pszText = szText;
				if(bDiffName)
				{
					lvi.iItem = MAX_INT;
					iDestItem = m_jListView.InsertItem(&lvi);
				}
				if(iFirstItem == INVALID_VALUE)
					iFirstItem = iDestItem;
			}
			else
			{
				if(e_pTPE->m_fst == CTomPoiEdr::FST_SC)
				{
					if(j == LVC_SPEED)
					{
						if(iSrcItem || !e_pTPE->m_bMyBP)
						{
							LoadString(e_hInstance, IDS_SPEED_COL, szSC, countof(szSC));
							lstrcat(szSC, szText);
						}
						lvi.pszText = NULL;
					}
					else if(j == LVC_CAMERA)
					{
						if(iSrcItem || !e_pTPE->m_bMyBP)
						{
							lstrcat(szSC, _T(", "));
							lstrcat(szSC, szText);
							lvi.pszText = szSC;
						}
						else
							lvi.pszText = NULL;
					}
				}
				m_jListView.SetItemText(iDestItem, &lvi);
				lvi.pszText = szText;
			}
			j++;
		}
		while(j <= LVC_MEMO);
	}
	if(iFirstItem != INVALID_VALUE)
	{
		lvi.state     = UNUSED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(&lvi);
		lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(iFirstItem, &lvi);
		AutoSizeLvCol();
		m_bDirty = TRUE;
	}
	m_jListView.SetWindowRedraw();
	EnableButtons(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_DELETE control sends a notification message

AFXINLINE void CMyBPDlg::OnDelete(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnDelete\n"));

	int ceLvItem, iLvItem, iFirstItem;
	m_jListView.SetWindowRedraw(FALSE);
	iLvItem    = m_jListView.GetNextItem(LVNI_SELECTED);
	iFirstItem = iLvItem;
	do
	{
		m_jListView.DeleteItem(iLvItem);
		if(m_iInsertBP == iLvItem)
			m_iInsertBP = 0;
		else if(m_iInsertBP > iLvItem)
			m_iInsertBP--;
		iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem - 1);
	}
	while(iLvItem != RV_ERR);
	ceLvItem = m_jListView.GetItemCount();
	if(ceLvItem)
	{
		LVITEM lvi;
		iLvItem       = min(iFirstItem, ceLvItem - 1);
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(iLvItem, &lvi);
		m_jListView.EnsureVisible(iLvItem);
	}
	AutoSizeLvCol();
	m_jListView.SetWindowRedraw();
	m_bDirty = TRUE;
	EnableButtons(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_DELETE_ALL control sends a notification message

AFXINLINE void CMyBPDlg::OnDeleteAll(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnDeleteAll\n"));

	m_jListView.SetWindowRedraw(FALSE);
	if(m_jListView.GetItemCount())
		m_jListView.DeleteAllItems();
	m_iInsertBP = 0;
	AutoSizeLvCol();
	m_jListView.SetWindowRedraw();
	m_bDirty = TRUE;
	EnableButtons(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BP_EXPORT control sends a notification message

AFXINLINE void CMyBPDlg::OnExport(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnExport\n"));

	TCHAR  szText[e_eccNameCol];
	int    iFirstItem, iLvItem;
	LVITEM lvi;
	e_pTPE->m_jListView.SetAction(AT_NEW);
	e_pTPE->m_jListView.SetWindowRedraw(FALSE);
	iFirstItem    = INVALID_VALUE;
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	e_pTPE->m_jListView.SetItemState(&lvi);
	lvi.state      = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	for(int i = 0, j, ceLvItem = m_jListView.GetItemCount(); i < ceLvItem; i++)
	{
		lvi.iSubItem = LVC_NAME;
		m_jListView.GetItemText(i, &lvi);
		lvi.mask  = LVIF_STATE | LVIF_TEXT;
		lvi.iItem = MAX_INT;
		iLvItem   = e_pTPE->m_jListView.InsertItem(&lvi);
		if(iFirstItem == INVALID_VALUE)
			iFirstItem = iLvItem;
		j = LVC_LNG;
		do
		{
			lvi.iSubItem = j;
			m_jListView.GetItemText(i, &lvi);
			e_pTPE->m_jListView.CListView::SetItemText(iLvItem, &lvi);
			j++;
		}
		while(j <= LVC_MEMO);
	}
	e_pTPE->SetBasePoint(e_pTPE->m_iLvBaseItem, FALSE);
	e_pTPE->AutoSizeLvCol(LVC_ALL, TRUE);
	e_pTPE->EnsureLvItemVisible(iFirstItem);
	e_pTPE->m_jListView.SetWindowRedraw();
	e_pTPE->m_bFileDirty  = TRUE;
	e_pTPE->m_bSetCaption = TRUE;
	m_bExport             = TRUE;
	EnableButtons(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_OK control sends a notification message

AFXINLINE void CMyBPDlg::OnOK(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnOK\n"));

	e_pTPE->m_iInsertBP = m_iInsertBP;
	WriteSection(TRUE, ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CMyBPDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnCancel\n"));

	WriteSection(FALSE, ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size or position is about to change

AFXINLINE void CMyBPDlg::OnGetMinMaxInfo(PMINMAXINFO pMMI)
{
	TRACE(_T("Entering CMyBPDlg::OnGetMinMaxInfo\n"));

	pMMI->ptMinTrackSize.x = max(pMMI->ptMinTrackSize.x, m_ecxMinDlg);
	pMMI->ptMinTrackSize.y = max(pMMI->ptMinTrackSize.y, m_ecyMinDlg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CMyBPDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CMyBPDlg::OnInitDialog\n"));

	SCUINT snButtonId[] =
	{
		IDC_BP_SET,
		IDC_BP_MOVE_UP,
		IDC_BP_MOVE_DOWN,
		IDC_BP_ADD,
		IDC_BP_DELETE,
		IDC_BP_DELETE_ALL,
		IDC_BP_EXPORT,
		ID_OK,
		ID_CANCEL,
	};
	TCHAR    szLvCol[64];
	int      i;
	DWORD    dwLvStyle;
	LVCOLUMN lvc;
	ASSERT(countof(m_jButton) == countof(snButtonId));
	SetDialogIcon(*this, ID_POS_MY_BP);
	m_iInsertBP        = e_pTPE->m_iInsertBP;
	m_bDirty           = FALSE;
	m_bExport          = FALSE;
	m_jListView.m_hWnd = GetChild(IDC_BP_LV);
	m_jListView.SetTextColor(e_pTPE->m_jListView.GetTextColor());
	m_jListView.SetTextBkColor(GetSysColor(COLOR_WINDOW));
	i = 0;
	do
	{
		m_jButton[i] = GetChild(snButtonId[i]);
		i++;
	}
	while(i < countof(m_jButton));
	lvc.mask    = LVCF_TEXT;
	lvc.pszText = szLvCol;
	i           = LVC_FIRST;
	do
	{
		LoadString(e_hInstance, IDS_FIRST_COL + i, szLvCol, countof(szLvCol));
		m_jListView.InsertColumn(i, &lvc);
		i++;
	}
	while(i <= LVC_MEMO);
	m_jListView.SetColumnOrder(e_iHdOrder + LVCR_NAME, countof(e_iHdOrder) - LVCR_NAME);
	if(e_dwWinVer > WV_XP)
	{
		m_jListView.SetWindowStyle(m_jListView.GetWindowStyle() | LVS_SHOWSELALWAYS);
		dwLvStyle = m_eLvExStyle;
	}
	else
		dwLvStyle = m_eLvExStyleXP;
	m_jListView.SetExStyle(dwLvStyle, dwLvStyle);
	m_jListView.SetWindowTheme(e_szAppTheme);

	TCHAR  szName[e_eccAllCol], szCoord[32], szEntry[32];
	PTSTR  pszBpData, pszText;
	int    iLvItem, iLvCol;
	LVITEM lvi;
	lvi.mask = LVIF_TEXT;
	i        = 1;
	do
	{
		wsprintf(szEntry, m_szListEntry, i);
		if(!GetPrivateProfileString(m_szSection, szEntry, e_szNull,
			szName, countof(szName), e_pszProfile))
			break;

		if(*szName != m_eSepChar)
		{
			pszBpData = StrChr(szName, m_eSepChar);
			if(pszBpData)
			{
				*pszBpData = CHR_NULL;
				pszBpData++;
			}
			lvi.iItem    = MAX_INT;
			lvi.iSubItem = 0;
			lvi.pszText  = szName;
			iLvItem      = m_jListView.InsertItem(&lvi);
			iLvCol       = LVC_LNG;
			while(pszBpData)
			{
				pszText = pszBpData;
				if(iLvCol == LVC_MEMO)
					pszBpData = NULL;
				else
				{
					pszBpData = StrChr(pszBpData, m_eSepChar);
					if(pszBpData)
					{
						*pszBpData = CHR_NULL;
						pszBpData++;
					}
					if(iLvCol != LVC_TEL)
					{
						ASSERT((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT));
						lstrcpyn(szCoord, pszText, countof(szCoord));
						GetCoordString(iLvCol, szCoord);
						pszText = szCoord;
					}
				}
				lvi.iSubItem = iLvCol;
				lvi.pszText  = pszText;
				m_jListView.SetItemText(iLvItem, &lvi);
				iLvCol++;
			}
		}
		i++;
	}
	while(i <= m_eceMaxLvItem);
	if(i > 1)
	{
		m_iInsertBP   = min(m_iInsertBP, i - 2);
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(m_iInsertBP, &lvi);
		m_jListView.EnsureVisible(m_iInsertBP);
	}
	AutoSizeLvCol();
	EnableButtons(FALSE);

	RECT rcWork;
	SystemParametersInfo(SPI_GETWORKAREA, &rcWork);
	SetWindowPos(NULL, UNUSED, UNUSED,
		clamp(GetPrivateProfileInt(m_szSection, e_szWidth, m_ecxMinDlg, e_pszProfile),
		m_ecxMinDlg, GetRectWidth(&rcWork)),
		clamp(GetPrivateProfileInt(m_szSection, e_szHeight, m_ecyMinDlg, e_pszProfile),
		m_ecyMinDlg, GetRectHeight(&rcWork)), SWP_NOMOVE | SWP_NOZORDER);
	MoveDialogCenter();
	FadeInDialog(*this);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes of an item in a list view have changed

AFXINLINE void CMyBPDlg::OnLvnItemChanged(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CMyBPDlg::OnLvnItemChanged\n"));

	EnableButtons(FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view is about to be drawn

AFXINLINE UINT CMyBPDlg::OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering CMyBPDlg::OnNmCustomDraw\n"));

	// Result value of the message processed
	return ((pNMCD->hdr.idFrom == IDC_BP_LV) ?
		CTomPoiEdr::CustomDrawListView((PNMLVCUSTOMDRAW)pNMCD) : CDRF_DODEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has double-clicked (an item with) the left mouse button within a control

AFXINLINE BOOL CMyBPDlg::OnNmDblClk(PCNMHDR pNMH)
{
	TRACE(_T("Entering CMyBPDlg::OnNmDblClk\n"));

	PCNMITEMACTIVATE pNMIA;
	pNMIA = (PCNMITEMACTIVATE)pNMH;
	if(pNMIA->iItem != -1)
		PostMessage(WM_COMMAND, MAKEWPARAM(IDC_BP_SET, BN_CLICKED));

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size has changed

AFXINLINE void CMyBPDlg::OnSize(UINT nType, int cxClient, int cyClient)
{
	TRACE(_T("Entering CMyBPDlg::OnSize\n"));

	int  i, cxGap, cyGap, xButton, yButton, cxButton, cyButton, cyBtnGap;
	RECT rcCtrl;
	m_jListView.GetWindowRect(&rcCtrl);
	ScreenToClient((PPOINT)&rcCtrl);
	cxGap    = rcCtrl.left;
	cyGap    = rcCtrl.top;
	yButton  = cyGap;
	cyBtnGap = cyGap / 3;
	m_jButton[m_eSet].GetWindowRect(&rcCtrl);
	cxButton = GetRectWidth(&rcCtrl);
	cyButton = GetRectHeight(&rcCtrl) + cyBtnGap;
	xButton  = cxClient - cxButton - cxGap;
	i        = 0;
	do
	{
		m_jButton[i].SetWindowPos(NULL, xButton, yButton, UNUSED, UNUSED,
			SWP_NOSIZE | SWP_NOZORDER);
		if(i == m_eExport)
			yButton += cyBtnGap;
		yButton += cyButton;
		i++;
	}
	while(i < countof(m_jButton));
	m_jListView.SetWindowPos(NULL, UNUSED, UNUSED,
		cxClient - cxButton - cxGap * 3, cyClient - cyGap * 2, SWP_NOMOVE | SWP_NOZORDER);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CMyBPDlg

BEGIN_MESSAGE_MAP(CMyBPDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_BP_SET, OnSet)
		ON_CONTROL(IDC_BP_MOVE_UP, OnMoveUp)
		ON_CONTROL(IDC_BP_MOVE_DOWN, OnMoveDown)
		ON_CONTROL(IDC_BP_ADD, OnAdd)
		ON_CONTROL(IDC_BP_DELETE, OnDelete)
		ON_CONTROL(IDC_BP_DELETE_ALL, OnDeleteAll)
		ON_CONTROL(IDC_BP_EXPORT, OnExport)
		ON_CONTROL(ID_OK, OnOK)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_GETMINMAXINFO(OnGetMinMaxInfo)
	ON_WM_INITDIALOG(OnInitDialog)
	BEGIN_NOTIFY_MAP()
		ON_LVN_ITEMCHANGED(OnLvnItemChanged)
		ON_NM_CUSTOMDRAW(OnNmCustomDraw)
		ON_NM_DBLCLK(OnNmDblClk)
	END_NOTIFY_MAP()
	ON_WM_SIZE(OnSize)
END_MESSAGE_MAP()
