// COpenDeviceDlg.cpp - Open Device Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "COpenDeviceDlg.hpp"
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR COpenDeviceDlg::m_szSection[] = _T("Open Device");

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has pressed

AFXINLINE LRESULT COpenDeviceDlg::CLvHeader::OnLButtonDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering COpenDeviceDlg::CLvHeader::OnLButtonDown\n"));

	HDHITTESTINFO hhti;
	hhti.pt = MAKEPOINT(lParam);
	HitTest(&hhti);
	if(hhti.flags & (HHT_ONDIVIDER | HHT_ONDIVOPEN))
		return WMR_DEFAULT;

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_PROCESSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for COpenDeviceDlg::CLvHeader

BEGIN_MESSAGE_MAP(COpenDeviceDlg::CLvHeader)
	ON_ANY_MAP(WM_LBUTTONDOWN, OnLButtonDown)
END_MESSAGE_MAP()

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item strings

int CALLBACK COpenDeviceDlg::CompareLvItem(int iItem1, int iItem2, LPARAM lSortParam)
{
	TRACE(_T("Entering COpenDeviceDlg::CompareLvItem\n"));

	COpenDeviceDlg* pODD;
	pODD = (COpenDeviceDlg*)lSortParam;
	if(pODD->m_bFavFile)
	{
		if(!iItem1)
			return -1;
		if(!iItem2)
			return 1;
	}

	TCHAR  szText1[MAX_FNAME], szText2[countof(szText1)];
	int    iResult;
	LVITEM lvi;
	lvi.iSubItem   = pODD->m_iLvCol;
	lvi.pszText    = szText1;
	lvi.cchTextMax = (pODD->m_iLvCol > LVC_SIZE) ? 2 : countof(szText1);
	pODD->m_jListView.GetItemText(iItem1, &lvi);
	lvi.pszText = szText2;
	pODD->m_jListView.GetItemText(iItem2, &lvi);
	switch(pODD->m_iLvCol)
	{
	case LVC_NAME:
#ifdef UNICODE
		iResult = StrCmpLogicalW(szText1, szText2);
		if(!iResult)
			iResult = lstrcmp(szText1, szText2);
#else
		iResult = lstrcmp(szText1, szText2);
#endif
		break;
	case LVC_SIZE:
		double dTemp, cbSize1, cbSize2;
		cbSize1 = _ttof(szText1);
		cbSize2 = _ttof(szText2);
		if(StrChrI(szText1, _T('k')))
			cbSize1 *= 1024;
		else if(StrChrI(szText1, _T('M')))
			cbSize1 *= 1048576;
		if(StrChrI(szText2, _T('k')))
			cbSize2 *= 1024;
		else if(StrChrI(szText2, _T('M')))
			cbSize2 *= 1048576;
		dTemp   = cbSize1 - cbSize2;
		iResult = (dTemp < 0) ? -1 : ((dTemp > 0) ? 1 : 0);
		break;
	default:
		ASSERT(pODD->m_iLvCol > LVC_SIZE);
		iResult = *szText1 ? (*szText2 ? 0 : 1) : (*szText2 ? -1 : 0);
	}
	return (pODD->m_bAscend ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item parameters

int CALLBACK COpenDeviceDlg::CompareLvParam(LPARAM lParam1, LPARAM lParam2, LPARAM lSortParam)
{
	TRACE(_T("Entering COpenDeviceDlg::CompareLvParam\n"));

	if(lParam1 == MIN_LPARAM)
		return -1;
	if(lParam2 == MIN_LPARAM)
		return 1;

	int iResult;
	iResult = (lParam1 < lParam2) ? -1 : 1;
	return (lSortParam ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from a menu or a control sends a notification

AFXINLINE void COpenDeviceDlg::OnCommand(UINT nItemId, UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COpenDeviceDlg::OnCommand\n"));

	TCHAR szText[32];
	if((nItemId == ID_OK) || (nItemId == IDC_GPS_DELETE))
	{
		int    iLvItem;
		LVITEM lvi;
		iLvItem = m_jListView.GetNextItem(LVNI_SELECTED);
		if(nItemId == IDC_GPS_DELETE)
		{
			PTCHAR pcExtPos;
			TCHAR  szMessage[MAX_STR];
			int    i, j;
			lvi.pszText    = szText;
			lvi.cchTextMax = 2;
			pcExtPos       = szMessage;
			pcExtPos      += LoadString(e_hInstance, IDS_GPS_DELETE_FILE,
				szMessage, countof(szMessage));
			i              = LVC_BMP;
			j              = GPSF_BMP;
			do
			{
				lvi.iSubItem = i;
				if(m_jListView.GetItemText(iLvItem, &lvi))
				{
					CopyMemory(pcExtPos, CTomPoiEdr::m_pszGpsfSep,
						sizeof(CTomPoiEdr::m_pszGpsfSep));
					pcExtPos += countof(CTomPoiEdr::m_pszGpsfSep);
					CopyMemory(pcExtPos, CTomPoiEdr::m_pszGPSF[j],
						sizeof(CTomPoiEdr::m_szOv2Ext));
					pcExtPos += countof(CTomPoiEdr::m_szOv2Ext) - 1;
				}
				i++;
				j++;
			}
			while(i < LVC_COUNT);
			if(MessageBox(szMessage, e_pszTpeAppName,
				MB_ICONQUESTION | MB_OKCANCEL) == ID_CANCEL)
			{
				m_jListView.SetFocus();
				return;
			}
		}
		lvi.iSubItem   = LVC_NAME;
		lvi.pszText    = m_pszSelName;
		lvi.cchTextMax = MAX_FNAME;
		m_jListView.GetItemText(iLvItem, &lvi);
	}
#ifdef DEBUG
	else
		ASSERT((nItemId == ID_CANCEL) || (nItemId == IDC_GPS_REFRESH));
#endif

	RECT rcDlg;
	GetWindowRect(&rcDlg);
	_ltot(GetRectWidth(&rcDlg), szText);
	WritePrivateProfileString(m_szSection, e_szWidth, szText, e_pszProfile);
	_ltot(GetRectHeight(&rcDlg), szText);
	WritePrivateProfileString(m_szSection, e_szHeight, szText, e_pszProfile);
	if((nItemId == ID_OK) || (nItemId == ID_CANCEL))
		FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(nItemId);
	m_jLvItemIL.Destroy();
	m_jDummyIL.Destroy();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size or position is about to change

AFXINLINE void COpenDeviceDlg::OnGetMinMaxInfo(PMINMAXINFO pMMI)
{
	TRACE(_T("Entering COpenDeviceDlg::OnGetMinMaxInfo\n"));

	pMMI->ptMinTrackSize.x = max(pMMI->ptMinTrackSize.x, m_ecxMinDlg);
	pMMI->ptMinTrackSize.y = max(pMMI->ptMinTrackSize.y, m_ecyMinDlg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL COpenDeviceDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering COpenDeviceDlg::OnInitDialog\n"));

	PTCHAR    pcStart, pcEnd;
	PTSTR     pszName;
	PGPSFILE  pOv2GPSF, pBmpGPSF;
	PPGPSFILE ppGPSF;
	TCHAR     szTempFile[MAX_PATH];
	TCHAR     szText[m_eccDateCol], szFavName[32], szItemMark[m_eccMarkCol];
	TCHAR     szToken3[32], szToken4[countof(szToken3)];
	BOOL      bBmpFile;
	int       i, j, ccToken3, ccToken4;
	int       iDateOrder, iLvItem, ceLvItem;
	DWORD     dwLvStyle;
	BITMAP    bm;
	LVCOLUMN  lvc;
	LVITEM    lvi;
	HBITMAP   hPoiBmp;
	SetDialogIcon(*this, ID_FILE_OPEN_DEVICE);
	m_jListView   = GetChild(IDC_GPS_OPEN_LV);
	m_jDeleteBtn  = GetChild(IDC_GPS_DELETE);
	m_jRefreshBtn = GetChild(IDC_GPS_REFRESH);
	m_jOKBtn      = GetChild(ID_OK);
	m_jCancelBtn  = GetChild(ID_CANCEL);
	if(e_dwWinVer > WV_XP)
	{
		m_jListView.SetWindowStyle(m_jListView.GetWindowStyle() | LVS_SHOWSELALWAYS);
		dwLvStyle = m_eLvExStyle;
		i         = m_ecxDummyImg;
		j         = m_ecyDummyImg;
	}
	else
	{
		dwLvStyle = m_eLvExStyleXP;
		i         = m_ecxDummyImgXP;
		j         = m_ecyDummyImgXP;
	}
	m_jListView.SetExStyle(dwLvStyle, dwLvStyle);
	m_jListView.SetWindowTheme(e_szAppTheme);
	m_jLvItemIL.LoadImage(e_hInstance, IDB_NO_POI_ICON, m_ecxPoiImg, m_eceGrowImg, CLR_NONE,
		IMAGE_BITMAP, LR_CREATEDIBSECTION);
	m_jDummyIL.Create(i, j, ILC_COLOR32, 0);
	m_jListView.SetImageList(m_jDummyIL, LVSIL_SMALL);
	m_jListView.SetTextColor(e_pTPE->m_jListView.GetTextColor());
	m_jListView.SetTextBkColor(GetSysColor(COLOR_WINDOW));
	LoadString(e_hInstance, IDS_FAVORITES_NAME, szFavName, countof(szFavName));
	LoadString(e_hInstance, IDS_GPS_ITEM_MARK, szItemMark, countof(szItemMark));
	ccToken3    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN3, szToken3, countof(szToken3));
	ccToken4    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN4, szToken4, countof(szToken4));
	lvc.mask    = LVCF_FMT | LVCF_TEXT;
	lvc.pszText = szText;
	i           = LVC_FIRST;
	do
	{
		lvc.fmt = (i < LVC_SIZE) ? LVCFMT_LEFT :
			((i == LVC_SIZE) ? LVCFMT_RIGHT : LVCFMT_CENTER);
		LoadString(e_hInstance, IDS_GPS_FIRST_COL + i, szText, countof(szText));
		m_jListView.InsertColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	m_bFavFile = FALSE;
	lvi.mask   = LVIF_IMAGE | LVIF_PARAM | LVIF_TEXT;
	iDateOrder = 0;
	pOv2GPSF   = m_pGPSFL->pGPSF[GPSF_OV2];
	pBmpGPSF   = m_pGPSFL->pGPSF[GPSF_BMP];
	while(pOv2GPSF)
	{
		pszName  = pOv2GPSF->pszName;
		bBmpFile = CTomPoiEdr::GpsFindFile(pBmpGPSF, pszName);
		if(m_bFavFile || lstrcmpi(pszName, szFavName))
		{
			if(bBmpFile && !e_pTPE->GpsGetFile(m_hInternet, TRUE, pszName, FALSE, szTempFile))
			{
				hPoiBmp = LoadBitmap(szTempFile);
				if(hPoiBmp)
				{
					if(!GetObject(hPoiBmp, &bm) ||
						(bm.bmWidth != m_ecxPoiImg) || (bm.bmHeight != m_ecyPoiImg))
						hPoiBmp = (HBITMAP)CopyImage(hPoiBmp, IMAGE_BITMAP,
							m_ecxPoiImg, m_ecyPoiImg, LR_COPYDELETEORG | LR_COPYRETURNORG);
				}
				DeleteFile(szTempFile);
			}
			else
				hPoiBmp = NULL;
			lvi.iItem  = MAX_INT;
			lvi.lParam = iDateOrder;
			iDateOrder++;
		}
		else
		{
			hPoiBmp    = (HBITMAP)LoadImage(e_hInstance, MAKERESPTR(IDB_FAVORITES),
				IMAGE_BITMAP, LR_CREATEDIBSECTION);
			lvi.iItem  = 0;
			lvi.lParam = MIN_LPARAM;
			m_bFavFile = TRUE;
		}
		if(hPoiBmp)
		{
			lvi.iImage = m_jLvItemIL.Add(hPoiBmp);
			DeleteBitmap(hPoiBmp);
		}
		else
		{
			lvi.iImage = 0;
			bBmpFile   = FALSE;
		}
		lvi.iSubItem = 0;
		lvi.pszText  = pszName;
		iLvItem      = m_jListView.InsertItem(&lvi);
		pcStart      = pOv2GPSF->pszAttr;
		while(pcStart = StrStrI(pcStart, szToken3))
		{
			pcStart += ccToken3;
			if(*pcStart != CHR_LT)
				break;
		}
		if(pcStart)
		{
			pcEnd = StrStrI(pcStart, szToken4);
			if(pcEnd)
			{
				lvi.pszText = szText;
				i           = pcEnd - pcStart;
				if(i < m_eccDateCol)
				{
					CopyMemory(szText, pcStart, i * sizeof(TCHAR));
					szText[i] = CHR_NULL;
					e_pTPE->DecodeEntity(szText);
					TrimString(szText);
					lvi.iSubItem = LVC_DATE;
					m_jListView.SetItemText(iLvItem, &lvi);
				}
				pcStart = pcEnd + ccToken4;
				while(pcStart = StrStrI(pcStart, szToken3))
				{
					pcStart += ccToken3;
					if(*pcStart != CHR_LT)
						break;
				}
				if(pcStart)
				{
					pcEnd = StrStrI(pcStart, szToken4);
					if(pcEnd)
					{
						i = pcEnd - pcStart;
						if(i < m_eccSizeCol)
						{
							CopyMemory(szText, pcStart, i * sizeof(TCHAR));
							szText[i] = CHR_NULL;
							e_pTPE->DecodeEntity(szText);
							TrimString(szText);
							lvi.iSubItem = LVC_SIZE;
							m_jListView.SetItemText(iLvItem, &lvi);
						}
					}
				}
			}
		}
		lvi.pszText = szItemMark;
		if(bBmpFile)
		{
			lvi.iSubItem = LVC_BMP;
			m_jListView.SetItemText(iLvItem, &lvi);
		}
		ppGPSF = m_pGPSFL->pGPSF;
		i      = LVC_OGG;
		j      = GPSF_OGG;
		do
		{
			if(CTomPoiEdr::GpsFindFile(ppGPSF[j], pszName))
			{
				lvi.iSubItem = i;
				m_jListView.SetItemText(iLvItem, &lvi);
			}
			i++;
			j++;
		}
		while(i < LVC_COUNT);
		pOv2GPSF = pOv2GPSF->pNext;
	}
	i = LVC_FIRST;
	do
	{
		m_jListView.SetColumnWidth(i, (i <= LVC_BMP) ? LVSCW_AUTOSIZE_USEHEADER : j);
		if(i == LVC_BMP)
			j = m_jListView.GetColumnWidth(LVC_BMP);
		i++;
	}
	while(i < LVC_COUNT);
	ceLvItem = m_jListView.GetItemCount();
	if(m_bFavFile)
		ceLvItem--;
	if(ceLvItem >= 2)
	{
		m_iLvCol  = LVC_NAME;
		m_bAscend = TRUE;
		m_jListView.SortItems(CompareLvItem, (LPARAM)this);
	}
	else
	{
		m_jLvHeader.m_hWnd = m_jListView.GetHeader();
		m_jLvHeader.SubclassWindow();
	}
	if(m_bFavFile || ceLvItem)
	{
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(0, &lvi);
	}

	RECT rcWork;
	SystemParametersInfo(SPI_GETWORKAREA, &rcWork);
	SetWindowPos(NULL, UNUSED, UNUSED,
		clamp(GetPrivateProfileInt(m_szSection, e_szWidth, m_ecxMinDlg, e_pszProfile),
		m_ecxMinDlg, GetRectWidth(&rcWork)),
		clamp(GetPrivateProfileInt(m_szSection, e_szHeight, m_ecyMinDlg, e_pszProfile),
		m_ecyMinDlg, GetRectHeight(&rcWork)), SWP_NOMOVE | SWP_NOZORDER);
	MoveDialogCenter();
	FadeInDialog(*this);
	SetTimer(m_eTimerId, m_eTimerIntl);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user clicked an item in a header

AFXINLINE void COpenDeviceDlg::OnHdnItemClick(PCNMHEADER pNMH)
{
	TRACE(_T("Entering COpenDeviceDlg::OnHdnItemClick\n"));

	int      i;
	LVCOLUMN lvc;
	if(m_iLvCol == pNMH->iItem)
		m_bAscend = !m_bAscend;
	else
		m_iLvCol = pNMH->iItem;
	lvc.mask = LVCF_FMT;
	i        = LVC_FIRST;
	do
	{
		lvc.fmt = (i < LVC_SIZE) ? LVCFMT_LEFT :
			((i == LVC_SIZE) ? LVCFMT_RIGHT : LVCFMT_CENTER);
		if(i == m_iLvCol)
			lvc.fmt |= m_bAscend ? HDF_SORTUP : HDF_SORTDOWN;
		m_jListView.SetColumn(i, &lvc);
		i++;
	}
	while(i < LVC_COUNT);
	if(m_iLvCol == LVC_DATE)
		m_jListView.SortItems(CompareLvParam, m_bAscend);
	else
		m_jListView.SortItems(CompareLvItem, (LPARAM)this);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes of an item in a list view have changed

AFXINLINE void COpenDeviceDlg::OnLvnItemChanged(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering COpenDeviceDlg::OnLvnItemChanged\n"));

	ASSERT(pNMLV->uChanged & LVIF_STATE);
	SetTimer(m_eTimerId, m_eTimerIntl);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view is about to be drawn

AFXINLINE UINT COpenDeviceDlg::OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering COpenDeviceDlg::OnNmCustomDraw\n"));

	// Result value of the message processed
	return ((pNMCD->hdr.idFrom == IDC_GPS_OPEN_LV) ?
		CTomPoiEdr::CustomDrawListView((PNMLVCUSTOMDRAW)pNMCD) : CDRF_DODEFAULT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has double-clicked (an item with) the left mouse button within a control

AFXINLINE BOOL COpenDeviceDlg::OnNmDblClk(PCNMHDR pNMH)
{
	TRACE(_T("Entering COpenDeviceDlg::OnNmDblClk\n"));

	PCNMITEMACTIVATE pNMIA;
	pNMIA = (PCNMITEMACTIVATE)pNMH;
	if(pNMIA->iItem != -1)
		PostMessage(WM_COMMAND, MAKEWPARAM(ID_OK, BN_CLICKED));

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size has changed

AFXINLINE void COpenDeviceDlg::OnSize(UINT nType, int cxClient, int cyClient)
{
	TRACE(_T("Entering COpenDeviceDlg::OnSize\n"));

	int  cxTemp, cxyGap, yListView, xButton, yButton, cxButton, cxBtnGap;
	RECT rcCtrl;
	m_jListView.GetWindowRect(&rcCtrl);
	ScreenToClient((PPOINT)&rcCtrl);
	cxyGap    = rcCtrl.left;
	yListView = rcCtrl.top;
	m_jDeleteBtn.GetWindowRect(&rcCtrl);
	cxButton = GetRectWidth(&rcCtrl);
	yButton  = cyClient - GetRectHeight(&rcCtrl) - cxyGap;
	xButton  = rcCtrl.right;
	m_jRefreshBtn.GetWindowRect(&rcCtrl);
	cxBtnGap = rcCtrl.left - xButton;
	cxTemp   = cxButton + cxBtnGap;
	m_jDeleteBtn.SetWindowPos(NULL, cxyGap, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	m_jRefreshBtn.SetWindowPos(NULL, cxyGap + cxTemp, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	xButton = cxClient - cxButton - cxyGap;
	m_jCancelBtn.SetWindowPos(NULL, xButton, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	m_jOKBtn.SetWindowPos(NULL, xButton - cxTemp, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	m_jListView.SetWindowPos(NULL, UNUSED, UNUSED,
		cxClient - cxyGap * 2, yButton - yListView - cxyGap, SWP_NOMOVE | SWP_NOZORDER);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A timer interval expires

AFXINLINE void COpenDeviceDlg::OnTimer(UINT nTimerId)
{
	TRACE(_T("Entering COpenDeviceDlg::OnTimer\n"));

	BOOL bEnable;
	bEnable = m_jListView.GetSelCount();
	if(bEnable != m_jOKBtn.IsWindowEnabled())
		m_jOKBtn.EnableWindow(bEnable);
	bEnable = bEnable && (!m_bFavFile || m_jListView.GetNextItem(LVNI_SELECTED));
	if(bEnable != m_jDeleteBtn.IsWindowEnabled())
		m_jDeleteBtn.EnableWindow(bEnable);
	KillTimer(m_eTimerId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for COpenDeviceDlg

BEGIN_MESSAGE_MAP(COpenDeviceDlg)
	ON_WM_COMMAND(OnCommand)
	ON_WM_GETMINMAXINFO(OnGetMinMaxInfo)
	ON_WM_INITDIALOG(OnInitDialog)
	BEGIN_NOTIFY_MAP()
		ON_HDN_ITEMCLICK(OnHdnItemClick)
		ON_LVN_ITEMCHANGED(OnLvnItemChanged)
		ON_NM_CUSTOMDRAW(OnNmCustomDraw)
		ON_NM_DBLCLK(OnNmDblClk)
	END_NOTIFY_MAP()
	ON_WM_SIZE(OnSize)
	ON_WM_TIMER(OnTimer)
END_MESSAGE_MAP()
