// COpenFavDlg.cpp - Open Favorites Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Initializes Browse Folder dialog box

AFXINLINE void COpenFavDlg::Initialized()
{
	TRACE(_T("Entering COpenFavDlg::Initialized\n"));

	TCHAR      szText[MAX_PATH + 64];
	int        xButton, yButton, cxButton, cyButton, yTvCtrl;
	SIZE       size;
	RECT       rc;
	TEXTMETRIC tm;
	HFONT      hFont, hOldFont;
	HDC        hDC;
	CButton    jButton;
	CTreeView  jTreeView;
	SetDialogIcon(*this, ID_FILE_OPEN_FAV);
	LoadString(e_hInstance, IDS_OFD_NAME, szText, countof(szText));
	SetWindowText(szText);
	m_jTitleStc = GetChild(IDC_BFD_STATIC1);
	m_jTitleStc.SetWindowStyle(m_jTitleStc.GetWindowStyle() | SS_PATHELLIPSIS);
	hDC   = m_jTitleStc.GetDC();
	hFont = m_jTitleStc.GetWindowFont();
	if(!hFont)
		hFont = GetStockFont(SYSTEM_FONT);
	SelectFont(hDC, hFont);
	GetTextMetrics(hDC, &tm);
	m_jTitleStc.MoveWindow(m_ecxBorder, m_ecyBorder,
		m_ecxTvCtrl, tm.tmHeight + tm.tmExternalLeading);
	jButton = GetChild(ID_CANCEL);
	jButton.SetWindowStyle(jButton.GetWindowStyle() & ~WS_GROUP);
	jButton.GetWindowRect(&rc);
	cxButton = GetRectWidth(&rc);
	cyButton = GetRectHeight(&rc);
	xButton  = m_ecxDialog - cxButton - m_ecxBorder + 1;
	yButton  = m_ecyDialog - GetRectHeight(&rc) - m_ecyBorder;
	jButton.SetWindowPos(NULL, xButton, yButton, UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	jButton = GetChild(ID_OK);
	jButton.SetWindowPos(NULL, xButton - cxButton - m_ecxGap, yButton,
		UNUSED, UNUSED, SWP_NOSIZE | SWP_NOZORDER);
	yTvCtrl   = tm.tmHeight + m_ecyBorder + m_ecyGap;
	jTreeView = GetChild(IDC_BFD_TREE_VIEW);
	jTreeView.SetWindowStyle(jTreeView.GetWindowStyle() |
		((e_dwWinVer > WV_XP) ? m_eTvStyle : m_eTvStyleXP));
	jTreeView.MoveWindow(m_ecxBorder, yTvCtrl, m_ecxTvCtrl, yButton - yTvCtrl - m_ecyGap);
	GetTextExtent(hDC, szText, LoadString(e_hInstance, IDS_OFD_RECENT_DEST,
		szText, countof(szText)), &size);
	m_jTitleStc.ReleaseDC(hDC);
	jButton.CreateWindow(szText, m_eRecentBtnStyle, m_ecxBorder, yButton,
		size.cx + m_ecxCheckBox + m_ecxPadding, cyButton, *this, m_eRecentBtnId);
	hOldFont = jButton.GetWindowFont();
	jButton.SetWindowFont(hFont);
	if(hOldFont)
		DeleteFont(hOldFont);
	jButton.SetWindowPos(jTreeView, UNUSED, UNUSED, UNUSED, UNUSED, SWP_NOMOVE | SWP_NOSIZE);
	if(m_bRecentDest)
		jButton.SetCheck();
	SetRect(&rc, 0, 0, m_ecxDialog, m_ecyDialog);
	AdjustWindowRect(&rc, GetWindowStyle(), FALSE);
	MoveWindow(0, 0, GetRectWidth(&rc), GetRectHeight(&rc));
	MoveDialogCenter();
	if(m_pIDL)
		SendMessage(BFFM_SETSELECTION, FALSE, (LPARAM)m_pIDL);
	else
	{
		int ccText;
		ccText = lstrlen(lpszTitle);
		CopyMemory(szText, lpszTitle, ccText * sizeof(TCHAR));
		szText[ccText]     = CHR_SP;
		szText[ccText + 1] = CHR_SP;
		lstrcpy(szText + ccText + 2, m_pszFavName);
		m_jTitleStc.SetWindowText(szText);
	}
	jTreeView.EnsureVisible(jTreeView.GetNextItem(TVGN_CARET));
	FadeInDialog(*this);
	m_wpPrev = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WndProc);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Selected item of Browse Folder dialog box has changed

AFXINLINE void COpenFavDlg::SelChanged(PITEMIDLIST pIDL)
{
	TRACE(_T("Entering COpenFavDlg::SelChanged\n"));

	if(!m_jTitleStc)
		return;

	TCHAR szText[MAX_PATH + 64];
	int   ccText;
	SHGetPathFromIDList(pIDL, m_pszFileName);
	ccText = lstrlen(m_pszFileName);
	if(ccText && (m_pszFileName[ccText - 1] != CHR_BSLH))
	{
		m_pszFileName[ccText] = CHR_BSLH;
		ccText++;
	}
	lstrcpy(m_pszFileName + ccText, m_pszFavName);
	SendMessage(BFFM_ENABLEOK, UNUSED, IsValidFile(m_pszFileName));
	ccText = lstrlen(lpszTitle);
	CopyMemory(szText, lpszTitle, ccText * sizeof(TCHAR));
	szText[ccText]     = CHR_SP;
	szText[ccText + 1] = CHR_SP;
	e_pTPE->GetPurePathName(m_pszFileName, szText + ccText + 2);
	m_jTitleStc.SetWindowText(szText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Callback procedure for Browse Folder dialog box

int CALLBACK COpenFavDlg::CbProc(HWND hWnd, UINT nMsg, LPARAM lData, LPARAM lParam)
{
	TRACE(_T("Entering COpenFavDlg::CbProc\n"));

	COpenFavDlg* pOFD;
	pOFD = (COpenFavDlg*)lParam;
	if(nMsg == BFFM_SELCHANGED)
	{
		pOFD->SelChanged((PITEMIDLIST)lData);
		return 0;
	}

	if(nMsg != BFFM_INITIALIZED)
		return 0;

	pOFD->m_hWnd = hWnd;
	pOFD->AttachObject(hWnd);
	pOFD->Initialized();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window procedure for Browse Folder dialog box

LRESULT CALLBACK COpenFavDlg::WndProc(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering COpenFavDlg::WndProc\n"));

	COpenFavDlg* pOFD;
	pOFD = &e_pTPE->m_jOpenFavDlg;
	if(nMsg == WM_COMMAND)
	{
		UINT nBtnId;
		nBtnId = LOWORD(wParam);
		if((nBtnId == ID_OK) || (nBtnId == ID_CANCEL))
		{
			if(nBtnId == ID_OK)
				pOFD->m_bRecentDest = pOFD->IsButtonChecked(m_eRecentBtnId) == BST_CHECKED;
			FadeOutDialog(hWnd);
			DestroyIcon(pOFD->SetWindowIcon(ICON_SMALL));
		}
	}
	return CallWindowProc(pOFD->m_wpPrev, hWnd, nMsg, wParam, lParam);
}
