// COptionDlg.cpp - Options Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CUINT COptionDlg::m_nCheckBoxId[] =
{
	IDC_SHOW_MENU,
	IDC_SHOW_TOOLBAR,
	IDC_LARGE_TOOLBAR,
	IDC_WRAP_TOOLBAR,
	IDC_LNG_FIRST,
	IDC_DMS_FORMAT,
	IDC_COMPASS_DIR,
	IDC_AUTO_COLUMN,
	IDC_SHOW_DISTANCE,
	IDC_METRIC_UNIT,
	IDC_RECENT_FILE,
	IDC_FADE_WINDOW,
	IDC_SHADE_TABLE,
	IDC_SHOW_STATUS,
	IDC_SIGNAL_MARK,
	IDC_OPT_POI,
	IDC_CHECK_NAME,
	IDC_AUTO_BP,
	IDC_OPEN_LAST,
	IDC_TRAN_NAME,
	IDC_BACKUP_FILE,
	IDC_GM_USE_COOKIE,
	IDC_GM_UPDATE_MEMO,
	IDC_GM_FLASH_ROUTE,
};

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes the dialog

void COptionDlg::CloseDialog(int iResult)
{
	TRACE(_T("Entering COptionDlg::CloseDialog\n"));

	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Validates the input values

AFXINLINE UINT COptionDlg::ValidateValue(CEdit** ppEdit)
{
	TRACE(_T("Entering COptionDlg::ValidateValue\n"));

	BOOL bError;
	int  i, ceRecentFile, ceBackupIntl;
	UINT nCheckBox[countof(m_nCheckBoxId)];
	i = 0;
	do
	{
		nCheckBox[i] = IsButtonChecked(m_nCheckBoxId[i]);
		i++;
	}
	while(i < countof(m_nCheckBoxId));
	if(nCheckBox[m_eRecentFile] == BST_CHECKED)
	{
		ceRecentFile = m_jRecentSpin.GetPos(&bError);
		if(bError)
		{
			*ppEdit = &m_jRecentBuddy;
			return IDS_FILE_COUNT_ERROR;
		}
	}
	else
		ceRecentFile = 0;
	if(nCheckBox[m_eBackupFile] == BST_CHECKED)
	{
		ceBackupIntl = m_jBackupSpin.GetPos(&bError);
		if(bError)
		{
			*ppEdit = &m_jBackupBuddy;
			return IDS_BACKUP_INTL_ERROR;
		}
	}
	else
		ceBackupIntl = 0;
	i = 0;
	do
	{
		*m_pnCheckBox[i] = nCheckBox[i];
		i++;
	}
	while(i < countof(nCheckBox));
	if(ceRecentFile)
		m_ceRecentFile = ceRecentFile;
	if(ceBackupIntl)
		m_ceBackupIntl = ceBackupIntl;
	m_iGmLang = m_jGmLang.GetCurSel();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_SHOW_TOOLBAR control sends a notification message

AFXINLINE void COptionDlg::OnShowToolbar(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnShowToolbar\n"));

	UINT nEnable;
	nEnable = IsButtonChecked(IDC_SHOW_TOOLBAR) == BST_CHECKED;
	m_jLargeTb.EnableWindow(nEnable);
	m_jAutoWrapTb.EnableWindow(nEnable);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_SHOW_DISTANCE control sends a notification message

AFXINLINE void COptionDlg::OnShowDistance(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnShowDistance\n"));

	m_jAutoBP.EnableWindow(IsButtonChecked(IDC_SHOW_DISTANCE) == BST_CHECKED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_RECENT_FILE control sends a notification message

AFXINLINE void COptionDlg::OnRecentFile(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnRecentFile\n"));

	BOOL bEnable;
	bEnable = IsButtonChecked(IDC_RECENT_FILE) == BST_CHECKED;
	m_jRecentBuddy.EnableWindow(bEnable);
	m_jRecentSpin.EnableWindow(bEnable);
	m_jOpenLast.EnableWindow(bEnable);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_BACKUP_FILE control sends a notification message

AFXINLINE void COptionDlg::OnBackupFile(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnBackupFile\n"));

	BOOL bEnable;
	bEnable = IsButtonChecked(IDC_BACKUP_FILE) == BST_CHECKED;
	m_jBackupBuddy.EnableWindow(bEnable);
	m_jBackupSpin.EnableWindow(bEnable);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_OK control sends a notification message

AFXINLINE void COptionDlg::OnOK(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnOK\n"));

	UINT   nResId;
	CEdit* pEdit;
	nResId = ValidateValue(&pEdit);
	if(!nResId)
	{
		CloseDialog(ID_OK);
		return;
	}

	TCHAR szMessage[MAX_STR];
	LoadString(e_hInstance, nResId, szMessage, countof(szMessage));
	MessageBox(szMessage, e_pszTpeAppName, MB_ICONINFORMATION | MB_OK);
	pEdit->SetSel(0);
	pEdit->SetFocus();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void COptionDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering COptionDlg::OnCancel\n"));

	CloseDialog(ID_CANCEL);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL COptionDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering COptionDlg::OnInitDialog\n"));

	TCHAR szText[32];
	int   i;
	ASSERT(countof(m_nCheckBoxId) == countof(m_pnCheckBox));
	SetDialogIcon(*this, ID_TOOLS_OPTION);
	i = 0;
	do
	{
		CheckButton(m_nCheckBoxId[i], *m_pnCheckBox[i]);
		i++;
	}
	while(i < countof(m_pnCheckBox));
	m_jLargeTb     = GetChild(IDC_LARGE_TOOLBAR);
	m_jAutoWrapTb  = GetChild(IDC_WRAP_TOOLBAR);
	m_jAutoBP      = GetChild(IDC_AUTO_BP);
	m_jOpenLast    = GetChild(IDC_OPEN_LAST);
	m_jRecentBuddy = GetChild(IDC_RECENT_BUDDY);
	m_jRecentSpin  = GetChild(IDC_RECENT_SPIN);
	m_jBackupBuddy = GetChild(IDC_BACKUP_BUDDY);
	m_jBackupSpin  = GetChild(IDC_BACKUP_SPIN);
	m_jRecentBuddy.SetLimitText(m_eccMaxRecentBuddy);
	m_jRecentSpin.SetRange(CTomPoiEdr::m_eceMaxRecentFile, CTomPoiEdr::m_eceMinRecentFile);
	m_jRecentSpin.SetPos(m_ceRecentFile);
	m_jBackupBuddy.SetLimitText(m_eccMaxBackupBuddy);
	m_jBackupSpin.SetRange(CTomPoiEdr::m_eceMaxBackupIntl, CTomPoiEdr::m_eceMinBackupIntl);
	m_jBackupSpin.SetPos(m_ceBackupIntl);
	if(m_nShowTb == BST_UNCHECKED)
	{
		m_jLargeTb.DisableWindow();
		m_jAutoWrapTb.DisableWindow();
	}
	if(e_pTPE->m_fst == CTomPoiEdr::FST_CPB)
		::EnableWindow(GetChild(IDC_SHOW_DISTANCE), FALSE);
	if(m_nShowDist == BST_UNCHECKED)
		m_jAutoBP.DisableWindow();
	if(m_nRecentFile == BST_UNCHECKED)
	{
		m_jRecentBuddy.DisableWindow();
		m_jRecentSpin.DisableWindow();
		m_jOpenLast.DisableWindow();
	}
	if(e_dwWinVer > WV_XP)
		::EnableWindow(GetChild(IDC_SHADE_TABLE), FALSE);
	if(m_nBackupFile == BST_UNCHECKED)
	{
		m_jBackupBuddy.DisableWindow();
		m_jBackupSpin.DisableWindow();
	}
	m_jGmLang = GetChild(IDC_GM_LANG);
	i         = 0;
	do
	{
		LoadString(e_hInstance, IDS_GM_LANG_NAME0 + i, szText, countof(szText));
		m_jGmLang.AddItem(szText);
		i++;
	}
	while(i < CTomPoiEdr::m_eceGmLang);
	m_jGmLang.SetCurSel(m_iGmLang);
	MoveDialogCenter();
	FadeInDialog(*this);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for COptionDlg

BEGIN_MESSAGE_MAP(COptionDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_SHOW_TOOLBAR, OnShowToolbar)
		ON_CONTROL(IDC_SHOW_DISTANCE, OnShowDistance)
		ON_CONTROL(IDC_RECENT_FILE, OnRecentFile)
		ON_CONTROL(IDC_BACKUP_FILE, OnBackupFile)
		ON_CONTROL(ID_OK, OnOK)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_INITDIALOG(OnInitDialog)
END_MESSAGE_MAP()
