// CQGPSfixDlg.cpp - Quick GPS Fix Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include <Dbt.h>
#include "CQGPSfixDlg.hpp"
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables or disables controls

void CQGPSfixDlg::EnableControls(BOOL bEnable)
{
	TRACE(_T("Entering CQGPSfixDlg::EnableControls\n"));

	BOOL bEnableDrive;
	int  iShowCmd;
	if(bEnable)
	{
		iShowCmd     = SW_HIDE;
		bEnableDrive = m_jDriveName.GetCount();
		KillTimer(m_eTimerId);
	}
	else
	{
		iShowCmd     = SW_SHOW;
		bEnableDrive = FALSE;
		SetTimer(m_eTimerId, m_eTimerIntl);
	}
	e_pTPE->m_jProgress.ShowWindow(iShowCmd);
	m_jChipGroup.EnableWindow(bEnable);
	m_jGLBtn.EnableWindow(bEnable);
	m_jSiRFBtn.EnableWindow(bEnable);
	m_jSaveGroup.EnableWindow(bEnableDrive);
	m_jDriveName.EnableWindow(bEnableDrive);
	m_jSavePath.EnableWindow(bEnableDrive);
	m_jSaveBtn.EnableWindow(bEnableDrive);
	m_jRetryBtn.EnableWindow(bEnable);
	m_jCancelBtn.EnableWindow(bEnable);
	SetFocus();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS fix: gets chip type

AFXINLINE void CQGPSfixDlg::GfGetChip()
{
	TRACE(_T("Entering CQGPSfixDlg::GfGetChip\n"));

	TCHAR szResText[MAX_STR];
	int   iDrive, ccPathName, ccFileName;
	UINT  nGLBtn, nSiRFBtn;
	LoadString(e_hInstance, IDS_RECOGNIZING_GPS_CHIP, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	iDrive = m_jDriveName.GetCount() - 1;
	do
	{
		ccPathName = m_jDriveName.GetItem(iDrive, m_pszFileName);
		if(GetDriveType(m_pszFileName) != DRIVE_REMOVABLE)
			continue;
		ccPathName                 += LoadString(e_hInstance, IDS_GPSFIX_DIRNAME,
			m_pszFileName + ccPathName, MAX_PATH);
		m_pszFileName[ccPathName++] = CHR_BSLH;
		ccFileName = LoadString(e_hInstance, IDS_GL_FILENAME,
			m_pszFileName + ccPathName, MAX_PATH);
		if(IsValidFile(m_pszFileName))
		{
			m_gpsc   = GPSC_GL;
			nGLBtn   = BST_CHECKED;
			nSiRFBtn = BST_UNCHECKED;
			break;
		}
		ccFileName = LoadString(e_hInstance, IDS_SIRF_FILENAME,
			m_pszFileName + ccPathName, MAX_PATH);
		if(IsValidFile(m_pszFileName))
		{
			m_gpsc   = GPSC_SIRF;
			nGLBtn   = BST_UNCHECKED;
			nSiRFBtn = BST_CHECKED;
			break;
		}
	}
	while(--iDrive >= 0);
	if(iDrive >= 0)
	{
		ccPathName--;
		CopyMemory(m_pszPathName, m_pszFileName, ccPathName * sizeof(TCHAR));
		m_pszPathName[ccPathName] = CHR_NULL;
		m_jGLBtn.SetCheck(nGLBtn);
		m_jSiRFBtn.SetCheck(nSiRFBtn);
		m_jDriveName.SetCurSel(iDrive);
		m_jSavePath.SetWindowText(m_pszPathName);
		m_gpsfs = GPSFS_GETFILE;
		return;
	}

	LoadString(e_hInstance, IDS_RECOGNIZE_CHIP_ERROR, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	EnableControls(TRUE);
	m_gpsfs = GPSFS_STOP_GETCHIP;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS fix: gets saving path

AFXINLINE void CQGPSfixDlg::GfGetPath()
{
	TRACE(_T("Entering CQGPSfixDlg::GfGetPath\n"));

	int  ccPathName;
	UINT nResId;
	ccPathName = m_jSavePath.GetWindowText(m_pszPathName, MAX_PATH);
	CopyMemory(m_pszFileName, m_pszPathName, ccPathName * sizeof(TCHAR));
	m_pszFileName[ccPathName] = CHR_BSLH;
	if(m_jGLBtn.GetCheck() == BST_CHECKED)
	{
		m_gpsc = GPSC_GL;
		nResId = IDS_GL_FILENAME;
	}
	else
	{
		m_gpsc = GPSC_SIRF;
		nResId = IDS_SIRF_FILENAME;
	}
	LoadString(e_hInstance, nResId, m_pszFileName + ccPathName + 1, MAX_PATH);
	m_gpsfs = GPSFS_GETFILE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS fix: gets fix file

AFXINLINE void CQGPSfixDlg::GfGetFile()
{
	TRACE(_T("Entering CQGPSfixDlg::GfGetFile\n"));

	TCHAR     szResText[MAX_STR];
	UINT      nResId;
	HINTERNET hInternet;
	LoadString(e_hInstance, IDS_CONNECTING_TOMTOM_HOME, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	nResId    = IDS_CONNECT_TOMTOM_ERROR;
	hInternet = InternetOpen(e_pszTpeAppName, INTERNET_OPEN_TYPE_PRECONFIG);
	if(hInternet)
	{
		HHTTP hHttp;
		LoadString(e_hInstance, (m_gpsc == GPSC_GL) ? IDS_GL_EPHEM_URL : IDS_SIRF_EPHEM_URL,
			szResText, countof(szResText));
		hHttp = InternetOpenUrl(hInternet, szResText, INTERNET_FLAG_DONT_CACHE);
		if(hHttp)
		{
			HFILE hFile;
			LoadString(e_hInstance, IDS_GPSFIX_FILENAME,
				m_pszTempFile + GetTempPath(MAX_PATH, m_pszTempFile), MAX_PATH);
			hFile = CreateFile(m_pszTempFile, GENERIC_WRITE, CREATE_ALWAYS);
			if(hFile != INVALID_HANDLE)
			{
				char  szDlBuffer[m_ecbDlBuffer];
				TCHAR szReadInfo[MAX_STR];
				DWORD cbRead, cbDone;
				LoadString(e_hInstance, IDS_DOWNLOADING_EPHEM, szResText, countof(szResText));
				cbDone = 0;
				nResId = IDS_DISK_ERROR;
				do
				{
					if(e_pTPE->PeekUpdateMessage(TRUE))
					{
						PostMessage(WM_COMMAND, MAKEWPARAM(ID_CANCEL, BN_CLICKED));
						KillTimer(m_eTimerId);
						nResId = MAX_UINT;
						break;
					}
					wsprintf(szReadInfo, szResText, cbDone);
					m_jGPSfixInfo.SetWindowText(szReadInfo);
					InternetReadFile(hHttp, szDlBuffer, sizeof(szDlBuffer), &cbRead);
					if(!cbRead)
					{
						nResId = cbDone ? 0 : IDS_DOWNLOAD_EPHEM_ERROR;
						break;
					}
					cbDone += cbRead;
				}
				while(WriteFile(hFile, szDlBuffer, cbRead, &cbRead));
				CloseHandle(hFile);
			}
			else
				nResId = IDS_CANNOT_WRITE_EPHEM;
			InternetCloseHandle(hHttp);
		}
		InternetCloseHandle(hInternet);
	}
	if(!nResId)
	{
		m_gpsfs = GPSFS_SAVEFILE;
		return;
	}
	if(nResId == MAX_UINT)
		return;

	LoadString(e_hInstance, nResId, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	EnableControls(TRUE);
	m_gpsfs = GPSFS_STOP_GETFILE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS fix: saves fix file

AFXINLINE void CQGPSfixDlg::GfSaveFile()
{
	TRACE(_T("Entering CQGPSfixDlg::GfSaveFile\n"));

	TCHAR szResText[MAX_STR];
	LoadString(e_hInstance, IDS_UPDATING_EPHEM, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	if((IsValidFile(m_pszFileName) ? DeleteFile(m_pszFileName) : TRUE) &&
		(IsValidDirectory(m_pszPathName) || CreateDirectory(m_pszPathName)))
	{
		TCHAR szExeName[MAX_STR], szCmdLine[MAX_STR];
		LoadString(e_hInstance, IDS_DECOMPRESS_APP_NAME, szExeName, countof(szExeName));
		LoadString(e_hInstance, IDS_DECOMPRESS_CMDLINE, szResText, countof(szResText));
		wsprintf(szCmdLine, szResText, m_pszTempFile, m_pszPathName);
		ShellExecute(NULL, NULL, szExeName, szCmdLine, NULL, SW_HIDE);
		m_ceTick = GetTickCount();
		m_gpsfs  = GPSFS_CHECKFILE;
		return;
	}

	LoadString(e_hInstance, IDS_CANNOT_WRITE_EPHEM, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	EnableControls(TRUE);
	m_gpsfs = GPSFS_STOP_SAVEFILE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS fix: checks fix file

AFXINLINE void CQGPSfixDlg::GfCheckFile()
{
	TRACE(_T("Entering CQGPSfixDlg::GfCheckFile\n"));

	TCHAR szResText[MAX_STR];
	if(IsValidFile(m_pszFileName))
	{
		DWORD ceTick;
		HFILE hFile;
		ceTick = GetTickCount();
		do
			e_pTPE->PeekUpdateMessage(TRUE);
		while((GetTickCount() - ceTick) < m_eFlushTime);
		hFile = CreateFile(m_pszFileName, GENERIC_WRITE, OPEN_EXISTING);
		if(hFile != INVALID_HANDLE)
		{
			TCHAR      szExpFile[MAX_PATH];
			FILETIME   ft;
			SYSTEMTIME st;
			GetSystemTime(&st);
			SystemTimeToFileTime(&st, &ft);
			SetFileTime(hFile, &ft, &ft, &ft);
			CloseHandle(hFile);
			LoadString(e_hInstance, IDS_EXPIRY_FILENAME, szResText, countof(szResText));
			PathCombine(szExpFile, m_pszPathName, szResText);
			if(IsValidFile(szExpFile))
			{
				hFile = CreateFile(szExpFile, GENERIC_WRITE, OPEN_EXISTING);
				if(hFile != INVALID_HANDLE)
				{
					SetFileTime(hFile, &ft, &ft, &ft);
					CloseHandle(hFile);
					LoadString(e_hInstance, IDS_UPDATED_EPHEM, szResText, countof(szResText));
					m_jGPSfixInfo.SetWindowText(szResText);
					LoadString(e_hInstance, IDS_CLOSE_BUTTON, szResText, countof(szResText));
					m_jCancelBtn.SetWindowText(szResText);
					m_jCancelBtn.EnableWindow();
					m_jCancelBtn.SetFocus();
					KillTimer(m_eTimerId);
					e_pTPE->m_jProgress.HideWindow();
					e_pTPE->m_bSetReady = TRUE;
					return;
				}
			}
		}
	}
	else if((GetTickCount() - m_ceTick) < m_eUpdTimeout)
		return;

	LoadString(e_hInstance, IDS_UPDATE_EPHEM_ERROR, szResText, countof(szResText));
	m_jGPSfixInfo.SetWindowText(szResText);
	EnableControls(TRUE);
	m_gpsfs = GPSFS_STOP_SAVEFILE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts drives into combo box control

void CQGPSfixDlg::InsertDrives()
{
	TRACE(_T("Entering CQGPSfixDlg::InsertDrives\n"));

	TCHAR szDrive[MAX_STR], szSavePath[MAX_PATH];
	BOOL  bDropDown;
	int   i;
	bDropDown = m_jDriveName.GetDroppedState();
	if(bDropDown)
		m_jDriveName.HideDropDown();
	m_jDriveName.ResetContent();
	GetLogicalDriveStrings(countof(szDrive), szDrive);
	i = 0;
	do
	{
		m_jDriveName.AddItem(szDrive + i);
		i += lstrlen(szDrive + i) + 1;
	}
	while(szDrive[i]);
	m_jDriveName.SetCurSel(0);
	if(bDropDown)
		m_jDriveName.ShowDropDown();
	LoadString(e_hInstance, IDS_GPSFIX_DIRNAME,
		szSavePath + m_jDriveName.GetItem(0, szSavePath), countof(szSavePath));
	m_jSavePath.SetWindowText(szSavePath);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_GL_CHIP control sends a notification message

AFXINLINE void CQGPSfixDlg::OnGLChip(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnGLChip\n"));

	m_jSiRFBtn.SetCheck(BST_UNCHECKED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_SIRF_CHIP control sends a notification message

AFXINLINE void CQGPSfixDlg::OnSiRFChip(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnSiRFChip\n"));

	m_jGLBtn.SetCheck(BST_UNCHECKED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_DRIVE_NAME control sends a notification message

AFXINLINE void CQGPSfixDlg::OnDriveName(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnDriveName\n"));

	if(nNotify != CBN_SELCHANGE)
		return;

	TCHAR szSavePath[MAX_PATH];
	LoadString(e_hInstance, IDS_GPSFIX_DIRNAME,
		szSavePath + m_jDriveName.GetItem(m_jDriveName.GetCurSel(), szSavePath),
		countof(szSavePath));
	m_jSavePath.SetWindowText(szSavePath);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// IDC_SAVE control sends a notification message

AFXINLINE void CQGPSfixDlg::OnSave(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnSave\n"));

	EnableControls(FALSE);
	m_gpsfs = GPSFS_GETPATH;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_RETRY control sends a notification message

AFXINLINE void CQGPSfixDlg::OnRetry(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnRetry\n"));

	EnableControls(FALSE);
	m_gpsfs = (GPSFIXSTEP)(m_gpsfs & ~GPSFS_STOP);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// ID_CANCEL control sends a notification message

AFXINLINE void CQGPSfixDlg::OnCancel(UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnCancel\n"));

	if(IsValidFile(m_pszTempFile))
		DeleteFile(m_pszTempFile);
	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A change to the hardware configuration of a device or the computer

AFXINLINE UINT CQGPSfixDlg::OnDeviceChange(UINT nEvent, PCVOID pvStruct)
{
	TRACE(_T("Entering CQGPSfixDlg::OnDeviceChange\n"));

	if((nEvent == DBT_DEVICEARRIVAL) || (nEvent == DBT_DEVICEREMOVECOMPLETE))
	{
		InsertDrives();
		if(m_jRetryBtn.IsWindowEnabled())
			m_jRetryBtn.PostMessage(BM_CLICK);
	}

	// Accepts (TRUE) or rejects (BROADCAST_QUERY_DENY) the request
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CQGPSfixDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CQGPSfixDlg::OnInitDialog\n"));

	SetDialogIcon(*this, ID_TOOLS_QGPSFIX);
	m_jGPSfixInfo = GetChild(IDC_GPSFIX_INFO);
	m_jChipGroup  = GetChild(IDC_CHIP_GROUP);
	m_jGLBtn      = GetChild(IDC_GL_CHIP);
	m_jSiRFBtn    = GetChild(IDC_SIRF_CHIP);
	m_jSaveGroup  = GetChild(IDC_SAVE_GROUP);
	m_jDriveName  = GetChild(IDC_DRIVE_NAME);
	m_jSavePath   = GetChild(IDC_SAVE_PATH);
	m_jSaveBtn    = GetChild(IDC_SAVE);
	m_jRetryBtn   = GetChild(ID_RETRY);
	m_jCancelBtn  = GetChild(ID_CANCEL);
	m_jGLBtn.SetCheck();
	InsertDrives();
	EnableControls(FALSE);
	MoveDialogCenter();
	FadeInDialog(*this);
	m_gpsfs = GPSFS_GETCHIP;

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A timer interval expires

AFXINLINE void CQGPSfixDlg::OnTimer(UINT nTimerId)
{
	TRACE(_T("Entering CQGPSfixDlg::OnTimer\n"));

	switch(m_gpsfs)
	{
	case GPSFS_GETCHIP:
		GfGetChip();
		break;
	case GPSFS_GETPATH:
		GfGetPath();
		break;
	case GPSFS_GETFILE:
		GfGetFile();
		break;
	case GPSFS_SAVEFILE:
		GfSaveFile();
		break;
	case GPSFS_CHECKFILE:
		GfCheckFile();
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CQGPSfixDlg

BEGIN_MESSAGE_MAP(CQGPSfixDlg)
	BEGIN_COMMAND_MAP()
		ON_CONTROL(IDC_GL_CHIP, OnGLChip)
		ON_CONTROL(IDC_SIRF_CHIP, OnSiRFChip)
		ON_CONTROL(IDC_DRIVE_NAME, OnDriveName)
		ON_CONTROL(IDC_SAVE, OnSave)
		ON_CONTROL(ID_RETRY, OnRetry)
		ON_CONTROL(ID_CANCEL, OnCancel)
	END_COMMAND_MAP()
	ON_WM_DEVICECHANGE(OnDeviceChange)
	ON_WM_INITDIALOG(OnInitDialog)
	ON_WM_TIMER(OnTimer)
END_MESSAGE_MAP()
