// CQGPSfixDlg.hpp - Quick GPS Fix Dialog Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CQGPSFIXDLG_INCLUDED_
#define _CQGPSFIXDLG_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CQGPSfixDlg - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

class CQGPSfixDlg : public CDialog
{
public:
	// Chip type (gpsc)
	enum GPSCHIP
	{
		// Global Locate
		GPSC_GL = 0,

		// SiRF Star III
		GPSC_SIRF,
	};

	// GPS fix step (gpsfs)
	enum GPSFIXSTEP
	{
		GPSFS_GETCHIP = 0,
		GPSFS_GETPATH,
		GPSFS_GETFILE,
		GPSFS_SAVEFILE,
		GPSFS_CHECKFILE,
		GPSFS_STOP          = 0x80000000,
		GPSFS_STOP_GETCHIP  = GPSFS_GETCHIP  | GPSFS_STOP,
		GPSFS_STOP_GETFILE  = GPSFS_GETFILE  | GPSFS_STOP,
		GPSFS_STOP_SAVEFILE = GPSFS_SAVEFILE | GPSFS_STOP,
	};

	enum
	{
		// Size of download buffer
		m_ecbDlBuffer = 1024,

		// Timer identifier and its interval
		m_eTimerId   = 7,
		m_eTimerIntl = 500,

		// Waits for flushing files
		m_eFlushTime = 2000,

		// Timeout when updated GPS file
		m_eUpdTimeout = 7000,
	};

public:
	PTSTR      m_pszPathName, m_pszFileName, m_pszTempFile;
	GPSCHIP    m_gpsc;
	GPSFIXSTEP m_gpsfs;
	DWORD      m_ceTick;
	CButton    m_jChipGroup, m_jGLBtn, m_jSiRFBtn, m_jSaveGroup;
	CButton    m_jSaveBtn, m_jRetryBtn, m_jCancelBtn;
	CComboBox  m_jDriveName;
	CStatic    m_jGPSfixInfo, m_jSavePath;

public:
	CQGPSfixDlg(PTSTR pszPathName, PTSTR pszFileName, PTSTR pszTempFile);
	void EnableControls(BOOL bEnable);
	void GfGetChip();
	void GfGetPath();
	void GfGetFile();
	void GfSaveFile();
	void GfCheckFile();
	void InsertDrives();

protected:
	void OnGLChip(UINT nNotify, HWND hCtrlWnd);
	void OnSiRFChip(UINT nNotify, HWND hCtrlWnd);
	void OnDriveName(UINT nNotify, HWND hCtrlWnd);
	void OnSave(UINT nNotify, HWND hCtrlWnd);
	void OnRetry(UINT nNotify, HWND hCtrlWnd);
	void OnCancel(UINT nNotify, HWND hCtrlWnd);
	UINT OnDeviceChange(UINT nEvent, PCVOID pvStruct);
	BOOL OnInitDialog(HWND hFocusWnd);
	void OnTimer(UINT nTimerId);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CQGPSfixDlg::CQGPSfixDlg(PTSTR pszPathName, PTSTR pszFileName, PTSTR pszTempFile)
{
	TRACE(_T("Entering CQGPSfixDlg::CQGPSfixDlg\n"));

	m_pszPathName  = pszPathName;
	m_pszFileName  = pszFileName;
	m_pszTempFile  = pszTempFile;
	*m_pszTempFile = CHR_NULL;
}

// End of _CQGPSFIXDLG_INCLUDED_
#endif
