// CSaveDeviceDlg.cpp - Save Device Dialog Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CSaveDeviceDlg.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from a menu or a control sends a notification

AFXINLINE void CSaveDeviceDlg::OnCommand(UINT nItemId, UINT nNotify, HWND hCtrlWnd)
{
	TRACE(_T("Entering CSaveDeviceDlg::OnCommand\n"));

	int i;
	if((nItemId != ID_OK) && (nItemId != ID_CANCEL))
	{
		BOOL bEnable;
		ASSERT((nItemId >= IDC_GPS_OV2_FILE) && (nItemId <= IDC_GPS_VIF_FILE));
		i = GPSF_FIRST;
		do
		{
			if(m_jButton[i].IsWindowEnabled() && (m_jButton[i].GetCheck() == BST_CHECKED))
				break;
			i++;
		}
		while(i < GPSF_COUNT);
		bEnable = i < GPSF_COUNT;
		if(bEnable != m_jOKBtn.IsWindowEnabled())
			m_jOKBtn.EnableWindow(bEnable);
		return;
	}

	ASSERT((nItemId == ID_OK) || (nItemId == ID_CANCEL));
	if(nItemId == ID_OK)
	{
		UINT nSaveFlags;
		i          = GPSF_COUNT;
		nSaveFlags = 0;
		do
		{
			i--;
			nSaveFlags <<= 1;
			if(!m_jButton[i].IsWindowEnabled())
				continue;
			if(m_jButton[i].GetCheck() == BST_CHECKED)
				nSaveFlags |= 1;
		}
		while(i);
		ASSERT(nSaveFlags);
		*m_pnSaveFlags = nSaveFlags;
	}

	FadeOutDialog(*this);
	DestroyIcon(SetWindowIcon(ICON_SMALL));
	EndDialog(nItemId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Dialog box is being displayed

AFXINLINE BOOL CSaveDeviceDlg::OnInitDialog(HWND hFocusWnd)
{
	TRACE(_T("Entering CSaveDeviceDlg::OnInitDialog\n"));

	PTSTR pszFileExt;
	TCHAR szFileName[MAX_PATH];
	BOOL  bValidFile;
	int   i, j;
	SetDialogIcon(*this, ID_FILE_SAVE_DEVICE);
	lstrcpy(szFileName, m_pszFileName);
	pszFileExt = PathFindExtension(szFileName);
	if(!*pszFileExt)
		*pszFileExt = CHR_DOT;
	pszFileExt++;
	i = GPSF_OV2;
	j = IDC_GPS_OV2_FILE;
	do
	{
		m_jButton[i] = GetChild(j);
		if(i == GPSF_OV2)
			bValidFile = TRUE;
		else
		{
			CopyMemory(pszFileExt, CTomPoiEdr::m_pszGPSF[i], sizeof(CTomPoiEdr::m_szOv2Ext));
			bValidFile = IsValidFile(szFileName);
		}
		m_jButton[i].EnableWindow(bValidFile);
		i++;
		j++;
	}
	while(i < countof(m_jButton));
	m_jButton[GPSF_OV2].SetCheck();
	m_jOKBtn = GetChild(ID_OK);
	MoveDialogCenter();
	FadeInDialog(*this);

	// System (TRUE) or application itself (FALSE) sets the keyboard focus
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CSaveDeviceDlg

BEGIN_MESSAGE_MAP(CSaveDeviceDlg)
	ON_WM_COMMAND(OnCommand)
	ON_WM_INITDIALOG(OnInitDialog)
END_MESSAGE_MAP()
