// CTbCtrl.cpp - Toolbar Control Implementation File
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include "CTomPoiEdr.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds the index to button structures associated with a command

int CTbCtrl::FindButton(UINT nCmdId)
{
	TRACE(_T("Entering CTbCtrl::FindButton\n"));

	PTBBUTTON pStartTBB, pMatchTBB;
	ASSERT((nCmdId >= ID_FIRST_MENU_ITEM) && (nCmdId <= ID_LAST_MENU_ITEM));
	pStartTBB = e_pTPE->m_tbb;
	pMatchTBB = pStartTBB;
	while(nCmdId != pMatchTBB->idCommand)
		pMatchTBB++;
	return (pMatchTBB - pStartTBB);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Checks or unchecks a button

BOOL CTbCtrl::CheckButton(UINT nCmdId, BOOL bCheck /* = TRUE */)
{
	TRACE(_T("Entering CTbCtrl::CheckButton\n"));

	if(CommandToIndex(nCmdId) != RV_ERR)
		return CToolbar::CheckButton(nCmdId, bCheck);

	PBYTE pbyState;
	pbyState = &e_pTPE->m_tbb[FindButton(nCmdId)].fsState;
	if(bCheck)
		*pbyState |= TBSTATE_CHECKED;
	else
		*pbyState &= ~TBSTATE_CHECKED;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables or disables a button

BOOL CTbCtrl::EnableButton(UINT nCmdId, BOOL bEnable /* = TRUE */)
{
	TRACE(_T("Entering CTbCtrl::EnableButton\n"));

	if(CommandToIndex(nCmdId) != RV_ERR)
		return CToolbar::EnableButton(nCmdId, bEnable);

	PBYTE pbyState;
	pbyState = &e_pTPE->m_tbb[FindButton(nCmdId)].fsState;
	if(bEnable)
		*pbyState |= TBSTATE_ENABLED;
	else
		*pbyState &= ~TBSTATE_ENABLED;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a button is enabled

BOOL CTbCtrl::IsButtonEnabled(UINT nCmdId)
{
	TRACE(_T("Entering CTbCtrl::IsButtonEnabled\n"));

	if(CommandToIndex(nCmdId) != RV_ERR)
		return CToolbar::IsButtonEnabled(nCmdId);

	return ((e_pTPE->m_tbb[FindButton(nCmdId)].fsState & TBSTATE_ENABLED) ? TRUE : FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a button is pressed

BOOL CTbCtrl::IsButtonPressed(UINT nCmdId)
{
	TRACE(_T("Entering CTbCtrl::IsButtonPressed\n"));

	if(CommandToIndex(nCmdId) != RV_ERR)
		return CToolbar::IsButtonPressed(nCmdId);

	return ((e_pTPE->m_tbb[FindButton(nCmdId)].fsState & TBSTATE_PRESSED) ? TRUE : FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Presses or releases a button

BOOL CTbCtrl::PressButton(UINT nCmdId, BOOL bPress /* = TRUE */)
{
	TRACE(_T("Entering CTbCtrl::PressButton\n"));

	if(CommandToIndex(nCmdId) != RV_ERR)
		return CToolbar::PressButton(nCmdId, bPress);

	PBYTE pbyState;
	pbyState = &e_pTPE->m_tbb[FindButton(nCmdId)].fsState;
	if(bPress)
		*pbyState |= TBSTATE_PRESSED;
	else
		*pbyState &= ~TBSTATE_PRESSED;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left or right mouse button has pressed/double clicked

AFXINLINE LRESULT CTbCtrl::OnLRBtnDblClkDown(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTbCtrl::OnLRBtnDblClkDown\n"));

	e_pTPE->UpdateTbButton();
	if(!e_pTPE->m_jLvEdit.IsWindowVisible())
		return WMR_DEFAULT;

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	e_pTPE->m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
	return (e_pTPE->m_jLvEdit.m_bTextOK ? WMR_DEFAULT : WMR_PROCESSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse cursor moves in the client area of a window

AFXINLINE LRESULT CTbCtrl::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTbCtrl::OnMouseMove\n"));

	if(e_pTPE->m_jOptionDlg.m_nWrapTb == BST_CHECKED)
		return WMR_DEFAULT;

	int iButton;
	iButton = ButtonCount();
	if(!iButton)
		return WMR_DEFAULT;

	RECT     rc;
	TBBUTTON tbb;
	do
	{
		iButton--;
		GetButton(iButton, &tbb);
		if(tbb.idCommand)
		{
			GetItemRect(iButton, &rc);
			if(rc.right <= e_pTPE->m_cxClient)
				return WMR_DEFAULT;

			SetRect(&rc, e_pTPE->m_cxClient - CTomPoiEdr::m_ecxTbChevron,
				-CTomPoiEdr::m_eyTbPad, e_pTPE->m_cxClient,
				e_pTPE->m_cyToolbar - CTomPoiEdr::m_eyTbPad);
			if(PtInRect(&rc, MAKEPOINT(lParam)))
				SetCursor(LoadCursor(e_hInstance, IDC_TOOLBAR_CHEVRON));
			return WMR_DEFAULT;
		}
	}
	while(iButton);

	// Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
	return WMR_DEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Size and position of a window's client area must be calculated

AFXINLINE int CTbCtrl::OnNcCalcSize(BOOL bCalcRects, LPARAM lParam)
{
	TRACE(_T("Entering CTbCtrl::OnNcCalcSize\n"));

	if(!bCalcRects)
		return 0;

	((PNCCALCSIZE_PARAMS)lParam)->rgrc[0].top += CTomPoiEdr::m_eyTbPad;
	return (WVR_ALIGNLEFT | WVR_ALIGNTOP);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Paints the frame of a window

AFXINLINE void CTbCtrl::OnNcPaint(HRGN hUpdateRgn)
{
	TRACE(_T("Entering CTbCtrl::OnNcPaint\n"));

	RECT   rc;
	HBRUSH hBrush;
	HDC    hDC;
	hDC = GetWindowDC();
	if(e_dwWinVer <= WV_XP)
	{
		SelectPen(hDC, CreatePen(PS_SOLID, 1, CTomPoiEdr::m_ecrTbBorderXP));
		MoveTo(hDC, 0, 0);
		LineTo(hDC, e_pTPE->m_cxClient, 0);
		DeletePen(SelectPen(hDC, GetStockPen(WHITE_PEN)));
		MoveTo(hDC, 0, 1);
		LineTo(hDC, e_pTPE->m_cxClient, 1);
	}
	hBrush = CreateSolidBrush(e_pTPE->m_crTbBkgnd);
	SetRect(&rc, 0, 2, e_pTPE->m_cxClient, CTomPoiEdr::m_eyTbPad);
	FillRect(hDC, &rc, hBrush);
	DeleteBrush(hBrush);
	ReleaseDC(hDC);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A tooltip is about to be hidden

AFXINLINE void CTbCtrl::OnTtnPop(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTbCtrl::OnTtnPop\n"));

	e_pTPE->m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CTbCtrl

BEGIN_MESSAGE_MAP(CTbCtrl)
	ON_SAME_MAP(WM_LBUTTONDBLCLK)
	ON_SAME_MAP(WM_LBUTTONDOWN)
	ON_ANY_MAP(WM_RBUTTONDOWN, OnLRBtnDblClkDown)
	ON_ANY_MAP(WM_MOUSEMOVE, OnMouseMove)
	ON_WM_NCCALCSIZE(OnNcCalcSize)
	ON_WM_NCPAINT(OnNcPaint)
	BEGIN_NOTIFY_MAP()
		ON_TTN_POP(OnTtnPop)
	END_NOTIFY_MAP()
END_MESSAGE_MAP()
