// WinMain.hpp - Windows WinMain Declaration File
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _WINMAIN_INCLUDED_
#define _WINMAIN_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  WinMain - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

// Major version of Windows
enum
{
	WV_XP = 5,
};

// List view editing column (LVCOL, POI)
enum
{
	LVC_FIRST = 0,
	LVC_NAME  = LVC_FIRST,
	LVC_LNG,
	LVC_LAT,
	LVC_TEL,
	LVC_MEMO,
	LVC_DIST,
	LVC_COUNT,
	LVC_SPEED  = LVC_TEL,
	LVC_CAMERA = LVC_MEMO,
	LVC_CPBT   = LVC_MEMO,
};

// List view editing column, reorder (LVCOLR, POI)
enum
{
	LVCR_FIRST = 0,
	LVCR_DIST  = LVCR_FIRST,
	LVCR_NAME,
	LVCR_LNG,
	LVCR_LAT,
	LVCR_TEL,
	LVCR_MEMO,
	LVCR_COUNT,
};

// List view editing column (LVCOLI, ITN)
enum
{
	LVCI_FIRST = 0,
	LVCI_NAME  = LVCI_FIRST,
	LVCI_LNG,
	LVCI_LAT,
	LVCI_COUNT,
};

// Automatic sizes column widths of list view control (CListView::AutoSizeLvCol)
enum
{
	LVC_ALL = LVC_COUNT, LVCA_FIRST, LVCA_ALL = LVCA_FIRST + LVC_COUNT,
};

// Range of coordinates (COORDRANGE)
enum
{
	CR_FIRST    = 0,
	CR_LNG_FROM = CR_FIRST,
	CR_LNG_TO,
	CR_LAT_FROM,
	CR_LAT_TO,
	CR_COUNT,
};

enum
{
	// Fades in/out the window/dialog
	e_eceFadeStep  = 16,
	e_eceFadeSleep = 20,
	e_eFadeAlpha   = MAX_BYTE - 12,

	// Minimum size of main window
	e_ecxMinWnd = 480,
	e_ecyMinWnd = 320,

	// Maximum number of characters, including null character
	e_eccNameCol = MAX_STR,
	e_eccLngCol  = 24,
	e_eccLatCol  = e_eccLngCol,
	e_eccTelCol  = 19,
	e_eccMemoCol = e_eccNameCol,
	e_eccAllCol  = e_eccNameCol + e_eccLngCol + e_eccLatCol + e_eccTelCol + e_eccMemoCol,
	e_eccTextRd  = e_eccNameCol + e_eccTelCol + e_eccMemoCol,

	// Pumps for coordinates
	e_eCoordPump = 100000,

	// Coordinate limit
	e_eMinLng = -180 * e_eCoordPump,
	e_eMaxLng = 180 * e_eCoordPump,
	e_eMinLat = -90 * e_eCoordPump,
	e_eMaxLat = 90 * e_eCoordPump,
};

// Base point of list view item
enum
{
	LVI_FIND = -1, LVI_ERR = RV_ERR, LVI_ALL = -1,
};

// Major version of Windows
EXTERN DWORD e_dwWinVer;

// General strings
EXTERN CWCHAR e_szAppTheme[];
EXTERN CTCHAR e_szNull[];
EXTERN CTCHAR e_szLeft[];
EXTERN CTCHAR e_szTop[];
EXTERN CTCHAR e_szWidth[];
EXTERN CTCHAR e_szHeight[];
EXTERN CTCHAR e_szCoordFmt[];
EXTERN CTCHAR e_szDistKmFmt[];
EXTERN CTCHAR e_szDistMFmt[];
EXTERN CTCHAR e_szDistMiFmt[];
EXTERN CTCHAR e_szDistFtFmt[];
EXTERN PCTSTR e_pszDistKm;
EXTERN PCTSTR e_pszDistM;
EXTERN PCTSTR e_pszDistMi;
EXTERN PCTSTR e_pszDistFt;

// Profile section
EXTERN CTCHAR e_szProfileSection[];

// Window section in profile
EXTERN CTCHAR e_szWndSection[];
EXTERN CTCHAR e_szWndShowEntry[];

// General variables
class CTomPoiEdr;
EXTERN HINSTANCE   e_hInstance;
EXTERN PCTSTR      e_pszProfile;
EXTERN PCTSTR      e_pszTpeAppName;
EXTERN PCTSTR      e_pszTpeClassName;
EXTERN PCTSTR      e_pszTpeVersion;
EXTERN PCTSTR      e_pszTieAppName;
EXTERN PCTSTR      e_pszTieClassName;
EXTERN CTomPoiEdr* e_pTPE;

// Header item order
EXTERN int e_iHdOrder[LVC_COUNT];

// Coordinate limit
EXTERN CLONG e_lCoordLimit[CR_COUNT];

// Fades in/out the window/dialog
void WINAPI FadeInWindow(CWnd jWnd, int iStartAlpha, int iFinalAlpha, int iShowCmd);
void WINAPI FadeOutWindow(CWnd jWnd, int iStartAlpha, int iFinalAlpha);
AFXINLINE void WINAPI FadeInDialog(CWnd jWnd)
{
	FadeInWindow(jWnd, 0, e_eFadeAlpha, SW_SHOW);
}
AFXINLINE void WINAPI FadeOutDialog(CWnd jWnd)
{
	FadeOutWindow(jWnd, e_eFadeAlpha, 0);
}

// Sets a coordinate with compass direction
int WINAPI SetCoordCompassDir(int iLvCol, PTSTR pszCoord, int ccCoord);

// Removes compass direction from a coordinate
void WINAPI RemoveCoordCompassDir(PTSTR pszCoord);

// Gets a coordinate string in DD/DMS format
int WINAPI GetCoordDdString(int iLvCol, PTSTR pszCoord, int ccCoord);
int WINAPI GetCoordDdString(PTSTR pszCoord, int ccCoord);
int WINAPI GetCoordDmsString(int iLvCol, double dCoord, PTSTR pszBuffer);

// Gets a coordinate string in a specific format
int WINAPI GetCoordString(int iLvCol, double dCoord, BOOL bDepump, PTSTR pszBuffer);
int WINAPI GetCoordString(int iLvCol, double dCoord, PTSTR pszBuffer);
AFXINLINE int WINAPI GetCoordString(int iLvCol, LONG lCoord, PTSTR pszBuffer)
{
	return GetCoordString(iLvCol, lCoord, TRUE, pszBuffer);
}
int WINAPI GetCoordString(int iLvCol, PCTSTR pszCoord, PTSTR pszBuffer);
int WINAPI GetCoordString(int iLvCol, PTSTR pszCoord);

// Gets a coordinate value
double WINAPI GetCoordValueD(PTSTR pszCoord);
LONG WINAPI GetCoordValueL(PTSTR pszCoord);
double WINAPI GetCoordDmsValue(PTSTR pszCoord);

// Gets a distance string
void WINAPI GetDistanceString(LPARAM lParam, PTSTR pszBuffer);

// Determines whether a string is a valid float value
BOOL WINAPI IsValidFloat(PTSTR pszText);

// Determines whether a character is a valid number
AFXINLINE BOOL WINAPI IsValidNumber(TCHAR cChar)
{
	return ((cChar >= _T('0')) && (cChar <= _T('9')));
}

// Determines whether a character is a valid telephone sign
BOOL WINAPI IsValidTelSign(TCHAR cChar);

// Loads a bitmap from a file
HBITMAP WINAPI LoadBitmap(PCTSTR pszFileName);

// Normalizes a coordinate string
BOOL WINAPI NormalizeCoord(int iLvCol, PCTSTR pszCoord, PTSTR pszBuffer);

// Sets an icon to associate with a dialog
void WINAPI SetDialogIcon(HWND hDialog, UINT nItemId);

// End of _WINMAIN_INCLUDED_
#endif
