////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// CPl.hpp - Control Panel Declaration File                                                   //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CPL_INCLUDED_
#define _CPL_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CPl - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// General purposes

// Exports required function name
EXTERN_C LONG APIENTRY CPlApplet(HWND hCPlWnd, UINT nMsg, LPARAM lParam1, LPARAM lParam2);
#pragma comment(linker, "/EXPORT:CPlApplet=_CPlApplet@16")

// Pointers
typedef CPLINFO*    PCPLINFO;
typedef NEWCPLINFO* PNEWCPLINFO;

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps

// Declares in Control Panel class
#define DECLARE_CTRLPANEL_MAP() \
	public: \
		LONG _CPlApplet(UINT nMsg, LPARAM lParam1, LPARAM lParam2);

// Implements Control Panel object
// When creates object by DYNAMIC storgae class, deletes the object in OnCPlExit
// CName   : Name of Control Panel's window class
// Storage : Storage class (DYNAMIC or STATIC) of window class
#define IMPLEMENT_CTRLPANEL(CName, Storage) _IMPLEMENT_CTRLPANEL_##Storage(CName)

#define _IMPLEMENT_CTRLPANEL_DYNAMIC(CName) \
	CName* _i_pCPl = VERIFY(new CName); \
	LONG APIENTRY CPlApplet(HWND hCPlWnd, UINT nMsg, LPARAM lParam1, LPARAM lParam2) \
	{ \
		_i_pCPl->m_hWnd = hCPlWnd; \
		return _i_pCPl->_CPlApplet(nMsg, lParam1, lParam2); \
	}

#define _IMPLEMENT_CTRLPANEL_STATIC(CName) \
	CName _i_jCPl; \
	LONG APIENTRY CPlApplet(HWND hCPlWnd, UINT nMsg, LPARAM lParam1, LPARAM lParam2) \
	{ \
		_i_jCPl.m_hWnd = hCPlWnd; \
		return _i_jCPl._CPlApplet(nMsg, lParam1, lParam2); \
	}

// Begins/ends Control Panel maps
// CName : Name of Control Panel class
#define BEGIN_CTRLPANEL_MAP(CName) \
	AFXINLINE LONG CName::_CPlApplet(UINT nMsg, LPARAM lParam1, LPARAM lParam2) \
	{ \
		switch(nMsg) \
		{

#define END_CTRLPANEL_MAP() \
		} \
		return _eResult; \
	}

// The user double-clicks the icon of a dialog box
#define ON_CPL_DBLCLK(fName) \
	case CPL_DBLCLK: \
		fName(lParam1); \
		return _eResult;

// Control Panel is being closed
#define ON_CPL_EXIT(fName) \
	case CPL_EXIT: \
		fName(); \
		return _eResult;

// Gets number of dialog boxes the Control Panel supports
// Returns: Number of dialog boxes
#define ON_CPL_GETCOUNT(fName) \
	case CPL_GETCOUNT: \
		return fName();

// Initializes the Control Panel
// Returns: Initialization is successful (TRUE) or not (FALSE)
#define ON_CPL_INIT(fName) \
	case CPL_INIT: \
		return fName();

// Requests information the Control Panel supports
// Returns: Information is supported (0) or not (nonzero)
#define ON_CPL_INQUIRE(fName) \
	case CPL_INQUIRE: \
		return fName(lParam1, (PCPLINFO)lParam2);

// Requests information the Control Panel supports
// Returns: Information is supported (0) or not (nonzero)
#define ON_CPL_NEWINQUIRE(fName) \
	case CPL_NEWINQUIRE: \
		return fName(lParam1, (PNEWCPLINFO)lParam2);

// The user has chosen the icon associated with a dialog box
// Returns: Message is handled (TRUE) or not (FALSE)
#define ON_CPL_STARTWPARMS(fName) \
	case CPL_STARTWPARMS: \
		return fName(lParam1, (PCTSTR)lParam2);

// A dialog box is being closed
#define ON_CPL_STOP(fName) \
	case CPL_STOP: \
		fName(lParam1); \
		return _eResult;

////////////////////////////////////////////////////////////////////////////////////////////////
// CCtrlPanel

// Only can be derived (cannot be instantiated), and only one derived object can be created
class CCtrlPanel : public CWnd
{
protected:
	CCtrlPanel();
};

AFXINLINE CCtrlPanel::CCtrlPanel()
{
	// Does nothing
}

// End of _CPL_INCLUDED_
#endif
