////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// CommDlg.hpp - Common Dialog Boxes Declaration File                                         //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _COMMDLG_INCLUDED_
#define _COMMDLG_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CommDlg - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Pointers
typedef const CHOOSEFONT* PCCHOOSEFONT;
typedef const OFNOTIFY*   PCOFNOTIFY;
typedef const OFNOTIFYEX* PCOFNOTIFYEX;

////////////////////////////////////////////////////////////////////////////////////////////////
// General purposes

// Gets the dialog procedure of a class object
// Returns: Address of the procedure
#define GetCommonDialogProc(CName) ((LPOFNHOOKPROC)CName::_WindowProc)

// Help button is clicked
// void OnCdmHelpMsg(HWND hDlg, PCVOID pvStruct);
#define ON_CDM_HELPMSG(jDlgName, fName) \
	_ON_WM_REGISTER_V(jDlgName.m_nHelpMsg, fName, ((HWND)wParam, (PCVOID)lParam))

// Gets the buffer size required for a file name
// Returns: Required buffer size
AFXINLINE short WINAPI GetFileTitle(PCTSTR pszFileName)
{
	short ccBuffer = GetFileTitle(pszFileName, NULL, UNUSED);
	ASSERT(ccBuffer > 0);
	return ccBuffer;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CColorDialog

// A color is selected and the OK button is clicked
// BOOL OnCdmColorOK();
// Returns: Accepts (FALSE) or rejects (TRUE) the selected color
#define ON_CDM_COLOROK(fName) _ON_WM_REGISTER_R(m_nColorMsg, fName, ())

// Only can be derived (cannot be instantiated)
class CColorDialog : public CDialog, public CHOOSECOLOR
{
public:
	// Registered messages
	UINT m_nColorMsg, m_nHelpMsg, m_nRgbMsg;

public:
	BOOL ChooseColor();
	BOOL RegisterColorOK();
	BOOL RegisterHelpMsg();
	BOOL RegisterSetRgb();
	void SetRgb(COLORREF crValue) const;

protected:
	CColorDialog();
};

AFXINLINE CColorDialog::CColorDialog()
{
#ifdef DEBUG
	m_nColorMsg = 0;
	m_nHelpMsg  = 0;
	m_nRgbMsg   = 0;
#endif
}

AFXINLINE BOOL CColorDialog::ChooseColor()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(CHOOSECOLOR)) && (Flags & CC_ENABLEHOOK));
	_m_pWnd = this;
	return ::ChooseColor(this);
}

AFXINLINE BOOL CColorDialog::RegisterColorOK()
{
	ASSERT(!m_nColorMsg);
	m_nColorMsg = ::RegisterWindowMessage(COLOROKSTRING);
	return VERIFY(m_nColorMsg);
}

AFXINLINE BOOL CColorDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

AFXINLINE BOOL CColorDialog::RegisterSetRgb()
{
	ASSERT(!m_nRgbMsg);
	m_nRgbMsg = ::RegisterWindowMessage(SETRGBSTRING);
	return VERIFY(m_nRgbMsg);
}

AFXINLINE void CColorDialog::SetRgb(COLORREF crValue) const
{
	ASSERT(m_nRgbMsg);
	::SendMessage(m_hWnd, m_nRgbMsg, UNUSED, crValue);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CFileDialog

// A file name is selected and the OK button is clicked
// BOOL OnCdmFileOK();
// Returns: Accepts (FALSE) or rejects (TRUE) the selected file name
#define ON_CDM_FILEOK(fName) _ON_WM_REGISTER_R(m_nFileMsg, fName, ())

// Selection changes in any of the combo boxes or list boxes of the dialog box
// void OnCdmLbSelCh(UINT nCtrlId, int iItem, int iType);
#define ON_CDM_LBSELCH(fName) \
	_ON_WM_REGISTER_V(m_nLbSelMsg, fName, (wParam, LOSHORT(lParam), HISHORT(lParam)))

// A sharing violation occurs for the selected file when the user clicks the OK button
// int OnCdmShareVi();
// Returns: Value to indicate how to handle the sharing violation
#define ON_CDM_SHAREVI(fName) _ON_WM_REGISTER_R(m_nShareMsg, fName, ())

// The user specifies a file name and clicks the OK button
// BOOL OnCdnFileOK(PCOFNOTIFY pOFN);
// Returns: Accepts (FALSE) or rejects (TRUE) the file name
#define ON_CDN_FILEOK(fName) _ON_NOTIFY_R(CDN_FILEOK, fName, PCOFNOTIFY)

// A new folder is opened
// void OnCdnFolderChange(PCOFNOTIFY pOFN);
#define ON_CDN_FOLDERCHANGE(fName) _ON_NOTIFY_V(CDN_FOLDERCHANGE, fName, PCOFNOTIFY)

// The user clicks the Help button
// void OnCdnHelp(PCOFNOTIFY pOFN);
#define ON_CDN_HELP(fName) _ON_NOTIFY_V(CDN_HELP, fName, PCOFNOTIFY)

// Determines whether to display an item in a shell folder's item list
// BOOL OnCdnIncludeItem(PCOFNOTIFYEX pOFNX);
// Returns: Excludes (FALSE) or includes (TRUE) the item
#define ON_CDN_INCLUDEITEM(fName) _ON_NOTIFY_R(CDN_INCLUDEITEM, fName, PCOFNOTIFYEX)

// The system has finished arranging the controls in the dialog box
// void OnCdnInitDone(PCOFNOTIFY pOFN);
#define ON_CDN_INITDONE(fName) _ON_NOTIFY_V(CDN_INITDONE, fName, PCOFNOTIFY)

// The selection changes in the contents of the opened folder or directory
// void OnCdnSelChange(PCOFNOTIFY pOFN);
#define ON_CDN_SELCHANGE(fName) _ON_NOTIFY_V(CDN_SELCHANGE, fName, PCOFNOTIFY)

// A sharing violation occurs for the selected file when the user clicks the OK button
// int OnCdnShareViolation(PCOFNOTIFY pOFN);
// Returns: The value to indicate how to handle the sharing violation
#define ON_CDN_SHAREVIOLATION(fName) _ON_NOTIFY_R(CDN_SHAREVIOLATION, fName, PCOFNOTIFY)

// The user selects a new file type from the file types combo box
// void OnCdnTypeChange(PCOFNOTIFY pOFN);
#define ON_CDN_TYPECHANGE(fName) _ON_NOTIFY_V(CDN_TYPECHANGE, fName, PCOFNOTIFY)

// Only can be derived (cannot be instantiated)
// Parent handle of dialog is set by SetParent, if necessary
class CFileDialog : public CDialog, public OPENFILENAME
{
public:
	// Explorer-style Open and Save As dialog box
	HWND m_hParentDlg;

	// Registered messages
	UINT m_nFileMsg, m_nHelpMsg, m_nLbSelMsg, m_nShareMsg;

public:
	int GetFilePath(PTSTR pszBuffer, int ccBuffer) const;
	int GetFolderIdList(PVOID pvBuffer, int cbBuffer) const;
	int GetFolderPath(PTSTR pszBuffer, int ccBuffer) const;
	BOOL GetOpenFileName();
	BOOL GetSaveFileName();
	int GetSpec(PTSTR pszBuffer, int ccBuffer) const;
	void HideControl(UINT nCtrlId) const;
	BOOL RegisterFileOK();
	BOOL RegisterHelpMsg();
	BOOL RegisterLbSelCh();
	BOOL RegisterShareVi();
	void SetControlText(UINT nCtrlId, PCSTR pszText) const;
	void SetDefExt(PCSTR pszFileExt) const;
	void SetParent(HWND hParentDlg);

protected:
	CFileDialog();
};

AFXINLINE CFileDialog::CFileDialog()
{
#ifdef DEBUG
	m_hParentDlg = NULL;
	m_nFileMsg   = 0;
	m_nHelpMsg   = 0;
	m_nLbSelMsg  = 0;
	m_nShareMsg  = 0;
#endif
}

AFXINLINE int CFileDialog::GetFilePath(PTSTR pszBuffer, int ccBuffer) const
{
	ASSERT_HWND(m_hParentDlg);
	int iResult = ::SendMessage(m_hParentDlg, CDM_GETFILEPATH, ccBuffer, (LPARAM)pszBuffer);
	ASSERT((0 <= iResult) && (iResult <= ccBuffer));
	return iResult;
}

AFXINLINE int CFileDialog::GetFolderIdList(PVOID pvBuffer, int cbBuffer) const
{
	ASSERT_HWND(m_hParentDlg);
	int iResult = ::SendMessage(m_hParentDlg, CDM_GETFOLDERIDLIST, cbBuffer, (LPARAM)pvBuffer);
	ASSERT((0 <= iResult) && (iResult <= cbBuffer));
	return iResult;
}

AFXINLINE int CFileDialog::GetFolderPath(PTSTR pszBuffer, int ccBuffer) const
{
	ASSERT_HWND(m_hParentDlg);
	int iResult = ::SendMessage(m_hParentDlg, CDM_GETFOLDERPATH, ccBuffer, (LPARAM)pszBuffer);
	ASSERT((0 <= iResult) && (iResult <= ccBuffer));
	return iResult;
}

AFXINLINE BOOL CFileDialog::GetOpenFileName()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(OPENFILENAME)) && (Flags & OFN_ENABLEHOOK));
	_m_pWnd = this;
	return ::GetOpenFileName(this);
}

AFXINLINE BOOL CFileDialog::GetSaveFileName()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(OPENFILENAME)) && (Flags & OFN_ENABLEHOOK));
	_m_pWnd = this;
	return ::GetSaveFileName(this);
}

AFXINLINE int CFileDialog::GetSpec(PTSTR pszBuffer, int ccBuffer) const
{
	ASSERT_HWND(m_hParentDlg);
	int iResult = ::SendMessage(m_hParentDlg, CDM_GETSPEC, ccBuffer, (LPARAM)pszBuffer);
	ASSERT((0 <= iResult) && (iResult <= ccBuffer));
	return iResult;
}

AFXINLINE void CFileDialog::HideControl(UINT nCtrlId) const
{
	ASSERT_HWND(m_hParentDlg);
	::SendMessage(m_hParentDlg, CDM_HIDECONTROL, nCtrlId, UNUSED);
}

AFXINLINE BOOL CFileDialog::RegisterFileOK()
{
	ASSERT(!m_nFileMsg);
	m_nFileMsg = ::RegisterWindowMessage(FILEOKSTRING);
	return VERIFY(m_nFileMsg);
}

AFXINLINE BOOL CFileDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

AFXINLINE BOOL CFileDialog::RegisterLbSelCh()
{
	ASSERT(!m_nLbSelMsg);
	m_nLbSelMsg = ::RegisterWindowMessage(LBSELCHSTRING);
	return VERIFY(m_nLbSelMsg);
}

AFXINLINE BOOL CFileDialog::RegisterShareVi()
{
	ASSERT(!m_nShareMsg);
	m_nShareMsg = ::RegisterWindowMessage(SHAREVISTRING);
	return VERIFY(m_nShareMsg);
}

AFXINLINE void CFileDialog::SetControlText(UINT nCtrlId, PCSTR pszText) const
{
	ASSERT_HWND(m_hParentDlg);
	::SendMessage(m_hParentDlg, CDM_SETCONTROLTEXT, nCtrlId, (LPARAM)pszText);
}

AFXINLINE void CFileDialog::SetDefExt(PCSTR pszFileExt) const
{
	ASSERT_HWND(m_hParentDlg);
	::SendMessage(m_hParentDlg, CDM_SETDEFEXT, UNUSED, (LPARAM)pszFileExt);
}

AFXINLINE void CFileDialog::SetParent(HWND hParentDlg)
{
	ASSERT(::IsWindow(hParentDlg));
	m_hParentDlg = hParentDlg;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CFindDialog

// Find Next, Replace or Replace All button is clicked, or the dialog box is closed
// void OnCdmFindMsg();
#define ON_CDM_FINDMSG(jDlgName, fName) _ON_WM_REGISTER_V(jDlgName.m_nFindMsg, fName, ())

// Only can be derived (cannot be instantiated)
class CFindDialog : public CDialog, public FINDREPLACE
{
public:
	// Registered messages
	UINT m_nFindMsg, m_nHelpMsg;

public:
	BOOL FindText();
	BOOL RegisterFindMsg();
	BOOL RegisterHelpMsg();
	BOOL ReplaceText();

protected:
	CFindDialog();
};

AFXINLINE CFindDialog::CFindDialog()
{
#ifdef DEBUG
	m_nFindMsg = 0;
	m_nHelpMsg = 0;
#endif
}

AFXINLINE BOOL CFindDialog::FindText()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(FINDREPLACE)) && (Flags & FR_ENABLEHOOK));
	_m_pWnd = this;
	return (BOOL)VERIFY(::FindText(this));
}

AFXINLINE BOOL CFindDialog::RegisterFindMsg()
{
	ASSERT(!m_nFindMsg);
	m_nFindMsg = ::RegisterWindowMessage(FINDMSGSTRING);
	return VERIFY(m_nFindMsg);
}

AFXINLINE BOOL CFindDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

AFXINLINE BOOL CFindDialog::ReplaceText()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(FINDREPLACE)) && (Flags & FR_ENABLEHOOK));
	_m_pWnd = this;
	return (BOOL)VERIFY(::ReplaceText(this));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CFontDialog

// Only can be derived (cannot be instantiated)
class CFontDialog : public CDialog, public CHOOSEFONT, public LOGFONT
{
public:
	// Registered message
	UINT m_nHelpMsg;

public:
	BOOL ChooseFont();
	void GetLogFont();
	BOOL RegisterHelpMsg();
	void SetFlags() const;
	void SetLogFont() const;

protected:
	CFontDialog();
};

AFXINLINE CFontDialog::CFontDialog()
{
#ifdef DEBUG
	m_nHelpMsg = 0;
#endif
}

AFXINLINE BOOL CFontDialog::ChooseFont()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(CHOOSEFONT)) && (Flags & CF_ENABLEHOOK));
	_m_pWnd = this;
	return ::ChooseFont(this);
}

AFXINLINE void CFontDialog::GetLogFont()
{
	::SendMessage(m_hWnd, WM_CHOOSEFONT_GETLOGFONT, UNUSED, (LPARAM)(PLOGFONT)this);
}

AFXINLINE BOOL CFontDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

AFXINLINE void CFontDialog::SetFlags() const
{
	::SendMessage(m_hWnd, WM_CHOOSEFONT_SETFLAGS, UNUSED, (LPARAM)(PCCHOOSEFONT)this);
}

AFXINLINE void CFontDialog::SetLogFont() const
{
	::SendMessage(m_hWnd, WM_CHOOSEFONT_SETLOGFONT, UNUSED, (LPARAM)(PCLOGFONT)this);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CPageDialog

// Draws the envelope-stamp rectangle of the sample page
// BOOL OnPsdEnvStampRect(HDC hDC, PCRECT pRect);
// Returns: Accepts (FALSE) to draw the sample page or not (TRUE)
#define _ON_WM_PSD_ENVSTAMPRECT(nMsg, fName) \
	case nMsg: \
		return pWnd->fName((HDC)wParam, (PCRECT)lParam);
#define ON_WM_PSD_ENVSTAMPRECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_ENVSTAMPRECT, fName)

// Notifies the coordinates of the sample page rectangle
// BOOL OnPsdFullPageRect(HDC hDC, PCRECT pRect);
// Returns: Sends (FALSE) the remaining messages or not (TRUE)
#define ON_WM_PSD_FULLPAGERECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_FULLPAGERECT, fName)

// Draws greek text inside the margin rectangle of the sample page
// BOOL OnPsdGreekTextRect(HDC hDC, PCRECT pRect);
// Returns: Accepts (FALSE) to draw the sample page or not (TRUE)
#define ON_WM_PSD_GREEKTEXTRECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_GREEKTEXTRECT, fName)

// Draw the margin rectangle of the sample page
// BOOL OnPsdMarginRect(HDC hDC, PCRECT pRect);
// Returns: Accepts (FALSE) to draw the sample page or not (TRUE)
#define ON_WM_PSD_MARGINRECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_MARGINRECT, fName)

// Notifies the coordinates of the minimum margin rectangle in the sample page
// BOOL OnPsdMinMarginRect(HDC hDC, PCRECT pRect);
// Returns: Sends (FALSE) the remaining messages or not (TRUE)
#define ON_WM_PSD_MINMARGINRECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_MINMARGINRECT, fName)

// Draws the contents of the sample page
// BOOL OnPsdPageSetupDlg(UINT nPaper, UINT nFlag);
// Returns: Sends (FALSE) the remaining messages or not (TRUE)
#define ON_WM_PSD_PAGESETUPDLG(fName) \
	case WM_PSD_PAGESETUPDLG: \
		return pWnd->fName(LOWORD(wParam), HIWORD(wParam));

// Draws the return address portion of an envelope sample page
// BOOL OnPsdYaFullPageRect(HDC hDC, PCRECT pRect);
// Returns: Accepts (FALSE) to draw the sample page or not (TRUE)
#define ON_WM_PSD_YAFULLPAGERECT(fName) _ON_WM_PSD_ENVSTAMPRECT(WM_PSD_YAFULLPAGERECT, fName)

// Only can be derived (cannot be instantiated)
class CPageDialog : public CDialog, public PAGESETUPDLG
{
public:
	// Registered message
	UINT m_nHelpMsg;

public:
	BOOL PageSetupDlg();
	BOOL RegisterHelpMsg();

protected:
	CPageDialog();
};

AFXINLINE CPageDialog::CPageDialog()
{
#ifdef DEBUG
	m_nHelpMsg = 0;
#endif
}

AFXINLINE BOOL CPageDialog::PageSetupDlg()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(PAGESETUPDLG)) &&
		((Flags & PSD_ENABLEPAGEPAINTHOOK) || (Flags & PSD_ENABLEPAGESETUPHOOK) ||
		(Flags & PSD_RETURNDEFAULT)));
	_m_pWnd = this;
	return ::PageSetupDlg(this);
}

AFXINLINE BOOL CPageDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CPrintDialog

// Only can be derived (cannot be instantiated)
class CPrintDialog : public CDialog, public PRINTDLG
{
public:
	// Registered message
	UINT m_nHelpMsg;

public:
	BOOL PrintDlg();
	BOOL RegisterHelpMsg();

protected:
	CPrintDialog();
};

AFXINLINE CPrintDialog::CPrintDialog()
{
#ifdef DEBUG
	m_nHelpMsg = 0;
#endif
}

AFXINLINE BOOL CPrintDialog::PrintDlg()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(PRINTDLG)) &&
		((Flags & PD_ENABLEPRINTHOOK) || (Flags & PD_ENABLESETUPHOOK) ||
		(Flags & PD_RETURNDEFAULT) || (Flags & PD_RETURNIC)));
	_m_pWnd = this;
	return ::PrintDlg(this);
}

AFXINLINE BOOL CPrintDialog::RegisterHelpMsg()
{
	ASSERT(!m_nHelpMsg);
	m_nHelpMsg = ::RegisterWindowMessage(HELPMSGSTRING);
	return VERIFY(m_nHelpMsg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CPrintDialogEx

// Only can be derived (cannot be instantiated)
#ifdef INCOLE
class CPrintDialogEx : public CDialog, public PRINTDLGEX
{
public:
	HRESULT PrintDlgEx();

protected:
	CPrintDialogEx();
};

AFXINLINE CPrintDialogEx::CPrintDialogEx()
{
	// Does nothing
}

AFXINLINE HRESULT CPrintDialogEx::PrintDlgEx()
{
	ASSERT(!_m_pWnd && (lStructSize == sizeof(PRINTDLGEX)) &&
		((Flags & PD_RETURNDEFAULT) || (Flags & PD_RETURNIC)));
	_m_pWnd = this;
	return VERIFY_HRESULT(::PrintDlgEx(this));
}
#endif

// End of _COMMDLG_INCLUDED_
#endif
