////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// D3D.hpp - Direct3D Declaration File                                                        //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _D3D_INCLUDED_
#define _D3D_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  D3D - Framework header file <WinAfx.hpp> should be included first.
#endif

// Direct3D header file should be matching with framework
#if    (DIRECT3D_VERSION != 0x0900)
#error D3D - Header file <D3d9.h> is inconsistent with framework.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Version control (used internally only)

// Makes a correct name with corresponding header files
#define _D3D_TYPEDEF(e) typedef e##9 e;

// Structures
_D3D_TYPEDEF(D3DCAPS)
_D3D_TYPEDEF(D3DLIGHT)
_D3D_TYPEDEF(D3DMATERIAL)
_D3D_TYPEDEF(D3DVERTEXELEMENT)
_D3D_TYPEDEF(D3DVIEWPORT)

// Interfaces
_D3D_TYPEDEF(IDirect3D)
_D3D_TYPEDEF(IDirect3DBaseTexture)
_D3D_TYPEDEF(IDirect3DDevice)
_D3D_TYPEDEF(IDirect3DIndexBuffer)
_D3D_TYPEDEF(IDirect3DPixelShader)
_D3D_TYPEDEF(IDirect3DStateBlock)
_D3D_TYPEDEF(IDirect3DTexture)
_D3D_TYPEDEF(IDirect3DVertexBuffer)
_D3D_TYPEDEF(IDirect3DVertexDeclaration)
_D3D_TYPEDEF(IDirect3DVertexShader)

////////////////////////////////////////////////////////////////////////////////////////////////
// General purposes

// Constant symbols
#define D3DFMT_DEFAULT D3DFMT_UNKNOWN  // Uses the default surface format
#define D3DFMT_UNUSED  D3DFMT_UNKNOWN  // Surface is unused
#define D3D_DEFAULT    0               // Uses the default value
#define D3D_PI         D3DX_PI         // PI value (3.141592654f)
#define D3D_2PI        (D3D_PI * 2)    // Twice of PI value (6.283185308f)
#define D3D_VPL        2               // Vertices per line
#define D3D_VPT        3               // Vertices per triangle

// Color reference codes for D3DCOLOR (ARGB and XRGB)
// Returns: A D3DCOLOR value
#define D3DARGB(a, c)        D3DCOLOR_ARGB(a,   c,   c,   c)
#define D3DARGB_BLACK(a)     D3DCOLOR_ARGB(a,   0,   0,   0)
#define D3DARGB_BLUE(a)      D3DCOLOR_ARGB(a,   0,   0, 255)
#define D3DARGB_GREEN(a)     D3DCOLOR_ARGB(a,   0, 255,   0)
#define D3DARGB_CYAN(a)      D3DCOLOR_ARGB(a,   0, 255, 255)
#define D3DARGB_RED(a)       D3DCOLOR_ARGB(a, 255,   0,   0)
#define D3DARGB_MAGENTA(a)   D3DCOLOR_ARGB(a, 255,   0, 255)
#define D3DARGB_YELLOW(a)    D3DCOLOR_ARGB(a, 255, 255,   0)
#define D3DARGB_WHITE(a)     D3DCOLOR_ARGB(a, 255, 255, 255)
#define D3DARGB_DKBLUE(a)    D3DCOLOR_ARGB(a,   0,   0, 128)
#define D3DARGB_DKGREEN(a)   D3DCOLOR_ARGB(a,   0, 128,   0)
#define D3DARGB_DKGRAY(a)    D3DCOLOR_ARGB(a,   0, 128, 128)
#define D3DARGB_DKBROWN(a)   D3DCOLOR_ARGB(a, 128,   0,   0)
#define D3DARGB_DKMAGENTA(a) D3DCOLOR_ARGB(a, 128,   0, 128)
#define D3DARGB_BROWN(a)     D3DCOLOR_ARGB(a, 128, 128,   0)
#define D3DARGB_GRAY(a)      D3DCOLOR_ARGB(a, 128, 128, 128)
#define D3DARGB_LTGRAY(a)    D3DCOLOR_ARGB(a, 192, 192, 192)
#define D3DARGB_BTGRAY(a)    D3DCOLOR_ARGB(a, 192, 220, 192)
#define D3DARGB_LTBLUE(a)    D3DCOLOR_ARGB(a, 166, 202, 240)
#define D3DARGB_MIDGRAY(a)   D3DCOLOR_ARGB(a, 160, 160, 164)
#define D3DARGB_MILKY(a)     D3DCOLOR_ARGB(a, 255, 251, 240)
#define D3DXRGB(c)           D3DARGB(255, c)
#define D3DXRGB_BLACK        D3DARGB_BLACK(255)
#define D3DXRGB_BLUE         D3DARGB_BLUE(255)
#define D3DXRGB_GREEN        D3DARGB_GREEN(255)
#define D3DXRGB_CYAN         D3DARGB_CYAN(255)
#define D3DXRGB_RED          D3DARGB_RED(255)
#define D3DXRGB_MAGENTA      D3DARGB_MAGENTA(255)
#define D3DXRGB_YELLOW       D3DARGB_YELLOW(255)
#define D3DXRGB_WHITE        D3DARGB_WHITE(255)
#define D3DXRGB_DKBLUE       D3DARGB_DKBLUE(255)
#define D3DXRGB_DKGREEN      D3DARGB_DKGREEN(255)
#define D3DXRGB_DKGRAY       D3DARGB_DKGRAY(255)
#define D3DXRGB_DKBROWN      D3DARGB_DKBROWN(255)
#define D3DXRGB_DKMAGENTA    D3DARGB_DKMAGENTA(255)
#define D3DXRGB_BROWN        D3DARGB_BROWN(255)
#define D3DXRGB_GRAY         D3DARGB_GRAY(255)
#define D3DXRGB_LTGRAY       D3DARGB_LTGRAY(255)
#define D3DXRGB_BTGRAY       D3DARGB_BTGRAY(255)
#define D3DXRGB_LTBLUE       D3DARGB_LTBLUE(255)
#define D3DXRGB_MIDGRAY      D3DARGB_MIDGRAY(255)
#define D3DXRGB_MILKY        D3DARGB_MILKY(255)

// Gets the alpha/red/green/blue component of a D3DCOLOR value (ARGB and XRGB)
// Returns: Intensity of the alpha/red/green/blue component
#define D3DARGB_GetAValue(c) ((BYTE)((c) >> 24))
#define D3DARGB_GetRValue(c) ((BYTE)((c) >> 16))
#define D3DARGB_GetGValue(c) ((BYTE)((c) >> 8))
#define D3DARGB_GetBValue(c) ((BYTE)(c))
#define D3DXRGB_GetRValue(c) D3DARGB_GetRValue(c)
#define D3DXRGB_GetGValue(c) D3DARGB_GetGValue(c)
#define D3DXRGB_GetBValue(c) D3DARGB_GetBValue(c)

// Color reference codes for D3DCOLORVALUE (RGBA and RGBX)
// Returns: A D3DCOLORVALUE value
#define D3DRGBA(c, a)        { c,         c,         c,         a }
#define D3DRGBA_BLACK(a)     { 0,         0,         0,         a }
#define D3DRGBA_BLUE(a)      { 0,         0,         1,         a }
#define D3DRGBA_GREEN(a)     { 0,         1,         0,         a }
#define D3DRGBA_CYAN(a)      { 0,         1,         1,         a }
#define D3DRGBA_RED(a)       { 1,         0,         0,         a }
#define D3DRGBA_MAGENTA(a)   { 1,         0,         1,         a }
#define D3DRGBA_YELLOW(a)    { 1,         1,         0,         a }
#define D3DRGBA_WHITE(a)     { 1,         1,         1,         a }
#define D3DRGBA_DKBLUE(a)    { 0,         0,         0.501961f, a }
#define D3DRGBA_DKGREEN(a)   { 0,         0.501961f, 0,         a }
#define D3DRGBA_DKGRAY(a)    { 0,         0.501961f, 0.501961f, a }
#define D3DRGBA_DKBROWN(a)   { 0.501961f, 0,         0,         a }
#define D3DRGBA_DKMAGENTA(a) { 0.501961f, 0,         0.501961f, a }
#define D3DRGBA_BROWN(a)     { 0.501961f, 0.501961f, 0,         a }
#define D3DRGBA_GRAY(a)      { 0.501961f, 0.501961f, 0.501961f, a }
#define D3DRGBA_LTGRAY(a)    { 0.752942f, 0.752942f, 0.752942f, a }
#define D3DRGBA_BTGRAY(a)    { 0.752942f, 0.862746f, 0.752942f, a }
#define D3DRGBA_LTBLUE(a)    { 0.650981f, 0.792157f, 0.941177f, a }
#define D3DRGBA_MIDGRAY(a)   { 0.627451f, 0.627451f, 0.643138f, a }
#define D3DRGBA_MILKY(a)     { 1,         0.984314f, 0.941177f, a }
#define D3DRGBX(c)           D3DRGBA(c, 1)
#define D3DRGBX_BLACK        D3DRGBA_BLACK(1)
#define D3DRGBX_BLUE         D3DRGBA_BLUE(1)
#define D3DRGBX_GREEN        D3DRGBA_GREEN(1)
#define D3DRGBX_CYAN         D3DRGBA_CYAN(1)
#define D3DRGBX_RED          D3DRGBA_RED(1)
#define D3DRGBX_MAGENTA      D3DRGBA_MAGENTA(1)
#define D3DRGBX_YELLOW       D3DRGBA_YELLOW(1)
#define D3DRGBX_WHITE        D3DRGBA_WHITE(1)
#define D3DRGBX_DKBLUE       D3DRGBA_DKBLUE(1)
#define D3DRGBX_DKGREEN      D3DRGBA_DKGREEN(1)
#define D3DRGBX_DKGRAY       D3DRGBA_DKGRAY(1)
#define D3DRGBX_DKBROWN      D3DRGBA_DKBROWN(1)
#define D3DRGBX_DKMAGENTA    D3DRGBA_DKMAGENTA(1)
#define D3DRGBX_BROWN        D3DRGBA_BROWN(1)
#define D3DRGBX_GRAY         D3DRGBA_GRAY(1)
#define D3DRGBX_LTGRAY       D3DRGBA_LTGRAY(1)
#define D3DRGBX_BTGRAY       D3DRGBA_BTGRAY(1)
#define D3DRGBX_LTBLUE       D3DRGBA_LTBLUE(1)
#define D3DRGBX_MIDGRAY      D3DRGBA_MIDGRAY(1)
#define D3DRGBX_MILKY        D3DRGBA_MILKY(1)

// Unused D3DVECTOR structure
#define D3DVECTOR_UNUSED { 0, 0, 0 }

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Structures
struct D3DPLANE
{
	float a, b, c, d;
};

struct D3DVECTOR4
{
	float x, y, z, w;
};

// Synonyms
typedef D3DXVECTOR3   D3DXVECTOR;
typedef _D3DXMESH     D3DXMESH;
typedef _D3DXMESHSIMP D3DXMESHSIMP;

// Pointers
typedef D3DCOLOR*            PD3DCOLOR;
typedef D3DMATERIAL*         PD3DMATERIAL;
typedef D3DMATRIX*           PD3DMATRIX;
typedef D3DPLANE*            PD3DPLANE;
typedef D3DVECTOR*           PD3DVECTOR;
typedef D3DXMATERIAL*        PD3DXMATERIAL;
typedef D3DXMATRIX*          PD3DXMATRIX;
typedef D3DXPLANE*           PD3DXPLANE;
typedef D3DXVECTOR*          PD3DXVECTOR;
typedef const D3DMATRIX*     PCD3DMATRIX;
typedef const D3DPLANE*      PCD3DPLANE;
typedef const D3DVECTOR*     PCD3DVECTOR;
typedef const D3DVECTOR4*    PCD3DVECTOR4;
typedef const D3DXFONT_DESC* PCD3DXFONT_DESC;
typedef const D3DXMATRIX*    PCD3DXMATRIX;
typedef const D3DXPLANE*     PCD3DXPLANE;
typedef const D3DXVECTOR*    PCD3DXVECTOR;
typedef const D3DXVECTOR4*   PCD3DXVECTOR4;

// Static
#define SD3DLIGHT          static D3DLIGHT
#define SD3DTEXTUREADDRESS static D3DTEXTUREADDRESS
#define SCD3DLIGHT         static const D3DLIGHT
#define SCD3DMATERIAL      static const D3DMATERIAL
#define SCD3DPLANE         static const D3DPLANE
#define SCD3DVECTOR        static const D3DVECTOR
#define SCD3DVECTOR4       static const D3DVECTOR4
#define SCD3DVERTEXELEMENT static const D3DVERTEXELEMENT

////////////////////////////////////////////////////////////////////////////////////////////////
// General functions

// Converts a float data type to a DWORD data type (bit image is not change)
// Returns: A converted DWORD
AFXINLINE DWORD WINAPI MAKEDWORD(float fValue)
{
	return *(PDWORD)&fValue;
}

// Verifies the D3DX version
// Returns: TRUE if the compiled version is matching with the running version
AFXINLINE BOOL WINAPI D3DXCheckVersion()
{
	return VERIFY(D3DXCheckVersion(D3D_SDK_VERSION, D3DX_SDK_VERSION));
}

// Creates an IDirect3D object
// Returns: A pointer to an IDirect3D interface
AFXINLINE IDirect3D* WINAPI Direct3DCreate()
{
	return (IDirect3D*)VERIFY(Direct3DCreate9(D3D_SDK_VERSION));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on color values

// Determines whether two colors are equal/unequal
// Returns: TRUE if two colors are (not) identical
AFXINLINE BOOL WINAPI operator ==(const D3DCOLORVALUE& rClr1, const D3DCOLORVALUE& rClr2)
{
	return !memcmp(&rClr1, &rClr2, sizeof(D3DCOLORVALUE));
}

AFXINLINE BOOL WINAPI operator !=(const D3DCOLORVALUE& rClr1, const D3DCOLORVALUE& rClr2)
{
	return memcmp(&rClr1, &rClr2, sizeof(D3DCOLORVALUE));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on effects

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateEffectFromFile(IDirect3DDevice* pD3DD, PCTSTR pszFileName,
	DWORD d3dfx, ID3DXEffectPool* pD3DEP, ID3DXEffect** ppD3DE, ID3DXBuffer** ppD3DB)
{
	return DXVERIFY(D3DXCreateEffectFromFile(pD3DD, pszFileName, NULL, NULL,
		d3dfx, pD3DEP, ppD3DE, ppD3DB));
}

AFXINLINE HRESULT WINAPI D3DXCreateEffectFromFile(IDirect3DDevice* pD3DD, PCTSTR pszFileName,
	DWORD d3dfx, ID3DXEffect** ppD3DE, ID3DXBuffer** ppD3DB)
{
	return DXVERIFY(D3DXCreateEffectFromFile(pD3DD, pszFileName, NULL, NULL,
		d3dfx, NULL, ppD3DE, ppD3DB));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on fonts

// Creates a font object
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateFont(IDirect3DDevice* pD3DD, int iHeight, UINT nWidth,
	UINT nWeight, UINT nMipLevels, BOOL bItalic, DWORD dwCharSet, DWORD dwOutputPrec,
	DWORD dwQuality, DWORD dwPitchFamily, ID3DXFont** ppD3DF)
{
	return DXVERIFY(D3DXCreateFont(pD3DD, iHeight, nWidth, nWeight, nMipLevels,
		bItalic, dwCharSet, dwOutputPrec, dwQuality, dwPitchFamily, NULL, ppD3DF));
}

AFXINLINE HRESULT WINAPI D3DXCreateFont(IDirect3DDevice* pD3DD, PCD3DXFONT_DESC pD3DFD,
	ID3DXFont** ppD3DF)
{
	return DXVERIFY(D3DXCreateFontIndirect(pD3DD, pD3DFD, ppD3DF));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on matrices

// Determines whether two matrices are equal/unequal
// Returns: TRUE if two matrices are (not) identical
AFXINLINE BOOL WINAPI operator ==(const D3DMATRIX& rMtx1, const D3DMATRIX& rMtx2)
{
	return !memcmp(&rMtx1, &rMtx2, sizeof(D3DMATRIX));
}

AFXINLINE BOOL WINAPI operator !=(const D3DMATRIX& rMtx1, const D3DMATRIX& rMtx2)
{
	return memcmp(&rMtx1, &rMtx2, sizeof(D3DMATRIX));
}

AFXINLINE float WINAPI D3DXMatrixDeterminant(PCD3DMATRIX pD3DM)
{
	return D3DXMatrixDeterminant((PCD3DXMATRIX)pD3DM);
}

// Creates an identity matrix
// Returns: Identity matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixIdentity(PD3DMATRIX pRetMtx)
{
	return D3DXMatrixIdentity((PD3DXMATRIX)pRetMtx);
}

// Calculates the inverse of a matrix
// Returns: Inverse of the matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixInverse(PD3DXMATRIX pRetMtx, PCD3DXMATRIX pSrcMtx)
{
	return (PD3DXMATRIX)VERIFY(D3DXMatrixInverse(pRetMtx, NULL, pSrcMtx));
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixInverse(PD3DMATRIX pRetMtx, PFLOAT pfDeterminant,
	PCD3DMATRIX pSrcMtx)
{
	return (PD3DXMATRIX)VERIFY(D3DXMatrixInverse((PD3DXMATRIX)pRetMtx, pfDeterminant,
		(PCD3DXMATRIX)pSrcMtx));
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixInverse(PD3DMATRIX pRetMtx, PCD3DMATRIX pSrcMtx)
{
	return (PD3DXMATRIX)VERIFY(D3DXMatrixInverse((PD3DXMATRIX)pRetMtx, NULL,
		(PCD3DXMATRIX)pSrcMtx));
}

AFXINLINE BOOL WINAPI D3DXMatrixIsIdentity(PCD3DMATRIX pD3DM)
{
	return D3DXMatrixIsIdentity((PCD3DXMATRIX)pD3DM);
}

// Creates a left-handed look-at matrix
// Returns: Look-at matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixLookAtLH(PD3DMATRIX pRetMtx, PCD3DVECTOR pEyeVtr,
	PCD3DVECTOR pAtVtr, PCD3DVECTOR pUpVtr)
{
	return D3DXMatrixLookAtLH((PD3DXMATRIX)pRetMtx, (PCD3DXVECTOR)pEyeVtr,
		(PCD3DXVECTOR)pAtVtr, (PCD3DXVECTOR)pUpVtr);
}

// Creates a right-handed look-at matrix
// Returns: Look-at matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixLookAtRH(PD3DMATRIX pRetMtx, PCD3DVECTOR pEyeVtr,
	PCD3DVECTOR pAtVtr, PCD3DVECTOR pUpVtr)
{
	return D3DXMatrixLookAtRH((PD3DXMATRIX)pRetMtx, (PCD3DXVECTOR)pEyeVtr,
		(PCD3DXVECTOR)pAtVtr, (PCD3DXVECTOR)pUpVtr);
}

// Calculates the product of two matrices
// Returns: Product of two matrices
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixMultiply(PD3DMATRIX pRetMtx, PCD3DMATRIX pSrcMtx1,
	PCD3DMATRIX pSrcMtx2)
{
	return D3DXMatrixMultiply((PD3DXMATRIX)pRetMtx, (PCD3DXMATRIX)pSrcMtx1,
		(PCD3DXMATRIX)pSrcMtx2);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixMultiplyTranspose(PD3DMATRIX pRetMtx,
	PCD3DMATRIX pSrcMtx1, PCD3DMATRIX pSrcMtx2)
{
	return D3DXMatrixMultiplyTranspose((PD3DXMATRIX)pRetMtx,
		(PCD3DXMATRIX)pSrcMtx1, (PCD3DXMATRIX)pSrcMtx2);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixOrthoLH(PD3DMATRIX pD3DM, float fWidth, float fHeight,
	float zNearPln, float zFarPln)
{
	return D3DXMatrixOrthoLH((PD3DXMATRIX)pD3DM, fWidth, fHeight, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixOrthoOffCenterLH(PD3DMATRIX pD3DM, float xLeft,
	float xRight, float yBottom, float yTop, float zNearPln, float zFarPln)
{
	return D3DXMatrixOrthoOffCenterLH((PD3DXMATRIX)pD3DM, xLeft,
		xRight, yBottom, yTop, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixOrthoOffCenterRH(PD3DMATRIX pD3DM, float xLeft,
	float xRight, float yBottom, float yTop, float zNearPln, float zFarPln)
{
	return D3DXMatrixOrthoOffCenterRH((PD3DXMATRIX)pD3DM, xLeft,
		xRight, yBottom, yTop, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixOrthoRH(PD3DMATRIX pD3DM, float fWidth, float fHeight,
	float zNearPln, float zFarPln)
{
	return D3DXMatrixOrthoRH((PD3DXMATRIX)pD3DM, fWidth, fHeight, zNearPln, zFarPln);
}

// Creates a left-handed perspective projection matrix
// Returns: Perspective projection matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveFovLH(PD3DMATRIX pRetMtx, float yFov,
	float fAspect, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveFovLH((PD3DXMATRIX)pRetMtx, yFov, fAspect, zNearPln, zFarPln);
}

// Creates a right-handed perspective projection matrix
// Returns: Perspective projection matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveFovRH(PD3DMATRIX pRetMtx, float yFov,
	float fAspect, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveFovRH((PD3DXMATRIX)pRetMtx, yFov, fAspect, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveLH(PD3DMATRIX pD3DM, float fWidth,
	float fHeight, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveLH((PD3DXMATRIX)pD3DM, fWidth, fHeight, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveOffCenterLH(PD3DMATRIX pD3DM, float xLeft,
	float xRight, float yBottom, float yTop, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveOffCenterLH((PD3DXMATRIX)pD3DM, xLeft,
		xRight, yBottom, yTop, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveOffCenterRH(PD3DMATRIX pD3DM, float xLeft,
	float xRight, float yBottom, float yTop, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveOffCenterRH((PD3DXMATRIX)pD3DM, xLeft,
		xRight, yBottom, yTop, zNearPln, zFarPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixPerspectiveRH(PD3DMATRIX pD3DM, float fWidth,
	float fHeight, float zNearPln, float zFarPln)
{
	return D3DXMatrixPerspectiveRH((PD3DXMATRIX)pD3DM, fWidth, fHeight, zNearPln, zFarPln);
}

// Creates a matrix that reflects the coordinate system about a plane
// Returns: Reflected matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixReflect(PD3DMATRIX pRetMtx, PCD3DPLANE pSrcPln)
{
	return D3DXMatrixReflect((PD3DXMATRIX)pRetMtx, (PCD3DXPLANE)pSrcPln);
}

AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixRotationAxis(PD3DMATRIX pD3DM, PCD3DVECTOR pD3DV,
	float fAngle)
{
	return D3DXMatrixRotationAxis((PD3DXMATRIX)pD3DM, (PCD3DXVECTOR)pD3DV, fAngle);
}

// Creates a matrix that rotates around the x-axis
// Returns: Rotated matrix around the x-axis
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixRotationX(PD3DMATRIX pRetMtx, float fAngle)
{
	return D3DXMatrixRotationX((PD3DXMATRIX)pRetMtx, fAngle);
}

// Creates a matrix that rotates around the y-axis
// Returns: Rotated matrix around the y-axis
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixRotationY(PD3DMATRIX pRetMtx, float fAngle)
{
	return D3DXMatrixRotationY((PD3DXMATRIX)pRetMtx, fAngle);
}

// Creates a matrix that rotates around the z-axis
// Returns: Rotated matrix around the z-axis
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixRotationZ(PD3DMATRIX pRetMtx, float fAngle)
{
	return D3DXMatrixRotationZ((PD3DXMATRIX)pRetMtx, fAngle);
}

// Creates a matrix that scales along the x-, y- and z-axes
// Returns: Scaling matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixScaling(PD3DMATRIX pRetMtx, float xFactor, float yFactor,
	float zFactor)
{
	return D3DXMatrixScaling((PD3DXMATRIX)pRetMtx, xFactor, yFactor, zFactor);
}

// Creates a matrix that flattens geometry into a plane
// Returns: Shadow matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixShadow(PD3DMATRIX pRetMtx, PCD3DVECTOR4 pLightVtr,
	PCD3DPLANE pSrcPln)
{
	return D3DXMatrixShadow((PD3DXMATRIX)pRetMtx, (PCD3DXVECTOR4)pLightVtr,
		(PCD3DXPLANE)pSrcPln);
}

// Creates a translation matrix
// Returns: Translation matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixTranslation(PD3DMATRIX pRetMtx, float xOffset,
	float yOffset, float zOffset)
{
	return D3DXMatrixTranslation((PD3DXMATRIX)pRetMtx, xOffset, yOffset, zOffset);
}

// Calculates the matrix transpose of a matrix
// Returns: Matrix transpose of a matrix
AFXINLINE PD3DXMATRIX WINAPI D3DXMatrixTranspose(PD3DMATRIX pRetMtx, PCD3DMATRIX pSrcMtx)
{
	return D3DXMatrixTranspose((PD3DXMATRIX)pRetMtx, (PCD3DXMATRIX)pSrcMtx);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on meshes

// Computes a bounding box
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXComputeBoundingBox(PCD3DVECTOR paVertex, DWORD ceVertex,
	DWORD cbStride, PD3DVECTOR pMinVtx, PD3DVECTOR pMaxVtx)
{
	return DXVERIFY(D3DXComputeBoundingBox((PCD3DXVECTOR)paVertex, ceVertex,
		cbStride, (PD3DXVECTOR)pMinVtx, (PD3DXVECTOR)pMaxVtx));
}

// Computes a bounding sphere
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXComputeBoundingSphere(PCD3DVECTOR paVertex, DWORD ceVertex,
	DWORD cbStride, PD3DVECTOR pCenterVtx, PFLOAT pfRadius)
{
	return DXVERIFY(D3DXComputeBoundingSphere((PCD3DXVECTOR)paVertex, ceVertex,
		cbStride, (PD3DXVECTOR)pCenterVtx, pfRadius));
}

// Computes unit normals for each vertex in a mesh
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXComputeNormals(ID3DXBaseMesh* pD3DBM)
{
	return DXVERIFY(D3DXComputeNormals(pD3DBM, NULL));
}

// Generates a progressive mesh
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXGeneratePMesh(ID3DXMesh* pD3DM, PCDWORD pdwaAdjInfo,
	DWORD ceMeshSimp, D3DXMESHSIMP d3dms, ID3DXPMesh** ppD3DPM)
{
	return DXVERIFY(D3DXGeneratePMesh(pD3DM, pdwaAdjInfo, NULL, NULL,
		ceMeshSimp, d3dms, ppD3DPM));
}

// Loads a mesh from a .x file
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXLoadMeshFromX(PCTSTR pszFileName, D3DXMESH d3dm,
	IDirect3DDevice* pD3DD, ID3DXBuffer** ppAdjBuf, ID3DXBuffer** ppMtlBuf,
	PDWORD pceRetMtl, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXLoadMeshFromX(pszFileName, d3dm, pD3DD, ppAdjBuf, ppMtlBuf,
		NULL, pceRetMtl, ppD3DM));
}

AFXINLINE HRESULT WINAPI D3DXLoadMeshFromX(PCTSTR pszFileName, D3DXMESH d3dm,
	IDirect3DDevice* pD3DD, ID3DXBuffer** ppMtlBuf, PDWORD pceRetMtl, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXLoadMeshFromX(pszFileName, d3dm, pD3DD, NULL, ppMtlBuf,
		NULL, pceRetMtl, ppD3DM));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXSplitMesh(ID3DXMesh* pD3DM, PCDWORD pdwaAdjInfo, DWORD dwMaxValue,
	D3DXMESH d3dm, PDWORD pceRetMsh, ID3DXBuffer** ppMshBuf, ID3DXBuffer** ppAdjBuf = NULL,
	ID3DXBuffer** ppFaceBuf = NULL)
{
	return DXVERIFY(D3DXSplitMesh(pD3DM, pdwaAdjInfo, dwMaxValue, d3dm, pceRetMsh,
		ppMshBuf, ppAdjBuf, ppFaceBuf, NULL));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on planes

// Determines whether two planes are equal/unequal
// Returns: TRUE if two planes are (not) identical
AFXINLINE BOOL WINAPI operator ==(const D3DPLANE& rPln1, const D3DPLANE& rPln2)
{
	return !memcmp(&rPln1, &rPln2, sizeof(D3DPLANE));
}

AFXINLINE BOOL WINAPI operator !=(const D3DPLANE& rPln1, const D3DPLANE& rPln2)
{
	return memcmp(&rPln1, &rPln2, sizeof(D3DPLANE));
}

// Calculates the dot product of a plane and a vector
// Returns: Dot product of plane and vector
AFXINLINE float WINAPI D3DXPlaneDot(PCD3DPLANE pSrcPln, PCD3DVECTOR4 pSrcVtr)
{
	return D3DXPlaneDot((PCD3DXPLANE)pSrcPln, (PCD3DXVECTOR4)pSrcVtr);
}

// Calculates the dot product of a plane and a vector (w = 1)
// Returns: Dot product of plane and vector
AFXINLINE float WINAPI D3DXPlaneDotCoord(PCD3DPLANE pSrcPln, PCD3DVECTOR pSrcVtr)
{
	return D3DXPlaneDotCoord((PCD3DXPLANE)pSrcPln, (PCD3DXVECTOR)pSrcVtr);
}

// Calculates the dot product of a plane and a vector (w = 0)
// Returns: Dot product of plane and vector
AFXINLINE float WINAPI D3DXPlaneDotNormal(PCD3DPLANE pSrcPln, PCD3DVECTOR pSrcVtr)
{
	return D3DXPlaneDotNormal((PCD3DXPLANE)pSrcPln, (PCD3DXVECTOR)pSrcVtr);
}

// Constructs a plane from a point and a normal vector
// Returns: Constructed plane
AFXINLINE PD3DXPLANE WINAPI D3DXPlaneFromPointNormal(PD3DPLANE pRetPln, PCD3DVECTOR pSrcPt,
	PCD3DVECTOR pSrcVtr)
{
	return D3DXPlaneFromPointNormal((PD3DXPLANE)pRetPln, (PCD3DXVECTOR)pSrcPt,
		(PCD3DXVECTOR)pSrcVtr);
}

// Constructs a plane from three points
// Returns: Constructed plane
AFXINLINE PD3DXPLANE WINAPI D3DXPlaneFromPoints(PD3DPLANE pRetPln, PCD3DVECTOR pSrcPt1,
	PCD3DVECTOR pSrcPt2, PCD3DVECTOR pSrcPt3)
{
	return D3DXPlaneFromPoints((PD3DXPLANE)pRetPln, (PCD3DXVECTOR)pSrcPt1,
		(PCD3DXVECTOR)pSrcPt2, (PCD3DXVECTOR)pSrcPt3);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXPlaneIntersectLine(PD3DVECTOR pRetVtr, PCD3DPLANE pD3DP,
	PCD3DVECTOR pStartVtr, PCD3DVECTOR pEndVtr)
{
	return D3DXPlaneIntersectLine((PD3DXVECTOR)pRetVtr, (PCD3DXPLANE)pD3DP,
		(PCD3DXVECTOR)pStartVtr, (PCD3DXVECTOR)pEndVtr);
}

// Normalizes a plane
// Returns: Normal of the plane
AFXINLINE PD3DXPLANE WINAPI D3DXPlaneNormalize(PD3DPLANE pRetPln, PCD3DPLANE pSrcPln)
{
	return D3DXPlaneNormalize((PD3DXPLANE)pRetPln, (PCD3DXPLANE)pSrcPln);
}

AFXINLINE PD3DXPLANE WINAPI D3DXPlaneScale(PD3DPLANE pRetPln, PCD3DPLANE pSrcPln, float fFactor)
{
	return D3DXPlaneScale((PD3DXPLANE)pRetPln, (PCD3DXPLANE)pSrcPln, fFactor);
}

// Transforms a plane by a matrix
// Returns: Transformed plane
AFXINLINE PD3DXPLANE WINAPI D3DXPlaneTransform(PD3DPLANE pRetPln, PCD3DPLANE pSrcPln,
	PCD3DMATRIX pSrcMtx)
{
	return D3DXPlaneTransform((PD3DXPLANE)pRetPln, (PCD3DXPLANE)pSrcPln, (PCD3DXMATRIX)pSrcMtx);
}

AFXINLINE PD3DXPLANE WINAPI D3DXPlaneTransformArray(PD3DPLANE paRetPln, UINT ceRetStride,
	PCD3DPLANE paSrcPln, UINT ceSrcStride, PCD3DMATRIX pD3DM, UINT ceArray)
{
	return D3DXPlaneTransformArray((PD3DXPLANE)paRetPln, ceRetStride,
		(PCD3DXPLANE)paSrcPln, ceSrcStride, (PCD3DXMATRIX)pD3DM, ceArray);
}

AFXINLINE PD3DXPLANE WINAPI D3DXPlaneTransformArray(PD3DPLANE paRetPln,
	PCD3DPLANE paSrcPln, PCD3DMATRIX pD3DM, UINT ceArray)
{
	return D3DXPlaneTransformArray((PD3DXPLANE)paRetPln, sizeof(D3DPLANE),
		(PCD3DXPLANE)paSrcPln, sizeof(D3DPLANE), (PCD3DXMATRIX)pD3DM, ceArray);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on shapes

// Creates a mesh containing a box
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateBox(IDirect3DDevice* pD3DD, float fWidth, float fHeight,
	float fDepth, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreateBox(pD3DD, fWidth, fHeight, fDepth, ppD3DM, NULL));
}

// Creates a mesh containing a cylinder
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateCylinder(IDirect3DDevice* pD3DD, float fRadius1,
	float fRadius2, float fLength, UINT ceSlice, UINT ceStack, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreateCylinder(pD3DD, fRadius1,
		fRadius2, fLength, ceSlice, ceStack, ppD3DM, NULL));
}

// Creates a mesh containing a polygon
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreatePolygon(IDirect3DDevice* pD3DD, float fLength, UINT ceSide,
	ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreatePolygon(pD3DD, fLength, ceSide, ppD3DM, NULL));
}

// Creates a mesh containing a sphere
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateSphere(IDirect3DDevice* pD3DD, float fRadius, UINT ceSlice,
	UINT ceStack, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreateSphere(pD3DD, fRadius, ceSlice, ceStack, ppD3DM, NULL));
}

// Creates a mesh containing a teapot
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateTeapot(IDirect3DDevice* pD3DD, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreateTeapot(pD3DD, ppD3DM, NULL));
}

// Creates a mesh containing a text
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateText(IDirect3DDevice* pD3DD, HDC hDC, PCTSTR pszText,
	float fDeviation, float fExtrusion, ID3DXMesh** ppD3DM, ID3DXBuffer** ppD3DB = NULL)
{
	return DXVERIFY(D3DXCreateText(pD3DD, hDC, pszText,
		fDeviation, fExtrusion, ppD3DM, ppD3DB, NULL));
}

AFXINLINE HRESULT WINAPI D3DXCreateText(IDirect3DDevice* pD3DD, HDC hDC, PCTSTR pszText,
	float fExtrusion, ID3DXMesh** ppD3DM, ID3DXBuffer** ppD3DB = NULL)
{
	return DXVERIFY(D3DXCreateText(pD3DD, hDC, pszText, 0, fExtrusion, ppD3DM, ppD3DB, NULL));
}

// Creates a mesh containing a torus
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCreateTorus(IDirect3DDevice* pD3DD, float fInnerRadius,
	float fOuterRadius, UINT ceSide, UINT ceRing, ID3DXMesh** ppD3DM)
{
	return DXVERIFY(D3DXCreateTorus(pD3DD, fInnerRadius,
		fOuterRadius, ceSide, ceRing, ppD3DM, NULL));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on shaders

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXCompileShaderFromFile(PCTSTR pszFileName, PCSTR pszFunName,
	PCSTR pszProfile, DWORD d3dshader, ID3DXBuffer** ppShaderBuf, ID3DXBuffer** ppErrorBuf,
	ID3DXConstantTable** ppD3DCT = NULL)
{
	return DXVERIFY(D3DXCompileShaderFromFile(pszFileName, NULL, NULL, pszFunName,
		pszProfile, d3dshader, ppShaderBuf, ppErrorBuf, ppD3DCT));
}

AFXINLINE HRESULT WINAPI D3DXCompileShaderFromFile(PCTSTR pszFileName, PCSTR pszFunName,
	PCSTR pszProfile, DWORD d3dshader, ID3DXBuffer** ppShaderBuf,
	ID3DXConstantTable** ppD3DCT = NULL)
{
	return DXVERIFY(D3DXCompileShaderFromFile(pszFileName, NULL, NULL, pszFunName,
		pszProfile, d3dshader, ppShaderBuf, NULL, ppD3DCT));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on textures

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI D3DXFilterTexture(IDirect3DBaseTexture* pD3DBT, UINT nSrcLevel,
	DWORD d3dfilter)
{
	return DXVERIFY(D3DXFilterTexture(pD3DBT, NULL, nSrcLevel, d3dfilter));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on vectors

// Determines whether two vectors are equal/unequal
// Returns: TRUE if two vectors are (not) identical
AFXINLINE BOOL WINAPI operator ==(const D3DVECTOR& rVtr1, const D3DVECTOR& rVtr2)
{
	return ((rVtr1.x == rVtr2.x) && (rVtr1.y == rVtr2.y) && (rVtr1.z == rVtr2.z));
}

AFXINLINE BOOL WINAPI operator !=(const D3DVECTOR& rVtr1, const D3DVECTOR& rVtr2)
{
	return ((rVtr1.x != rVtr2.x) || (rVtr1.y != rVtr2.y) || (rVtr1.z != rVtr2.z));
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Add(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Add((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1, (PCD3DXVECTOR)pSrcVtr2);
}

// Calculates the cross product of two vectors
// Returns: Cross product of vectors
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Cross(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Cross((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1, (PCD3DXVECTOR)pSrcVtr2);
}

// Calculates the dot product of two vectors
// Returns: Dot product of vectors
AFXINLINE float WINAPI D3DXVec3Dot(PCD3DVECTOR pSrcVtr1, PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Dot((PCD3DXVECTOR)pSrcVtr1, (PCD3DXVECTOR)pSrcVtr2);
}

// Calculates the length of a vector
// Returns: Length of the vector
AFXINLINE float WINAPI D3DXVec3Length(PCD3DVECTOR pSrcVtr)
{
	return D3DXVec3Length((PCD3DXVECTOR)pSrcVtr);
}

AFXINLINE float WINAPI D3DXVec3LengthSq(PCD3DVECTOR pD3DV)
{
	return D3DXVec3LengthSq((PCD3DXVECTOR)pD3DV);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Lerp(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2, float fParam)
{
	return D3DXVec3Lerp((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1,
		(PCD3DXVECTOR)pSrcVtr2, fParam);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Maximize(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Maximize((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1,
		(PCD3DXVECTOR)pSrcVtr2);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Minimize(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Minimize((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1,
		(PCD3DXVECTOR)pSrcVtr2);
}

// Calculates the normalized version of a vector
// Returns: Normalized version of the vector
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Normalize(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr)
{
	return D3DXVec3Normalize((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Scale(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr,
	float fFactor)
{
	return D3DXVec3Scale((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr, fFactor);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3Subtract(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr1,
	PCD3DVECTOR pSrcVtr2)
{
	return D3DXVec3Subtract((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr1,
		(PCD3DXVECTOR)pSrcVtr2);
}

// Transforms a vector by a matrix, projecting the result back into w = 1
// Returns: Transformed vector
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformCoord(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr,
	PCD3DMATRIX pSrcMtx)
{
	return D3DXVec3TransformCoord((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr,
		(PCD3DXMATRIX)pSrcMtx);
}

// Transforms vectors by a matrix, projecting the result back into w = 1
// Returns: Transformed vectors
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformCoordArray(PD3DVECTOR paRetVtr, UINT cbRetStride,
	PCD3DVECTOR paSrcVtr, UINT cbSrcStride, PCD3DMATRIX pSrcMtx, UINT ceArray)
{
	return D3DXVec3TransformCoordArray((PD3DXVECTOR)paRetVtr, cbRetStride,
		(PCD3DXVECTOR)paSrcVtr, cbSrcStride, (PCD3DXMATRIX)pSrcMtx, ceArray);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformCoordArray(PD3DVECTOR paRetVtr,
	PCD3DVECTOR paSrcVtr, PCD3DMATRIX pSrcMtx, UINT ceArray)
{
	return D3DXVec3TransformCoordArray((PD3DXVECTOR)paRetVtr, sizeof(D3DVECTOR),
		(PCD3DXVECTOR)paSrcVtr, sizeof(D3DVECTOR), (PCD3DXMATRIX)pSrcMtx, ceArray);
}

// Transforms a vector normal by a matrix
// Returns: Transformed vector
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformNormal(PD3DVECTOR pRetVtr, PCD3DVECTOR pSrcVtr,
	PCD3DMATRIX pSrcMtx)
{
	return D3DXVec3TransformNormal((PD3DXVECTOR)pRetVtr, (PCD3DXVECTOR)pSrcVtr,
		(PCD3DXMATRIX)pSrcMtx);
}

// Transforms vectors normal by a matrix
// Returns: Transformed vectors
AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformNormalArray(PD3DVECTOR paRetVtr, UINT cbRetStride,
	PCD3DVECTOR paSrcVtr, UINT cbSrcStride, PCD3DMATRIX pSrcMtx, UINT ceArray)
{
	return D3DXVec3TransformNormalArray((PD3DXVECTOR)paRetVtr, cbRetStride,
		(PCD3DXVECTOR)paSrcVtr, cbSrcStride, (PCD3DXMATRIX)pSrcMtx, ceArray);
}

AFXINLINE PD3DXVECTOR WINAPI D3DXVec3TransformNormalArray(PD3DVECTOR paRetVtr,
	PCD3DVECTOR paSrcVtr, PCD3DMATRIX pSrcMtx, UINT ceArray)
{
	return D3DXVec3TransformNormalArray((PD3DXVECTOR)paRetVtr, sizeof(D3DVECTOR),
		(PCD3DXVECTOR)paSrcVtr, sizeof(D3DVECTOR), (PCD3DXMATRIX)pSrcMtx, ceArray);
}

// End of _D3D_INCLUDED_
#endif
