////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// DDE.hpp - Dynamic Data Exchange Declaration File                                           //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _DDE_INCLUDED_
#define _DDE_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  DDE - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Pointers
typedef DDEACK*    PDDEACK;
typedef DDEADVISE* PDDEADVISE;
typedef DDEDATA*   PDDEDATA;
typedef DDEPOKE*   PDDEPOKE;

////////////////////////////////////////////////////////////////////////////////////////////////
// Transaction maps

// Acknowledges the receipt and processing of a DDE message
#define ON_WM_DDE_ACK(fName) \
	case WM_DDE_ACK: \
		pWnd->fName((HWND)wParam, lParam); \
		return _eResult;

// Requests an update for a data item whenever the item changes
#define _ON_WM_DDE_ADVISE(nMsg, fName) \
	case nMsg: \
		VERIFY(::UnpackDDElParam(nMsg, lParam, (PUINT)&hMemory, &nParam)); \
		VERIFY(::FreeDDElParam(nMsg, lParam)); \
		pWnd->fName((HWND)wParam, hMemory, (ATOM)nParam); \
		return _eResult;
#define ON_WM_DDE_ADVISE(fName) _ON_WM_DDE_ADVISE(WM_DDE_ADVISE, fName)

// A data item is received or is available
#define ON_WM_DDE_DATA(fName) _ON_WM_DDE_ADVISE(WM_DDE_DATA, fName)

// Requests to process a series of commands
#define ON_WM_DDE_EXECUTE(fName) _ON_WM_PAINTCLIPBOARD(WM_DDE_EXECUTE, fName)

// Initiates a conversation
#define ON_WM_DDE_INITIATE(fName) \
	case WM_DDE_INITIATE: \
		pWnd->fName((HWND)wParam, LOWORD(lParam), HIWORD(lParam)); \
		return _eResult;

// Requests to accept an unsolicited data item
#define ON_WM_DDE_POKE(fName) _ON_WM_DDE_ADVISE(WM_DDE_POKE, fName)

// Requests a data item
#define _ON_WM_DDE_REQUEST(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HWND)wParam, HIWORD(lParam), LOWORD(lParam)); \
		return _eResult;
#define ON_WM_DDE_REQUEST(fName) _ON_WM_DDE_REQUEST(WM_DDE_REQUEST, fName)

// Terminates a conversation
#define ON_WM_DDE_TERMINATE(fName) _ON_WM_PALETTECHANGED(WM_DDE_TERMINATE, fName)

// No longer updates for a data item
#define ON_WM_DDE_UNADVISE(fName) _ON_WM_DDE_REQUEST(WM_DDE_UNADVISE, fName)

////////////////////////////////////////////////////////////////////////////////////////////////
// General functions

// Frees the memory specified by an lParam of a WM_DDE_ACK message
// Returns: TRUE if the value is freed
AFXINLINE BOOL WINAPI DdeFreeAck(LPARAM lParam)
{
	return VERIFY(FreeDDElParam(WM_DDE_ACK, lParam));
}

// Unpacks an lParam value received from a posted WM_DDE_ACK message
// Returns: TRUE if the value is unpacked
AFXINLINE BOOL WINAPI DdeUnpackAck(LPARAM lParam, PDDEACK pDDEA, PATOM paItemName)
{
	UINT nLoParam, nHiParam;
	BOOL bSuccess = UnpackDDElParam(WM_DDE_ACK, lParam, &nLoParam, &nHiParam);
	*(PWORD)pDDEA = LOWORD(nLoParam);
	*paItemName   = (ATOM)nHiParam;
	return VERIFY(bSuccess);
}

AFXINLINE BOOL WINAPI DdeUnpackAck(LPARAM lParam, PDDEACK pDDEA, PHMEMORY phCmdStr)
{
	UINT nLoParam, nHiParam;
	BOOL bSuccess = UnpackDDElParam(WM_DDE_ACK, lParam, &nLoParam, &nHiParam);
	*(PWORD)pDDEA = LOWORD(nLoParam);
	*phCmdStr     = (HMEMORY)nHiParam;
	return VERIFY(bSuccess);
}

// End of _DDE_INCLUDED_
#endif
