////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// DDEML.hpp - DDEML Declaration File                                                         //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _DDEML_INCLUDED_
#define _DDEML_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  DDEML - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Transaction maps

// Declares in DDE class
#define DECLARE_DDE_MAP() \
	public: \
		static HDDEDATA CALLBACK _DdeCallback(UINT nType, UINT nFormat, HCONV hConv, \
			HSZ hSZ1, HSZ hSZ2, HDDEDATA hData, DWORD dwData1, DWORD dwData2); \
		UINT Initialize(DWORD dwFlags = APPCMD_CLIENTONLY) \
		{ \
			return EQUAL(::DdeInitialize(&m_dwInstId, _DdeCallback, dwFlags, UNUSED), \
				DMLERR_NO_ERROR); \
		}

// Begins/ends transaction maps
// CName : Name of DDE class
#define BEGIN_DDE_MAP(CName) \
	CDdeObject* CDdeObject::_m_pThis; \
	HDDEDATA CALLBACK CName::_DdeCallback(UINT nType, UINT nFormat, HCONV hConv, \
		HSZ hSZ1, HSZ hSZ2, HDDEDATA hData, DWORD dwData1, DWORD dwData2) \
	{ \
		CName* pThis = (CName*)_m_pThis; \
		ASSERT_WRITE(pThis, sizeof(CName)); \
		switch(nType) \
		{

#define END_DDE_MAP() \
		} \
		return (HDDEDATA)_eResult; \
	}

// A data item has changed
// Returns: Result value of the transaction processed
#define ON_XTYP_ADVDATA(fName) \
	case XTYP_ADVDATA: \
		return (HDDEDATA)pThis->fName(hConv, hSZ1, hSZ2, hData, nFormat);

// An advise transaction is outstanding on a topic and item pair
// Returns: Data handle of the item that has changed or NULL
#define ON_XTYP_ADVREQ(fName) \
	case XTYP_ADVREQ: \
		return pThis->fName(hConv, hSZ1, hSZ2, nFormat, (WORD)dwData1);

// Establishes an advise loop
// Returns: Allows (TRUE) the advise loop or not (FALSE)
#define ON_XTYP_ADVSTART(fName) \
	case XTYP_ADVSTART: \
		return (HDDEDATA)pThis->fName(hConv, hSZ1, hSZ2, nFormat);

// Stops an advise loop
#define ON_XTYP_ADVSTOP(fName) \
	case XTYP_ADVSTOP: \
		pThis->fName(hConv, hSZ1, hSZ2, nFormat); \
		return (HDDEDATA)_eResult;

// Establishes a conversation
// Returns: Allows (TRUE) to establish the conversation or not (FALSE)
#define ON_XTYP_CONNECT(fName) \
	case XTYP_CONNECT: \
		return (HDDEDATA)pThis->fName(hSZ2, hSZ1, (PCONVCONTEXT)dwData1, dwData2);

// Confirms a conversation has been established
#define ON_XTYP_CONNECT_CONFIRM(fName) \
	case XTYP_CONNECT_CONFIRM: \
		pThis->fName(hConv, hSZ2, hSZ1, dwData2); \
		return (HDDEDATA)_eResult;

// Terminates a conversation
#define ON_XTYP_DISCONNECT(fName) \
	case XTYP_DISCONNECT: \
		pThis->fName(hConv, dwData2); \
		return (HDDEDATA)_eResult;

// A critical error occurs
#define ON_XTYP_ERROR(fName) \
	case XTYP_ERROR: \
		pThis->fName(hConv, (WORD)dwData1); \
		return (HDDEDATA)_eResult;

// Requests to process a series of commands
// Returns: Result value of the transaction processed
#define ON_XTYP_EXECUTE(fName) \
	case XTYP_EXECUTE: \
		return (HDDEDATA)pThis->fName(hConv, hSZ1, hData);

// Monitors DDE activity in the system
#define ON_XTYP_MONITOR(fName) \
	case XTYP_MONITOR: \
		pThis->fName(hData, dwData2); \
		return (HDDEDATA)_eResult;

// Requests to accept an unsolicited data item
// Returns: Result value of the transaction processed
#define ON_XTYP_POKE(fName) \
	case XTYP_POKE: \
		return (HDDEDATA)pThis->fName(hConv, hSZ1, hSZ2, hData, nFormat);

// Registers a service name or a system topic is started
#define ON_XTYP_REGISTER(fName) \
	case XTYP_REGISTER: \
		pThis->fName(hSZ1, hSZ2); \
		return (HDDEDATA)_eResult;

// Requests a data item
// Returns: Data handle of the item that has changed or NULL
#define ON_XTYP_REQUEST(fName) \
	case XTYP_REQUEST: \
		return pThis->fName(hConv, hSZ1, hSZ2, nFormat);

// Unregisters a service name or a system topic is terminated
#define ON_XTYP_UNREGISTER(fName) \
	case XTYP_UNREGISTER: \
		pThis->fName(hSZ1, hSZ2); \
		return (HDDEDATA)_eResult;

// Establishes a conversation on each service and topic name pairs
// Returns: Data handle of an array contains HSZPAIR structures or NULL
#define ON_XTYP_WILDCONNECT(fName) \
	case XTYP_WILDCONNECT: \
		return pThis->fName(hSZ2, hSZ1, (PCONVCONTEXT)dwData1, dwData2);

// An asynchronous transaction has completed
#define ON_XTYP_XACT_COMPLETE(fName) \
	case XTYP_XACT_COMPLETE: \
		pThis->fName(hConv, hSZ1, hSZ2, hData, nFormat, dwData1); \
		return (HDDEDATA)_eResult;

////////////////////////////////////////////////////////////////////////////////////////////////
// CDdeObject

// Only can be derived (cannot be instantiated), and only one derived object can be created
class CDdeObject
{
public:
	// Instance identifier
	DWORD m_dwInstId;

public:
	operator DWORD() const;
	BOOL AbandonTransaction(HCONV hConv = NULL, DWORD dwTransId = 0) const;
	BOOL CheckQueue(HCONV hConv = NULL) const;
	HCONV Connect(HSZ hService, HSZ hTopic, PCONVCONTEXT pCC = NULL) const;
	HCONVLIST ConnectList(HSZ hService, HSZ hTopic, HCONVLIST hConvList = NULL,
		PCONVCONTEXT pCC = NULL) const;
	HDDEDATA CreateData(HSZ hItem, PVOID pvData, DWORD cbData, UINT nFormat, DWORD cbOffset = 0,
		UINT nFlag = UNUSED) const;
	HDDEDATA CreateData(PCTSTR pszCmdStr, DWORD cbCmdStr, DWORD cbOffset = 0,
		UINT nFlag = UNUSED) const;
	HSZ CreateString(PCTSTR pszName, int iCodePage = CP_WINNEUTRAL) const;
	BOOL EnableCallback(HCONV hConv, UINT nCode) const;
	BOOL EnableCallback(UINT nCode) const;
	BOOL FreeString(HSZ hSZ) const;
	UINT GetLastError() const;
	DWORD GetString(HSZ hSZ, PTSTR pszBuffer, DWORD ccBuffer,
		int iCodePage = CP_WINNEUTRAL) const;
	DWORD GetStringLength(HSZ hSZ, int iCodePage = CP_WINNEUTRAL) const;
	UINT Initialize(DWORD dwFlags = APPCMD_CLIENTONLY);
	BOOL KeepString(HSZ hSZ) const;
	BOOL NameService(HSZ hService, UINT nFlag) const;
	BOOL NameService(UINT nFlag) const;
	BOOL PostAdvise(HSZ hTopic = NULL, HSZ hItem = NULL) const;
	BOOL Uninitialize() const;

protected:
	// Default return value (used internally only)
	enum { _eResult = 0 };

protected:
	// Object pointer to itself (used internally only)
	static CDdeObject* _m_pThis;

protected:
	CDdeObject();
};

AFXINLINE CDdeObject::CDdeObject()
{
	ASSERT(!_m_pThis);
	_m_pThis = this;
}

AFXINLINE CDdeObject::operator DWORD() const
{
	return m_dwInstId;
}

AFXINLINE BOOL CDdeObject::AbandonTransaction(HCONV hConv /* = NULL */,
	DWORD dwTransId /* = 0 */) const
{
	return VERIFY(::DdeAbandonTransaction(m_dwInstId, hConv, dwTransId));
}

AFXINLINE BOOL CDdeObject::CheckQueue(HCONV hConv /* = NULL */) const
{
	BOOL bWaiting = ::DdeEnableCallback(m_dwInstId, hConv, EC_QUERYWAITING);
	ASSERT(bWaiting || (::DdeGetLastError(m_dwInstId) == DMLERR_NO_ERROR));
	return bWaiting;
}

AFXINLINE HCONV CDdeObject::Connect(HSZ hService, HSZ hTopic,
	PCONVCONTEXT pCC /* = NULL */) const
{
	HCONV hConv = ::DdeConnect(m_dwInstId, hService, hTopic, pCC);
	ASSERT(hConv || (::DdeGetLastError(m_dwInstId) == DMLERR_NO_CONV_ESTABLISHED));
	return hConv;
}

AFXINLINE HCONVLIST CDdeObject::ConnectList(HSZ hService, HSZ hTopic,
	HCONVLIST hConvList /* = NULL */, PCONVCONTEXT pCC /* = NULL */) const
{
	HCONVLIST hNewList = ::DdeConnectList(m_dwInstId, hService, hTopic, hConvList, pCC);
	ASSERT(hNewList || (::DdeGetLastError(m_dwInstId) == DMLERR_NO_CONV_ESTABLISHED));
	return hNewList;
}

AFXINLINE HDDEDATA CDdeObject::CreateData(HSZ hItem, PVOID pvData, DWORD cbData,
	UINT nFormat, DWORD cbOffset /* = 0 */, UINT nFlag /* = UNUSED */) const
{
	return (HDDEDATA)VERIFY(::DdeCreateDataHandle(m_dwInstId, (PBYTE)pvData, cbData,
		cbOffset, hItem, nFormat, nFlag));
}

AFXINLINE HDDEDATA CDdeObject::CreateData(PCTSTR pszCmdStr, DWORD cbCmdStr,
	DWORD cbOffset /* = 0 */, UINT nFlag /* = UNUSED */) const
{
	return (HDDEDATA)VERIFY(::DdeCreateDataHandle(m_dwInstId, (PBYTE)pszCmdStr, cbCmdStr,
		cbOffset, NULL, UNUSED, nFlag));
}

AFXINLINE HSZ CDdeObject::CreateString(PCTSTR pszName,
	int iCodePage /* = CP_WINNEUTRAL */) const
{
	return (HSZ)VERIFY(::DdeCreateStringHandle(m_dwInstId, pszName, iCodePage));
}

AFXINLINE BOOL CDdeObject::EnableCallback(HCONV hConv, UINT nCode) const
{
	return VERIFY(::DdeEnableCallback(m_dwInstId, hConv, nCode));
}

AFXINLINE BOOL CDdeObject::EnableCallback(UINT nCode) const
{
	return VERIFY(::DdeEnableCallback(m_dwInstId, NULL, nCode));
}

AFXINLINE BOOL CDdeObject::FreeString(HSZ hSZ) const
{
	return VERIFY(::DdeFreeStringHandle(m_dwInstId, hSZ));
}

AFXINLINE UINT CDdeObject::GetLastError() const
{
	return ::DdeGetLastError(m_dwInstId);
}

AFXINLINE DWORD CDdeObject::GetString(HSZ hSZ, PTSTR pszBuffer, DWORD ccBuffer,
	int iCodePage /* = CP_WINNEUTRAL */) const
{
	return VERIFY(::DdeQueryString(m_dwInstId, hSZ, pszBuffer, ccBuffer, iCodePage));
}

AFXINLINE DWORD CDdeObject::GetStringLength(HSZ hSZ, int iCodePage /* = CP_WINNEUTRAL */) const
{
	return VERIFY(::DdeQueryString(m_dwInstId, hSZ, NULL, UNUSED, iCodePage));
}

AFXINLINE BOOL CDdeObject::KeepString(HSZ hSZ) const
{
	return VERIFY(::DdeKeepStringHandle(m_dwInstId, hSZ));
}

AFXINLINE BOOL CDdeObject::NameService(HSZ hService, UINT nFlag) const
{
	return (BOOL)VERIFY(::DdeNameService(m_dwInstId, hService, NULL, nFlag));
}

AFXINLINE BOOL CDdeObject::NameService(UINT nFlag) const
{
	return (BOOL)VERIFY(::DdeNameService(m_dwInstId, NULL, NULL, nFlag));
}

AFXINLINE BOOL CDdeObject::PostAdvise(HSZ hTopic /* = NULL */, HSZ hItem /* = NULL */) const
{
	return VERIFY(::DdePostAdvise(m_dwInstId, hTopic, hItem));
}

AFXINLINE BOOL CDdeObject::Uninitialize() const
{
	return VERIFY(::DdeUninitialize(m_dwInstId));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Helper functions

AFXINLINE PBYTE WINAPI DdeAccessData(HDDEDATA hData)
{
	return (PBYTE)VERIFY(DdeAccessData(hData, NULL));
}

AFXINLINE HDDEDATA WINAPI DdeAddData(HDDEDATA hData, PVOID pvData, DWORD cbData)
{
	return (HDDEDATA)VERIFY(DdeAddData(hData, (PBYTE)pvData, cbData, 0));
}

AFXINLINE int WINAPI DdeCompareString(HSZ hSZ1, HSZ hSZ2)
{
	return DdeCmpStringHandles(hSZ1, hSZ2);
}

AFXINLINE BOOL WINAPI DdeFreeData(HDDEDATA hData)
{
	return VERIFY(DdeFreeDataHandle(hData));
}

AFXINLINE DWORD WINAPI DdeGetData(HDDEDATA hData, PVOID pvBuffer, DWORD cbBuffer)
{
	return VERIFY(DdeGetData(hData, (PBYTE)pvBuffer, cbBuffer, 0));
}

AFXINLINE DWORD WINAPI DdeGetDataSize(HDDEDATA hData, DWORD cbOffset = 0)
{
	return VERIFY(DdeGetData(hData, NULL, UNUSED, cbOffset));
}

AFXINLINE UINT WINAPI DdeQueryConvInfo(HCONV hConv, PCONVINFO pCI, DWORD dwTransId = QID_SYNC)
{
	return VERIFY(DdeQueryConvInfo(hConv, dwTransId, pCI));
}

AFXINLINE HCONV WINAPI DdeQueryNextServer(HCONVLIST hConvList)
{
	return DdeQueryNextServer(hConvList, NULL);
}

AFXINLINE BOOL WINAPI DdeSetUserData(HCONV hConv, DWORD dwUser, DWORD dwTransId = QID_SYNC)
{
	return VERIFY(DdeSetUserHandle(hConv, dwTransId, dwUser));
}

AFXINLINE BOOL WINAPI DdeTransAdvStart(HCONV hConv, HSZ hItem, UINT nFormat,
	UINT nFlags = UNUSED, DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction(NULL, UNUSED, hConv, hItem, nFormat,
		XTYP_ADVSTART | nFlags, dwTimeout, pdwTransId));
}

AFXINLINE BOOL WINAPI DdeTransAdvStop(HCONV hConv, HSZ hItem, UINT nFormat,
	DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction(NULL, UNUSED, hConv, hItem, nFormat,
		XTYP_ADVSTOP, dwTimeout, pdwTransId));
}

AFXINLINE BOOL WINAPI DdeTransExecute(HCONV hConv, PCTSTR pszCmdStr, DWORD cbCmdStr,
	DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction((PBYTE)pszCmdStr, cbCmdStr, hConv, NULL, UNUSED,
		XTYP_EXECUTE, dwTimeout, pdwTransId));
}

AFXINLINE BOOL WINAPI DdeTransExecute(HCONV hConv, HDDEDATA hCmdStr,
	DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction((PBYTE)hCmdStr, -1, hConv, NULL, UNUSED,
		XTYP_EXECUTE, dwTimeout, pdwTransId));
}

AFXINLINE BOOL WINAPI DdeTransPoke(HCONV hConv, HSZ hItem, PVOID pvData, DWORD cbData,
	UINT nFormat, DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction((PBYTE)pvData, cbData, hConv, hItem,
		nFormat, XTYP_POKE, dwTimeout, pdwTransId));
}

AFXINLINE BOOL WINAPI DdeTransPoke(HCONV hConv, HSZ hItem, HDDEDATA hData, UINT nFormat,
	DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (BOOL)VERIFY(DdeClientTransaction((PBYTE)hData, -1, hConv, hItem, nFormat,
		XTYP_POKE, dwTimeout, pdwTransId));
}

AFXINLINE HDDEDATA WINAPI DdeTransRequest(HCONV hConv, HSZ hItem, UINT nFormat,
	DWORD dwTimeout = TIMEOUT_ASYNC, PDWORD pdwTransId = NULL)
{
	return (HDDEDATA)VERIFY(DdeClientTransaction(NULL, UNUSED, hConv, hItem, nFormat,
		XTYP_REQUEST, dwTimeout, pdwTransId));
}

// End of _DDEML_INCLUDED_
#endif
