////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// DirectX.hpp - DirectX Macros and Functions Declaration File                                //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _DIRECTX_INCLUDED_
#define _DIRECTX_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  DirectX - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// DxErr

// Debug version
#ifdef DEBUG

// Displays a message by a message box or into the debug window (used internally only)
HRESULT WINAPI _DXTRACE(PCSTR pszFileName, int iLine, HRESULT hResult, PCWSTR pszMsg,
	BOOL bMsgBox);
HRESULT WINAPI _DXTRACE(PCSTR pszFileName, int iLine, HRESULT hResult, PCSTR pszMsg,
	BOOL bMsgBox);

// Displays a message by a message box or into the debug window
#define DXTRACE(hResult, pszMsg, bMsgBox) _DXTRACE(__FILE__, __LINE__, hResult, pszMsg, bMsgBox)

// Determines whether a resulting value is succeeded
#define DXASSERT(hResult) \
	do \
	{ \
		HRESULT _hResult = (hResult); \
		if(FAILED(_hResult)) \
			_DXTRACE(__FILE__, __LINE__, _hResult, "DXASSERT(" #hResult ")", TRUE); \
	} \
	while(0)

// Determines whether a resulting value is succeeded (used internally only)
HRESULT WINAPI _DXVERIFY(PCSTR pszFileName, int iLine, HRESULT hResult, PCSTR pszMsg);

// Determines whether a resulting value is succeeded
// Returns: Original resulting value
#define DXVERIFY(hResult) _DXVERIFY(__FILE__, __LINE__, hResult, #hResult)

// Release version
#else

// Removes the following macros
#define DXTRACE(hResult, pszMsg, bMsgBox) (hResult)
#define DXASSERT(hResult)                 ((void)0)
#define DXVERIFY(hResult)                 (hResult)

// End of DEBUG and _DIRECTX_INCLUDED_
#endif
#endif
