////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// MsgMap.hpp - Message Map Declaration File                                                  //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _MSGMAP_INCLUDED_
#define _MSGMAP_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  MsgMap - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

// Declares in window class
#define DECLARE_MESSAGE_MAP() \
	public: \
		static LRESULT CALLBACK _WindowProc(HWND hWnd, UINT nMsg, \
			WPARAM wParam, LPARAM lParam); \
		BOOL CreateDialog(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return (BOOL)VERIFY(::CreateDialogParam(hInstance, pszTemplate, \
				hOwnerWnd, (DLGPROC)_WindowProc, UNUSED)); \
		} \
		BOOL CreateDialog(HINSTANCE hInstance, UINT nTmplId, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return (BOOL)VERIFY(::CreateDialogParam(hInstance, MAKERESPTR(nTmplId), \
				hOwnerWnd, (DLGPROC)_WindowProc, UNUSED)); \
		} \
		BOOL CreateDialog(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return (BOOL)VERIFY(::CreateDialogIndirectParam(hInstance, pDT, hOwnerWnd, \
				(DLGPROC)_WindowProc, UNUSED)); \
		} \
		int DialogBox(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return UNEQUAL(::DialogBoxParam(hInstance, pszTemplate, hOwnerWnd, \
				(DLGPROC)_WindowProc, UNUSED), RV_ERR); \
		} \
		int DialogBox(HINSTANCE hInstance, UINT nTmplId, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return UNEQUAL(::DialogBoxParam(hInstance, MAKERESPTR(nTmplId), hOwnerWnd, \
				(DLGPROC)_WindowProc, UNUSED), RV_ERR); \
		} \
		int DialogBox(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hOwnerWnd) \
		{ \
			ASSERT(!_m_pWnd && (_eWndType == _WT_DIALOG)); \
			_m_pWnd = this; \
			return UNEQUAL(::DialogBoxIndirectParam(hInstance, pDT, hOwnerWnd, \
				(DLGPROC)_WindowProc, UNUSED), RV_ERR); \
		} \
		void SubclassWindow() \
		{ \
			ASSERT((_eWndType == _WT_SUBCLASS) && ::IsWindow(m_hWnd) && !m_wpPrev); \
			::SetWindowLong(m_hWnd, GWL_USERDATA, (LONG)(CWnd*)this); \
			m_wpPrev = (WNDPROC)::SetWindowLong(m_hWnd, GWL_WNDPROC, (LONG)_WindowProc); \
			ASSERT(m_wpPrev && (m_wpPrev != _WindowProc)); \
		}

// Begins message maps
// CName : Name of window class
#define BEGIN_MESSAGE_MAP(CName) \
	LRESULT CALLBACK CName::_WindowProc(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam) \
	{ \
		int     iError; \
		UINT    nItemId, nNotify, nCode, nParam; \
		LRESULT lResult; \
		HMEMORY hMemory; \
		HWND    hChildWnd; \
		CName*  pWnd; \
		DEBUG_ONLY(_m_cbObject = sizeof(CName)); \
		pWnd = (CName*)_GetObjectPtr(hWnd); \
		switch(nMsg) \
		{

// Ends message maps
#define END_MESSAGE_MAP() \
		} \
		xDefProc: \
		lResult = pWnd->DefWindowProc(nMsg, wParam, lParam); \
		if(nMsg == WM_NCDESTROY) \
		{ \
			pWnd->DetachObject(); \
			DEBUG_ONLY(pWnd->m_wpPrev = NULL); \
		} \
		return lResult; \
		UNREFERENCED_LABEL(xDefProc); \
		UNREFERENCED_VARIABLE(iError); \
		UNREFERENCED_VARIABLE(nItemId); \
		UNREFERENCED_VARIABLE(nNotify); \
		UNREFERENCED_VARIABLE(nCode); \
		UNREFERENCED_VARIABLE(nParam); \
		UNREFERENCED_VARIABLE(hMemory); \
		UNREFERENCED_VARIABLE(hChildWnd); \
	}

// Gets the window/dialog procedure of a class object
// Returns: Address of the procedure
#define GetWindowProc(CName) (CName::_WindowProc)
#define GetDialogProc(CName) ((DLGPROC)CName::_WindowProc)

// Gets a pointer to the attached object associated with a window
// Returns: A pointer to the object
template<class tClassName>
AFXINLINE void WINAPI GetObjectPtr(HWND hWnd, tClassName** tppObjName)
{
	*tppObjName = (tClassName*)GetWindowLong(hWnd, GWL_USERDATA);
	ASSERT_WRITE(*tppObjName, sizeof(tClassName));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps

// Custom messages used by applications
#define WM_REGISTER 0xC000             // Registered message
#define _WM_CTMCTRL (WM_REGISTER - 1)  // Custom control creation (used internally only)
#define WM_IDLE     (WM_REGISTER - 2)  // Window is entering an idle state

// General return values for message maps
#define WMR_PROCESSED _eResult  // Message is processed
#define WMR_DEFAULT   (-2013)   // Message uses the default processing

// Same as the following map
#define ON_SAME_MAP(nMsg) \
	case nMsg:

// Handles any sort of a message
// LRESULT OnAny(WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
#define ON_ANY_MAP(nMsg, fName) \
	case nMsg: \
		ASSERT(nMsg <= MAX_WORD); \
		lResult = pWnd->fName(wParam, lParam); \
		if(lResult != WMR_DEFAULT) \
			return lResult; \
		goto xDefProc;

// Begins/ends mapping a group of messages
// LRESULT OnGroup(UINT nMsg, WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
#define BEGIN_GROUP_MAP() {
#define ON_GROUP(nMsg) ON_SAME_MAP(nMsg)
#define END_GROUP_MAP(fName) \
		ASSERT(nMsg <= MAX_WORD); \
		lResult = pWnd->fName(nMsg, wParam, lParam); \
		if(lResult != WMR_DEFAULT) \
			return lResult; \
		goto xDefProc; \
	}

// Handles any sort of a message, and falls through the handler, if necessary
// LRESULT OnThrough(WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT, WMR_THROUGH)
#define WMR_THROUGH (WMR_DEFAULT + 1)
#define ON_THROUGH_MAP(nMsg, fName) \
	case nMsg: \
		ASSERT(nMsg <= MAX_WORD); \
		lResult = pWnd->fName(wParam, lParam); \
		if(lResult == WMR_DEFAULT) \
			goto xDefProc; \
		if(lResult != WMR_THROUGH) \
			return lResult;

// Top-level window is being activated or deactivated
// void OnActivate(UINT nState, HWND hOtherWnd, BOOL bMinimized);
#define ON_WM_ACTIVATE(fName) \
	case WM_ACTIVATE: \
		pWnd->fName(LOWORD(wParam), (HWND)lParam, HIWORD(wParam)); \
		goto xDefProc;

// Application is being activated or deactivated
// void OnActivateApp(BOOL bActive, DWORD dwThreadId);
#define _ON_WM_ACTIVATEAPP(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, lParam); \
		return _eResult;
#define ON_WM_ACTIVATEAPP(fName) _ON_WM_ACTIVATEAPP(WM_ACTIVATEAPP, fName)

// Integer message for use by application
// LRESULT OnApp(WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed (or WMR_PROCESSED)
#define _ON_WM_APP(nMsg, nMinMsg, nMaxMsg, fName) \
	case nMsg: \
		ASSERT((nMinMsg <= nMsg) && (nMsg < nMaxMsg)); \
		return pWnd->fName(wParam, lParam);
#define ON_WM_APP(nMsg, fName) _ON_WM_APP(nMsg, WM_APP, WM_REGISTER, fName)

// Requests the name of a CF_OWNERDISPLAY clipboard format
// void OnAskCbFormatName(PTSTR pszBuffer, int ccBuffer);
#define ON_WM_ASKCBFORMATNAME(fName) \
	case WM_ASKCBFORMATNAME: \
		pWnd->fName((PTSTR)lParam, wParam); \
		return _eResult;

// Cancels certain modes
// void OnCancelMode();
#define _ON_WM_CANCELMODE(nMsg, fName) \
	case nMsg: \
		pWnd->fName(); \
		goto xDefProc;
#define ON_WM_CANCELMODE(fName) _ON_WM_CANCELMODE(WM_CANCELMODE, fName)

// Window is losing the mouse capture
// void OnCaptureChanged(HWND hNewWnd);
#define ON_WM_CAPTURECHANGED(fName) \
	case WM_CAPTURECHANGED: \
		pWnd->fName((HWND)lParam); \
		return _eResult;

// Window is being removed from the clipboard viewer chain
// void OnChangeCbChain(HWND hRemoveWnd, HWND hNextWnd);
#define ON_WM_CHANGECBCHAIN(fName) \
	case WM_CHANGECBCHAIN: \
		pWnd->fName((HWND)wParam, (HWND)lParam); \
		return _eResult;

// A character is pressed
// void OnChar(TCHAR cChar, LPARAM lKeyData);
#define _ON_WM_CHAR(nMsg, fName) \
	case nMsg: \
		pWnd->fName((TCHAR)wParam, lParam); \
		goto xDefProc;
#define ON_WM_CHAR(fName) _ON_WM_CHAR(WM_CHAR, fName)

// Owner-drawn list box in response to a WM_CHAR message
// int OnCharToItem(TCHAR cChar, int iItem, HWND hListBox);
// Returns: No further action (-1) or default action on an item
#define _ON_WM_CHARTOITEM(nMsg, fName) \
	case nMsg: \
		return pWnd->fName((TCHAR)LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
#define ON_WM_CHARTOITEM(fName) _ON_WM_CHARTOITEM(WM_CHARTOITEM, fName)

// Child window is activated, moved or sized, or the user clicks the title bar
// void OnChildActivate();
#define ON_WM_CHILDACTIVATE(fName) \
	case WM_CHILDACTIVATE: \
		pWnd->fName(); \
		if(_eWndType != _WT_MDICHILD) \
			return _eResult; \
		goto xDefProc;

// Contents of the clipboard have changed
// void OnClipboardUpdate();
#define _ON_WM_CLIPBOARDUPDATE(nMsg, fName) \
	case nMsg: \
		pWnd->fName(); \
		return _eResult;
#define ON_WM_CLIPBOARDUPDATE(fName) _ON_WM_CLIPBOARDUPDATE(WM_CLIPBOARDUPDATE, fName)

// Window is being closed
// BOOL OnClose();
// Returns: Accepts (TRUE) or rejects (FALSE) the window to close
#define ON_WM_CLOSE(fName) \
	case WM_CLOSE: \
		if(!pWnd->fName()) \
			return _eResult; \
		if(_eWndType != _WT_DIALOG) \
			goto xDefProc; \
		::DestroyWindow(hWnd); \
		return _eResult;

// A command item from a menu or a control sends a notification
// void OnCommand(UINT nItemId, UINT nNotify, HWND hCtrlWnd);
#define ON_WM_COMMAND(fName) \
	case WM_COMMAND: \
		pWnd->fName(LOWORD(wParam), HIWORD(wParam), (HWND)lParam); \
		if(_eWndType != _WT_MDIFRAME) \
			return _eResult; \
		hChildWnd = (HWND)::SendMessage(pWnd->m_hClientWnd, WM_MDIGETACTIVE, UNUSED, UNUSED); \
		if(hChildWnd) \
			::SendMessage(hChildWnd, WM_COMMAND, wParam, lParam); \
		goto xDefProc;

// Begins command message maps
#define BEGIN_COMMAND_MAP() \
	case WM_COMMAND: \
		nItemId = LOWORD(wParam); \
		nNotify = HIWORD(wParam); \
		switch(nItemId) \
		{

// A command item from a menu
// void OnCmdItem();
#define ON_COMMAND(nItemId, fName) \
		case nItemId: \
			pWnd->fName(); \
			if(_eWndType != _WT_MDIFRAME) \
				return _eResult; \
			goto xForCmd;

// void OnCmdItemParam(LPARAM lParam);
#define ON_COMMAND_PARAM(nItemId, fName) \
		case nItemId: \
			pWnd->fName(lParam); \
			if(_eWndType != _WT_MDIFRAME) \
				return _eResult; \
			goto xForCmd;

// A command item in the range from a menu
// void OnCmdItemRange(UINT nItemId);
#define ON_COMMAND_RANGE(nItemId1, nItemId2, fName) \
		} \
		if((nItemId1 <= nItemId) && (nItemId <= nItemId2)) \
		{ \
			pWnd->fName(nItemId); \
			if(_eWndType != _WT_MDIFRAME) \
				return _eResult; \
			goto xForCmd; \
		} \
		switch(nItemId) \
		{

// A control sends a notification message
// void OnCmdCtrl(UINT nNotify, HWND hCtrlWnd);
#define ON_CONTROL(nCtrlId, fName) \
		case nCtrlId: \
			pWnd->fName(nNotify, (HWND)lParam); \
			return _eResult;

// A control in the range sends a notification message
// void OnCmdCtrlRange(UINT nCtrlId, UINT nNotify, HWND hCtrlWnd);
#define ON_CONTROL_RANGE(nCtrlId1, nCtrlId2, fName) \
		} \
		if((nCtrlId1 <= nItemId) && (nItemId <= nCtrlId2)) \
		{ \
			pWnd->fName(nItemId, nNotify, (HWND)lParam); \
			return _eResult; \
		} \
		switch(nItemId) \
		{

// Begins control notification maps
#define BEGIN_CONTROL_NOTIFY_MAP(nCtrlId) \
		case nCtrlId: \
			switch(nNotify) \
			{

// Control notification map
#define _ON_CONTROL_NOTIFY(nNotify, fName) \
			case nNotify: \
				pWnd->fName((HWND)lParam); \
				return _eResult;

// Ends control notification maps
#define END_CONTROL_NOTIFY_MAP() \
			} \
			return _eResult;

// Ends command message maps
#define END_COMMAND_MAP() \
		} \
		if(_eWndType != _WT_MDIFRAME) \
			return _eResult; \
		xForCmd: \
		hChildWnd = (HWND)::SendMessage(pWnd->m_hClientWnd, WM_MDIGETACTIVE, UNUSED, UNUSED); \
		if(hChildWnd) \
			::SendMessage(hChildWnd, WM_COMMAND, wParam, lParam); \
		goto xDefProc; \
		UNREFERENCED_LABEL(xForCmd);

// The system is being spent compacting memory
// void OnCompacting(UINT nRatio);
#define _ON_WM_COMPACTING(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam); \
		return _eResult;
#define ON_WM_COMPACTING(fName) _ON_WM_COMPACTING(WM_COMPACTING, fName)

// Determines the relative position of a new item in an owner-drawn combo box or list box
// int OnCompareItem(PCCOMPAREITEMSTRUCT pCIS);
// Returns: Item 1 precedes (-1), equals (0) or follows (1) the item 2
#define ON_WM_COMPAREITEM(fName) \
	case WM_COMPAREITEM: \
		return pWnd->fName((PCCOMPAREITEMSTRUCT)lParam);

// The user clicked the right mouse button in a window
// void OnContextMenu(HWND hContextWnd, int xCursor, int yCursor);
#define ON_WM_CONTEXTMENU(fName) \
	case WM_CONTEXTMENU: \
		pWnd->fName((HWND)wParam, LOSHORT(lParam), HISHORT(lParam)); \
		return _eResult;

// Window is being created
// int OnCreate(PCCREATESTRUCT pCS);
// Returns: Continues creation (0) or destroys (-1) the window
#define _ON_WM_CREATE(nMsg, fName) \
	case nMsg: \
		return pWnd->fName((PCCREATESTRUCT)lParam);
#define ON_WM_CREATE(fName) _ON_WM_CREATE(WM_CREATE, fName)

// Button is about to be drawn
// HBRUSH OnCtlColorBtn(HDC hDC, HWND hButton);
// Returns: Brush to paint the background (or NULL)
#define _ON_WM_CTLCOLORBTN(nMsg, fName) \
	case nMsg: \
		return (LRESULT)pWnd->fName((HDC)wParam, (HWND)lParam);
#define ON_WM_CTLCOLORBTN(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLORBTN, fName)

// Dialog box is about to be drawn
// HBRUSH OnCtlColorDlg(HDC hDC);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLORDLG(fName) \
	case WM_CTLCOLORDLG: \
		return (LRESULT)pWnd->fName((HDC)wParam);

// Edit is about to be drawn
// HBRUSH OnCtlColorEdit(HDC hDC, HWND hEdit);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLOREDIT(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLOREDIT, fName)

// List box is about to be drawn
// HBRUSH OnCtlColorListBox(HDC hDC, HWND hListBox);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLORLISTBOX(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLORLISTBOX, fName)

// Message box is about to be drawn
// HBRUSH OnCtlColorMsgBox(HDC hDC, HWND hMsgBox);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLORMSGBOX(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLORMSGBOX, fName)

// Scroll bar is about to be drawn
// HBRUSH OnCtlColorScrollBar(HDC hDC, HWND hScrollBar);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLORSCROLLBAR(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLORSCROLLBAR, fName)

// Static is about to be drawn
// HBRUSH OnCtlColorStatic(HDC hDC, HWND hStatic);
// Returns: Brush to paint the background (or NULL)
#define ON_WM_CTLCOLORSTATIC(fName) _ON_WM_CTLCOLORBTN(WM_CTLCOLORSTATIC, fName)

// A dead character is pressed
// void OnDeadChar(TCHAR cChar, LPARAM lKeyData);
#define ON_WM_DEADCHAR(fName) _ON_WM_CHAR(WM_DEADCHAR, fName)

// A combo box or list box is destroyed or its items are deleted
// void OnDeleteItem(PCDELETEITEMSTRUCT pDIS);
#define ON_WM_DELETEITEM(fName) \
	case WM_DELETEITEM: \
		pWnd->fName((PCDELETEITEMSTRUCT)lParam); \
		return TRUE;

// Window is being destroyed
// void OnDestroy();
#define ON_WM_DESTROY(fName) \
	case WM_DESTROY: \
		pWnd->fName(); \
		if(_eWndType != _WT_SCRNSAVE) \
			return _eResult; \
		goto xDefProc;

// Delayed rendering for clipboard data is no longer needed
// void OnDestroyClipboard();
#define ON_WM_DESTROYCLIPBOARD(fName) _ON_WM_CLIPBOARDUPDATE(WM_DESTROYCLIPBOARD, fName)

// A change to the hardware configuration of a device or the computer
// UINT OnDeviceChange(UINT nEvent, PCVOID pvStruct);
// Returns: Accepts (TRUE) or rejects (BROADCAST_QUERY_DENY) the request
#define ON_WM_DEVICECHANGE(fName) \
	case WM_DEVICECHANGE: \
		return pWnd->fName(wParam, (PCVOID)lParam);

// Display resolution has changed
// void OnDisplayChange(UINT nBitsPixel, int cxScreen, int cyScreen);
#define _ON_WM_DISPLAYCHANGE(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, LOSHORT(lParam), HISHORT(lParam)); \
		return _eResult;
#define ON_WM_DISPLAYCHANGE(fName) _ON_WM_DISPLAYCHANGE(WM_DISPLAYCHANGE, fName)

// Content of the clipboard changes
// void OnDrawClipboard();
#define ON_WM_DRAWCLIPBOARD(fName) _ON_WM_CLIPBOARDUPDATE(WM_DRAWCLIPBOARD, fName)

// A visual aspect of an owner-drawn control or menu item has changed
// void OnDrawItem(PCDRAWITEMSTRUCT pDIS);
#define ON_WM_DRAWITEM(fName) \
	case WM_DRAWITEM: \
		pWnd->fName((PCDRAWITEMSTRUCT)lParam); \
		return TRUE;

// Drops files in a window under the cursor
// void OnDropFiles(HDROP hDropFile);
#define ON_WM_DROPFILES(fName) \
	case WM_DROPFILES: \
		pWnd->fName((HDROP)wParam); \
		return _eResult;

// Enabled state of a window is changing
// void OnEnable(BOOL bEnable);
#define ON_WM_ENABLE(fName) _ON_WM_COMPACTING(WM_ENABLE, fName)

// Windows session is ending
// void OnEndSession(BOOL bEnding, UINT nFlags);
#define ON_WM_ENDSESSION(fName) _ON_WM_ACTIVATEAPP(WM_ENDSESSION, fName)

// Modal dialog box or menu enters an idle state
// void OnEnterIdle(UINT nWhy, HWND hWho);
#define ON_WM_ENTERIDLE(fName) \
	case WM_ENTERIDLE: \
		pWnd->fName(wParam, (HWND)lParam); \
		return _eResult;

// Window enters a menu modal loop
// void OnEnterMenuLoop(BOOL bPopupMenu);
#define ON_WM_ENTERMENULOOP(fName) _ON_WM_COMPACTING(WM_ENTERMENULOOP, fName)

// Window enters the moving or sizing mode
// void OnEnterSizeMove();
#define ON_WM_ENTERSIZEMOVE(fName) _ON_WM_CLIPBOARDUPDATE(WM_ENTERSIZEMOVE, fName)

// Window background should be erased
// BOOL OnEraseBkgnd(HDC hDC);
// Returns: Background is erased (TRUE) or not (FALSE)
#define ON_WM_ERASEBKGND(fName) \
	case WM_ERASEBKGND: \
		return pWnd->fName((HDC)wParam);

// Window exits a menu modal loop
// void OnExitMenuLoop(BOOL bPopupMenu);
#define ON_WM_EXITMENULOOP(fName) _ON_WM_COMPACTING(WM_EXITMENULOOP, fName)

// Window exits the moving or sizing mode
// void OnExitSizeMove();
#define ON_WM_EXITSIZEMOVE(fName) _ON_WM_CLIPBOARDUPDATE(WM_EXITSIZEMOVE, fName)

// Gets the type of a control
// UINT OnGetDlgCode(TCHAR cVirtKey, PCMSG pMsg);
// Returns: Type of input the application processes
#define ON_WM_GETDLGCODE(fName) \
	case WM_GETDLGCODE: \
		return pWnd->fName((TCHAR)wParam, (PCMSG)lParam);

// Gets the icon handle associated with a window
// HICON OnGetIcon(UINT nType);
// Returns: Handle to the large or small icon (or NULL)
#define ON_WM_GETICON(fName) \
	case WM_GETICON: \
		return (LRESULT)pWnd->fName(wParam);

// Window size or position is about to change
// void OnGetMinMaxInfo(PMINMAXINFO pMMI);
#define ON_WM_GETMINMAXINFO(fName) \
	case WM_GETMINMAXINFO: \
		if(_eWndType == _WT_MDICHILD) \
			::DefMDIChildProc(hWnd, WM_GETMINMAXINFO, UNUSED, lParam); \
		pWnd->fName((PMINMAXINFO)lParam); \
		return _eResult;

// Gets the window text
// int OnGetText(PTSTR pszBuffer, int ccBuffer);
// Returns: Number of characters writted into the buffer
#define ON_WM_GETTEXT(fName) \
	case WM_GETTEXT: \
		return pWnd->fName((PTSTR)lParam, wParam);

// Gets the length of the window text
// int OnGetTextLength();
// Returns: Length of the text
#define _ON_WM_GETTEXTLENGTH(nMsg, fName) \
	case nMsg: \
		return pWnd->fName();
#define ON_WM_GETTEXTLENGTH(fName) _ON_WM_GETTEXTLENGTH(WM_GETTEXTLENGTH, fName)

// F1 key is pressed
// void OnHelp(PCHELPINFO pHI);
#define ON_WM_HELP(fName) \
	case WM_HELP: \
		pWnd->fName((PCHELPINFO)lParam); \
		return TRUE;

// An event occurs in a horizontal scroll bar
// void OnHScroll(UINT nCode, int iPos, HWND hScrollBar);
#define _ON_WM_HSCROLL(nMsg, fName) \
	case nMsg: \
		pWnd->fName(LOWORD(wParam), HISHORT(wParam), (HWND)lParam); \
		if(_eWndType != _WT_SUBCLASS) \
			return _eResult; \
		goto xDefProc;
#define ON_WM_HSCROLL(fName) _ON_WM_HSCROLL(WM_HSCROLL, fName)

// An event occurs in a clipboard viewer's horizontal scroll bar
// void OnHScrollClipboard(HWND hViewerWnd, UINT nCode, int iPos);
#define _ON_WM_HSCROLLCLIPBOARD(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HWND)wParam, LOWORD(lParam), HISHORT(lParam)); \
		return _eResult;
#define ON_WM_HSCROLLCLIPBOARD(fName) _ON_WM_HSCROLLCLIPBOARD(WM_HSCROLLCLIPBOARD, fName)

// Window is entering an idle state
// BOOL OnIdle(long lCount);
// Returns: More idle processing time (TRUE) or not (FALSE)
#define _ON_WM_IDLE(nMsg, fName) \
	case nMsg: \
		return pWnd->fName(lParam);
#define ON_WM_IDLE(fName) _ON_WM_IDLE(WM_IDLE, fName)

// Dialog box is being displayed
// BOOL OnInitDialog(HWND hFocusWnd);
// Returns: System (TRUE) or application itself (FALSE) sets the keyboard focus
#define _ON_WM_INITDIALOG(nMsg, fName) \
	case nMsg: \
		return pWnd->fName((HWND)wParam);
#define ON_WM_INITDIALOG(fName) _ON_WM_INITDIALOG(WM_INITDIALOG, fName)

// A menu is about to become active
// void OnInitMenu(HMENU hMenu);
#define _ON_WM_INITMENU(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HMENU)wParam); \
		return _eResult;
#define ON_WM_INITMENU(fName) _ON_WM_INITMENU(WM_INITMENU, fName)

// A pop-up menu is about to become active
// void OnInitMenuPopup(HMENU hMenu, UINT nPos, BOOL bSysMenu);
#define ON_WM_INITMENUPOPUP(fName) \
	case WM_INITMENUPOPUP: \
		pWnd->fName((HMENU)wParam, LOWORD(lParam), HIWORD(lParam)); \
		return _eResult;

// Input language has been changed
// void OnInputLangChange(LCID lcName, DWORD dwCharSet);
#define ON_WM_INPUTLANGCHANGE(fName) \
	case WM_INPUTLANGCHANGE: \
		pWnd->fName(lParam, wParam); \
		goto xDefProc;

// Requests the input language to change
// BOOL OnInputLangChangeRequest(LCID lcName, UINT nFlags);
// Returns: Accepts (TRUE) or rejects (FALSE) the change
#define ON_WM_INPUTLANGCHANGEREQUEST(fName) \
	case WM_INPUTLANGCHANGEREQUEST: \
		if(!pWnd->fName(lParam, wParam)) \
			return _eResult; \
		goto xDefProc;

// A nonsystem key is pressed
// void OnKeyDown(TCHAR cVirtKey, LPARAM lKeyData);
#define ON_WM_KEYDOWN(fName) _ON_WM_CHAR(WM_KEYDOWN, fName)

// A nonsystem key is released
// void OnKeyUp(TCHAR cVirtKey, LPARAM lKeyData);
#define ON_WM_KEYUP(fName) _ON_WM_CHAR(WM_KEYUP, fName)

// Window is being lost the keyboard focus
// void OnKillFocus(HWND hNewWnd);
#define ON_WM_KILLFOCUS(fName) \
	case WM_KILLFOCUS: \
		pWnd->fName((HWND)wParam); \
		if(_eWndType != _WT_SUBCLASS) \
			return _eResult; \
		goto xDefProc;

// Left mouse button has double-clicked
// void OnLButtonDblClk(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_LBUTTONDBLCLK(fName) _ON_WM_DISPLAYCHANGE(WM_LBUTTONDBLCLK, fName)

// Left mouse button has pressed
// void OnLButtonDown(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_LBUTTONDOWN(fName) _ON_WM_DISPLAYCHANGE(WM_LBUTTONDOWN, fName)

// Left mouse button has released
// void OnLButtonUp(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_LBUTTONUP(fName) _ON_WM_DISPLAYCHANGE(WM_LBUTTONUP, fName)

// Middle mouse button has double-clicked
// void OnMButtonDblClk(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_MBUTTONDBLCLK(fName) _ON_WM_DISPLAYCHANGE(WM_MBUTTONDBLCLK, fName)

// Middle mouse button has pressed
// void OnMButtonDown(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_MBUTTONDOWN(fName) _ON_WM_DISPLAYCHANGE(WM_MBUTTONDOWN, fName)

// Middle mouse button has released
// void OnMButtonUp(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_MBUTTONUP(fName) _ON_WM_DISPLAYCHANGE(WM_MBUTTONUP, fName)

// MDI child window is being activated or deactivated
// void OnMdiActivate(HWND hActiveWnd, HWND hDeactWnd);
#define ON_WM_MDIACTIVATE(fName) \
	case WM_MDIACTIVATE: \
		pWnd->fName((HWND)lParam, (HWND)wParam); \
		return _eResult;

// An owner-drawn control or menu item is created
// void OnMeasureItem(PMEASUREITEMSTRUCT pMIS);
#define ON_WM_MEASUREITEM(fName) \
	case WM_MEASUREITEM: \
		pWnd->fName((PMEASUREITEMSTRUCT)lParam); \
		return TRUE;

// A key does not correspond to any mnemonic or accelerator key
// LRESULT OnMenuChar(TCHAR cChar, UINT nFlag, HMENU hMenu);
// Returns: Code informs the system how to process the pressed key (or WMR_DEFAULT)
#define ON_WM_MENUCHAR(fName) \
	case WM_MENUCHAR: \
		lResult = pWnd->fName((TCHAR)LOWORD(wParam), HIWORD(wParam), (HMENU)lParam); \
		if(lResult != WMR_DEFAULT) \
			return lResult; \
		goto xDefProc;

// The user makes a selection from a menu
// void OnMenuCommand(HMENU hMenu, UINT nItemId, UINT nIndex);
#define ON_WM_MENUCOMMAND(fName) \
	case WM_MENUCOMMAND: \
		pWnd->fName((HMENU)lParam, LOWORD(wParam), HIWORD(wParam)); \
		return _eResult;

// The user drags a menu item
// UINT OnMenuDrag(HMENU hMenu, UINT nPos);
// Returns: Menu remains active (MND_CONTINUE) or not (MND_ENDMENU)
#define ON_WM_MENUDRAG(fName) \
	case WM_MENUDRAG: \
		return pWnd->fName((HMENU)lParam, wParam);

// Mouse cursor enters a menu item
// UINT OnMenuGetObject(PMENUGETOBJECTINFO pMGOI);
// Returns: Interface is supported (MNGO_NOERROR) or not (MNGO_NOINTERFACE)
#define ON_WM_MENUGETOBJECT(fName) \
	case WM_MENUGETOBJECT: \
		return pWnd->fName((PMENUGETOBJECTINFO)lParam);

// The user releases the right mouse button while the cursor is on a menu item
// void OnMenuRButtonUp(HMENU hMenu, UINT nPos);
#define ON_WM_MENURBUTTONUP(fName) \
	case WM_MENURBUTTONUP: \
		pWnd->fName((HMENU)lParam, wParam); \
		return _eResult;

// A menu item is selected
// void OnMenuSelect(UINT nItemId, UINT nFlags, HMENU hMenu);
#define ON_WM_MENUSELECT(fName) \
	case WM_MENUSELECT: \
		pWnd->fName(LOWORD(wParam), HIWORD(wParam), (HMENU)lParam); \
		return _eResult;

// Mouse's horizontal wheel is rotated
// void OnMouseHWheel(int iDelta, UINT nFlags, int xCursor, int yCursor);
#define _ON_WM_MOUSEHWHEEL(nMsg, fName) \
	case nMsg: \
		pWnd->fName(HISHORT(wParam), LOWORD(wParam), LOSHORT(lParam), HISHORT(lParam)); \
		return _eResult;
#define ON_WM_MOUSEHWHEEL(fName) _ON_WM_MOUSEHWHEEL(WM_MOUSEHWHEEL, fName)

// Mouse cursor moves in the client area of a window
// void OnMouseMove(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_MOUSEMOVE(fName) _ON_WM_DISPLAYCHANGE(WM_MOUSEMOVE, fName)

// Mouse wheel is rotated
// void OnMouseWheel(int iDelta, UINT nFlags, int xCursor, int yCursor);
#define ON_WM_MOUSEWHEEL(fName) _ON_WM_MOUSEHWHEEL(WM_MOUSEWHEEL, fName)

// Window has been moved
// void OnMove(int xClient, int yClient);
#define ON_WM_MOVE(fName) \
	case WM_MOVE: \
		pWnd->fName(LOSHORT(lParam), HISHORT(lParam)); \
		if(_eWndType != _WT_MDICHILD) \
			return _eResult; \
		goto xDefProc;

// Window is being moving
// void OnMoving(UINT nSide, PRECT pRect);
#define _ON_WM_MOVING(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, (PRECT)lParam); \
		return TRUE;
#define ON_WM_MOVING(fName) _ON_WM_MOVING(WM_MOVING, fName)

// Nonclient area of a window is being changed to indicate an active or inactive state
// BOOL OnNcActivate(BOOL bActive);
// Returns: Allows (TRUE) the default processing or not (FALSE)
#define _ON_WM_NCACTIVATE(nMsg, fName) \
	case nMsg: \
		return pWnd->fName(wParam);
#define ON_WM_NCACTIVATE(fName) _ON_WM_NCACTIVATE(WM_NCACTIVATE, fName)

// Size and position of a window's client area must be calculated
// int OnNcCalcSize(BOOL bCalcRects, LPARAM lParam);
// Returns: Result value of the message processed
#define ON_WM_NCCALCSIZE(fName) \
	case WM_NCCALCSIZE: \
		return pWnd->fName(wParam, lParam);

// Nonclient area of a window is being created
// BOOL OnNcCreate(PCCREATESTRUCT pCS);
// Returns: Continues creation (TRUE) or destroys (FALSE) the window
#define ON_WM_NCCREATE(fName) _ON_WM_CREATE(WM_NCCREATE, fName)

// Nonclient area of a window is being destroyed
// void OnNcDestroy();
#define ON_WM_NCDESTROY(fName) _ON_WM_CANCELMODE(WM_NCDESTROY, fName)

// Mouse cursor moves, a mouse button is pressed or released
// int OnNcHitTest(int xCursor, int yCursor);
// Returns: Position of the cursor hot spot (or HTDEFAULT)
#define HTDEFAULT WMR_DEFAULT
#define ON_WM_NCHITTEST(fName) \
	case WM_NCHITTEST: \
		lResult = pWnd->fName(LOSHORT(lParam), HISHORT(lParam)); \
		if(lResult != HTDEFAULT) \
			return lResult; \
		goto xDefProc;

// Left mouse button has double-clicked in the nonclient area
// void OnNcLButtonDblClk(int iHitTest, int xCursor, int yCursor);
#define _ON_WM_NCLBUTTONDBLCLK(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, LOSHORT(lParam), HISHORT(lParam)); \
		goto xDefProc;
#define ON_WM_NCLBUTTONDBLCLK(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCLBUTTONDBLCLK, fName)

// Left mouse button has pressed in the nonclient area
// void OnNcLButtonDown(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCLBUTTONDOWN(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCLBUTTONDOWN, fName)

// Left mouse button has released in the nonclient area
// void OnNcLButtonUp(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCLBUTTONUP(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCLBUTTONUP, fName)

// Middle mouse button has double-clicked in the nonclient area
// void OnNcMButtonDblClk(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCMBUTTONDBLCLK(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCMBUTTONDBLCLK, fName)

// Middle mouse button has pressed in the nonclient area
// void OnNcMButtonDown(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCMBUTTONDOWN(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCMBUTTONDOWN, fName)

// Middle mouse button has released in the nonclient area
// void OnNcMButtonUp(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCMBUTTONUP(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCMBUTTONUP, fName)

// Mouse cursor moves in the nonclient area of a window
// void OnNcMouseMove(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCMOUSEMOVE(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCMOUSEMOVE, fName)

// Paints the frame of a window
// void OnNcPaint(HRGN hUpdateRgn);
#define ON_WM_NCPAINT(fName) \
	case WM_NCPAINT: \
		pWnd->fName((HRGN)wParam); \
		return _eResult;

// Right mouse button has double-clicked in the nonclient area
// void OnNcRButtonDblClk(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCRBUTTONDBLCLK(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCRBUTTONDBLCLK, fName)

// Right mouse button has pressed in the nonclient area
// void OnNcRButtonDown(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCRBUTTONDOWN(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCRBUTTONDOWN, fName)

// Right mouse button has released in the nonclient area
// void OnNcRButtonUp(int iHitTest, int xCursor, int yCursor);
#define ON_WM_NCRBUTTONUP(fName) _ON_WM_NCLBUTTONDBLCLK(WM_NCRBUTTONUP, fName)

// X mouse button has double-clicked in the nonclient area
// void OnNcXButtonDblClk(int iHitTest, UINT nButton, int xCursor, int yCursor);
#define _ON_WM_NCXBUTTONDBLCLK(nMsg, fName) \
	case nMsg: \
		pWnd->fName(LOSHORT(wParam), HIWORD(wParam), LOSHORT(lParam), HISHORT(lParam)); \
		return TRUE;
#define ON_WM_NCXBUTTONDBLCLK(fName) _ON_WM_NCXBUTTONDBLCLK(WM_NCXBUTTONDBLCLK, fName)

// X mouse button has pressed in the nonclient area
// void OnNcXButtonDown(int iHitTest, UINT nButton, int xCursor, int yCursor);
#define ON_WM_NCXBUTTONDOWN(fName) _ON_WM_NCXBUTTONDBLCLK(WM_NCXBUTTONDOWN, fName)

// X mouse button has released in the nonclient area
// void OnNcXButtonUp(int iHitTest, UINT nButton, int xCursor, int yCursor);
#define ON_WM_NCXBUTTONUP(fName) _ON_WM_NCXBUTTONDBLCLK(WM_NCXBUTTONUP, fName)

// Arrow key is used to switch between the menu bar and the system menu
// void OnNextMenu(TCHAR cVirtKey, PMDINEXTMENU pMDINM);
#define ON_WM_NEXTMENU(fName) \
	case WM_NEXTMENU: \
		pWnd->fName((TCHAR)wParam, (PMDINEXTMENU)lParam); \
		return _eResult;

// An event has occurred in, or requires some kind of information for, a control
// int OnNotify(PCNMHDR pNMH);
// Returns: Result value of the message processed (or WMR_PROCESSED, WMR_DEFAULT)
#define ON_WM_NOTIFY(fName) \
	case WM_NOTIFY: \
		lResult = pWnd->fName((PCNMHDR)lParam); \
		if(lResult == WMR_DEFAULT) \
			goto xDefProc; \
		if(_eWndType != _WT_DIALOG) \
			return lResult; \
		::SetWindowLong(hWnd, DWL_MSGRESULT, lResult); \
		return _eResult;

// Begins notification maps
#define BEGIN_NOTIFY_MAP() \
	case WM_NOTIFY: \
		nCode = ((PCNMHDR)lParam)->code; \
		switch(nCode) \
		{

// Notification map
#define _ON_NOTIFY_V(nCode, fName, pType) \
		case nCode: \
			pWnd->fName((pType)lParam); \
			return _eResult;

// Notification map
#define _ON_NOTIFY_R(nCode, fName, pType) \
		case nCode: \
			lResult = pWnd->fName((pType)lParam); \
			if(_eWndType != _WT_DIALOG) \
				return lResult; \
			goto xDefNty;

// An event has occurred in, or requires some kind of information for, a control
// int OnNotifyRange(PCNMHDR pNMH);
// Returns: Ignored except for notification messages that specify otherwise
#define ON_NOTIFY_RANGE(nCode1, nCode2, fName) \
		} \
		if((nCode1 <= nCode) && (nCode <= nCode2)) \
		{ \
			lResult = pWnd->fName((PCNMHDR)lParam); \
			if(lResult == WMR_DEFAULT) \
				goto xDefProc; \
			if(_eWndType != _WT_DIALOG) \
				return lResult; \
			goto xDefNty; \
		} \
		switch(nCode) \
		{

// Ends notification maps
#define END_NOTIFY_MAP() \
		} \
		goto xDefProc; \
		xDefNty: \
		::SetWindowLong(hWnd, DWL_MSGRESULT, lResult); \
		return _eResult; \
		UNREFERENCED_LABEL(xDefNty);

// Determines whether a control should use ANSI or Unicode structures in WM_NOTIFY messages
// UINT OnNotifyFormat(HWND hCtrlWnd);
// Returns: Value of the query operation
#define ON_WM_NOTIFYFORMAT(fName) _ON_WM_INITDIALOG(WM_NOTIFYFORMAT, fName)

// Paints the client area of a window
// void OnPaint();
#define ON_WM_PAINT(fName) _ON_WM_CLIPBOARDUPDATE(WM_PAINT, fName)

// Paints the client area of a clipboard viewer
// void OnPaintClipboard(HWND hViewerWnd, HMEMORY hPS);
#define _ON_WM_PAINTCLIPBOARD(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HWND)wParam, (HMEMORY)lParam); \
		return _eResult;
#define ON_WM_PAINTCLIPBOARD(fName) _ON_WM_PAINTCLIPBOARD(WM_PAINTCLIPBOARD, fName)

// A window has realized its logical palette
// void OnPaletteChanged(HWND hFocusWnd);
#define _ON_WM_PALETTECHANGED(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HWND)wParam); \
		return _eResult;
#define ON_WM_PALETTECHANGED(fName) _ON_WM_PALETTECHANGED(WM_PALETTECHANGED, fName)

// A window is about to realize its logical palette
// void OnPaletteIsChanging(HWND hRealizeWnd);
#define ON_WM_PALETTEISCHANGING(fName) _ON_WM_PALETTECHANGED(WM_PALETTEISCHANGING, fName)

// Child window is created, destroyed or a mouse button is pressed on it
// void OnParentNotify(UINT nMsg, UINT nChildId, HWND hChildWnd);
#define ON_WM_PARENTNOTIFY(fName) \
	case WM_PARENTNOTIFY: \
		pWnd->fName(LOWORD(wParam), HIWORD(wParam), (HWND)lParam); \
		return _eResult;

// The system is about to enter suspended mode
// int OnPower(UINT nEvent);
// Returns: Accept (PWR_OK) or prevent (PWR_FAIL) the operation
#define ON_WM_POWER(fName) _ON_WM_NCACTIVATE(WM_POWER, fName)

// Minimized window (no icon defined) is about to be dragged by the user
// HICON OnQueryDragIcon();
// Returns: Handle to a cursor or icon (or NULL)
#define ON_WM_QUERYDRAGICON(fName) \
	case WM_QUERYDRAGICON: \
		return (LRESULT)pWnd->fName();

// Requests to end the Windows session
// BOOL OnQueryEndSession(UINT nFlags);
// Returns: Accepts (TRUE) or rejects (FALSE) the request
#define ON_WM_QUERYENDSESSION(fName) _ON_WM_IDLE(WM_QUERYENDSESSION, fName)

// Requests to realize the logical palette
// BOOL OnQueryNewPalette();
// Returns: Realizes (TRUE) the logical palette or not (FALSE)
#define ON_WM_QUERYNEWPALETTE(fName) _ON_WM_GETTEXTLENGTH(WM_QUERYNEWPALETTE, fName)

// Requests a window to be restored to its previous size and position
// BOOL OnQueryOpen();
// Returns: Accepts (TRUE) the window to be restored or not (FALSE)
#define ON_WM_QUERYOPEN(fName) _ON_WM_GETTEXTLENGTH(WM_QUERYOPEN, fName)

// Right mouse button has double-clicked
// void OnRButtonDblClk(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_RBUTTONDBLCLK(fName) _ON_WM_DISPLAYCHANGE(WM_RBUTTONDBLCLK, fName)

// Right mouse button has pressed
// void OnRButtonDown(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_RBUTTONDOWN(fName) _ON_WM_DISPLAYCHANGE(WM_RBUTTONDOWN, fName)

// Right mouse button has released
// void OnRButtonUp(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_RBUTTONUP(fName) _ON_WM_DISPLAYCHANGE(WM_RBUTTONUP, fName)

// Registered message for use by application
#define _ON_WM_REGISTER_V(nRegMsg, fName, pList) \
	} \
	if(nMsg == pWnd->nRegMsg) \
	{ \
		ASSERT((WM_REGISTER <= pWnd->nRegMsg) && (pWnd->nRegMsg <= MAX_WORD)); \
		pWnd->fName pList; \
		return _eResult; \
	} \
	switch(nMsg) \
	{

#define _ON_WM_REGISTER_R(nRegMsg, fName, pList) \
	} \
	if(nMsg == pWnd->nRegMsg) \
	{ \
		ASSERT((WM_REGISTER <= pWnd->nRegMsg) && (pWnd->nRegMsg <= MAX_WORD)); \
		return pWnd->fName pList; \
	} \
	switch(nMsg) \
	{

// Registered message for use by application
// LRESULT OnRegister(WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed
#define ON_WM_REGISTER(nRegMsg, fName) _ON_WM_REGISTER_R(nRegMsg, fName, (wParam, lParam))

// Delayed rendering one or more clipboard formats
// void OnRenderAllFormats();
#define ON_WM_RENDERALLFORMATS(fName) _ON_WM_CLIPBOARDUPDATE(WM_RENDERALLFORMATS, fName)

// Delayed rendering a clipboard format
// void OnRenderFormat(UINT nFormat);
#define ON_WM_RENDERFORMAT(fName) _ON_WM_COMPACTING(WM_RENDERFORMAT, fName)

// Mouse cursor shape should be set
// BOOL OnSetCursor(HWND hCursorWnd, int iHitTest, UINT nMsg);
// Returns: Cursor shape is set (TRUE) or not (FALSE)
#define ON_WM_SETCURSOR(fName) \
	case WM_SETCURSOR: \
		if(pWnd->fName((HWND)wParam, LOSHORT(lParam), HIWORD(lParam))) \
			return TRUE; \
		goto xDefProc;

// Window has gained the keyboard focus
// void OnSetFocus(HWND hOldWnd);
#define ON_WM_SETFOCUS(fName) \
	case WM_SETFOCUS: \
		pWnd->fName((HWND)wParam); \
		if((_eWndType != _WT_MDIFRAME) && (_eWndType != _WT_MDICHILD) && \
			(_eWndType != _WT_SUBCLASS)) \
			return _eResult; \
		goto xDefProc;

// Sets the text of a window
// BOOL OnSetText(PCTSTR pszText);
// Returns: Text is set (TRUE) or insufficient space is available (FALSE)
#define ON_WM_SETTEXT(fName) \
	case WM_SETTEXT: \
		return pWnd->fName((PCTSTR)lParam);

// System setting has changed
// void OnSettingChange(UINT nAction, PCTSTR pszParam);
#define ON_WM_SETTINGCHANGE(fName) \
	case WM_SETTINGCHANGE: \
		pWnd->fName(wParam, (PCTSTR)lParam); \
		return _eResult;

// Window is about to be shown or hidden
// void OnShowWindow(BOOL bShow, UINT nStatus);
#define ON_WM_SHOWWINDOW(fName) _ON_WM_ACTIVATEAPP(WM_SHOWWINDOW, fName)

// Window size has changed
// void OnSize(UINT nType, int cxClient, int cyClient);
#define ON_WM_SIZE(fName) \
	case WM_SIZE: \
		pWnd->fName(wParam, LOSHORT(lParam), HISHORT(lParam)); \
		if((_eWndType != _WT_MDICHILD) && (_eWndType != _WT_SUBCLASS)) \
			return _eResult; \
		goto xDefProc;

// Size of the clipboard viewer has changed
// void OnSizeClipboard(HWND hViewerWnd, HMEMORY hRect);
#define ON_WM_SIZECLIPBOARD(fName) _ON_WM_PAINTCLIPBOARD(WM_SIZECLIPBOARD, fName)

// Window is being resizing
// void OnSizing(UINT nSide, PRECT pRect);
#define ON_WM_SIZING(fName) _ON_WM_MOVING(WM_SIZING, fName)

// Window styles have changed
// void OnStyleChanged(int iType, PCSTYLESTRUCT pSS);
#define ON_WM_STYLECHANGED(fName) \
	case WM_STYLECHANGED: \
		pWnd->fName((int)wParam, (PCSTYLESTRUCT)lParam); \
		return _eResult;

// Window styles are being changing
// void OnStyleChanging(int iType, PSTYLESTRUCT pSS);
#define ON_WM_STYLECHANGING(fName) \
	case WM_STYLECHANGING: \
		pWnd->fName((int)wParam, (PSTYLESTRUCT)lParam); \
		return _eResult;

// A system character is pressed
// void OnSysChar(TCHAR cChar, LPARAM lKeyData);
#define ON_WM_SYSCHAR(fName) _ON_WM_CHAR(WM_SYSCHAR, fName)

// A system color is changed
// void OnSysColorChange();
#define ON_WM_SYSCOLORCHANGE(fName) _ON_WM_CLIPBOARDUPDATE(WM_SYSCOLORCHANGE, fName)

// A command item from the System menu
// void OnSysCommand(UINT nItemId, LPARAM lParam);
#define _SC_MIN SC_SIZE
#define ON_WM_SYSCOMMAND(fName) \
	case WM_SYSCOMMAND: \
		pWnd->fName(wParam, lParam); \
		if(wParam < _SC_MIN) \
			return _eResult; \
		goto xDefProc;

// Begins system-command message maps
#define BEGIN_SYSCOMMAND_MAP() \
	case WM_SYSCOMMAND: \
		nItemId = ((wParam < _SC_MIN) ? wParam : (wParam & 0xFFF0)); \
		switch(nItemId) \
		{

// A command item from the System menu
// void OnSysCmdItem(LPARAM lParam);
#define ON_SYSCOMMAND(nItemId, fName) \
		case nItemId: \
			pWnd->fName(lParam); \
			goto xDefSysCmd;

// A command item in the range from the System menu
// void OnSysCmdItemRange(UINT nItemId, LPARAM lParam);
#define ON_SYSCOMMAND_RANGE(nItemId1, nItemId2, fName) \
		} \
		if((nItemId1 <= nItemId) && (nItemId <= nItemId2)) \
		{ \
			pWnd->fName(nItemId, lParam); \
			goto xDefSysCmd; \
		} \
		switch(nItemId) \
		{

// Ends system-command message maps
#define END_SYSCOMMAND_MAP() \
		} \
		ASSERT(nItemId >= _SC_MIN); \
		xDefSysCmd: \
		if(nItemId < _SC_MIN) \
			return _eResult; \
		goto xDefProc; \
		UNREFERENCED_LABEL(xDefSysCmd);

// A system dead character is pressed
// void OnSysDeadChar(TCHAR cChar, LPARAM lKeyData);
#define ON_WM_SYSDEADCHAR(fName) _ON_WM_CHAR(WM_SYSDEADCHAR, fName)

// A system key is pressed
// void OnSysKeyDown(TCHAR cVirtKey, LPARAM lKeyData);
#define ON_WM_SYSKEYDOWN(fName) _ON_WM_CHAR(WM_SYSKEYDOWN, fName)

// A system key is released
// void OnSysKeyUp(TCHAR cVirtKey, LPARAM lKeyData);
#define ON_WM_SYSKEYUP(fName) _ON_WM_CHAR(WM_SYSKEYUP, fName)

// System time has changed
// void OnTimeChange();
#define ON_WM_TIMECHANGE(fName) _ON_WM_CLIPBOARDUPDATE(WM_TIMECHANGE, fName)

// A timer interval expires
// void OnTimer(UINT nTimerId);
#define ON_WM_TIMER(fName) _ON_WM_COMPACTING(WM_TIMER, fName)

// A pop-up menu or submenu has been inactive
// void OnUninitMenuPopup(HMENU hMenu);
#define ON_WM_UNINITMENUPOPUP(fName) _ON_WM_INITMENU(WM_UNINITMENUPOPUP, fName)

// Integer message for use by private window class
// LRESULT OnUser(WPARAM wParam, LPARAM lParam);
// Returns: Result value of the message processed (or WMR_PROCESSED)
#define ON_WM_USER(nMsg, fName) _ON_WM_APP(nMsg, WM_USER, WM_APP, fName)

// The user has logged on or off
// void OnUserChanged();
#define ON_WM_USERCHANGED(fName) _ON_WM_CLIPBOARDUPDATE(WM_USERCHANGED, fName)

// Owner-drawn list box in response to a WM_KEYDOWN message
// int OnVKeyToItem(TCHAR cVirtKey, int iItem, HWND hListBox);
// Returns: No further action (-2), default action (-1) or default action on an item
#define ON_WM_VKEYTOITEM(fName) _ON_WM_CHARTOITEM(WM_VKEYTOITEM, fName)

// An event occurs in a vertical scroll bar
// void OnVScroll(UINT nCode, int iPos, HWND hScrollBar);
#define ON_WM_VSCROLL(fName) _ON_WM_HSCROLL(WM_VSCROLL, fName)

// An event occurs in a clipboard viewer's vertical scroll bar
// void OnVScrollClipboard(HWND hViewerWnd, UINT nCode, int iPos);
#define ON_WM_VSCROLLCLIPBOARD(fName) _ON_WM_HSCROLLCLIPBOARD(WM_VSCROLLCLIPBOARD, fName)

// Window size, position or place in the Z order has changed
// void OnWindowPosChanged(PCWINDOWPOS pWP);
#define ON_WM_WINDOWPOSCHANGED(fName) \
	case WM_WINDOWPOSCHANGED: \
		pWnd->fName((PCWINDOWPOS)lParam); \
		goto xDefProc;

// Window size, position or place in the Z order is about to change
// void OnWindowPosChanging(PWINDOWPOS pWP);
#define ON_WM_WINDOWPOSCHANGING(fName) \
	case WM_WINDOWPOSCHANGING: \
		pWnd->fName((PWINDOWPOS)lParam); \
		goto xDefProc;

// Initialization file has changed
// void OnWinIniChange(PCTSTR pszSection);
#define ON_WM_WININICHANGE(fName) \
	case WM_WININICHANGE: \
		pWnd->fName((PCTSTR)lParam); \
		return _eResult;

// X mouse button has double-clicked
// void OnXButtonDblClk(UINT nFlags, int xCursor, int yCursor);
#define _ON_WM_XBUTTONDBLCLK(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, LOSHORT(lParam), HISHORT(lParam)); \
		return TRUE;
#define ON_WM_XBUTTONDBLCLK(fName) _ON_WM_XBUTTONDBLCLK(WM_XBUTTONDBLCLK, fName)

// X mouse button has pressed
// void OnXButtonDown(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_XBUTTONDOWN(fName) _ON_WM_XBUTTONDBLCLK(WM_XBUTTONDOWN, fName)

// X mouse button has released
// void OnXButtonUp(UINT nFlags, int xCursor, int yCursor);
#define ON_WM_XBUTTONUP(fName) _ON_WM_XBUTTONDBLCLK(WM_XBUTTONUP, fName)

// End of _MSGMAP_INCLUDED_
#endif
