////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// OleObj.hpp - OLE Declaration File                                                          //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _OLEOBJ_INCLUDED_
#define _OLEOBJ_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  OleObj - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Exports required functions for an in-process server/handler, if necessary

#ifdef  OLEINPROC
#pragma comment(linker, "/EXPORT:DllGetClassObject=_DllGetClassObject@12,PRIVATE")
#pragma comment(linker, "/EXPORT:DllCanUnloadNow=_DllCanUnloadNow@0,PRIVATE")
#endif
#ifdef  OLEINPROCREG
#pragma comment(linker, "/EXPORT:DllRegisterServer=_DllRegisterServer@0,PRIVATE")
#pragma comment(linker, "/EXPORT:DllUnregisterServer=_DllUnregisterServer@0,PRIVATE")
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// General purposes

// Declares a CLSID/IID
#define DECLARE_CLSID(CLSID_Name, dw, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
	EXTVAR const CLSID CLSID_Name = { dw, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8 };
#define DECLARE_IID DECLARE_CLSID

// Declares an interface
#undef  interface
#define interface struct __declspec(novtable)

// Attaches a CLSID/IID to a class/interface
#define clsid(g) __declspec(uuid(g))
#define iid(g)   clsid(g)

// Gets a CLSID/IID attached to an expression
// Returns: CLSID/IID attached to the expression
#define clsidof(e) __uuidof(e)
#define iidof(e)   clsidof(e)

// Gets the severity bit from a resulting value
// Returns: Severity bit of the value
#undef  HRESULT_SEVERITY
#undef  SCODE_SEVERITY
#define HRESULT_SEVERITY(h) (((h) < 0) ? SEVERITY_ERROR : SEVERITY_SUCCESS)
#define SCODE_SEVERITY(s)   HRESULT_SEVERITY(s)

// Generic test for errors on any status value
// Returns: TRUE if the result is error
#undef  IS_ERROR
#define IS_ERROR(h) FAILED(h)

// Maximum length of storage (nonroot) and stream names (32, including a null character)
#define MAX_STGNAME CWCSTORAGENAME
#define MAX_STMNAME CWCSTORAGENAME

// Data type mapping for OLE characters and strings
#define _O(x) OLESTR(x)

////////////////////////////////////////////////////////////////////////////////////////////////
// Character constants and string literals

// Characters
#define OCHR_NULL _O('\0')    // Null (0x00)
#define OCHR_CR   _O('\r')    // Carriage return (0x0D)
#define OCHR_LF   _O('\n')    // Linefeed (0x0A)
#define OCHR_FF   _O('\f')    // Formfeed (0x0C)
#define OCHR_TAB  _O('\t')    // Horizontal tab (0x09)
#define OCHR_VT   _O('\v')    // Vertical tab (0x0B)
#define OCHR_BS   _O('\b')    // Backspace (0x08)
#define OCHR_BEL  _O('\a')    // Bell (0x07)
#define OCHR_SQM  _O('\'')    // Single quotation mark (0x27)
#define OCHR_DQM  _O('"')     // Double quotation mark (0x22)
#define OCHR_SLH  _O('/')     // Slash (0x2F)
#define OCHR_BSLH _O('\\')    // Backslash (0x5C)
#define OCHR_DSH  _O('-')     // Dash (0x2D)
#define OCHR_SP   _O(' ')     // Space (0x20)
#define OCHR_ESC  _O('\x1B')  // Escape (0x1B)

// Strings
#define OSTR_NULL _O("")      // Null
#define OSTR_CR   _O("\r")    // Carriage return
#define OSTR_LF   _O("\n")    // Linefeed
#define OSTR_FF   _O("\f")    // Formfeed
#define OSTR_TAB  _O("\t")    // Horizontal tab
#define OSTR_VT   _O("\v")    // Vertical tab
#define OSTR_BS   _O("\b")    // Backspace
#define OSTR_BEL  _O("\a")    // Bell
#define OSTR_SQM  _O("'")     // Single quotation mark
#define OSTR_DQM  _O("\"")    // Double quotation mark
#define OSTR_SLH  _O("/")     // Slash
#define OSTR_BSLH _O("\\")    // Backslash
#define OSTR_DSH  _O("-")     // Dash
#define OSTR_SP   _O(" ")     // Space
#define OSTR_CRLF _O("\r\n")  // Carriage return/linefeed combination
#define OSTR_NL   OSTR_CRLF   // New line (same as OSTR_CRLF)

////////////////////////////////////////////////////////////////////////////////////////////////
// Debugging information may corrupt (uuid.lib), uses another way to retrieve IIDs

#if     (MSC_VER < 1300)
#ifdef  DEBUG
#define IID_IClassFactory             iidof(IClassFactory)
#define IID_IDataObject               iidof(IDataObject)
#define IID_IDropSource               iidof(IDropSource)
#define IID_IDropTarget               iidof(IDropTarget)
#define IID_IExternalConnection       iidof(IExternalConnection)
#define IID_IMarshal                  iidof(IMarshal)
#define IID_IMessageFilter            iidof(IMessageFilter)
#define IID_IOleClientSite            iidof(IOleClientSite)
#define IID_IOleContainer             iidof(IOleContainer)
#define IID_IOleInPlaceActiveObject   iidof(IOleInPlaceActiveObject)
#define IID_IOleInPlaceFrame          iidof(IOleInPlaceFrame)
#define IID_IOleInPlaceObject         iidof(IOleInPlaceObject)
#define IID_IOleInPlaceSite           iidof(IOleInPlaceSite)
#define IID_IOleInPlaceUIWindow       iidof(IOleInPlaceUIWindow)
#define IID_IOleItemContainer         iidof(IOleItemContainer)
#define IID_IOleObject                iidof(IOleObject)
#define IID_IOleWindow                iidof(IOleWindow)
#define IID_IParseDisplayName         iidof(IParseDisplayName)
#define IID_IPersist                  iidof(IPersist)
#define IID_IPersistFile              iidof(IPersistFile)
#define IID_IPersistStorage           iidof(IPersistStorage)
#define IID_IPersistStream            iidof(IPersistStream)
#define IID_IRootStorage              iidof(IRootStorage)
#define IID_IStorage                  iidof(IStorage)
#define IID_IStream                   iidof(IStream)
#define IID_IUnknown                  iidof(IUnknown)
#ifdef  MSHTML
#define IID_IDocHostUIHandler         iidof(IDocHostUIHandler)
#endif
#ifdef  OLECTRL
#define IID_IClassFactory2            iidof(IClassFactory2)
#define IID_IConnectionPoint          iidof(IConnectionPoint)
#define IID_IConnectionPointContainer iidof(IConnectionPointContainer)
#define IID_IEnumConnectionPoints     iidof(IEnumConnectionPoints)
#define IID_IEnumConnections          iidof(IEnumConnections)
#define IID_IPersistStreamInit        iidof(IPersistStreamInit)
#endif
#ifdef  OLEEXDISP
#define CLSID_InternetExplorer        clsidof(InternetExplorer)
#define CLSID_WebBrowser              clsidof(WebBrowser)
#define IID_IWebBrowser2              iidof(IWebBrowser2)
#endif
#endif
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Synonyms
typedef OLECHAR     OCHAR;
typedef const OCHAR COCHAR;

// Pointers
typedef OCHAR*               POCHAR;
typedef OCHAR*               POSTR;
typedef BSTR*                PBSTR;
typedef CLSID*               PCLSID;
typedef COSERVERINFO*        PCOSERVERINFO;
typedef FORMATETC*           PFORMATETC;
typedef IID*                 PIID;
typedef INTERFACEINFO*       PINTERFACEINFO;
typedef MULTI_QI*            PMULTI_QI;
typedef OLEINPLACEFRAMEINFO* POLEINPLACEFRAMEINFO;
typedef OLEMENUGROUPWIDTHS*  POLEMENUGROUPWIDTHS;
typedef STGMEDIUM*           PSTGMEDIUM;
typedef OCHAR**              PPOSTR;
typedef const OCHAR*         PCOCHAR;
typedef const OCHAR*         PCOSTR;
typedef const BORDERWIDTHS*  PCBORDERWIDTHS;
typedef const CLSID*         PCCLSID;
typedef const GUID*          PCGUID;
typedef const IID*           PCIID;

#ifdef  MSHTML
typedef DOCHOSTUIINFO* PDOCHOSTUIINFO;
#endif

#ifdef  OLECTRL
typedef LICINFO* PLICINFO;
#endif

// Pointer to DllRegisterServer or DllUnregisterServer
typedef HRESULT (WINAPI* POLESELFREG)();

// Static
#define SDISPPARAMS static DISPPARAMS
#define SPOSTR      static OCHAR*
#define SCOCHAR     static const OCHAR

////////////////////////////////////////////////////////////////////////////////////////////////
// Controlling IUnknown, used for an aggregate, multiple inheritance object

interface ICtrlUnknown
{
	// Same behavior as IUnknown methods
	STDMETHOD(CtrlQueryInterface)(REFIID rIId, PPVOID ppvItf) PURE;
	STDMETHOD_(ULONG, CtrlAddRef)() PURE;
	STDMETHOD_(ULONG, CtrlRelease)() PURE;

	template<class tItfName>
	HRESULT CtrlQueryInterface(tItfName** tppItfName)
	{
		return CtrlQueryInterface(iidof(tItfName), (PPVOID)tppItfName);
	}

	// Gets a pointer to the interface
	// Returns: A pointer of type IUnknown*
	IUnknown* GetCtrlUnknown()
	{
		return (IUnknown*)this;
	}
};

////////////////////////////////////////////////////////////////////////////////////////////////
// General macros and functions

// Gets a value that is unique to the current thread
// Returns: Value identifying the current thread
AFXINLINE DWORD WINAPI CoGetCurrentThread()
{
	return CoGetCurrentProcess();
}

// Gets a pointer to the default task memory allocator
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoGetMalloc(IMalloc** ppMA)
{
	return VERIFY_HRESULT(CoGetMalloc(MEMCTX_TASK, ppMA));
}

// Sets a 64-bit integer value from a 32-bit integer value
#undef LISet32
#undef ULISet32
AFXINLINE void WINAPI LISet32(LARGE_INTEGER liValue, LONG lValue)
{
	liValue.QuadPart = lValue;
}

AFXINLINE void WINAPI ULISet32(ULARGE_INTEGER uliValue, ULONG ulValue)
{
	uliValue.QuadPart = ulValue;
}

// Converts a multi-byte string into/from an OLE string
// Returns: Number of wide/multi-byte characters writted into the buffer
#define MBStoOCS MBStoWCS
#define OCStoMBS WCStoMBS

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a CLSID/IID

// Creates a CLSID/IID
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoCreateCLSID(PCLSID pClsId)
{
	return VERIFY_HRESULT(CoCreateGuid(pClsId));
}
#define CoCreateIID CoCreateCLSID

// Converts a CLSID/IID into a string of printable characters
// Returns: Length of the string
AFXINLINE int WINAPI StringFromCLSID(REFCLSID rClsId, POSTR pszBuffer, int ccBuffer)
{
	return VERIFY(StringFromGUID2(rClsId, pszBuffer, ccBuffer));
}

AFXINLINE int WINAPI StringFromCLSID(REFCLSID rClsId, PSTR pszBuffer, int cbBuffer)
{
	OCHAR szBuffer[64];
	return VERIFY(WideCharToMultiByte(CP_ACP, UNUSED, szBuffer,
		StringFromGUID2(rClsId, szBuffer, countof(szBuffer)), pszBuffer, cbBuffer, NULL, NULL));
}
#define StringFromIID StringFromCLSID

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on DLLs

AFXINLINE void WINAPI CoFreeUnusedLibraries(DWORD dwDelay)
{
	CoFreeUnusedLibrariesEx(dwDelay, UNUSED);
}

// Initializes the COM library
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoInitialize(COINIT coinit)
{
	return VERIFY_HRESULT(CoInitializeEx(NULL, coinit));
}

AFXINLINE HRESULT WINAPI CoInitialize()
{
	return VERIFY_HRESULT(CoInitialize(NULL));
}

AFXINLINE HINSTANCE WINAPI CoLoadLibrary(POSTR pszFileName)
{
	return (HINSTANCE)VERIFY(CoLoadLibrary(pszFileName, UNUSED));
}

// Initializes the OLE library
// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI OleInitialize()
{
	return VERIFY_HRESULT(OleInitialize(NULL));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on marshaling

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoGetMarshalSizeMax(PULONG pcbSize, REFIID rIId, IUnknown* pUnk,
	MSHCTX mshctx, MSHLFLAGS mshlflags)
{
	return VERIFY_HRESULT(CoGetMarshalSizeMax(pcbSize, rIId, pUnk, mshctx, NULL, mshlflags));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoGetStandardMarshal(REFIID rIId, IUnknown* pUnk, MSHCTX mshctx,
	MSHLFLAGS mshlflags, IMarshal** ppMsh)
{
	return VERIFY_HRESULT(CoGetStandardMarshal(rIId, pUnk, mshctx, NULL, mshlflags, ppMsh));
}

AFXINLINE HRESULT WINAPI CoGetStandardMarshal(REFIID rIId, MSHCTX mshctx,
	MSHLFLAGS mshlflags, IMarshal** ppMsh)
{
	return VERIFY_HRESULT(CoGetStandardMarshal(rIId, NULL, mshctx, NULL, mshlflags, ppMsh));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoMarshalInterface(IStream* pStm, REFIID rIId, IUnknown* pUnk,
	MSHCTX mshctx, MSHLFLAGS mshlflags)
{
	return VERIFY_HRESULT(CoMarshalInterface(pStm, rIId, pUnk, mshctx, NULL, mshlflags));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a message filter

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoRegisterMessageFilter(IMessageFilter* pNewMF)
{
	return VERIFY_HRESULT(CoRegisterMessageFilter(pNewMF, NULL));
}

AFXINLINE HRESULT WINAPI CoRevokeMessageFilter(IMessageFilter** ppPrevMF = NULL)
{
	return VERIFY_HRESULT(CoRegisterMessageFilter(NULL, ppPrevMF));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a moniker

// Returns: A generic resulting value
template<class tItfName>
AFXINLINE HRESULT WINAPI BindMoniker(IMoniker* pMnk, tItfName** tppItfName)
{
	return VERIFY_HRESULT(BindMoniker(pMnk, UNUSED, iidof(tItfName), (PPVOID)tppItfName));
}

AFXINLINE HRESULT WINAPI BindMoniker(IMoniker* pMnk, REFIID rIId, PPVOID ppvItf)
{
	return VERIFY_HRESULT(BindMoniker(pMnk, UNUSED, rIId, ppvItf));
}

AFXINLINE HRESULT WINAPI CreateBindCtx(IBindCtx** ppBC)
{
	return VERIFY_HRESULT(CreateBindCtx(UNUSED, ppBC));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI GetRunningObjectTable(IRunningObjectTable** ppROT)
{
	return VERIFY_HRESULT(GetRunningObjectTable(UNUSED, ppROT));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI MonikerRelativePathTo(IMoniker* pSrcMnk, IMoniker* pDestMnk,
	IMoniker** ppRetMnk)
{
	return VERIFY_HRESULT(MonikerRelativePathTo(pSrcMnk, pDestMnk, ppRetMnk, TRUE));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on an object

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, IUnknown* pOuterUnk, CLSCTX clsctx,
	DWORD ceMQI, PMULTI_QI paMQI)
{
	return VERIFY_HRESULT(CoCreateInstanceEx(rClsId, pOuterUnk, clsctx, NULL, ceMQI, paMQI));
}

AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, CLSCTX clsctx, PCOSERVERINFO pCOSI,
	DWORD ceMQI, PMULTI_QI paMQI)
{
	return VERIFY_HRESULT(CoCreateInstanceEx(rClsId, NULL, clsctx, pCOSI, ceMQI, paMQI));
}

AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, CLSCTX clsctx,
	DWORD ceMQI, PMULTI_QI paMQI)
{
	return VERIFY_HRESULT(CoCreateInstanceEx(rClsId, NULL, clsctx, NULL, ceMQI, paMQI));
}

template<class tItfName>
AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, IUnknown* pOuterUnk, CLSCTX clsctx,
	tItfName** tppItfName)
{
	return VERIFY_HRESULT(CoCreateInstance(rClsId, pOuterUnk, clsctx,
		iidof(tItfName), (PPVOID)tppItfName));
}

template<class tItfName>
AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, CLSCTX clsctx, tItfName** tppItfName)
{
	return VERIFY_HRESULT(CoCreateInstance(rClsId, NULL, clsctx,
		iidof(tItfName), (PPVOID)tppItfName));
}

AFXINLINE HRESULT WINAPI CoCreateInstance(REFCLSID rClsId, CLSCTX clsctx,
	REFIID rIId, PPVOID ppvItf)
{
	return VERIFY_HRESULT(CoCreateInstance(rClsId, NULL, clsctx, rIId, ppvItf));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoDisconnectObject(IUnknown* pUnk)
{
	return VERIFY_HRESULT(CoDisconnectObject(pUnk, UNUSED));
}

// Returns: A generic resulting value
template<class tItfName>
AFXINLINE HRESULT WINAPI CoGetClassObject(REFCLSID rClsId, CLSCTX clsctx, PCOSERVERINFO pCOSI,
	tItfName** tppItfName)
{
	return VERIFY_HRESULT(CoGetClassObject(rClsId, clsctx, pCOSI,
		iidof(tItfName), (PPVOID)tppItfName));
}

template<class tItfName>
AFXINLINE HRESULT WINAPI CoGetClassObject(REFCLSID rClsId, CLSCTX clsctx, tItfName** tppItfName)
{
	return VERIFY_HRESULT(CoGetClassObject(rClsId, clsctx, NULL,
		iidof(tItfName), (PPVOID)tppItfName));
}

AFXINLINE HRESULT WINAPI CoGetClassObject(REFCLSID rClsId, CLSCTX clsctx,
	REFIID rIId, PPVOID ppvItf)
{
	return VERIFY_HRESULT(CoGetClassObject(rClsId, clsctx, NULL, rIId, ppvItf));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CoLockObjectExternal(IUnknown* pUnk)
{
	return VERIFY_HRESULT(CoLockObjectExternal(pUnk, TRUE, UNUSED));
}

AFXINLINE HRESULT WINAPI CoUnlockObjectExternal(IUnknown* pUnk, BOOL bReleaseAll = FALSE)
{
	return VERIFY_HRESULT(CoLockObjectExternal(pUnk, FALSE, bReleaseAll));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a storage

// Returns: A generic resulting value
template<class tItfName>
AFXINLINE HRESULT WINAPI StgCreateStorage(PCOSTR pszFileName, DWORD stgm, DWORD stgfmt,
	tItfName** tppItfName)
{
	return VERIFY_HRESULT(StgCreateStorageEx(pszFileName, stgm, stgfmt, UNUSED, NULL, NULL,
		iidof(tItfName), (PPVOID)tppItfName));
}

template<class tItfName>
AFXINLINE HRESULT WINAPI StgCreateStorage(DWORD stgm, DWORD stgfmt, tItfName** tppItfName)
{
	return VERIFY_HRESULT(StgCreateStorageEx(NULL, stgm, stgfmt, UNUSED, NULL, NULL,
		iidof(tItfName), (PPVOID)tppItfName));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI StgCreateDocfile(PCOSTR pszFileName, DWORD stgm, IStorage** ppStg)
{
	return VERIFY_HRESULT(StgCreateDocfile(pszFileName, stgm, UNUSED, ppStg));
}

AFXINLINE HRESULT WINAPI StgCreateDocfile(DWORD stgm, IStorage** ppStg)
{
	return VERIFY_HRESULT(StgCreateDocfile(NULL, stgm, UNUSED, ppStg));
}

// Returns: A generic resulting value
template<class tItfName>
AFXINLINE HRESULT WINAPI StgOpenStorage(PCOSTR pszFileName, DWORD stgm, DWORD stgfmt,
	tItfName** tppItfName)
{
	return VERIFY_HRESULT(StgOpenStorageEx(pszFileName, stgm, stgfmt, UNUSED, NULL, NULL,
		iidof(tItfName), (PPVOID)tppItfName));
}

AFXINLINE HRESULT WINAPI StgOpenStorage(PCOSTR pszFileName, DWORD stgm, SNB snb,
	IStorage** ppStg)
{
	return VERIFY_HRESULT(StgOpenStorage(pszFileName, NULL, stgm, snb, UNUSED, ppStg));
}

AFXINLINE HRESULT WINAPI StgOpenStorage(PCOSTR pszFileName, DWORD stgm, IStorage** ppStg)
{
	return VERIFY_HRESULT(StgOpenStorage(pszFileName, NULL, stgm, NULL, UNUSED, ppStg));
}

AFXINLINE HRESULT WINAPI StgOpenStorage(IStorage* pStg, DWORD stgm, SNB snb, IStorage** ppStg)
{
	return VERIFY_HRESULT(StgOpenStorage(NULL, pStg, stgm, snb, UNUSED, ppStg));
}

AFXINLINE HRESULT WINAPI StgOpenStorage(IStorage* pStg, DWORD stgm, IStorage** ppStg)
{
	return VERIFY_HRESULT(StgOpenStorage(NULL, pStg, stgm, NULL, UNUSED, ppStg));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a string

AFXINLINE POSTR WINAPI ocscat(POSTR pszDest, PCOSTR pszSrc)
{
	return (POSTR)VERIFY(lstrcatW(pszDest, pszSrc));
}

AFXINLINE PCOCHAR WINAPI ocschr(PCOSTR pszText, OCHAR cChar)
{
	return wcschr(pszText, cChar);
}

AFXINLINE POCHAR WINAPI ocschr(POSTR pszText, OCHAR cChar)
{
	return wcschr(pszText, cChar);
}

AFXINLINE int WINAPI ocscmp(PCOSTR pszText1, PCOSTR pszText2)
{
	return lstrcmpW(pszText1, pszText2);
}

AFXINLINE int WINAPI ocscmpi(PCOSTR pszText1, PCOSTR pszText2)
{
	return lstrcmpiW(pszText1, pszText2);
}

AFXINLINE POSTR WINAPI ocscpy(POSTR pszDest, PCOSTR pszSrc)
{
	return (POSTR)VERIFY(lstrcpyW(pszDest, pszSrc));
}

AFXINLINE POSTR WINAPI ocscpyn(POSTR pszDest, PCOSTR pszSrc, int ccMax)
{
	return (POSTR)VERIFY(lstrcpynW(pszDest, pszSrc, ccMax));
}

AFXINLINE int WINAPI ocslen(PCOSTR pszText)
{
	return lstrlenW(pszText);
}

AFXINLINE PCOCHAR WINAPI ocsrchr(PCOSTR pszText, OCHAR cChar)
{
	return wcsrchr(pszText, cChar);
}

AFXINLINE POCHAR WINAPI ocsrchr(POSTR pszText, OCHAR cChar)
{
	return wcsrchr(pszText, cChar);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Manipulates on a type library

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI CreateTypeLib(PCOSTR pszFileName, ICreateTypeLib** ppCTL)
{
	return VERIFY_HRESULT(CreateTypeLib(SYS_WIN32, pszFileName, ppCTL));
}

AFXINLINE HRESULT WINAPI CreateTypeLib(PCOSTR pszFileName, ICreateTypeLib2** ppCTL2)
{
	return VERIFY_HRESULT(CreateTypeLib2(SYS_WIN32, pszFileName, ppCTL2));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI LoadTypeLib(PCOSTR pszFileName, REGKIND regkind, ITypeLib** ppTL)
{
	return VERIFY_HRESULT(LoadTypeLibEx(pszFileName, regkind, ppTL));
}

// Returns: A generic resulting value
AFXINLINE HRESULT WINAPI RegisterTypeLib(ITypeLib* pTL, POSTR pszFileName)
{
	return VERIFY_HRESULT(RegisterTypeLib(pTL, pszFileName, NULL));
}

// End of _OLEOBJ_INCLUDED_
#endif
