////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// PreCtrl.hpp - Predefined Classes Declaration File                                          //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _PRECTRL_INCLUDED_
#define _PRECTRL_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  PreCtrl - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Control class names

#ifndef COMMCTRL
#define WC_BUTTON    _T("Button")
#define WC_COMBOBOX  _T("ComboBox")
#define WC_EDIT      _T("Edit")
#define WC_LISTBOX   _T("ListBox")
#define WC_SCROLLBAR _T("ScrollBar")
#define WC_STATIC    _T("Static")
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

#if     (WINVER >= 0x0600)
typedef const BUTTON_SPLITINFO* PCBUTTON_SPLITINFO;
typedef const NMBCDROPDOWN*     PCNMBCDROPDOWN;
#endif

#ifdef  COMMCTRL
typedef const BUTTON_IMAGELIST* PCBUTTON_IMAGELIST;
typedef const EDITBALLOONTIP*   PCEDITBALLOONTIP;
typedef const NMBCHOTITEM*      PCNMBCHOTITEM;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// CButton

// The user clicks a button
// void OnBnClicked(HWND hButton);
#define ON_BN_CLICKED(fName) _ON_CONTROL_NOTIFY(BN_CLICKED, fName)

// The user double-clicks a button
// void OnBnDblClk(HWND hButton);
#define ON_BN_DBLCLK(fName) _ON_CONTROL_NOTIFY(BN_DBLCLK, fName)

// Button is disabled
// void OnBnDisable(HWND hButton);
#define ON_BN_DISABLE(fName) _ON_CONTROL_NOTIFY(BN_DISABLE, fName)

// Button loses the keyboard focus
// void OnBnKillFocus(HWND hButton);
#define ON_BN_KILLFOCUS(fName) _ON_CONTROL_NOTIFY(BN_KILLFOCUS, fName)

// Button should be painted
// void OnBnPaint(HWND hButton);
#define ON_BN_PAINT(fName) _ON_CONTROL_NOTIFY(BN_PAINT, fName)

// Button is set to pushed
// void OnBnPushed(HWND hButton);
#define ON_BN_PUSHED(fName) _ON_CONTROL_NOTIFY(BN_PUSHED, fName)

// Button receives the keyboard focus
// void OnBnSetFocus(HWND hButton);
#define ON_BN_SETFOCUS(fName) _ON_CONTROL_NOTIFY(BN_SETFOCUS, fName)

// Button is set to unpushed
// void OnBnUnpushed(HWND hButton);
#define ON_BN_UNPUSHED(fName) _ON_CONTROL_NOTIFY(BN_UNPUSHED, fName)

#if (WINVER >= 0x0600)
// The user clicks a drop down arrow on a button
// void OnBcnDropDown(PCNMBCDROPDOWN pNMBCDD);
#define ON_BCN_DROPDOWN(fName) _ON_NOTIFY_V(BCN_DROPDOWN, fName, PCNMBCDROPDOWN)
#endif

#ifdef COMMCTRL
// Mouse cursor is entering or leaving the client area of a button
// void OnBcnHotItemChange(PCNMBCHOTITEM pNMBCHI);
#define ON_BCN_HOTITEMCHANGE(fName) _ON_NOTIFY_V(BCN_HOTITEMCHANGE, fName, PCNMBCHOTITEM)
#endif

class CButton : public CWnd
{
public:
	CButton();
	CButton(HWND hButton);
	void operator =(HWND hButton);
	void Click() const;
	BOOL CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nButtonId);
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nButtonId);
	HBITMAP GetBitmap() const;
	UINT GetCheck() const;
	HICON GetIcon() const;
	UINT GetState() const;
	void Highlight() const;
	HBITMAP SetBitmap(HBITMAP hBitmap = NULL) const;
	void SetCheck(UINT nState = BST_CHECKED) const;
	HICON SetIcon(HICON hIcon = NULL) const;
	void SetStyle(DWORD dwStyle, BOOL bRedraw = TRUE) const;
	void Unhighlight() const;

#if (WINVER >= 0x0600)
	BOOL GetNote(PTSTR pszBuffer, int ccBuffer) const;
	int GetNoteLength() const;
	BOOL GetSplitInfo(PBUTTON_SPLITINFO pBSI) const;
	void SetDontClick(BOOL bState) const;
	BOOL SetDropDownState(BOOL bDropDown = TRUE) const;
	BOOL SetNote(PCTSTR pszText = NULL) const;
	BOOL SetShield(BOOL bRequired = TRUE) const;
	BOOL SetSplitInfo(PCBUTTON_SPLITINFO pBSI) const;
#endif

#ifdef COMMCTRL
	BOOL GetIdealSize(PSIZE pSize) const;
	BOOL GetImageList(PBUTTON_IMAGELIST pBIL) const;
	BOOL GetTextMargin(PRECT pRect) const;
	BOOL SetImageList(PCBUTTON_IMAGELIST pBIL) const;
	BOOL SetTextMargin(PCRECT pRect) const;
#endif
};

AFXINLINE CButton::CButton()
{
	// Does nothing
}

AFXINLINE CButton::CButton(HWND hButton)
{
	m_hWnd = hButton;
}

AFXINLINE void CButton::operator =(HWND hButton)
{
	m_hWnd = hButton;
}

AFXINLINE void CButton::Click() const
{
	::SendMessage(m_hWnd, BM_CLICK, UNUSED, UNUSED);
}

AFXINLINE BOOL CButton::CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y,
	int iWidth, int iHeight, HWND hParentWnd, UINT nButtonId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_BUTTON, pszText, dwStyle, x, y,
		iWidth, iHeight, hParentWnd, (HMENU)nButtonId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE BOOL CButton::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nButtonId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_BUTTON, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nButtonId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE HBITMAP CButton::GetBitmap() const
{
	return (HBITMAP)::SendMessage(m_hWnd, BM_GETIMAGE, IMAGE_BITMAP, UNUSED);
}

AFXINLINE UINT CButton::GetCheck() const
{
	return ::SendMessage(m_hWnd, BM_GETCHECK, UNUSED, UNUSED);
}

AFXINLINE HICON CButton::GetIcon() const
{
	return (HICON)::SendMessage(m_hWnd, BM_GETIMAGE, IMAGE_ICON, UNUSED);
}

AFXINLINE UINT CButton::GetState() const
{
	return ::SendMessage(m_hWnd, BM_GETSTATE, UNUSED, UNUSED);
}

AFXINLINE void CButton::Highlight() const
{
	::SendMessage(m_hWnd, BM_SETSTATE, TRUE, UNUSED);
}

AFXINLINE HBITMAP CButton::SetBitmap(HBITMAP hBitmap /* = NULL */) const
{
	return (HBITMAP)::SendMessage(m_hWnd, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)hBitmap);
}

AFXINLINE void CButton::SetCheck(UINT nState /* = BST_CHECKED */) const
{
	::SendMessage(m_hWnd, BM_SETCHECK, nState, UNUSED);
}

AFXINLINE HICON CButton::SetIcon(HICON hIcon /* = NULL */) const
{
	return (HICON)::SendMessage(m_hWnd, BM_SETIMAGE, IMAGE_ICON, (LPARAM)hIcon);
}

AFXINLINE void CButton::SetStyle(DWORD dwStyle, BOOL bRedraw /* = TRUE */) const
{
	::SendMessage(m_hWnd, BM_SETSTYLE, dwStyle, bRedraw);
}

AFXINLINE void CButton::Unhighlight() const
{
	::SendMessage(m_hWnd, BM_SETSTATE, FALSE, UNUSED);
}

#if (WINVER >= 0x0600)
AFXINLINE BOOL CButton::GetNote(PTSTR pszBuffer, int ccBuffer) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_GETNOTE, ccBuffer, (LPARAM)pszBuffer));
}

AFXINLINE int CButton::GetNoteLength() const
{
	return ::SendMessage(m_hWnd, BCM_GETNOTELENGTH, UNUSED, UNUSED);
}

AFXINLINE BOOL CButton::GetSplitInfo(PBUTTON_SPLITINFO pBSI) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_GETSPLITINFO, UNUSED, (LPARAM)pBSI));
}

AFXINLINE void CButton::SetDontClick(BOOL bState) const
{
	::SendMessage(m_hWnd, BM_SETDONTCLICK, bState, UNUSED);
}

AFXINLINE BOOL CButton::SetDropDownState(BOOL bDropDown /* = TRUE */) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETDROPDOWNSTATE, bDropDown, UNUSED));
}

AFXINLINE BOOL CButton::SetNote(PCTSTR pszText /* = NULL */) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETNOTE, UNUSED, (LPARAM)pszText));
}

AFXINLINE BOOL CButton::SetShield(BOOL bRequired /* = TRUE */) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETSHIELD, UNUSED, bRequired));
}

AFXINLINE BOOL CButton::SetSplitInfo(PCBUTTON_SPLITINFO pBSI) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETSPLITINFO, UNUSED, (LPARAM)pBSI));
}
#endif

#ifdef COMMCTRL
AFXINLINE BOOL CButton::GetIdealSize(PSIZE pSize) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_GETIDEALSIZE, UNUSED, (LPARAM)pSize));
}

AFXINLINE BOOL CButton::GetImageList(PBUTTON_IMAGELIST pBIL) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_GETIMAGELIST, UNUSED, (LPARAM)pBIL));
}

AFXINLINE BOOL CButton::GetTextMargin(PRECT pRect) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_GETTEXTMARGIN, UNUSED, (LPARAM)pRect));
}

AFXINLINE BOOL CButton::SetImageList(PCBUTTON_IMAGELIST pBIL) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETIMAGELIST, UNUSED, (LPARAM)pBIL));
}

AFXINLINE BOOL CButton::SetTextMargin(PCRECT pRect) const
{
	return VERIFY(::SendMessage(m_hWnd, BCM_SETTEXTMARGIN, UNUSED, (LPARAM)pRect));
}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// CComboBox

// List box of a combo box has been closed
// void OnCbnCloseUp(HWND hComboBox);
#define ON_CBN_CLOSEUP(fName) _ON_CONTROL_NOTIFY(CBN_CLOSEUP, fName)

// The user double-clicks a string in the list box of a combo box
// void OnCbnDblClk(HWND hComboBox);
#define ON_CBN_DBLCLK(fName) _ON_CONTROL_NOTIFY(CBN_DBLCLK, fName)

// List box of a combo box is about to be made visible
// void OnCbnDropDown(HWND hComboBox);
#define ON_CBN_DROPDOWN(fName) _ON_CONTROL_NOTIFY(CBN_DROPDOWN, fName)

// The user has taken an action in the edit of a combo box
// void OnCbnEditChange(HWND hComboBox);
#define ON_CBN_EDITCHANGE(fName) _ON_CONTROL_NOTIFY(CBN_EDITCHANGE, fName)

// Edit portion of a combo box is about to redraw itself
// void OnCbnEditUpdate(HWND hComboBox);
#define ON_CBN_EDITUPDATE(fName) _ON_CONTROL_NOTIFY(CBN_EDITUPDATE, fName)

// A combo box cannot allocate enough memory to meet a specific request
// void OnCbnErrSpace(HWND hComboBox);
#define ON_CBN_ERRSPACE(fName) _ON_CONTROL_NOTIFY(CBN_ERRSPACE, fName)

// A combo box loses the keyboard focus
// void OnCbnKillFocus(HWND hComboBox);
#define ON_CBN_KILLFOCUS(fName) _ON_CONTROL_NOTIFY(CBN_KILLFOCUS, fName)

// Selection in the list box of a combo box has changed
// void OnCbnSelChange(HWND hComboBox);
#define ON_CBN_SELCHANGE(fName) _ON_CONTROL_NOTIFY(CBN_SELCHANGE, fName)

// The user cancels a selected item in a combo box
// void OnCbnSelEndCancel(HWND hComboBox);
#define ON_CBN_SELENDCANCEL(fName) _ON_CONTROL_NOTIFY(CBN_SELENDCANCEL, fName)

// The user selects an item in a combo box
// void OnCbnSelEndOK(HWND hComboBox);
#define ON_CBN_SELENDOK(fName) _ON_CONTROL_NOTIFY(CBN_SELENDOK, fName)

// A combo box receives the keyboard focus
// void OnCbnSetFocus(HWND hComboBox);
#define ON_CBN_SETFOCUS(fName) _ON_CONTROL_NOTIFY(CBN_SETFOCUS, fName)

class CComboBox : public CWnd
{
public:
	CComboBox();
	CComboBox(HWND hComboBox);
	void operator =(HWND hComboBox);
	int AddItem(PCTSTR pszText) const;
	int AddItem(DWORD dwData) const;
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nComboBoxId);
	int DeleteItem(int iItem) const;
	int Dir(PCTSTR pszPathSpec, UINT nAttrs) const;
	int FindItem(PCTSTR pszText, int iStart = -1) const;
	int FindItem(DWORD dwData, int iStart = -1) const;
	int FindItemExact(PCTSTR pszText, int iStart = -1) const;
	int GetCount() const;
	int GetCurSel() const;
	BOOL GetDroppedRect(PRECT pRect) const;
	BOOL GetDroppedState() const;
	int GetDroppedWidth() const;
	DWORD GetEditSel(PINT piStart = NULL, PINT piEnd = NULL) const;
	BOOL GetExtendedUi() const;
	int GetHExtent() const;
	BOOL GetInfo(PCOMBOBOXINFO pCBI) const;
	int GetItem(int iItem, PTSTR pszBuffer) const;
	DWORD GetItem(int iItem) const;
	int GetItemHeight(int iItem = 0) const;
	int GetItemLen(int iItem) const;
	LCID GetLocale() const;
	int GetTopIndex() const;
	void HideDropDown() const;
	int InitStorage(int ceItem, UINT cbSize) const;
	int InsertItem(PCTSTR pszText, int iItem = -1) const;
	int InsertItem(DWORD dwData, int iItem = -1) const;
	void LimitText(UINT cbSize) const;
	void ResetContent() const;
	int SelectItem(PCTSTR pszText, int iStart = -1) const;
	int SelectItem(DWORD dwData, int iStart = -1) const;
	int SetCurSel(int iItem = -1) const;
	int SetDroppedWidth(int iWidth) const;
	int SetEditHeight(int iHeight) const;
	int SetEditSel(int iStart = -1, int iEnd = -1) const;
	int SetExtendedUi(BOOL bExtend = TRUE) const;
	void SetHExtent(int iWidth) const;
	int SetItem(int iItem, DWORD dwData) const;
	int SetItemHeight(int iItem, int iHeight) const;
	int SetItemHeight(int iHeight) const;
	LCID SetLocale(LCID lcName) const;
	int SetTopIndex(int iItem) const;
	void ShowDropDown() const;

#ifdef COMMCTRL
	BOOL GetCueBanner(PWSTR pszBuffer, int cwBuffer) const;
	int GetMinVisible() const;
	BOOL SetCueBanner(PCWSTR pszText) const;
	BOOL SetMinVisible(int ceItem) const;
#endif
};

AFXINLINE CComboBox::CComboBox()
{
	// Does nothing
}

AFXINLINE CComboBox::CComboBox(HWND hComboBox)
{
	m_hWnd = hComboBox;
}

AFXINLINE void CComboBox::operator =(HWND hComboBox)
{
	m_hWnd = hComboBox;
}

AFXINLINE int CComboBox::AddItem(PCTSTR pszText) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, CB_ADDSTRING, UNUSED, (LPARAM)pszText),
		CB_ERR), CB_ERRSPACE);
}

AFXINLINE int CComboBox::AddItem(DWORD dwData) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, CB_ADDSTRING, UNUSED, dwData),
		CB_ERR), CB_ERRSPACE);
}

AFXINLINE BOOL CComboBox::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nComboBoxId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_COMBOBOX, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nComboBoxId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE int CComboBox::DeleteItem(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_DELETESTRING, iItem, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::Dir(PCTSTR pszPathSpec, UINT nAttrs) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, CB_DIR, nAttrs, (LPARAM)pszPathSpec),
		CB_ERR), CB_ERRSPACE);
}

AFXINLINE int CComboBox::FindItem(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, CB_FINDSTRING, iStart, (LPARAM)pszText);
}

AFXINLINE int CComboBox::FindItem(DWORD dwData, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, CB_FINDSTRING, iStart, dwData);
}

AFXINLINE int CComboBox::FindItemExact(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, CB_FINDSTRINGEXACT, iStart, (LPARAM)pszText);
}

AFXINLINE int CComboBox::GetCount() const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETCOUNT, UNUSED, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::GetCurSel() const
{
	return ::SendMessage(m_hWnd, CB_GETCURSEL, UNUSED, UNUSED);
}

AFXINLINE BOOL CComboBox::GetDroppedRect(PRECT pRect) const
{
	return VERIFY(::SendMessage(m_hWnd, CB_GETDROPPEDCONTROLRECT, UNUSED, (LPARAM)pRect));
}

AFXINLINE BOOL CComboBox::GetDroppedState() const
{
	return ::SendMessage(m_hWnd, CB_GETDROPPEDSTATE, UNUSED, UNUSED);
}

AFXINLINE int CComboBox::GetDroppedWidth() const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETDROPPEDWIDTH, UNUSED, UNUSED), CB_ERR);
}

AFXINLINE DWORD CComboBox::GetEditSel(PINT piStart /* = NULL */, PINT piEnd /* = NULL */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETEDITSEL, (WPARAM)piStart, (LPARAM)piEnd),
		CB_ERR);
}

AFXINLINE BOOL CComboBox::GetExtendedUi() const
{
	return ::SendMessage(m_hWnd, CB_GETEXTENDEDUI, UNUSED, UNUSED);
}

AFXINLINE int CComboBox::GetHExtent() const
{
	return ::SendMessage(m_hWnd, CB_GETHORIZONTALEXTENT, UNUSED, UNUSED);
}

AFXINLINE BOOL CComboBox::GetInfo(PCOMBOBOXINFO pCBI) const
{
	return VERIFY(::GetComboBoxInfo(m_hWnd, pCBI));
}

AFXINLINE int CComboBox::GetItem(int iItem, PTSTR pszBuffer) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETLBTEXT, iItem, (LPARAM)pszBuffer), CB_ERR);
}

AFXINLINE DWORD CComboBox::GetItem(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETITEMDATA, iItem, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::GetItemHeight(int iItem /* = 0 */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETITEMHEIGHT, iItem, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::GetItemLen(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETLBTEXTLEN, iItem, UNUSED), CB_ERR);
}

AFXINLINE LCID CComboBox::GetLocale() const
{
	return ::SendMessage(m_hWnd, CB_GETLOCALE, UNUSED, UNUSED);
}

AFXINLINE int CComboBox::GetTopIndex() const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_GETTOPINDEX, UNUSED, UNUSED), CB_ERR);
}

AFXINLINE void CComboBox::HideDropDown() const
{
	::SendMessage(m_hWnd, CB_SHOWDROPDOWN, FALSE, UNUSED);
}

AFXINLINE int CComboBox::InitStorage(int ceItem, UINT cbSize) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_INITSTORAGE, ceItem, cbSize), CB_ERRSPACE);
}

AFXINLINE int CComboBox::InsertItem(PCTSTR pszText, int iItem /* = -1 */) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, CB_INSERTSTRING, iItem, (LPARAM)pszText),
		CB_ERR), CB_ERRSPACE);
}

AFXINLINE int CComboBox::InsertItem(DWORD dwData, int iItem /* = -1 */) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, CB_INSERTSTRING, iItem, dwData),
		CB_ERR), CB_ERRSPACE);
}

AFXINLINE void CComboBox::LimitText(UINT cbSize) const
{
	::SendMessage(m_hWnd, CB_LIMITTEXT, cbSize, UNUSED);
}

AFXINLINE void CComboBox::ResetContent() const
{
	::SendMessage(m_hWnd, CB_RESETCONTENT, UNUSED, UNUSED);
}

AFXINLINE int CComboBox::SelectItem(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, CB_SELECTSTRING, iStart, (LPARAM)pszText);
}

AFXINLINE int CComboBox::SelectItem(DWORD dwData, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, CB_SELECTSTRING, iStart, dwData);
}

AFXINLINE int CComboBox::SetCurSel(int iItem /* = -1 */) const
{
	int iResult = ::SendMessage(m_hWnd, CB_SETCURSEL, iItem, UNUSED);
	ASSERT((iItem == -1) || (iResult != CB_ERR));
	return iResult;
}

AFXINLINE int CComboBox::SetDroppedWidth(int iWidth) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETDROPPEDWIDTH, iWidth, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::SetEditHeight(int iHeight) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETITEMHEIGHT, -1, iHeight), CB_ERR);
}

AFXINLINE int CComboBox::SetEditSel(int iStart /* = -1 */, int iEnd /* = -1 */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETEDITSEL,
		UNUSED, MAKELPARAM(iStart, iEnd)), CB_ERR);
}

AFXINLINE int CComboBox::SetExtendedUi(BOOL bExtend /* = TRUE */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETEXTENDEDUI, bExtend, UNUSED), CB_ERR);
}

AFXINLINE void CComboBox::SetHExtent(int iWidth) const
{
	::SendMessage(m_hWnd, CB_SETHORIZONTALEXTENT, iWidth, UNUSED);
}

AFXINLINE int CComboBox::SetItem(int iItem, DWORD dwData) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETITEMDATA, iItem, dwData), CB_ERR);
}

AFXINLINE int CComboBox::SetItemHeight(int iItem, int iHeight) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETITEMHEIGHT, iItem, iHeight), CB_ERR);
}

AFXINLINE int CComboBox::SetItemHeight(int iHeight) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETITEMHEIGHT, 0, iHeight), CB_ERR);
}

AFXINLINE LCID CComboBox::SetLocale(LCID lcName) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETLOCALE, lcName, UNUSED), CB_ERR);
}

AFXINLINE int CComboBox::SetTopIndex(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, CB_SETTOPINDEX, iItem, UNUSED), CB_ERR);
}

AFXINLINE void CComboBox::ShowDropDown() const
{
	::SendMessage(m_hWnd, CB_SHOWDROPDOWN, TRUE, UNUSED);
}

#ifdef  COMMCTRL
#ifndef CB_GETCUEBANNER
#define CB_GETCUEBANNER (CBM_FIRST + 4)
#endif
AFXINLINE BOOL CComboBox::GetCueBanner(PWSTR pszBuffer, int cwBuffer) const
{
	return VERIFY(::SendMessage(m_hWnd, CB_GETCUEBANNER, (WPARAM)pszBuffer, cwBuffer));
}

AFXINLINE int CComboBox::GetMinVisible() const
{
	return ::SendMessage(m_hWnd, CB_GETMINVISIBLE, UNUSED, UNUSED);
}

#ifndef CB_SETCUEBANNER
#define CB_SETCUEBANNER (CBM_FIRST + 3)
#endif
AFXINLINE BOOL CComboBox::SetCueBanner(PCWSTR pszText) const
{
	return VERIFY(::SendMessage(m_hWnd, CB_SETCUEBANNER, UNUSED, (LPARAM)pszText));
}

AFXINLINE BOOL CComboBox::SetMinVisible(int ceItem) const
{
	return VERIFY(::SendMessage(m_hWnd, CB_SETMINVISIBLE, ceItem, UNUSED));
}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// CEdit

// The user has changed an edit direction to left-to-right
// void OnEnAlignLTR(HWND hEdit);
#define ON_EN_ALIGN_LTR_EC(fName) _ON_CONTROL_NOTIFY(EN_ALIGN_LTR_EC, fName)

// The user has changed an edit direction to right-to-left
// void OnEnAlignRTL(HWND hEdit);
#define ON_EN_ALIGN_RTL_EC(fName) _ON_CONTROL_NOTIFY(EN_ALIGN_RTL_EC, fName)

// The user has altered the text in an edit
// void OnEnChange(HWND hEdit);
#define ON_EN_CHANGE(fName) _ON_CONTROL_NOTIFY(EN_CHANGE, fName)

// Edit cannot allocate enough memory
// void OnEnErrSpace(HWND hEdit);
#define ON_EN_ERRSPACE(fName) _ON_CONTROL_NOTIFY(EN_ERRSPACE, fName)

// The user clicks the horizontal scroll bar of an edit
// void OnEnHScroll(HWND hEdit);
#define ON_EN_HSCROLL(fName) _ON_CONTROL_NOTIFY(EN_HSCROLL, fName)

// Edit loses the keyboard focus
// void OnEnKillFocus(HWND hEdit);
#define ON_EN_KILLFOCUS(fName) _ON_CONTROL_NOTIFY(EN_KILLFOCUS, fName)

// The current text insertion has exceeded the specified number of characters for an edit
// void OnEnMaxText(HWND hEdit);
#define ON_EN_MAXTEXT(fName) _ON_CONTROL_NOTIFY(EN_MAXTEXT, fName)

// Edit receives the keyboard focus
// void OnEnSetFocus(HWND hEdit);
#define ON_EN_SETFOCUS(fName) _ON_CONTROL_NOTIFY(EN_SETFOCUS, fName)

// Edit is about to redraw itself
// void OnEnUpdate(HWND hEdit);
#define ON_EN_UPDATE(fName) _ON_CONTROL_NOTIFY(EN_UPDATE, fName)

// The user clicks the vertical scroll bar of an edit
// void OnEnVScroll(HWND hEdit);
#define ON_EN_VSCROLL(fName) _ON_CONTROL_NOTIFY(EN_VSCROLL, fName)

class CEdit : public CWnd
{
public:
	CEdit();
	CEdit(HWND hEdit);
	void operator =(HWND hEdit);
	BOOL CanUndo() const;
	DWORD CharFromPos(int x, int y) const;
	BOOL CreateWindow(DWORD dwExStyle, PCTSTR pszText, DWORD dwStyle, int x, int y,
		int iWidth, int iHeight, HWND hParentWnd, UINT nEditId);
	BOOL CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y,
		int iWidth, int iHeight, HWND hParentWnd, UINT nEditId);
	BOOL CreateWindow(DWORD dwStyle, int x, int y,
		int iWidth, int iHeight, HWND hParentWnd, UINT nEditId);
	void EmptyUndoBuffer() const;
	void FmtLines(BOOL bAddEol = TRUE) const;
	int GetFirstVisibleLine() const;
	HMEMORY GetHandle() const;
	UINT GetImeStatus(UINT nStatus = EMSIS_COMPOSITIONSTRING) const;
	int GetLimitText() const;
	int GetLine(int iLine, PTSTR psBuffer, int ccBuffer) const;
	int GetLineCount() const;
	DWORD GetMargins() const;
	BOOL GetModify() const;
	TCHAR GetPasswordChar() const;
	void GetRect(PRECT pRect) const;
	DWORD GetSel(PINT piStart = NULL, PINT piEnd = NULL) const;
	int GetThumb() const;
	EDITWORDBREAKPROC GetWordBreakProc() const;
	void LimitText(UINT cbSize) const;
	int LineFromChar(int iChar = -1) const;
	int LineIndex(int iLine = -1) const;
	int LineLength(int iChar = -1) const;
	BOOL LineScroll(int ceLine, int ceChar = 0) const;
	DWORD PosFromChar(int iChar) const;
	void ReplaceSel(PCTSTR pszText, BOOL bCanUndo = TRUE) const;
	DWORD Scroll(UINT nCode) const;
	void ScrollCaret() const;
	void SetHandle(HMEMORY hMemory) const;
	UINT SetImeStatus(UINT nData, UINT nStatus = EMSIS_COMPOSITIONSTRING) const;
	void SetLimitText(UINT cbSize) const;
	void SetMargins(UINT nLeft, UINT nRight,
		UINT nFlags = EC_LEFTMARGIN | EC_RIGHTMARGIN) const;
	void SetModify(BOOL bModified = FALSE) const;
	void SetPasswordChar(TCHAR cChar = 0) const;
	BOOL SetReadOnly(BOOL bReadOnly = TRUE) const;
	void SetRect(PCRECT pRect, BOOL bRelative = FALSE) const;
	void SetRectNp(PCRECT pRect, BOOL bRelative = FALSE) const;
	void SetSel(int iStart = -1, int iEnd = -1) const;
	BOOL SetTabStops(PCINT piaTabPos, int ceTabPos = 1) const;
	void SetWordBreakProc(EDITWORDBREAKPROC epNew) const;
	BOOL Undo() const;

#ifdef COMMCTRL
	BOOL GetCueBanner(PWSTR pszBuffer, int cwBuffer) const;
	BOOL HideBalloonTip() const;
	BOOL SetCueBanner(PCWSTR pszText, BOOL bAlwaysShow = FALSE) const;
	BOOL ShowBalloonTip(PCEDITBALLOONTIP pEBT) const;
#endif
};

AFXINLINE CEdit::CEdit()
{
	// Does nothing
}

AFXINLINE CEdit::CEdit(HWND hEdit)
{
	m_hWnd = hEdit;
}

AFXINLINE void CEdit::operator =(HWND hEdit)
{
	m_hWnd = hEdit;
}

AFXINLINE BOOL CEdit::CanUndo() const
{
	return ::SendMessage(m_hWnd, EM_CANUNDO, UNUSED, UNUSED);
}

AFXINLINE DWORD CEdit::CharFromPos(int x, int y) const
{
	return ::SendMessage(m_hWnd, EM_CHARFROMPOS, UNUSED, MAKELPARAM(x, y));
}

AFXINLINE BOOL CEdit::CreateWindow(DWORD dwExStyle, PCTSTR pszText, DWORD dwStyle, int x, int y,
	int iWidth, int iHeight, HWND hParentWnd, UINT nEditId)
{
	m_hWnd = ::CreateWindowEx(dwExStyle, WC_EDIT, pszText, dwStyle, x, y,
		iWidth, iHeight, hParentWnd, (HMENU)nEditId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE BOOL CEdit::CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y,
	int iWidth, int iHeight, HWND hParentWnd, UINT nEditId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_EDIT, pszText, dwStyle, x, y,
		iWidth, iHeight, hParentWnd, (HMENU)nEditId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE BOOL CEdit::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nEditId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_EDIT, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nEditId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE void CEdit::EmptyUndoBuffer() const
{
	::SendMessage(m_hWnd, EM_EMPTYUNDOBUFFER, UNUSED, UNUSED);
}

AFXINLINE void CEdit::FmtLines(BOOL bAddEol /* = TRUE */) const
{
	::SendMessage(m_hWnd, EM_FMTLINES, bAddEol, UNUSED);
}

AFXINLINE int CEdit::GetFirstVisibleLine() const
{
	return ::SendMessage(m_hWnd, EM_GETFIRSTVISIBLELINE, UNUSED, UNUSED);
}

AFXINLINE HLOCAL CEdit::GetHandle() const
{
	return (HLOCAL)::SendMessage(m_hWnd, EM_GETHANDLE, UNUSED, UNUSED);
}

AFXINLINE UINT CEdit::GetImeStatus(UINT nStatus /* = EMSIS_COMPOSITIONSTRING */) const
{
	return ::SendMessage(m_hWnd, EM_GETIMESTATUS, nStatus, UNUSED);
}

AFXINLINE int CEdit::GetLimitText() const
{
	return ::SendMessage(m_hWnd, EM_GETLIMITTEXT, UNUSED, UNUSED);
}

AFXINLINE int CEdit::GetLine(int iLine, PTSTR psBuffer, int ccBuffer) const
{
	*(PWORD)psBuffer = ccBuffer;
	return ::SendMessage(m_hWnd, EM_GETLINE, iLine, (LPARAM)psBuffer);
}

AFXINLINE int CEdit::GetLineCount() const
{
	return ::SendMessage(m_hWnd, EM_GETLINECOUNT, UNUSED, UNUSED);
}

AFXINLINE DWORD CEdit::GetMargins() const
{
	return ::SendMessage(m_hWnd, EM_GETMARGINS, UNUSED, UNUSED);
}

AFXINLINE BOOL CEdit::GetModify() const
{
	return ::SendMessage(m_hWnd, EM_GETMODIFY, UNUSED, UNUSED);
}

AFXINLINE TCHAR CEdit::GetPasswordChar() const
{
	return (TCHAR)::SendMessage(m_hWnd, EM_GETPASSWORDCHAR, UNUSED, UNUSED);
}

AFXINLINE void CEdit::GetRect(PRECT pRect) const
{
	::SendMessage(m_hWnd, EM_GETRECT, UNUSED, (LPARAM)pRect);
}

AFXINLINE DWORD CEdit::GetSel(PINT piStart /* = NULL */, PINT piEnd /* = NULL */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, EM_GETSEL, (WPARAM)piStart, (LPARAM)piEnd), RV_ERR);
}

AFXINLINE int CEdit::GetThumb() const
{
	return ::SendMessage(m_hWnd, EM_GETTHUMB, UNUSED, UNUSED);
}

AFXINLINE EDITWORDBREAKPROC CEdit::GetWordBreakProc() const
{
	return (EDITWORDBREAKPROC)::SendMessage(m_hWnd, EM_GETWORDBREAKPROC, UNUSED, UNUSED);
}

AFXINLINE void CEdit::LimitText(UINT cbSize) const
{
	::SendMessage(m_hWnd, EM_LIMITTEXT, cbSize, UNUSED);
}

AFXINLINE int CEdit::LineFromChar(int iChar /* = -1 */) const
{
	return ::SendMessage(m_hWnd, EM_LINEFROMCHAR, iChar, UNUSED);
}

AFXINLINE int CEdit::LineIndex(int iLine /* = -1 */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, EM_LINEINDEX, iLine, UNUSED), RV_ERR);
}

AFXINLINE int CEdit::LineLength(int iChar /* = -1 */) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_LINELENGTH, iChar, UNUSED));
}

AFXINLINE BOOL CEdit::LineScroll(int ceLine, int ceChar /* = 0 */) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_LINESCROLL, ceChar, ceLine));
}

AFXINLINE DWORD CEdit::PosFromChar(int iChar) const
{
	return ::SendMessage(m_hWnd, EM_POSFROMCHAR, iChar, UNUSED);
}

AFXINLINE void CEdit::ReplaceSel(PCTSTR pszText, BOOL bCanUndo /* = TRUE */) const
{
	::SendMessage(m_hWnd, EM_REPLACESEL, bCanUndo, (LPARAM)pszText);
}

AFXINLINE DWORD CEdit::Scroll(UINT nCode) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_SCROLL, nCode, UNUSED));
}

AFXINLINE void CEdit::ScrollCaret() const
{
	::SendMessage(m_hWnd, EM_SCROLLCARET, UNUSED, UNUSED);
}

AFXINLINE void CEdit::SetHandle(HLOCAL hMemory) const
{
	::SendMessage(m_hWnd, EM_SETHANDLE, (WPARAM)hMemory, UNUSED);
}

AFXINLINE UINT CEdit::SetImeStatus(UINT nData,
	UINT nStatus /* = EMSIS_COMPOSITIONSTRING */) const
{
	return ::SendMessage(m_hWnd, EM_SETIMESTATUS, nStatus, nData);
}

AFXINLINE void CEdit::SetLimitText(UINT cbSize) const
{
	::SendMessage(m_hWnd, EM_SETLIMITTEXT, cbSize, UNUSED);
}

AFXINLINE void CEdit::SetMargins(UINT nLeft, UINT nRight,
	UINT nFlags /* = EC_LEFTMARGIN | EC_RIGHTMARGIN */) const
{
	::SendMessage(m_hWnd, EM_SETMARGINS, nFlags, MAKELPARAM(nLeft, nRight));
}

AFXINLINE void CEdit::SetModify(BOOL bModified /* = FALSE */) const
{
	::SendMessage(m_hWnd, EM_SETMODIFY, bModified, UNUSED);
}

AFXINLINE void CEdit::SetPasswordChar(TCHAR cChar /* = 0 */) const
{
	::SendMessage(m_hWnd, EM_SETPASSWORDCHAR, cChar, UNUSED);
}

AFXINLINE BOOL CEdit::SetReadOnly(BOOL bReadOnly /* = TRUE */) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_SETREADONLY, bReadOnly, UNUSED));
}

AFXINLINE void CEdit::SetRect(PCRECT pRect, BOOL bRelative /* = FALSE */) const
{
	::SendMessage(m_hWnd, EM_SETRECT, bRelative, (LPARAM)pRect);
}

AFXINLINE void CEdit::SetRectNp(PCRECT pRect, BOOL bRelative /* = FALSE */) const
{
	::SendMessage(m_hWnd, EM_SETRECTNP, bRelative, (LPARAM)pRect);
}

AFXINLINE void CEdit::SetSel(int iStart /* = -1 */, int iEnd /* = -1 */) const
{
	::SendMessage(m_hWnd, EM_SETSEL, iStart, iEnd);
}

AFXINLINE BOOL CEdit::SetTabStops(PCINT piaTabPos, int ceTabPos /* = 1 */) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_SETTABSTOPS, ceTabPos, (LPARAM)piaTabPos));
}

AFXINLINE void CEdit::SetWordBreakProc(EDITWORDBREAKPROC epNew) const
{
	::SendMessage(m_hWnd, EM_SETWORDBREAKPROC, UNUSED, (LPARAM)epNew);
}

AFXINLINE BOOL CEdit::Undo() const
{
	return VERIFY(::SendMessage(m_hWnd, EM_UNDO, UNUSED, UNUSED));
}

#ifdef COMMCTRL
AFXINLINE BOOL CEdit::GetCueBanner(PWSTR pszBuffer, int cwBuffer) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_GETCUEBANNER, (WPARAM)pszBuffer, cwBuffer));
}

AFXINLINE BOOL CEdit::HideBalloonTip() const
{
	return VERIFY(::SendMessage(m_hWnd, EM_HIDEBALLOONTIP, UNUSED, UNUSED));
}

AFXINLINE BOOL CEdit::SetCueBanner(PCWSTR pszText, BOOL bAlwaysShow /* = FALSE */) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_SETCUEBANNER, bAlwaysShow, (LPARAM)pszText));
}

AFXINLINE BOOL CEdit::ShowBalloonTip(PCEDITBALLOONTIP pEBT) const
{
	return VERIFY(::SendMessage(m_hWnd, EM_SHOWBALLOONTIP, UNUSED, (LPARAM)pEBT));
}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// CListBox

// The user double-clicks an item in a list box
// void OnLbnDblClk(HWND hListBox);
#define ON_LBN_DBLCLK(fName) _ON_CONTROL_NOTIFY(LBN_DBLCLK, fName)

// List box cannot allocate enough memory
// void OnLbnErrSpace(HWND hListBox);
#define ON_LBN_ERRSPACE(fName) _ON_CONTROL_NOTIFY(LBN_ERRSPACE, fName)

// List box loses the keyboard focus
// void OnLbnKillFocus(HWND hListBox);
#define ON_LBN_KILLFOCUS(fName) _ON_CONTROL_NOTIFY(LBN_KILLFOCUS, fName)

// The user cancels the selection in a list box
// void OnLbnSelCancel(HWND hListBox);
#define ON_LBN_SELCANCEL(fName) _ON_CONTROL_NOTIFY(LBN_SELCANCEL, fName)

// The selection in a list box has changed
// void OnLbnSelChange(HWND hListBox);
#define ON_LBN_SELCHANGE(fName) _ON_CONTROL_NOTIFY(LBN_SELCHANGE, fName)

// List box receives the keyboard focus
// void OnLbnSetFocus(HWND hListBox);
#define ON_LBN_SETFOCUS(fName) _ON_CONTROL_NOTIFY(LBN_SETFOCUS, fName)

class CListBox : public CWnd
{
public:
	CListBox();
	CListBox(HWND hListBox);
	void operator =(HWND hListBox);
	int AddFile(PCTSTR pszFileName) const;
	int AddItem(PCTSTR pszText) const;
	int AddItem(DWORD dwData) const;
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nListBoxId);
	int DeleteItem(int iItem) const;
	int Dir(PCTSTR pszPathSpec, UINT nAttrs) const;
	int FindItem(PCTSTR pszText, int iStart = -1) const;
	int FindItem(DWORD dwData, int iStart = -1) const;
	int FindItemExact(PCTSTR pszText, int iStart = -1) const;
	int GetAnchorIndex() const;
	int GetCaretIndex() const;
	int GetCount() const;
	int GetCurSel() const;
	int GetHExtent() const;
	int GetInfo() const;
	int GetItem(int iItem, PTSTR pszBuffer) const;
	DWORD GetItem(int iItem) const;
	int GetItem(int x, int y) const;
	int GetItemHeight(int iItem = 0) const;
	int GetItemLen(int iItem) const;
	int GetItemRect(int iItem, PRECT pRect) const;
	LCID GetLocale() const;
	int GetSel(PINT piaItem, int ceItem) const;
	int GetSel(int iItem) const;
	int GetSelCount() const;
	int GetTopIndex() const;
	int InitStorage(int ceItem, UINT cbSize) const;
	int InsertItem(PCTSTR pszText, int iItem = -1) const;
	int InsertItem(DWORD dwData, int iItem = -1) const;
	void ResetContent() const;
	int SelectItem(PCTSTR pszText, int iStart = -1) const;
	int SelectItem(DWORD dwData, int iStart = -1) const;
	int SelectItem(int iFirst, int iLast) const;
	int SetAnchorIndex(int iItem) const;
	int SetCaretIndex(int iItem, BOOL bPartial = FALSE) const;
	void SetColumnWidth(int iWidth) const;
	int SetCount(int ceItem) const;
	int SetCurSel(int iItem = -1) const;
	void SetHExtent(int iWidth) const;
	int SetItem(int iItem, DWORD dwData) const;
	int SetItem(DWORD dwData) const;
	int SetItemHeight(int iItem, int iHeight) const;
	int SetItemHeight(int iHeight) const;
	LCID SetLocale(LCID lcName) const;
	int SetSel(int iItem, BOOL bSelect = TRUE) const;
	int SetSel(BOOL bSelect = TRUE) const;
	BOOL SetTabStops(PCINT piaTabPos, int ceTabPos = 1) const;
	int SetTopIndex(int iItem) const;
};

AFXINLINE CListBox::CListBox()
{
	// Does nothing
}

AFXINLINE CListBox::CListBox(HWND hListBox)
{
	m_hWnd = hListBox;
}

AFXINLINE void CListBox::operator =(HWND hListBox)
{
	m_hWnd = hListBox;
}

AFXINLINE int CListBox::AddFile(PCTSTR pszFileName) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_ADDFILE, UNUSED, (LPARAM)pszFileName), LB_ERR);
}

AFXINLINE int CListBox::AddItem(PCTSTR pszText) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_ADDSTRING, UNUSED, (LPARAM)pszText),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE int CListBox::AddItem(DWORD dwData) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_ADDSTRING, UNUSED, dwData),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE BOOL CListBox::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nListBoxId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_LISTBOX, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nListBoxId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE int CListBox::DeleteItem(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_DELETESTRING, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::Dir(PCTSTR pszPathSpec, UINT nAttrs) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_DIR, nAttrs, (LPARAM)pszPathSpec),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE int CListBox::FindItem(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, LB_FINDSTRING, iStart, (LPARAM)pszText);
}

AFXINLINE int CListBox::FindItem(DWORD dwData, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, LB_FINDSTRING, iStart, dwData);
}

AFXINLINE int CListBox::FindItemExact(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, LB_FINDSTRINGEXACT, iStart, (LPARAM)pszText);
}

AFXINLINE int CListBox::GetAnchorIndex() const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETANCHORINDEX, UNUSED, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetCaretIndex() const
{
	return ::SendMessage(m_hWnd, LB_GETCARETINDEX, UNUSED, UNUSED);
}

AFXINLINE int CListBox::GetCount() const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETCOUNT, UNUSED, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetCurSel() const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETCURSEL, UNUSED, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetHExtent() const
{
	return ::SendMessage(m_hWnd, LB_GETHORIZONTALEXTENT, UNUSED, UNUSED);
}

AFXINLINE int CListBox::GetInfo() const
{
	return ::GetListBoxInfo(m_hWnd);
}

AFXINLINE int CListBox::GetItem(int iItem, PTSTR pszBuffer) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETTEXT, iItem, (LPARAM)pszBuffer), LB_ERR);
}

AFXINLINE DWORD CListBox::GetItem(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETITEMDATA, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetItem(int x, int y) const
{
	return ::SendMessage(m_hWnd, LB_ITEMFROMPOINT, UNUSED, MAKELPARAM(x, y));
}

AFXINLINE int CListBox::GetItemHeight(int iItem /* = 0 */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETITEMHEIGHT, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetItemLen(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETTEXTLEN, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetItemRect(int iItem, PRECT pRect) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETITEMRECT, iItem, (LPARAM)pRect), LB_ERR);
}

AFXINLINE LCID CListBox::GetLocale() const
{
	return ::SendMessage(m_hWnd, LB_GETLOCALE, UNUSED, UNUSED);
}

AFXINLINE int CListBox::GetSel(PINT piaItem, int ceItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETSELITEMS, ceItem, (LPARAM)piaItem), LB_ERR);
}

AFXINLINE int CListBox::GetSel(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETSEL, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetSelCount() const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETSELCOUNT, UNUSED, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::GetTopIndex() const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_GETTOPINDEX, UNUSED, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::InitStorage(int ceItem, UINT cbSize) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_INITSTORAGE, ceItem, cbSize), LB_ERRSPACE);
}

AFXINLINE int CListBox::InsertItem(PCTSTR pszText, int iItem /* = -1 */) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_INSERTSTRING, iItem, (LPARAM)pszText),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE int CListBox::InsertItem(DWORD dwData, int iItem /* = -1 */) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_INSERTSTRING, iItem, dwData),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE void CListBox::ResetContent() const
{
	::SendMessage(m_hWnd, LB_RESETCONTENT, UNUSED, UNUSED);
}

AFXINLINE int CListBox::SelectItem(PCTSTR pszText, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, LB_SELECTSTRING, iStart, (LPARAM)pszText);
}

AFXINLINE int CListBox::SelectItem(DWORD dwData, int iStart /* = -1 */) const
{
	return ::SendMessage(m_hWnd, LB_SELECTSTRING, iStart, dwData);
}

AFXINLINE int CListBox::SelectItem(int iFirst, int iLast) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SELITEMRANGEEX, iFirst, iLast), LB_ERR);
}

AFXINLINE int CListBox::SetAnchorIndex(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETANCHORINDEX, iItem, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::SetCaretIndex(int iItem, BOOL bPartial /* = FALSE */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETCARETINDEX, iItem, bPartial), LB_ERR);
}

AFXINLINE void CListBox::SetColumnWidth(int iWidth) const
{
	::SendMessage(m_hWnd, LB_SETCOLUMNWIDTH, iWidth, UNUSED);
}

AFXINLINE int CListBox::SetCount(int ceItem) const
{
	return UNEQUAL(UNEQUAL(::SendMessage(m_hWnd, LB_SETCOUNT, ceItem, UNUSED),
		LB_ERR), LB_ERRSPACE);
}

AFXINLINE int CListBox::SetCurSel(int iItem /* = -1 */) const
{
	int iResult = ::SendMessage(m_hWnd, LB_SETCURSEL, iItem, UNUSED);
	ASSERT((iItem == -1) || (iResult != LB_ERR));
	return iResult;
}

AFXINLINE void CListBox::SetHExtent(int iWidth) const
{
	::SendMessage(m_hWnd, LB_SETHORIZONTALEXTENT, iWidth, UNUSED);
}

AFXINLINE int CListBox::SetItem(int iItem, DWORD dwData) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETITEMDATA, iItem, dwData), LB_ERR);
}

AFXINLINE int CListBox::SetItem(DWORD dwData) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETITEMDATA, -1, dwData), LB_ERR);
}

AFXINLINE int CListBox::SetItemHeight(int iItem, int iHeight) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETITEMHEIGHT, iItem, iHeight), LB_ERR);
}

AFXINLINE int CListBox::SetItemHeight(int iHeight) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETITEMHEIGHT, 0, iHeight), LB_ERR);
}

AFXINLINE LCID CListBox::SetLocale(LCID lcName) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETLOCALE, lcName, UNUSED), LB_ERR);
}

AFXINLINE int CListBox::SetSel(int iItem, BOOL bSelect /* = TRUE */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETSEL, bSelect, iItem), LB_ERR);
}

AFXINLINE int CListBox::SetSel(BOOL bSelect /* = TRUE */) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETSEL, bSelect, -1), LB_ERR);
}

AFXINLINE BOOL CListBox::SetTabStops(PCINT piaTabPos, int ceTabPos /* = 1 */) const
{
	return VERIFY(::SendMessage(m_hWnd, LB_SETTABSTOPS, ceTabPos, (LPARAM)piaTabPos));
}

AFXINLINE int CListBox::SetTopIndex(int iItem) const
{
	return UNEQUAL(::SendMessage(m_hWnd, LB_SETTOPINDEX, iItem, UNUSED), LB_ERR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CScrollBar

class CScrollBar : public CWnd
{
public:
	CScrollBar();
	CScrollBar(HWND hScrollBar);
	void operator =(HWND hScrollBar);
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nScrollBarId);
	BOOL Enable(UINT nArrows = ESB_ENABLE_BOTH) const;
	BOOL GetInfo(PSCROLLINFO pSI) const;
	BOOL GetInfo(PSCROLLBARINFO pSBI) const;
	int GetPos() const;
	BOOL GetRange(PINT piMinPos, PINT piMaxPos) const;
	BOOL Hide() const;
	int SetInfo(PCSCROLLINFO pSI, BOOL bRedraw = TRUE) const;
	int SetPos(int iPos, BOOL bRedraw = TRUE) const;
	int SetRange(int iMinPos, int iMaxPos, BOOL bRedraw = FALSE) const;
	BOOL Show() const;
};

AFXINLINE CScrollBar::CScrollBar()
{
	// Does nothing
}

AFXINLINE CScrollBar::CScrollBar(HWND hScrollBar)
{
	m_hWnd = hScrollBar;
}

AFXINLINE void CScrollBar::operator =(HWND hScrollBar)
{
	m_hWnd = hScrollBar;
}

AFXINLINE BOOL CScrollBar::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nScrollBarId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_SCROLLBAR, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nScrollBarId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE BOOL CScrollBar::Enable(UINT nArrows /* = ESB_ENABLE_BOTH */) const
{
	return ::EnableScrollBar(m_hWnd, SB_CTL, nArrows);
}

AFXINLINE BOOL CScrollBar::GetInfo(PSCROLLINFO pSI) const
{
	return VERIFY(::GetScrollInfo(m_hWnd, SB_CTL, pSI));
}

AFXINLINE BOOL CScrollBar::GetInfo(PSCROLLBARINFO pSBI) const
{
	return VERIFY(::GetScrollBarInfo(m_hWnd, OBJID_CLIENT, pSBI));
}

AFXINLINE int CScrollBar::GetPos() const
{
	return ::GetScrollPos(m_hWnd, SB_CTL);
}

AFXINLINE BOOL CScrollBar::GetRange(PINT piMinPos, PINT piMaxPos) const
{
	return VERIFY(::GetScrollRange(m_hWnd, SB_CTL, piMinPos, piMaxPos));
}

AFXINLINE BOOL CScrollBar::Hide() const
{
	return VERIFY(::ShowScrollBar(m_hWnd, SB_CTL, FALSE));
}

AFXINLINE int CScrollBar::SetInfo(PCSCROLLINFO pSI, BOOL bRedraw /* = TRUE */) const
{
	return ::SetScrollInfo(m_hWnd, SB_CTL, pSI, bRedraw);
}

AFXINLINE int CScrollBar::SetPos(int iPos, BOOL bRedraw /* = TRUE */) const
{
	return ::SetScrollPos(m_hWnd, SB_CTL, iPos, bRedraw);
}

AFXINLINE int CScrollBar::SetRange(int iMinPos, int iMaxPos, BOOL bRedraw /* = FALSE */) const
{
	return VERIFY(::SetScrollRange(m_hWnd, SB_CTL, iMinPos, iMaxPos, bRedraw));
}

AFXINLINE BOOL CScrollBar::Show() const
{
	return VERIFY(::ShowScrollBar(m_hWnd, SB_CTL, TRUE));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CStatic

// The user clicks a static
// void OnStnClicked(HWND hStatic);
#define ON_STN_CLICKED(fName) _ON_CONTROL_NOTIFY(STN_CLICKED, fName)

// The user double-clicks a static
// void OnStnDblClk(HWND hStatic);
#define ON_STN_DBLCLK(fName) _ON_CONTROL_NOTIFY(STN_DBLCLK, fName)

// Static is disabled
// void OnStnDisable(HWND hStatic);
#define ON_STN_DISABLE(fName) _ON_CONTROL_NOTIFY(STN_DISABLE, fName)

// Static is enabled
// void OnStnEnable(HWND hStatic);
#define ON_STN_ENABLE(fName) _ON_CONTROL_NOTIFY(STN_ENABLE, fName)

class CStatic : public CWnd
{
public:
	CStatic();
	CStatic(HWND hStatic);
	void operator =(HWND hStatic);
	BOOL CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nStaticId);
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		HWND hParentWnd, UINT nStaticId);
	HBITMAP GetBitmap() const;
	HCURSOR GetCursor() const;
	HENHMETAFILE GetEnhMetaFile() const;
	HICON GetIcon() const;
	HBITMAP SetBitmap(HBITMAP hBitmap = NULL) const;
	HCURSOR SetCursor(HCURSOR hCursor = NULL) const;
	HENHMETAFILE SetEnhMetaFile(HENHMETAFILE hMetaFile = NULL) const;
	HICON SetIcon(HICON hIcon = NULL) const;
};

AFXINLINE CStatic::CStatic()
{
	// Does nothing
}

AFXINLINE CStatic::CStatic(HWND hStatic)
{
	m_hWnd = hStatic;
}

AFXINLINE void CStatic::operator =(HWND hStatic)
{
	m_hWnd = hStatic;
}

AFXINLINE BOOL CStatic::CreateWindow(PCTSTR pszText, DWORD dwStyle, int x, int y,
	int iWidth, int iHeight, HWND hParentWnd, UINT nStaticId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_STATIC, pszText, dwStyle, x, y,
		iWidth, iHeight, hParentWnd, (HMENU)nStaticId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE BOOL CStatic::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HWND hParentWnd, UINT nStaticId)
{
	m_hWnd = ::CreateWindowEx(UNUSED, WC_STATIC, NULL, dwStyle, x, y, iWidth, iHeight,
		hParentWnd, (HMENU)nStaticId, NULL, NULL);
	return (BOOL)VERIFY(m_hWnd);
}

AFXINLINE HBITMAP CStatic::GetBitmap() const
{
	return (HBITMAP)::SendMessage(m_hWnd, STM_GETIMAGE, IMAGE_BITMAP, UNUSED);
}

AFXINLINE HCURSOR CStatic::GetCursor() const
{
	return (HCURSOR)::SendMessage(m_hWnd, STM_GETIMAGE, IMAGE_CURSOR, UNUSED);
}

AFXINLINE HENHMETAFILE CStatic::GetEnhMetaFile() const
{
	return (HENHMETAFILE)::SendMessage(m_hWnd, STM_GETIMAGE, IMAGE_ENHMETAFILE, UNUSED);
}

AFXINLINE HICON CStatic::GetIcon() const
{
	return (HICON)::SendMessage(m_hWnd, STM_GETICON, UNUSED, UNUSED);
}

AFXINLINE HBITMAP CStatic::SetBitmap(HBITMAP hBitmap /* = NULL */) const
{
	return (HBITMAP)::SendMessage(m_hWnd, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)hBitmap);
}

AFXINLINE HCURSOR CStatic::SetCursor(HCURSOR hCursor /* = NULL */) const
{
	return (HCURSOR)::SendMessage(m_hWnd, STM_SETIMAGE, IMAGE_CURSOR, (LPARAM)hCursor);
}

AFXINLINE HENHMETAFILE CStatic::SetEnhMetaFile(HENHMETAFILE hMetaFile /* = NULL */) const
{
	return (HENHMETAFILE)::SendMessage(m_hWnd, STM_SETIMAGE,
		IMAGE_ENHMETAFILE, (LPARAM)hMetaFile);
}

AFXINLINE HICON CStatic::SetIcon(HICON hIcon /* = NULL */) const
{
	return (HICON)::SendMessage(m_hWnd, STM_SETICON, (WPARAM)hIcon, UNUSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CCustomCtrl

// Only can be derived (cannot be instantiated)
class CCustomCtrl : public CWnd
{
protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_CTMCTRL };

protected:
	CCustomCtrl(UINT nCtrlId);

protected:
	// Gets object pointer to this control (used internally only)
	static CWnd* WINAPI _GetObjectPtr(HWND hWnd);
};

AFXINLINE CCustomCtrl::CCustomCtrl(UINT nCtrlId)
{
	VERIFY(::PostMessage(NULL, _WM_CTMCTRL, nCtrlId, (LPARAM)this));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CSubclassWnd

// Only can be derived (cannot be instantiated)
// Selects a control as the base class
template<class tBaseClass>
class CSubclassWnd : public tBaseClass
{
public:
	// Previous window procedure
	WNDPROC m_wpPrev;

public:
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	void SubclassWindow();
	void UnsubclassWindow();

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_SUBCLASS };

protected:
	CSubclassWnd();

protected:
	// Gets object pointer to this control (used internally only)
	static CWnd* WINAPI _GetObjectPtr(HWND hWnd);
};

template<class tBaseClass>
AFXINLINE CSubclassWnd<tBaseClass>::CSubclassWnd()
{
	DEBUG_ONLY(m_wpPrev = NULL);
}

template<class tBaseClass>
AFXINLINE CWnd* WINAPI CSubclassWnd<tBaseClass>::_GetObjectPtr(HWND hWnd)
{
	CWnd* pWnd = (CWnd*)::GetWindowLong(hWnd, GWL_USERDATA);
	ASSERT(pWnd->m_hWnd == hWnd);
	return pWnd;
}

template<class tBaseClass>
AFXINLINE LRESULT CSubclassWnd<tBaseClass>::DefWindowProc(UINT nMsg,
	WPARAM wParam /* = UNUSED */, LPARAM lParam /* = UNUSED */) const
{
	return ::CallWindowProc(m_wpPrev, m_hWnd, nMsg, wParam, lParam);
}

template<class tBaseClass>
AFXINLINE void CSubclassWnd<tBaseClass>::UnsubclassWindow()
{
	ASSERT_PTR(m_wpPrev);
	::SetWindowLong(m_hWnd, GWL_WNDPROC, (LONG)m_wpPrev);
	DEBUG_ONLY(m_wpPrev = NULL);
}

// End of _PRECTRL_INCLUDED_
#endif
