////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// ScrnSave.hpp - Screen Saver Declaration File                                               //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _SCRNSAVE_INCLUDED_
#define _SCRNSAVE_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  ScrnSave - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Exports required function names

#pragma comment(linker, "/EXPORT:ScreenSaverProc=_ScreenSaverProc@16")
#pragma comment(linker, "/EXPORT:ScreenSaverConfigureDialog=_ScreenSaverConfigureDialog@16")

////////////////////////////////////////////////////////////////////////////////////////////////
// CScreenSaverDialog

// Implements configuration dialog object (used internally only)
// CDlgName : Name of screen saver's configuration dialog box
#define _SCREEN_SAVER_CONFIGURE_DIALOG \
	BOOL WINAPI ScreenSaverConfigureDialog(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)

#define _REGISTER_DIALOG_CLASSES(CDlgName) \
	BOOL WINAPI RegisterDialogClasses(HANDLE hInstance) \
	{ \
		return CDlgName::RegisterDialogClasses((HINSTANCE)hInstance); \
	}

#define _IMPLEMENT_SCRNSAVE_DIALOG_DYNAMIC(CDlgName) \
	CDlgName* _i_pSSD = VERIFY(new CDlgName); \
	_SCREEN_SAVER_CONFIGURE_DIALOG \
	{ \
		_i_pSSD->m_hWnd = hWnd; \
		return _i_pSSD->__WindowProc(nMsg, wParam, lParam); \
	} \
	_REGISTER_DIALOG_CLASSES(CDlgName)

#define _IMPLEMENT_SCRNSAVE_DIALOG_STATIC(CDlgName) \
	CDlgName _i_jSSD; \
	_SCREEN_SAVER_CONFIGURE_DIALOG \
	{ \
		_i_jSSD.m_hWnd = hWnd; \
		return _i_jSSD.__WindowProc(nMsg, wParam, lParam); \
	} \
	_REGISTER_DIALOG_CLASSES(CDlgName)

// Only can be derived (cannot be instantiated), and only one derived object can be created
class CScreenSaverDialog : public CDialog
{
public:
	static BOOL WINAPI RegisterDialogClasses(HINSTANCE hInstance);

protected:
	CScreenSaverDialog();
};

AFXINLINE CScreenSaverDialog::CScreenSaverDialog()
{
	// Does nothing
}

AFXINLINE BOOL WINAPI CScreenSaverDialog::RegisterDialogClasses(HINSTANCE hInstance)
{
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CScreenSaver

// Implements screen saver and configuration dialog objects
// When creates object by DYNAMIC storgae class, deletes the object in DetachObject
// CName      : Name of screen saver's window class
// WndStorage : Storage class (DYNAMIC or STATIC) of window class
// CDlgName   : Name of screen saver's configuration dialog box
// DlgStorage : Storage class (DYNAMIC or STATIC) of configuration dialog box
#define IMPLEMENT_SCRNSAVE(CName, WndStorage, CDlgName, DlgStorage) \
	_IMPLEMENT_SCRNSAVE_##WndStorage(CName) \
	_IMPLEMENT_SCRNSAVE_DIALOG_##DlgStorage(CDlgName)

#define _SCREEN_SAVER_PROC \
	LRESULT WINAPI ScreenSaverProc(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)

#define _IMPLEMENT_SCRNSAVE_DYNAMIC(CName) \
	CName* _i_pSS = VERIFY(new CName); \
	_SCREEN_SAVER_PROC \
	{ \
		_i_pSS->m_hWnd = hWnd; \
		return _i_pSS->__WindowProc(nMsg, wParam, lParam); \
	}

#define _IMPLEMENT_SCRNSAVE_STATIC(CName) \
	CName _i_jSS; \
	_SCREEN_SAVER_PROC \
	{ \
		_i_jSS.m_hWnd = hWnd; \
		return _i_jSS.__WindowProc(nMsg, wParam, lParam); \
	}

// Only can be derived (cannot be instantiated), and only one derived object can be created
class CScreenSaver : public CWnd
{
public:
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_SCRNSAVE };

protected:
	CScreenSaver();
};

AFXINLINE CScreenSaver::CScreenSaver()
{
	// Does nothing
}

AFXINLINE LRESULT CScreenSaver::DefWindowProc(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return ::DefScreenSaverProc(m_hWnd, nMsg, wParam, lParam);
}

// End of _SCRNSAVE_INCLUDED_
#endif
