////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// TypeDef.hpp - General Data Types and Constants Definition File                             //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _TYPEDEF_INCLUDED_
#define _TYPEDEF_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  TypeDef - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// General purposes

// Constant symbols
#define UNUSED          0                     // Any unused item
#define MIN_CHAR        CHAR_MIN              // Minimum char value
#define MAX_CHAR        CHAR_MAX              // Maximum char value
#define MAX_UCHAR       UCHAR_MAX             // Maximum unsigned char value
#define MAX_BYTE        UCHAR_MAX             // Maximum BYTE value
#define MIN_SHORT       SHRT_MIN              // Minimum short value
#define MAX_SHORT       SHRT_MAX              // Maximum short value
#define MAX_USHORT      USHRT_MAX             // Maximum unsigned short value
#define MAX_WORD        USHRT_MAX             // Maximum WORD value
#define MIN_INT         INT_MIN               // Minimum int value
#define MAX_INT         INT_MAX               // Maximum int value
#define MAX_UINT        UINT_MAX              // Maximum unsigned int value
#define MIN_LONG        LONG_MIN              // Minimum long value
#define MAX_LONG        LONG_MAX              // Maximum long value
#define MIN_LPARAM      MIN_LONG              // Minimum LPARAM value
#define MAX_LPARAM      MAX_LONG              // Maximum LPARAM value
#define MAX_ULONG       ULONG_MAX             // Maximum unsigned long value
#define MAX_DWORD       ULONG_MAX             // Maximum DWORD value
#define MIN_LLONG       _I64_MIN              // Minimum __int64 (LLONG) value
#define MAX_LLONG       _I64_MAX              // Maximum __int64 (LLONG) value
#define MAX_ULLONG      _UI64_MAX             // Maximum unsigned __int64 (ULLONG) value
#define MAX_DLONG       _UI64_MAX             // Maximum unsigned __int64 (DLONG) value
#define MIN_FLOAT       FLT_MIN               // Minimum float value
#define MAX_FLOAT       FLT_MAX               // Maximum float value
#define FLOAT_EPSILON   FLT_EPSILON           // Epsilon value for float
#define MIN_DOUBLE      DBL_MIN               // Minimum double value
#define MAX_DOUBLE      DBL_MAX               // Maximum double value
#define DOUBLE_EPSILON  DBL_EPSILON           // Epsilon value for double
#define MIN_LDOUBLE     LDBL_MIN              // Minimum long double value
#define MAX_LDOUBLE     LDBL_MAX              // Maximum long double value
#define LDOUBLE_EPSILON LDBL_EPSILON          // Epsilon value for long double
#define MAX_VOLUME      12                    // Maximum length of volume (including a null)
#define MAX_DRIVE       _MAX_DRIVE            // Maximum length of drive (including a null)
#define MAX_DIR         _MAX_DIR              // Maximum length of directory (including a null)
#define MAX_FNAME       _MAX_FNAME            // Maximum length of file name (including a null)
#define MAX_EXT         _MAX_EXT              // Maximum length of extension (including a null)
#define MAX_RESID       USHRT_MAX             // Maximum value of resource identifier
#define MAX_STR         256                   // Maximum length of string (including a null)
#define INVALID_HANDLE  INVALID_HANDLE_VALUE  // Invalid handle
#define INVALID_VALUE   (-1)                  // Invalid value
#define RH_ERR          INVALID_HANDLE        // Return handle is error
#define RV_ERR          INVALID_VALUE         // Return value is error
#define RV_IGNORE       0                     // Return value is ignored

// Expression for specific character set only
#ifdef  UNICODE
#define UNICODE_ONLY(e) e
#define MBCS_ONLY(e)
#define SBCS_ONLY(e)
#elif   defined(MBCS)
#define UNICODE_ONLY(e)
#define MBCS_ONLY(e) e
#define SBCS_ONLY(e)
#else
#define UNICODE_ONLY(e)
#define MBCS_ONLY(e)
#define SBCS_ONLY(e) e
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Color reference codes for COLORREF

#define RGB_BLACK     RGB(  0,   0,   0)
#define RGB_BLUE      RGB(  0,   0, 255)
#define RGB_GREEN     RGB(  0, 255,   0)
#define RGB_CYAN      RGB(  0, 255, 255)
#define RGB_RED       RGB(255,   0,   0)
#define RGB_MAGENTA   RGB(255,   0, 255)
#define RGB_YELLOW    RGB(255, 255,   0)
#define RGB_WHITE     RGB(255, 255, 255)
#define RGB_DKBLUE    RGB(  0,   0, 128)
#define RGB_DKGREEN   RGB(  0, 128,   0)
#define RGB_DKGRAY    RGB(  0, 128, 128)
#define RGB_DKBROWN   RGB(128,   0,   0)
#define RGB_DKMAGENTA RGB(128,   0, 128)
#define RGB_BROWN     RGB(128, 128,   0)
#define RGB_GRAY      RGB(128, 128, 128)
#define RGB_LTGRAY    RGB(192, 192, 192)
#define RGB_BTGRAY    RGB(192, 220, 192)
#define RGB_LTBLUE    RGB(166, 202, 240)
#define RGB_MIDGRAY   RGB(160, 160, 164)
#define RGB_MILKY     RGB(255, 251, 240)

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Handles
#undef  HFILE
typedef HANDLE  HEVENT;
typedef HANDLE  HFILE;
typedef HGLOBAL HMEMORY;
typedef HANDLE  HPRINTER;
typedef HANDLE  HTHREAD;

// Synonyms
typedef _TXCHAR        TXCHAR;
typedef _TUCHAR        TUCHAR;
typedef INT_PTR        XINT;
typedef LONG_PTR       XLONG;
typedef ULONG_PTR      XULONG;
typedef LONGLONG       LLONG;
typedef ULONGLONG      ULLONG;
typedef DWORDLONG      DLONG;
typedef long double    LDOUBLE;
typedef const BYTE     CBYTE;
typedef const WCHAR    CWCHAR;
typedef const TCHAR    CTCHAR;
typedef const WORD     CWORD;
typedef const int      CINT;
typedef const UINT     CUINT;
typedef const LONG     CLONG;
typedef const DWORD    CDWORD;
typedef const LOGBRUSH CLOGBRUSH;

// Pointers
typedef bool*                     pbool;
typedef ATOM*                     PATOM;
typedef COLORREF*                 PCOLORREF;
typedef double*                   PDOUBLE;
typedef LDOUBLE*                  PLDOUBLE;
typedef HMEMORY*                  PHMEMORY;
typedef HPRINTER*                 PHPRINTER;
typedef HPALETTE*                 PHPALETTE;
typedef DRAWTEXTPARAMS*           PDRAWTEXTPARAMS;
typedef ENUMLOGFONT*              PENUMLOGFONT;
typedef ENUMLOGFONTEX*            PENUMLOGFONTEX;
typedef MEMORYSTATUS*             PMEMORYSTATUS;
typedef MEMORYSTATUSEX*           PMEMORYSTATUSEX;
typedef NCCALCSIZE_PARAMS*        PNCCALCSIZE_PARAMS;
typedef NEWTEXTMETRIC*            PNEWTEXTMETRIC;
typedef NEWTEXTMETRICEX*          PNEWTEXTMETRICEX;
typedef NMHDR*                    PNMHDR;
typedef OVERLAPPED*               POVERLAPPED;
typedef RGBQUAD*                  PRGBQUAD;
typedef RGBTRIPLE*                PRGBTRIPLE;
typedef SCROLLINFO*               PSCROLLINFO;
typedef STYLESTRUCT*              PSTYLESTRUCT;
typedef TPMPARAMS*                PTPMPARAMS;
typedef HWND*                     PHWND;
typedef void**                    PPVOID;
typedef char**                    PPCHAR;
typedef BYTE**                    PPBYTE;
typedef TCHAR**                   PPTCHAR;
typedef const void*               PCVOID;
typedef const char*               PCCHAR;
typedef const BYTE*               PCBYTE;
typedef const TCHAR*              PCTCHAR;
typedef const WCHAR*              PCWCHAR;
typedef const int*                PCINT;
typedef const LONG*               PCLONG;
typedef const DWORD*              PCDWORD;
typedef const HWND*               PCHWND;
typedef const BITMAP*             PCBITMAP;
typedef const BITMAPINFO*         PCBITMAPINFO;
typedef const BITMAPINFOHEADER*   PCBITMAPINFOHEADER;
typedef const CLIENTCREATESTRUCT* PCCLIENTCREATESTRUCT;
typedef const CREATESTRUCT*       PCCREATESTRUCT;
typedef const DEVMODE*            PCDEVMODE;
typedef const DLGTEMPLATE*        PCDLGTEMPLATE;
typedef const DRAWITEMSTRUCT*     PCDRAWITEMSTRUCT;
typedef const ENHMETARECORD*      PCENHMETARECORD;
typedef const HELPINFO*           PCHELPINFO;
typedef const LOGBRUSH*           PCLOGBRUSH;
typedef const LOGFONT*            PCLOGFONT;
typedef const LOGPEN*             PCLOGPEN;
typedef const MDICREATESTRUCT*    PCMDICREATESTRUCT;
typedef const MENUTEMPLATE*       PCMENUTEMPLATE;
typedef const METAFILEPICT*       PCMETAFILEPICT;
typedef const MSG*                PCMSG;
typedef const MSGBOXPARAMS*       PCMSGBOXPARAMS;
typedef const NMHDR*              PCNMHDR;
typedef const PAINTSTRUCT*        PCPAINTSTRUCT;
typedef const POINT*              PCPOINT;
typedef const RECT*               PCRECT;
typedef const SCROLLINFO*         PCSCROLLINFO;
typedef const STYLESTRUCT*        PCSTYLESTRUCT;
typedef const SYSTEMTIME*         PCSYSTEMTIME;
typedef const WNDCLASSEX*         PCWNDCLASSEX;
typedef const WINDOWPLACEMENT*    PCWINDOWPLACEMENT;
typedef const WINDOWPOS*          PCWINDOWPOS;
typedef const TCHAR* const        CPCTSTR;

// Volatile pointers
typedef const BOOL volatile* PCBOOLV;

// Callback functions used with EnumFontFamilies and EnumFontFamiliesEx
typedef BOOL (CALLBACK* ENUMFONTFAMPROCEX)(PENUMLOGFONTEX, PNEWTEXTMETRICEX, DWORD, LPARAM);
typedef BOOL (CALLBACK* ENUMFONTFAMPROC)(PENUMLOGFONT, PNEWTEXTMETRIC, DWORD, LPARAM);

// Overlapped completion routine
typedef void (WINAPI* POCR)(DWORD, DWORD, POVERLAPPED);

// Thread procedures for BeginThread and CreateThread
typedef PTHREAD_START_ROUTINE THREADPROC;
typedef void (WINAPIV* THREADPROCV)(PVOID);

// Static
#define SCHAR        static char
#define STCHAR       static TCHAR
#define SBOOL        static BOOL
#define SWORD        static WORD
#define SINT         static int
#define SUINT        static UINT
#define SDWORD       static DWORD
#define SCOLORREF    static COLORREF
#define SFLOAT       static float
#define SDOCINFO     static DOCINFO
#define SLOGFONT     static LOGFONT
#define SRECT        static RECT
#define SWNDCLASS    static WNDCLASS
#define SWNDCLASSEX  static WNDCLASSEX
#define SCCHAR       static const char
#define SCBYTE       static const BYTE
#define SCTCHAR      static const TCHAR
#define SCWCHAR      static const WCHAR
#define SCBOOL       static const BOOL
#define SCWORD       static const WORD
#define SCINT        static const int
#define SCUINT       static const UINT
#define SCLONG       static const LONG
#define SCDWORD      static const DWORD
#define SCCOLORREF   static const COLORREF
#define SCDOCINFO    static const DOCINFO
#define SCLOGBRUSH   static const LOGBRUSH
#define SCPOINT      static const POINT
#define SCWNDCLASS   static const WNDCLASS
#define SCWNDCLASSEX static const WNDCLASSEX
#define SCPCTSTR     static const TCHAR* const

////////////////////////////////////////////////////////////////////////////////////////////////
// Character constants and string literals

// Characters
#define CHR_NULL _T('\0')    // Null (0x00)
#define CHR_BEL  _T('\a')    // Bell (0x07)
#define CHR_BS   _T('\b')    // Backspace (0x08)
#define CHR_TAB  _T('\t')    // Horizontal tab (0x09)
#define CHR_LF   _T('\n')    // Linefeed (0x0A)
#define CHR_VT   _T('\v')    // Vertical tab (0x0B)
#define CHR_FF   _T('\f')    // Formfeed (0x0C)
#define CHR_CR   _T('\r')    // Carriage return (0x0D)
#define CHR_ESC  _T('\x1B')  // Escape (0x1B)
#define CHR_SP   _T(' ')     // Space (0x20)
#define CHR_EM   _T('!')     // Exclamation mark (0x21)
#define CHR_DQM  _T('"')     // Double quotation mark (0x22)
#define CHR_NS   _T('#')     // Number sign (0x23)
#define CHR_DS   _T('$')     // Dollar sign (0x24)
#define CHR_PER  _T('%')     // Percent sign (0x25)
#define CHR_AMP  _T('&')     // Ampersand (0x26)
#define CHR_SQM  _T('\'')    // Single quotation mark (0x27)
#define CHR_LP   _T('(')     // Left parenthesis (0x28)
#define CHR_RP   _T(')')     // Right parenthesis (0x29)
#define CHR_AST  _T('*')     // Asterisk (0x2A)
#define CHR_PS   _T('+')     // Plus sign (0x2B)
#define CHR_CA   _T(',')     // Comma (0x2C)
#define CHR_DSH  _T('-')     // Dash (0x2D)
#define CHR_DOT  _T('.')     // Dot (0x2E)
#define CHR_SLH  _T('/')     // Slash (0x2F)
#define CHR_CLN  _T(':')     // Colon (0x3A)
#define CHR_SC   _T(';')     // Semicolon (0x3B)
#define CHR_LT   _T('<')     // Less than (0x3C)
#define CHR_EQL  _T('=')     // Equals sign (0x3D)
#define CHR_GT   _T('>')     // Greater than (0x3E)
#define CHR_QM   _T('?')     // Question mark (0x3F)
#define CHR_AT   _T('@')     // At sign (0x40)
#define CHR_LSB  _T('[')     // Left square bracket (0x5B)
#define CHR_BSLH _T('\\')    // Backslash (0x5C)
#define CHR_RSB  _T(']')     // Right square bracket (0x5D)
#define CHR_CRT  _T('^')     // Caret (0x5E)
#define CHR_US   _T('_')     // Underscore (0x5F)
#define CHR_GA   _T('`')     // Grave accent (0x60)
#define CHR_LCB  _T('{')     // Left curly brace (0x7B)
#define CHR_VL   _T('|')     // Vertical line (0x7C)
#define CHR_RCB  _T('}')     // Right curly brace (0x7D)
#define CHR_TLD  _T('~')     // Tilde (0x7E)

// Strings
#define STR_NULL _T("")      // Null
#define STR_BEL  _T("\a")    // Bell
#define STR_BS   _T("\b")    // Backspace
#define STR_TAB  _T("\t")    // Horizontal tab
#define STR_LF   _T("\n")    // Linefeed
#define STR_VT   _T("\v")    // Vertical tab
#define STR_FF   _T("\f")    // Formfeed
#define STR_CR   _T("\r")    // Carriage return
#define STR_ESC  _T("\x1B")  // Escape
#define STR_SP   _T(" ")     // Space
#define STR_EM   _T("!")     // Exclamation mark
#define STR_DQM  _T("\"")    // Double quotation mark
#define STR_NS   _T("#")     // Number sign
#define STR_DS   _T("$")     // Dollar sign
#define STR_PER  _T("%")     // Percent sign
#define STR_AMP  _T("&")     // Ampersand
#define STR_SQM  _T("'")     // Single quotation mark
#define STR_LP   _T("(")     // Left parenthesis
#define STR_RP   _T(")")     // Right parenthesis
#define STR_AST  _T("*")     // Asterisk
#define STR_PS   _T("+")     // Plus sign
#define STR_CA   _T(",")     // Comma
#define STR_DSH  _T("-")     // Dash
#define STR_DOT  _T(".")     // Dot
#define STR_SLH  _T("/")     // Slash
#define STR_CLN  _T(":")     // Colon
#define STR_SC   _T(";")     // Semicolon
#define STR_LT   _T("<")     // Less than
#define STR_EQL  _T("=")     // Equals sign
#define STR_GT   _T(">")     // Greater than
#define STR_QM   _T("?")     // Question mark
#define STR_AT   _T("@")     // At sign
#define STR_LSB  _T("[")     // Left square bracket
#define STR_BSLH _T("\\")    // Backslash
#define STR_RSB  _T("]")     // Right square bracket
#define STR_CRT  _T("^")     // Caret
#define STR_US   _T("_")     // Underscore
#define STR_GA   _T("`")     // Grave accent
#define STR_LCB  _T("{")     // Left curly brace
#define STR_VL   _T("|")     // Vertical line
#define STR_RCB  _T("}")     // Right curly brace
#define STR_TLD  _T("~")     // Tilde
#define STR_CRLF _T("\r\n")  // Carriage return/linefeed combination
#define STR_NL   STR_CRLF    // New line (same as STR_CRLF)

////////////////////////////////////////////////////////////////////////////////////////////////
// Calling conventions, storage classes and extension keywords

// Embeds assembly-language instructions
#define ASM __asm

// Imports/exports a variable/function
#define DLLEXPORT __declspec(dllexport)
#define DLLIMPORT __declspec(dllimport)

// Declares a variable or function has external linkage
#define EXTERN extern

// Declares and initializes a global variable in a multiple included file
#define EXTVAR extern __declspec(selectany)

// Forces to insert a copy of a function body in each place the function is called
#define FORCEINLINE __forceinline
#define AFXINLINE   FORCEINLINE

// Declares a variable or function has global linkage
#define GLOBAL

// Declares a variable or function has internal linkage
#define INTERN static

// Function is defined without prolog and epilog codes
#define NAKED __declspec(naked)

// Pure virtual function
#define PURE = 0

// Variable is declared/defined for each thread in a multithreaded process
// Must not be used in a DLL when loading the DLL explicitly by LoadLibrary* functions
#ifdef  MT
#define THREAD __declspec(thread)
#else
#define THREAD
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// General macros

// Gets a count of elements in an array
// Returns: Number of elements in the array
#define countof(a) (sizeof(a) / sizeof(a[0]))

// Gets the offset of a member in a containing class
// Returns: Byte offset of the member
#define memberof(c, m) ((UINT)&((c*)0)->m)

// Gets the address of a containing class by a pointer to a member
// Returns: A pointer to the class
#define classof(c, m, p) ((c*)((UINT)(p) - memberof(c, m)))

// Gets the alignment requirement for a type
// Returns: Number of bytes for alignment
#define alignof(t) memberof(struct { char x; t y; }, y)

// Gets the green component of a RGB value
// Returns: Intensity of the green component
#undef  GetGValue
#define GetGValue(cr) ((BYTE)((cr) >> 8))

// Gets the high/low-order BYTE from a 16-bit value
// Returns: High/low-order BYTE of the value
#undef  HIBYTE
#undef  LOBYTE
#define HIBYTE(w) ((BYTE)((WORD)(w) >> 8))
#define LOBYTE(w) ((BYTE)(w))

// Gets the high/low-order short from a 32-bit value
// Returns: High/low-order short of the value
#define HISHORT(dw) ((short)((DWORD)(dw) >> 16))
#define LOSHORT(dw) ((short)(dw))

// Gets the high/low-order WORD from a 32-bit value
// Returns: High/low-order WORD of the value
#undef  HIWORD
#undef  LOWORD
#define HIWORD(dw) ((WORD)((DWORD)(dw) >> 16))
#define LOWORD(dw) ((WORD)(dw))

// Creates a WORD value by concatenating two BYTE values
// Returns: Requested value
#undef  MAKEWORD
#define MAKEWORD(l, h) (((WORD)(h) << 8) | (BYTE)(l))

// Creates a LONG value by concatenating two WORD values
// Returns: Requested value
#undef  MAKELONG
#define MAKELONG(l, h) ((LONG)(((DWORD)(h) << 16) | (WORD)(l)))

// Converts an ULARGE_INTEGER structure to/from a FILETIME structure
// Returns: A converted FILETIME/ULARGE_INTEGER
#define MAKEFILETIME(uli) (*(PFILETIME)&uli)
#define MAKEULLONG(ft)    (*(PULARGE_INTEGER)&ft)

// Creates an atom that represents a string of decimal digits
// Returns: A pointer represents the atom
#undef  MAKEINTATOM
#define MAKEINTATOM(a) ((PCTSTR)(DWORD)(WORD)(a))

// Creates a resource type by an integer value
// Returns: A pointer represents the resource
#undef  MAKEINTRESOURCE
#define MAKEINTRESOURCE(i) ((PCTSTR)(DWORD)(WORD)(i))
#define MAKERESPTR(i)      MAKEINTRESOURCE(i)

// Determines whether a resource identifier is valid
// Returns: TRUE if the identifier is valid
#undef  IS_INTRESOURCE
#define IS_INTRESOURCE(i) ((UINT)(i) <= MAX_RESID)
#define IS_RESID(i)       IS_INTRESOURCE(i)

// Eliminates compiler warning generated when a label or local variable is not referenced
#define UNREFERENCED_LABEL(l)    while(0) goto l
#define UNREFERENCED_VARIABLE(v) (v)

// End of _TYPEDEF_INCLUDED_
#endif
