////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// WinAfx.hpp - Application Framework for Windows                                             //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
//                                                                                            //
// Predefines the symbols for building the specified version                                  //
// ------------------------------------------------------------------------------------------ //
// WINDOWS or _WINDOWS - Windows application (always defined)                                 //
// WINVER  or _WINVER  - Platform version control (default setting for Windows XP, 0x0501)    //
// WIN64   or _WIN64   - Win64 application                                                    //
// WIN32   or _WIN32   - Win32 application (default setting)                                  //
// WINLIB  or _WINLIB  - Static-link library (LIB)                                            //
// WINDLL  or _WINDLL  - Dynamic-link library (DLL)                                           //
// UNICODE or _UNICODE - Unicode character set                                                //
// MBCS    or _MBCS    - Multibyte character set                                              //
// SBCS    or _SBCS    - Single-byte character set (default setting)                          //
// DEBUG   or _DEBUG   - Debug version                                                        //
// NDEBUG  or _NDEBUG  - Release version (default setting)                                    //
// MSC_VER or _MSC_VER - Major and minor versions of the compiler (automatically defined)     //
// MT      or _MT      - Multithreaded application (automatically defined)                    //
// STRICT  or _STRICT  - Enables type-safe checking (always defined)                          //
// WINAFX  or _WINAFX  - Builds WinAfx static-link library (used internally only)             //
//                                                                                            //
//                                                                                            //
// Predefines the symbols for including the specified files                                   //
// '<symbol>' meaning 'symbol' is also defined                                                //
// ------------------------------------------------------------------------------------------ //
// COMMCTRL  - Common controls <PRECTRL>                                                      //
// COMMDLG   - Common dialog boxes                                                            //
// CTRLPANEL - Control Panel <WINDLL>                                                         //
// DDE       - Dynamic data exchange                                                          //
// DDEML     - DDEML                                                                          //
// DIRECTX   - DirectX features: uses the following symbols to include additional behavior    //
//                 D3D - Direct3D components <DIRECTX>                                        //
// GDIPLUS   - GDI+ functions and classes                                                     //
// INCOLE    - OLE features: uses the following symbols to include additional behavior        //
//                 MSHTML       - MS HTML parsing and rendering engine <OLECTRL><OLEEXDISP>   //
//                 OLECTRL      - OLE control interfaces <INCOLE>                             //
//                 OLEDLG       - OLE UI dialog boxes <INCOLE>                                //
//                 OLEEXDISP    - Explorer dispatch <INCOLE>                                  //
//                 OLEINPROC    - Exports DllGetClassObject/DllCanUnloadNow <WINDLL><INCOLE>  //
//                 OLEINPROCREG - Exports DllRegisterServer/DllUnregisterServer <OLEINPROC>   //
// MATH      - Mathematical                                                                   //
// MMSYSTEM  - Multimedia system                                                              //
// OLEMSH    - Builds OLE marshaling proxy/stub DLL <WINDLL>                                  //
// PRECTRL   - Predefined controls                                                            //
// RICHEDIT  - Rich edit control <PRECTRL>                                                    //
// SCRNSAVE  - Screen saver                                                                   //
// SHLAPI    - Shell utilities                                                                //
// SHLWAPI   - Shell lightweight utilities                                                    //
// WININET   - Windows Internet                                                               //
// WINSOCK   - Windows Sockets                                                                //
// WINSPOOL  - Windows printer spooler                                                        //
// WINTHEME  - Windows theme                                                                  //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _WINAFX_INCLUDED_
#define _WINAFX_INCLUDED_

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// General symbols

// Windows application (WINDOWS = _WINDOWS, always defined)
#ifndef WINDOWS
#define WINDOWS
#endif
#ifndef _WINDOWS
#define _WINDOWS
#endif

// Platform version control (WINVER = _WINVER, default setting for Windows XP, 0x0501)
#if     (!defined(WINVER) && !defined(_WINVER))
#define WINVER 0x0501
#endif
#ifdef  WINVER
#ifndef _WINVER
#define _WINVER WINVER
#endif
#elif   defined(_WINVER)
#define WINVER _WINVER
#endif
#ifdef  WIN32_WINDOWS
#ifndef _WIN32_WINDOWS
#define _WIN32_WINDOWS WIN32_WINDOWS
#endif
#elif   defined(_WIN32_WINDOWS)
#define WIN32_WINDOWS _WIN32_WINDOWS
#endif
#if     (!defined(WIN32_WINDOWS) && !defined(WIN32_WINNT))
#define WIN32_WINNT WINVER
#endif
#ifdef  WIN32_WINNT
#ifndef _WIN32_WINNT
#define _WIN32_WINNT WIN32_WINNT
#endif
#elif   defined(_WIN32_WINNT)
#define WIN32_WINNT _WIN32_WINNT
#endif
#ifdef  WIN32_WINNT
#ifndef WINNT
#define WINNT
#endif
#ifndef _WINNT
#define _WINNT
#endif
#endif

// Win64 application (WIN64 = _WIN64)
#ifdef  WIN64
#ifndef _WIN64
#define _WIN64
#endif
#elif   defined(_WIN64)
#define WIN64
#endif

// Win32 application (WIN32 = _WIN32, default setting)
#if     (!defined(WIN64) && !defined(WIN32))
#define WIN32
#endif
#ifdef  WIN32
#ifndef _WIN32
#define _WIN32
#endif
#elif   defined(_WIN32)
#define WIN32
#endif

// Builds WinAfx static-link library (WINAFX = _WINAFX, used internally only)
#ifdef  WINAFX
#ifndef _WINAFX
#define _WINAFX
#endif
#elif   defined(_WINAFX)
#define WINAFX
#endif
#if     (defined(WINAFX) && !defined(WINLIB))
#define WINLIB
#endif

// Static-link library (WINLIB = _WINLIB)
#ifdef  WINLIB
#ifndef _WINLIB
#define _WINLIB
#endif
#elif   defined(_WINLIB)
#define WINLIB
#endif

// Dynamic-link library (WINDLL = _WINDLL)
#if     (defined(OLEINPROCREG) && !defined(OLEINPROC))
#define OLEINPROC
#endif
#if     ((defined(CTRLPANEL) || defined(OLEINPROC) || defined(OLEMSH)) && !defined(WINDLL))
#define WINDLL
#endif
#ifdef  WINDLL
#ifndef _WINDLL
#define _WINDLL
#endif
#elif   defined(_WINDLL)
#define WINDLL
#endif

// Only one library version is allowed
#if    (defined(WINLIB) && defined(WINDLL))
#error WinAfx - Only one library version is allowed.
#endif

// Unicode character set (UNICODE = _UNICODE)
#ifdef  UNICODE
#ifndef _UNICODE
#define _UNICODE
#endif
#elif   defined(_UNICODE)
#define UNICODE
#endif

// Multibyte character set (MBCS = _MBCS)
#ifdef  MBCS
#ifndef _MBCS
#define _MBCS
#endif
#elif   defined(_MBCS)
#define MBCS
#endif

// Single-byte character set (SBCS = _SBCS, default setting)
#if     (!defined(UNICODE) && !defined(MBCS) && !defined(SBCS))
#define SBCS
#endif
#ifdef  SBCS
#ifndef _SBCS
#define _SBCS
#endif
#elif   defined(_SBCS)
#define SBCS
#endif

// Only one character set is allowed
#if    (defined(UNICODE) + defined(MBCS) + defined(SBCS) != 1)
#error WinAfx - Only one character set is allowed.
#endif

// Debug version (DEBUG = _DEBUG)
#ifdef  DEBUG
#ifndef _DEBUG
#define _DEBUG
#endif
#elif   defined(_DEBUG)
#define DEBUG
#endif

// Release version (NDEBUG = _NDEBUG, default setting)
#if     (!defined(DEBUG) && !defined(NDEBUG))
#define NDEBUG
#endif
#ifdef  NDEBUG
#ifndef _NDEBUG
#define _NDEBUG
#endif
#elif   defined(_NDEBUG)
#define NDEBUG
#endif

// Only one project version is allowed
#if    (defined(DEBUG) && defined(NDEBUG))
#error WinAfx - Only one project version is allowed.
#endif

// Major and minor versions of the compiler (MSC_VER = _MSC_VER, automatically defined)
#define MSC_VER _MSC_VER

// Multithreaded application (MT = _MT, automatically defined)
#ifdef  MT
#ifndef _MT
#define _MT
#endif
#elif   defined(_MT)
#define MT
#endif

// Enables type-safe checking (STRICT = _STRICT, always defined)
#ifndef STRICT
#define STRICT
#endif
#ifndef _STRICT
#define _STRICT
#endif

// Displays the building version
#ifdef  DEBUG
#define _szMode "Debug"
#else
#define _szMode "Release"
#endif
#ifdef  UNICODE
#define _szCS "Unicode"
#elif   defined(MBCS)
#define _szCS "MBCS"
#else
#define _szCS "SBCS"
#endif
#ifdef  MT
#define _szThd "Multithreaded"
#else
#define _szThd "Single-threaded"
#endif
#ifdef  WINAFX
#define _szType "WinAfx library"
#elif   defined(WINLIB)
#define _szType "Static-link library"
#elif   defined(WINDLL)
#define _szType "Dynamic-link library"
#else
#define _szType "Application"
#endif
#pragma message("Building the " _szMode ", " _szCS ", " _szThd " version for " _szType "...")
#undef  _szMode
#undef  _szCS
#undef  _szThd
#undef  _szType

// Marshaling proxy and stub
#ifdef  OLEMSH
#pragma comment(lib, "RpcRt4.lib")
#else

// Import library for WinAfx
#ifndef WINAFX
#ifdef  DEBUG
#pragma comment(lib, "dWinAfx.lib")
#else
#pragma comment(lib, "rWinAfx.lib")
#endif
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Main header file (excluding rarely-used stuff) for Windows applications

#ifdef   _WINDOWS_
#error   WinAfx - Header file <Windows.h> must be included internally only.
#else
#if      (defined(RC_INVOKED) && defined(APSTUDIO_INVOKED) && !defined(APSTUDIO_HIDDEN_SYMBOLS))
#define  APSTUDIO_HIDDEN_SYMBOLS
#endif
#ifndef  WIN32_LEAN_AND_MEAN
#define  WIN32_LEAN_AND_MEAN
#endif
#ifndef  OEMRESOURCE
#define  OEMRESOURCE
#endif
#define  HFILE _HFILE_
#include <Windows.h>
#if      (defined(RC_INVOKED) && defined(APSTUDIO_INVOKED))
#undef   APSTUDIO_HIDDEN_SYMBOLS
#endif
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Additional resource identifiers

// Standard identifiers used by message and dialog boxes
#define ID_ABORT    IDABORT
#define ID_CANCEL   IDCANCEL
#define ID_CLOSE    IDCLOSE
#define ID_CONTINUE IDCONTINUE
#define ID_HELP     IDHELP
#define ID_IGNORE   IDIGNORE
#define ID_NO       IDNO
#define ID_OK       IDOK
#define ID_RETRY    IDRETRY
#define ID_TRYAGAIN IDTRYAGAIN
#define ID_YES      IDYES

// Any static window or control
#ifdef  RC_INVOKED
#define IDC_STATIC (-1)
#else
#define IDC_STATIC MAX_RESID
#endif
#define IDC_NULL   IDC_STATIC

////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef RC_INVOKED  // Not resource compiled

// Data type constants
#ifndef  _INC_LIMITS
#include <Limits.h>
#endif

// Standard I/O routines
#ifndef  _INC_STDIO
#include <StdIO.h>
#endif

// Standard library
#ifndef  _INC_STDLIB
#include <StdLib.h>
#endif

// Variable argument
#ifndef  _INC_STDARG
#include <StdArg.h>
#endif

// Floating point values
#ifndef  _INC_FLOAT
#include <Float.h>
#endif

// Memory allocation
#if      (MSC_VER < 1300)
#ifndef  _INC_MALLOC
#include <MAlloc.h>
#endif
#endif

// CRT debugging features
#ifdef   _INC_CRTDBG
#ifndef  _CRTDBG_MAP_ALLOC
#error   WinAfx - Header file <CrtDbg.h> must be included internally only.
#endif
#else
#ifndef  _CRTDBG_MAP_ALLOC
#define  _CRTDBG_MAP_ALLOC
#endif
#include <CrtDbg.h>
#endif

// Memory allocation
#if      (MSC_VER >= 1300)
#ifndef  _INC_MALLOC
#include <MAlloc.h>
#endif
#endif

// Mathematical
#if      (defined(MATH) && !defined(_INC_MATH))
#include <Math.h>
#endif

// Process and thread control
#if      (defined(MT) && !defined(_INC_PROCESS))
#include <Process.h>
#endif

// Generic international text
#ifndef  _INC_TCHAR
#include <TChar.h>
#ifndef _ttof
#define _ttof _tstof
#endif
#endif

// Character conversion and string manipulation
#ifdef   MBCS
#ifndef  _INC_MBCTYPE
#include <MbcType.h>
#endif
#ifndef  _INC_MBSTRING
#include <MbString.h>
#endif
#endif

// OLE features
#if      ((defined(MSHTML) || defined(OLECTRL) || defined(OLEDLG) || defined(OLEEXDISP) || \
         defined(OLEINPROC)) && !defined(INCOLE))
#define  INCOLE
#endif
#ifdef   INCOLE
#ifndef  INC_OLE2
#define  INC_OLE2
#endif
#ifndef  WIN32_DCOM
#define  WIN32_DCOM
#endif
#ifndef  _WIN32_DCOM
#define  _WIN32_DCOM
#endif
#ifndef  _OLE2_H_
#include <Ole2.h>
#pragma  comment(lib, "Ole32.lib")
#endif
#ifdef   MSHTML
#define  OLECTRL
#define  OLEEXDISP
#ifndef  __mshtml_h__
#include <MsHtml.h>
#endif
#ifndef  __mshtmhst_h__
#include <MsHtmHst.h>
#endif
#endif
#if      (defined(OLECTRL) && !defined(_OLECTL_H_))
#include <OleCtl.h>
#pragma  comment(lib, "OleAut32.lib")
#endif
#if      (defined(OLEDLG) && !defined(_OLEDLG_H_))
#include <OleDlg.h>
#pragma  comment(lib, "OleDlg.lib")
#endif
#if      (defined(OLEEXDISP) && !defined(__exdisp_h__))
#include <ExDisp.h>
#endif
#endif

// Common controls
#ifdef   COMMCTRL
#ifndef  PRECTRL
#define  PRECTRL
#endif
#ifndef  __objidl_h__
#include <ObjIdl.h>
#endif
#ifndef  _INC_COMMCTRL
#include <CommCtrl.h>
#pragma  comment(lib, "ComCtl32.lib")
#endif
#endif

// Common dialog boxes
#ifdef   COMMDLG
#ifndef  _INC_COMMDLG
#include <CommDlg.h>
#pragma  comment(lib, "ComDlg32.lib")
#endif
#ifndef  _INC_CDERR
#include <CDErr.h>
#endif
#endif

// Control Panel
#if      (defined(CTRLPANEL) && !defined(_INC_CPL))
#include <CPl.h>
#endif

// Dynamic data exchange
#if      (defined(DDE) && !defined(_DDEHEADER_INCLUDED_))
#include <DDE.h>
#endif
#if      (defined(DDEML) && !defined(_INC_DDEMLH))
#include <DDEML.h>
#endif

// DirectX features
#if     (defined(D3D) && !defined(DIRECTX))
#define DIRECTX
#endif

// Direct3D components
#if      (defined(D3D) && !defined(__D3DX9_H__))
#ifdef   DEBUG
#ifndef  D3D_DEBUG_INFO
#define  D3D_DEBUG_INFO
#endif
#define  _szD3dX9Lib "D3dX9d.lib"
#else
#define  _szD3dX9Lib "D3dX9.lib"
#endif
#include <D3dX9.h>
#pragma  comment(lib, "D3d9.lib")
#pragma  comment(lib, _szD3dX9Lib)
#undef   _szD3dX9Lib
#endif

// GDI+ functions and classes
#if      (defined(GDIPLUS) && !defined(_GDIPLUS_H))
#include <gdiplus.h>
#pragma  comment(lib, "GdiPlus.lib")
#endif

// Multimedia system
#if      (defined(MMSYSTEM) && !defined(_INC_MMSYSTEM))
#include <MmSystem.h>
#pragma  comment(lib, "WinMm.lib")
#endif

// Rich edit control
#ifdef   RICHEDIT
#ifndef  PRECTRL
#define  PRECTRL
#endif
#ifndef  _RICHEDIT_
#include <RichEdit.h>
#endif
#if      (defined(INCOLE) && !defined(_RICHOLE_))
#include <RichOle.h>
#endif
#endif

// Screen saver
#if      (defined(SCRNSAVE) && !defined(_INC_SCRNSAVE))
#include <ScrnSave.h>
#ifdef   UNICODE
#pragma  comment(lib, "ScrnSavW.lib")
#else
#pragma  comment(lib, "ScrnSave.lib")
#endif
#endif

// Shell utilities
#ifdef   SHLAPI
#ifndef  _INC_SHELLAPI
#include <ShellApi.h>
#endif
#ifndef  _SHLOBJ_H_
#include <ShlObj.h>
#endif
#pragma  comment(lib, "Shell32.lib")
#endif

// Shell lightweight utilities
#ifdef   SHLWAPI
#ifndef  _INC_SHLWAPI
#include <ShlWApi.h>
#endif
#pragma  comment(lib, "ShlWApi.lib")
#endif

// Windows Internet
#if      (defined(WININET) && !defined(_WININET_))
#include <WinINet.h>
#pragma  comment(lib, "WinINet.lib")
#endif

// Windows Sockets
#ifdef   WINSOCK
#ifndef  _WINSOCK2API_
#include <WinSock2.h>
#pragma  comment(lib, "WS2_32.lib")
#endif
#ifndef  _MSWSOCK_
#include <MsWsock.h>
#pragma  comment(lib, "MsWSock.lib")
#endif
#endif

// Windows printer spooler
#if      (defined(WINSPOOL) && !defined(_WINSPOOL_))
#include <WinSpool.h>
#pragma  comment(lib, "WinSpool.lib")
#endif

// Windows theme
#if      (defined(WINTHEME) && !defined(_UXTHEME_H_))
#include <UxTheme.h>
#pragma  comment(lib, "UxTheme.lib")
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// General declarations, definitions and functions

#pragma  warning(disable : 4060)
#pragma  warning(disable : 4355)
#pragma  warning(disable : 4996)
#ifndef  _TYPEDEF_INCLUDED_
#include <TypeDef.hpp>
#endif
#ifndef  _DEBUG_INCLUDED_
#include <Debug.hpp>
#endif
#ifndef  _GENERAL_INCLUDED_
#include <General.hpp>
#endif
#ifndef  _MSGMAP_INCLUDED_
#include <MsgMap.hpp>
#endif
#if      (defined(DDE) && !defined(_DDE_INCLUDED_))
#include <DDE.hpp>
#endif
#if      (defined(DDEML) && !defined(_DDEML_INCLUDED_))
#include <DDEML.hpp>
#endif
#if      (defined(DIRECTX) && !defined(_DIRECTX_INCLUDED_))
#include <DirectX.hpp>
#endif
#if      (defined(D3D) && !defined(_D3D_INCLUDED_))
#include <D3D.hpp>
#endif
#if      (defined(MMSYSTEM) && !defined(_MMSYSTEM_INCLUDED_))
#include <MmSystem.hpp>
#endif
#if      (defined(INCOLE) && !defined(_OLEOBJ_INCLUDED_))
#include <OleObj.hpp>
#endif
#if      (defined(WININET) && !defined(_WININET_INCLUDED_))
#include <WinINet.hpp>
#endif
#if      (defined(WINSOCK) && !defined(_WINSOCK_INCLUDED_))
#include <WinSock.hpp>
#endif
#if      (defined(WINSPOOL) && !defined(_WINSPOOL_INCLUDED_))
#include <WinSpool.hpp>
#endif
#if      (defined(WINTHEME) && !defined(_WINTHEME_INCLUDED_))
#include <WinTheme.hpp>
#endif
#ifndef  _WINDOW_INCLUDED_
#include <Window.hpp>
#endif
#if      (defined(PRECTRL) && !defined(_PRECTRL_INCLUDED_))
#include <PreCtrl.hpp>
#endif
#if      (defined(COMMCTRL) && !defined(_COMMCTRL_INCLUDED_))
#include <CommCtrl.hpp>
#endif
#if      (defined(COMMDLG) && !defined(_COMMDLG_INCLUDED_))
#include <CommDlg.hpp>
#endif
#if      (defined(CTRLPANEL) && !defined(_CPL_INCLUDED_))
#include <CPl.hpp>
#endif
#if      (defined(OLEDLG) && !defined(_OLEDLG_INCLUDED_))
#include <OleDlg.hpp>
#endif
#if      (defined(RICHEDIT) && !defined(_RICHEDIT_INCLUDED_))
#include <RichEdit.hpp>
#endif
#if      (defined(SCRNSAVE) && !defined(_SCRNSAVE_INCLUDED_))
#include <ScrnSave.hpp>
#endif
#if      (defined(SHLAPI) && !defined(_SHLAPI_INCLUDED_))
#include <ShlApi.hpp>
#endif

// End of RC_INVOKED, OLEMSH and _WINAFX_INCLUDED_
#endif
#endif
#endif
