////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// WinINet.hpp - Windows Internet Declaration File                                            //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _WININET_INCLUDED_
#define _WININET_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  WinINet - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Handles
typedef HINTERNET HFTP;
typedef HINTERNET HFTPFIND;
typedef HINTERNET HGOPHER;
typedef HINTERNET HHTTP;

////////////////////////////////////////////////////////////////////////////////////////////////
// General functions

// Searches the directory of an FTP session
// Returns: A valid handle if the function succeeds
AFXINLINE HFTPFIND WINAPI FtpFindFirstFile(HFTP hFtp, PCTSTR pszPathSpec,
	PWIN32_FIND_DATA pWFD, DWORD dwFlags = UNUSED)
{
	HFTPFIND hFtpFind;
	hFtpFind = FtpFindFirstFile(hFtp, pszPathSpec, pWFD, dwFlags, UNUSED);
	ASSERT(hFtpFind || (GetLastError() == ERROR_NO_MORE_FILES));
	return hFtpFind;
}

AFXINLINE HFTPFIND WINAPI FtpFindFirstFile(HFTP hFtp, PWIN32_FIND_DATA pWFD,
	DWORD dwFlags = UNUSED, DWORD dwContext = UNUSED)
{
	HFTPFIND hFtpFind;
	hFtpFind = FtpFindFirstFile(hFtp, NULL, pWFD, dwFlags, dwContext);
	ASSERT(hFtpFind || (GetLastError() == ERROR_NO_MORE_FILES));
	return hFtpFind;
}

// Gets a file from an FTP server
// Returns: TRUE if the function succeeds
AFXINLINE BOOL WINAPI FtpGetFile(HFTP hFtp, PCTSTR pszRemoteFile, PCTSTR pszNewFile,
	BOOL bFailExist, DWORD dwFileAttrs, DWORD dwFlags)
{
	return VERIFY(FtpGetFile(hFtp, pszRemoteFile, pszNewFile,
		bFailExist, dwFileAttrs, dwFlags, UNUSED));
}

// Creates an HTTP request session
// Returns: A handle to a request session
AFXINLINE HHTTP WINAPI HttpOpenRequest(HHTTP hHttp, PCTSTR pszVerb, PCTSTR pszObjectName,
	DWORD dwFlags = INTERNET_FLAG_DONT_CACHE)
{
	return (HHTTP)VERIFY(HttpOpenRequest(hHttp, pszVerb, pszObjectName, NULL, NULL, NULL,
		dwFlags, UNUSED));
}

AFXINLINE HHTTP WINAPI HttpOpenRequest(HHTTP hHttp, PCTSTR pszObjectName,
	DWORD dwFlags = INTERNET_FLAG_DONT_CACHE)
{
	return (HHTTP)VERIFY(HttpOpenRequest(hHttp, NULL, pszObjectName, NULL, NULL, NULL,
		dwFlags, UNUSED));
}

// Sends a request to an HTTP server
// Returns: TRUE if the function succeeds
AFXINLINE BOOL WINAPI HttpSendRequest(HHTTP hRequest, PCTSTR pszHeaders,
	PVOID pvOptData = NULL, DWORD cbOptData = UNUSED)
{
	return VERIFY(HttpSendRequest(hRequest, pszHeaders, -1, pvOptData, cbOptData));
}

AFXINLINE BOOL WINAPI HttpSendRequest(HHTTP hRequest)
{
	return VERIFY(HttpSendRequest(hRequest, NULL, UNUSED, NULL, UNUSED));
}

// Attempts to make a connection to the internet
// Returns: NO_ERROR if the function succeeds
AFXINLINE DWORD WINAPI InternetAttemptConnect()
{
	return InternetAttemptConnect(UNUSED);
}

// Checks if a connection to the internet can be established
// Returns: TRUE if a connection is made successfully
AFXINLINE BOOL WINAPI InternetCheckConnection(PCTSTR pszUrl = NULL)
{
	return InternetCheckConnection(pszUrl, FLAG_ICC_FORCE_CONNECTION, UNUSED);
}

// Connects an FTP or HTTP session for a given site
// Returns: A handle to an FTP or HTTP session
AFXINLINE HINTERNET WINAPI InternetConnect(HINTERNET hInternet, PCTSTR pszServerName,
	INTERNET_PORT ipServer, PCTSTR pszUserName, PCTSTR pszPassword, DWORD dwService)
{
	return (HINTERNET)VERIFY(InternetConnect(hInternet, pszServerName,
		ipServer, pszUserName, pszPassword, dwService, UNUSED, UNUSED));
}

AFXINLINE HINTERNET WINAPI InternetConnect(HINTERNET hInternet, PCTSTR pszServerName,
	INTERNET_PORT ipServer, DWORD dwService, DWORD dwFlags = UNUSED, DWORD dwContext = UNUSED)
{
	return (HINTERNET)VERIFY(InternetConnect(hInternet, pszServerName,
		ipServer, NULL, NULL, dwService, dwFlags, dwContext));
}

// Initializes the Windows internet library
// Returns: A generic resulting handle
AFXINLINE HINTERNET WINAPI InternetOpen(PCTSTR pszAgent, DWORD dwAccess, DWORD dwFlags = UNUSED)
{
	return (HINTERNET)VERIFY(InternetOpen(pszAgent, dwAccess, NULL, NULL, dwFlags));
}

// Opens a resource by a complete FTP or HTTP URL
// Returns: A handle to an FTP or HTTP session
AFXINLINE HINTERNET WINAPI InternetOpenUrl(HINTERNET hInternet, PCTSTR pszUrl,
	DWORD dwFlags = UNUSED)
{
	return (HINTERNET)VERIFY(InternetOpenUrl(hInternet, pszUrl, NULL, UNUSED, dwFlags, UNUSED));
}

// End of _WININET_INCLUDED_
#endif
