////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// WinSock.hpp - Windows Sockets Definitions File                                             //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _WINSOCK_INCLUDED_
#define _WINSOCK_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  WinSock - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// Constant definitions

// Network notification events
#define FD_ALC FD_ADDRESS_LIST_CHANGE
#define FD_ALL FD_ALL_EVENTS
#define FD_RIC FD_ROUTING_INTERFACE_CHANGE

// Array indexes for associated error codes of network notification events
#define FDI_ACCEPT  FD_ACCEPT_BIT
#define FDI_ALC     FD_ADDRESS_LIST_CHANGE_BIT
#define FDI_CLOSE   FD_CLOSE_BIT
#define FDI_CONNECT FD_CONNECT_BIT
#define FDI_OOB     FD_OOB_BIT
#define FDI_QOS     FD_QOS_BIT
#define FDI_READ    FD_READ_BIT
#define FDI_RIC     FD_ROUTING_INTERFACE_CHANGE_BIT
#define FDI_WRITE   FD_WRITE_BIT

// Unspecifies a port number
#define IPPORT_UNSPEC 0

// Protocol names
#define IPPN_TCP    _T("TCP")
#define IPPN_UDP    _T("UDP")
#define IPPN_UNSPEC NULL

// Service names
#define IPSN_AUTH       _T("AUTH")        // TCP     - Port 113
#define IPSN_BIFF       _T("BIFF")        // UDP     - Port 512
#define IPSN_BOOTP      _T("BOOTP")       // UDP     - Port  67
#define IPSN_CONFERENCE _T("CONFERENCE")  // TCP/UDP - Port 531
#define IPSN_COURIER    _T("COURIER")     // TCP/UDP - Port 530
#define IPSN_DAYTIME    _T("DAYTIME")     // TCP/UDP - Port  13
#define IPSN_DISCARD    _T("DISCARD")     // TCP/UDP - Port   9
#define IPSN_DOMAIN     _T("DOMAIN")      // TCP/UDP - Port  53
#define IPSN_ECHO       _T("ECHO")        // TCP/UDP - Port   7
#define IPSN_EFS        _T("EFS")         // TCP     - Port 520
#define IPSN_ELCSD      _T("ELCSD")       // TCP/UDP - Port 704
#define IPSN_EXEC       _T("EXEC")        // TCP     - Port 512
#define IPSN_FINGER     _T("FINGER")      // TCP/UDP - Port  42, TCP - Port 79
#define IPSN_FTP        _T("FTP")         // TCP     - Port  21
#define IPSN_FTPDATA    _T("FTP-DATA")    // TCP     - Port  20
#define IPSN_HOSTNAMES  _T("HOSTNAMES")   // TCP     - Port 101
#define IPSN_KLOGIN     _T("KLOGIN")      // TCP/UDP - Port 543
#define IPSN_KSHELL     _T("KSHELL")      // TCP/UDP - Port 544
#define IPSN_LOGIN      _T("LOGIN")       // TCP     - Port 513
#define IPSN_MONITOR    _T("MONITOR")     // TCP/UDP - Port 561
#define IPSN_MTP        _T("MTP")         // TCP     - Port  57
#define IPSN_NAMESERVER _T("NAMESERVER")  // TCP/UDP - Port  53
#define IPSN_NETNEWS    _T("NETNEWS")     // TCP/UDP - Port 532
#define IPSN_NETWALL    _T("NETWALL")     // TCP/UDP - Port 533
#define IPSN_NEWRWHO    _T("NEW-RWHO")    // UDP     - Port 550
#define IPSN_NNTP       _T("NNTP")        // TCP     - Port 119
#define IPSN_NTALK      _T("NTALK")       // TCP/UDP - Port 518
#define IPSN_NTP        _T("NTP")         // UDP     - Port 123
#define IPSN_POP2       _T("POP2")        // TCP     - Port 109
#define IPSN_POP3       _T("POP3")        // TCP     - Port 110
#define IPSN_PORTMAP    _T("PORTMAP")     // TCP/UDP - Port 111
#define IPSN_PRINTER    _T("PRINTER")     // TCP/UDP - Port 515
#define IPSN_QOTD       _T("QOTD")        // TCP/UDP - Port  17
#define IPSN_REMOTEFS   _T("REMOTEFS")    // TCP/UDP - Port 556
#define IPSN_RLP        _T("RLP")         // UDP     - Port  39
#define IPSN_RMONITOR   _T("RMONITOR")    // TCP/UDP - Port 560
#define IPSN_ROUTE      _T("ROUTE")       // UDP     - Port 520
#define IPSN_SFTP       _T("SFTP")        // TCP     - Port 115
#define IPSN_SGMP       _T("SGMP")        // UDP     - Port 153
#define IPSN_SHELL      _T("SHELL")       // TCP     - Port 514
#define IPSN_SMTP       _T("SMTP")        // TCP     - Port  25
#define IPSN_SNMP       _T("SNMP")        // UDP     - Port 161
#define IPSN_SNMPTRAP   _T("SNMP-TRAP")   // UDP     - Port 167
#define IPSN_SUNRPC     _T("SUNRPC")      // TCP/UDP - Port 111
#define IPSN_SUPDUP     _T("SUPDUP")      // TCP     - Port  95
#define IPSN_TALK       _T("TALK")        // TCP/UDP - Port 517
#define IPSN_TCPREPO    _T("TCPREPO")     // TCP     - Port 158
#define IPSN_TELNET     _T("TELNET")      // TCP     - Port  23
#define IPSN_TEMPO      _T("TEMPO")       // TCP/UDP - Port 526
#define IPSN_TFTP       _T("TFTP")        // UDP     - Port  69
#define IPSN_TIME       _T("TIME")        // TCP/UDP - Port  37
#define IPSN_TIMED      _T("TIMED")       // TCP/UDP - Port 525
#define IPSN_UUCP       _T("UUCP")        // TCP/UDP - Port 540
#define IPSN_UUCPPATH   _T("UUCP-PATH")   // TCP     - Port 117
#define IPSN_WHO        _T("WHO")         // UDP     - Port 513
#define IPSN_WHOIS      _T("WHOIS")       // TCP     - Port  43

// Socket operation is OK
#define SOCKET_OK 0

////////////////////////////////////////////////////////////////////////////////////////////////
// Extended structures for database operations

struct HOSTENTEX : public HOSTENT
{
protected:
	char _cPadding[MAXGETHOSTSTRUCT - sizeof(HOSTENT)];
};

struct PROTOENTEX : public PROTOENT
{
protected:
	char _cPadding[MAXGETHOSTSTRUCT - sizeof(PROTOENT)];
};

struct SERVENTEX : public SERVENT
{
protected:
	char _cPadding[MAXGETHOSTSTRUCT - sizeof(SERVENT)];
};

////////////////////////////////////////////////////////////////////////////////////////////////
// Custom data types

// Handles
typedef SOCKET   HSOCKET;
typedef WSAEVENT HWSAEVENT;
typedef HANDLE   HWSATASK;

// Pointers
typedef HOSTENTEX*         PHOSTENTEX;
typedef PROTOENTEX*        PPROTOENTEX;
typedef SERVENTEX*         PSERVENTEX;
typedef WSAPROTOCOL_INFO*  PWSAPROTOCOL_INFO;
typedef WSABUF*            PWSABUF;
typedef WSADATA*           PWSADATA;
typedef WSAOVERLAPPED*     PWSAOVERLAPPED;
typedef const SOCKADDR*    PCSOCKADDR;
typedef const SOCKADDR_IN* PCSOCKADDR_IN;

// Overlapped completion routine
typedef void (CALLBACK* PWSAOCR)(DWORD, DWORD, PWSAOVERLAPPED, DWORD);

////////////////////////////////////////////////////////////////////////////////////////////////
// Network communication maps

#define ON_WSM_ASYNCTASK(nMsg, fName) \
	case nMsg: \
		pWnd->fName((HWSATASK)wParam, WSAGETASYNCERROR(lParam)); \
		return _eResult;

#define ON_WSM_EVENT(nMsg, fName) \
	case nMsg: \
		pWnd->fName(wParam, WSAGETSELECTEVENT(lParam), WSAGETSELECTERROR(lParam)); \
		return _eResult;

#define BEGIN_SOCKET_EVENT_MAP(nMsg) \
	case nMsg: \
		iError = WSAGETSELECTERROR(lParam); \
		switch(WSAGETSELECTEVENT(lParam)) \
		{

#define ON_SOCKET_EVENT(lEvent, fName) \
		case lEvent: \
			pWnd->fName(wParam, iError); \
			return _eResult;

#define END_SOCKET_EVENT_MAP() \
		} \
		return _eResult;

////////////////////////////////////////////////////////////////////////////////////////////////
// General macros and functions

// Converts an address string (in any byte order) to an IN_ADDR structure
// Returns: A converted IN_ADDR
#define MAKEINADDR(pszAddr) (*(PIN_ADDR)pszAddr)

// Accepts an incoming connection attempt on a socket
// Returns: A new socket for the accepted packet
AFXINLINE HSOCKET WINAPI accept(HSOCKET hSocket, PSOCKADDR_IN pSAI)
{
	int     cbSAI         = sizeof(SOCKADDR_IN);
	HSOCKET hAcceptSocket = accept(hSocket, (PSOCKADDR)pSAI, &cbSAI);
	ASSERT((hAcceptSocket != INVALID_SOCKET) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return hAcceptSocket;
}

AFXINLINE HSOCKET WINAPI accept(HSOCKET hSocket)
{
	HSOCKET hAcceptSocket = accept(hSocket, NULL, NULL);
	ASSERT((hAcceptSocket != INVALID_SOCKET) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return hAcceptSocket;
}

// Associates a local address with a socket
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI bind(HSOCKET hSocket, PCSOCKADDR_IN pSAI)
{
	return UNEQUAL(bind(hSocket, (PCSOCKADDR)pSAI, sizeof(SOCKADDR)), SOCKET_ERROR);
}

// Establishes a connection to a peer
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI connect(HSOCKET hSocket, PCSOCKADDR_IN pSAI)
{
	int iResult = connect(hSocket, (PCSOCKADDR)pSAI, sizeof(SOCKADDR));
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return iResult;
}

// Gets host information corresponding to an address
// Returns : A pointer to a HOSTENT structure
AFXINLINE PHOSTENT WINAPI gethostbyaddr(PULONG pulAddr, int iAddr)
{
	return (PHOSTENT)VERIFY(gethostbyaddr((PCCHAR)pulAddr, sizeof(ULONG), iAddr));
}

// Gets the name of the peer to which a socket is connected
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI getpeername(HSOCKET hSocket, PSOCKADDR_IN pSAI)
{
	int cbSAI = sizeof(SOCKADDR_IN);
	return UNEQUAL(getpeername(hSocket, (PSOCKADDR)pSAI, &cbSAI), SOCKET_ERROR);
}

// Gets service information corresponding to a service name
// Returns: A pointer to a SERVENT structure
AFXINLINE PSERVENT WINAPI getservbyname(PCSTR pszService)
{
	return (PSERVENT)VERIFY(getservbyname(pszService, NULL));
}

// Gets service information corresponding to a port number
// Returns: A pointer to a SERVENT structure
AFXINLINE PSERVENT WINAPI getservbyport(int iPort)
{
	return (PSERVENT)VERIFY(getservbyport(iPort, NULL));
}

// Gets the local name for a socket
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI getsockname(HSOCKET hSocket, PSOCKADDR_IN pSAI)
{
	int cbSAI = sizeof(SOCKADDR_IN);
	return UNEQUAL(getsockname(hSocket, (PSOCKADDR)pSAI, &cbSAI), SOCKET_ERROR);
}

// Gets/sets a socket option
// Returns: SOCKET_OK if the function succeeds
template<typename tDataType>
AFXINLINE int WINAPI getsockopt(HSOCKET hSocket, int iLevel, int iOption, tDataType* tpBuffer)
{
	int cbBuffer = sizeof(tDataType);
	return UNEQUAL(getsockopt(hSocket, iLevel, iOption, (PCHAR)tpBuffer, &cbBuffer),
		SOCKET_ERROR);
}

template<typename tDataType>
AFXINLINE int WINAPI setsockopt(HSOCKET hSocket, int iLevel, int iOption, tDataType* tpData)
{
	int cbData = sizeof(tDataType);
	return UNEQUAL(setsockopt(hSocket, iLevel, iOption, (PCCHAR)tpData, &cbData), SOCKET_ERROR);
}

// Receives/sends data from/to a connected socket
// Returns: Number of bytes received/sent/SOCKET_OK if the function succeeds
AFXINLINE int WINAPI recv(HSOCKET hSocket, PVOID pvBuffer, int cbBuffer, int iFlags = UNUSED)
{
	int cbReceived = recv(hSocket, (PCHAR)pvBuffer, cbBuffer, iFlags);
	ASSERT((cbReceived != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbReceived;
}

AFXINLINE int WINAPI WSARecv(HSOCKET hSocket, PWSABUF paWSAB, DWORD ceWSAB, PDWORD pcbReceived,
	PDWORD pdwFlags, PWSAOVERLAPPED pWSAOL = NULL)
{
	int iResult = WSARecv(hSocket, paWSAB, ceWSAB, pcbReceived, pdwFlags, pWSAOL, NULL);
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSA_IO_PENDING));
	return iResult;
}

AFXINLINE int WINAPI WSARecv(HSOCKET hSocket, PVOID pvBuffer, int cbBuffer, PINT piFlags)
{
	int iResult = WSARecvEx(hSocket, (PCHAR)pvBuffer, cbBuffer, piFlags);
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return iResult;
}

AFXINLINE int WINAPI send(HSOCKET hSocket, PCVOID pvData, int cbData, int iFlags = UNUSED)
{
	int cbSent = send(hSocket, (PCCHAR)pvData, cbData, iFlags);
	ASSERT((cbSent != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbSent;
}

AFXINLINE int WINAPI WSASend(HSOCKET hSocket, PWSABUF paWSAB, DWORD ceWSAB, PDWORD pcbSent,
	DWORD dwFlags, PWSAOVERLAPPED pWSAOL = NULL)
{
	int iResult = WSASend(hSocket, paWSAB, ceWSAB, pcbSent, dwFlags, pWSAOL, NULL);
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSA_IO_PENDING));
	return iResult;
}

// Receives a datagram from/to an address
// Returns: Number of bytes received/sent/SOCKET_OK if the function succeeds
AFXINLINE int WINAPI recvfrom(HSOCKET hSocket, PVOID pvBuffer, int cbBuffer, int iFlags,
	PSOCKADDR_IN pSAI)
{
	int cbSAI      = sizeof(SOCKADDR_IN);
	int cbReceived = recvfrom(hSocket, (PCHAR)pvBuffer, cbBuffer, iFlags,
		(PSOCKADDR)pSAI, &cbSAI);
	ASSERT((cbReceived != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbReceived;
}

AFXINLINE int WINAPI recvfrom(HSOCKET hSocket, PVOID pvBuffer, int cbBuffer, PSOCKADDR_IN pSAI)
{
	int cbSAI      = sizeof(SOCKADDR_IN);
	int cbReceived = recvfrom(hSocket, (PCHAR)pvBuffer, cbBuffer, UNUSED,
		(PSOCKADDR)pSAI, &cbSAI);
	ASSERT((cbReceived != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbReceived;
}

AFXINLINE int WINAPI WSARecvFrom(HSOCKET hSocket, PWSABUF paWSAB, DWORD ceWSAB,
	PDWORD pcbReceived, PDWORD pdwFlags, PSOCKADDR_IN pSAI, int cbSAI,
	PWSAOVERLAPPED pWSAOL = NULL, PWSAOCR pWSAOCR = NULL)
{
	int iResult = WSARecvFrom(hSocket, paWSAB, ceWSAB, pcbReceived, pdwFlags,
		(PSOCKADDR)pSAI, &cbSAI, pWSAOL, pWSAOCR);
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSA_IO_PENDING));
	return iResult;
}

AFXINLINE int WINAPI sendto(HSOCKET hSocket, PCVOID pvData, int cbData, int iFlags,
	PCSOCKADDR_IN pSAI)
{
	int cbSent = sendto(hSocket, (PCCHAR)pvData, cbData, iFlags,
		(PCSOCKADDR)pSAI, sizeof(SOCKADDR));
	ASSERT((cbSent != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbSent;
}

AFXINLINE int WINAPI sendto(HSOCKET hSocket, PCVOID pvData, int cbData, PCSOCKADDR_IN pSAI)
{
	int cbSent = sendto(hSocket, (PCCHAR)pvData, cbData, UNUSED,
		(PCSOCKADDR)pSAI, sizeof(SOCKADDR));
	ASSERT((cbSent != SOCKET_ERROR) || (WSAGetLastError() == WSAEWOULDBLOCK));
	return cbSent;
}

AFXINLINE int WINAPI WSASendTo(HSOCKET hSocket, PWSABUF paWSAB, DWORD ceWSAB, PDWORD pcbSent,
	DWORD dwFlags, PCSOCKADDR_IN pSAI, PWSAOVERLAPPED pWSAOL = NULL, PWSAOCR pWSAOCR = NULL)
{
	int iResult = WSASendTo(hSocket, paWSAB, ceWSAB, pcbSent, dwFlags,
		(PCSOCKADDR)pSAI, sizeof(SOCKADDR), pWSAOL, pWSAOCR);
	ASSERT((iResult != SOCKET_ERROR) || (WSAGetLastError() == WSA_IO_PENDING));
	return iResult;
}

// Converts components of SOCKADDR_IN into a readable address string
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI WSAAddressToString(PSOCKADDR_IN pSAI, PWSAPROTOCOL_INFO pPI,
	PTSTR pszBuffer, PDWORD pccBuffer)
{
	return UNEQUAL(WSAAddressToString((PSOCKADDR)pSAI, sizeof(SOCKADDR), pPI,
		pszBuffer, pccBuffer), SOCKET_ERROR);
}

AFXINLINE int WINAPI WSAAddressToString(PSOCKADDR_IN pSAI, PTSTR pszBuffer, PDWORD pccBuffer)
{
	return UNEQUAL(WSAAddressToString((PSOCKADDR)pSAI, sizeof(SOCKADDR), NULL,
		pszBuffer, pccBuffer), SOCKET_ERROR);
}

// Removes messages associated with a socket/task (used internally only)
void WINAPI _WSARemoveMessage(HWND hWnd, UINT nMsg, HSOCKET hSocket);

// Removes messages associated with a socket/task
AFXINLINE void WINAPI WSARemoveMessage(UINT nMsg, HWSATASK hWsaTask)
{
	_WSARemoveMessage(NULL, nMsg, (HSOCKET)hWsaTask);
}

AFXINLINE void WINAPI WSARemoveMessage(UINT nMsg, HSOCKET hSocket)
{
	_WSARemoveMessage(NULL, nMsg, hSocket);
}

AFXINLINE void WINAPI WSARemoveMessage(UINT nMsg)
{
	MSG msg;
	while(PeekMessage(&msg, NULL, nMsg, nMsg, PM_REMOVE));
}

// Creates a socket
// Returns: A handle to the socket
AFXINLINE HSOCKET WINAPI WSASocket(int iAddr, int iType, int iProtocol, PWSAPROTOCOL_INFO pPI,
	DWORD dwFlags = UNUSED)
{
	return UNEQUAL(WSASocket(iAddr, iType, iProtocol, pPI, UNUSED, dwFlags), INVALID_SOCKET);
}

AFXINLINE HSOCKET WINAPI WSASocket(int iAddr, int iType, int iProtocol, DWORD dwFlags = UNUSED)
{
	return UNEQUAL(WSASocket(iAddr, iType, iProtocol, NULL, UNUSED, dwFlags), INVALID_SOCKET);
}

// Initializes the Windows Sockets library
// Returns: SOCKET_OK if the function succeeds
AFXINLINE int WINAPI WSAStartup(PWSADATA pWSAD)
{
	return EQUAL(WSAStartup(WINSOCK_VERSION, pWSAD), SOCKET_OK);
}

// End of _WINSOCK_INCLUDED_
#endif
