////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// Window.hpp - Window Classes Declaration File                                               //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _WINDOW_INCLUDED_
#define _WINDOW_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  Window - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
// CWnd

class CWnd
{
public:
	union
	{
		// Window handle
		HWND m_hWnd;

		// For CMdiFrameWnd overriding (used internally only)
		HWND m_hClientWnd;

		// For CSubclassWnd overriding (used internally only)
		WNDPROC m_wpPrev;
	};

public:
	CWnd();
	CWnd(HWND hWnd);
	operator HWND() const;
	void operator =(HWND hWnd);
	BOOL operator ==(HWND hWnd) const;
	BOOL operator !=(HWND hWnd) const;
	BOOL AnimateWindow(DWORD dwTime, DWORD dwFlags) const;
	UINT ArrangeIconicWindows() const;
	HDC BeginPaint(PPAINTSTRUCT pPS) const;
	BOOL BringWindowToTop() const;
	WORD CascadeWindows(UINT nFlag, PCRECT pRect, UINT ceChildWnd, PCHWND phaChildWnd) const;
	void ChangeCbChain(HWND hRemoveWnd, HWND hNextWnd) const;
	BOOL ChangeClipboardChain(HWND hNextWnd) const;
	void ChangeUiState(UINT nState, UINT nElement) const;
	BOOL CheckButton(UINT nButtonId, UINT nCheck = BST_CHECKED) const;
	BOOL CheckButton(UINT nFirstId, UINT nLastId, UINT nCheckId) const;
	void Clear() const;
	BOOL ClientToScreen(PPOINT pPoint) const;
	BOOL CloseWindow(BOOL bForce = TRUE) const;
	void Copy() const;
	BOOL CreateCaret(HBITMAP hBitmap) const;
	BOOL CreateCaret(int iWidth = UNUSED, int iHeight = UNUSED) const;
	void Cut() const;
	HDWP DeferWindowPos(HDWP hDwp, HWND hPrecWnd, int x, int y, int iWidth, int iHeight,
		UINT nFlags) const;
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	BOOL DestroyWindow() const;
	BOOL DirComboBox(PTSTR pszPathSpec, UINT nComboBoxId, UINT nStaticId, UINT nAttrs) const;
	BOOL DirListBox(PTSTR pszPathSpec, UINT nListBoxId, UINT nStaticId, UINT nAttrs) const;
	BOOL DirSelComboBox(PTSTR pszBuffer, int ccBuffer, UINT nComboBoxId) const;
	BOOL DirSelListBox(PTSTR pszBuffer, int ccBuffer, UINT nListBoxId) const;
	BOOL DisableWindow() const;
	void DrawClipboard() const;
	BOOL DrawMenuBar() const;
	BOOL EnableScrollBar(UINT nBar, UINT nArrows = ESB_ENABLE_BOTH) const;
	BOOL EnableWindow(BOOL bEnable = TRUE) const;
	BOOL EndPaint(PCPAINTSTRUCT pPS) const;
	BOOL EnumChildWindows(WNDENUMPROC epName, LPARAM lParam = UNUSED) const;
	int EnumProps(PROPENUMPROCEX epName, LPARAM lParam) const;
	int EnumProps(PROPENUMPROC epName) const;
	HWND FindWindow(HWND hChildWnd, PCTSTR pszClassName = NULL, PCTSTR pszWndName = NULL) const;
	BOOL FindWindow(PCTSTR pszClassName = NULL, PCTSTR pszWndName = NULL);
	BOOL GetAltTabInfo(int iItem, PALTTABINFO pATI, PTSTR pszBuffer, UINT ccBuffer) const;
	BOOL GetAltTabInfo(PALTTABINFO pATI) const;
	HWND GetAncestor(UINT nFlag) const;
	void GetCapture();
	HWND GetChild(UINT nChildId) const;
	HWND GetChild(POINT ptClient, UINT nFlags) const;
	HWND GetChild(POINT ptClient) const;
	UINT GetChildInt(UINT nChildId, BOOL bSigned = FALSE, PBOOL pbSuccess = NULL) const;
	UINT GetChildText(UINT nChildId, PTSTR pszBuffer, int ccBuffer) const;
	DWORD GetClassLong(int iIndex) const;
	int GetClassName(PTSTR pszBuffer, int ccBuffer) const;
	WORD GetClassWord(int iIndex) const;
	XULONG GetClassXLong(int iIndex) const;
	BOOL GetClientRect(PRECT pRect) const;
	HDC GetDC(HRGN hClipRgn, DWORD dwFlags) const;
	HDC GetDC(DWORD dwFlags) const;
	HDC GetDC() const;
	HWND GetFirstChild() const;
	void GetForegroundWindow();
	HWND GetLastActivePopup() const;
	HMENU GetMenu() const;
	BOOL GetMenuBarInfo(LONG lObjectId, LONG lItemId, PMENUBARINFO pMBI) const;
	BOOL GetMessage(PMSG pMsg, UINT nMinMsg, UINT nMaxMsg) const;
	BOOL GetMessage(PMSG pMsg, UINT nMsg = UNUSED) const;
	HWND GetNextGroupChild(HWND hChildWnd, BOOL bPrevCtrl = FALSE) const;
	HWND GetNextGroupChild(BOOL bPrevCtrl = FALSE) const;
	HWND GetNextTabChild(HWND hChildWnd, BOOL bPrevCtrl = FALSE) const;
	HWND GetNextTabChild(BOOL bPrevCtrl = FALSE) const;
	HWND GetParent() const;
	HANDLE GetProp(PCTSTR pszEntry) const;
	HANDLE GetProp(ATOM aEntry) const;
	HWND GetRealChild(POINT ptClient) const;
	BOOL GetScrollInfo(LONG lBar, PSCROLLBARINFO pSBI) const;
	BOOL GetScrollInfo(int iBar, PSCROLLINFO pSI) const;
	int GetScrollPos(int iBar) const;
	BOOL GetScrollRange(int iBar, PINT piMinPos, PINT piMaxPos) const;
	HMENU GetSystemMenu(BOOL bReset = FALSE) const;
	BOOL GetTitleBarInfo(PTITLEBARINFO pTBI) const;
	BOOL GetUpdateRect(PRECT pRect = NULL, BOOL bErase = FALSE) const;
	int GetUpdateRgn(HRGN hRgn, BOOL bErase = FALSE) const;
	HWND GetWindow(UINT nType) const;
	HDC GetWindowDC() const;
	DWORD GetWindowExStyle() const;
	UINT GetWindowFile(PTSTR pszBuffer, UINT ccBuffer) const;
	HFONT GetWindowFont() const;
	HICON GetWindowIcon(UINT nType) const;
	UINT GetWindowId() const;
	BOOL GetWindowInfo(PWINDOWINFO pWI) const;
	LONG GetWindowLong(int iIndex) const;
	BOOL GetWindowPlacement(PWINDOWPLACEMENT pWP) const;
	BOOL GetWindowRect(PRECT pRect) const;
	int GetWindowRgn(HRGN hRgn) const;
	int GetWindowRgn(PRECT pRect) const;
	DWORD GetWindowStyle() const;
	int GetWindowText(PTSTR pszBuffer, int ccBuffer) const;
	int GetWindowTextLength() const;
	DWORD GetWindowThreadId(PDWORD pdwProcessId = NULL) const;
	UINT GetWindowType(PTSTR pszBuffer, UINT ccBuffer) const;
	WORD GetWindowWord(int iIndex) const;
	XLONG GetWindowXLong(int iIndex) const;
	BOOL HideCaret() const;
	BOOL HideOwnedPopups() const;
	BOOL HideScrollBar(int iBar) const;
	BOOL HideWindow() const;
	BOOL HiliteMenuItem(HMENU hMenu, UINT nItemId, UINT nFlags) const;
	BOOL InvalidateRect(PCRECT pRect, BOOL bErase = TRUE) const;
	BOOL InvalidateRect(BOOL bErase = TRUE) const;
	BOOL InvalidateRgn(HRGN hRgn, BOOL bErase = TRUE) const;
	UINT IsButtonChecked(UINT nButtonId) const;
	BOOL IsChild(HWND hChildWnd) const;
	BOOL IsMaximized() const;
	BOOL IsMinimized() const;
	BOOL IsRestored() const;
	BOOL IsWindow() const;
	BOOL IsWindowEnabled() const;
	BOOL IsWindowUnicode() const;
	BOOL IsWindowVisible() const;
	BOOL KillTimer(UINT nTimerId) const;
	BOOL LockWindowUpdate() const;
	int MapWindowPoints(HWND hToWnd, PRECT paRect, UINT ceRect = 1) const;
	int MapWindowPoints(HWND hToWnd, PPOINT paPoint, UINT cePoint = 1) const;
	int MessageBox(PCTSTR pszText, PCTSTR pszCaption, UINT nType, LANGID liButton) const;
	int MessageBox(PCTSTR pszText, PCTSTR pszCaption = NULL, UINT nType = MB_OK) const;
	BOOL MinimizeWindow() const;
	BOOL MoveWindow(int x, int y, int iWidth, int iHeight, BOOL bRepaint = TRUE) const;
	BOOL OpenClipboard() const;
	void Paste() const;
	BOOL PeekMessage(PMSG pMsg, UINT nMinMsg, UINT nMaxMsg, UINT nRemove) const;
	BOOL PeekMessage(PMSG pMsg, UINT nMsg, UINT nRemove) const;
	BOOL PeekMessage(PMSG pMsg, UINT nRemove) const;
	BOOL PostMessage(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	UINT QueryUiState() const;
	BOOL RedrawWindow(PCRECT pUpdateRect, UINT nFlags) const;
	BOOL RedrawWindow(HRGN hUpdateRgn, UINT nFlags) const;
	BOOL RedrawWindow(UINT nFlags) const;
	BOOL RegisterShellHookWindow() const;
	BOOL ReleaseDC(HDC hDC) const;
	HANDLE RemoveProp(PCTSTR pszEntry) const;
	HANDLE RemoveProp(ATOM aEntry) const;
	BOOL RestoreWindow() const;
	BOOL ScreenToClient(PPOINT pPoint) const;
	int ScrollWindow(int cxAmount, int cyAmount, PCRECT pScrollRect, PCRECT pClipRect,
		HRGN hUpdateRgn, PRECT pUpdateRect, UINT nFlags) const;
	BOOL ScrollWindow(int cxAmount, int cyAmount, PCRECT pScrollRect, PCRECT pClipRect) const;
	BOOL ScrollWindow(int cxAmount, int cyAmount, PCRECT pClipRect = NULL) const;
	LRESULT SendChildMessage(UINT nChildId, UINT nMsg, WPARAM wParam = UNUSED,
		LPARAM lParam = UNUSED) const;
	LRESULT SendMessage(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	BOOL SendMessageTimeout(UINT nMsg, WPARAM wParam, LPARAM lParam,
		UINT nFlags, UINT nTimeout, PDWORD pdwResult) const;
	BOOL SendNotifyMessage(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	HWND SetActiveWindow() const;
	HWND SetCapture() const;
	HWND SetChild(HWND hChildWnd) const;
	BOOL SetChildInt(UINT nChildId, UINT nValue, BOOL bSigned = FALSE) const;
	BOOL SetChildText(UINT nChildId, PCTSTR pszText) const;
	DWORD SetClassLong(int iIndex, LONG lValue) const;
	WORD SetClassWord(int iIndex, WORD wValue) const;
	XULONG SetClassXLong(int iIndex, XLONG xlValue) const;
	HWND SetClipboardViewer() const;
	HWND SetFocus() const;
	BOOL SetForegroundWindow() const;
	BOOL SetLayeredWindowAttributes(COLORREF crKey, BYTE byAlpha, DWORD dwFlags) const;
	BOOL SetMenu(HMENU hMenu = NULL) const;
	HWND SetParent(HWND hParentWnd) const;
	BOOL SetProp(PCTSTR pszEntry, HANDLE hData) const;
	BOOL SetProp(ATOM aEntry, HANDLE hData) const;
	int SetScrollInfo(int iBar, PCSCROLLINFO pSI, BOOL bRedraw = TRUE) const;
	int SetScrollPos(int iBar, int iPos, BOOL bRedraw = TRUE) const;
	BOOL SetScrollRange(int iBar, int iMinPos, int iMaxPos, BOOL bRedraw = FALSE) const;
	UINT SetTimer(UINT nTimerId, UINT nInterval, TIMERPROC tpName = NULL) const;
	DWORD SetWindowExStyle(DWORD dwExStyle) const;
	void SetWindowFont(HFONT hFont = NULL, BOOL bRedraw = TRUE) const;
	HICON SetWindowIcon(UINT nType, HICON hIcon = NULL) const;
	UINT SetWindowId(UINT nWndId) const;
	LONG SetWindowLong(int iIndex, LONG lValue) const;
	BOOL SetWindowPlacement(PCWINDOWPLACEMENT pWP) const;
	BOOL SetWindowPos(HWND hPrecWnd, int x, int y, int iWidth, int iHeight, UINT nFlags) const;
	void SetWindowRedraw(BOOL bRedraw = TRUE) const;
	BOOL SetWindowRgn(HRGN hRgn, BOOL bRedraw = TRUE) const;
	DWORD SetWindowStyle(DWORD dwStyle) const;
	BOOL SetWindowText(PCTSTR pszText = NULL) const;
	WORD SetWindowWord(int iIndex, WORD wValue) const;
	XLONG SetWindowXLong(int iIndex, XLONG xlValue) const;
	BOOL ShowCaret() const;
	BOOL ShowOwnedPopups() const;
	BOOL ShowScrollBar(int iBar) const;
	BOOL ShowWindow(int iShowCmd = SW_SHOW) const;
	BOOL ShowWindowAsync(int iShowCmd) const;
	WORD TileWindows(UINT nFlags, PCRECT pRect, UINT ceChildWnd = UNUSED,
		PCHWND phaChildWnd = NULL) const;
	WORD TileWindows(UINT nFlags, UINT ceChildWnd = UNUSED, PCHWND phaChildWnd = NULL) const;
	BOOL TrackPopupMenu(HMENU hMenu, int x, int y, PTPMPARAMS pTPMP, UINT nFlags) const;
	BOOL TrackPopupMenu(HMENU hMenu, int x, int y, UINT nFlags) const;
	BOOL TranslateAccelerator(HACCEL hAccel, PMSG pMsg) const;
	BOOL UnregisterShellHookWindow() const;
	BOOL UpdateLayeredWindow(HDC hDestDC, PPOINT pptDest, PSIZE pSize, HDC hSrcDC,
		PPOINT pptSrc, COLORREF crKey, PBLENDFUNCTION pBF, DWORD dwFlags) const;
	void UpdateUiState(UINT nState, UINT nElement) const;
	BOOL UpdateWindow() const;
	BOOL ValidateRect(PCRECT pRect = NULL) const;
	BOOL ValidateRgn(HRGN hRgn = NULL) const;
	BOOL WindowFromPoint(POINT pt);

#if (WINVER >= 0x0600)
	BOOL AddClipboardFormatListener() const;
	BOOL RemoveClipboardFormatListener() const;
#endif

#ifdef COMMCTRL
	BOOL DragEnter(int x, int y) const;
	BOOL DragLeave() const;
	BOOL EnableFlatSB(UINT nBar, UINT nArrows = ESB_ENABLE_BOTH) const;
	void GetClientRect(PRECT pRect, PINT piaInfo) const;
	BOOL GetFlatSBInfo(int iBar, PSCROLLINFO pSI) const;
	int GetFlatSBPos(int iBar) const;
	BOOL GetFlatSBProp(int iProp, PINT piValue) const;
	BOOL GetFlatSBProp(int iProp, PCOLORREF pcrValue) const;
	BOOL GetFlatSBProp(int iProp, PHPALETTE phPalette) const;
	BOOL GetFlatSBRange(int iBar, PINT piMinPos, PINT piMaxPos) const;
	BOOL HideFlatSB(int iBar) const;
	BOOL InitializeFlatSB() const;
	int SetFlatSBInfo(int iBar, PSCROLLINFO pSI, BOOL bRedraw = TRUE) const;
	int SetFlatSBPos(int iBar, int iPos, BOOL bRedraw = TRUE) const;
	BOOL SetFlatSBProp(UINT nProp, int iValue, BOOL bRedraw = TRUE) const;
	BOOL SetFlatSBProp(UINT nProp, HPALETTE hPalette, BOOL bRedraw = TRUE) const;
	BOOL SetFlatSBRange(int iBar, int iMinPos, int iMaxPos, BOOL bRedraw = FALSE) const;
	BOOL ShowFlatSB(int iBar) const;
	BOOL ShowMenuCtrl(UINT nItemId, PINT piaInfo) const;
	HRESULT UninitializeFlatSB() const;
#endif

#ifdef DDE
	BOOL DdeAck(HWND hPartnerWnd, DDEACK ddea, ATOM aItemName) const;
	BOOL DdeAck(HWND hClientWnd, DDEACK ddea, HMEMORY hCmdStr) const;
	void DdeAck(HWND hClientWnd, ATOM aAppName, ATOM aTopicName) const;
	BOOL DdeAdvise(HWND hServerWnd, HMEMORY hDDEA, ATOM aItemName) const;
	BOOL DdeData(HWND hClientWnd, HMEMORY hDDED, ATOM aItemName) const;
	BOOL DdeExecute(HWND hServerWnd, HMEMORY hCmdStr) const;
	BOOL DdeImpersonateClient(HWND hClientWnd) const;
	void DdeInitiate(HWND hServerWnd, ATOM aAppName, ATOM aTopicName) const;
	void DdeInitiate(ATOM aAppName, ATOM aTopicName) const;
	BOOL DdePoke(HWND hServerWnd, HMEMORY hDDEP, ATOM aItemName) const;
	BOOL DdeRequest(HWND hServerWnd, ATOM aItemName, UINT nFormat) const;
	BOOL DdeSetQualityService(const PSECURITY_QUALITY_OF_SERVICE pNewSQOS,
		PSECURITY_QUALITY_OF_SERVICE pPrevSQOS = NULL) const;
	BOOL DdeTerminate(HWND hPartnerWnd) const;
	BOOL DdeUnadvise(HWND hServerWnd, ATOM aItemName, UINT nFormat) const;
#endif

#ifdef SHLAPI
	void DragAcceptFiles(BOOL bAccept = TRUE) const;
#endif

#ifdef WINSOCK
	HWSATASK WSAAsyncGetHostByAddr(UINT nMsg, PULONG pulAddr, int iAddr, PHOSTENTEX pHEX) const;
	HWSATASK WSAAsyncGetHostByName(UINT nMsg, PCSTR pszName, PHOSTENTEX pHEX) const;
	HWSATASK WSAAsyncGetProtoByName(UINT nMsg, PCSTR pszName, PPROTOENTEX pPEX) const;
	HWSATASK WSAAsyncGetProtoByNumber(UINT nMsg, int iProtocol, PPROTOENTEX pPEX) const;
	HWSATASK WSAAsyncGetServByName(UINT nMsg, PCSTR pszName, PCSTR pszProtocol,
		PSERVENTEX pSEX) const;
	HWSATASK WSAAsyncGetServByName(UINT nMsg, PCSTR pszName, PSERVENTEX pSEX) const;
	HWSATASK WSAAsyncGetServByPort(UINT nMsg, int iPort, PCSTR pszProtocol,
		PSERVENTEX pSEX) const;
	HWSATASK WSAAsyncGetServByPort(UINT nMsg, int iPort, PSERVENTEX pSEX) const;
	int WSAAsyncSelect(HSOCKET hSocket, UINT nMsg = UNUSED, LONG lEvent = UNUSED) const;
	void WSARemoveMessage(UINT nMsg, HWSATASK hWsaTask) const;
	void WSARemoveMessage(UINT nMsg, HSOCKET hSocket) const;
	void WSARemoveMessage(UINT nMsg) const;
#endif

#ifdef WINSPOOL
	BOOL PrinterProperties(HPRINTER hPrinter) const;
#endif

#ifdef WINTHEME
	HRESULT SetWindowTheme(PCWSTR pszAppName = NULL, PCWSTR pszClsIdList = NULL);
#endif

	// Attaches the object to a window
	void AttachObject(HWND hWnd);

	// Called after the window has been detached from the object
	// Overrides in derived class (to delete the object, for example), if necessary
	void DetachObject();

protected:
	// Window types for message maps (used internally only)
	enum
	{
		_WT_FRAME,     // General window
		_WT_CHILD,     // General child window
		_WT_FORM,      // Form window (based on dialog template, no keyboard navigation support)
		_WT_DIALOG,    // Dialog box (modal and modeless)
		_WT_CTMCTRL,   // Custom control (contained in a dialog box)
		_WT_SUBCLASS,  // Subclass window (derived from a control class)
		_WT_MDIFRAME,  // MDI frame window
		_WT_MDICHILD,  // MDI child window
		_WT_SCRNSAVE,  // Screen saver window
	};

	// Default return value (used internally only)
	enum { _eResult = 0 };

protected:
	// Window object pointer for message maps (used internally only)
	static CWnd* _m_pWnd;

#ifdef DEBUG
	// Size, in bytes, associated with this object (used internally only)
	static UINT _m_cbObject;
#endif

protected:
	// Gets object pointer to this window (used internally only)
	static CWnd* WINAPI _GetObjectPtr(HWND hWnd);
};

AFXINLINE CWnd::CWnd()
{
	// Does nothing
}

AFXINLINE CWnd::CWnd(HWND hWnd)
{
	m_hWnd = hWnd;
}

AFXINLINE CWnd::operator HWND() const
{
	return m_hWnd;
}

AFXINLINE void CWnd::operator =(HWND hWnd)
{
	m_hWnd = hWnd;
}

AFXINLINE BOOL CWnd::operator ==(HWND hWnd) const
{
	return (m_hWnd == hWnd);
}

AFXINLINE BOOL CWnd::operator !=(HWND hWnd) const
{
	return (m_hWnd != hWnd);
}

AFXINLINE CWnd* WINAPI CWnd::_GetObjectPtr(HWND hWnd)
{
	CWnd* pWnd = (CWnd*)::GetWindowLong(hWnd, GWL_USERDATA);
	if(pWnd)
	{
		ASSERT(pWnd->m_hWnd == hWnd);
		return pWnd;
	}

	ASSERT_WRITE(_m_pWnd, _m_cbObject);
	pWnd         = _m_pWnd;
	pWnd->m_hWnd = hWnd;
	::SetWindowLong(hWnd, GWL_USERDATA, (LONG)pWnd);
	DEBUG_ONLY(_m_pWnd = NULL);
	return pWnd;
}

AFXINLINE BOOL CWnd::AnimateWindow(DWORD dwTime, DWORD dwFlags) const
{
	return VERIFY(::AnimateWindow(m_hWnd, dwTime, dwFlags));
}

AFXINLINE UINT CWnd::ArrangeIconicWindows() const
{
	return VERIFY(::ArrangeIconicWindows(m_hWnd));
}

AFXINLINE void CWnd::AttachObject(HWND hWnd)
{
	ASSERT_HWND(hWnd);
	m_hWnd = hWnd;
	::SetWindowLong(m_hWnd, GWL_USERDATA, (LONG)this);
}

AFXINLINE HDC CWnd::BeginPaint(PPAINTSTRUCT pPS) const
{
	return (HDC)VERIFY(::BeginPaint(m_hWnd, pPS));
}

AFXINLINE BOOL CWnd::BringWindowToTop() const
{
	return VERIFY(::BringWindowToTop(m_hWnd));
}

AFXINLINE WORD CWnd::CascadeWindows(UINT nFlag, PCRECT pRect,
	UINT ceChildWnd, PCHWND phaChildWnd) const
{
	return ::CascadeWindows(m_hWnd, nFlag, pRect, ceChildWnd, phaChildWnd);
}

AFXINLINE void CWnd::ChangeCbChain(HWND hRemoveWnd, HWND hNextWnd) const
{
	::SendMessage(m_hWnd, WM_CHANGECBCHAIN, (WPARAM)hRemoveWnd, (LPARAM)hNextWnd);
}

AFXINLINE BOOL CWnd::ChangeClipboardChain(HWND hNextWnd) const
{
	return ::ChangeClipboardChain(m_hWnd, hNextWnd);
}

AFXINLINE void CWnd::ChangeUiState(UINT nState, UINT nElement) const
{
	::SendMessage(m_hWnd, WM_CHANGEUISTATE, MAKEWPARAM(nState, nElement), UNUSED);
}

AFXINLINE BOOL CWnd::CheckButton(UINT nButtonId, UINT nCheck /* = BST_CHECKED */) const
{
	return VERIFY(::CheckDlgButton(m_hWnd, nButtonId, nCheck));
}

AFXINLINE BOOL CWnd::CheckButton(UINT nFirstId, UINT nLastId, UINT nCheckId) const
{
	return VERIFY(::CheckRadioButton(m_hWnd, nFirstId, nLastId, nCheckId));
}

AFXINLINE void CWnd::Clear() const
{
	::SendMessage(m_hWnd, WM_CLEAR, UNUSED, UNUSED);
}

AFXINLINE BOOL CWnd::ClientToScreen(PPOINT pPoint) const
{
	return VERIFY(::ClientToScreen(m_hWnd, pPoint));
}

AFXINLINE BOOL CWnd::CloseWindow(BOOL bForce /* = TRUE */) const
{
	return VERIFY(::EndTask(m_hWnd, FALSE, bForce));
}

AFXINLINE void CWnd::Copy() const
{
	::SendMessage(m_hWnd, WM_COPY, UNUSED, UNUSED);
}

AFXINLINE BOOL CWnd::CreateCaret(HBITMAP hBitmap) const
{
	return VERIFY(::CreateCaret(m_hWnd, hBitmap, UNUSED, UNUSED));
}

AFXINLINE BOOL CWnd::CreateCaret(int iWidth /* = UNUSED */, int iHeight /* = UNUSED */) const
{
	return VERIFY(::CreateCaret(m_hWnd, NULL, iWidth, iHeight));
}

AFXINLINE void CWnd::Cut() const
{
	::SendMessage(m_hWnd, WM_CUT, UNUSED, UNUSED);
}

AFXINLINE HDWP CWnd::DeferWindowPos(HDWP hDwp, HWND hPrecWnd, int x, int y,
	int iWidth, int iHeight, UINT nFlags) const
{
	return (HDWP)VERIFY(::DeferWindowPos(hDwp, m_hWnd, hPrecWnd, x, y,
		iWidth, iHeight, nFlags));
}

AFXINLINE LRESULT CWnd::DefWindowProc(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return ::DefWindowProc(m_hWnd, nMsg, wParam, lParam);
}

AFXINLINE BOOL CWnd::DestroyWindow() const
{
	return VERIFY(::DestroyWindow(m_hWnd));
}

AFXINLINE void CWnd::DetachObject()
{
	// Does nothing
}

AFXINLINE BOOL CWnd::DirComboBox(PTSTR pszPathSpec, UINT nComboBoxId, UINT nStaticId,
	UINT nAttrs) const
{
	return VERIFY(::DlgDirListComboBox(m_hWnd, pszPathSpec, nComboBoxId, nStaticId, nAttrs));
}

AFXINLINE BOOL CWnd::DirListBox(PTSTR pszPathSpec, UINT nListBoxId, UINT nStaticId,
	UINT nAttrs) const
{
	return VERIFY(::DlgDirList(m_hWnd, pszPathSpec, nListBoxId, nStaticId, nAttrs));
}

AFXINLINE BOOL CWnd::DirSelComboBox(PTSTR pszBuffer, int ccBuffer, UINT nComboBoxId) const
{
	return ::DlgDirSelectComboBoxEx(m_hWnd, pszBuffer, ccBuffer, nComboBoxId);
}

AFXINLINE BOOL CWnd::DirSelListBox(PTSTR pszBuffer, int ccBuffer, UINT nListBoxId) const
{
	return ::DlgDirSelectEx(m_hWnd, pszBuffer, ccBuffer, nListBoxId);
}

AFXINLINE BOOL CWnd::DisableWindow() const
{
	return ::EnableWindow(m_hWnd, FALSE);
}

AFXINLINE void CWnd::DrawClipboard() const
{
	::SendMessage(m_hWnd, WM_DRAWCLIPBOARD, UNUSED, UNUSED);
}

AFXINLINE BOOL CWnd::DrawMenuBar() const
{
	return VERIFY(::DrawMenuBar(m_hWnd));
}

AFXINLINE BOOL CWnd::EnableScrollBar(UINT nBar, UINT nArrows /* = ESB_ENABLE_BOTH */) const
{
	return ::EnableScrollBar(m_hWnd, nBar, nArrows);
}

AFXINLINE BOOL CWnd::EnableWindow(BOOL bEnable /* = TRUE */) const
{
	return ::EnableWindow(m_hWnd, bEnable);
}

AFXINLINE BOOL CWnd::EndPaint(PCPAINTSTRUCT pPS) const
{
	return ::EndPaint(m_hWnd, pPS);
}

AFXINLINE BOOL CWnd::EnumChildWindows(WNDENUMPROC epName, LPARAM lParam /* = UNUSED */) const
{
	return ::EnumChildWindows(m_hWnd, epName, lParam);
}

AFXINLINE int CWnd::EnumProps(PROPENUMPROCEX epName, LPARAM lParam) const
{
	return ::EnumPropsEx(m_hWnd, epName, lParam);
}

AFXINLINE int CWnd::EnumProps(PROPENUMPROC epName) const
{
	return ::EnumProps(m_hWnd, epName);
}

AFXINLINE HWND CWnd::FindWindow(HWND hChildWnd, PCTSTR pszClassName /* = NULL */,
	PCTSTR pszWndName /* = NULL */) const
{
	return ::FindWindowEx(m_hWnd, hChildWnd, pszClassName, pszWndName);
}

AFXINLINE BOOL CWnd::FindWindow(PCTSTR pszClassName /* = NULL */,
	PCTSTR pszWndName /* = NULL */)
{
	m_hWnd = ::FindWindow(pszClassName, pszWndName);
	return (BOOL)m_hWnd;
}

AFXINLINE BOOL CWnd::GetAltTabInfo(int iItem, PALTTABINFO pATI, PTSTR pszBuffer, UINT ccBuffer) const
{
	return VERIFY(::GetAltTabInfo(m_hWnd, iItem, pATI, pszBuffer, ccBuffer));
}

AFXINLINE BOOL CWnd::GetAltTabInfo(PALTTABINFO pATI) const
{
	return VERIFY(::GetAltTabInfo(m_hWnd, -1, pATI, NULL, UNUSED));
}

AFXINLINE HWND CWnd::GetAncestor(UINT nFlag) const
{
	return (HWND)VERIFY(::GetAncestor(m_hWnd, nFlag));
}

AFXINLINE void CWnd::GetCapture()
{
	m_hWnd = ::GetCapture();
}

AFXINLINE HWND CWnd::GetChild(UINT nChildId) const
{
	return (HWND)VERIFY(::GetDlgItem(m_hWnd, nChildId));
}

AFXINLINE HWND CWnd::GetChild(POINT ptClient, UINT nFlags) const
{
	return (HWND)VERIFY(::ChildWindowFromPointEx(m_hWnd, ptClient, nFlags));
}

AFXINLINE HWND CWnd::GetChild(POINT ptClient) const
{
	return (HWND)VERIFY(::ChildWindowFromPoint(m_hWnd, ptClient));
}

AFXINLINE UINT CWnd::GetChildInt(UINT nChildId, BOOL bSigned /* = FALSE */,
	PBOOL pbSuccess /* = NULL */) const
{
#ifdef DEBUG
	BOOL bSuccess;
	UINT nResult = ::GetDlgItemInt(m_hWnd, nChildId, &bSuccess, bSigned);
	ASSERT(bSuccess);
	if(pbSuccess)
		*pbSuccess = bSuccess;
	return nResult;
#else
	return ::GetDlgItemInt(m_hWnd, nChildId, pbSuccess, bSigned);
#endif
}

AFXINLINE UINT CWnd::GetChildText(UINT nChildId, PTSTR pszBuffer, int ccBuffer) const
{
	return ::GetDlgItemText(m_hWnd, nChildId, pszBuffer, ccBuffer);
}

AFXINLINE DWORD CWnd::GetClassLong(int iIndex) const
{
	return ::GetClassLong(m_hWnd, iIndex);
}

AFXINLINE int CWnd::GetClassName(PTSTR pszBuffer, int ccBuffer) const
{
	return VERIFY(::GetClassName(m_hWnd, pszBuffer, ccBuffer));
}

AFXINLINE WORD CWnd::GetClassWord(int iIndex) const
{
	return ::GetClassWord(m_hWnd, iIndex);
}

AFXINLINE XULONG CWnd::GetClassXLong(int iIndex) const
{
	return ::GetClassLongPtr(m_hWnd, iIndex);
}

AFXINLINE BOOL CWnd::GetClientRect(PRECT pRect) const
{
	return VERIFY(::GetClientRect(m_hWnd, pRect));
}

AFXINLINE HDC CWnd::GetDC(HRGN hClipRgn, DWORD dwFlags) const
{
	return (HDC)VERIFY(::GetDCEx(m_hWnd, hClipRgn, dwFlags));
}

AFXINLINE HDC CWnd::GetDC(DWORD dwFlags) const
{
	return (HDC)VERIFY(::GetDCEx(m_hWnd, NULL, dwFlags));
}

AFXINLINE HDC CWnd::GetDC() const
{
	return (HDC)VERIFY(::GetDC(m_hWnd));
}

AFXINLINE HWND CWnd::GetFirstChild() const
{
	return ::GetTopWindow(m_hWnd);
}

AFXINLINE void CWnd::GetForegroundWindow()
{
	m_hWnd = ::GetForegroundWindow();
}

AFXINLINE HWND CWnd::GetLastActivePopup() const
{
	return (HWND)VERIFY(::GetLastActivePopup(m_hWnd));
}

AFXINLINE HMENU CWnd::GetMenu() const
{
	return (HMENU)::GetMenu(m_hWnd);
}

AFXINLINE BOOL CWnd::GetMenuBarInfo(LONG lObjectId, LONG lItemId, PMENUBARINFO pMBI) const
{
	return VERIFY(::GetMenuBarInfo(m_hWnd, lObjectId, lItemId, pMBI));
}

AFXINLINE BOOL CWnd::GetMessage(PMSG pMsg, UINT nMinMsg, UINT nMaxMsg) const
{
	return UNEQUAL(::GetMessage(pMsg, m_hWnd, nMinMsg, nMaxMsg), RV_ERR);
}

AFXINLINE BOOL CWnd::GetMessage(PMSG pMsg, UINT nMsg /* = UNUSED */) const
{
	return UNEQUAL(::GetMessage(pMsg, m_hWnd, nMsg, nMsg), RV_ERR);
}

AFXINLINE HWND CWnd::GetNextGroupChild(HWND hChildWnd, BOOL bPrevCtrl /* = FALSE */) const
{
	return (HWND)VERIFY(::GetNextDlgGroupItem(m_hWnd, hChildWnd, bPrevCtrl));
}

AFXINLINE HWND CWnd::GetNextGroupChild(BOOL bPrevCtrl /* = FALSE */) const
{
	return (HWND)VERIFY(::GetNextDlgGroupItem(m_hWnd, NULL, bPrevCtrl));
}

AFXINLINE HWND CWnd::GetNextTabChild(HWND hChildWnd, BOOL bPrevCtrl /* = FALSE */) const
{
	return (HWND)VERIFY(::GetNextDlgTabItem(m_hWnd, hChildWnd, bPrevCtrl));
}

AFXINLINE HWND CWnd::GetNextTabChild(BOOL bPrevCtrl /* = FALSE */) const
{
	return (HWND)VERIFY(::GetNextDlgTabItem(m_hWnd, NULL, bPrevCtrl));
}

AFXINLINE HWND CWnd::GetParent() const
{
	return (HWND)::GetParent(m_hWnd);
}

AFXINLINE HANDLE CWnd::GetProp(PCTSTR pszEntry) const
{
	return ::GetProp(m_hWnd, pszEntry);
}

AFXINLINE HANDLE CWnd::GetProp(ATOM aEntry) const
{
	return ::GetProp(m_hWnd, MAKEINTATOM(aEntry));
}

AFXINLINE HWND CWnd::GetRealChild(POINT ptClient) const
{
	return ::RealChildWindowFromPoint(m_hWnd, ptClient);
}

AFXINLINE BOOL CWnd::GetScrollInfo(LONG lBar, PSCROLLBARINFO pSBI) const
{
	return VERIFY(::GetScrollBarInfo(m_hWnd, lBar, pSBI));
}

AFXINLINE BOOL CWnd::GetScrollInfo(int iBar, PSCROLLINFO pSI) const
{
	return VERIFY(::GetScrollInfo(m_hWnd, iBar, pSI));
}

AFXINLINE int CWnd::GetScrollPos(int iBar) const
{
	return ::GetScrollPos(m_hWnd, iBar);
}

AFXINLINE BOOL CWnd::GetScrollRange(int iBar, PINT piMinPos, PINT piMaxPos) const
{
	return VERIFY(::GetScrollRange(m_hWnd, iBar, piMinPos, piMaxPos));
}

AFXINLINE HMENU CWnd::GetSystemMenu(BOOL bReset /* = FALSE */) const
{
	return ::GetSystemMenu(m_hWnd, bReset);
}

AFXINLINE BOOL CWnd::GetTitleBarInfo(PTITLEBARINFO pTBI) const
{
	return VERIFY(::GetTitleBarInfo(m_hWnd, pTBI));
}

AFXINLINE BOOL CWnd::GetUpdateRect(PRECT pRect /* = NULL */, BOOL bErase /* = FALSE */) const
{
	return ::GetUpdateRect(m_hWnd, pRect, bErase);
}

AFXINLINE int CWnd::GetUpdateRgn(HRGN hRgn, BOOL bErase /* = FALSE */) const
{
	return UNEQUAL(::GetUpdateRgn(m_hWnd, hRgn, bErase), RGN_ERROR);
}

AFXINLINE HWND CWnd::GetWindow(UINT nType) const
{
	return ::GetWindow(m_hWnd, nType);
}

AFXINLINE HDC CWnd::GetWindowDC() const
{
	return (HDC)VERIFY(::GetWindowDC(m_hWnd));
}

AFXINLINE DWORD CWnd::GetWindowExStyle() const
{
	return ::GetWindowLong(m_hWnd, GWL_EXSTYLE);
}

AFXINLINE UINT CWnd::GetWindowFile(PTSTR pszBuffer, UINT ccBuffer) const
{
	return ::GetWindowModuleFileName(m_hWnd, pszBuffer, ccBuffer);
}

AFXINLINE HFONT CWnd::GetWindowFont() const
{
	return (HFONT)::SendMessage(m_hWnd, WM_GETFONT, UNUSED, UNUSED);
}

AFXINLINE HICON CWnd::GetWindowIcon(UINT nType) const
{
	return (HICON)::SendMessage(m_hWnd, WM_GETICON, nType, UNUSED);
}

AFXINLINE UINT CWnd::GetWindowId() const
{
	return VERIFY(::GetDlgCtrlID(m_hWnd));
}

AFXINLINE BOOL CWnd::GetWindowInfo(PWINDOWINFO pWI) const
{
	return VERIFY(::GetWindowInfo(m_hWnd, pWI));
}

AFXINLINE LONG CWnd::GetWindowLong(int iIndex) const
{
	return ::GetWindowLong(m_hWnd, iIndex);
}

AFXINLINE BOOL CWnd::GetWindowPlacement(PWINDOWPLACEMENT pWP) const
{
	return VERIFY(::GetWindowPlacement(m_hWnd, pWP));
}

AFXINLINE BOOL CWnd::GetWindowRect(PRECT pRect) const
{
	return VERIFY(::GetWindowRect(m_hWnd, pRect));
}

AFXINLINE int CWnd::GetWindowRgn(HRGN hRgn) const
{
	return UNEQUAL(::GetWindowRgn(m_hWnd, hRgn), RGN_ERROR);
}

AFXINLINE int CWnd::GetWindowRgn(PRECT pRect) const
{
	return UNEQUAL(::GetWindowRgnBox(m_hWnd, pRect), RGN_ERROR);
}

AFXINLINE DWORD CWnd::GetWindowStyle() const
{
	return ::GetWindowLong(m_hWnd, GWL_STYLE);
}

AFXINLINE int CWnd::GetWindowText(PTSTR pszBuffer, int ccBuffer) const
{
	return ::GetWindowText(m_hWnd, pszBuffer, ccBuffer);
}

AFXINLINE int CWnd::GetWindowTextLength() const
{
	return ::GetWindowTextLength(m_hWnd);
}

AFXINLINE DWORD CWnd::GetWindowThreadId(PDWORD pdwProcessId /* = NULL */) const
{
	return ::GetWindowThreadProcessId(m_hWnd, pdwProcessId);
}

AFXINLINE UINT CWnd::GetWindowType(PTSTR pszBuffer, UINT ccBuffer) const
{
	return VERIFY(::RealGetWindowClass(m_hWnd, pszBuffer, ccBuffer));
}

AFXINLINE WORD CWnd::GetWindowWord(int iIndex) const
{
	return ::GetWindowWord(m_hWnd, iIndex);
}

AFXINLINE XLONG CWnd::GetWindowXLong(int iIndex) const
{
	return ::GetWindowLongPtr(m_hWnd, iIndex);
}

AFXINLINE BOOL CWnd::HideCaret() const
{
	return VERIFY(::HideCaret(m_hWnd));
}

AFXINLINE BOOL CWnd::HideOwnedPopups() const
{
	return VERIFY(::ShowOwnedPopups(m_hWnd, FALSE));
}

AFXINLINE BOOL CWnd::HideScrollBar(int iBar) const
{
	return VERIFY(::ShowScrollBar(m_hWnd, iBar, FALSE));
}

AFXINLINE BOOL CWnd::HideWindow() const
{
	return ::ShowWindow(m_hWnd, SW_HIDE);
}

AFXINLINE BOOL CWnd::HiliteMenuItem(HMENU hMenu, UINT nItemId, UINT nFlags) const
{
	return ::HiliteMenuItem(m_hWnd, hMenu, nItemId, nFlags);
}

AFXINLINE BOOL CWnd::InvalidateRect(PCRECT pRect, BOOL bErase /* = TRUE */) const
{
	return VERIFY(::InvalidateRect(m_hWnd, pRect, bErase));
}

AFXINLINE BOOL CWnd::InvalidateRect(BOOL bErase /* = TRUE */) const
{
	return VERIFY(::InvalidateRect(m_hWnd, NULL, bErase));
}

AFXINLINE BOOL CWnd::InvalidateRgn(HRGN hRgn, BOOL bErase /* = TRUE */) const
{
	return ::InvalidateRgn(m_hWnd, hRgn, bErase);
}

AFXINLINE UINT CWnd::IsButtonChecked(UINT nButtonId) const
{
	return ::IsDlgButtonChecked(m_hWnd, nButtonId);
}

AFXINLINE BOOL CWnd::IsChild(HWND hChildWnd) const
{
	return ::IsChild(m_hWnd, hChildWnd);
}

AFXINLINE BOOL CWnd::IsMaximized() const
{
	return ::IsZoomed(m_hWnd);
}

AFXINLINE BOOL CWnd::IsMinimized() const
{
	return ::IsIconic(m_hWnd);
}

AFXINLINE BOOL CWnd::IsRestored() const
{
	return !(::GetWindowLong(m_hWnd, GWL_STYLE) & (WS_MAXIMIZE | WS_MINIMIZE));
}

AFXINLINE BOOL CWnd::IsWindow() const
{
	return ::IsWindow(m_hWnd);
}

AFXINLINE BOOL CWnd::IsWindowEnabled() const
{
	return ::IsWindowEnabled(m_hWnd);
}

AFXINLINE BOOL CWnd::IsWindowUnicode() const
{
	return ::IsWindowUnicode(m_hWnd);
}

AFXINLINE BOOL CWnd::IsWindowVisible() const
{
	return ::IsWindowVisible(m_hWnd);
}

AFXINLINE BOOL CWnd::KillTimer(UINT nTimerId) const
{
	return VERIFY(::KillTimer(m_hWnd, nTimerId));
}

AFXINLINE BOOL CWnd::LockWindowUpdate() const
{
	return VERIFY(::LockWindowUpdate(m_hWnd));
}

AFXINLINE int CWnd::MapWindowPoints(HWND hToWnd, PRECT paRect, UINT ceRect /* = 1 */) const
{
	return VERIFY(::MapWindowPoints(m_hWnd, hToWnd, (PPOINT)paRect, ceRect * 2));
}

AFXINLINE int CWnd::MapWindowPoints(HWND hToWnd, PPOINT paPoint, UINT cePoint /* = 1 */) const
{
	return VERIFY(::MapWindowPoints(m_hWnd, hToWnd, paPoint, cePoint));
}

AFXINLINE int CWnd::MessageBox(PCTSTR pszText, PCTSTR pszCaption, UINT nType,
	LANGID liButton) const
{
	return VERIFY(::MessageBoxEx(m_hWnd, pszText, pszCaption, nType, liButton));
}

AFXINLINE int CWnd::MessageBox(PCTSTR pszText, PCTSTR pszCaption /* = NULL */,
	UINT nType /* = MB_OK */) const
{
	return VERIFY(::MessageBox(m_hWnd, pszText, pszCaption, nType));
}

AFXINLINE BOOL CWnd::MinimizeWindow() const
{
	return VERIFY(::CloseWindow(m_hWnd));
}

AFXINLINE BOOL CWnd::MoveWindow(int x, int y, int iWidth, int iHeight,
	BOOL bRepaint /* = TRUE */) const
{
	return VERIFY(::MoveWindow(m_hWnd, x, y, iWidth, iHeight, bRepaint));
}

AFXINLINE BOOL CWnd::OpenClipboard() const
{
	return VERIFY(::OpenClipboard(m_hWnd));
}

AFXINLINE void CWnd::Paste() const
{
	::SendMessage(m_hWnd, WM_PASTE, UNUSED, UNUSED);
}

AFXINLINE BOOL CWnd::PeekMessage(PMSG pMsg, UINT nMinMsg, UINT nMaxMsg, UINT nRemove) const
{
	return ::PeekMessage(pMsg, m_hWnd, nMinMsg, nMaxMsg, nRemove);
}

AFXINLINE BOOL CWnd::PeekMessage(PMSG pMsg, UINT nMsg, UINT nRemove) const
{
	return ::PeekMessage(pMsg, m_hWnd, nMsg, nMsg, nRemove);
}

AFXINLINE BOOL CWnd::PeekMessage(PMSG pMsg, UINT nRemove) const
{
	return ::PeekMessage(pMsg, m_hWnd, UNUSED, UNUSED, nRemove);
}

AFXINLINE BOOL CWnd::PostMessage(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return VERIFY(::PostMessage(m_hWnd, nMsg, wParam, lParam));
}

AFXINLINE UINT CWnd::QueryUiState() const
{
	return ::SendMessage(m_hWnd, WM_QUERYUISTATE, UNUSED, UNUSED);
}

AFXINLINE BOOL CWnd::RedrawWindow(PCRECT pUpdateRect, UINT nFlags) const
{
	return VERIFY(::RedrawWindow(m_hWnd, pUpdateRect, NULL, nFlags));
}

AFXINLINE BOOL CWnd::RedrawWindow(HRGN hUpdateRgn, UINT nFlags) const
{
	return VERIFY(::RedrawWindow(m_hWnd, NULL, hUpdateRgn, nFlags));
}

AFXINLINE BOOL CWnd::RedrawWindow(UINT nFlags) const
{
	return VERIFY(::RedrawWindow(m_hWnd, NULL, NULL, nFlags));
}

AFXINLINE BOOL CWnd::RegisterShellHookWindow() const
{
	return VERIFY(::RegisterShellHookWindow(m_hWnd));
}

AFXINLINE BOOL CWnd::ReleaseDC(HDC hDC) const
{
	return VERIFY(::ReleaseDC(m_hWnd, hDC));
}

AFXINLINE HANDLE CWnd::RemoveProp(PCTSTR pszEntry) const
{
	return ::RemoveProp(m_hWnd, pszEntry);
}

AFXINLINE HANDLE CWnd::RemoveProp(ATOM aEntry) const
{
	return ::RemoveProp(m_hWnd, MAKEINTATOM(aEntry));
}

AFXINLINE BOOL CWnd::RestoreWindow() const
{
	return VERIFY(::OpenIcon(m_hWnd));
}

AFXINLINE BOOL CWnd::ScreenToClient(PPOINT pPoint) const
{
	return VERIFY(::ScreenToClient(m_hWnd, pPoint));
}

AFXINLINE int CWnd::ScrollWindow(int cxAmount, int cyAmount, PCRECT pScrollRect,
	PCRECT pClipRect, HRGN hUpdateRgn, PRECT pUpdateRect, UINT nFlags) const
{
	return UNEQUAL(::ScrollWindowEx(m_hWnd, cxAmount, cyAmount, pScrollRect,
		pClipRect, hUpdateRgn, pUpdateRect, nFlags), RGN_ERROR);
}

AFXINLINE BOOL CWnd::ScrollWindow(int cxAmount, int cyAmount, PCRECT pScrollRect,
	PCRECT pClipRect) const
{
	return VERIFY(::ScrollWindow(m_hWnd, cxAmount, cyAmount, pScrollRect, pClipRect));
}

AFXINLINE BOOL CWnd::ScrollWindow(int cxAmount, int cyAmount,
	PCRECT pClipRect /* = NULL */) const
{
	return VERIFY(::ScrollWindow(m_hWnd, cxAmount, cyAmount, NULL, pClipRect));
}

AFXINLINE LRESULT CWnd::SendChildMessage(UINT nChildId, UINT nMsg,
	WPARAM wParam /* = UNUSED */, LPARAM lParam /* = UNUSED */) const
{
	return ::SendDlgItemMessage(m_hWnd, nChildId, nMsg, wParam, lParam);
}

AFXINLINE LRESULT CWnd::SendMessage(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return ::SendMessage(m_hWnd, nMsg, wParam, lParam);
}

AFXINLINE BOOL CWnd::SendMessageTimeout(UINT nMsg, WPARAM wParam, LPARAM lParam,
	UINT nFlags, UINT nTimeout, PDWORD pdwResult) const
{
	return ::SendMessageTimeout(m_hWnd, nMsg, wParam, lParam, nFlags, nTimeout, pdwResult);
}

AFXINLINE BOOL CWnd::SendNotifyMessage(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return VERIFY(::SendNotifyMessage(m_hWnd, nMsg, wParam, lParam));
}

AFXINLINE HWND CWnd::SetActiveWindow() const
{
	return (HWND)VERIFY(::SetActiveWindow(m_hWnd));
}

AFXINLINE HWND CWnd::SetCapture() const
{
	return ::SetCapture(m_hWnd);
}

AFXINLINE HWND CWnd::SetChild(HWND hChildWnd) const
{
	return (HWND)VERIFY(::SetParent(hChildWnd, m_hWnd));
}

AFXINLINE BOOL CWnd::SetChildInt(UINT nChildId, UINT nValue, BOOL bSigned /* = FALSE */) const
{
	return VERIFY(::SetDlgItemInt(m_hWnd, nChildId, nValue, bSigned));
}

AFXINLINE BOOL CWnd::SetChildText(UINT nChildId, PCTSTR pszText) const
{
	return VERIFY(::SetDlgItemText(m_hWnd, nChildId, pszText));
}

AFXINLINE DWORD CWnd::SetClassLong(int iIndex, LONG lValue) const
{
	return ::SetClassLong(m_hWnd, iIndex, lValue);
}

AFXINLINE WORD CWnd::SetClassWord(int iIndex, WORD wValue) const
{
	return ::SetClassWord(m_hWnd, iIndex, wValue);
}

AFXINLINE XULONG CWnd::SetClassXLong(int iIndex, XLONG xlValue) const
{
	return ::SetClassLongPtr(m_hWnd, iIndex, xlValue);
}

AFXINLINE HWND CWnd::SetClipboardViewer() const
{
	return ::SetClipboardViewer(m_hWnd);
}

AFXINLINE HWND CWnd::SetFocus() const
{
	return (HWND)::SetFocus(m_hWnd);
}

AFXINLINE BOOL CWnd::SetForegroundWindow() const
{
	return ::SetForegroundWindow(m_hWnd);
}

AFXINLINE BOOL CWnd::SetLayeredWindowAttributes(COLORREF crKey, BYTE byAlpha,
	DWORD dwFlags) const
{
	return VERIFY(::SetLayeredWindowAttributes(m_hWnd, crKey, byAlpha, dwFlags));
}

AFXINLINE BOOL CWnd::SetMenu(HMENU hMenu /* = NULL */) const
{
	return VERIFY(::SetMenu(m_hWnd, hMenu));
}

AFXINLINE HWND CWnd::SetParent(HWND hParentWnd) const
{
	return (HWND)VERIFY(::SetParent(m_hWnd, hParentWnd));
}

AFXINLINE BOOL CWnd::SetProp(PCTSTR pszEntry, HANDLE hData) const
{
	return VERIFY(::SetProp(m_hWnd, pszEntry, hData));
}

AFXINLINE BOOL CWnd::SetProp(ATOM aEntry, HANDLE hData) const
{
	return VERIFY(::SetProp(m_hWnd, MAKEINTATOM(aEntry), hData));
}

AFXINLINE int CWnd::SetScrollInfo(int iBar, PCSCROLLINFO pSI, BOOL bRedraw /* = TRUE */) const
{
	return ::SetScrollInfo(m_hWnd, iBar, pSI, bRedraw);
}

AFXINLINE int CWnd::SetScrollPos(int iBar, int iPos, BOOL bRedraw /* = TRUE */) const
{
	return ::SetScrollPos(m_hWnd, iBar, iPos, bRedraw);
}

AFXINLINE BOOL CWnd::SetScrollRange(int iBar, int iMinPos, int iMaxPos,
	BOOL bRedraw /* = FALSE */) const
{
	return VERIFY(::SetScrollRange(m_hWnd, iBar, iMinPos, iMaxPos, bRedraw));
}

AFXINLINE UINT CWnd::SetTimer(UINT nTimerId, UINT nInterval,
	TIMERPROC tpName /* = NULL */) const
{
	return VERIFY(::SetTimer(m_hWnd, nTimerId, nInterval, tpName));
}

AFXINLINE DWORD CWnd::SetWindowExStyle(DWORD dwExStyle) const
{
	return ::SetWindowLong(m_hWnd, GWL_EXSTYLE, dwExStyle);
}

AFXINLINE void CWnd::SetWindowFont(HFONT hFont /* = NULL */, BOOL bRedraw /* = TRUE */) const
{
	::SendMessage(m_hWnd, WM_SETFONT, (WPARAM)hFont, bRedraw);
}

AFXINLINE HICON CWnd::SetWindowIcon(UINT nType, HICON hIcon /* = NULL */) const
{
	return (HICON)::SendMessage(m_hWnd, WM_SETICON, nType, (LPARAM)hIcon);
}

AFXINLINE UINT CWnd::SetWindowId(UINT nWndId) const
{
	return VERIFY(::SetWindowLong(m_hWnd, GWL_ID, nWndId));
}

AFXINLINE LONG CWnd::SetWindowLong(int iIndex, LONG lValue) const
{
	return ::SetWindowLong(m_hWnd, iIndex, lValue);
}

AFXINLINE BOOL CWnd::SetWindowPlacement(PCWINDOWPLACEMENT pWP) const
{
	return VERIFY(::SetWindowPlacement(m_hWnd, pWP));
}

AFXINLINE BOOL CWnd::SetWindowPos(HWND hPrecWnd, int x, int y, int iWidth, int iHeight,
	UINT nFlags) const
{
	return VERIFY(::SetWindowPos(m_hWnd, hPrecWnd, x, y, iWidth, iHeight, nFlags));
}

AFXINLINE void CWnd::SetWindowRedraw(BOOL bRedraw /* = TRUE */) const
{
	::SendMessage(m_hWnd, WM_SETREDRAW, bRedraw, UNUSED);
}

AFXINLINE BOOL CWnd::SetWindowRgn(HRGN hRgn, BOOL bRedraw /* = TRUE */) const
{
	return VERIFY(::SetWindowRgn(m_hWnd, hRgn, bRedraw));
}

AFXINLINE DWORD CWnd::SetWindowStyle(DWORD dwStyle) const
{
	return ::SetWindowLong(m_hWnd, GWL_STYLE, dwStyle);
}

AFXINLINE BOOL CWnd::SetWindowText(PCTSTR pszText /* = NULL */) const
{
	return VERIFY(::SetWindowText(m_hWnd, pszText));
}

AFXINLINE WORD CWnd::SetWindowWord(int iIndex, WORD wValue) const
{
	return ::SetWindowWord(m_hWnd, iIndex, wValue);
}

AFXINLINE XLONG CWnd::SetWindowXLong(int iIndex, XLONG xlValue) const
{
	return ::SetWindowLongPtr(m_hWnd, iIndex, xlValue);
}

AFXINLINE BOOL CWnd::ShowCaret() const
{
	return VERIFY(::ShowCaret(m_hWnd));
}

AFXINLINE BOOL CWnd::ShowOwnedPopups() const
{
	return VERIFY(::ShowOwnedPopups(m_hWnd, TRUE));
}

AFXINLINE BOOL CWnd::ShowScrollBar(int iBar) const
{
	return VERIFY(::ShowScrollBar(m_hWnd, iBar, TRUE));
}

AFXINLINE BOOL CWnd::ShowWindow(int iShowCmd /* = SW_SHOW */) const
{
	return ::ShowWindow(m_hWnd, iShowCmd);
}

AFXINLINE BOOL CWnd::ShowWindowAsync(int iShowCmd) const
{
	return ::ShowWindowAsync(m_hWnd, iShowCmd);
}

AFXINLINE WORD CWnd::TileWindows(UINT nFlags, PCRECT pRect, UINT ceChildWnd /* = UNUSED */,
	PCHWND phaChildWnd /* = NULL */) const
{
	return VERIFY(::TileWindows(m_hWnd, nFlags, pRect, ceChildWnd, phaChildWnd));
}

AFXINLINE WORD CWnd::TileWindows(UINT nFlags, UINT ceChildWnd /* = UNUSED */,
	PCHWND phaChildWnd /* = NULL */) const
{
	return VERIFY(::TileWindows(m_hWnd, nFlags, NULL, ceChildWnd, phaChildWnd));
}

AFXINLINE BOOL CWnd::TrackPopupMenu(HMENU hMenu, int x, int y,
	PTPMPARAMS pTPMP, UINT nFlags) const
{
	return ::TrackPopupMenuEx(hMenu, nFlags, x, y, m_hWnd, pTPMP);
}

AFXINLINE BOOL CWnd::TrackPopupMenu(HMENU hMenu, int x, int y, UINT nFlags) const
{
	return ::TrackPopupMenu(hMenu, nFlags, x, y, UNUSED, m_hWnd, NULL);
}

AFXINLINE BOOL CWnd::TranslateAccelerator(HACCEL hAccel, PMSG pMsg) const
{
	return ::TranslateAccelerator(m_hWnd, hAccel, pMsg);
}

AFXINLINE BOOL CWnd::UnregisterShellHookWindow() const
{
	return VERIFY(::DeregisterShellHookWindow(m_hWnd));
}

AFXINLINE BOOL CWnd::UpdateLayeredWindow(HDC hDestDC, PPOINT pptDest, PSIZE pSize, HDC hSrcDC,
	PPOINT pptSrc, COLORREF crKey, PBLENDFUNCTION pBF, DWORD dwFlags) const
{
	return VERIFY(::UpdateLayeredWindow(m_hWnd, hDestDC, pptDest, pSize, hSrcDC,
		pptSrc, crKey, pBF, dwFlags));
}

AFXINLINE void CWnd::UpdateUiState(UINT nState, UINT nElement) const
{
	::SendMessage(m_hWnd, WM_UPDATEUISTATE, MAKEWPARAM(nState, nElement), UNUSED);
}

AFXINLINE BOOL CWnd::UpdateWindow() const
{
	return VERIFY(::UpdateWindow(m_hWnd));
}

AFXINLINE BOOL CWnd::ValidateRect(PCRECT pRect /* = NULL */) const
{
	return VERIFY(::ValidateRect(m_hWnd, pRect));
}

AFXINLINE BOOL CWnd::ValidateRgn(HRGN hRgn /* = NULL */) const
{
	return VERIFY(::ValidateRgn(m_hWnd, hRgn));
}

AFXINLINE BOOL CWnd::WindowFromPoint(POINT pt)
{
	m_hWnd = ::WindowFromPoint(pt);
	return (BOOL)m_hWnd;
}

#if (WINVER >= 0x0600)
AFXINLINE BOOL CWnd::AddClipboardFormatListener() const
{
	return VERIFY(::AddClipboardFormatListener(m_hWnd));
}

AFXINLINE BOOL CWnd::RemoveClipboardFormatListener() const
{
	return VERIFY(::RemoveClipboardFormatListener(m_hWnd));
}
#endif

#ifdef COMMCTRL
AFXINLINE BOOL CWnd::DragEnter(int x, int y) const
{
	return VERIFY(::ImageList_DragEnter(m_hWnd, x, y));
}

AFXINLINE BOOL CWnd::DragLeave() const
{
	return VERIFY(::ImageList_DragLeave(m_hWnd));
}

AFXINLINE BOOL CWnd::EnableFlatSB(UINT nBar, UINT nArrows /* = ESB_ENABLE_BOTH */) const
{
	return ::FlatSB_EnableScrollBar(m_hWnd, nBar, nArrows);
}

AFXINLINE void CWnd::GetClientRect(PRECT pRect, PINT piaInfo) const
{
	::GetEffectiveClientRect(m_hWnd, pRect, piaInfo);
}

AFXINLINE BOOL CWnd::GetFlatSBInfo(int iBar, PSCROLLINFO pSI) const
{
	return VERIFY(::FlatSB_GetScrollInfo(m_hWnd, iBar, pSI));
}

AFXINLINE int CWnd::GetFlatSBPos(int iBar) const
{
	return ::FlatSB_GetScrollPos(m_hWnd, iBar);
}

AFXINLINE BOOL CWnd::GetFlatSBProp(int iProp, PINT piValue) const
{
	return VERIFY(::FlatSB_GetScrollProp(m_hWnd, iProp, piValue));
}

AFXINLINE BOOL CWnd::GetFlatSBProp(int iProp, PCOLORREF pcrValue) const
{
	return VERIFY(::FlatSB_GetScrollProp(m_hWnd, iProp, (PINT)pcrValue));
}

AFXINLINE BOOL CWnd::GetFlatSBProp(int iProp, PHPALETTE phPalette) const
{
	return VERIFY(::FlatSB_GetScrollProp(m_hWnd, iProp, (PINT)phPalette));
}

AFXINLINE BOOL CWnd::GetFlatSBRange(int iBar, PINT piMinPos, PINT piMaxPos) const
{
	return VERIFY(::FlatSB_GetScrollRange(m_hWnd, iBar, piMinPos, piMaxPos));
}

AFXINLINE BOOL CWnd::HideFlatSB(int iBar) const
{
	return VERIFY(::FlatSB_ShowScrollBar(m_hWnd, iBar, FALSE));
}

AFXINLINE BOOL CWnd::InitializeFlatSB() const
{
	return VERIFY(::InitializeFlatSB(m_hWnd));
}

AFXINLINE int CWnd::SetFlatSBInfo(int iBar, PSCROLLINFO pSI, BOOL bRedraw /* = TRUE */) const
{
	return ::FlatSB_SetScrollInfo(m_hWnd, iBar, pSI, bRedraw);
}

AFXINLINE int CWnd::SetFlatSBPos(int iBar, int iPos, BOOL bRedraw /* = TRUE */) const
{
	return ::FlatSB_SetScrollPos(m_hWnd, iBar, iPos, bRedraw);
}

AFXINLINE BOOL CWnd::SetFlatSBProp(UINT nProp, int iValue, BOOL bRedraw /* = TRUE */) const
{
	return VERIFY(::FlatSB_SetScrollProp(m_hWnd, nProp, iValue, bRedraw));
}

AFXINLINE BOOL CWnd::SetFlatSBProp(UINT nProp, HPALETTE hPalette,
	BOOL bRedraw /* = TRUE */) const
{
	return VERIFY(::FlatSB_SetScrollProp(m_hWnd, nProp, (int)hPalette, bRedraw));
}

AFXINLINE BOOL CWnd::SetFlatSBRange(int iBar, int iMinPos, int iMaxPos,
	BOOL bRedraw /* = FALSE */) const
{
	return VERIFY(::FlatSB_SetScrollRange(m_hWnd, iBar, iMinPos, iMaxPos, bRedraw));
}

AFXINLINE BOOL CWnd::ShowFlatSB(int iBar) const
{
	return VERIFY(::FlatSB_ShowScrollBar(m_hWnd, iBar, TRUE));
}

AFXINLINE BOOL CWnd::ShowMenuCtrl(UINT nItemId, PINT piaInfo) const
{
	return VERIFY(::ShowHideMenuCtl(m_hWnd, nItemId, piaInfo));
}

AFXINLINE HRESULT CWnd::UninitializeFlatSB() const
{
	return VERIFY_HRESULT(::UninitializeFlatSB(m_hWnd));
}
#endif

#ifdef DDE
AFXINLINE BOOL CWnd::DdeAck(HWND hPartnerWnd, DDEACK ddea, ATOM aItemName) const
{
	return VERIFY(::PostMessage(hPartnerWnd, WM_DDE_ACK, (WPARAM)m_hWnd,
		VERIFY(::PackDDElParam(WM_DDE_ACK, *(PWORD)&ddea, aItemName))));
}

AFXINLINE BOOL CWnd::DdeAck(HWND hClientWnd, DDEACK ddea, HMEMORY hCmdStr) const
{
	return VERIFY(::PostMessage(hClientWnd, WM_DDE_ACK, (WPARAM)m_hWnd,
		VERIFY(::PackDDElParam(WM_DDE_ACK, *(PWORD)&ddea, (UINT)hCmdStr))));
}

AFXINLINE void CWnd::DdeAck(HWND hClientWnd, ATOM aAppName, ATOM aTopicName) const
{
	::SendMessage(hClientWnd, WM_DDE_ACK, (WPARAM)m_hWnd, MAKELPARAM(aAppName, aTopicName));
}

AFXINLINE BOOL CWnd::DdeAdvise(HWND hServerWnd, HMEMORY hDDEA, ATOM aItemName) const
{
	return VERIFY(::PostMessage(hServerWnd, WM_DDE_ADVISE, (WPARAM)m_hWnd,
		VERIFY(::PackDDElParam(WM_DDE_ADVISE, (UINT)hDDEA, aItemName))));
}

AFXINLINE BOOL CWnd::DdeData(HWND hClientWnd, HMEMORY hDDED, ATOM aItemName) const
{
	return VERIFY(::PostMessage(hClientWnd, WM_DDE_DATA, (WPARAM)m_hWnd,
		VERIFY(::PackDDElParam(WM_DDE_DATA, (UINT)hDDED, aItemName))));
}

AFXINLINE BOOL CWnd::DdeExecute(HWND hServerWnd, HMEMORY hCmdStr) const
{
	return VERIFY(::PostMessage(hServerWnd, WM_DDE_EXECUTE, (WPARAM)m_hWnd, (LPARAM)hCmdStr));
}

AFXINLINE BOOL CWnd::DdeImpersonateClient(HWND hClientWnd) const
{
	return VERIFY(::ImpersonateDdeClientWindow(hClientWnd, m_hWnd));
}

AFXINLINE void CWnd::DdeInitiate(HWND hServerWnd, ATOM aAppName, ATOM aTopicName) const
{
	::SendMessage(hServerWnd, WM_DDE_INITIATE, (WPARAM)m_hWnd,
		MAKELPARAM(aAppName, aTopicName));
}

AFXINLINE void CWnd::DdeInitiate(ATOM aAppName, ATOM aTopicName) const
{
	::SendMessage(HWND_BROADCAST, WM_DDE_INITIATE, (WPARAM)m_hWnd,
		MAKELPARAM(aAppName, aTopicName));
}

AFXINLINE BOOL CWnd::DdePoke(HWND hServerWnd, HMEMORY hDDEP, ATOM aItemName) const
{
	return VERIFY(::PostMessage(hServerWnd, WM_DDE_POKE, (WPARAM)m_hWnd,
		VERIFY(::PackDDElParam(WM_DDE_POKE, (UINT)hDDEP, aItemName))));
}

AFXINLINE BOOL CWnd::DdeRequest(HWND hServerWnd, ATOM aItemName, UINT nFormat) const
{
	return VERIFY(::PostMessage(hServerWnd, WM_DDE_REQUEST, (WPARAM)m_hWnd,
		MAKELPARAM(nFormat, aItemName)));
}

AFXINLINE BOOL CWnd::DdeSetQualityService(const PSECURITY_QUALITY_OF_SERVICE pNewSQOS,
	PSECURITY_QUALITY_OF_SERVICE pPrevSQOS /* = NULL */) const
{
	return VERIFY(::DdeSetQualityOfService(m_hWnd, pNewSQOS, pPrevSQOS));
}

AFXINLINE BOOL CWnd::DdeTerminate(HWND hPartnerWnd) const
{
	return VERIFY(::PostMessage(hPartnerWnd, WM_DDE_TERMINATE, (WPARAM)m_hWnd, UNUSED));
}

AFXINLINE BOOL CWnd::DdeUnadvise(HWND hServerWnd, ATOM aItemName, UINT nFormat) const
{
	return VERIFY(::PostMessage(hServerWnd, WM_DDE_UNADVISE, (WPARAM)m_hWnd,
		MAKELPARAM(nFormat, aItemName)));
}
#endif

#ifdef SHLAPI
AFXINLINE void CWnd::DragAcceptFiles(BOOL bAccept /* = TRUE */) const
{
	::DragAcceptFiles(m_hWnd, bAccept);
}
#endif

#ifdef WINSOCK
AFXINLINE HWSATASK CWnd::WSAAsyncGetHostByAddr(UINT nMsg, PULONG pulAddr, int iAddr,
	PHOSTENTEX pHEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetHostByAddr(m_hWnd, nMsg, (PCCHAR)pulAddr,
		sizeof(ULONG), iAddr, (PCHAR)pHEX, sizeof(HOSTENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetHostByName(UINT nMsg, PCSTR pszName, PHOSTENTEX pHEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetHostByName(m_hWnd, nMsg, pszName,
		(PCHAR)pHEX, sizeof(HOSTENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetProtoByName(UINT nMsg, PCSTR pszName,
	PPROTOENTEX pPEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetProtoByName(m_hWnd, nMsg, pszName,
		(PCHAR)pPEX, sizeof(PROTOENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetProtoByNumber(UINT nMsg, int iProtocol,
	PPROTOENTEX pPEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetProtoByNumber(m_hWnd, nMsg, iProtocol,
		(PCHAR)pPEX, sizeof(PROTOENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetServByName(UINT nMsg, PCSTR pszName, PCSTR pszProtocol,
	PSERVENTEX pSEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetServByName(m_hWnd, nMsg, pszName, pszProtocol,
		(PCHAR)pSEX, sizeof(SERVENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetServByName(UINT nMsg, PCSTR pszName, PSERVENTEX pSEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetServByName(m_hWnd, nMsg, pszName, NULL,
		(PCHAR)pSEX, sizeof(SERVENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetServByPort(UINT nMsg, int iPort, PCSTR pszProtocol,
	PSERVENTEX pSEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetServByPort(m_hWnd, nMsg, iPort, pszProtocol,
		(PCHAR)pSEX, sizeof(SERVENTEX)));
}

AFXINLINE HWSATASK CWnd::WSAAsyncGetServByPort(UINT nMsg, int iPort, PSERVENTEX pSEX) const
{
	return (HWSATASK)VERIFY(::WSAAsyncGetServByPort(m_hWnd, nMsg, iPort, NULL,
		(PCHAR)pSEX, sizeof(SERVENTEX)));
}

AFXINLINE int CWnd::WSAAsyncSelect(HSOCKET hSocket, UINT nMsg /* = UNUSED */,
	LONG lEvent /* = UNUSED */) const
{
	return UNEQUAL(::WSAAsyncSelect(hSocket, m_hWnd, nMsg, lEvent), SOCKET_ERROR);
}

AFXINLINE void CWnd::WSARemoveMessage(UINT nMsg, HWSATASK hWsaTask) const
{
	::_WSARemoveMessage(m_hWnd, nMsg, (HSOCKET)hWsaTask);
}

AFXINLINE void CWnd::WSARemoveMessage(UINT nMsg, HSOCKET hSocket) const
{
	::_WSARemoveMessage(m_hWnd, nMsg, hSocket);
}

AFXINLINE void CWnd::WSARemoveMessage(UINT nMsg) const
{
	MSG msg;
	while(::PeekMessage(&msg, m_hWnd, nMsg, nMsg, PM_REMOVE));
}
#endif

#ifdef WINSPOOL
AFXINLINE BOOL CWnd::PrinterProperties(HPRINTER hPrinter) const
{
	return VERIFY(::PrinterProperties(m_hWnd, hPrinter));
}
#endif

#ifdef WINTHEME
AFXINLINE HRESULT CWnd::SetWindowTheme(PCWSTR pszAppName /* = NULL */,
	PCWSTR pszClsIdList /* = NULL */)
{
	return VERIFY_HRESULT(::SetWindowTheme(m_hWnd, pszAppName, pszClsIdList));
}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// CFrameWnd

// Only can be derived (cannot be instantiated)
class CFrameWnd : public CWnd
{
public:
	BOOL CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hParentWnd, HMENU hMenu = NULL);
	BOOL CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hParentWnd, HMENU hMenu = NULL);
	BOOL CreateWindow(PCTSTR pszClassName);

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_FRAME };

protected:
	CFrameWnd();
};

AFXINLINE CFrameWnd::CFrameWnd()
{
	// Does nothing
}

AFXINLINE BOOL CFrameWnd::CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName,
	DWORD dwStyle, int x, int y, int iWidth, int iHeight, HWND hParentWnd,
	HMENU hMenu /* = NULL */)
{
	ASSERT(!_m_pWnd && !(dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(dwExStyle, pszClassName, pszWndName,
		dwStyle, x, y, iWidth, iHeight, hParentWnd, hMenu, NULL, NULL));
}

AFXINLINE BOOL CFrameWnd::CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
	int x, int y, int iWidth, int iHeight, HWND hParentWnd, HMENU hMenu /* = NULL */)
{
	ASSERT(!_m_pWnd && !(dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(UNUSED, pszClassName, pszWndName, dwStyle,
		x, y, iWidth, iHeight, hParentWnd, hMenu, NULL, NULL));
}

AFXINLINE BOOL CFrameWnd::CreateWindow(PCTSTR pszClassName)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(UNUSED, pszClassName, NULL, UNUSED,
		0, 0, 0, 0, HWND_MESSAGE, NULL, NULL, NULL));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CChildWnd

// Only can be derived (cannot be instantiated)
class CChildWnd : public CWnd
{
public:
	BOOL CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hParentWnd, UINT nChildId);
	BOOL CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hParentWnd, UINT nChildId);
	BOOL CreateWindow(PCTSTR pszClassName, HWND hParentWnd, UINT nChildId);

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_CHILD };

protected:
	CChildWnd();
};

AFXINLINE CChildWnd::CChildWnd()
{
	// Does nothing
}

AFXINLINE BOOL CChildWnd::CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName,
	DWORD dwStyle, int x, int y, int iWidth, int iHeight, HWND hParentWnd, UINT nChildId)
{
	ASSERT(!_m_pWnd && (dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(dwExStyle, pszClassName, pszWndName,
		dwStyle, x, y, iWidth, iHeight, hParentWnd, (HMENU)nChildId, NULL, NULL));
}

AFXINLINE BOOL CChildWnd::CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName,
	DWORD dwStyle, int x, int y, int iWidth, int iHeight, HWND hParentWnd, UINT nChildId)
{
	ASSERT(!_m_pWnd && (dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(UNUSED, pszClassName, pszWndName,
		dwStyle, x, y, iWidth, iHeight, hParentWnd, (HMENU)nChildId, NULL, NULL));
}

AFXINLINE BOOL CChildWnd::CreateWindow(PCTSTR pszClassName, HWND hParentWnd, UINT nChildId)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(UNUSED, pszClassName, NULL, WS_CHILD,
		0, 0, 0, 0, hParentWnd, (HMENU)nChildId, NULL, NULL));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CFormWnd

// Only can be derived (cannot be instantiated)
// CLASS directive in template must be used, and
// lpfnWndProc of WNDCLASS must be also set to WindowProc
class CFormWnd : public CWnd
{
public:
	BOOL CreateForm(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hParentWnd);
	BOOL CreateForm(HINSTANCE hInstance, UINT nTmplId, HWND hParentWnd);
	BOOL CreateForm(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hParentWnd);
	int DialogForm(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hParentWnd);
	int DialogForm(HINSTANCE hInstance, UINT nTmplId, HWND hParentWnd);
	int DialogForm(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hParentWnd);

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_FORM };

protected:
	CFormWnd();
};

AFXINLINE CFormWnd::CFormWnd()
{
	// Does nothing
}

AFXINLINE BOOL CFormWnd::CreateForm(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateDialogParam(hInstance, pszTemplate, hParentWnd, NULL, UNUSED));
}

AFXINLINE BOOL CFormWnd::CreateForm(HINSTANCE hInstance, UINT nTmplId, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateDialogParam(hInstance, MAKERESPTR(nTmplId), hParentWnd,
		NULL, UNUSED));
}

AFXINLINE BOOL CFormWnd::CreateForm(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateDialogIndirectParam(hInstance, pDT, hParentWnd, NULL, UNUSED));
}

AFXINLINE int CFormWnd::DialogForm(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return UNEQUAL(::DialogBoxParam(hInstance, pszTemplate, hParentWnd, NULL, UNUSED), RV_ERR);
}

AFXINLINE int CFormWnd::DialogForm(HINSTANCE hInstance, UINT nTmplId, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return UNEQUAL(::DialogBoxParam(hInstance, MAKERESPTR(nTmplId), hParentWnd,
		NULL, UNUSED), RV_ERR);
}

AFXINLINE int CFormWnd::DialogForm(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hParentWnd)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return UNEQUAL(::DialogBoxIndirectParam(hInstance, pDT, hParentWnd, NULL, UNUSED), RV_ERR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CDialog

// Only can be derived (cannot be instantiated)
// lpfnWndProc of WNDCLASS must be set to DefDlgProc if CLASS directive in template is used
class CDialog : public CWnd
{
public:
	BOOL CreateDialog(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hOwnerWnd);
	BOOL CreateDialog(HINSTANCE hInstance, UINT nTmplId, HWND hOwnerWnd);
	BOOL CreateDialog(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hOwnerWnd);
	LRESULT DefWindowProc(UINT nMsg = UNUSED, WPARAM wParam = UNUSED,
		LPARAM lParam = UNUSED) const;
	int DialogBox(HINSTANCE hInstance, PCTSTR pszTemplate, HWND hOwnerWnd);
	int DialogBox(HINSTANCE hInstance, UINT nTmplId, HWND hOwnerWnd);
	int DialogBox(HINSTANCE hInstance, PCDLGTEMPLATE pDT, HWND hOwnerWnd);
	BOOL EndDialog(int iResult = UNUSED) const;
	UINT GetDefId() const;
	BOOL IsDialogMessage(PMSG pMsg) const;
	BOOL MapDialogRect(PRECT pRect) const;
	void Reposition() const;
	void SetDefId(UINT nButtonId) const;
	void SetNextTabItem(HWND hFocusWnd) const;
	void SetNextTabItem(BOOL bPrevCtrl = FALSE) const;

	// Moves the dialog box to the center of its owner window
	// Should be used when processing WM_INITDIALOG only
	// Returns: TRUE if the function succeeds
	BOOL MoveDialogCenter() const;

protected:
	// Window type and default return value (used internally only)
	enum { _eWndType = _WT_DIALOG, _eResult = TRUE };

protected:
	CDialog();
};

AFXINLINE CDialog::CDialog()
{
	// Does nothing
}

AFXINLINE LRESULT CDialog::DefWindowProc(UINT nMsg /* = UNUSED */,
	WPARAM wParam /* = UNUSED */, LPARAM lParam /* = UNUSED */) const
{
	return FALSE;
}

AFXINLINE BOOL CDialog::EndDialog(int iResult /* = UNUSED */) const
{
	return VERIFY(::EndDialog(m_hWnd, iResult));
}

AFXINLINE UINT CDialog::GetDefId() const
{
	return ::SendMessage(m_hWnd, DM_GETDEFID, UNUSED, UNUSED);
}

AFXINLINE BOOL CDialog::IsDialogMessage(PMSG pMsg) const
{
	ASSERT_HWND(m_hWnd);
	return ::IsDialogMessage(m_hWnd, pMsg);
}

AFXINLINE BOOL CDialog::MapDialogRect(PRECT pRect) const
{
	return VERIFY(::MapDialogRect(m_hWnd, pRect));
}

AFXINLINE void CDialog::Reposition() const
{
	::SendMessage(m_hWnd, DM_REPOSITION, UNUSED, UNUSED);
}

AFXINLINE void CDialog::SetDefId(UINT nButtonId) const
{
	::SendMessage(m_hWnd, DM_SETDEFID, nButtonId, UNUSED);
}

AFXINLINE void CDialog::SetNextTabItem(HWND hFocusWnd) const
{
	::PostMessage(m_hWnd, WM_NEXTDLGCTL, (WPARAM)hFocusWnd, TRUE);
}

AFXINLINE void CDialog::SetNextTabItem(BOOL bPrevCtrl /* = FALSE */) const
{
	::PostMessage(m_hWnd, WM_NEXTDLGCTL, bPrevCtrl, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CMdiFrameWnd

// Window class name
#define WC_MDICLIENT _T("MdiClient")

// Only can be derived (cannot be instantiated)
class CMdiFrameWnd : public CWnd
{
public:
	// MDI client window
	HWND m_hClientWnd;

public:
	BOOL CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HMENU hMenu = NULL);
	BOOL CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HMENU hMenu = NULL);
	BOOL CreateWindow(DWORD dwExStyle, DWORD dwStyle, int x, int y, int iWidth, int iHeight,
		UINT nClientId, PCCLIENTCREATESTRUCT pCCS);
	BOOL CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight, UINT nClientId,
		PCCLIENTCREATESTRUCT pCCS);
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	BOOL EnumChildWindows(WNDENUMPROC epName, LPARAM lParam = UNUSED) const;
	void MdiActivate(HWND hChildWnd) const;
	void MdiCascade(UINT nFlags = MDITILE_SKIPDISABLED) const;
	void MdiDestroy(HWND hChildWnd) const;
	HWND MdiGetActive(PBOOL pbMaximized = NULL) const;
	void MdiIconArrange() const;
	void MdiMaximize(HWND hChildWnd) const;
	void MdiNext(HWND hChildWnd = NULL, BOOL bPrevWnd = FALSE) const;
	HMENU MdiRefreshMenu() const;
	void MdiRestore(HWND hChildWnd) const;
	HMENU MdiSetMenu(HMENU hFrameMenu, HMENU hWndMenu = NULL) const;
	BOOL MdiTile(UINT nFlags = MDITILE_SKIPDISABLED) const;
	BOOL TranslateMDISysAccel(PMSG pMsg) const;

	// Updates the edge of MDI client window
	// Should be used when processing WM_DESTROY, WM_MDIACTIVATE and WM_WINDOWPOSCHANGING
	// messages in all MDI child window procedures only
	// Returns: TRUE if the window is updated
	BOOL UpdateClientEdge(HWND hChildWnd) const;

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_MDIFRAME };

protected:
	CMdiFrameWnd();
};

AFXINLINE CMdiFrameWnd::CMdiFrameWnd()
{
	// Does nothing
}

AFXINLINE BOOL CMdiFrameWnd::CreateWindow(DWORD dwExStyle, PCTSTR pszClassName,
	PCTSTR pszWndName, DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	HMENU hMenu /* = NULL */)
{
	ASSERT(!_m_pWnd && !(dwStyle & WS_CHILD));
	_m_pWnd      = this;
	m_hClientWnd = NULL;
	return (BOOL)VERIFY(::CreateWindowEx(dwExStyle, pszClassName, pszWndName, dwStyle,
		x, y, iWidth, iHeight, HWND_DESKTOP, hMenu, NULL, NULL));
}

AFXINLINE BOOL CMdiFrameWnd::CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
	int x, int y, int iWidth, int iHeight, HMENU hMenu /* = NULL */)
{
	ASSERT(!_m_pWnd && !(dwStyle & WS_CHILD));
	_m_pWnd      = this;
	m_hClientWnd = NULL;
	return (BOOL)VERIFY(::CreateWindowEx(UNUSED, pszClassName, pszWndName, dwStyle,
		x, y, iWidth, iHeight, HWND_DESKTOP, hMenu, NULL, NULL));
}

AFXINLINE BOOL CMdiFrameWnd::CreateWindow(DWORD dwExStyle, DWORD dwStyle,
	int x, int y, int iWidth, int iHeight, UINT nClientId, PCCLIENTCREATESTRUCT pCCS)
{
	ASSERT(!_m_pWnd && (dwStyle & WS_CHILD));
	m_hClientWnd = ::CreateWindowEx(dwExStyle, WC_MDICLIENT, NULL, dwStyle,
		x, y, iWidth, iHeight, m_hWnd, (HMENU)nClientId, NULL, (PVOID)pCCS);
	return (BOOL)VERIFY(m_hClientWnd);
}

AFXINLINE BOOL CMdiFrameWnd::CreateWindow(DWORD dwStyle, int x, int y, int iWidth, int iHeight,
	UINT nClientId, PCCLIENTCREATESTRUCT pCCS)
{
	ASSERT(!_m_pWnd && (dwStyle & WS_CHILD));
	m_hClientWnd = ::CreateWindowEx(UNUSED, WC_MDICLIENT, NULL, dwStyle, x, y, iWidth, iHeight,
		m_hWnd, (HMENU)nClientId, NULL, (PVOID)pCCS);
	return (BOOL)VERIFY(m_hClientWnd);
}

AFXINLINE LRESULT CMdiFrameWnd::DefWindowProc(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return ::DefFrameProc(m_hWnd, m_hClientWnd, nMsg, wParam, lParam);
}

AFXINLINE BOOL CMdiFrameWnd::EnumChildWindows(WNDENUMPROC epName,
	LPARAM lParam /* = UNUSED */) const
{
	return ::EnumChildWindows(m_hClientWnd, epName, lParam);
}

AFXINLINE void CMdiFrameWnd::MdiActivate(HWND hChildWnd) const
{
	::SendMessage(m_hClientWnd, WM_MDIACTIVATE, (WPARAM)hChildWnd, UNUSED);
}

AFXINLINE void CMdiFrameWnd::MdiCascade(UINT nFlags /* = MDITILE_SKIPDISABLED */) const
{
	::SendMessage(m_hClientWnd, WM_MDICASCADE, nFlags, UNUSED);
}

AFXINLINE void CMdiFrameWnd::MdiDestroy(HWND hChildWnd) const
{
	::SendMessage(m_hClientWnd, WM_MDIDESTROY, (WPARAM)hChildWnd, UNUSED);
}

AFXINLINE HWND CMdiFrameWnd::MdiGetActive(PBOOL pbMaximized /* = NULL */) const
{
	return (HWND)::SendMessage(m_hClientWnd, WM_MDIGETACTIVE, UNUSED, (LPARAM)pbMaximized);
}

AFXINLINE void CMdiFrameWnd::MdiIconArrange() const
{
	::SendMessage(m_hClientWnd, WM_MDIICONARRANGE, UNUSED, UNUSED);
}

AFXINLINE void CMdiFrameWnd::MdiMaximize(HWND hChildWnd) const
{
	::SendMessage(m_hClientWnd, WM_MDIMAXIMIZE, (WPARAM)hChildWnd, UNUSED);
}

AFXINLINE void CMdiFrameWnd::MdiNext(HWND hChildWnd /* = NULL */,
	BOOL bPrevWnd /* = FALSE */) const
{
	::SendMessage(m_hClientWnd, WM_MDINEXT, (WPARAM)hChildWnd, bPrevWnd);
}

AFXINLINE HMENU CMdiFrameWnd::MdiRefreshMenu() const
{
	return (HMENU)VERIFY(::SendMessage(m_hClientWnd, WM_MDIREFRESHMENU, UNUSED, UNUSED));
}

AFXINLINE void CMdiFrameWnd::MdiRestore(HWND hChildWnd) const
{
	::SendMessage(m_hClientWnd, WM_MDIRESTORE, (WPARAM)hChildWnd, UNUSED);
}

AFXINLINE HMENU CMdiFrameWnd::MdiSetMenu(HMENU hFrameMenu, HMENU hWndMenu /* = NULL */) const
{
	return (HMENU)VERIFY(::SendMessage(m_hClientWnd, WM_MDISETMENU,
		(WPARAM)hFrameMenu, (LPARAM)hWndMenu));
}

AFXINLINE BOOL CMdiFrameWnd::MdiTile(UINT nFlags /* = MDITILE_SKIPDISABLED */) const
{
	return VERIFY(::SendMessage(m_hClientWnd, WM_MDITILE, nFlags, UNUSED));
}

AFXINLINE BOOL CMdiFrameWnd::TranslateMDISysAccel(PMSG pMsg) const
{
	return ::TranslateMDISysAccel(m_hClientWnd, pMsg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// CMdiChildWnd

// Only can be derived (cannot be instantiated)
class CMdiChildWnd : public CWnd
{
public:
	BOOL CreateWindow(DWORD dwExStyle, PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hClientWnd);
	BOOL CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
		int x, int y, int iWidth, int iHeight, HWND hClientWnd);
	BOOL CreateWindow(HWND hClientWnd, PCMDICREATESTRUCT pMDICS);
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam = UNUSED, LPARAM lParam = UNUSED) const;
	void DestroyWindow(HWND hClientWnd) const;

protected:
	// Window type (used internally only)
	enum { _eWndType = _WT_MDICHILD };

protected:
	CMdiChildWnd();
};

AFXINLINE CMdiChildWnd::CMdiChildWnd()
{
	// Does nothing
}

AFXINLINE BOOL CMdiChildWnd::CreateWindow(DWORD dwExStyle, PCTSTR pszClassName,
	PCTSTR pszWndName, DWORD dwStyle, int x, int y, int iWidth, int iHeight, HWND hClientWnd)
{
	ASSERT(!_m_pWnd && (dwExStyle & WS_EX_MDICHILD) && (dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateWindowEx(dwExStyle, pszClassName, pszWndName, dwStyle,
		x, y, iWidth, iHeight, hClientWnd, NULL, NULL, NULL));
}

AFXINLINE BOOL CMdiChildWnd::CreateWindow(PCTSTR pszClassName, PCTSTR pszWndName, DWORD dwStyle,
	int x, int y, int iWidth, int iHeight, HWND hClientWnd)
{
	ASSERT(!_m_pWnd && (dwStyle & WS_CHILD));
	_m_pWnd = this;
	return (BOOL)VERIFY(::CreateMDIWindow(pszClassName, pszWndName, dwStyle,
		x, y, iWidth, iHeight, hClientWnd, NULL, UNUSED));
}

AFXINLINE BOOL CMdiChildWnd::CreateWindow(HWND hClientWnd, PCMDICREATESTRUCT pMDICS)
{
	ASSERT(!_m_pWnd);
	_m_pWnd = this;
	return VERIFY(::SendMessage(hClientWnd, WM_MDICREATE, UNUSED, (LPARAM)pMDICS));
}

AFXINLINE LRESULT CMdiChildWnd::DefWindowProc(UINT nMsg, WPARAM wParam /* = UNUSED */,
	LPARAM lParam /* = UNUSED */) const
{
	return ::DefMDIChildProc(m_hWnd, nMsg, wParam, lParam);
}

AFXINLINE void CMdiChildWnd::DestroyWindow(HWND hClientWnd) const
{
	::SendMessage(hClientWnd, WM_MDIDESTROY, (WPARAM)m_hWnd, UNUSED);
}

// End of _WINDOW_INCLUDED_
#endif
