////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// General.cpp - General Macros and Functions Implementation File                             //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>

////////////////////////////////////////////////////////////////////////////////////////////////
// Duplicates a string
// Returns: A pointer to the storage location for the copied string

PWSTR WINAPI lstrdup(PCWSTR pszText)
{
	// String pointer cannot be NULL
	ASSERT_STR(pszText);
	if(!pszText)
		return NULL;

	// Allocates the required storage
	UINT  cbAlloc   = (lstrlenW(pszText) + 1) * sizeof(WCHAR);
	PWSTR pszMemory = (PWSTR)VERIFY(malloc(cbAlloc));
	if(!pszMemory)
		return NULL;

	// Copies the string
	memcpy(pszMemory, pszText, cbAlloc);
	return pszMemory;
}

PSTR WINAPI lstrdup(PCSTR pszText)
{
	// String pointer cannot be NULL
	ASSERT_STR(pszText);
	if(!pszText)
		return NULL;

	// Allocates the required storage
	UINT cbAlloc   = lstrlenA(pszText) + 1;
	PSTR pszMemory = (PSTR)VERIFY(malloc(cbAlloc));
	if(!pszMemory)
		return NULL;

	// Copies the string
	memcpy(pszMemory, pszText, cbAlloc);
	return pszMemory;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Normalizes a rectangle so that both the width and height are positive

void WINAPI NormalizeRect(PRECT pRect)
{
	// Exchanges the horizontal values, if necessary
	LONG lTemp;
	if(pRect->left > pRect->right)
	{
		lTemp        = pRect->left;
		pRect->left  = pRect->right;
		pRect->right = lTemp;
	}

	// Exchanges the vertical values, if necessary
	if(pRect->top > pRect->bottom)
	{
		lTemp         = pRect->top;
		pRect->top    = pRect->bottom;
		pRect->bottom = lTemp;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a character is a control character
// Returns: TRUE if the character is a control character

BOOL WINAPI IsControl(WCHAR cChar)
{
	// Normal controls
	if(cChar && (cChar < 0x20))
		return TRUE;

	// Unicode control page
	if((cChar & 0xFF00) != 0x2000)
		return FALSE;

	// Unicode controls
	SCWCHAR scCtrl[] =
	{
		0x200E,  // Left-to-right mark (LRM)
		0x200F,  // Right-to-left mark (RLM)
		0x200D,  // Zero width joiner (ZWJ)
		0x200C,  // Zero width non-joiner (ZWNJ)
		0x202A,  // Start of left-to-right embedding (LRE)
		0x202B,  // Start of right-to-left embedding (RLE)
		0x202D,  // Start of left-to-right override (LRO)
		0x202E,  // Start of right-to-left override (RLO)
		0x202C,  // Pop directional formatting (PDF)
		0x206E,  // National digit shapes substitution (NADS)
		0x206F,  // Nominal (European) digit shapes (NODS)
		0x206B,  // Activate symmetric swapping (ASS)
		0x206A,  // Inhibit symmetric swapping (ISS)
		0x206D,  // Activate Arabic form shaping (AAFS)
		0x206C,  // Inhibit Arabic form shaping (IAFS)
	};
	int i;
	i = 0;
	do
	{
		if(cChar == scCtrl[i])
			return TRUE;

		i++;
	}
	while(i < countof(scCtrl));
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Trims leading and trailing whitespace characters from a string
// Returns: Length of the string

int WINAPI TrimString(PWSTR pszText)
{
	// Finds the first non-whitespace character
	ASSERT_STR(pszText);
	for(PWCHAR pcStart = pszText; IsSpace(*pcStart); pcStart++);

	// Finds the first whitespace of trailing character
	for(PWCHAR pcEnd = pcStart, pcTemp = NULL; *pcEnd; pcEnd++)
	{
		if(IsSpace(*pcEnd))
		{
			if(!pcTemp)
				pcTemp = pcEnd;
			continue;
		}
		pcTemp = NULL;
	}

	// Gets the length of the string and trims it, if necessary
	if(pcTemp)
		*pcTemp = CHR_NULL;
	pcStart--;
	pcTemp = pszText - 1;
	do
	{
		pcTemp++;
		do
			pcStart++;
		while(IsControl(*pcStart));
		*pcTemp = *pcStart;
	}
	while(*pcTemp);
	return (pcTemp - pszText);
}

int WINAPI TrimString(PSTR pszText)
{
	// Finds the first non-whitespace character
	ASSERT_STR(pszText);
	for(PCHAR pcStart = pszText; IsSpace(*pcStart); pcStart++);

	// Finds the first whitespace of trailing character
	for(PCHAR pcEnd = pcStart, pcTemp = NULL; *pcEnd; pcEnd++)
	{
		if(IsSpace(*pcEnd))
		{
			if(!pcTemp)
				pcTemp = pcEnd;
			continue;
		}
		pcTemp = NULL;
	}

	// Gets the length of the string and trims it, if necessary
	if(pcTemp)
		*pcTemp = CHR_NULL;
	pcStart--;
	pcTemp = pszText - 1;
	do
	{
		pcTemp++;
		do
			pcStart++;
		while(IsControl(*pcStart));
		*pcTemp = *pcStart;
	}
	while(*pcTemp);
	return (pcTemp - pszText);
}
