////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// Window.cpp - Window Classes Implementation File                                            //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>

// Window object pointer and size of this object (used internally only)
CWnd* CWnd::_m_pWnd;
#ifdef DEBUG
UINT  CWnd::_m_cbObject;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
// Moves the dialog box to the center of its owner window
// Should be used when processing WM_INITDIALOG only
// Returns: TRUE if the function succeeds

BOOL CDialog::MoveDialogCenter() const
{
	// Gets the owner window, if any, or the desktop window
	ASSERT_HWND(m_hWnd);
	HWND hOwnerWnd = ::GetWindow(m_hWnd, GW_OWNER);
	if(!hOwnerWnd)
		hOwnerWnd = ::GetDesktopWindow();

	// Gets rectangle areas
	RECT rcWork, rcOwner, rcDlg;
	::SystemParametersInfo(SPI_GETWORKAREA, UNUSED, &rcWork, UNUSED);
	::GetWindowRect(hOwnerWnd, &rcOwner);
	::GetWindowRect(m_hWnd, &rcDlg);
	ASSERT(!::IsRectEmpty(&rcOwner) && !::IsRectEmpty(&rcDlg));

	// Moves the dialog box
	SIZE sizeDlg = { rcDlg.right - rcDlg.left, rcDlg.bottom - rcDlg.top };
	return VERIFY(::SetWindowPos(m_hWnd, NULL,
		clamp((rcOwner.left + rcOwner.right - sizeDlg.cx) / 2,
		rcWork.left + 7, rcWork.right - sizeDlg.cx - 7),
		clamp((rcOwner.top + rcOwner.bottom - sizeDlg.cy) / 2,
		rcWork.top + 7, rcWork.bottom - sizeDlg.cy - 7), UNUSED, UNUSED,
		SWP_NOACTIVATE | SWP_NOREDRAW | SWP_NOSIZE | SWP_NOZORDER));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Updates the edge of MDI client window
// Should be used when processing WM_DESTROY, WM_MDIACTIVATE and WM_WINDOWPOSCHANGING
// messages in all MDI child window procedures only
// Returns: TRUE if the window is updated

BOOL CMdiFrameWnd::UpdateClientEdge(HWND hChildWnd) const
{
	// Gets the currently active child window
	// No update is necessary if the specified child window is not currently active
	ASSERT(::IsWindow(m_hClientWnd) && (!hChildWnd || ::IsWindow(hChildWnd)));
	BOOL bMaximized;
	HWND hActiveWnd = (HWND)::SendMessage(m_hClientWnd, WM_MDIGETACTIVE,
		UNUSED, (LPARAM)&bMaximized);
	if(hActiveWnd && (hActiveWnd != hChildWnd))
		return FALSE;

	// Adjusts the style of the MDI client window when maximize/restore happens
	// No update is necessary if the style is matching already
	DWORD dwOldStyle = ::GetWindowLong(m_hClientWnd, GWL_EXSTYLE);
	DWORD dwNewStyle = (hActiveWnd && bMaximized) ?
		(dwOldStyle & ~WS_EX_CLIENTEDGE) : (dwOldStyle | WS_EX_CLIENTEDGE);
	if(dwNewStyle == dwOldStyle)
		return FALSE;

	// Updates the style of the MDI client window
	::SetWindowLong(m_hClientWnd, GWL_EXSTYLE, dwNewStyle);
	return VERIFY(::SetWindowPos(m_hClientWnd, NULL, UNUSED, UNUSED, UNUSED, UNUSED,
		SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER));
}
