// CTomPoiEdr.cpp - POI Editor for TomTom Navigation
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>
#include <Dbt.h>
#include "CAboutDlg.hpp"
#include "CClearColDlg.hpp"
#include "CDumpDeviceDlg.hpp"
#include "CDupPoiDlg.hpp"
#include "CFileExistDlg.hpp"
#include "CMyBPDlg.hpp"
#include "COpenDeviceDlg.hpp"
#include "CQGPSfixDlg.hpp"
#include "CQueryNameDlg.hpp"
#include "CSaveDeviceDlg.hpp"
#include "CTbBkDlg.hpp"
#include "CTomPoiEdr.hpp"

// Converts degrees to radians
#define GetRadians(d) (d * 0.017453292519943295)

// Radius of earth, in meters
#define GetEarthRadius() 6378137.527506567418844

////////////////////////////////////////////////////////////////////////////////////////////////
// Static data

CTCHAR  CTomPoiEdr::m_szOv2Ext[4]         = _T("ov2");
CTCHAR  CTomPoiEdr::m_szTxtExt[]          = _T("txt");
CTCHAR  CTomPoiEdr::m_szKmlExt[]          = _T("kml");
CTCHAR  CTomPoiEdr::m_szGpxExt[]          = _T("gpx");
CTCHAR  CTomPoiEdr::m_szGpiExt[]          = _T("gpi");
CTCHAR  CTomPoiEdr::m_szCsvExt[]          = _T("csv");
CTCHAR  CTomPoiEdr::m_szAscExt[]          = _T("asc");
CTCHAR  CTomPoiEdr::m_szPoiExt[]          = _T("poi");
CTCHAR  CTomPoiEdr::m_szPngExt[]          = _T("png");
CTCHAR  CTomPoiEdr::m_szBmpExt[]          = _T("bmp");
CTCHAR  CTomPoiEdr::m_szRpfExt[]          = _T("rpf");
CTCHAR  CTomPoiEdr::m_szOggExt[]          = _T("ogg");
CTCHAR  CTomPoiEdr::m_szVifExt[]          = _T("vif");
CTCHAR  CTomPoiEdr::m_szErrToken[]        = _T(">|");
CTCHAR  CTomPoiEdr::m_szCtrlToken[]       = _T("\t\r\n");
CTCHAR  CTomPoiEdr::m_szAmpEntity[]       = _T("&amp;");
CTCHAR  CTomPoiEdr::m_szLtEntity[]        = _T("&lt;");
CTCHAR  CTomPoiEdr::m_szGtEntity[]        = _T("&gt;");
CTCHAR  CTomPoiEdr::m_szQuotEntity[]      = _T("&quot;");
CTCHAR  CTomPoiEdr::m_szNbspEntity[]      = _T("&nbsp;");
CTCHAR  CTomPoiEdr::m_szLastNotifyEntry[] = _T("Last Notify");
CTCHAR  CTomPoiEdr::m_szOptionSection[]   = _T("Options");
CTCHAR  CTomPoiEdr::m_szShowMbEntry[]     = _T("Show Menu Bar");
CTCHAR  CTomPoiEdr::m_szShowTbEntry[]     = _T("Show Toolbar");
CTCHAR  CTomPoiEdr::m_szLargeTbEntry[]    = _T("Large Toolbar");
CTCHAR  CTomPoiEdr::m_szWrapTbEntry[]     = _T("Wrap Toolbar");
CTCHAR  CTomPoiEdr::m_szTbBkgndEntry[]    = _T("Toolbar Bkgnd");
CTCHAR  CTomPoiEdr::m_szTbButtonEntry[]   = _T("Toolbar Button");
CTCHAR  CTomPoiEdr::m_szTbSpacingEntry[]  = _T("Toolbar Spacing");
CTCHAR  CTomPoiEdr::m_szLngFirstEntry[]   = _T("Lng First");
CTCHAR  CTomPoiEdr::m_szDmsFormatEntry[]  = _T("DMS Format");
CTCHAR  CTomPoiEdr::m_szCompassDirEntry[] = _T("Compass Direction");
CTCHAR  CTomPoiEdr::m_szAutoColEntry[]    = _T("Auto Column");
CTCHAR  CTomPoiEdr::m_szShowDistEntry[]   = _T("Show Distance");
CTCHAR  CTomPoiEdr::m_szMetricUnitEntry[] = _T("Metric Unit");
CTCHAR  CTomPoiEdr::m_szAdjDistEntry[]    = _T("Adjacent Distance");
CTCHAR  CTomPoiEdr::m_szRecentFileEntry[] = _T("Recent File");
CTCHAR  CTomPoiEdr::m_szFileCountEntry[]  = _T("File Count");
CTCHAR  CTomPoiEdr::m_szRecentListEntry[] = _T("Recent List %d");
CTCHAR  CTomPoiEdr::m_szFadeWndEntry[]    = _T("Fade Window");
CTCHAR  CTomPoiEdr::m_szShadeTableEntry[] = _T("Shade Table");
CTCHAR  CTomPoiEdr::m_szShowSbEntry[]     = _T("Show Status Bar");
CTCHAR  CTomPoiEdr::m_szSignalMarkEntry[] = _T("Signal Mark");
CTCHAR  CTomPoiEdr::m_szOptPoiEntry[]     = _T("Optimise POI");
CTCHAR  CTomPoiEdr::m_szCheckNameEntry[]  = _T("Check Name");
CTCHAR  CTomPoiEdr::m_szAutoBPEntry[]     = _T("Auto BP");
CTCHAR  CTomPoiEdr::m_szOpenLastEntry[]   = _T("Open Last");
CTCHAR  CTomPoiEdr::m_szTranNameEntry[]   = _T("Translate Name");
CTCHAR  CTomPoiEdr::m_szBackupFileEntry[] = _T("Backup File");
CTCHAR  CTomPoiEdr::m_szBackupIntlEntry[] = _T("Backup Interval");
CTCHAR  CTomPoiEdr::m_szGmLangEntry[]     = _T("Google Maps Lang");
CTCHAR  CTomPoiEdr::m_szGmCookieEntry[]   = _T("Google Maps Cookie");
CTCHAR  CTomPoiEdr::m_szGmMemoEntry[]     = _T("Google Maps Memo");
CTCHAR  CTomPoiEdr::m_szGmRouteEntry[]    = _T("Google Maps Route");
CTCHAR  CTomPoiEdr::m_szFontSection[]     = _T("Font");
CTCHAR  CTomPoiEdr::m_szFontNameEntry[]   = _T("Name");
CTCHAR  CTomPoiEdr::m_szFontWeightEntry[] = _T("Weight");
CTCHAR  CTomPoiEdr::m_szFontItalicEntry[] = _T("Italic");
CTCHAR  CTomPoiEdr::m_szFontColorEntry[]  = _T("Color");
CTCHAR  CTomPoiEdr::m_szFontBkgndEntry[]  = _T("Bkgnd");
CTCHAR  CTomPoiEdr::m_szFontUseBkEntry[]  = _T("Use Bkgnd");
CTCHAR  CTomPoiEdr::m_szIEPaneEntry[]     = _T("IE Pane");
CBYTE   CTomPoiEdr::m_byUnicodeMark[]     = { LOBYTE(m_eUBOM), HIBYTE(m_eUBOM) };
CBYTE   CTomPoiEdr::m_byUtf8Mark[]        = { LOBYTE(FT_UTF8L), HIBYTE(FT_UTF8L), FT_UTF8H };

// Acceptable POI files in GPS device
CPCTSTR CTomPoiEdr::m_pszGPSF[GPSF_COUNT] =
{
	m_szOv2Ext,
	m_szBmpExt,
	m_szOggExt,
	m_szRpfExt,
	m_szVifExt,
};

// Separator between GPS files
CTCHAR CTomPoiEdr::m_pszGpsfSep[3] =
{
	CHR_SP,
	CHR_VL,
	CHR_SP,
};

// Character codes for TomTom file names
CBYTE CTomPoiEdr::m_byTfnCode[] =
{
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'{', '-', '+', '=', '}',
};

// Options entries
CPCTSTR CTomPoiEdr::m_pszOptionEntry[] =
{
	m_szShowMbEntry,
	m_szShowTbEntry,
	m_szLargeTbEntry,
	m_szWrapTbEntry,
	m_szLngFirstEntry,
	m_szDmsFormatEntry,
	m_szCompassDirEntry,
	m_szAutoColEntry,
	m_szShowDistEntry,
	m_szMetricUnitEntry,
	m_szRecentFileEntry,
	m_szFadeWndEntry,
	m_szShadeTableEntry,
	m_szShowSbEntry,
	m_szSignalMarkEntry,
	m_szOptPoiEntry,
	m_szCheckNameEntry,
	m_szAutoBPEntry,
	m_szOpenLastEntry,
	m_szTranNameEntry,
	m_szBackupFileEntry,
	m_szGmCookieEntry,
	m_szGmMemoEntry,
	m_szGmRouteEntry,
};

// Default values of Options entries
CUINT CTomPoiEdr::m_nOptionEntry[] =
{
	BST_CHECKED,    // m_szShowMbEntry
	BST_CHECKED,    // m_szShowTbEntry
	BST_CHECKED,    // m_szLargeTbEntry
	BST_CHECKED,    // m_szWrapTbEntry
	BST_UNCHECKED,  // m_szLngFirstEntry
	BST_UNCHECKED,  // m_szDmsFormatEntry
	BST_UNCHECKED,  // m_szCompassDirEntry
	BST_CHECKED,    // m_szAutoColEntry
	BST_UNCHECKED,  // m_szShowDistEntry
#ifdef VER_EN_USA   // m_szMetricUnitEntry
	BST_UNCHECKED,
#else
	BST_CHECKED,
#endif
	BST_CHECKED,    // m_szRecentFileEntry
	BST_UNCHECKED,  // m_szFadeWndEntry
	BST_UNCHECKED,  // m_szShadeTableEntry
	BST_CHECKED,    // m_szShowSbEntry
	BST_UNCHECKED,  // m_szSignalMarkEntry
	BST_CHECKED,    // m_szOptPoiEntry
	BST_UNCHECKED,  // m_szCheckNameEntry
	BST_UNCHECKED,  // m_szAutoBPEntry
	BST_UNCHECKED,  // m_szOpenLastEntry
	BST_UNCHECKED,  // m_szTranNameEntry
	BST_CHECKED,    // m_szBackupFileEntry
	BST_CHECKED,    // m_szGmCookieEntry
	BST_CHECKED,    // m_szGmMemoEntry
	BST_CHECKED,    // m_szGmRouteEntry
};

// Decoded table for record 9
CTomPoiEdr::CPOIRECORD9DT CTomPoiEdr::m_pr9dt[] =
{
	"1011",                     sizeof("1011") - 1,                     0,
	"0010",                     sizeof("0010") - 1,                     L' ',
	"01100001110111001001",     sizeof("01100001110111001001") - 1,     L'!',
	"010001100011",             sizeof("010001100011") - 1,             L'"',
	"10101111110000100001110",  sizeof("10101111110000100001110") - 1,  L'#',
	"101011111100001001100",    sizeof("101011111100001001100") - 1,    L'$',
	/*
	"",                         sizeof("") - 1,                         L'%',
	*/
	"0100010010",               sizeof("0100010010") - 1,               L'&',
	"0100011010",               sizeof("0100011010") - 1,               L'\'',
	"01001010011",              sizeof("01001010011") - 1,              L'(',
	"01100001100",              sizeof("01100001100") - 1,              L')',
	"101011111100000100",       sizeof("101011111100000100") - 1,       L'*',
	"00001011011111",           sizeof("00001011011111") - 1,           L'+',
	"1010111011",               sizeof("1010111011") - 1,               L',',
	"01001011",                 sizeof("01001011") - 1,                 L'-',
	"0000110",                  sizeof("0000110") - 1,                  L'.',
	"0000001101",               sizeof("0000001101") - 1,               L'/',
	"10101110001",              sizeof("10101110001") - 1,              L'0',
	"00001011010",              sizeof("00001011010") - 1,              L'1',
	"01000110111",              sizeof("01000110111") - 1,              L'2',
	"10101110101",              sizeof("10101110101") - 1,              L'3',
	"10101111111",              sizeof("10101111111") - 1,              L'4',
	"010010100100",             sizeof("010010100100") - 1,             L'5',
	"101000110110",             sizeof("101000110110") - 1,             L'6',
	"101000110100",             sizeof("101000110100") - 1,             L'7',
	"000010110110",             sizeof("000010110110") - 1,             L'8',
	"101011100001",             sizeof("101011100001") - 1,             L'9',
	"10101111110000011",        sizeof("10101111110000011") - 1,        L':',
	"00001011011101010100",     sizeof("00001011011101010100") - 1,     L';',
	/*
	"",                         sizeof("") - 1,                         L'<',
	"",                         sizeof("") - 1,                         L'=',
	*/
	"10101111110000100001101",  sizeof("10101111110000100001101") - 1,  L'>',
	"101011111100001010110",    sizeof("101011111100001010110") - 1,    L'?',
	"0000101101110111011110",   sizeof("0000101101110111011110") - 1,   L'@',
	"1010110",                  sizeof("1010110") - 1,                  L'A',
	"00001010",                 sizeof("00001010") - 1,                 L'B',
	"00000010",                 sizeof("00000010") - 1,                 L'C',
	"01100011",                 sizeof("01100011") - 1,                 L'D',
	"010010101",                sizeof("010010101") - 1,                L'E',
	"010001000",                sizeof("010001000") - 1,                L'F',
	"01000111",                 sizeof("01000111") - 1,                 L'G',
	"10100000",                 sizeof("10100000") - 1,                 L'H',
	"0000101100",               sizeof("0000101100") - 1,               L'I',
	"0100010011",               sizeof("0100010011") - 1,               L'J',
	"000000111",                sizeof("000000111") - 1,                L'K',
	"10100001",                 sizeof("10100001") - 1,                 L'L',
	"00001110",                 sizeof("00001110") - 1,                 L'M',
	"0000001100",               sizeof("0000001100") - 1,               L'N',
	"0100011001",               sizeof("0100011001") - 1,               L'O',
	"01000101",                 sizeof("01000101") - 1,                 L'P',
	"1010111000000",            sizeof("1010111000000") - 1,            L'Q',
	"01100000",                 sizeof("01100000") - 1,                 L'R',
	"0000100",                  sizeof("0000100") - 1,                  L'S',
	"000011111",                sizeof("000011111") - 1,                L'T',
	"01100001101",              sizeof("01100001101") - 1,              L'U',
	"000011110",                sizeof("000011110") - 1,                L'V',
	"011000010",                sizeof("011000010") - 1,                L'W',
	"01001010010101",           sizeof("01001010010101") - 1,           L'X',
	"1010111111001",            sizeof("1010111111001") - 1,            L'Y',
	"01000110000",              sizeof("01000110000") - 1,              L'Z',
	"000010110111010111",       sizeof("000010110111010111") - 1,       L'[',
	"011000011101110010000",    sizeof("011000011101110010000") - 1,    L'\\',
	"000010110111011100",       sizeof("000010110111011100") - 1,       L']',
	/*
	"",                         sizeof("") - 1,                         L'^',
	*/
	"0000101101110101011",      sizeof("0000101101110101011") - 1,      L'_',
	"011000011101110011",       sizeof("011000011101110011") - 1,       L'`',
	"0011",                     sizeof("0011") - 1,                     L'a',
	"101010",                   sizeof("101010") - 1,                   L'b',
	"11010",                    sizeof("11010") - 1,                    L'c',
	"01101",                    sizeof("01101") - 1,                    L'd',
	"111",                      sizeof("111") - 1,                      L'e',
	"0100100",                  sizeof("0100100") - 1,                  L'f',
	"010011",                   sizeof("010011") - 1,                   L'g',
	"000001",                   sizeof("000001") - 1,                   L'h',
	"0111",                     sizeof("0111") - 1,                     L'i',
	"000010111",                sizeof("000010111") - 1,                L'j',
	"0000000",                  sizeof("0000000") - 1,                  L'k',
	"00011",                    sizeof("00011") - 1,                    L'l',
	"010000",                   sizeof("010000") - 1,                   L'm',
	"1001",                     sizeof("1001") - 1,                     L'n',
	"1000",                     sizeof("1000") - 1,                     L'o',
	"011001",                   sizeof("011001") - 1,                   L'p',
	"1010111001",               sizeof("1010111001") - 1,               L'q',
	"0101",                     sizeof("0101") - 1,                     L'r',
	"00010",                    sizeof("00010") - 1,                    L's',
	"1100",                     sizeof("1100") - 1,                     L't',
	"11011",                    sizeof("11011") - 1,                    L'u',
	"1010011",                  sizeof("1010011") - 1,                  L'v',
	"10100010",                 sizeof("10100010") - 1,                 L'w',
	"1010001100",               sizeof("1010001100") - 1,               L'x',
	"01100010",                 sizeof("01100010") - 1,                 L'y',
	"1010010",                  sizeof("1010010") - 1,                  L'z',
	"101011111100000101000010", sizeof("101011111100000101000010") - 1, L'{',
	/*
	"",                         sizeof("") - 1,                         L'|',
	*/
	"101011111100000101000000", sizeof("101011111100000101000000") - 1, L'}',
	/*
	"",                         sizeof("") - 1,                         L'~',
	*/
	"1010111111011001",         sizeof("1010111111011001") - 1,         0xAA,
	"0110000111011100101",      sizeof("0110000111011100101") - 1,      0xBA,
	"000010110111010110",       sizeof("000010110111010110") - 1,       0xC0,
	"101011111101101",          sizeof("101011111101101") - 1,          0xC1,
	"000010110111011101010",    sizeof("000010110111011101010") - 1,    0xC2,
	"101011111100001001101",    sizeof("101011111100001001101") - 1,    0xC3,
	"1010001101010",            sizeof("1010001101010") - 1,            0xC4,
	"01100001110110",           sizeof("01100001110110") - 1,           0xC5,
	"1010111111000010100",      sizeof("1010111111000010100") - 1,      0xC6,
	"011000011101110010001",    sizeof("011000011101110010001") - 1,    0xC7,
	"0100101001010001",         sizeof("0100101001010001") - 1,         0xC8,
	"1010111111011000",         sizeof("1010111111011000") - 1,         0xC9,
	"10101111110000100000110",  sizeof("10101111110000100000110") - 1,  0xCA,
	"00001011011101110100100",  sizeof("00001011011101110100100") - 1,  0xCB,
	"01100001110111010",        sizeof("01100001110111010") - 1,        0xCD,
	"00001011011101110100101",  sizeof("00001011011101110100101") - 1,  0xCE,
	"101011111100001000010",    sizeof("101011111100001000010") - 1,    0xCF,
	"10101111110000101010",     sizeof("10101111110000101010") - 1,     0xD1,
	"000010110111011101000",    sizeof("000010110111011101000") - 1,    0xD2,
	"00001011011101110110",     sizeof("00001011011101110110") - 1,     0xD3,
	"101011111100001010111",    sizeof("101011111100001010111") - 1,    0xD4,
	"0100011011011",            sizeof("0100011011011") - 1,            0xD6,
	"011000011101111",          sizeof("011000011101111") - 1,          0xD8,
	"011000011101110001",       sizeof("011000011101110001") - 1,       0xDA,
	"1010111111000001011011",   sizeof("1010111111000001011011") - 1,   0xDB,
	"00001011011100",           sizeof("00001011011100") - 1,           0xDC,
	"101011110",                sizeof("101011110") - 1,                0xDF,
	"0110000111010",            sizeof("0110000111010") - 1,            0xE0,
	"010001101100",             sizeof("010001101100") - 1,             0xE1,
	"010001101101001",          sizeof("010001101101001") - 1,          0xE2,
	"01000110110101",           sizeof("01000110110101") - 1,           0xE3,
	"1010111110",               sizeof("1010111110") - 1,               0xE4,
	"1010001101011",            sizeof("1010001101011") - 1,            0xE5,
	"010001100010",             sizeof("010001100010") - 1,             0xE6,
	"1010111000001",            sizeof("1010111000001") - 1,            0xE7,
	"101000110111",             sizeof("101000110111") - 1,             0xE8,
	"1010001111",               sizeof("1010001111") - 1,               0xE9,
	"0000101101110110",         sizeof("0000101101110110") - 1,         0xEA,
	"10101111110001",           sizeof("10101111110001") - 1,           0xEB,
	"01001010010110",           sizeof("01001010010110") - 1,           0xEC,
	"10101110100",              sizeof("10101110100") - 1,              0xED,
	"00001011011101111",        sizeof("00001011011101111") - 1,        0xEE,
	"01100001110111011",        sizeof("01100001110111011") - 1,        0xEF,
	"1010111111010",            sizeof("1010111111010") - 1,            0xF1,
	"10101111110111",           sizeof("10101111110111") - 1,           0xF2,
	"01100001111",              sizeof("01100001111") - 1,              0xF3,
	"00001011011110",           sizeof("00001011011110") - 1,           0xF4,
	"000010110111010100",       sizeof("000010110111010100") - 1,       0xF5,
	"1010001110",               sizeof("1010001110") - 1,               0xF6,
	"011000011100",             sizeof("011000011100") - 1,             0xF8,
	"010001101101000",          sizeof("010001101101000") - 1,          0xF9,
	"01001010010111",           sizeof("01001010010111") - 1,           0xFA,
	"101011111100001011",       sizeof("101011111100001011") - 1,       0xFB,
	"0100101000",               sizeof("0100101000") - 1,               0xFC,
	"0100101001010000",         sizeof("0100101001010000") - 1,         0xFD,
	"000010110111010101011",    sizeof("000010110111010101011") - 1,    0xFF,
	/*
	"101011111100001000110111", sizeof("101011111100001000110111") - 1, 0x141,
	"0000101101110100",         sizeof("0000101101110100") - 1,         0x142,
	"000010110111010101010",    sizeof("000010110111010101010") - 1,    0x2DA,
	"1010111111000011",         sizeof("1010111111000011") - 1,         0x2019,
	"1010111111000001011010",   sizeof("1010111111000001011010") - 1,   ???
	*/
};

// Toolbar buttons
#define TBB_UNUSED  { UNUSED }, UNUSED, UNUSED
#define TBB_SEP_POS 5
TBBUTTON CTomPoiEdr::m_tbb[] =
{
	// File menu
	m_eFileNew,        ID_FILE_NEW,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileOpen,       ID_FILE_OPEN,        TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileOpenFav,    ID_FILE_OPEN_FAV,    TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileOpenDevice, ID_FILE_OPEN_DEVICE, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileDumpDevice, ID_FILE_DUMP_DEVICE, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_eFileSave,       ID_FILE_SAVE,        TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileSaveAs,     ID_FILE_SAVE_AS,     TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileSaveDevice, ID_FILE_SAVE_DEVICE, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileImport,     ID_FILE_IMPORT,      TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileExport,     ID_FILE_EXPORT,      TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_eFileQuery,      ID_FILE_QUERY,       TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileTranslate,  ID_FILE_TRANSLATE,   TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eFileConvert,    ID_FILE_CONVERT,     TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,

	// POI menu
	m_ePoiUndo,        ID_POI_UNDO,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiRedo,        ID_POI_REDO,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiAdd,         ID_POI_ADD,          TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiInsert,      ID_POI_INSERT,       TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiCombineCol,  ID_POI_COMBINE_COL,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiEdit,        ID_POI_EDIT,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiDelete,      ID_POI_DELETE,       TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiClearCol,    ID_POI_CLEAR_COL,    TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_ePoiCut,         ID_POI_CUT,          TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiCopy,        ID_POI_COPY,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiPaste,       ID_POI_PASTE,        TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiQueryCoord,  ID_POI_QUERY_COORD,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiQueryAddr,   ID_POI_QUERY_ADDR,   TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_ePoiFind,        ID_POI_FIND,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiFindNext,    ID_POI_FIND_NEXT,    TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiReplace,     ID_POI_REPLACE,      TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiReplaceNext, ID_POI_REPLACE_NEXT, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_ePoiMessCoord,   ID_POI_MESS_COORD,   TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiCheck,       ID_POI_CHECK,        TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiAddFav,      ID_POI_ADD_FAV,      TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePoiAddItn,      ID_POI_ADD_ITN,      TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,

	// Position menu
	m_ePosUp,          ID_POS_UP,           TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosDown,        ID_POS_DOWN,         TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosSetBP,       ID_POS_SET_BP,       TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosMyBP,        ID_POS_MY_BP,        TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosSortAscend,  ID_POS_SORT_ASCEND,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosSortDescend, ID_POS_SORT_DESCEND, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,
	m_ePosGoogleMap,   ID_POS_GOOGLE_MAP,   TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_ePosGoogleEarth, ID_POS_GOOGLE_EARTH, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,            UNUSED,              UNUSED,          BTNS_SEP,    TBB_UNUSED,

	// Tools menu
	m_eToolsQGPSfix, ID_TOOLS_QGPSFIX, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eToolsLvFont,  ID_TOOLS_LVFONT,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eToolsLvBkgnd, ID_TOOLS_LVBKGND, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eToolsOption,  ID_TOOLS_OPTION,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	UNUSED,          UNUSED,           UNUSED,          BTNS_SEP,    TBB_UNUSED,

	// Help menu
	m_eHelpTomHome,  ID_HELP_TOM_HOME,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eHelpTomUsers, ID_HELP_TOM_USERS, TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eHelpTpeHome,  ID_HELP_TPE_HOME,  TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eHelpMailto,   ID_HELP_MAILTO,    TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
	m_eHelpAbout,    ID_HELP_ABOUT,     TBSTATE_ENABLED, BTNS_BUTTON, TBB_UNUSED,
};

VARIANT CTomPoiEdr::m_vtEmpty = { VT_EMPTY };

////////////////////////////////////////////////////////////////////////////////////////////////
// Adds a POI record

void CTomPoiEdr::AddPoiRecord(int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::AddPoiRecord\n"));

	TCHAR  szText[MAX_STR];
	LVITEM lvi;
	m_jListView.SetAction(AT_TEMP);
	m_iLvCol     = LVC_NAME;
	lvi.mask     = UNUSED;
	lvi.iItem    = (!iLvItem && m_bMyBP) ? 1 : iLvItem;
	lvi.iSubItem = 0;
	iLvItem      = m_jListView.InsertItem(&lvi);
	if(m_iLvBaseItem != LVI_ERR)
	{
		if(iLvItem <= m_iLvBaseItem)
			m_iLvBaseItem++;
		SetDistanceText2(iLvItem);
	}
	if(m_fst == FST_SC)
	{
		lvi.iSubItem = LVC_SPEED;
		lvi.pszText  = szText;
		_ltot(e_eDefSpeed, szText);
		m_jListView.CListView::SetItemText(iLvItem, &lvi);
		lvi.iSubItem = LVC_CAMERA;
		LoadString(e_hInstance, IDS_SCM_FIXED, szText, countof(szText));
		m_jListView.CListView::SetItemText(iLvItem, &lvi);
	}
	else if(m_fst == FST_CPB)
	{
		lvi.iSubItem = LVC_CPBT;
		lvi.pszText  = szText;
		LoadString(e_hInstance, IDS_CPBM_DEFAULT, szText, countof(szText));
		m_jListView.CListView::SetItemText(iLvItem, &lvi);
	}
#ifdef DEBUG
	else
		ASSERT(m_fst == FST_GENERAL);
#endif
	BeginLvEdit(iLvItem);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Automatic sizes column widths of list view control

void CTomPoiEdr::AutoSizeLvCol(int iLvCol, BOOL bSetInfo)
{
	TRACE(_T("Entering CTomPoiEdr::AutoSizeLvCol\n"));

	if(iLvCol <= LVC_ALL)
	{
		if(m_jOptionDlg.m_nAutoColumn != BST_CHECKED)
			return;
	}
	else
		iLvCol -= LVCA_FIRST;

	BOOL     bInsLvCol, bNameCol, bRedraw;
	int      iLvColR, iLastColR, cxLvCol, iLeft;
	RECT     rc;
	HDITEM   hi;
	LVCOLUMN lvc;
	if(bSetInfo)
	{
		TCHAR szText[MAX_STR];
		LoadString(e_hInstance, IDP_AUTOSIZING_COL, szText, countof(szText));
		m_jStatusBar.SetWindowText(szText);
	}
	iLvColR = LVCR_FIRST;
	if(iLvCol == LVC_ALL)
		iLastColR = LVCR_COUNT - 1;
	else
	{
		while(iLvCol != e_iHdOrder[iLvColR])
			iLvColR++;
		iLastColR = iLvColR;
	}
	hi.mask = HDI_WIDTH;
	bRedraw = m_jListView.m_bRedraw;
	do
	{
		iLvCol    = e_iHdOrder[iLvColR];
		cxLvCol   = LVSCW_AUTOSIZE_USEHEADER;
		bInsLvCol = FALSE;
		hi.cxy    = 0;
		if((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT))
		{
			if(m_fst == FST_CPB)
				cxLvCol = 0;
		}
		else if(iLvCol == LVC_DIST)
		{
			if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
			{
				if(e_dwWinVer <= WV_XP)
					bInsLvCol = TRUE;
			}
			else
				cxLvCol = 0;
			hi.cxy = (e_dwWinVer > WV_XP) ? m_ecxHdColPad : m_ecxHdColPadXP;
		}
		if((iLvColR == iLastColR) && (e_dwWinVer > WV_XP))
			bInsLvCol = TRUE;
		if(bInsLvCol)
		{
			lvc.mask = LVCF_WIDTH;
			lvc.cx   = 0;
			m_jListView.InsertColumn(LVC_COUNT, &lvc);
		}
		bNameCol = (iLvCol == LVC_NAME) && (e_dwWinVer > WV_XP);
		if(bNameCol && bRedraw)
			m_jListView.SetWindowRedraw(FALSE);
		m_jListView.SetColumnWidth(iLvCol, cxLvCol);
		if(bNameCol)
		{
			m_jListView.SetColumnWidth(LVC_NAME,
				m_jListView.GetColumnWidth(LVC_NAME) + m_exLvNameAdj);
			if(bRedraw)
				m_jListView.SetWindowRedraw();
		}
		if(bInsLvCol)
			m_jListView.DeleteColumn(LVC_COUNT);
		hi.cxy += m_jListView.GetColumnWidth(iLvCol);
		m_jLvHeader.SetItem(iLvCol, &hi);
		iLvColR++;
		if(e_dwWinVer > WV_XP)
			PeekUpdateMessage(FALSE);
	}
	while(iLvColR <= iLastColR);
	::GetWindowRect(m_jListView.GetHeader(), &rc);
	iLeft = rc.left;
	m_jLvHeader.GetWindowRect(&rc);
	if(rc.left != iLeft)
	{
		MSG msg;
		if(!PeekMessage(&msg, WM_COMMAND, PM_NOREMOVE) || (msg.wParam != ID_RESIZE_WINDOW))
			PostMessage(WM_COMMAND, ID_RESIZE_WINDOW, FALSE);
	}
	if(bSetInfo)
		m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Begins to edit POI record

void CTomPoiEdr::BeginLvEdit(int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::BeginLvEdit\n"));

	SCINT sccMaxText[] =
	{
		e_eccNameCol,
		e_eccLngCol,
		e_eccLatCol,
		e_eccTelCol,
		e_eccMemoCol,
	};
	TCHAR  szText[MAX_STR];
	int    iDelta, ccText;
	UINT   nEdLMargin, nResId;
	RECT   rc, rcCtrl;
	LVITEM lvi;
	CWnd   jFocusWnd;
	m_jListView.SetAction(AT_NEW);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	m_jLvEdit.SetLimitText(sccMaxText[m_iLvCol] - 1);
	if(m_iLvCol == LVC_DIST)
		m_iLvCol = LVC_NAME;
	m_jListView.EnsureVisible(iLvItem);
	m_jListView.GetClientRect(&rcCtrl);
	m_jListView.GetItemRect(iLvItem, m_iLvCol, LVIR_ICON, &rc);
	if(m_iLvCol == LVC_NAME)
		rc.left -= m_exEdAdjL;
	if((rc.left < m_exEdAdjL) || (rc.left > (rcCtrl.right - m_exEdAdjR)))
		m_jListView.Scroll(rc.left - m_exEdAdjL, 0);
	m_jListView.GetItemRect(iLvItem, m_iLvCol, LVIR_LABEL, &rc);
	if(m_iLvCol == LVC_NAME)
	{
		if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
		{
			if(e_dwWinVer > WV_XP)
				nEdLMargin = m_exEdAdjL + m_exEdPad + 1;
			else
				nEdLMargin = m_exEdAdjL + 1;
		}
		else
			nEdLMargin = m_exEdAdjL;
		rc.left -= nEdLMargin;
	}
	else
	{
		rc.left   -= m_exEdPad;
		nEdLMargin = m_exEdAdjL + m_exEdPad;
	}
	if(e_dwWinVer > WV_XP)
		iDelta = 0;
	else
	{
		iDelta    = 1;
		rc.top   -= m_eyEdPad;
		rc.right += m_exEdPad + 1;
		rc.bottom++;
	}
	rc.right       = min(rc.right, rcCtrl.right) - rc.left;
	rc.bottom      = GetRectHeight(&rc);
	lvi.iSubItem   = m_iLvCol;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	ccText         = m_jListView.GetItemText(iLvItem, &lvi);
	if(((m_fst == FST_SC) && ((m_iLvCol == LVC_SPEED) || (m_iLvCol == LVC_CAMERA))) ||
		((m_fst == FST_CPB) && (m_iLvCol == LVC_CPBT)))
	{
		TCHAR szCbItem[64];
		int   i, cxMaxCombo, cxCombo;
		m_jLvEdit.MoveWindow(0, 0, 0, 0);
		cxMaxCombo = 0;
		if(m_fst == FST_SC)
		{
			if(m_iLvCol == LVC_SPEED)
			{
				i = e_eNonSpeed;
				do
				{
					_ltot(i, szCbItem);
					m_jLvCombo.AddItem(szCbItem);
					cxCombo = m_jListView.GetStringWidth(szCbItem);
					if(cxCombo > cxMaxCombo)
						cxMaxCombo = cxCombo;
					if(i == e_eNonSpeed)
						i = e_eMinSpeed;
					else
						i += e_eDivSpeed;
				}
				while(i <= e_eMaxSpeed);
				i      = (e_eDefSpeed - e_eMinSpeed) / e_eDivSpeed + 1;
				nResId = IDP_SPEED_TIP;
			}
			else
			{
				i = IDS_SCM_FIRST;
				do
				{
					LoadString(e_hInstance, i, szCbItem, countof(szCbItem));
					m_jLvCombo.AddItem(szCbItem);
					cxCombo = m_jListView.GetStringWidth(szCbItem);
					if(cxCombo > cxMaxCombo)
						cxMaxCombo = cxCombo;
					i++;
				}
				while(i <= IDS_SCM_LAST);
				i      = 0;
				nResId = IDP_CAMERA_TIP;
			}
		}
		else
		{
			i = IDS_CPBM_DEFAULT;
			do
			{
				LoadString(e_hInstance, i, szCbItem, countof(szCbItem));
				m_jLvCombo.AddItem(szCbItem);
				cxCombo = m_jListView.GetStringWidth(szCbItem);
				if(cxCombo > cxMaxCombo)
					cxMaxCombo = cxCombo;
				i++;
			}
			while(i <= IDS_CPBM_HOME);
			i      = 0;
			nResId = IDP_CPBT_TIP;
		}
		if(m_jLvCombo.SelectItem(szText) == CB_ERR)
			m_jLvCombo.SetCurSel(i);
		if(e_dwWinVer > WV_XP)
			rc.left -= m_exEdPad;
		cxCombo = min(cxMaxCombo + m_jLvCombo.m_cxDropArrow, rcCtrl.right - rc.left);
		m_jLvCombo.GetWindowRect(&rcCtrl);
		m_jLvCombo.MoveWindow(rc.left, rc.top - (GetRectHeight(&rcCtrl) - rc.bottom) / 2,
			cxCombo, m_ecyCbLb);
		m_jLvCombo.ShowWindow();
		m_jLvCombo.PostMessage(CB_SHOWDROPDOWN, TRUE, UNUSED);
		jFocusWnd = m_jLvCombo;
	}
	else
	{
		m_jLvEdit.MoveWindow(rc.left, rc.top + (rc.bottom - m_cyLvEdit + iDelta) / 2,
			rc.right, m_cyLvEdit);
		m_jLvEdit.SetMargins(nEdLMargin, m_exEdPad);
		if((m_iLvCol == LVC_NAME) && !ccText)
		{

			if(m_fst == FST_SC)
				LoadString(e_hInstance, IDS_SC_NAME, szText, countof(szText));
			else
			{
				TCHAR      szNameFmt[64], cAmPm;
				SYSTEMTIME st;
				ASSERT((m_fst == FST_GENERAL) || (m_fst == FST_CPB));
				GetLocalTime(&st);
				if(st.wHour >= 12)
				{
					st.wHour -= 12;
					cAmPm     = _T('P');
				}
				else
					cAmPm = _T('A');
				if(!st.wHour)
					st.wHour = 12;
				LoadString(e_hInstance, (m_fst == FST_GENERAL) ?
					IDS_NEW_POI_FMT : IDS_NEW_CPB_FMT, szNameFmt, countof(szNameFmt));
				wsprintf(szText, szNameFmt, st.wMonth, st.wDay,
					cAmPm, st.wHour, st.wMinute, st.wSecond);
			}
		}
		m_jLvEdit.SetWindowText(szText);
		m_jLvEdit.SetSel(0);
		jFocusWnd = m_jLvEdit;
		nResId    = m_iLvCol + IDP_NAME_TIP;
	}
	m_jLvEdit.ShowWindow();
	jFocusWnd.SetFocus();
	ccText         = LoadString(e_hInstance, nResId, szText, countof(szText));
	szText[ccText] = CHR_SP;
	ccText++;
	LoadString(e_hInstance, IDP_TAB_COL_TIP, szText + ccText, countof(szText) - ccText);
	m_jStatusBar.SetWindowText(szText);
	m_jLvEdit.m_iLvItem  = iLvItem;
	m_jLvEdit.m_bMsgShow = FALSE;
	m_bSetReady          = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Calculates the distance between two points of POIs

LPARAM CTomPoiEdr::CalcPoiDistance(int iLvItem1, int iLvItem2)
{
	TRACE(_T("Entering CTomPoiEdr::CalcPoiDistance\n"));

	TCHAR  szCoord[32];
	LVITEM lvi;
	lvi.iSubItem   = LVC_LNG;
	lvi.pszText    = szCoord;
	lvi.cchTextMax = countof(szCoord);
	if(!m_jListView.GetItemText(iLvItem1, &lvi))
		return MAX_LPARAM;

	double dLng1;
	dLng1 = GetCoordValueD(szCoord);
	if(!m_jListView.GetItemText(iLvItem2, &lvi))
		return MAX_LPARAM;

	double dLng2;
	dLng2        = GetCoordValueD(szCoord);
	lvi.iSubItem = LVC_LAT;
	if(!m_jListView.GetItemText(iLvItem1, &lvi))
		return MAX_LPARAM;

	double dLat1;
	dLat1 = GetCoordValueD(szCoord);
	if(!m_jListView.GetItemText(iLvItem2, &lvi))
		return MAX_LPARAM;

	double dLat2;
	dLat2 = GetCoordValueD(szCoord);
	if((dLng1 == dLng2) && (dLat1 == dLat2))
		return 0;

	double dLatRed1, dLatRed2;
	dLatRed1 = GetRadians(dLat1);
	dLatRed2 = GetRadians(dLat2);
	return (LPARAM)(acos(sin(dLatRed2) * sin(dLatRed1) +
		cos(dLatRed2) * cos(dLatRed1) * cos(GetRadians(dLng2) - GetRadians(dLng1))) *
		GetEarthRadius());
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Calculates the distance between base point and a point of POIs

AFXINLINE LPARAM CTomPoiEdr::CalcPoiDistance(int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::CalcPoiDistance\n"));

	if(m_iLvBaseItem == LVI_ERR)
		return MAX_LPARAM;

	TCHAR  szCoord[32];
	LVITEM lvi;
	lvi.iSubItem   = LVC_LNG;
	lvi.pszText    = szCoord;
	lvi.cchTextMax = countof(szCoord);
	if(!m_jListView.GetItemText(iLvItem, &lvi))
		return MAX_LPARAM;

	double dLng;
	dLng         = GetCoordValueD(szCoord);
	lvi.iSubItem = LVC_LAT;
	if(!m_jListView.GetItemText(iLvItem, &lvi))
		return MAX_LPARAM;

	double dLat;
	dLat = GetCoordValueD(szCoord);
	if((dLng == m_dBaseLng) && (dLat == m_dBaseLat))
		return 0;

	double dLatRed;
	dLatRed = GetRadians(dLat);
	return (LPARAM)(acos(sin(dLatRed) * m_dBaseSinLat +
		cos(dLatRed) * m_dBaseCosLat * cos(GetRadians(dLng) - m_dBaseRedLng)) *
		GetEarthRadius());
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Checks a signal mark record

BOOL CTomPoiEdr::CheckSignalMark(PPOIRECORD64 pPR64, UINT nCheckSum)
{
	TRACE(_T("Entering CTomPoiEdr::CheckSignalMark\n"));

	if(pPR64->byType != FRT_SIGNAL)
		return FALSE;

	if(pPR64->cbSize != sizeof(POIRECORD64))
		return FALSE;

	int  i;
	UINT nSignalSum;
	i          = sizeof(pPR64->bySignal);
	nSignalSum = pPR64->byAddDelta - pPR64->bySubDelta;
	do
	{
		i--;
		nSignalSum += pPR64->bySignal[i];
	}
	while(i);
	return (nSignalSum == nCheckSum);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes the IE pane

void CTomPoiEdr::CloseIEPane()
{
	TRACE(_T("Entering CTomPoiEdr::CloseIEPane\n"));

	POSTR pszGmUrl;
	TCHAR szHtmBlank[32];
	BSTR  bszGmUrl;
	if(m_jIEPane.IsChild(GetFocus()))
		SetFocus();
#ifdef UNICODE
	LoadStringW(e_hInstance, IDS_GM_HTM_BLANK, szHtmBlank, countof(szHtmBlank));
	pszGmUrl = szHtmBlank;
#else
	WCHAR szwHtmBlank[sizeof(szHtmBlank)];
	MBStoWCS(szHtmBlank, LoadStringA(e_hInstance, IDS_GM_HTM_BLANK,
		szHtmBlank, sizeof(szHtmBlank)) + 1, szwHtmBlank, countof(szwHtmBlank));
	pszGmUrl = szwHtmBlank;
#endif
	bszGmUrl = SysAllocString(pszGmUrl);
	m_pWB2->Navigate(bszGmUrl, &m_vtEmpty, &m_vtEmpty, &m_vtEmpty, &m_vtEmpty);
	SysFreeString(bszGmUrl);
	m_jIEPane.HideWindow();
	ResizeWindow(FALSE);
	m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Combines two coordinate blocks

void CTomPoiEdr::CombineCoordBlock(PLONG plDestCB, PCLONG plSrcCB)
{
	TRACE(_T("Entering CTomPoiEdr::CombineCoordBlock\n"));

	if(*plDestCB < *plSrcCB)
		*plDestCB = *plSrcCB;
	if(plDestCB[1] < plSrcCB[1])
		plDestCB[1] = plSrcCB[1];
	if(plDestCB[2] > plSrcCB[2])
		plDestCB[2] = plSrcCB[2];
	if(plDestCB[3] > plSrcCB[3])
		plDestCB[3] = plSrcCB[3];
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Copies POI records into the clipboard

BOOL CTomPoiEdr::CopyPoiRecord()
{
	TRACE(_T("Entering CTomPoiEdr::CopyPoiRecord\n"));

	UINT nResId;
	m_jToolbar.PressButton(ID_POI_COPY);
	m_jToolbar.UpdateWindow();
	if(OpenClipboard())
	{
		TCHAR szResText[MAX_STR];
		SetCursor(LoadCursor(IDC_WAIT));
		ShowCursor();
		LoadString(e_hInstance, IDP_COPYING_POI, szResText, countof(szResText));
		m_jStatusBar.SetWindowText(szResText);
		nResId = SetCbData() ? 0 : IDS_MEMORY_ERROR_CB;
		CloseClipboard();
		HideCursor();
		SetCursor(LoadCursor(IDC_ARROW));
	}
	else
		nResId = IDS_OPEN_CB_ERROR;
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(ID_POI_COPY, FALSE);
	m_bSetReady = TRUE;
	return !nResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Creates a toolbar control

void CTomPoiEdr::CreateTbCtrl()
{
	TRACE(_T("Entering CTomPoiEdr::CreateTbCtrl\n"));

	PBYTE      pbySrc;
	PARGB      pargbSrc, pargbSrcEnd, pargbDest;
	TCHAR      szValue[MAX_STR];
	int        i, cxImage, cyImage, cbImage;
	UINT       nBitmap;
	DIBSECTION dibs;
	IMAGEINFO  ii;
	TBMETRICS  tbm;
	HBITMAP    hBitmap;
	if(m_jToolbar)
		m_jToolbar.DestroyWindow();
	m_jToolbar.CreateWindow((m_jOptionDlg.m_nWrapTb == BST_CHECKED) ?
		(m_eTbStyle | TBSTYLE_WRAPABLE) : m_eTbStyle, *this, IDW_TOOLBAR);
	m_jToolbar.ButtonStructSize();
	if(m_jOptionDlg.m_nLargeTb == BST_CHECKED)
	{
		nBitmap = IDB_TOOLBAR_LARGE;
		cxImage = m_ecxTbImageL;
		cyImage = m_ecyTbImageL;
	}
	else
	{
		nBitmap = IDB_TOOLBAR_SMALL;
		cxImage = m_ecxTbImageS;
		cyImage = m_ecyTbImageS;
	}
	if(m_jToolbarIL)
		m_jToolbarIL.Destroy();
	m_jToolbarIL.LoadImage(e_hInstance, nBitmap, cxImage, 0, CLR_NONE,
		IMAGE_BITMAP, LR_CREATEDIBSECTION);
	ASSERT(m_jToolbarIL.GetImageCount() == m_eceTbImage);
	m_jToolbar.SetImageList(m_jToolbarIL);
	if(GetPrivateProfileString(m_szOptionSection, m_szTbButtonEntry,
		e_szNull, szValue, countof(szValue), e_pszProfile))
	{
		PTSTR     pszValue;
		PTBBUTTON pTBB;
		pszValue = szValue - 1;
		do
		{
			do
				pszValue++;
			while(*pszValue == CHR_SP);
			nBitmap = _ttoi(pszValue);
			if(nBitmap > m_eceTbImage)
				continue;
			if(nBitmap == m_eceTbImage)
				pTBB = m_tbb + TBB_SEP_POS;
			else
			{
				pTBB = m_tbb;
				while(nBitmap != pTBB->iBitmap)
					pTBB++;
				ASSERT(pTBB->idCommand);
				if(m_jToolbar.CommandToIndex(pTBB->idCommand) != RV_ERR)
					continue;
			}
			m_jToolbar.AddButtons(pTBB, 1);
		}
		while(pszValue = StrChr(pszValue, CHR_SP));
	}
	else
		m_jToolbar.AddButtons(m_tbb, countof(m_tbb));
	if(m_jToolbarHotIL)
		m_jToolbarHotIL.Destroy();
	m_jToolbarHotIL.Create(cxImage, cyImage, ILC_COLOR32, m_eceTbImage);
	m_jToolbarIL.GetImageInfo(0, &ii);
	GetObject(ii.hbmImage, &dibs);
	dibs.dsBmih.biHeight    = cyImage;
	dibs.dsBmih.biSizeImage = 0;
	cbImage                 = dibs.dsBm.bmWidthBytes * cyImage;
	pbySrc                  = (PBYTE)dibs.dsBm.bmBits +
		dibs.dsBm.bmWidthBytes * dibs.dsBm.bmHeight;
	i                       = 0;
	do
	{
		hBitmap     = CreateDIBSection((PCBITMAPINFO)&dibs.dsBmih,
			DIB_RGB_COLORS, (PPVOID)&pargbDest);
		pargbSrcEnd = (PARGB)pbySrc;
		pbySrc     -= cbImage;
		pargbSrc    = (PARGB)pbySrc;
		do
		{
			pargbDest->byAlpha = pargbSrc->byAlpha;
			pargbDest->byRed   = min(pargbSrc->byRed * 110 / 100, MAX_BYTE);
			pargbDest->byGreen = min(pargbSrc->byGreen * 110 / 100, MAX_BYTE);
			pargbDest->byBlue  = min(pargbSrc->byBlue * 110 / 100, MAX_BYTE);
			pargbDest++;
			pargbSrc++;
		}
		while(pargbSrc < pargbSrcEnd);
		m_jToolbarHotIL.Add(hBitmap);
		DeleteBitmap(hBitmap);
		i++;
	}
	while(i < m_eceTbImage);
	m_jToolbar.SetHotImageList(m_jToolbarHotIL);
	tbm.cbSize          = sizeof(tbm);
	tbm.dwMask          = TBMF_BUTTONSPACING;
	tbm.cxButtonSpacing = m_jCustTbDlg.m_cxSpacing;
	tbm.cyButtonSpacing = 0;
	m_jToolbar.SetMetrics(&tbm);
	m_jToolbar.SetExStyle(m_eTbExStyle);
	m_jToolbar.SetIndent(m_ecxTbPad);
	m_jToolbar.SubclassWindow();
	ResizeWindow(TRUE);
	m_jToolbar.ShowWindow();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A toolbar is about to be drawn

AFXINLINE UINT CTomPoiEdr::CustomDrawToolbar(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering CTomPoiEdr::CustomDrawToolbar\n"));

	if(pNMCD->dwDrawStage != CDDS_PREPAINT)
		return CDRF_DODEFAULT;

	BOOL     bRepeat, bChevron;
	int      i, iButton, ceButton, cyButton, cyRow, cyHalfRow, cyLine, yFirstLine, yLastLine;
	int      iRValue, iGValue, iBValue;
	COLORREF cr, crTbDivider;
	RECT     rc, rcDraw;
	HBRUSH   hBrush;
	bChevron     = FALSE;
	rcDraw.right = pNMCD->rc.right;
	ceButton     = m_jToolbar.ButtonCount();
	if((m_jOptionDlg.m_nWrapTb == BST_CHECKED) && ceButton)
	{
		bRepeat = TRUE;
		iButton = 0;
	}
	else
	{
		TBBUTTON tbb;
		bRepeat = FALSE;
		if(rcDraw.right > (m_cxClient - m_ecxTbChevron))
		{
			iButton = ceButton;
			while(iButton)
			{
				iButton--;
				m_jToolbar.GetButton(iButton, &tbb);
				if(!tbb.idCommand)
					continue;
				m_jToolbar.GetItemRect(iButton, &rc);
				if(rc.right > m_cxClient)
					bChevron = TRUE;
				break;
			}
		}
	}
	rcDraw.left = pNMCD->rc.left;
	cyButton    = HIWORD(m_jToolbar.GetButtonSize());
	cyRow       = cyButton + m_ecyTbDiv + m_ecyTbPad * 2;
	cyHalfRow   = cyRow / 2;
	yLastLine   = -m_eyTbPad;
	do
	{
		yFirstLine = yLastLine;
		yLastLine += cyButton + m_ecyTbDiv + m_ecyTbPad;
		if(bRepeat)
		{
			do
			{
				m_jToolbar.GetItemRect(iButton, &rc);
				if(rc.left == m_ecxTbPad)
				{
					cyLine = rc.bottom - yLastLine;
					if(cyLine)
					{
						if(cyLine == cyRow)
							break;
						ASSERT(cyLine < cyRow);
						if(cyLine >= cyButton)
							yLastLine = rc.bottom;
					}
				}
				iButton++;
			}
			while(iButton < ceButton);
			if(iButton == ceButton)
				bRepeat = FALSE;
		}
		if(yFirstLine > 0)
			i = 0;
		else
		{
			yFirstLine = 0;
			i          = m_eyTbPad;
		}
		rcDraw.top  = yFirstLine;
		yLastLine  += m_ecyTbPad;
		crTbDivider = (e_dwWinVer > WV_XP) ? m_ecrTbDivider : m_ecrTbDividerXP;
		iRValue     = GetRValue(m_crTbBkgnd);
		iGValue     = GetGValue(m_crTbBkgnd);
		iBValue     = GetBValue(m_crTbBkgnd);
		do
		{
			if(!i)
				cr = crTbDivider;
			else if(i == 1)
				cr = RGB_WHITE;
			else if(((e_dwWinVer <= WV_XP) && (i < m_eyTbPad)) ||
				((e_dwWinVer > WV_XP) && (i < cyHalfRow)))
				cr = m_crTbBkgnd;
			else
			{
				if((e_dwWinVer > WV_XP) && (i == cyHalfRow))
				{
					iRValue -= 24;
					iGValue -= 8;
					iBValue -= 16;
				}
				else
				{
					iRValue--;
					if(i % 2)
						iGValue--;
					iBValue--;
				}
				cr = RGB(max(0, iRValue), max(0, iGValue), max(0, iBValue));
			}
			if(i < cyRow)
			{
				i++;
				rcDraw.bottom = rcDraw.top + 1;
			}
			else
				rcDraw.bottom = yLastLine;
			hBrush = CreateSolidBrush(cr);
			FillRect(pNMCD->hdc, &rcDraw, hBrush);
			DeleteBrush(hBrush);
			rcDraw.top = rcDraw.bottom;
		}
		while(rcDraw.top < yLastLine);
	}
	while(bRepeat);
	if(bChevron)
		m_jTbChevronIL.Draw((m_jOptionDlg.m_nLargeTb == BST_CHECKED) ?
			m_eTbChevronImageL : m_eTbChevronImageS,
			pNMCD->hdc, m_cxClient - m_ecxTbChevron,
			cyRow - m_ecyTbChevronImage - m_eyTbPad, ILD_TRANSPARENT);

	// Result value of the message processed
	return CDRF_DODEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Decodes character entities

void CTomPoiEdr::DecodeEntity(PTSTR pszData)
{
	TRACE(_T("Entering CTomPoiEdr::DecodeEntity\n"));

	SCPCTSTR spszEntity[] =
	{
		m_szAmpEntity,
		m_szLtEntity,
		m_szGtEntity,
		m_szQuotEntity,
		m_szNbspEntity,
	};
	SCINT sccEntity[] =
	{
		countof(m_szAmpEntity) - 1,
		countof(m_szLtEntity) - 1,
		countof(m_szGtEntity) - 1,
		countof(m_szQuotEntity) - 1,
		countof(m_szNbspEntity) - 1,
	};
	SCTCHAR scEntity[] =
	{
		CHR_AMP,
		CHR_LT,
		CHR_GT,
		CHR_DQM,
		CHR_SP,
	};
	for(int i = 0, j = 0, k; pszData[i]; i++, j++)
	{
#ifndef UNICODE
		if(IsDBCSLeadByte(pszData[i]))
		{
			*(PWCHAR)(pszData + j) = *(PWCHAR)(pszData + i);
			i++;
			j++;
			continue;
		}
#endif
		k = 0;
		do
		{
			if(EqualMemory(pszData + i, spszEntity[k], sccEntity[k] * sizeof(TCHAR)))
				break;
			k++;
		}
		while(k < countof(spszEntity));
		if(k == countof(spszEntity))
		{
			pszData[j] = pszData[i];
			continue;
		}
		pszData[j] = scEntity[k];
		i         += sccEntity[k] - 1;
	}
	pszData[j] = CHR_NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Deletes POI records

void CTomPoiEdr::DeletePoiRecord(UINT nResId)
{
	TRACE(_T("Entering CTomPoiEdr::DeletePoiRecord\n"));

	TCHAR  szText[MAX_STR];
	BOOL   bSelMyBP;
	int    ceLvItem, ceSelItem, iLvItem, iFirstItem;
	UINT   nTbBtnId;
	LVITEM lvi;
	ASSERT((nResId == IDP_DELETING_POI) || (nResId == IDP_CUTTING_POI));
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	nTbBtnId = (nResId == IDP_DELETING_POI) ? ID_POI_DELETE : ID_POI_CUT;
	m_jToolbar.PressButton(nTbBtnId);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, nResId, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	m_jListView.UpdateWindow();
	m_jListView.SetWindowRedraw(FALSE);
	m_jListView.SetAction(AT_NEW);
	ceSelItem  = m_jListView.GetSelCount();
	iLvItem    = m_jListView.GetNextItem(LVNI_SELECTED);
	iFirstItem = iLvItem;
	do
	{
		PutGoogleMaps(m_eGmPutDelete, iLvItem);
		m_jListView.DeleteItem(iLvItem);
		iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem - 1);
	}
	while(iLvItem != RV_ERR);
	ceLvItem = m_jListView.GetItemCount();
	if(ceLvItem)
	{
		iLvItem       = min(iFirstItem, ceLvItem - 1);
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(iLvItem, &lvi);
		m_jListView.EnsureVisible(iLvItem);
	}
	SetBasePoint(LVI_FIND, TRUE);
	m_jListView.SetWindowRedraw();
	m_jToolbar.PressButton(nTbBtnId, FALSE);
	bSelMyBP = !iFirstItem && m_bMyBP;
	if((ceSelItem > 1) || !bSelMyBP)
		m_bFileDirty = TRUE;
	if(bSelMyBP)
		m_bMyBP = FALSE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Displays the file processing progress

void CTomPoiEdr::DisplayFileProgress(DWORD dwDividend, DWORD dwDivisor)
{
	TRACE(_T("Entering CTomPoiEdr::DisplayFileProgress\n"));

	if(m_bBackup || !dwDivisor)
		return;

	SINT siPrecent;
	int  iPrecent;
	iPrecent = min(dwDividend * 100 / dwDivisor, 100);
	if(iPrecent == siPrecent)
		return;

	TCHAR szInfo[MAX_STR], szText[MAX_STR + countof(szInfo)];
	int   iRightPos[m_eceSbPart];
	SIZE  size;
	siPrecent = iPrecent;
	SendMessage(WM_TIMER, m_eMainTimerId);
	LoadString(e_hInstance, (m_jFileDlg.nFilterIndex == FTR_CFG) ?
		IDP_READING_FAV_FILE : (m_opf + IDP_READING_FILE), szInfo, countof(szInfo));
	GetTextExtent(m_hSbDC, szText, wsprintf(szText, szInfo, iPrecent), &size);
	size.cx += m_ecxPbPadL;
	m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
	m_jProgress.MoveWindow(size.cx, m_ecyPbPadT,
		iRightPos[m_eInfoPart] - size.cx - m_ecxPbPadR, m_cyStatusBar - m_ecyPbPadB);
	m_jProgress.SetPos(iPrecent);
	m_jStatusBar.SetWindowText(szText);
	if(e_dwWinVer > WV_XP)
		PeekUpdateMessage(FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Displays a message

int CTomPoiEdr::DisplayMessage(UINT nResId, UINT nType, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::DisplayMessage\n"));

	TCHAR szResText[MAX_STR], szMessage[MAX_FNAME + countof(szResText)];
	int   iResult;
	LoadString(e_hInstance, nResId, szResText, countof(szResText));
	wsprintf(szMessage, szResText, lParam);
	m_ceDragDrop++;
	iResult = MessageBox(szMessage, e_pszTpeAppName, nType);
	m_ceDragDrop--;
	return iResult;
}

int CTomPoiEdr::DisplayMessage(UINT nResId, UINT nType)
{
	TRACE(_T("Entering CTomPoiEdr::DisplayMessage\n"));

	TCHAR szFileTitle[MAX_FNAME];
	TomFile2WinFile(m_jFileDlg.lpstrFileTitle, szFileTitle);
	return DisplayMessage(nResId, nType, (LPARAM)szFileTitle);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Undoes/Redoes actions of layers

void CTomPoiEdr::DoAction(UINT nResId)
{
	TRACE(_T("Entering CTomPoiEdr::DoAction\n"));

	TCHAR  szText[MAX_STR];
	UINT   nTbBtnId;
	LVITEM lvi;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	nTbBtnId = (nResId == IDP_UNDOING_ACTION) ? ID_POI_UNDO : ID_POI_REDO;
	m_jToolbar.PressButton(nTbBtnId);
	m_jToolbar.UpdateWindow();
	m_jListView.SetFocus();
	LoadString(e_hInstance, nResId, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	if(nResId == IDP_UNDOING_ACTION)
		m_jListView.UndoAction();
	else
		m_jListView.RedoAction();
	m_jListView.SetWindowRedraw();
	m_jToolbar.PressButton(nTbBtnId, FALSE);
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Encodes character entities

int CTomPoiEdr::EncodeEntity(PTSTR pszData)
{
	TRACE(_T("Entering CTomPoiEdr::EncodeEntity\n"));

	SCTCHAR scEntity[] =
	{
		CHR_AMP,
		CHR_LT,
		CHR_GT,
		CHR_DQM,
	};
	SCPCTSTR spszEntity[] =
	{
		m_szAmpEntity,
		m_szLtEntity,
		m_szGtEntity,
		m_szQuotEntity,
	};
	SCINT sccEntity[] =
	{
		countof(m_szAmpEntity) - 1,
		countof(m_szLtEntity) - 1,
		countof(m_szGtEntity) - 1,
		countof(m_szQuotEntity) - 1,
	};
	TCHAR szBuffer[e_eccNameCol * countof(m_szQuotEntity)];
	for(int i = 0, j = 0, k; pszData[i]; i++, j++)
	{
#ifndef UNICODE
		if(IsDBCSLeadByte(pszData[i]))
		{
			*(PWCHAR)(szBuffer + j) = *(PWCHAR)(pszData + i);
			i++;
			j++;
			continue;
		}
#endif
		k = 0;
		do
		{
			if(pszData[i] == scEntity[k])
				break;
			k++;
		}
		while(k < countof(scEntity));
		if(k == countof(scEntity))
		{
			szBuffer[j] = pszData[i];
			continue;
		}
		CopyMemory(szBuffer + j, spszEntity[k], sccEntity[k] * sizeof(TCHAR));
		j += sccEntity[k] - 1;
	}
	CopyMemory(pszData, szBuffer, j * sizeof(TCHAR));
	pszData[j] = CHR_NULL;
	return j;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Encodes a URL string

int CTomPoiEdr::EncodeUrl(PTSTR pszUrl)
{
	TRACE(_T("Entering CTomPoiEdr::EncodeUrl\n"));

	char szUtf8[e_eccNameCol * FT_UTF8S3];
	int  i, j, k;
#ifdef UNICODE
	WCStoMBS(CP_UTF8, pszUrl, szUtf8, sizeof(szUtf8));
#else
	WCHAR szwUrl[e_eccNameCol];
	WCStoMBS(CP_UTF8, szwUrl, MBStoWCS(pszUrl, szwUrl, countof(szwUrl)),
		szUtf8, sizeof(szUtf8));
#endif
	i = 0;
	j = 0;
	do
	{
		if(szUtf8[j] < 0)
			k = (szUtf8[j] >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
		else
		{
			if(szUtf8[j] != CHR_SP)
			{
				pszUrl[i] = (BYTE)szUtf8[j];
				i++;
				j++;
				continue;
			}
			k = 1;
		}
		do
		{
			wsprintf(pszUrl + i, _T("%%%X"), (BYTE)szUtf8[j]);
			i += 3;
			j++;
			k--;
		}
		while(k);
	}
	while(szUtf8[j]);
	pszUrl[i] = CHR_NULL;
	return i;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Ensures the list view item is visible

void CTomPoiEdr::EnsureLvItemVisible(int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::EnsureLvItemVisible\n"));

	if(iLvItem == RV_ERR)
		return;

	int ceLvItem;
	ceLvItem = m_jListView.GetItemCount();
	if(!ceLvItem)
		return;

	int ceItemPage, iTopItem, iBottomItem;
	iTopItem    = m_jListView.GetTopIndex();
	ceItemPage  = min(m_jListView.GetCountPerPage(), ceLvItem);
	iBottomItem = min(iTopItem + ceItemPage, ceLvItem) - 1;
	if(iLvItem >= ceLvItem)
		iLvItem = ceLvItem - 1;
	if((iLvItem < iTopItem) || (iLvItem > iBottomItem))
	{
		iTopItem    = max(0, iLvItem - ceItemPage / 2);
		iBottomItem = min(iTopItem + ceItemPage, ceLvItem) - 1;
		m_jListView.EnsureVisible(iTopItem);
		m_jListView.EnsureVisible(iBottomItem);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds the base point of POIs

int CTomPoiEdr::FindBasePoint()
{
	TRACE(_T("Entering CTomPoiEdr::FindBasePoint\n"));

	int    ceLvItem;
	LVITEM lvi;
	ceLvItem     = m_jListView.GetItemCount();
	lvi.mask     = LVIF_PARAM;
	lvi.iSubItem = 0;
	for(lvi.iItem = 0; lvi.iItem < ceLvItem; lvi.iItem++)
	{
		m_jListView.GetItem(&lvi);
		if(lvi.lParam == MIN_LPARAM)
			return lvi.iItem;
	}
	return LVI_ERR;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds a duplicated string on POI names except itself

int CTomPoiEdr::FindPoiName(PCTSTR pszPoiName, int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::FindPoiName\n"));

	TCHAR      szHitText[e_eccNameCol];
	int        iHitItem;
	LVITEM     lvi;
	LVFINDINFO lvfi;
	lvfi.flags = LVFI_STRING;
	lvfi.psz   = pszPoiName;
	iHitItem   = -1;
	while((iHitItem = m_jListView.FindItem(&lvfi, iHitItem)) != RV_ERR)
	{
		if(iHitItem == iLvItem)
			continue;
		lvi.iSubItem   = LVC_NAME;
		lvi.pszText    = szHitText;
		lvi.cchTextMax = countof(szHitText);
		m_jListView.GetItemText(iHitItem, &lvi);
		if(!lstrcmp(szHitText, pszPoiName))
			return iHitItem;
	}
	return RV_ERR;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds a duplicated string on POI names

BOOL CTomPoiEdr::FindPoiName(PCTSTR pszPoiName)
{
	TRACE(_T("Entering CTomPoiEdr::FindPoiName\n"));

	if(m_nPoiDupSel == IDC_RETAIN_ALL)
		return FALSE;

	TCHAR      szText[e_eccNameCol];
	int        iHitItem;
	LVFINDINFO lvfi;
	LVITEM     lvi;
	lvfi.flags     = LVFI_STRING;
	lvfi.psz       = pszPoiName;
	lvi.iSubItem   = LVC_NAME;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	iHitItem       = -1;
	do
	{
		iHitItem = m_jListView.FindItem(&lvfi, iHitItem);
		if(iHitItem == RV_ERR)
			return FALSE;

		m_jListView.GetItemText(iHitItem, &lvi);
	}
	while(lstrcmp(szText, pszPoiName));

	if(m_nPoiDupSel == IDC_DELETE_ALL)
	{
		m_cePoiDupDel++;
		return TRUE;
	}

	CDupPoiDlg jDupPoiDlg(pszPoiName);
	m_ceDragDrop++;
	m_nPoiDupSel = jDupPoiDlg.DialogBox(e_hInstance, IDD_DUP_POI, *this);
	m_ceDragDrop--;
	if((m_nPoiDupSel != IDC_DELETE) && (m_nPoiDupSel != IDC_DELETE_ALL))
		return FALSE;
	m_cePoiDupDel++;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds a POI record

int CTomPoiEdr::FindPoiRecord(int iLvItem, BOOL bMarkAll)
{
	TRACE(_T("Entering CTomPoiEdr::FindPoiRecord\n"));

	TCHAR  szText[e_eccNameCol];
	PTSTR  (WINAPI* StrStrx)(PCTSTR, PCTSTR);
	int    ceLvItem, iLastCol;
	LVITEM lvi;
	ASSERT(!bMarkAll || !iLvItem);
	StrStrx        = (m_jFindReplDlg.m_nMatchCase == BST_CHECKED) ? StrStr : StrStrI;
	ceLvItem       = m_jListView.GetItemCount();
	iLastCol       = (m_jFindReplDlg.m_nOnlyName == BST_CHECKED) ? LVC_NAME : LVC_MEMO;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	lvi.state      = UNUSED;
	lvi.stateMask  = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
	for(int i = iLvItem, j; i < ceLvItem; i++)
	{
		j = LVC_NAME;
		do
		{
			lvi.iSubItem = j;
			m_jListView.GetItemText(i, &lvi);
			if(StrStrx(szText, m_jFindReplDlg.m_pszFindText))
				break;
			j++;
		}
		while(j <= iLastCol);
		if(j <= iLastCol)
		{
			m_jListView.SetItemState(i, &lvi);
			if(bMarkAll)
				continue;
			return i;
		}
		if(bMarkAll)
			continue;
		i = (i + 1) % ceLvItem;
		if(i == iLvItem)
			return RV_ERR;
		i--;
	}
	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets command from Google Maps

void CTomPoiEdr::GetGoogleMaps()
{
	TRACE(_T("Entering CTomPoiEdr::GetGoogleMaps\n"));

	if(!m_jIEPane.IsWindowVisible() || !m_pHD2)
		return;

	IDispatch* pDsp;
	m_pHD2->get_Script(&pDsp);
	if(!pDsp)
		return;

	SPOSTR spszGetMethod = _O("GetGoogleMaps");
	DISPID di;
	if(pDsp->GetIDsOfNames(IID_NULL, &spszGetMethod, 1, LOCALE_USER_DEFAULT, &di) == S_OK)
	{
		SDISPPARAMS dp;
		VARIANT     vtResult;
		vtResult.vt      = VT_BSTR;
		vtResult.bstrVal = SysAllocString(OSTR_NULL);
		if((pDsp->Invoke(di, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dp,
			&vtResult, NULL, NULL) == S_OK) && vtResult.bstrVal && *vtResult.bstrVal)
		{
			PTSTR pszRecord;
			TCHAR szText[e_eccNameCol];
#ifdef UNICODE
			pszRecord = vtResult.bstrVal;
#else
			char szRecord[e_eccAllCol * 2];
			WCStoMBS(vtResult.bstrVal, szRecord, sizeof(szRecord));
			pszRecord = szRecord;
#endif
			if(m_bGmReady)
			{
				if((*pszRecord == m_eGmTokenGet) && (pszRecord[1] == m_eGmTokenSep))
				{
					if(m_jLvEdit.IsWindowVisible())
						m_jLvEdit.PostMessage(WM_CHAR, CHR_ESC);
					else
					{
						PTSTR pszValue[m_eceGmRdItem];
						int   i, iGetCmd, iSelItem;
						pszRecord += 2;
						iGetCmd    = *pszRecord - _T('0');
						ASSERT((iGetCmd >= 0) && (iGetCmd < m_eceGmGet));
						ZeroMemory(pszValue, sizeof(pszValue));
						for(i = m_eGmRdName; pszRecord = StrChr(pszRecord, m_eGmTokenSep); i++)
						{
							*pszRecord = NULL;
							pszRecord++;
							pszValue[i] = pszRecord;
						}
						while(i > m_eGmRdLng)
						{
							i--;
							if((i != m_eGmRdLng) && (i != m_eGmRdLat) &&
								(i != m_eGmRdNewLng) && (i != m_eGmRdNewLat))
								continue;
							if(!lstrcmp(pszValue[i], _T("-0.00000")))
								pszValue[i]++;
						}

						SBOOL  sbUpdateAll;
						LVITEM lvi;
						lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
						if(!sbUpdateAll)
						{
							if(iGetCmd == m_eGmGetAdd)
								iSelItem = MAX_INT;
							else if(iGetCmd == m_eGmGetInsert)
							{
								iGetCmd  = m_eGmGetAdd;
								iSelItem = m_jListView.GetNextItem(LVNI_SELECTED);
								if(iSelItem == RV_ERR)
									iSelItem = MAX_INT;
								else if(!iSelItem && m_bMyBP)
									iSelItem = 1;
							}
							lvi.state = UNUSED;
							m_jListView.SetItemState(&lvi);
							if((iGetCmd != m_eGmGetSelect) && (iGetCmd != m_eGmGetEdit))
								m_jListView.SetAction(AT_NEW);
						}
						if(pszValue[m_eGmRdName])
						{
							BOOL bSelItem;
							int  iLvItem;
							bSelItem = TRUE;
							if(iGetCmd != m_eGmGetAdd)
							{
								int ceLvItem;
								ceLvItem       = m_jListView.GetItemCount();
								lvi.pszText    = szText;
								lvi.cchTextMax = countof(szText);
								for(iLvItem = 0; iLvItem < ceLvItem; iLvItem++)
								{
									i = LVC_NAME;
									do
									{
										lvi.iSubItem = i;
										GetCoordDdString(i, szText,
											m_jListView.GetItemText(iLvItem, &lvi));
										if(lstrcmp(szText, pszValue[i]))
											break;
										i++;
									}
									while(i <= LVC_MEMO);
									if(i > LVC_MEMO)
										break;
								}
								if(iLvItem == ceLvItem)
								{
									iGetCmd  = m_eGmGetSelect;
									bSelItem = FALSE;
								}
								else if(iGetCmd == m_eGmGetUpdate)
								{
									pszValue[m_eGmRdLng]  = pszValue[m_eGmRdNewLng];
									pszValue[m_eGmRdLat]  = pszValue[m_eGmRdNewLat];
									pszValue[m_eGmRdMemo] = pszValue[m_eGmRdNewMemo];
								}
								else if(iGetCmd == m_eGmGetDelete)
								{
									m_jListView.DeleteItem(iLvItem);
									SetBasePoint(LVI_FIND, FALSE);
									if(!iLvItem && m_bMyBP)
										m_bMyBP = FALSE;
									else
										m_bFileDirty = TRUE;
									bSelItem = FALSE;
								}
							}
							if(iGetCmd == m_eGmGetAdd)
							{
								ValidateNameCol(pszValue[m_eGmRdName]);
								lvi.mask     = LVIF_TEXT;
								lvi.iItem    = iSelItem;
								lvi.iSubItem = 0;
								lvi.pszText  = pszValue[m_eGmRdName];
								iLvItem      = m_jListView.InsertItem(&lvi);
								ValidateTelCol(iLvItem, pszValue[m_eGmRdTel]);
							}
							if((iGetCmd != m_eGmGetSelect) && (iGetCmd != m_eGmGetEdit))
							{
								if(iGetCmd != m_eGmGetDelete)
								{
									ValidateCoordCol(iLvItem, LVC_LNG,
										pszValue[m_eGmRdLng], TRUE);
									ValidateCoordCol(iLvItem, LVC_LAT,
										pszValue[m_eGmRdLat], TRUE);
									ValidateMemoCol(iLvItem, pszValue[m_eGmRdMemo]);
									if(m_iLvBaseItem != LVI_ERR)
										SetDistanceText2(iLvItem);
									if(iLvItem || !m_bMyBP)
										m_bFileDirty = TRUE;
									AutoSizeLvCol(LVC_ALL, TRUE);
								}
								m_bSetCaption = TRUE;
							}
							if(bSelItem)
							{
								lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
								m_jListView.SetItemState(iLvItem, &lvi);
								if(!sbUpdateAll)
									EnsureLvItemVisible(iLvItem);
							}
							if(iGetCmd == m_eGmGetEdit)
							{
								m_iLvCol = *pszValue[m_eGmRdNewLng] - _T('0');
								BeginLvEdit(iLvItem);
							}
						}
						else if(iGetCmd == m_eGmGetUpdate)
						{
							sbUpdateAll = TRUE;
							PutGoogleMaps(m_eGmPutUpdate, UNUSED);
							sbUpdateAll = FALSE;
						}
						m_bGmSelItem = FALSE;
					}
				}
			}
			else if(lstrcmp(pszRecord, e_pszTpeAppName))
			{
				LoadString(e_hInstance, IDS_TERMINATE_GOOGLE_MAP, szText, countof(szText));
				if(!lstrcmp(pszRecord, szText))
					CloseIEPane();
			}
			else
			{
				m_bGmReady   = TRUE;
				m_bGmSelItem = TRUE;
				m_bSetReady  = TRUE;
			}
		}
		SysFreeString(vtResult.bstrVal);
	}
	pDsp->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the pure name of a path/file

void CTomPoiEdr::GetPurePathName(PCTSTR pszFileName, PTSTR pszBuffer)
{
	TRACE(_T("Entering CTomPoiEdr::GetPurePathName\n"));

	TCHAR szPureName[MAX_PATH];
	if(PathUnExpandEnvStrings(pszFileName, szPureName, countof(szPureName)))
	{
		SCTCHAR  szCurUser[]  = _T("%USERPROFILE%\\");
		SCTCHAR  szAllUser[]  = _T("%ALLUSERSPROFILE%\\");
		SCTCHAR  szSysDrive[] = _T("%SystemDrive%\\");
		SCPCTSTR spszUser[]   = { szCurUser, szAllUser, szSysDrive };
		SCINT    sccUser[]    =
		{
			countof(szCurUser) - 1,
			countof(szAllUser) - 1,
			countof(szSysDrive) - 1,
		};
		SCTCHAR  szDesktop[]  = _T("Desktop");
		SCTCHAR  szDoc[]      = _T("Documents");
		SCTCHAR  szMyDoc[]    = _T("My Documents");
		SCTCHAR  szShareDoc[] = _T("Users\\Public\\Documents");
		SCPCTSTR spszDoc[]    = { szDesktop, szDoc, szMyDoc, szShareDoc };
		SCINT    sccDoc[]     =
		{
			countof(szDesktop) - 1,
			countof(szDoc) - 1,
			countof(szMyDoc) - 1,
			countof(szShareDoc) - 1,
		};
		int    i, j, k;
		PCTSTR pszDoc, pszRemain;
		i = 0;
		do
		{
			if(!StrCmpNI(szPureName, spszUser[i], sccUser[i]))
			{
				pszDoc = szPureName + sccUser[i];
				if(!*pszDoc)
					break;
				j = 0;
				do
				{
					if(!StrCmpNI(pszDoc, spszDoc[j], sccDoc[j]))
					{
						pszRemain = pszDoc + sccDoc[j];
						if(*pszRemain && (*pszRemain != CHR_BSLH))
							break;
						k = j + IDS_FIRST_DOCUMENT;
						if(e_dwWinVer > WV_XP)
						{
							if(k == IDS_DOCUMENT)
								k = IDS_MY_DOCUMENT;
							else if(k == IDS_SHARE_DOCUMENT_XP)
								k = IDS_SHARE_DOCUMENT;
						}
						else if(k == IDS_DOCUMENT)
							k = IDS_SHARE_DOCUMENT_XP;
						k = LoadString(e_hInstance, k, pszBuffer, countof(szPureName));
						if(*pszRemain)
							lstrcpy(pszBuffer + k, pszRemain);
						return;
					}
					j++;
				}
				while(j < countof(spszDoc));
				if(i != (countof(spszUser) - 1))
					pszFileName = pszDoc;
				break;
			}
			i++;
		}
		while(i < countof(spszUser));
	}
	lstrcpy(pszBuffer, pszFileName);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the character which is used in TomTom file name

AFXINLINE BYTE CTomPoiEdr::GetTomFileCode(WCHAR wChar)
{
	TRACE(_T("Entering CTomPoiEdr::GetTomFileCode\n"));

	if(IsValidNumber(wChar))
		return (wChar - _T('0'));
	if(IsCharAlpha(wChar))
		return (CharUpper(wChar) - _T('A') + m_eceNumber);
	if(wChar == CHR_LCB)
		return (m_eceNumber + m_eceAlpha);
	if(wChar == CHR_DSH)
		return (m_eceNumber + m_eceAlpha + 1);
	if(wChar == CHR_PS)
		return (m_eceNumber + m_eceAlpha + 2);
	if(wChar == CHR_EQL)
		return (m_eceNumber + m_eceAlpha + 3);
	if(wChar == CHR_RCB)
		return (m_eceNumber + m_eceAlpha + 4);
	return MAX_BYTE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Broadcasts the message about GPS device has been changed

void CTomPoiEdr::GpsBroadcastChange(BOOL bGpsDevice)
{
	TRACE(_T("Entering CTomPoiEdr::GpsBroadcastChange\n"));

	HWND hTpeWnd;
	hTpeWnd = NULL;
	while(hTpeWnd = ::FindWindow(HWND_DESKTOP, hTpeWnd, e_pszTpeClassName))
	{
		if(hTpeWnd == *this)
			continue;
		::PostMessage(hTpeWnd, WM_GPS_CHANGE, bGpsDevice);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Closes any dialog associated with GPS device

BOOL CTomPoiEdr::GpsCloseDialog(BOOL bPrevGpsDevice)
{
	TRACE(_T("Entering CTomPoiEdr::GpsCloseDialog\n"));

	if(!m_bGpsDevice && bPrevGpsDevice)
	{
		if(m_jToolbar.IsButtonPressed(ID_FILE_OPEN_DEVICE) ||
			m_jToolbar.IsButtonPressed(ID_FILE_DUMP_DEVICE) ||
			m_jToolbar.IsButtonPressed(ID_FILE_SAVE_DEVICE))
		{
			HWND hDialog;
			hDialog = GetLastActivePopup();
			ASSERT(hDialog != m_jCheckDataDlg);
			while(hDialog != *this)
			{
				::PostMessage(hDialog, WM_COMMAND, MAKEWPARAM(ID_CANCEL, BN_CLICKED));
				::PostMessage(hDialog, WM_COMMAND, MAKEWPARAM(ID_NO, BN_CLICKED));
				hDialog = ::GetWindow(hDialog, GW_OWNER);
			}
		}
	}
	if(m_bGpsDevice == bPrevGpsDevice)
		return FALSE;

	EndMenu();
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Creates a marquee window

void CTomPoiEdr::GpsCreateMarquee(UINT nResId)
{
	TRACE(_T("Entering CTomPoiEdr::GpsCreateMarquee\n"));

	TCHAR szText[MAX_STR];
	int   iRightPos[m_eceSbPart];
	SIZE  size;
	m_bSetReady = FALSE;
	GetTextExtent(m_hSbDC, szText, LoadString(e_hInstance, nResId,
		szText, countof(szText)), &size);
	size.cx += m_ecxPbPadL;
	m_jStatusBar.SetWindowText(szText);
	m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
	m_jProgress.CreateWindow(m_ePbMqStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
	m_jProgress.SetMarquee(m_ePbMqIntl);
	m_jProgress.MoveWindow(size.cx, m_ecyPbPadT,
		iRightPos[m_eInfoPart] - size.cx - m_ecxPbPadR, m_cyStatusBar - m_ecyPbPadB);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets a file from GPS device

DWORD CTomPoiEdr::GpsGetFile(HINTERNET hInternet, BOOL bPoiPath,
	PCTSTR pszFileName, BOOL bOv2File, PTSTR pszTempFile)
{
	TRACE(_T("Entering CTomPoiEdr::GpsGetFile\n"));

	PTSTR  pszNextPos;
	PCTSTR pszImageExt;
	TCHAR  szUrl[MAX_STR + MAX_FNAME * FT_UTF8S3 * 3];
	UINT   nResId;
	HHTTP  hHttp;
	if(bPoiPath)
	{
		nResId      = IDS_GPS_POI_PATH;
		pszImageExt = bOv2File ? m_szOv2Ext : m_szBmpExt;
	}
	else
	{
		ASSERT((m_gpsd == GPSD_V3) || (m_gpsd == GPSD_V4));
		if(m_gpsd == GPSD_V3)
		{
			nResId      = IDS_GPS_DUMP_PATH3;
			pszImageExt = m_szBmpExt;
		}
		else
		{
			nResId      = IDS_GPS_DUMP_PATH4;
			pszImageExt = m_szPngExt;
		}
	}
	pszNextPos  = szUrl;
	pszNextPos += LoadString(e_hInstance, IDS_GPS_HTTP_PREFIX, pszNextPos, MAX_STR);
	pszNextPos += LoadString(e_hInstance, IDS_GPS_SITE_IP, pszNextPos, MAX_STR);
	pszNextPos += LoadString(e_hInstance, nResId, pszNextPos, MAX_STR);
	*pszNextPos = CHR_SLH;
	pszNextPos++;
	lstrcpy(pszNextPos, pszFileName);
	pszNextPos += EncodeUrl(pszNextPos);
	*pszNextPos = CHR_DOT;
	CopyMemory(pszNextPos + 1, pszImageExt, sizeof(m_szOv2Ext));
	hHttp = InternetOpenUrl(hInternet, szUrl, INTERNET_FLAG_DONT_CACHE);
	if(!hHttp)
		return IDS_GPS_CONNECT_ERROR;

	DWORD dwResId;
	HFILE hFile;
	lstrcpy(pszTempFile, m_pszGmTempFile);
	PathRenameExtension(pszTempFile, pszNextPos);
	hFile = CreateFile(pszTempFile, GENERIC_WRITE, CREATE_ALWAYS);
	if(hFile != INVALID_HANDLE)
	{
		char  szBuffer[m_ecbGpsBuffer], szError[16];
		DWORD cbError, cbRead, cbDone;
		dwResId = IDS_GPS_READ_ERROR;
		cbError = LoadStringA(e_hInstance, IDS_GPS_ERROR, szError, sizeof(szError));
		cbDone  = 0;
		do
		{
			InternetReadFile(hHttp, szBuffer, sizeof(szBuffer), &cbRead);
			if(!cbRead && cbDone)
			{
				dwResId = UNUSED;
				break;
			}
			if(!cbDone)
			{
				if(cbRead < cbError)
					break;
				if(!StrCmpNIA(szBuffer, szError, cbError))
					break;
			}
			cbDone += cbRead;
		}
		while(WriteFile(hFile, szBuffer, cbRead, &cbRead));
		CloseHandle(hFile);
		if(dwResId)
			DeleteFile(pszTempFile);
	}
	else
		dwResId = IDS_GPS_FILE_ERROR;
	InternetCloseHandle(hHttp);
	return dwResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Gets the file list for GPS device

DWORD CTomPoiEdr::GpsGetFileList(HINTERNET hInternet, BOOL bPoiPath, PGPSFILELIST pGPSFL)
{
	TRACE(_T("Entering CTomPoiEdr::GpsGetFileList\n"));

	PTSTR  pszNextPos, pszPath;
	PCTSTR pszImageExt;
	TCHAR  szText[MAX_STR], szLatest[64];
	UINT   nResId;
	HHTTP  hHttp;
	if(bPoiPath)
		nResId = IDS_GPS_POI_PATH;
	else
	{
		ASSERT((m_gpsd == GPSD_V3) || (m_gpsd == GPSD_V4));
		if(m_gpsd == GPSD_V3)
		{
			nResId      = IDS_GPS_DUMP_PATH3;
			pszImageExt = m_szBmpExt;
			LoadString(e_hInstance, IDS_GPS_NAME_LATEST, szLatest, countof(szLatest));
		}
		else
		{
			nResId      = IDS_GPS_DUMP_PATH4;
			pszImageExt = m_szPngExt;
		}
	}
	LoadString(e_hInstance, IDP_GPS_READING_LIST, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	pszNextPos  = szText;
	pszNextPos += LoadString(e_hInstance, IDS_GPS_HTTP_PREFIX, pszNextPos, countof(szText));
	pszNextPos += LoadString(e_hInstance, IDS_GPS_SITE_IP, pszNextPos, countof(szText));
	pszPath     = pszNextPos;
	pszNextPos += LoadString(e_hInstance, nResId, pszNextPos, countof(szText));
	if(bPoiPath)
	{
		*pszNextPos = CHR_SLH;
		pszNextPos++;
		LoadString(e_hInstance, IDS_GPS_SORT_TYPE, pszNextPos, countof(szText));
	}
	hHttp = InternetOpenUrl(hInternet, szText, INTERNET_FLAG_DONT_CACHE);
	if(!hHttp)
		return IDS_GPS_CONNECT_ERROR;

	PSTR  pszBuffer;
	int   i;
	DWORD dwResId, cbAlloc, cbRead, cbDone;
	pszBuffer = NULL;
	cbAlloc   = 0;
	cbRead    = 0;
	cbDone    = 0;
	do
	{
		cbDone += cbRead;
		i       = cbAlloc - cbDone - 1;
		if(i <= 0)
		{
			cbAlloc  += m_ecbGpsAlloc;
			i        += m_ecbGpsAlloc;
			pszBuffer = (PSTR)realloc(pszBuffer, cbAlloc);
		}
		InternetReadFile(hHttp, pszBuffer + cbDone, i, &cbRead);
	}
	while(cbRead);
	InternetCloseHandle(hHttp);
	dwResId = IDS_GPS_LIST_ERROR;
	if(cbDone)
	{
		char  szError[16];
		DWORD cbError;
		cbError = LoadStringA(e_hInstance, IDS_GPS_ERROR, szError, sizeof(szError));
		if((cbDone >= cbError) && StrCmpNIA(pszBuffer, szError, cbError))
		{
			PTSTR pszHtmData;
			PWSTR pszwBuffer;
			pszwBuffer = (PWSTR)malloc(cbAlloc * sizeof(WCHAR));
			i          = MBStoWCS(CP_UTF8, pszBuffer, cbDone, pszwBuffer, cbAlloc);
#ifdef UNICODE
			pszHtmData = pszwBuffer;
#else
			i          = WCStoMBS(pszwBuffer, i, pszBuffer, cbAlloc);
			pszHtmData = pszBuffer;
			free(pszwBuffer);
#endif
			pszHtmData[i]      = CHR_NULL;
			pGPSFL->pszHtmData = pszHtmData;

			PTCHAR    pcTemp, pcStart, pcEnd, pcNext;
			PTSTR     pszFileExt;
			PGPSFILE  pGPSF;
			PPGPSFILE ppGPSF;
			TCHAR     szToken1[32], szToken2[countof(szToken1)];
			int       ccPath, ccToken1, ccToken2;
			pszPath--;
			*pszPath = CHR_DQM;
			if(!bPoiPath)
			{
				*pszNextPos = CHR_SLH;
				pszNextPos++;
			}
			*pszNextPos = CHR_NULL;
			ccPath      = pszNextPos - pszPath;
			ccToken1    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN1,
				szToken1, countof(szToken1));
			ccToken2    = LoadString(e_hInstance, IDS_GPS_LIST_TOKEN2,
				szToken2, countof(szToken2));
			ppGPSF      = pGPSFL->pGPSF;
			ZeroMemory(ppGPSF, sizeof(pGPSFL->pGPSF));
			pcStart = pszHtmData;
			while(pcStart = StrStrI(pcStart, pszPath))
			{
				pcStart += ccPath;
				pcTemp   = StrStr(pcStart, szToken1);
				if(!pcTemp)
					continue;
				pcStart = pcTemp + ccToken1;
				if(!bPoiPath && (m_gpsd == GPSD_V3))
					pcStart += ccPath - 1;
				pcEnd = StrStr(pcStart, szToken2);
				if(!pcEnd)
					continue;
				i      = pcEnd - pcStart;
				pcNext = pcEnd + ccToken2;
				if(i < MAX_FNAME)
				{
					*pcEnd     = CHR_NULL;
					pszFileExt = PathFindExtension(pcStart);
					pcTemp     = pszFileExt;
					if(*pszFileExt)
					{
						pszFileExt++;
						if(*pszFileExt)
						{
							if(bPoiPath)
							{
								i = 0;
								do
								{
									if(!lstrcmpi(pszFileExt, m_pszGPSF[i]))
										break;
									i++;
								}
								while(i < GPSF_COUNT);
							}
							else
								i = lstrcmpi(pszFileExt, pszImageExt) ? GPSF_COUNT : GPSF_BMP;
							if(i < GPSF_COUNT)
							{
								*pcTemp = CHR_NULL;
								TrimString(pcStart);
								if(bPoiPath || (m_gpsd != GPSD_V3) ||
									lstrcmpi(pcStart, szLatest))
								{
									pGPSF          = (PGPSFILE)malloc(sizeof(GPSFILE));
									pGPSF->pszName = pcStart;
									pGPSF->pszAttr = pcNext;
									pGPSF->pNext   = ppGPSF[i];
									ppGPSF[i]      = pGPSF;
								}
							}
						}
					}
				}
				pcStart = pcNext;
			}
			dwResId = UNUSED;
		}
	}
#ifdef UNICODE
	free(pszBuffer);
#else
	if(dwResId)
		free(pszBuffer);
#endif
	return dwResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Finds an item by name in GPS device

BOOL CTomPoiEdr::GpsFindFile(PGPSFILE pGPSF, PCTSTR pszName)
{
	TRACE(_T("Entering CTomPoiEdr::GpsFindFile\n"));

	if(!pGPSF)
		return FALSE;

	do
	{
		if(!lstrcmpi(pGPSF->pszName, pszName))
			return TRUE;
		pGPSF = pGPSF->pNext;
	}
	while(pGPSF);
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Frees the file list for GPS device

void CTomPoiEdr::GpsFreeFileList(PGPSFILELIST pGPSFL)
{
	TRACE(_T("Entering CTomPoiEdr::GpsFreeFileList\n"));

	PGPSFILE  pGPSF, pNextGPSF;
	PPGPSFILE ppGPSF;
	int       i;
	ppGPSF = pGPSFL->pGPSF;
	i      = GPSF_FIRST;
	do
	{
		pGPSF = ppGPSF[i];
		while(pGPSF)
		{
			pNextGPSF = pGPSF->pNext;
			free(pGPSF);
			pGPSF = pNextGPSF;
		}
		i++;
	}
	while(i < GPSF_COUNT);
	free(pGPSFL->pszHtmData);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Initializes WinINet functions for GPS device

HINTERNET CTomPoiEdr::GpsInternetOpen()
{
	TRACE(_T("Entering CTomPoiEdr::GpsInternetOpen\n"));

	HINTERNET hInternet;
	hInternet = InternetOpen(e_pszTpeAppName, INTERNET_OPEN_TYPE_PRECONFIG);
	if(!hInternet)
		return NULL;

	SDWORD sdwTimeout = m_eGpsTimeout;
	InternetSetOption(hInternet, INTERNET_OPTION_CONNECT_TIMEOUT,
		&sdwTimeout, sizeof(sdwTimeout));
	return hInternet;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether the dump path is valid

BOOL CTomPoiEdr::GpsIsValidDump(HINTERNET hInternet, GPSDUMP gpsd)
{
	TRACE(_T("Entering CTomPoiEdr::GpsIsValidDump\n"));

	PTSTR pszPath;
	TCHAR szText[MAX_STR];
	BOOL  bCloseHandle;
	HHTTP hHttp;
	ASSERT((gpsd == GPSD_V3) || (gpsd == GPSD_V4));
	if(hInternet)
		bCloseHandle = FALSE;
	else
	{
		hInternet = GpsInternetOpen();
		if(!hInternet)
			return FALSE;
		bCloseHandle = TRUE;
	}
	m_gpsd   = GPSD_NONE;
	pszPath  = szText + LoadString(e_hInstance, IDS_GPS_HTTP_PREFIX, szText, countof(szText));
	pszPath += LoadString(e_hInstance, IDS_GPS_SITE_IP, pszPath, countof(szText));
	LoadString(e_hInstance, (gpsd == GPSD_V3) ? IDS_GPS_DUMP_PATH3 : IDS_GPS_DUMP_PATH4,
		pszPath, countof(szText));
	hHttp = InternetOpenUrl(hInternet, szText, INTERNET_FLAG_DONT_CACHE);
	if(hHttp)
	{
		char  szResponse[16];
		DWORD cbRead;
		InternetReadFile(hHttp, szResponse, sizeof(szResponse), &cbRead);
		if(cbRead)
		{
			char  szError[sizeof(szResponse)];
			DWORD cbError;
			cbError = LoadStringA(e_hInstance, IDS_GPS_ERROR, szError, sizeof(szError));
			if((cbRead >= cbError) && StrCmpNIA(szResponse, szError, cbError))
				m_gpsd = gpsd;
		}
		InternetCloseHandle(hHttp);
	}
	if(bCloseHandle)
		InternetCloseHandle(hInternet);
	return (m_gpsd != GPSD_NONE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether the dump path is valid

void CTomPoiEdr::GpsVerifyDump(HINTERNET hInternet)
{
	TRACE(_T("Entering CTomPoiEdr::GpsVerifyDump\n"));

	if(!GpsIsValidDump(hInternet, GPSD_V3))
		GpsIsValidDump(hInternet, GPSD_V4);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts a recent file into menu

void CTomPoiEdr::InsertRecentFile(PCTSTR pszFileName, int iListNumber)
{
	TRACE(_T("Entering CTomPoiEdr::InsertRecentFile\n"));

	TCHAR szPureName[MAX_PATH], szItemText[countof(szPureName) + 3];
	PTSTR pszItemName;
	szItemText[0] = CHR_AMP;
	szItemText[1] = _T('1') + iListNumber;
	szItemText[2] = CHR_SP;
	pszItemName   = szItemText + 3;
	GetPurePathName(pszFileName, szPureName);
	TomFile2WinFile(szPureName, pszItemName);
	PathCompactPath(m_hSbDC, pszItemName, m_ecxRecentItem);
	InsertMenu(m_hFileMenu, m_eRecentInsPos + iListNumber, MF_BYPOSITION | MF_STRING,
		ID_RECENT_FILE1 + iListNumber, szItemText);
	if(e_dwWinVer > WV_XP)
		return;

	MENUITEMINFO mii;
	mii.cbSize     = sizeof(mii);
	mii.fMask      = MIIM_BITMAP | MIIM_DATA;
	mii.hbmpItem   = HBMMENU_CALLBACK;
	mii.dwItemData = MAX_DWORD;
	SetMenuItemInfo(m_hMainMenu, ID_RECENT_FILE1 + iListNumber, FALSE, &mii);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts a recent file into menu

void CTomPoiEdr::InsertRecentFile(PCTSTR pszFileName)
{
	TRACE(_T("Entering CTomPoiEdr::InsertRecentFile\n"));

	if(m_jOptionDlg.m_nRecentFile == BST_UNCHECKED)
		return;

	int ceMoveItem;
	if(!m_ceRecentFile)
		InsertMenu(m_hFileMenu, m_eRecentInsPos, MF_BYPOSITION | MF_SEPARATOR);
	for(int i = 0; i < m_ceRecentFile; i++)
		if(!lstrcmpi(pszFileName, m_pszRecentFile + i * MAX_PATH))
			break;
	if(i == m_ceRecentFile)
		m_ceRecentFile = min(m_ceRecentFile + 1, m_jOptionDlg.m_ceRecentFile);
	ceMoveItem = min(i, m_ceRecentFile - 1);
	MoveMemory(m_pszRecentFile + MAX_PATH, m_pszRecentFile,
		ceMoveItem * MAX_PATH * sizeof(TCHAR));
	lstrcpy(m_pszRecentFile, pszFileName);
	for(i = 0; i <= ceMoveItem; i++)
	{
		RemoveMenu(m_hMainMenu, ID_RECENT_FILE1 + i, MF_BYCOMMAND);
		InsertRecentFile(m_pszRecentFile + i * MAX_PATH, i);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Moves an item

void CTomPoiEdr::MoveLvItem(BOOL bMoveUp, ACTIONTYPE at)
{
	TRACE(_T("Entering CTomPoiEdr::MoveLvItem\n"));

	TCHAR     szText[MAX_STR], szSelText[e_eccNameCol], szSwapText[countof(szSelText)];
	int       i, iSelItem, iSwapItem, iLastItem, iVisibleItem;
	UINT      nTbBtnId;
	ACTIONCMD ac;
	LVITEM    lvi;
	ASSERT((at == AT_NEW) || (at == AT_CUT));
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	iSelItem = m_jListView.GetNextItem(LVNI_SELECTED);
	if(bMoveUp)
	{
		nTbBtnId     = ID_POS_UP;
		iSwapItem    = iSelItem - 1;
		iLastItem    = iSelItem + 1;
		iVisibleItem = max(0, iSelItem - 3);
		ac           = AC_MOVEUP;
	}
	else
	{
		nTbBtnId     = ID_POS_DOWN;
		iSwapItem    = iSelItem + 1;
		iLastItem    = iSwapItem + 1;
		iVisibleItem = min(iSelItem + 3, m_jListView.GetItemCount() - 1);
		ac           = AC_MOVEDOWN;
	}
	m_jToolbar.PressButton(nTbBtnId);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDP_MOVING_POI, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	m_jListView.SetAction(at);
	if(at == AT_NEW)
		m_jListView.SetAction(ac, iSwapItem);
	lvi.pszText    = szSelText;
	lvi.cchTextMax = countof(szSelText);
	i              = LVC_NAME;
	do
	{
		lvi.iSubItem = i;
		m_jListView.GetItemText(iSelItem, &lvi);
		lvi.pszText = szSwapText;
		m_jListView.GetItemText(iSwapItem, &lvi);
		m_jListView.CListView::SetItemText(iSelItem, &lvi);
		lvi.pszText = szSelText;
		m_jListView.CListView::SetItemText(iSwapItem, &lvi);
		i++;
	}
	while(i <= LVC_MEMO);
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(iSelItem, &lvi);
	lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(iSwapItem, &lvi);
	if(m_iLvBaseItem != LVI_ERR)
	{
		if(m_iLvBaseItem == iSelItem)
			m_iLvBaseItem = iSwapItem;
		else if(m_iLvBaseItem == iSwapItem)
			m_iLvBaseItem = iSelItem;
		SetDistanceText(iSelItem, FALSE);
		SetDistanceText(iSwapItem, FALSE);
		SetDistanceText(iLastItem, FALSE);
	}
	m_jListView.EnsureVisible(iVisibleItem);
	m_jCheckDataDlg.SwapItemId(iSelItem, iSwapItem);
	m_jToolbar.PressButton(nTbBtnId, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_bFileDirty  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Opens a recent or a full-path file

void CTomPoiEdr::OpenPoiFile(UINT nRecentId)
{
	TRACE(_T("Entering CTomPoiEdr::OpenPoiFile\n"));

	// Prompts to save the file, if necessary
	m_jToolbar.PressButton(ID_FILE_OPEN);
	m_jToolbar.UpdateWindow();
	if(!m_bFileDirty || PromptMessage())
	{
		// Opens the POI file
		PCTSTR pszFileName, pszFileExt;
		pszFileName = HIWORD(nRecentId) ?
			(PCTSTR)nRecentId : (m_pszRecentFile + nRecentId * MAX_PATH);
		lstrcpy(m_jFileDlg.lpstrFile, pszFileName);
		lstrcpy(m_jFileDlg.lpstrFileTitle, pszFileName);
		PathStripPath(m_jFileDlg.lpstrFileTitle);
		pszFileExt = PathFindExtension(m_jFileDlg.lpstrFileTitle);
		if(*pszFileExt)
			pszFileExt++;
		SetFileFilter(pszFileExt, FALSE);
		PathRemoveExtension(m_jFileDlg.lpstrFileTitle);
		if(OperatePoiFile(OPF_READ))
		{
			InsertRecentFile(m_jFileDlg.lpstrFile);
			m_bSetBmp = TRUE;
		}
	}
	m_jToolbar.PressButton(ID_FILE_OPEN, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Displays the Open or Save As dialog box and operates the POI file

void CTomPoiEdr::OperateFileName(OPERPOIFILE opf)
{
	TRACE(_T("Entering CTomPoiEdr::OperateFileName\n"));

	// Fills information of the dialog box structure
	TCHAR szFileName[MAX_PATH], szFileTitle[MAX_FNAME], szInitDir[MAX_PATH], szTitle[MAX_STR];
	PTSTR pszFileName, pszFileTitle;
	BOOL  bSuccess;
	UINT  nFilterIndex;
	m_ceDragDrop++;
	LoadString(e_hInstance, opf + IDS_TITLE_OPEN, szTitle, countof(szTitle));
	pszFileName               = m_jFileDlg.lpstrFile;
	pszFileTitle              = m_jFileDlg.lpstrFileTitle;
	nFilterIndex              = m_jFileDlg.nFilterIndex;
	m_jFileDlg.lpstrFile      = szFileName;
	m_jFileDlg.lpstrFileTitle = szFileTitle;
	m_jFileDlg.lpstrTitle     = szTitle;
	lstrcpy(szFileName, pszFileTitle);
	if(IsValidFile(pszFileName))
	{
		lstrcpy(szInitDir, pszFileName);
		PathRemoveFileSpec(szInitDir);
		m_jFileDlg.lpstrInitialDir = szInitDir;
	}
	else
		m_jFileDlg.lpstrInitialDir = NULL;
	if(opf & OPF_WE_MASK)
	{
		if(m_jFileDlg.nFilterIndex == FTW_GPI)
			m_jFileDlg.nFilterIndex = FTW_OV2;
		else if(m_jFileDlg.nFilterIndex > FTW_GPI)
			m_jFileDlg.nFilterIndex -= FTW_CSVA - FTW_GPI;
		m_jFileDlg.lpstrFilter = m_pszFileFilterW;
		m_jFileDlg.Flags       = OFN_ENABLEHOOK | OFN_ENABLESIZING | OFN_EXPLORER |
			OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
		bSuccess               = m_jFileDlg.GetSaveFileName(opf == OPF_EXPORT);
		if(m_jFileDlg.nFilterIndex >= FTW_GPI)
			m_jFileDlg.nFilterIndex += FTW_CSVA - FTW_GPI;
	}
	else
	{
		m_jFileDlg.lpstrFilter = m_pszFileFilterR;
		m_jFileDlg.Flags       = OFN_ENABLEHOOK | OFN_ENABLESIZING | OFN_EXPLORER |
			OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
		if(m_jFileDlg.nFilterIndex >= FTR_CSVA)
			m_jFileDlg.nFilterIndex = FTR_CSV +
				(m_jFileDlg.nFilterIndex - FTR_CSVA) / (FTR_CSVU8 - FTR_CSVA + 1);
		bSuccess = m_jFileDlg.GetOpenFileName(opf == OPF_IMPORT);
	}
	if(bSuccess)
	{
		if(m_jFileDlg.nFilterIndex != FTR_CPB)
		{
			PTSTR pszFileExt;
			pszFileExt = szFileName + m_jFileDlg.nFileExtension;
			SetFileFilter(pszFileExt, opf & OPF_WE_MASK);
			if((opf & OPF_WE_MASK) && (m_jFileDlg.nFilterIndex == FTW_GPI))
			{
				CopyMemory(pszFileExt, m_szOv2Ext, sizeof(m_szOv2Ext));
				m_jFileDlg.nFilterIndex = FTW_OV2;
			}
		}
		if(OperatePoiFile(opf))
		{
			InsertRecentFile(szFileName);
			if((opf == OPF_READ) || ((opf == OPF_WRITE) && (m_fst != FST_CPB)))
			{
				szFileTitle[m_jFileDlg.nFileExtension - m_jFileDlg.nFileOffset - 1] = CHR_NULL;
				lstrcpy(pszFileName, szFileName);
				lstrcpy(pszFileTitle, szFileTitle);
				nFilterIndex = m_jFileDlg.nFilterIndex;
				m_bSetBmp    = TRUE;
			}
		}
	}
	m_jFileDlg.lpstrFile      = pszFileName;
	m_jFileDlg.lpstrFileTitle = pszFileTitle;
	m_jFileDlg.nFilterIndex   = nFilterIndex;
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Operates the POI file

BOOL CTomPoiEdr::OperatePoiFile(OPERPOIFILE opf)
{
	TRACE(_T("Entering CTomPoiEdr::OperatePoiFile\n"));

	BOOL bSuccess;
	int  cePrevPoi, ceCurPoi;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_opf = opf;
	if((opf == OPF_READ) && m_jCheckDataDlg)
		m_jCheckDataDlg.SendMessage(WM_COMMAND, MAKEWPARAM(ID_CANCEL, BN_CLICKED));
	m_jProgress.CreateWindow(m_ePbStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
	if(opf == OPF_IMPORT)
		cePrevPoi = m_jListView.GetItemCount();
	bSuccess = (opf & OPF_WE_MASK) ? WritePoiFile() : ReadPoiFile();
	if((m_jOptionDlg.m_nBackupFile == BST_CHECKED) &&
		((opf == OPF_READ) || (opf == OPF_WRITE)))
		SetTimer(m_eBackupTimerId, m_jOptionDlg.m_ceBackupIntl * m_eMinuteIntl);
	m_jProgress.DestroyWindow();
	ceCurPoi    = m_jListView.GetItemCount();
	m_bSetReady = TRUE;
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	switch(opf)
	{
	case OPF_READ:
		if(!bSuccess)
		{
			m_bFileDirty = FALSE;
			PostMessage(WM_COMMAND, ID_FILE_NEW);
			return FALSE;
		}

		TCHAR    szText[32];
		int      i;
		UINT     nTelId, nMemoId;
		HDITEM   hi;
		LVCOLUMN lvc;
		lvc.mask = LVCF_FMT;
		hi.mask  = HDI_FORMAT;
		i        = LVC_FIRST;
		do
		{
			if(i == LVC_DIST)
			{
				lvc.fmt = LVCFMT_RIGHT;
				hi.fmt  = HDF_RIGHT | HDF_STRING;
			}
			else
			{
				lvc.fmt = LVCFMT_LEFT;
				hi.fmt  = HDF_LEFT | HDF_STRING;
			}
			m_jListView.SetColumn(i, &lvc);
			m_jLvHeader.SetItem(i, &hi);
			i++;
		}
		while(i < LVC_COUNT);
		lvc.mask    = LVCF_TEXT;
		lvc.pszText = szText;
		hi.mask     = HDI_TEXT;
		hi.pszText  = szText;
		if(m_fst == FST_SC)
		{
			nTelId  = IDS_SPEED_COL;
			nMemoId = IDS_CAMERA_COL;
		}
		else
		{
			nTelId  = IDS_TEL_COL;
			nMemoId = (m_fst == FST_GENERAL) ? IDS_MEMO_COL : IDS_CPBT_COL;
		}
		LoadString(e_hInstance, nTelId, szText, countof(szText));
		m_jListView.SetColumn(LVC_TEL, &lvc);
		m_jLvHeader.SetItem(LVC_TEL, &hi);
		LoadString(e_hInstance, nMemoId, szText, countof(szText));
		m_jListView.SetColumn(LVC_MEMO, &lvc);
		m_jLvHeader.SetItem(LVC_MEMO, &hi);
		if(m_fst == FST_CPB)
		{
			m_jOptionDlg.m_nShowDist = BST_CHECKED;
			SendMessage(WM_COMMAND, ID_SHOW_DISTANCE);
			m_jListView.FreeStorage();
			if(m_jIEPane.IsWindowVisible())
				CloseIEPane();
		}

		if(ceCurPoi)
		{
			LVITEM lvi;
			lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			m_jListView.SetItemState(0, &lvi);
		}
		m_bFileDirty = m_cePoiDupDel;
		if(m_jIEPane.IsWindowVisible())
			PostMessage(WM_COMMAND, ID_POS_GOOGLE_MAP);
		break;
	case OPF_WRITE:
		if(bSuccess)
			m_bFileDirty = FALSE;
		break;
	case OPF_IMPORT:
		EnsureLvItemVisible(m_jListView.GetNextItem(LVNI_SELECTED));
		if(ceCurPoi != cePrevPoi)
			m_bFileDirty = TRUE;
		break;
	case OPF_EXPORT:
		return bSuccess;
	default:
		ASSERT_UNREACHABLE();
	}
	m_bSetCaption = TRUE;
	return bSuccess;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Peeks and processes window updated messages

BOOL CTomPoiEdr::PeekUpdateMessage(BOOL bCheckEsc)
{
	TRACE(_T("Entering CTomPoiEdr::PeekUpdateMessage\n"));

	MSG msg;
	while(::PeekMessage(&msg, WM_PAINT, PM_REMOVE) || PeekMessage(&msg, WM_TIMER, PM_REMOVE))
		DispatchMessage(&msg);
	if(!bCheckEsc)
		return FALSE;
	if(!::PeekMessage(&msg, WM_KEYDOWN, PM_REMOVE) || (msg.wParam != VK_ESCAPE))
		return FALSE;
	while(::PeekMessage(&msg, WM_SYSKEYDOWN, PM_REMOVE));
	while(::PeekMessage(&msg, WM_NCLBUTTONDOWN, PM_REMOVE));
	while(::PeekMessage(&msg, WM_LBUTTONDOWN, PM_REMOVE));
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Prompts to save or cancel the file

BOOL CTomPoiEdr::PromptMessage()
{
	TRACE(_T("Entering CTomPoiEdr::PromptMessage\n"));

	int  ceLvItem, iResult;
	UINT nResId, nType;
	ceLvItem = m_jListView.GetItemCount();
	if(ceLvItem && ((ceLvItem > 1) || !m_bMyBP))
	{
		nResId = IDS_SAVE_PROMPT;
		nType  = MB_ICONQUESTION | MB_YESNOCANCEL;
	}
	else
	{
		nResId = IDS_CANCEL_FILE;
		nType  = MB_ICONQUESTION | MB_OKCANCEL | MB_DEFBUTTON2;
	}
	iResult = DisplayMessage(nResId, nType);
	if(iResult != ID_YES)
		return (iResult != ID_CANCEL);
	SendMessage(WM_COMMAND, ID_FILE_SAVE);
	return !m_bFileDirty;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether the mouse is on the IE pane separator

BOOL CTomPoiEdr::PtInIEPaneSep()
{
	TRACE(_T("Entering CTomPoiEdr::PtInIEPaneSep\n"));

	if(!m_jIEPane.IsWindowVisible())
		return FALSE;

	POINT pt;
	RECT  rc;
	m_jIEPane.GetWindowRect(&rc);
	::MapWindowPoints(*this, &rc);
	rc.right = rc.left;
	rc.left -= m_ecxIEPaneSep;
	GetCursorPos(&pt);
	ScreenToClient(&pt);
	return PtInRect(&rc, pt);
}

BOOL CTomPoiEdr::PtInIEPaneSep(int xCursor, int yCursor)
{
	TRACE(_T("Entering CTomPoiEdr::PtInIEPaneSep\n"));

	if(!m_jIEPane.IsWindowVisible())
		return FALSE;

	POINT pt;
	RECT  rc;
	m_jIEPane.GetWindowRect(&rc);
	::MapWindowPoints(*this, &rc);
	rc.right = rc.left;
	rc.left -= m_ecxIEPaneSep;
	SetPoint(&pt, xCursor, yCursor);
	return PtInRect(&rc, pt);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Puts a command into Google Maps

void CTomPoiEdr::PutGoogleMaps(int iPutCmd, int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::PutGoogleMaps\n"));

	if(!m_jIEPane.IsWindowVisible())
		return;

	if(!m_bGmReady)
		return;

	IDispatch* pDsp;
	m_pHD2->get_Script(&pDsp);
	if(!pDsp)
		return;

	SPOSTR spszPutMethod = _O("PutGoogleMaps");
	DISPID di;
	ASSERT((iPutCmd >= 0) && (iPutCmd < m_eceGmPut));
	if(pDsp->GetIDsOfNames(IID_NULL, &spszPutMethod, 1, LOCALE_USER_DEFAULT, &di) == S_OK)
	{
		// Parameters: iPutCmd, szName, szLng, szLat, szTel, szMemo
		// Note:       Parameters are delivered in reverse order
		int        i;
		VARIANTARG vta[6];
		DISPPARAMS dp;
		dp.rgvarg            = vta;
		dp.rgdispidNamedArgs = NULL;
		dp.cNamedArgs        = 0;
		if((iPutCmd == m_eGmPutSelect) || (iPutCmd == m_eGmPutDelete))
		{
			TCHAR  szText[e_eccNameCol];
			POSTR  pszText;
			int    j, ccText;
			LVITEM lvi;
			dp.cArgs       = countof(vta);
			vta[5].vt      = VT_INT;
			vta[5].intVal  = iPutCmd;
			i              = LVC_NAME;
			j              = 4;
			lvi.pszText    = szText;
			lvi.cchTextMax = countof(szText);
			do
			{
				lvi.iSubItem = i;
				ccText       = GetCoordDdString(i, szText,
					m_jListView.GetItemText(iLvItem, &lvi));
				if(ccText)
				{
#ifdef UNICODE
					pszText = szText;
#else
					WCHAR szwText[countof(szText)];
					MBStoWCS(szText, ccText + 1, szwText, countof(szwText));
					pszText        = szwText;
#endif
					vta[j].vt      = VT_BSTR;
					vta[j].bstrVal = SysAllocString(pszText);
				}
				else
					vta[j].vt = VT_EMPTY;
				i++;
				j--;
			}
			while(i <= LVC_MEMO);
			pDsp->Invoke(di, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dp,
				NULL, NULL, NULL);
			do
			{
				j++;
				if(vta[j].vt == VT_BSTR)
					SysFreeString(vta[j].bstrVal);
			}
			while(j < 4);
		}
		else
		{
			dp.cArgs  = 1;
			vta[0].vt = VT_INT;
			if(iPutCmd == m_eGmPutUpdate)
			{
				VARIANT vtResult;
				vta[0].intVal = m_eceGmPut;
				vtResult.vt   = VT_I4;
				while(TRUE)
				{
					pDsp->Invoke(di, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dp,
						&vtResult, NULL, NULL);
					if(!vtResult.lVal)
						break;
					GetGoogleMaps();
					vta[0].intVal = vtResult.lVal + m_eceGmPut;
				}
			}
			else
			{
				vta[0].intVal = iPutCmd;
				pDsp->Invoke(di, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dp,
					NULL, NULL, NULL);
			}
		}
	}
	pDsp->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads records of cell phonebook

UINT CTomPoiEdr::ReadCpbRecord(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadCpbRecord\n"));

	// Resets the file structure type, if necessary
	if(m_opf == OPF_READ)
		m_fst = FST_CPB;

	BOOL  bTomTom, bDQM;
	PCHAR pcAnalyze, pcStart, pcEnd;
	pcAnalyze = (PCHAR)pvFile;
	pcStart   = pcAnalyze;
	bTomTom   = FALSE;
	bDQM      = FALSE;
	for(DWORD i = 0, j = 0; i <= cbFile; DisplayFileProgress(i, cbFile), i++)
	{
		if((pcAnalyze[i] == CHR_DQM) && (!bTomTom || (pcAnalyze[i + 1] == CHR_CA)))
		{
			if(bDQM)
				bTomTom = FALSE;
			bDQM = !bDQM;
			continue;
		}
		if(!j && (*(PWCHAR)(pcAnalyze + i) == FT_TOMTOM))
			bTomTom = TRUE;
		if(bTomTom)
		{
			*(PWCHAR)(pcStart + j) = *(PWCHAR)(pcAnalyze + i);
			i++;
			j += sizeof(WCHAR);
			continue;
		}
		if(pcAnalyze[i] == CHR_CA)
		{
			pcStart[j++] = bDQM ? CHR_CA : CHR_NULL;
			continue;
		}
		if((pcAnalyze[i] != CHR_CR) && (pcAnalyze[i] != CHR_LF))
		{
			pcStart[j++] = pcAnalyze[i];
			continue;
		}
		if(j)
		{
			pcEnd  = pcStart + j;
			*pcEnd = CHR_NULL;
			j      = 0;
			if(*pcStart)
				ReadCpbRecord(pcStart, pcEnd);
		}
		bTomTom = FALSE;
		bDQM    = FALSE;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads a record of cell phonebook

AFXINLINE void CTomPoiEdr::ReadCpbRecord(PCHAR pcStart, PCHAR pcEnd)
{
	TRACE(_T("Entering CTomPoiEdr::ReadCpbRecord\n"));

	if((pcEnd - pcStart + 1) > e_eccTextRd)
		return;

	TCHAR szText[e_eccTextRd];
	PWSTR pszwText;
	int   i, j;
	if(*(PWCHAR)pcStart == FT_TOMTOM)
	{
#ifdef UNICODE
		pszwText = szText;
#else
		WCHAR szwText[countof(szText)];
		pszwText = szwText;
#endif
		for(i = sizeof(WCHAR), j = 0; pcStart[i] && (j < (countof(szText) - 1));
			i += sizeof(WCHAR), j++)
			pszwText[j] = TomChar2WinChar(*(PWCHAR)(pcStart + i));
		pszwText[j] = CHR_NULL;
#ifndef UNICODE
		WCStoMBS(szwText, j + 1, szText, sizeof(szText));
#endif
	}
	else
	{
		i = lstrlenA(pcStart);
#ifdef UNICODE
		for(j = 0; j <= i; j++)
			szText[j] = (BYTE)pcStart[j];
#else
		CopyMemory(szText, pcStart, i + 1);
#endif
	}
	if(!ValidateNameCol(szText) ||
		((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(szText)))
		return;

	LVITEM lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = szText;
	lvi.iItem    = m_jListView.InsertItem(&lvi);
	pcStart     += i + 1;
	if(pcStart >= pcEnd)
		return;

	i = lstrlenA(pcStart) + 1;
#ifdef UNICODE
	for(j = 0; j < i; j++)
		szText[j] = (BYTE)pcStart[j];
#else
	CopyMemory(szText, pcStart, i);
#endif
	ValidateTelCol(lvi.iItem, szText);
	if(m_bRPF)
		return;

	pcStart += i;
	if(pcStart >= pcEnd)
		i = IDS_CPBT_DEFAULT;
	else
	{
		char szCpbType[32];
		TrimString(pcStart);
		i = IDS_CPBT_FIRST;
		do
		{
			LoadStringA(e_hInstance, i, szCpbType, sizeof(szCpbType));
			if(!lstrcmpA(pcStart, szCpbType))
				break;
			i++;
		}
		while(i <= IDS_CPBT_LAST);
		if(i > IDS_CPBT_LAST)
			i = IDS_CPBT_DEFAULT;
	}
	LoadString(e_hInstance, i - IDS_CPBT_FIRST + IDS_CPBM_FIRST, szText, countof(szText));
	ValidateMemoCol(lvi.iItem, szText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads a csv/asc record of POI file

AFXINLINE void CTomPoiEdr::ReadCsvRecord(PWCHAR pwStart, PWCHAR pwEnd)
{
	TRACE(_T("Entering CTomPoiEdr::ReadCsvRecord\n"));

	int cwRecord;
	cwRecord = pwEnd - pwStart + 1;
	if(cwRecord > e_eccAllCol)
	{
		pwEnd    = pwStart + e_eccAllCol - 1;
		*pwEnd   = CHR_NULL;
		cwRecord = e_eccAllCol;
	}

	PTSTR pszLng, pszLat;
#ifdef UNICODE
	pszLng = pwStart;
#else
	char szText[e_eccAllCol];
	pszLng = szText;
	pwEnd  = (PWCHAR)(szText + WCStoMBS(pwStart, cwRecord, szText, sizeof(szText)) - 1);
#endif
	pszLat = pszLng + lstrlen(pszLng) + 1;
	if(pszLat >= (PTSTR)pwEnd)
		return;

	PTSTR pszName;
	pszName = pszLat + lstrlen(pszLat) + 1;
	if(pszName >= (PTSTR)pwEnd)
		return;

	int ccName;
	ccName = lstrlen(pszName);
	if(!ValidateNameCol(pszName) ||
		((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszName)))
		return;

	PTSTR  pszMemo;
	LVITEM lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = pszName;
	lvi.iItem    = m_jListView.InsertItem(&lvi);
	if((m_jFileDlg.nFilterIndex >= FTR_ASCA) && (m_jFileDlg.nFilterIndex <= FTR_ASCU8))
	{
		TCHAR szCoord[32];
		int   ccCoord;
		ccCoord = TrimString(pszLng);
		if(ccCoord)
		{
			_stprintf(szCoord, e_szCoordFmt, GetCoordDmsValue(pszLng));
			ValidateCoordCol(lvi.iItem, LVC_LNG, szCoord, FALSE);
		}
		ccCoord = TrimString(pszLat);
		if(ccCoord)
		{
			_stprintf(szCoord, e_szCoordFmt, GetCoordDmsValue(pszLat));
			ValidateCoordCol(lvi.iItem, LVC_LAT, szCoord, FALSE);
		}
	}
	else
	{
		ValidateCoordCol(lvi.iItem, LVC_LNG, pszLng, FALSE);
		ValidateCoordCol(lvi.iItem, LVC_LAT, pszLat, FALSE);
	}
	pszMemo = pszName + ccName + 1;
	if(pszMemo >= (PTSTR)pwEnd)
		return;

	PTSTR pszTel;
	pszTel = pszMemo + lstrlen(pszMemo) + 1;
	if(pszTel > (PTSTR)pwEnd)
	{
		PTCHAR pcAnalyze, pcLeftP;
		pcAnalyze = pszMemo;
		pcLeftP   = NULL;
		pszTel    = NULL;
		do
		{
			if(IsValidNumber(*pcAnalyze) || IsValidTelSign(*pcAnalyze))
			{
				if(!pszTel || (*pcAnalyze == CHR_PS))
					pszTel = pcAnalyze;
			}
			else
			{
				if(*pcAnalyze == CHR_LP)
				{
					pcLeftP = pcAnalyze;
					pszTel  = pcAnalyze;
				}
				else
				{
					if((*pcAnalyze != CHR_RP) || !pcLeftP)
					{
						pcLeftP = NULL;
						pszTel  = NULL;
					}
				}
			}
#ifdef UNICODE
			pcAnalyze++;
#else
			pcAnalyze = CharNext(pcAnalyze);
#endif
		}
		while(*pcAnalyze);
	}

	if(pszTel && *pszTel)
	{
		ValidateTelCol(lvi.iItem, pszTel);
		*pszTel = CHR_NULL;
	}

	if(*pszMemo && !m_bRPF)
		ValidateMemoCol(lvi.iItem, pszMemo);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads csv/asc (ANSI) records of POI file

UINT CTomPoiEdr::ReadCsvRecordA(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadCsvRecordA\n"));

	int iDelta;
	iDelta = (m_jFileDlg.nFilterIndex - FTR_CSV) * (FTR_CSVU8 - FTR_CSVA + 1);
	if(*(PWCHAR)pvFile == m_eUBOM)
	{
		m_jFileDlg.nFilterIndex = FTR_CSVU + iDelta;
		ReadCsvRecordU((PWCHAR)pvFile + 1, cbFile / sizeof(WCHAR));
		return 0;
	}

	PCHAR  pcFile;
	PWCHAR pwFile;
	UINT   nCodePage;
	pcFile = (PCHAR)pvFile;
	cbFile++;
	if((*(PWORD)pvFile == FT_UTF8L) && (*((PCHAR)pvFile + 2) == (char)FT_UTF8H))
	{
		nCodePage               = CP_UTF8;
		pcFile                 += FT_UTF8S3;
		cbFile                 -= FT_UTF8S3;
		m_jFileDlg.nFilterIndex = FTR_CSVU8;
	}
	else
	{
		nCodePage               = CP_ACP;
		m_jFileDlg.nFilterIndex = FTR_CSVA;
	}
	m_jFileDlg.nFilterIndex += iDelta;
	pwFile                   = (PWCHAR)malloc(cbFile * sizeof(WCHAR));
	ReadCsvRecordU(pwFile, MBStoWCS(nCodePage, pcFile, cbFile, pwFile, cbFile));
	free(pwFile);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads csv/asc (Unicode) records of POI file

void CTomPoiEdr::ReadCsvRecordU(PWCHAR pwFile, DWORD cwFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadCsvRecordU\n"));

	BOOL   bDQM, bAscExt;
	PWCHAR pwStart, pwEnd, pwSC;
	pwStart = pwFile;
	pwEnd   = pwFile;
	bDQM    = FALSE;
	bAscExt = (m_jFileDlg.nFilterIndex >= FTR_ASCA) && (m_jFileDlg.nFilterIndex <= FTR_ASCU8);
	for(DWORD i = 0, ceComma = 0; i < cwFile; DisplayFileProgress(i, cwFile), i++, pwEnd++)
	{
		*pwEnd = pwFile[i];
		if(pwFile[i] == CHR_CA)
		{
			if(bDQM)
				continue;
			*pwEnd = CHR_NULL;
			ceComma++;
			continue;
		}
		if(bAscExt && (ceComma < 2))
		{
			if((pwFile[i] == CHR_SQM) || (pwFile[i] == CHR_DQM) || (pwFile[i] == CHR_CLN))
				*pwEnd = CHR_SP;
		}
		if(pwFile[i] == CHR_DQM)
		{
			if(pwFile[i + 1] == CHR_DQM)
			{
				if(pwFile[i + 2] == CHR_DQM)
				{
					if(pwFile[i + 3] == CHR_DQM)
					{
						if(pwFile[i + 4] == CHR_DQM)
						{
							i++;
							if(pwFile[i + 4] == CHR_DQM)
								continue;
							bDQM = !bDQM;
						}
						i += 3;
						pwEnd++;
						*pwEnd = CHR_DQM;
						continue;
					}
					i   += 2;
					bDQM = !bDQM;
					continue;
				}
				i++;
				continue;
			}
			pwEnd--;
			bDQM = !bDQM;
			continue;
		}
		if((pwFile[i] != CHR_CR) && (pwFile[i] != CHR_LF))
			continue;
		*pwEnd = CHR_NULL;
		if(pwStart != pwEnd)
		{
			pwSC = pwStart;
			do
			{
				if(*pwSC != CHR_SP)
					break;
				pwSC++;
			}
			while(pwSC < pwEnd);
			if(*pwSC != CHR_SC)
				ReadCsvRecord(pwStart, pwEnd);
		}
		pwEnd   = pwFile + i;
		pwStart = pwEnd + 1;
		bDQM    = FALSE;
		ceComma = 0;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads Favorites records

AFXINLINE UINT CTomPoiEdr::ReadFavRecord(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadFavRecord\n"));

	TCHAR           szName[e_eccNameCol], szCoord[32], szMemo[e_eccMemoCol], szFavMemo[64];
	TCHAR           szHomeMemo[countof(szFavMemo)], szRecentMemo[countof(szFavMemo)];
	PCHAR           pcName, pcEnd, pcEof;
	PTSTR           pszMemo;
	PWSTR           pszwName;
	PFAVORITERECORD pFR;
	int             i, j, iLvItem, cbName;
	LVITEM          lvi;
	pcName         = (PCHAR)pvFile;
	pcEof          = pcName + cbFile;
	cbFile        -= m_ecbMinFavRd;
	pcEnd          = pcName + cbFile;
	lvi.cchTextMax = countof(szCoord);
	if(m_jOpenFavDlg.m_bRecentDest)
	{
		LoadString(e_hInstance, IDS_FAV_POI_MEMO, szFavMemo, countof(szFavMemo));
		LoadString(e_hInstance, IDS_FAV_HOME_MEMO, szHomeMemo, countof(szHomeMemo));
		LoadString(e_hInstance, IDS_FAV_RECENT_MEMO, szRecentMemo, countof(szRecentMemo));
	}
	for(DWORD cbRead = 0; pcName <= pcEnd; pcName++, cbRead++)
	{
		DisplayFileProgress(cbRead, cbFile);
		pFR = (PFAVORITERECORD)pcName;
		if(pFR->bySignal[0] != 0x04)
			continue;
		if(pFR->bySignal[1] != 0x00)
			continue;
		if(!m_jOpenFavDlg.m_bRecentDest && (pFR->bySignal[2] != 0x03))
			continue;
		if(pFR->bySignal[3] != 0x00)
			continue;
		if(pFR->bySignal[4] != 0x00)
			continue;
		if(pFR->bySignal[5] != 0x00)
			continue;
		if((pFR->cbSizeOC != (sizeof(LONG) * 2)) || (pFR->cbSizeRC != (sizeof(LONG) * 2)))
			continue;
		pcName = pFR->szName + *pFR->szName - m_eFavNameDelta + 1;
		if(pcName >= pcEof)
			break;
		pcName += *pcName - m_eFavNameDelta + 1;
		if(pcName >= pcEof)
			break;
		cbName = *pcName - m_eFavNameDelta;
		if((pcName + cbName) >= pcEof)
			break;
		pcName++;
		if(*(PWCHAR)pcName == FT_TOMTOM)
		{
#ifdef UNICODE
			pszwName = szName;
#else
			WCHAR szwName[countof(szName)];
			pszwName = szwName;
#endif
			for(i = sizeof(WCHAR), j = 0; i < cbName; i += sizeof(WCHAR), j++)
				pszwName[j] = TomChar2WinChar(*(PWCHAR)(pcName + i));
			pszwName[j] = CHR_NULL;
#ifndef UNICODE
			WCStoMBS(szwName, j + 1, szName, sizeof(szName));
#endif
		}
		else
		{
#ifdef UNICODE
			for(i = 0; i < cbName; i++)
				szName[i] = (BYTE)pcName[i];
#else
			CopyMemory(szName, pcName, cbName);
#endif
			szName[cbName] = CHR_NULL;
		}
		if(!ValidateNameCol(szName))
			continue;
		if(m_jOpenFavDlg.m_bRecentDest)
		{
			switch(pFR->bySignal[2])
			{
			case 0x03:
				pszMemo = szFavMemo;
				break;
			case 0x04:
				pszMemo = szHomeMemo;
				break;
			default:
				pszMemo = szRecentMemo;
			}
		}
		iLvItem = FindPoiName(szName, MAX_INT);
		if(iLvItem != RV_ERR)
		{
			lvi.iSubItem = LVC_LNG;
			lvi.pszText  = szCoord;
			if(m_jListView.GetItemText(iLvItem, &lvi) &&
				(pFR->lLngOC == GetCoordValueL(szCoord)))
			{
				lvi.iSubItem = LVC_LAT;
				if(m_jListView.GetItemText(iLvItem, &lvi) &&
					(pFR->lLatOC == GetCoordValueL(szCoord)))
				{
					if(m_jOpenFavDlg.m_bRecentDest)
					{
						lvi.iSubItem = LVC_MEMO;
						lvi.pszText  = szMemo;
						if(!m_jListView.GetItemText(iLvItem, &lvi) || lstrcmp(szMemo, pszMemo))
							iLvItem = RV_ERR;
					}
				}
				else
					iLvItem = RV_ERR;
			}
			else
				iLvItem = RV_ERR;
		}
		if(iLvItem == RV_ERR)
		{
			lvi.mask     = LVIF_TEXT;
			lvi.iItem    = MAX_INT;
			lvi.iSubItem = 0;
			lvi.pszText  = szName;
			iLvItem      = m_jListView.InsertItem(&lvi);
			lvi.pszText  = szCoord;
			if(GetCoordString(LVC_LNG, pFR->lLngOC, szCoord) > 0)
			{
				lvi.iSubItem = LVC_LNG;
				m_jListView.CListView::SetItemText(iLvItem, &lvi);
			}
			if(GetCoordString(LVC_LAT, pFR->lLatOC, szCoord) > 0)
			{
				lvi.iSubItem = LVC_LAT;
				m_jListView.CListView::SetItemText(iLvItem, &lvi);
			}
			if(m_jOpenFavDlg.m_bRecentDest)
			{
				lvi.iSubItem = LVC_MEMO;
				lvi.pszText  = pszMemo;
				m_jListView.CListView::SetItemText(iLvItem, &lvi);
			}
		}
		pcName += cbName - 1;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads gpi records of POI file

UINT CTomPoiEdr::ReadGpiRecord(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadGpiRecord\n"));

	if(cbFile < sizeof(GPIHEADER))
		return IDS_STRUCT_ERROR;

	PGPIHEADER pGPIH;
	pGPIH = (PGPIHEADER)pvFile;
	if(!EqualMemory(pGPIH->szGrmRec, "GRMREC00", sizeof(pGPIH->szGrmRec)))
		return IDS_RECORD_ERROR;

	if(pGPIH->cbName != sizeof(pGPIH->szName))
		pGPIH = (PGPIHEADER)((PCHAR)pGPIH + pGPIH->cbName - sizeof(pGPIH->szName));
	if(!EqualMemory(pGPIH->szPOI, "POI", sizeof(pGPIH->szPOI)))
		return IDS_RECORD_ERROR;

	PCHAR      pcSeek, pcEnd, pNextGPIR;
	PSTR       pszDesc[8];
	PTCHAR     pcNumber;
	PTSTR      pszText, pszTel;
	PGPIRECORD pGPIR;
	char       szText[e_eccNameCol];
	TCHAR      szMemo[e_eccMemoCol];
	WCHAR      szwText[sizeof(szText)];
	BOOL       bCrLf, bSkipCrLf, bTelOK, bMemoOK;
	int        i, j, ccText, ccMemo, ccNumber, cbDesc[countof(pszDesc)];
	LVITEM     lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
#ifdef UNICODE
	lvi.pszText  = szwText;
	pszText      = szwText;
#else
	lvi.pszText  = szText;
	pszText      = szText;
#endif
	pcSeek       = pGPIH->szTitle;
	if(*(PWORD)pGPIH->szEN == 'NE')
		pcSeek += pGPIH->cbTitle;
	pcEnd = (PCHAR)pvFile + cbFile - sizeof(GPIRECORD);
	do
	{
		if(*(PDWORD)pcSeek == 0x00080002)
		{
			pGPIR = (PGPIRECORD)pcSeek;
			do
			{
				pNextGPIR = (PCHAR)pGPIR + pGPIR->dwNextRec + 12;
				if(pNextGPIR >= pcEnd)
					return 0;

				DisplayFileProgress(pcSeek - (PCHAR)pvFile, cbFile);
				if((*(PWORD)pGPIR->szEN == 'NE') && *pGPIR->szName && pGPIR->cbName)
				{
					pcSeek = pGPIR->szName + pGPIR->cbName;
					if(pcSeek < pNextGPIR)
					{
						ccText          = MBStoWCS(pGPIH->dwCodePage,
							pGPIR->szName, pGPIR->cbName, szwText, countof(szwText) - 1);
#ifndef UNICODE
						ccText          = WCStoMBS(CP_ACP, szwText, ccText,
							szText, sizeof(szText) - 1);
#endif
						pszText[ccText] = CHR_NULL;
						if(ValidateNameCol(pszText) &&
							((m_jOptionDlg.m_nCheckName == BST_UNCHECKED) ||
							!FindPoiName(pszText)))
						{
							lvi.iItem    = MAX_INT;
							lvi.iSubItem = 0;
							lvi.iItem    = m_jListView.InsertItem(&lvi);
							if(GetCoordString(LVC_LNG, pGPIR->lLng / (2147483648.0 / 180),
								FALSE, pszText) > 0)
							{
								lvi.iSubItem = LVC_LNG;
								m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
							}
							if(GetCoordString(LVC_LAT, pGPIR->lLat / (2147483648.0 / 180),
								FALSE, pszText) > 0)
							{
								lvi.iSubItem = LVC_LAT;
								m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
							}
							while((pcSeek < pNextGPIR) && (*(PWORD)pcSeek != 'NE'))
								pcSeek++;
							if(pcSeek < pNextGPIR)
							{
								pcSeek    += sizeof(pGPIR->szEN) + sizeof(WORD);
								pszDesc[0] = pcSeek;
								bSkipCrLf  = FALSE;
								for(i = 0; pcSeek < pNextGPIR; pcSeek++)
								{
									bCrLf = (*pcSeek == CHR_CR) || (*pcSeek == CHR_LF);
									if(bSkipCrLf)
									{
										if(bCrLf)
											continue;
										pszDesc[i] = pcSeek;
										bSkipCrLf  = FALSE;
										continue;
									}
									if(!bCrLf)
										continue;
									cbDesc[i] = pcSeek - pszDesc[i];
									i++;
									if(i == countof(pszDesc))
										break;
									pszDesc[i] = NULL;
									bSkipCrLf  = TRUE;
								}
								if((pcSeek == pNextGPIR) && pszDesc[i])
								{
									cbDesc[i] = pcSeek - pszDesc[i];
									i++;
								}
								bTelOK  = FALSE;
								bMemoOK = m_bRPF;
								ccMemo  = 0;
								for(j = 0; j < i; j++)
								{
									if(!cbDesc[j])
										break;
									if(cbDesc[j] >= sizeof(szText))
										cbDesc[j] = sizeof(szText) - 1;
									CopyMemory(szText, pszDesc[j], cbDesc[j]);
									szText[cbDesc[j]] = CHR_NULL;
									cbDesc[j]         = TrimString(szText);
									if(!cbDesc[j])
										continue;
									ccText = MBStoWCS(pGPIH->dwCodePage,
										szText, cbDesc[j] + 1, szwText, countof(szwText));
#ifdef UNICODE
									ccText--;
#else
									ccText = WCStoMBS(CP_ACP, szwText, ccText,
										szText, sizeof(szText)) - 1;
#endif
									if((ccMemo + ccText) >= countof(szMemo))
									{
										ccText          = countof(szMemo) - ccMemo - 1;
										pszText[ccText] = CHR_NULL;
									}
									if(bMemoOK)
									{
										if(bTelOK)
											break;
									}
									else
									{
										if(ccMemo)
										{
											szMemo[ccMemo] = CHR_AMP;
											ccMemo++;
											if((ccMemo + ccText) >= countof(szMemo))
											{
												ccText          = countof(szMemo) - ccMemo - 1;
												pszText[ccText] = CHR_NULL;
											}
										}
										CopyMemory(szMemo + ccMemo,
											pszText, (ccText + 1) * sizeof(TCHAR));
										ccMemo += ccText;
										if(ccMemo == (countof(szMemo) - 1))
										{
											if(bTelOK)
												break;
											bMemoOK = TRUE;
										}
										else if(bTelOK)
											continue;
									}
									pszTel   = NULL;
									pcNumber = pszText;
									do
									{
										if(IsValidNumber(*pcNumber))
										{
											if(pszTel)
											{
												ccNumber++;
												if(ccNumber >= 7)
													break;
											}
											else
											{
												pszTel   = pcNumber;
												ccNumber = 1;
											}
										}
										else if(!IsValidTelSign(*pcNumber) &&
											(*pcNumber != CHR_RP))
											pszTel = NULL;
										pcNumber++;
									}
									while(*pcNumber);
									if(!pszTel || (ccNumber < 7))
										continue;
									ValidateTelCol(lvi.iItem, pszTel);
									if(bMemoOK)
										break;
									bTelOK = TRUE;
								}
								if(ccMemo)
									ValidateMemoCol(lvi.iItem, szMemo);
							}
						}
					}
				}
				pGPIR = (PGPIRECORD)pNextGPIR;
			}
			while(pGPIR->dwSignal == 0x00080002);
			pcSeek = pNextGPIR;
		}
		pcSeek++;
	}
	while(pcSeek < pcEnd);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads gpx records of POI file

UINT CTomPoiEdr::ReadGpxRecord(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadGpxRecord\n"));

	char   szWpt[16], szWptEnd[sizeof(szWpt)];
	char   szLng[sizeof(szWpt)], szLat[sizeof(szWpt)];
	char   szName[sizeof(szWpt)], szNameEnd[sizeof(szWpt)];
	char   szCmt[sizeof(szWpt)], szCmtEnd[sizeof(szWpt)];
	char   szDesc[sizeof(szWpt)], szDescEnd[sizeof(szWpt)];
	WCHAR  szwText[e_eccNameCol];
	PCCHAR pcEnd;
	PSTR   pszToken, pszWptEnd, pszTemp, pszName, pszLng, pszLat, pszTel, pszMemo;
	PTSTR  pszText;
	BOOL   bLng, bLat;
	int    ccName, ccTel, ccMemo;
	int    cbWpt, cbWptEnd, cbLng, cbLat, cbName, cbNameEnd, cbCmt, cbCmtEnd, cbDesc, cbDescEnd;
	LVITEM lvi;
#ifdef UNICODE
	int    i;
#else
	char   szText[countof(szwText)];
#endif
	pszToken  = (PSTR)pvFile;
	pcEnd     = pszToken + cbFile;
	cbWpt     = LoadStringA(e_hInstance, IDS_GPXT_WPT, szWpt, sizeof(szWpt));
	cbWptEnd  = LoadStringA(e_hInstance, IDS_GPXT_WPT_END, szWptEnd, sizeof(szWptEnd));
	cbLng     = LoadStringA(e_hInstance, IDS_GPXT_LNG, szLng, sizeof(szLng));
	cbLat     = LoadStringA(e_hInstance, IDS_GPXT_LAT, szLat, sizeof(szLat));
	cbName    = LoadStringA(e_hInstance, IDS_GPXT_NAME, szName, sizeof(szName));
	cbNameEnd = LoadStringA(e_hInstance, IDS_GPXT_NAME_END, szNameEnd, sizeof(szNameEnd));
	cbCmt     = LoadStringA(e_hInstance, IDS_GPXT_CMT, szCmt, sizeof(szCmt));
	cbCmtEnd  = LoadStringA(e_hInstance, IDS_GPXT_CMT_END, szCmtEnd, sizeof(szCmtEnd));
	cbDesc    = LoadStringA(e_hInstance, IDS_GPXT_DESC, szDesc, sizeof(szDesc));
	cbDescEnd = LoadStringA(e_hInstance, IDS_GPXT_DESC_END, szDescEnd, sizeof(szDescEnd));
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iSubItem = 0;
#ifdef UNICODE
	lvi.pszText  = szwText;
	pszText      = szwText;
#else
	lvi.pszText  = szText;
#endif
	while(pszToken = StrStrIA(pszToken, szWpt))
	{
		pszToken += cbWpt;
		if(*pszToken != CHR_SP)
			continue;

		pszToken++;
		pszLng = NULL;
		pszLat = NULL;
		do
		{
			while(*pszToken == CHR_SP)
			{
				pszToken++;
				if(pszToken >= pcEnd)
					return IDS_STRUCT_ERROR;
			}
			bLng = !StrCmpNIA(pszToken, szLng, cbLng);
			bLat = !StrCmpNIA(pszToken, szLat, cbLat);
			if(bLng)
				pszToken += cbLng;
			else if(bLat)
				pszToken += cbLat;
			else
				return IDS_STRUCT_ERROR;

			while(*pszToken != '=')
			{
				if(*pszToken != CHR_SP)
					return IDS_STRUCT_ERROR;
				pszToken++;
				if(pszToken >= pcEnd)
					return IDS_STRUCT_ERROR;
			}
			pszToken++;
			while(*pszToken != CHR_DQM)
			{
				if(*pszToken != CHR_SP)
					return IDS_STRUCT_ERROR;
				pszToken++;
				if(pszToken >= pcEnd)
					return IDS_STRUCT_ERROR;
			}
			pszToken++;
			(bLng ? pszLng : pszLat) = pszToken;
			while(*pszToken != CHR_DQM)
			{
				pszToken++;
				if(pszToken >= pcEnd)
					return IDS_STRUCT_ERROR;
			}
			*pszToken = CHR_NULL;
			pszToken++;
		}
		while(*pszToken != CHR_GT);
		if(!pszLng || !pszLat)
			return IDS_STRUCT_ERROR;

		pszToken = StrStrIA(pszToken + 1, szName);
		if(!pszToken)
			return IDS_STRUCT_ERROR;

		pszToken += cbName;
		pszName   = pszToken;
		do
		{
			if(*pszToken < 0)
			{
				pszToken += (*pszToken >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
				continue;
			}
			if(!StrCmpNIA(pszToken, szNameEnd, cbNameEnd))
				break;
			pszToken++;
		}
		while(pszToken < pcEnd);
		if(pszToken >= pcEnd)
			return IDS_STRUCT_ERROR;

		ccName    = pszToken - pszName;
		pszToken += cbNameEnd;
		pszWptEnd = StrStrIA(pszToken, szWptEnd);
		if(!pszWptEnd)
			return IDS_STRUCT_ERROR;

		pszTemp = pszToken;
		do
		{
			if(!StrCmpNIA(pszTemp, szCmt, cbCmt))
				break;
			pszTemp++;
		}
		while(pszTemp < pszWptEnd);
		if(pszTemp < pszWptEnd)
		{
			pszTemp += cbCmt;
			pszTel   = pszTemp;
			do
			{
				if(*pszTemp < 0)
				{
					pszTemp += (*pszTemp >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
					continue;
				}
				if(!StrCmpNIA(pszTemp, szCmtEnd, cbCmtEnd))
					break;
				pszTemp++;
			}
			while(pszTemp < pszWptEnd);
			if(pszTemp >= pszWptEnd)
				return IDS_STRUCT_ERROR;

			ccTel = pszTemp - pszTel;
		}
		else
			pszTel = NULL;
		do
		{
			if(!StrCmpNIA(pszToken, szDesc, cbDesc))
				break;
			pszToken++;
		}
		while(pszToken < pszWptEnd);
		if(pszToken < pszWptEnd)
		{
			pszToken += cbDesc;
			pszMemo   = pszToken;
			do
			{
				if(*pszToken < 0)
				{
					pszToken += (*pszToken >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
					continue;
				}
				if(!StrCmpNIA(pszToken, szDescEnd, cbDescEnd))
					break;
				pszToken++;
			}
			while(pszToken < pszWptEnd);
			if(pszToken >= pszWptEnd)
				return IDS_STRUCT_ERROR;

			ccMemo = pszToken - pszMemo;
		}
		else
			pszMemo = NULL;

		pszToken = pszWptEnd + cbWptEnd;
		ccName   = MBStoWCS(CP_UTF8, pszName, ccName, szwText, countof(szwText) - 1);
		if(!ccName)
			ccName = countof(szwText) - 1;
#ifndef UNICODE
		ccName          = WCStoMBS(szwText, ccName, szText, sizeof(szText) - 1);
		pszText         = szText;
#endif
		pszText[ccName] = CHR_NULL;
		DecodeEntity(pszText);
		if(!ValidateNameCol(pszText) ||
			((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszText)))
			continue;

		lvi.iItem = MAX_INT;
		lvi.iItem = m_jListView.InsertItem(&lvi);
#ifdef UNICODE
		for(i = 0; pszLng[i]; i++)
			szwText[i] = (BYTE)pszLng[i];
		szwText[i] = CHR_NULL;
#else
		pszText    = pszLng;
#endif
		ValidateCoordCol(lvi.iItem, LVC_LNG, pszText, FALSE);
#ifdef UNICODE
		for(i = 0; pszLat[i]; i++)
			szwText[i] = (BYTE)pszLat[i];
		szwText[i] = CHR_NULL;
#else
		pszText    = pszLat;
#endif
		ValidateCoordCol(lvi.iItem, LVC_LAT, pszText, FALSE);
		if(pszTel)
		{
			if(ccTel >= e_eccTelCol)
				ccTel = e_eccTelCol - 1;
#ifdef UNICODE
			for(i = 0; i < ccTel; i++)
				szwText[i] = (BYTE)pszTel[i];
#else
			pszText        = pszTel;
#endif
			pszText[ccTel] = CHR_NULL;
			ValidateTelCol(lvi.iItem, pszText);
		}
		if(!m_bRPF && pszMemo)
		{
			ccMemo = MBStoWCS(CP_UTF8, pszMemo, ccMemo, szwText, countof(szwText) - 1);
			if(!ccMemo)
				ccMemo = countof(szwText) - 1;
#ifndef UNICODE
			ccMemo          = WCStoMBS(szwText, ccMemo, szText, sizeof(szText) - 1);
			pszText         = szText;
#endif
			pszText[ccMemo] = CHR_NULL;
			DecodeEntity(pszText);
			ValidateMemoCol(lvi.iItem, pszText);
		}
		DisplayFileProgress(pszToken - (PCCHAR)pvFile, cbFile);
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads kml records of POI file

UINT CTomPoiEdr::ReadKmlRecord(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadKmlRecord\n"));

	char   szPm[32], szPmEnd[sizeof(szPm)];
	char   szName[sizeof(szPm)], szNameEnd[sizeof(szPm)];
	char   szDesc[sizeof(szPm)], szDescEnd[sizeof(szPm)];
	char   szCData[sizeof(szPm)], szCDataEnd[sizeof(szPm)];
	TCHAR  szBr[sizeof(szPm)];
	char   szPoint[sizeof(szPm)], szPointEnd[sizeof(szPm)];
	char   szCoord[sizeof(szPm)], szCoordEnd[sizeof(szPm)];
	WCHAR  szwText[e_eccNameCol], szwTelMemo[e_eccTelCol + e_eccMemoCol];
	PCCHAR pcEnd;
	PSTR   pszToken, pszPmEnd, pszTemp, pszName, pszLng, pszLat, pszDesc;
	PTCHAR pcTemp;
	PTSTR  pszText, pszMemo;
	BOOL   bCData;
	int    i, j, ccName, ccDesc;
	int    cbPm, cbPmEnd, cbName, cbNameEnd, cbDesc, cbDescEnd, cbCData, cbCDataEnd, ccBr;
	int    cbPoint, cbCoord;
	LVITEM lvi;
#ifndef UNICODE
	char   szText[countof(szwText)], szTelMemo[countof(szwTelMemo)];
#endif
	pszToken   = (PSTR)pvFile;
	pcEnd      = pszToken + cbFile;
	cbPm       = LoadStringA(e_hInstance, IDS_KMLT_PLACEMARK, szPm, sizeof(szPm));
	cbPmEnd    = LoadStringA(e_hInstance, IDS_KMLT_PLACEMARK_END, szPmEnd, sizeof(szPmEnd));
	cbName     = LoadStringA(e_hInstance, IDS_KMLT_NAME, szName, sizeof(szName));
	cbNameEnd  = LoadStringA(e_hInstance, IDS_KMLT_NAME_END, szNameEnd, sizeof(szNameEnd));
	cbDesc     = LoadStringA(e_hInstance, IDS_KMLT_DESC, szDesc, sizeof(szDesc));
	cbDescEnd  = LoadStringA(e_hInstance, IDS_KMLT_DESC_END, szDescEnd, sizeof(szDescEnd));
	cbCData    = LoadStringA(e_hInstance, IDS_KMLT_CDATA, szCData, sizeof(szCData));
	cbCDataEnd = LoadStringA(e_hInstance, IDS_KMLT_CDATA_END, szCDataEnd, sizeof(szCDataEnd));
	ccBr       = LoadString(e_hInstance, IDS_KMLT_BR, szBr, countof(szBr));
	cbPoint    = LoadStringA(e_hInstance, IDS_KMLT_POINT, szPoint, sizeof(szPoint));
	LoadStringA(e_hInstance, IDS_KMLT_POINT_END, szPointEnd, sizeof(szPointEnd));
	cbCoord = LoadStringA(e_hInstance, IDS_KMLT_COORD, szCoord, sizeof(szCoord));
	LoadStringA(e_hInstance, IDS_KMLT_COORD_END, szCoordEnd, sizeof(szCoordEnd));
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iSubItem = 0;
#ifdef UNICODE
	lvi.pszText  = szwText;
#else
	lvi.pszText  = szText;
#endif
	while(pszToken = StrStrIA(pszToken, szPm))
	{
		pszToken += cbPm;
		if(*pszToken != CHR_GT)
		{
			pszToken = StrChrA(pszToken, CHR_GT);
			if(!pszToken)
				return IDS_STRUCT_ERROR;
		}
		pszToken = StrStrIA(pszToken + 1, szName);
		if(!pszToken)
			return IDS_STRUCT_ERROR;

		pszToken += cbName;
		pszName   = pszToken;
		do
		{
			if(*pszToken < 0)
			{
				pszToken += (*pszToken >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
				continue;
			}
			if(!StrCmpNIA(pszToken, szNameEnd, cbNameEnd))
				break;
			pszToken++;
		}
		while(pszToken < pcEnd);
		if(pszToken >= pcEnd)
			return IDS_STRUCT_ERROR;

		ccName    = pszToken - pszName;
		pszToken += cbNameEnd;
		pszPmEnd  = StrStrIA(pszToken, szPmEnd);
		if(!pszPmEnd)
			return IDS_STRUCT_ERROR;

		pszTemp = pszToken;
		do
		{
			if(!StrCmpNIA(pszTemp, szDesc, cbDesc))
				break;
			pszTemp++;
		}
		while(pszTemp < pszPmEnd);
		if(pszTemp < pszPmEnd)
		{
			pszTemp += cbDesc;
			pszDesc  = pszTemp;
			bCData   = FALSE;
			do
			{
				if(*pszTemp < 0)
				{
					pszTemp += (*pszTemp >= (char)FT_UTF8S3FC) ? FT_UTF8S3 : FT_UTF8S2;
					continue;
				}
				if(bCData)
				{
					if(!StrCmpNIA(pszTemp, szCDataEnd, cbCDataEnd))
						break;
				}
				else
				{
					if(!StrCmpNIA(pszTemp, szCData, cbCData))
					{
						bCData   = TRUE;
						pszTemp += cbCData;
						pszDesc  = pszTemp;
						continue;
					}
					if(!StrCmpNIA(pszTemp, szDescEnd, cbDescEnd))
						break;
				}
				pszTemp++;
			}
			while(pszTemp < pszPmEnd);
			if(pszTemp >= pszPmEnd)
				return IDS_STRUCT_ERROR;

			ccDesc = pszTemp - pszDesc;
		}
		else
			pszDesc = NULL;

		pszToken = StrStrIA(pszToken, szPoint);
		if(!pszToken || (pszToken >= pszPmEnd))
		{
			pszToken = pszPmEnd + cbPmEnd;
			continue;
		}

		pszToken += cbPoint;
		if(*pszToken != CHR_GT)
		{
			pszToken = StrChrA(pszToken, CHR_GT);
			if(!pszToken || (pszToken >= pszPmEnd))
				return IDS_STRUCT_ERROR;
		}
		pszToken = StrStrIA(pszToken + 1, szCoord);
		if(!pszToken || (pszToken >= pszPmEnd))
			return IDS_STRUCT_ERROR;

		pszToken += cbCoord;
		pszLng    = pszToken;
		pszToken  = StrStrIA(pszToken, szCoordEnd);
		if(!pszToken || (pszToken >= pszPmEnd))
			return IDS_STRUCT_ERROR;

		*pszToken = CHR_NULL;
		pszToken  = pszPmEnd + cbPmEnd;
		ccName    = MBStoWCS(CP_UTF8, pszName, ccName, szwText, countof(szwText) - 1);
		if(!ccName)
			ccName = countof(szwText) - 1;
#ifdef UNICODE
		pszText         = szwText;
#else
		ccName          = WCStoMBS(szwText, ccName, szText, sizeof(szText) - 1);
		pszText         = szText;
#endif
		pszText[ccName] = CHR_NULL;
		DecodeEntity(pszText);
		if(!ValidateNameCol(pszText) ||
			((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszText)))
			continue;

		lvi.iItem = MAX_INT;
		lvi.iItem = m_jListView.InsertItem(&lvi);
		pszLat    = StrChrA(pszLng, CHR_CA);
		if(pszLat)
		{
			*pszLat = CHR_NULL;
			pszLat++;
			pszTemp = StrChrA(pszLat, CHR_CA);
			if(pszTemp)
				*pszTemp = CHR_NULL;
		}
		if(pszLng && *pszLng)
		{
#ifdef UNICODE
			for(i = 0; pszLng[i]; i++)
				szwText[i] = (BYTE)pszLng[i];
			szwText[i] = CHR_NULL;
#else
			pszText    = pszLng;
#endif
			ValidateCoordCol(lvi.iItem, LVC_LNG, pszText, FALSE);
		}
		if(pszLat && *pszLat)
		{
#ifdef UNICODE
			for(i = 0; pszLat[i]; i++)
				szwText[i] = (BYTE)pszLat[i];
			szwText[i] = CHR_NULL;
#else
			pszText    = pszLat;
#endif
			ValidateCoordCol(lvi.iItem, LVC_LAT, pszText, FALSE);
		}
		if(pszDesc)
		{
			ccDesc = MBStoWCS(CP_UTF8, pszDesc, ccDesc, szwTelMemo, countof(szwTelMemo) - 1);
			if(!ccDesc)
				ccDesc = countof(szwTelMemo) - 1;
#ifdef UNICODE
			pszText         = szwTelMemo;
#else
			ccDesc          = WCStoMBS(szwTelMemo, ccDesc, szTelMemo, sizeof(szTelMemo) - 1);
			pszText         = szTelMemo;
#endif
			pszText[ccDesc] = CHR_NULL;
			pszMemo         = pszText;
			while(pszMemo = StrStrI(pszMemo, szBr))
			{
				pcTemp = pszMemo + ccBr;
				while(*pcTemp == CHR_SP)
					pcTemp++;
				if(*pcTemp == CHR_SLH)
					pcTemp++;
				if(*pcTemp == CHR_GT)
				{
					for(i = 0, j = pcTemp - pszMemo; i < j; i++)
						pszMemo[i] = CHR_SP;
					pszMemo[i] = CHR_LF;
				}
				pszMemo = pcTemp + 1;
			}
			pszMemo = StrChr(pszText, CHR_LF);
			if(pszMemo)
			{
				*pszMemo = CHR_NULL;
				pszMemo++;
				if(!*pszText)
				{
					pcTemp = StrChr(pszMemo, CHR_LF);
					if(pcTemp)
					{
						pszText  = pszMemo;
						pszMemo  = pcTemp;
						*pszMemo = CHR_NULL;
						pszMemo++;
					}
				}
				if(!m_bRPF)
				{
					DecodeEntity(pszMemo);
					ValidateMemoCol(lvi.iItem, pszMemo);
				}
			}
			ValidateTelCol(lvi.iItem, pszText);
		}
		DisplayFileProgress(pszToken - (PCCHAR)pvFile, cbFile);
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads ov2 records of POI file

UINT CTomPoiEdr::ReadOv2Record(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record\n"));

	UINT         nType;
	DWORD        cbRemain, cbRecord;
	PPOIRECORD1  pPR1;
	PPOIRECORD2  pPR2;
	PPOIRECORD17 pPR17;
	PPOIRECORD1D pPR1D;
	m_lRangeLngW = MIN_LONG;
	m_lRangeLngE = MAX_LONG;
	pPR2         = (PPOIRECORD2)pvFile;
	cbRemain     = cbFile;
	while(cbRemain)
	{
		nType = pPR2->byType;
		switch(nType)
		{
		case FRT_SIMPLE:
		case FRT_EXSIMPLE:
		case FRT_VARSIMPLE:
		case FRT_ENSIMPLE:
		case FRT_RPF:
			if(m_fst == FST_SC)
				return IDS_FILE_COMPATIBLE_ERROR;
		}

		if(cbRemain < sizeof(POIRECORD0))
			return IDS_STRUCT_ERROR;

		switch(nType)
		{
		case FRT_ENSIMPLE:
			pPR17    = (PPOIRECORD17)pPR2;
			cbRecord = pPR17->cbSize + sizeof(POIRECORD17) - 1;
			break;
		case FRT_SC:
			cbRecord = sizeof(POIRECORD1E);
			break;
		case FRT_RPF:
			pPR1D    = (PPOIRECORD1D)pPR2;
			cbRecord = pPR1D->cbSize + m_eRpfRdDelta;
			break;
		default:
			cbRecord = pPR2->cbSize;
		}
		if(cbRemain < cbRecord)
			return IDS_STRUCT_ERROR;

		switch(nType)
		{
		case FRT_DELETED:
			break;
		case FRT_SKIPPER:
			if(cbRecord < sizeof(POIRECORD1))
				return IDS_STRUCT_ERROR;
			cbRecord = sizeof(POIRECORD1);
			pPR1     = (PPOIRECORD1)pPR2;
			if(pPR1->lLngW > m_lRangeLngW)
				m_lRangeLngW = pPR1->lLngW;
			if(pPR1->lLngE < m_lRangeLngE)
				m_lRangeLngE = pPR1->lLngE;
			break;
		case FRT_SIMPLE:
		case FRT_VARSIMPLE:
			if(cbRecord <= sizeof(POIRECORD2))
				return IDS_STRUCT_ERROR;
			ReadOv2Record2(pPR2, nType == FRT_VARSIMPLE);
			break;
		case FRT_EXSIMPLE:
			if(cbRecord <= sizeof(POIRECORD3))
				return IDS_STRUCT_ERROR;
			ReadOv2Record3((PPOIRECORD3)pPR2);
			break;
		case FRT_ENSIMPLE:
			if(cbRecord <= sizeof(PPOIRECORD17))
				return IDS_STRUCT_ERROR;
			ReadOv2Record17(pPR17);
			break;
		case FRT_RPF:
			if(cbRecord <= sizeof(POIRECORD1D))
				return IDS_STRUCT_ERROR;
			if(m_opf == OPF_READ)
				m_bRPF = TRUE;
			ReadOv2Record1D(pPR1D);
			break;
		case FRT_SC:
			if(cbRecord != sizeof(POIRECORD1E))
				return IDS_STRUCT_ERROR;
			if(m_opf == OPF_READ)
				m_fst = FST_SC;
			else if(m_fst != FST_SC)
				return IDS_FILE_COMPATIBLE_ERROR;
			ReadOv2Record1E((PPOIRECORD1E)pPR2);
			break;
		case FRT_SIGNAL:
			if(cbRecord != sizeof(POIRECORD64))
				return IDS_STRUCT_ERROR;
			if(m_opf == OPF_READ)
				CopyMemory(&m_pr64, pPR2, sizeof(POIRECORD64));
			break;
		default:
			return IDS_RECORD_ERROR;
		}

		cbRemain -= cbRecord;
		pPR2      = (PPOIRECORD2)((PBYTE)pPR2 + cbRecord);
		DisplayFileProgress(cbFile - cbRemain, cbFile);
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads an ov2 record (type 2) of POI file

void CTomPoiEdr::ReadOv2Record2(PPOIRECORD2 pPR2, BOOL bVariety)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record2\n"));

	int i, cbName;
	cbName = pPR2->cbSize - sizeof(POIRECORD2);
	if(bVariety)
	{
		i = cbName;
		do
		{
			i--;
			pPR2->szName[i] ^= m_eVarDelta;
		}
		while(i);
	}

	TCHAR szText[e_eccNameCol + e_eccTelCol];
	PTSTR pszText;
	int   j;
	if(*(PWCHAR)pPR2->szName == FT_TOMTOM)
	{
		WCHAR szwText[countof(szText)];
		i = 0;
		j = sizeof(WCHAR);
		while(pPR2->szName[j])
		{
			szwText[i] = TomChar2WinChar(*(PWCHAR)(pPR2->szName + j));
			i++;
			if(i == (countof(szwText) - 1))
				break;
			j += sizeof(WCHAR);
		}
		szwText[i] = CHR_NULL;
#ifdef UNICODE
		pszText    = szwText;
#else
		WCStoMBS(szwText, i + 1, szText, sizeof(szText));
		pszText = szText;
#endif
		i       = sizeof(WCHAR);
	}
	else
	{
#ifdef UNICODE
		j = 0;
		while(pPR2->szName[j])
		{
			szText[j] = (BYTE)pPR2->szName[j];
			j++;
			if(j == (countof(szText) - 1))
				break;
		}
		szText[j] = CHR_NULL;
		pszText   = szText;
#else
		if(lstrlen(pPR2->szName) > (sizeof(szText) - 1))
			pPR2->szName[sizeof(szText) - 1] = CHR_NULL;
		pszText = pPR2->szName;
#endif
		i       = sizeof(char);
	}

	PTSTR pszTel;
	pszTel = StrChr(pszText, FT_POI_SEP);
	if(pszTel)
		*pszTel = CHR_NULL;
	if(!ValidateNameCol(pszText) ||
		((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszText)))
		return;

	LVITEM lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = pszText;
	lvi.iItem    = m_jListView.InsertItem(&lvi);

	TCHAR szCoord[32];
	lvi.pszText = szCoord;
	if(GetCoordString(LVC_LNG, pPR2->lLng, szCoord) > 0)
	{
		lvi.iSubItem = LVC_LNG;
		m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
	}
	if(GetCoordString(LVC_LAT, pPR2->lLat, szCoord) > 0)
	{
		lvi.iSubItem = LVC_LAT;
		m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
	}

	if(pszTel)
		ValidateTelCol(lvi.iItem, pszTel + 1);

	if(m_bRPF)
		return;

	PWSTR pszMemo, pcNextPR2;
	while(pPR2->szName[j] && (j < cbName))
		j += i;
	pszMemo   = (PWSTR)(pPR2->szName + j + 1);
	pcNextPR2 = (PWSTR)((PCHAR)pPR2 + pPR2->cbSize);
	if(pszMemo >= pcNextPR2)
		return;

	i       = min(pcNextPR2 - pszMemo, e_eccMemoCol);
#ifdef UNICODE
	pszText = pszMemo;
	if(pszMemo[i - 1])
	{
		if(i == e_eccMemoCol)
			pszMemo[e_eccMemoCol - 1] = CHR_NULL;
		else
		{
			CopyMemory(szText, pszMemo, i * sizeof(WCHAR));
			szText[i] = CHR_NULL;
			pszText   = szText;
		}
	}
#else
	i       = WCStoMBS(pszMemo, i, szText, e_eccMemoCol);
	pszText = AnsiPrev(szText, szText + i);
	if(*pszText)
	{
		if(i == e_eccMemoCol)
			*pszText = CHR_NULL;
		else
			szText[i] = CHR_NULL;
	}
	pszText = szText;
#endif
	ValidateMemoCol(lvi.iItem, pszText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads an ov2 record (type 3) of POI file

AFXINLINE void CTomPoiEdr::ReadOv2Record3(PPOIRECORD3 pPR3)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record3\n"));

	int i, j;
	if(*(PWCHAR)pPR3->szName == FT_TOMTOM)
	{
		i = sizeof(WCHAR);
		j = sizeof(WCHAR);
	}
	else
	{
		i = 0;
		j = 1;
	}
	while(pPR3->szName[i])
		i += j;
	pPR3->cbSize = i + sizeof(POIRECORD2);
	ReadOv2Record2((PPOIRECORD2)pPR3, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads an ov2 record (type 17) of POI file

AFXINLINE void CTomPoiEdr::ReadOv2Record17(PPOIRECORD17 pPR17)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record17\n"));

	char        cPR17[MAX_BYTE + sizeof(POIRECORD17)];
	int         cbName;
	LONG        lLng;
	PPOIRECORD2 pPR2;
	pPR2   = (PPOIRECORD2)cPR17;
	cbName = pPR17->cbSize;
	lLng   = MAKELONG(*(PWORD)pPR17->byLng, pPR17->byLng[2]);
	if(m_lRangeLngW != MIN_LONG)
	{
		while((lLng < m_lRangeLngE) || (lLng > m_lRangeLngW))
		{
			lLng -= m_eEnCoordSubDelta;
			if(lLng < m_eEnCoordAdjDelta)
				lLng += m_eEnCoordRollDelta;
		}
	}
	pPR2->byType = FRT_SIMPLE;
	pPR2->cbSize = cbName + sizeof(POIRECORD2);
	pPR2->lLng   = lLng;
	pPR2->lLat   = MAKELONG(*(PWORD)pPR17->byLat, pPR17->byLat[2]) - m_eEnCoordSubDelta;
	CopyMemory(pPR2->szName, pPR17->szName, cbName);
	pPR2->szName[cbName] = CHR_NULL;
	ReadOv2Record2(pPR2, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads an ov2 record (type 1D) of POI file

AFXINLINE void CTomPoiEdr::ReadOv2Record1D(PPOIRECORD1D pPR1D)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record1D\n"));

	WCHAR szwText[e_eccNameCol + e_eccTelCol];
	PTSTR pszText;
	int   i, j, cbName;
	cbName = pPR1D->cbSize - m_eRpfNameDelta;
	if((pPR1D->byNET == NET_TOMTOM_5) || (pPR1D->byNET == NET_TOMTOM_6) ||
		(pPR1D->byNET == NET_TOMTOM_7))
	{
		PWSTR pszwName;
		int   cwName;
		cwName   = cbName / sizeof(WCHAR);
		pszwName = (PWSTR)pPR1D->szName;
		if(pPR1D->byNET == NET_TOMTOM_5)
		{
			cwName++;
			pszwName--;
		}
		else if(pPR1D->byNET == NET_TOMTOM_7)
			pszwName = (PWSTR)((PCHAR)pszwName + 1);
		for(i = 0; i < cwName; i++)
			szwText[i] = TomChar2WinChar(pszwName[i]);
	}
	else if(pPR1D->byNET == NET_UNICODE)
	{
		for(i = 0, j = sizeof(WCHAR); j < cbName; i++, j++)
		{
			if((BYTE)pPR1D->szName[j] == FT_RPF_UCS)
			{
				j += sizeof(WCHAR);
				if(j >= cbName)
					break;
				szwText[i] = *(PWCHAR)(pPR1D->szName + j - 1);
				continue;
			}
			if((BYTE)pPR1D->szName[j] < FT_RPF_UEM)
			{
				szwText[i] = (BYTE)pPR1D->szName[j] + CHR_SP;
				continue;
			}
			if((BYTE)pPR1D->szName[j] == FT_RPF_UEM)
			{
				szwText[i] = *(PWCHAR)pPR1D->szName;
				continue;
			}
			szwText[i] = *(PWCHAR)pPR1D->szName + (BYTE)pPR1D->szName[j] - FT_RPF_UEM;
		}
	}
	else
	{
		ASSERT((pPR1D->byNET == NET_ANSI) || (pPR1D->byNET == NET_EXANSI));
		if((pPR1D->byNET != NET_ANSI) && (pPR1D->byNET != NET_EXANSI))
			return;

		char szCode[m_ecbMaxRpfName * 8], cName;
		int  k, cbCode;
		for(i = 0, j = 0, k; i < cbName; i++)
		{
			cName = pPR1D->szName[i];
			k     = 8;
			do
			{
				szCode[j] = (cName & 0x01) ? '1' : '0';
				cName   >>= 1;
				j++;
				k--;
			}
			while(k);
		}

		cbCode = j;
		for(i = 0, j = 0; j < cbCode; i++)
		{
			k = 0;
			do
			{
				if(EqualMemory(szCode + j, m_pr9dt[k].pszCode, m_pr9dt[k].cbCode))
					break;
				k++;
			}
			while(k < countof(m_pr9dt));
			if(!k || (k == countof(m_pr9dt)))
				break;

			szwText[i] = m_pr9dt[k].wChar;
			j         += m_pr9dt[k].cbCode;
		}
	}
	szwText[i] = CHR_NULL;
#ifdef UNICODE
	pszText = szwText;
#else
	char szText[countof(szwText)];
	WCStoMBS(szwText, i + 1, szText, sizeof(szText));
	pszText = szText;
#endif

	PTSTR pszTel;
	pszTel = StrChr(pszText, FT_POI_SEP);
	if(pszTel)
		*pszTel = CHR_NULL;
	if(!ValidateNameCol(pszText) ||
		((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszText)))
		return;

	LVITEM lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = pszText;
	lvi.iItem    = m_jListView.InsertItem(&lvi);

	TCHAR szCoord[32];
	lvi.pszText = szCoord;
	if(GetCoordString(LVC_LNG, pPR1D->lLng, szCoord) > 0)
	{
		lvi.iSubItem = LVC_LNG;
		m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
	}
	if(GetCoordString(LVC_LAT, pPR1D->lLat, szCoord) > 0)
	{
		lvi.iSubItem = LVC_LAT;
		m_jListView.CListView::SetItemText(lvi.iItem, &lvi);
	}

	if(pszTel)
		ValidateTelCol(lvi.iItem, pszTel + 1);

	if(pPR1D->byNET == NET_TOMTOM_5)
		return;

	if(m_opf == OPF_IMPORT)
		return;

	TCHAR szMemo[32];
	PBYTE pbyInfoId;
	int   iInfoId;
	pbyInfoId = pPR1D->byInfoId;
	if((pPR1D->byNET == NET_EXANSI) || (pPR1D->byNET == NET_TOMTOM_7))
		pbyInfoId++;
	iInfoId = MAKELONG(*(PWORD)pbyInfoId, pbyInfoId[2]);
	_ltot(iInfoId, szMemo);
	ValidateMemoCol(lvi.iItem, szMemo);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads an ov2 record (type 1E) of POI file

AFXINLINE void CTomPoiEdr::ReadOv2Record1E(PPOIRECORD1E pPR1E)
{
	TRACE(_T("Entering CTomPoiEdr::ReadOv2Record1E\n"));

	TCHAR  szText[64];
	int    iLvItem;
	UINT   nCamera, nSpeed;
	LVITEM lvi;
	LoadString(e_hInstance, IDS_SC_NAME, szText, countof(szText));
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = szText;
	iLvItem      = m_jListView.InsertItem(&lvi);
	if(GetCoordString(LVC_LNG, pPR1E->lLng, szText) > 0)
	{
		lvi.iSubItem = LVC_LNG;
		m_jListView.CListView::SetItemText(iLvItem, &lvi);
	}
	if(GetCoordString(LVC_LAT, pPR1E->lLat, szText) > 0)
	{
		lvi.iSubItem = LVC_LAT;
		m_jListView.CListView::SetItemText(iLvItem, &lvi);
	}
	nCamera = pPR1E->byCamera;
	if((nCamera < FT_SCM_FIRST) || (nCamera > FT_SCM_LAST))
		nCamera = FT_SCM_OTHER;
	LoadString(e_hInstance, nCamera - FT_SCM_FIRST + IDS_SCM_FIRST, szText, countof(szText));
	lvi.iSubItem = LVC_CAMERA;
	m_jListView.CListView::SetItemText(iLvItem, &lvi);
	nSpeed = pPR1E->bySpeed;
	if(nSpeed != e_eNonSpeed)
		nSpeed = clamp(nSpeed * e_eBaseSpeed / e_eDivSpeed * e_eDivSpeed,
			e_eMinSpeed, e_eMaxSpeed);
	_ltot(nSpeed, szText);
	lvi.iSubItem = LVC_SPEED;
	m_jListView.CListView::SetItemText(iLvItem, &lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads the POI file

AFXINLINE BOOL CTomPoiEdr::ReadPoiFile()
{
	TRACE(_T("Entering CTomPoiEdr::ReadPoiFile\n"));

	// Opens the existing file
	UINT  nResId;
	HFILE hFile;
	hFile = CreateFile(m_jFileDlg.lpstrFile, GENERIC_READ, OPEN_EXISTING);
	if(hFile != INVALID_HANDLE)
	{
		// Allocates the required memory block, if posible
		DWORD cbFile;
		PVOID pvFile;
		cbFile = GetFileSize(hFile);
		pvFile = malloc(cbFile + sizeof(WCHAR));  // 'sizeof(WCHAR)' for ending
		if(pvFile)
		{
			// Reads the POI records
			if(ReadFile(hFile, pvFile, cbFile, &cbFile))
			{
				PWCHAR     pwEnding;
				int        ceLvItem;
				ACTIONTYPE at;
				LVITEM     lvi;
				m_jListView.UpdateWindow();
				m_jListView.SetWindowRedraw(FALSE);
				ceLvItem = m_jListView.GetItemCount();
				if(m_opf == OPF_READ)
				{
					at            = AT_CUT;
					m_fst         = FST_GENERAL;
					m_pr64.byType = UNUSED;
					m_iLvCol      = LVC_NAME;
					m_bRPF        = FALSE;
					if(ceLvItem)
						m_jListView.DeleteAllItems();
					if((e_dwWinVer > WV_XP) || !ceLvItem)
						m_jListView.Scroll(MIN_INT, 0);
					m_jListView.FreeStorage();
				}
				else
				{
					at = AT_NEW;
					if(ceLvItem)
					{
						lvi.state     = UNUSED;
						lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
						m_jListView.SetItemState(&lvi);
					}
				}
				m_jListView.SetAction(at);
				m_nPoiDupSel  = IDC_RETAIN;
				m_cePoiDupDel = 0;
				pwEnding      = (PWCHAR)((PBYTE)pvFile + cbFile);
				switch(m_jFileDlg.nFilterIndex)
				{
				case FTR_OV2:
					nResId = ReadOv2Record(pvFile, cbFile);
					break;
				case FTR_CFG:
					nResId                  = ReadFavRecord(pvFile, cbFile);
					m_jFileDlg.nFilterIndex = FTR_OV2;
					break;
				case FTR_CPB:
					*pwEnding = CHR_LF;
					nResId    = ReadCpbRecord(pvFile, cbFile);
					break;
				case FTR_KML:
					*pwEnding = CHR_NULL;
					nResId    = ReadKmlRecord(pvFile, cbFile);
					break;
				case FTR_GPX:
					*pwEnding = CHR_NULL;
					nResId    = ReadGpxRecord(pvFile, cbFile);
					break;
				case FTR_GPI:
					nResId = ReadGpiRecord(pvFile, cbFile);
					break;
				case FTR_CSV:
				case FTR_ASC:
				case FTR_POI:
					*pwEnding = CHR_LF;
					nResId    = ReadCsvRecordA(pvFile, cbFile);
					break;
				default:
					*pwEnding = CHR_CR;
					nResId    = ReadTxtRecordA(pvFile, cbFile);
				}
				if(!nResId)
				{
					DisplayFileProgress(1, 1);
					if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
					{
						int iLvItem;
						if((m_opf == OPF_READ) && (m_fst != FST_CPB))
						{
							m_bMyBP = FALSE;
							if(m_jOptionDlg.m_nAutoBP == BST_CHECKED)
							{
								TCHAR szName[e_eccAllCol], szEntry[32];
								int   iNewInsBP;
								iNewInsBP = m_iInsertBP + 2;
								do
								{
									iNewInsBP--;
									if(!iNewInsBP)
										break;
									wsprintf(szEntry, CMyBPDlg::m_szListEntry, iNewInsBP);
								}
								while(!GetPrivateProfileString(CMyBPDlg::m_szSection, szEntry,
									e_szNull, szName, countof(szName), e_pszProfile));
								if(iNewInsBP && (*szName != CMyBPDlg::m_eSepChar))
								{
									TCHAR szText[32];
									PTSTR pszBpData, pszText;
									int   iLvCol, iInsertBP;
									pszBpData = StrChr(szName, CMyBPDlg::m_eSepChar);
									if(pszBpData)
									{
										*pszBpData = CHR_NULL;
										pszBpData++;
									}
									ValidateNameCol(szName);
									lvi.mask     = LVIF_TEXT;
									lvi.iItem    = 0;
									lvi.iSubItem = 0;
									lvi.pszText  = szName;
									iLvItem      = m_jListView.InsertItem(&lvi);
									iLvCol       = LVC_LNG;
									while(pszBpData)
									{
										pszText = pszBpData;
										if(iLvCol == LVC_MEMO)
											pszBpData = NULL;
										else
										{
											pszBpData = StrChr(pszBpData, CMyBPDlg::m_eSepChar);
											if(pszBpData)
											{
												*pszBpData = CHR_NULL;
												pszBpData++;
											}
											if(iLvCol != LVC_TEL)
											{
												ASSERT((iLvCol == LVC_LNG) ||
													(iLvCol == LVC_LAT));
												lstrcpyn(szText, pszText, countof(szText));
												GetCoordString(iLvCol, szText);
												pszText = szText;
											}
										}
										lvi.iSubItem = iLvCol;
										lvi.pszText  = pszText;
										m_jListView.CListView::SetItemText(iLvItem, &lvi);
										iLvCol++;
										if((m_fst == FST_SC) && (iLvCol == LVC_SPEED))
											break;
									}
									iInsertBP = iNewInsBP - 1;
									if(m_iInsertBP != iInsertBP)
									{
										_ltot(iNewInsBP, szText);
										WritePrivateProfileString(CMyBPDlg::m_szSection,
											CMyBPDlg::m_szInsertEntry, szText, e_pszProfile);
										m_iInsertBP = iInsertBP;
									}
									m_bMyBP = TRUE;
								}
							}
							iLvItem = 0;
						}
						else
							iLvItem = m_iLvBaseItem;
						SetBasePoint(iLvItem, FALSE);
					}
					AutoSizeLvCol(LVCA_ALL, FALSE);
					if(m_cePoiDupDel)
						DisplayMessage(IDS_POI_DUP_DEL,
							MB_ICONINFORMATION | MB_OK, m_cePoiDupDel);
				}
				m_jListView.SetWindowRedraw();
			}
			else
				nResId = IDS_CANNOT_READ;
			free(pvFile);
		}
		else
			nResId = IDS_MEMORY_ERROR_FILE;
		CloseHandle(hFile);
	}
	else
		nResId = IDS_CANNOT_OPEN;

	// Determines whether any error has occurred
	if(!nResId)
		return TRUE;

	// An error has occurred
	DisplayMessage(nResId, MB_ICONWARNING | MB_OK);
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads txt records of POI file

AFXINLINE void CTomPoiEdr::ReadTxtRecord(PWCHAR pwStart, PWCHAR pwEnd)
{
	TRACE(_T("Entering CTomPoiEdr::ReadTxtRecord\n"));

	int cwRecord;
	cwRecord = pwEnd - pwStart + 1;
	if(cwRecord > e_eccAllCol)
	{
		pwEnd    = pwStart + e_eccAllCol - 1;
		*pwEnd   = CHR_NULL;
		cwRecord = e_eccAllCol;
	}

	PTSTR pszText;
	int   ccText;
#ifdef UNICODE
	pszText = pwStart;
#else
	char szText[e_eccAllCol];
	pszText = szText;
	pwEnd   = (PWCHAR)(szText + WCStoMBS(pwStart, cwRecord, szText, sizeof(szText)) - 1);
#endif
	ccText = lstrlen(pszText);
	if(!ValidateNameCol(pszText) ||
		((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(pszText)))
		return;

	LVITEM lvi;
	if(m_opf == OPF_IMPORT)
	{
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	}
	else
		lvi.mask = LVIF_TEXT;
	lvi.iItem    = MAX_INT;
	lvi.iSubItem = 0;
	lvi.pszText  = pszText;
	lvi.iItem    = m_jListView.InsertItem(&lvi);
	pszText     += ccText + 1;
	if(pszText >= (PTSTR)pwEnd)
		return;

	if(*pszText)
	{
		ccText = lstrlen(pszText);
		ValidateCoordCol(lvi.iItem, e_iHdOrder[LVCR_LNG], pszText, FALSE);
	}
	else
		ccText = 0;
	pszText += ccText + 1;
	if(pszText >= (PTSTR)pwEnd)
		return;

	if(*pszText)
	{
		ccText = lstrlen(pszText);
		ValidateCoordCol(lvi.iItem, e_iHdOrder[LVCR_LAT], pszText, FALSE);
	}
	else
		ccText = 0;
	pszText += ccText + 1;
	if(pszText >= (PTSTR)pwEnd)
		return;

	if(*pszText)
	{
		ccText = lstrlen(pszText);
		ValidateTelCol(lvi.iItem, pszText);
	}
	else
		ccText = 0;
	if(m_bRPF)
		return;

	pszText += ccText + 1;
	if((pszText < (PTSTR)pwEnd) && *pszText)
		ValidateMemoCol(lvi.iItem, pszText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads txt (ANSI) records of POI file

UINT CTomPoiEdr::ReadTxtRecordA(PVOID pvFile, DWORD cbFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadTxtRecordA\n"));

	if(*(PCHAR)pvFile == CHR_DQM)
	{
		m_jFileDlg.nFilterIndex = FTR_CPB;
		return ReadCpbRecord(pvFile, cbFile);
	}

	if(*(PWCHAR)pvFile == m_eUBOM)
	{
		m_jFileDlg.nFilterIndex = FTR_TXTU;
		ReadTxtRecordU((PWCHAR)pvFile + 1, cbFile / sizeof(WCHAR));
		return 0;
	}

	PCHAR  pcFile;
	PWCHAR pwFile;
	UINT   nCodePage;
	pcFile = (PCHAR)pvFile;
	cbFile++;
	if((*(PWORD)pvFile == FT_UTF8L) && (*((PCHAR)pvFile + 2) == (char)FT_UTF8H))
	{
		nCodePage               = CP_UTF8;
		pcFile                 += FT_UTF8S3;
		cbFile                 -= FT_UTF8S3;
		m_jFileDlg.nFilterIndex = FTR_TXTU8;
	}
	else
	{
		nCodePage               = CP_ACP;
		m_jFileDlg.nFilterIndex = FTR_TXTA;
	}
	pwFile = (PWCHAR)malloc(cbFile * sizeof(WCHAR));
	ReadTxtRecordU(pwFile, MBStoWCS(nCodePage, pcFile, cbFile, pwFile, cbFile));
	free(pwFile);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reads txt (Unicode) records of POI file

void CTomPoiEdr::ReadTxtRecordU(PWCHAR pwFile, DWORD cwFile)
{
	TRACE(_T("Entering CTomPoiEdr::ReadTxtRecordU\n"));

	PWCHAR pwStart, pwEnd, pwSC;
	pwStart = pwFile;
	for(DWORD i = 0; i < cwFile; DisplayFileProgress(i, cwFile), i++)
	{
		if(pwFile[i] == CHR_TAB)
		{
			pwFile[i] = CHR_NULL;
			continue;
		}
		if((pwFile[i] != CHR_CR) && (pwFile[i] != CHR_LF))
			continue;
		pwEnd  = pwFile + i;
		*pwEnd = CHR_NULL;
		if((pwStart != pwEnd) && *pwStart)
		{
			pwSC = pwStart;
			do
			{
				if(*pwSC != CHR_SP)
					break;
				pwSC++;
			}
			while(pwSC < pwEnd);
			if(*pwSC != CHR_SC)
				ReadTxtRecord(pwStart, pwEnd);
		}
		pwStart = pwEnd + 1;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Releases IE pane objects

void CTomPoiEdr::ReleaseIEObject()
{
	TRACE(_T("Entering CTomPoiEdr::ReleaseIEObject\n"));

	if(m_pHD2)
	{
		m_pHD2->Release();
		m_pHD2 = NULL;
	}

	if(m_pDsp)
	{
		m_pDsp->Release();
		m_pDsp = NULL;
	}

	if(m_pOIPAO)
	{
		m_pOIPAO->Release();
		m_pOIPAO = NULL;
	}

	if(m_pWB2)
	{
		m_pWB2->Stop();
		m_pWB2->Release();
		m_pWB2 = NULL;
	}

	if(m_pOO)
	{
		m_pOO->Close(OLECLOSE_NOSAVE);
		m_pOO->Release();
		m_pOO = NULL;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Removes ampersand characters within a menu item string

void CTomPoiEdr::RemoveMenuItemAmp(PTSTR pszText)
{
	TRACE(_T("Entering CTomPoiEdr::RemoveMenuItemAmp\n"));

	PTCHAR pcSrc, pcDest;
	BOOL   bSkipAmp;
	pcSrc    = pszText;
	pcDest   = pszText;
	bSkipAmp = FALSE;
	do
	{
		if(*pcSrc == CHR_LP)
			bSkipAmp = TRUE;
		else if(*pcSrc == CHR_RP)
			bSkipAmp = FALSE;
		else if(!bSkipAmp && (*pcSrc != CHR_AMP))
		{
			*pcDest = *pcSrc;
			pcDest++;
		}
		pcSrc++;
	}
	while(*pcSrc);
	*pcDest = CHR_NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Replaces a POI record

BOOL CTomPoiEdr::ReplacePoiRecord(int iLvItem, BOOL bReplAll)
{
	TRACE(_T("Entering CTomPoiEdr::ReplacePoiRecord\n"));

	TCHAR  szText[e_eccNameCol], szNewText[countof(szText)];
	PTCHAR pcMatch, pcPrevPos;
	PTSTR  (WINAPI* StrStrx)(PCTSTR, PCTSTR);
	BOOL   bReplaced;
	int    ceLvItem, iLastCol, ccFindText, ccReplaceText, ccNewText, ccRetainText, ccTemp;
	LVITEM lvi;
	ASSERT(!bReplAll || !iLvItem);
	m_jListView.SetAction(AT_NEW);
	StrStrx        = (m_jFindReplDlg.m_nMatchCase == BST_CHECKED) ? StrStr : StrStrI;
	ceLvItem       = m_jListView.GetItemCount();
	iLastCol       = (m_jFindReplDlg.m_nOnlyName == BST_CHECKED) ? LVC_NAME : LVC_MEMO;
	ccFindText     = lstrlen(m_jFindReplDlg.m_pszFindText);
	ccReplaceText  = lstrlen(m_jFindReplDlg.m_pszReplaceText);
	bReplaced      = FALSE;
	lvi.cchTextMax = countof(szText);
	lvi.state      = UNUSED;
	lvi.stateMask  = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
	for(int i = iLvItem, j; i < ceLvItem; i++)
	{
		for(j = LVC_NAME; j <= iLastCol; j++)
		{
			lvi.iSubItem = j;
			lvi.pszText  = szText;
			m_jListView.GetItemText(i, &lvi);
			pcMatch = StrStrx(szText, m_jFindReplDlg.m_pszFindText);
			if(!pcMatch)
				continue;

			pcPrevPos = szText;
			ccNewText = 0;
			do
			{
				ccRetainText = pcMatch - pcPrevPos;
				ccTemp       = ccNewText + ccRetainText;
				if(ccTemp >= countof(szText))
					break;
				CopyMemory(szNewText + ccNewText, pcPrevPos, ccRetainText * sizeof(TCHAR));
				ccNewText = ccTemp;
				ccTemp   += ccReplaceText;
				if(ccTemp >= countof(szText))
					break;
				CopyMemory(szNewText + ccNewText, m_jFindReplDlg.m_pszReplaceText,
					ccReplaceText * sizeof(TCHAR));
				ccNewText = ccTemp;
				pcMatch  += ccFindText;
				pcPrevPos = pcMatch;
			}
			while(pcMatch = StrStrx(pcMatch, m_jFindReplDlg.m_pszFindText));
			if(ccTemp >= countof(szText))
				continue;
			ccRetainText = lstrlen(pcPrevPos);
			ccTemp      += ccRetainText;
			if(ccTemp >= countof(szText))
				continue;
			CopyMemory(szNewText + ccNewText, pcPrevPos, (ccRetainText + 1) * sizeof(TCHAR));
			ccNewText = TrimString(szNewText);
			if(!lstrcmp(szNewText, szText))
				continue;

			PutGoogleMaps(m_eGmPutDelete, i);
			switch(j)
			{
			case LVC_NAME:
				if(!ccNewText)
					continue;
				if((m_jOptionDlg.m_nCheckName == BST_CHECKED) &&
					(FindPoiName(szNewText, iLvItem) != RV_ERR))
					continue;
				if(StrPBrk(szNewText, m_szErrToken))
					continue;
				lvi.pszText = szNewText;
				m_jListView.SetItemText(i, &lvi);
				break;
			case LVC_LNG:
			case LVC_LAT:
				GetCoordDdString(szNewText, ccNewText);
				ValidateCoordCol(i, j, szNewText, FALSE);
				break;
			case LVC_TEL:
				ValidateTelCol(i, szNewText);
				break;
			case LVC_MEMO:
				if(!m_bRPF)
					ValidateMemoCol(i, szNewText);
				break;
			default:
				ASSERT_UNREACHABLE();
			}
			if(i || !m_bMyBP)
			{
				m_bFileDirty  = TRUE;
				m_bSetCaption = TRUE;
			}
			bReplaced = TRUE;
			if(bReplAll)
				m_jListView.SetItemState(i, &lvi);
		}
		if(bReplAll)
			continue;
		if(bReplaced)
			return TRUE;
		i = (i + 1) % ceLvItem;
		if(i == iLvItem)
			return FALSE;
		i--;
	}
	return bReplaced;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Resizes child windows

void CTomPoiEdr::ResizeWindow(BOOL bAllWnd)
{
	TRACE(_T("Entering CTomPoiEdr::ResizeWindow\n"));

	if(IsMinimized())
		return;

	int  iDelta;
	RECT rc;
	iDelta = (e_dwWinVer > WV_XP) ? m_eyTbTop : m_eyTbTopXP;
	if(bAllWnd)
	{
		if(m_jOptionDlg.m_nShowTb == BST_CHECKED)
		{
			// Toolbar control
			int cyToolbar, ceButton;
			m_jToolbar.GetWindowRect(&rc);
			if(GetRectWidth(&rc) == m_cxClient)
				cyToolbar = GetRectHeight(&rc);
			else
			{
				m_jToolbar.MoveWindow(0, iDelta, m_cxClient, m_cyToolbar);
				cyToolbar = m_cyToolbar;
			}
			ceButton = m_jToolbar.ButtonCount();
			if(ceButton)
			{
				m_jToolbar.GetItemRect(ceButton - 1, &rc);
				m_cyToolbar = rc.bottom;
			}
			else
				m_cyToolbar = HIWORD(m_jToolbar.GetButtonSize());
			m_cyToolbar += m_eyTbPad + m_ecyTbPad;
			if(m_cyToolbar != cyToolbar)
				m_jToolbar.MoveWindow(0, iDelta, m_cxClient, m_cyToolbar);
		}
		else
			m_cyToolbar = 0;

		if(m_jOptionDlg.m_nShowSb == BST_CHECKED)
		{
			// Status bar control
			int iRightPos[m_eceSbPart];
			m_jStatusBar.AutoSize();
			iRightPos[m_eTimePart] = -1;
			iRightPos[m_ePoiPart]  = m_cxClient - m_cxTimePart;
			iRightPos[m_eBmpPart]  = iRightPos[m_ePoiPart] - m_cxPoiPart;
			iRightPos[m_eInfoPart] = iRightPos[m_eBmpPart] - m_cxBmpPart;
			m_jStatusBar.SetParts(iRightPos, countof(iRightPos));
			m_jStatusBar.GetWindowRect(&rc);
			m_cyStatusBar = GetRectHeight(&rc);
		}
		else
			m_cyStatusBar = 0;
	}

	// List view, header and edit controls
	int yListView, cxListView, cyListView, yHeader, cxIEPane, cyIEPane;
	yHeader    = m_cyToolbar + m_eyHdPad + iDelta;
	cyIEPane   = m_cyClient - yHeader - m_cyStatusBar;
	yListView  = yHeader + m_cyLvHeader;
	cyListView = cyIEPane - m_cyLvHeader;
	if(m_jIEPane.IsWindowVisible())
	{
		m_jIEPane.GetWindowRect(&rc);
		ScreenToClient((PPOINT)&rc);
		cxIEPane = clamp(m_cxClient - rc.left, m_ecxMinIEPane, m_cxClient - m_ecxMinIEPane);
		m_jIEPane.MoveWindow(m_cxClient - cxIEPane, yHeader - 1, cxIEPane, cyIEPane + 1);
		m_pWB2->put_Width(cxIEPane - CIEPane::m_ecxBorder);
		m_pWB2->put_Height(cyIEPane - CIEPane::m_ecyBorder + 1);
		cxIEPane += m_ecxIEPaneSep;
	}
	else
		cxIEPane = 0;
	cxListView = m_cxClient - cxIEPane;
	m_jListView.MoveWindow(0, yListView, cxListView, cyListView);
	::GetWindowRect(m_jListView.GetHeader(), &rc);
	ScreenToClient((PPOINT)&rc);
	m_jLvHeader.MoveWindow(rc.left, yHeader, cxListView - rc.left, m_cyLvHeader);
	if(m_jLvEdit.IsWindowVisible())
	{
		int  iLeft, iRight;
		RECT rcCtrl;
		m_jListView.GetClientRect(&rcCtrl);
		m_jListView.GetItemRect(m_jLvEdit.m_iLvItem, m_iLvCol, LVIR_LABEL, &rc);
		if(m_iLvCol == LVC_NAME)
		{
			if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
			{
				if(e_dwWinVer > WV_XP)
					rc.left -= m_exEdAdjL + m_exEdPad + 1;
				else
					rc.left -= m_exEdAdjL + 1;
			}
			else
				rc.left -= m_exEdAdjL;
		}
		else
			rc.left -= m_exEdPad;
		iLeft   = rc.left;
		rc.left = clamp(iLeft, 0, rcCtrl.right);
		if(e_dwWinVer > WV_XP)
			iDelta = 0;
		else
		{
			iDelta    = 1;
			rc.top   -= m_eyEdPad;
			rc.right += m_exEdPad + 1;
			rc.bottom++;
		}
		iRight    = rc.right;
		rc.right  = clamp(iRight, 0, rcCtrl.right) - rc.left;
		rc.bottom = GetRectHeight(&rc);
		if(m_jLvCombo.IsWindowVisible())
		{
			TCHAR szCbItem[64];
			int   i, cxMaxCombo, cxCombo;
			i          = m_jLvCombo.GetCount();
			cxMaxCombo = 0;
			do
			{
				i--;
				m_jLvCombo.GetItem(i, szCbItem);
				cxCombo = m_jListView.GetStringWidth(szCbItem);
				if(cxCombo > cxMaxCombo)
					cxMaxCombo = cxCombo;
			}
			while(i);
			if(e_dwWinVer > WV_XP)
			{
				rc.left = max(0, rc.left - m_exEdPad);
				iLeft  -= m_exEdPad;
				iRight  = min(iRight + m_exEdPad, rcCtrl.right);
			}
			cxCombo = min(cxMaxCombo + m_jLvCombo.m_cxDropArrow, rcCtrl.right - rc.left);
			if((rc.left + cxCombo) > iRight)
				rc.left = ((iLeft + cxCombo) > iRight) ? iLeft : (iRight - cxCombo);
			m_jLvCombo.GetWindowRect(&rcCtrl);
			m_jLvCombo.MoveWindow(rc.left, rc.top - (GetRectHeight(&rcCtrl) - rc.bottom) / 2,
				cxCombo, m_ecyCbLb);
		}
		else
			m_jLvEdit.MoveWindow(rc.left, rc.top + (rc.bottom - m_cyLvEdit + iDelta) / 2,
				rc.right, m_cyLvEdit);
	}
	InvalidateRect();
	UpdateWindow();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the base point of POIs

void CTomPoiEdr::SetBasePoint(int iLvItem, BOOL bAutoSize)
{
	TRACE(_T("Entering CTomPoiEdr::SetBasePoint\n"));

	if(m_jOptionDlg.m_nShowDist == BST_UNCHECKED)
		return;

	if(iLvItem >= m_jListView.GetItemCount())
		return;

	m_iLvBaseItem = LVI_ERR;
	if((iLvItem != LVI_FIND) || ((iLvItem = FindBasePoint()) != LVI_ERR))
	{
		TCHAR  szCoord[32];
		LVITEM lvi;
		lvi.iSubItem   = LVC_LNG;
		lvi.pszText    = szCoord;
		lvi.cchTextMax = countof(szCoord);
		if(m_jListView.GetItemText(iLvItem, &lvi))
		{
			m_dBaseLng   = GetCoordValueD(szCoord);
			lvi.iSubItem = LVC_LAT;
			if(m_jListView.GetItemText(iLvItem, &lvi))
			{
				double dLatRed;
				m_dBaseRedLng = GetRadians(m_dBaseLng);
				m_dBaseLat    = GetCoordValueD(szCoord);
				dLatRed       = GetRadians(m_dBaseLat);
				m_dBaseSinLat = sin(dLatRed);
				m_dBaseCosLat = cos(dLatRed);
				m_iLvBaseItem = iLvItem;
			}
		}
	}
	SetDistanceText(LVI_ALL, bAutoSize);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the clipboard data, in registered and Unicode text format

AFXINLINE BOOL CTomPoiEdr::SetCbData()
{
	TRACE(_T("Entering CTomPoiEdr::SetCbData\n"));

	HMEMORY hTomMem;
	hTomMem = AllocMemory(m_ecbMinAllocCP);
	if(!hTomMem)
		return FALSE;

	HMEMORY hUniMem;
	hUniMem = AllocMemory(m_ecbMinAllocCP);
	if(!hUniMem)
	{
		FreeMemory(hTomMem);
		return FALSE;
	}

	int    i, cbTomMem, ccTomData, ccUniData, iLvItem, iLvCol, ccText;
	UINT   cbTomAlloc, cbUniAlloc;
	PCHAR  pcTomMem;
	PWSTR  pszwUniMem, pszwData[LVC_COUNT - 1];
	PCBD   pCBD;
	LVITEM lvi;
	pcTomMem        = (PCHAR)LockMemory(hTomMem);
	*(PINT)pcTomMem = m_jListView.GetSelCount();
	cbTomAlloc      = m_ecbMinAllocCP;
	cbTomMem        = sizeof(int);
	pszwUniMem      = (PWSTR)LockMemory(hUniMem);
	*pszwUniMem     = m_eUBOM;
	cbUniAlloc      = m_ecbMinAllocCP;
	ccUniData       = 1;
	iLvItem         = m_jListView.GetNextItem(LVNI_SELECTED);
	lvi.cchTextMax  = MAX_STR;
#ifndef UNICODE
	char szText[e_eccNameCol];
	lvi.pszText = szText;
#endif
	do
	{
		if((cbTomMem + e_eccAllCol * sizeof(WCHAR) + m_ecbCBD) > cbTomAlloc)
		{
			UnlockMemory(hTomMem);
			cbTomAlloc += m_ecbMinAllocCP;
			if(!ReAllocMemory(hTomMem, cbTomAlloc))
			{
				FreeMemory(hTomMem);
				FreeMemory(hUniMem);
				return FALSE;
			}
			pcTomMem = (PCHAR)LockMemory(hTomMem);
		}
		pCBD      = (PCBD)(pcTomMem + cbTomMem);
		ccTomData = 0;
		if(((ccUniData + e_eccAllCol) * sizeof(WCHAR)) > cbUniAlloc)
		{
			UnlockMemory(hUniMem);
			cbUniAlloc += m_ecbMinAllocCP;
			if(!ReAllocMemory(hUniMem, cbUniAlloc))
			{
				FreeMemory(hTomMem);
				FreeMemory(hUniMem);
				return FALSE;
			}
			pszwUniMem = (PWSTR)LockMemory(hUniMem);
		}
		iLvCol = LVC_NAME;
		do
		{
			lvi.iSubItem         = iLvCol;
#ifdef UNICODE
			lvi.pszText          = pCBD->szwData + ccTomData;
			pCBD->ccData[iLvCol] = GetCoordDdString(iLvCol, lvi.pszText,
				m_jListView.GetItemText(iLvItem, &lvi)) + 1;
#else
			pCBD->ccData[iLvCol] = MBStoWCS(szText, GetCoordDdString(iLvCol, szText,
				m_jListView.GetItemText(iLvItem, &lvi)) + 1,
				pCBD->szwData + ccTomData, sizeof(szText));
#endif
			ccTomData           += pCBD->ccData[iLvCol];
			iLvCol++;
		}
		while(iLvCol <= LVC_MEMO);
		cbTomMem          += ccTomData * sizeof(WCHAR) + m_ecbCBD;
		pszwData[LVC_NAME] = pCBD->szwData;
		iLvCol             = LVC_NAME;
		do
		{
			pszwData[iLvCol + 1] = pszwData[iLvCol] + pCBD->ccData[iLvCol];
			iLvCol++;
		}
		while(iLvCol < LVC_MEMO);
		i = LVCR_NAME;
		do
		{
			if((m_fst != FST_CPB) || ((i != LVCR_LNG) && (i != LVCR_LAT)))
			{
				iLvCol = e_iHdOrder[i];
				ccText = pCBD->ccData[iLvCol] - 1;
				CopyMemory(pszwUniMem + ccUniData, pszwData[iLvCol], ccText * sizeof(WCHAR));
				ccUniData              += ccText;
				pszwUniMem[ccUniData++] = CHR_TAB;
			}
			i++;
		}
		while(i <= LVCR_MEMO);
		while(pszwUniMem[ccUniData - 1] == CHR_TAB)
			ccUniData--;
		pszwUniMem[ccUniData++] = CHR_CR;
		pszwUniMem[ccUniData++] = CHR_LF;
		iLvItem                 = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem);
	}
	while(iLvItem != RV_ERR);
	EmptyClipboard();
	ReAllocMemory(hTomMem, cbTomMem);
	UnlockMemory(hTomMem);
	SetClipboardData((m_fst == FST_SC) ? m_nPoiScCbFmt : m_nPoiCbFmt , hTomMem);
	pszwUniMem[ccUniData] = CHR_NULL;
	ReAllocMemory(hUniMem, (ccUniData + 1) * sizeof(WCHAR));
	UnlockMemory(hUniMem);
	SetClipboardData(CF_UNICODETEXT, hUniMem);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the text of Distance column

void CTomPoiEdr::SetDistanceText(int iLvItem, BOOL bAutoSize)
{
	TRACE(_T("Entering CTomPoiEdr::SetDistanceText\n"));

	int ceLvItem;
	ceLvItem = m_jListView.GetItemCount();
	if(ceLvItem && (iLvItem < ceLvItem))
	{
		TCHAR  szDistance[32];
		int    iLastItem;
		LPARAM lParam;
		LVITEM lvi;
		if(iLvItem == LVI_ALL)
		{
			iLvItem   = 0;
			iLastItem = ceLvItem - 1;
		}
		else
			iLastItem = iLvItem;
		lvi.mask = LVIF_PARAM;
		do
		{
			lvi.pszText = NULL;
			if(iLvItem == m_iLvBaseItem)
				lvi.lParam = MIN_LPARAM;
			else
			{
				lvi.lParam = CalcPoiDistance(iLvItem);
				if(lvi.lParam != MAX_LPARAM)
				{
					if(m_bAdjDistance)
					{
						if(iLvItem)
						{
							lParam = CalcPoiDistance(iLvItem, iLvItem - 1);
							if(lParam != MAX_LPARAM)
								lvi.pszText = szDistance;
						}
					}
					else
					{
						lvi.pszText = szDistance;
						lParam      = lvi.lParam;
					}
					if(lvi.pszText)
						GetDistanceString(lParam, szDistance);
				}
			}
			lvi.iSubItem = LVC_DIST;
			m_jListView.CListView::SetItemText(iLvItem, &lvi);
			lvi.iItem    = iLvItem;
			lvi.iSubItem = 0;
			m_jListView.SetItem(&lvi);
			iLvItem++;
		}
		while(iLvItem <= iLastItem);
	}
	if(bAutoSize)
		AutoSizeLvCol(LVC_DIST, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the text of distance columns

void CTomPoiEdr::SetDistanceText2(int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::SetDistanceText2\n"));

	if(iLvItem == m_iLvBaseItem)
	{
		SetBasePoint(iLvItem, TRUE);
		return;
	}

	SetDistanceText(iLvItem, FALSE);
	if(m_bAdjDistance)
		SetDistanceText(iLvItem + 1, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets file filter index

void CTomPoiEdr::SetFileFilter(PCTSTR pszFileExt, BOOL bWriteExport)
{
	TRACE(_T("Entering CTomPoiEdr::SetFileFilter\n"));

	SCPCTSTR spszFTR[] =
	{
		m_szOv2Ext,
		m_szKmlExt,
		m_szGpxExt,
		m_szGpiExt,
		m_szCsvExt,
		m_szAscExt,
		m_szPoiExt,
	};
	SCINT siFTR[] =
	{
		FTR_OV2,
		FTR_KML,
		FTR_GPX,
		FTR_GPI,
		FTR_CSV,
		FTR_ASC,
		FTR_POI,
		FTR_TXT,
	};
	int i;
	i = 0;
	do
	{
		if(!lstrcmpi(pszFileExt, spszFTR[i]))
			break;
		i++;
	}
	while(i < countof(spszFTR));
	if(!bWriteExport || (i < (countof(spszFTR) - 3)))
	{
		m_jFileDlg.nFilterIndex = siFTR[i];
		return;
	}

	UINT nFilterIndex;
	nFilterIndex = (i - countof(spszFTR) + 3) * (FTW_CSVU8 - FTW_CSVA + 1) + FTW_CSVA;
	if(m_jFileDlg.nFilterIndex >= FTW_CSVA)
		nFilterIndex += (m_jFileDlg.nFilterIndex - FTW_CSVA) % (FTW_CSVU8 - FTW_CSVA + 1);
	m_jFileDlg.nFilterIndex = nFilterIndex;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sets the font of list view control

void CTomPoiEdr::SetLvFont()
{
	TRACE(_T("Entering CTomPoiEdr::SetLvFont\n"));

	TCHAR      szText[MAX_STR];
	TEXTMETRIC tm;
	HFONT      hLvFont;
	HDC        hDC;
	LoadString(e_hInstance, IDP_SETTING_FONT, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	hLvFont = CreateFont(&m_jFontDlg);
	m_jListView.SetWindowFont(hLvFont);
	m_jListView.SetTextColor(m_jFontDlg.rgbColors);
	DeleteFont(m_hLvFont);
	m_hLvFont = hLvFont;
	m_jLvHeader.SetWindowFont(hLvFont);
	hDC = m_jLvHeader.GetDC();
	SelectFont(hDC, hLvFont);
	GetTextMetrics(hDC, &tm);
	m_cyLvHeader = tm.tmHeight + GetSystemMetrics(SM_CYBORDER) + m_ecyHdPad;
	m_cyLvEdit   = tm.tmHeight + (GetSystemMetrics(SM_CYBORDER) + 1) * 2;
	m_jLvHeader.ReleaseDC(hDC);
	m_jLvEdit.SetWindowFont(hLvFont);
	m_jLvCombo.SetWindowFont(hLvFont);
	ResizeWindow(FALSE);
	AutoSizeLvCol(LVC_ALL, FALSE);
	m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Displays the IE pane

AFXINLINE int CTomPoiEdr::ShowIEPane()
{
	TRACE(_T("Entering CTomPoiEdr::ShowIEPane\n"));

	if(!m_pWB2)
	{
		if(FAILED(CoCreateInstance(CLSID_WebBrowser, CLSCTX_INPROC_SERVER, &m_pWB2)))
			return -IDS_GOOGLE_MAP_ERROR;
		if(FAILED(m_pWB2->QueryInterface(IID_IOleObject, (PPVOID)&m_pOO)))
			return -IDS_GOOGLE_MAP_ERROR;

		POSTR pszHostName;
		m_pOO->SetClientSite(&m_jIEPane);
#ifdef UNICODE
		pszHostName = (POSTR)e_pszTpeAppName;
#else
		WCHAR szwAppName[MAX_STR];
		MBStoWCS(e_pszTpeAppName, szwAppName, countof(szwAppName));
		pszHostName = szwAppName;
#endif
		m_pOO->SetHostNames(pszHostName, NULL);
		if(FAILED(OleSetContainedObject(m_pOO, TRUE)))
			return -IDS_GOOGLE_MAP_ERROR;

		RECT rc;
		SetRectEmpty(&rc);
		if(FAILED(m_pOO->DoVerb(OLEIVERB_SHOW, NULL, &m_jIEPane, UNUSED, m_jIEPane, &rc)))
			return -IDS_GOOGLE_MAP_ERROR;
		if(FAILED(m_pWB2->QueryInterface(IID_IOleInPlaceActiveObject, (PPVOID)&m_pOIPAO)))
			return -IDS_GOOGLE_MAP_ERROR;
		m_pWB2->put_Left(CIEPane::m_ecxBorder);
		m_pWB2->put_Top(CIEPane::m_ecyBorder);
		m_jIEPane.MoveWindow(m_cxClient - GetPrivateProfileInt(e_szWndSection, m_szIEPaneEntry,
			m_cxClient / 2, e_pszProfile), 0, 0, 0);
	}
	if(!m_jIEPane.IsWindowVisible())
	{
		m_jIEPane.ShowWindow();
		ResizeWindow(FALSE);
	}

	SCCHAR szTrue[]   = "true";
	SCCHAR szFalse[]  = "false";
	SCUINT snGmFoot[] =
	{
#ifdef VER_EN_USA
		IDX_GM_HTM_FOOT_EN,
		IDX_GM_HTM_FOOT_GB,
		IDX_GM_HTM_FOOT_AU,
		IDX_GM_HTM_FOOT_TW,
		IDX_GM_HTM_FOOT_CN,
		IDX_GM_HTM_FOOT_FR,
		IDX_GM_HTM_FOOT_DE,
		IDX_GM_HTM_FOOT_JA,
		IDX_GM_HTM_FOOT_ES,
#elif defined(VER_ESPANA)
		IDX_GM_HTM_FOOT_ES,
		IDX_GM_HTM_FOOT_EN,
		IDX_GM_HTM_FOOT_GB,
		IDX_GM_HTM_FOOT_AU,
		IDX_GM_HTM_FOOT_TW,
		IDX_GM_HTM_FOOT_CN,
		IDX_GM_HTM_FOOT_FR,
		IDX_GM_HTM_FOOT_DE,
		IDX_GM_HTM_FOOT_JA,
#elif defined(VER_FRANCE)
		IDX_GM_HTM_FOOT_FR,
		IDX_GM_HTM_FOOT_EN,
		IDX_GM_HTM_FOOT_GB,
		IDX_GM_HTM_FOOT_AU,
		IDX_GM_HTM_FOOT_TW,
		IDX_GM_HTM_FOOT_CN,
		IDX_GM_HTM_FOOT_DE,
		IDX_GM_HTM_FOOT_JA,
		IDX_GM_HTM_FOOT_ES,
#else
		IDX_GM_HTM_FOOT_TW,
		IDX_GM_HTM_FOOT_CN,
		IDX_GM_HTM_FOOT_EN,
		IDX_GM_HTM_FOOT_GB,
		IDX_GM_HTM_FOOT_AU,
		IDX_GM_HTM_FOOT_FR,
		IDX_GM_HTM_FOOT_DE,
		IDX_GM_HTM_FOOT_JA,
		IDX_GM_HTM_FOOT_ES,
#endif
	};
	char     szuText[MAX_PATH * FT_UTF8S3], szGmHtm[MAX_STR], szGmLang[32];
	TCHAR    szFileName[MAX_PATH];
	TCHAR    szAddMarker[e_eccAllCol * 2], szMarkerRes[MAX_STR];
	TCHAR    szName[e_eccNameCol * 2], szLng[32], szLat[countof(szLng)];
	TCHAR    szTel[e_eccTelCol], szMemo[e_eccMemoCol * 2], szTemp[countof(szName)];
	WCHAR    szwText[countof(szFileName)];
	PTCHAR   pcBS;
	PCHAR    pcGmHtm;
	BOOL     bAllItem;
	int      i, j, ccFileName, ceLvItem, ceSelItem, iLvItem, iFirstItem, ccLng, ccLat;
	DWORD    cbGmHtm, cbRsrc;
	COLORREF cr;
	LOGFONT  lf;
	LVITEM   lvi;
	HRSRC    hRsrc;
	ceLvItem  = m_jListView.GetItemCount();
	ceSelItem = m_jListView.GetSelCount();
	if(ceSelItem && (ceSelItem != ceLvItem))
		bAllItem = FALSE;
	else
	{
		bAllItem  = TRUE;
		ceSelItem = ceLvItem;
	}
	iLvItem    = -1;
	iFirstItem = LVI_ERR;
	pcGmHtm    = (PCHAR)malloc(m_ecbGmHtmDoc + ceSelItem * e_eccAllCol);
	hRsrc      = FindResource(IDX_GM_HTM_HEAD, RT_RCDATA);
	cbRsrc     = SizeofResource(hRsrc);
	CopyMemory(pcGmHtm, LockResource(LoadResource(hRsrc)), cbRsrc);
	cbGmHtm = cbRsrc;
	LoadStringA(e_hInstance, IDS_GM_API_SCRIPT, szGmHtm, sizeof(szGmHtm));
	LoadStringA(e_hInstance, IDS_GM_LANG_CODE0 + m_jOptionDlg.m_iGmLang,
		szGmLang, sizeof(szGmLang));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm, szGmLang);
	lstrcpy(szFileName, e_pszProfile);
	PathRemoveFileSpec(szFileName);
	ccFileName             = lstrlen(szFileName);
	szFileName[ccFileName] = CHR_BSLH;
	LoadString(e_hInstance, IDS_GM_MAP_TYPE_ID4, szFileName + ccFileName + 1, MAX_PATH);
	if(IsValidFile(szFileName))
	{
		pcBS = szFileName;
		while(pcBS = StrChr(pcBS, CHR_BSLH))
		{
			*pcBS = CHR_SLH;
			pcBS++;
		}
#ifdef UNICODE
		WCStoMBS(CP_UTF8, szFileName, szuText, sizeof(szuText));
#else
		WCStoMBS(CP_UTF8, szwText, MBStoWCS(szFileName, szwText, countof(szwText)),
			szuText, sizeof(szuText));
#endif
		LoadStringA(e_hInstance, IDS_GM_MTI4_SCRIPT, szGmHtm, sizeof(szGmHtm));
		cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm, szuText);
	}
	cbGmHtm += LoadStringA(e_hInstance, IDS_GM_CSS_STYLE, pcGmHtm + cbGmHtm, sizeof(szGmHtm));
	LoadStringA(e_hInstance, IDS_GM_INFOWND_STYLE, szGmHtm, sizeof(szGmHtm));
	GetObject(m_hLvFont, &lf);
	if(lf.lfHeight > 0)
		lf.lfHeight = lf.lfHeight * 96 / 72;
	else if(lf.lfHeight < 0)
		lf.lfHeight = -lf.lfHeight;
	else
		lf.lfHeight = 13;
#ifdef UNICODE
	WCStoMBS(CP_UTF8, lf.lfFaceName, szuText, sizeof(szuText));
#else
	WCStoMBS(CP_UTF8, szwText, MBStoWCS(lf.lfFaceName, szwText, countof(szwText)),
		szuText, sizeof(szuText));
#endif
	cr       = m_jListView.GetTextColor();
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm, lf.lfItalic ? "italic" : "normal",
		lf.lfWeight, lf.lfHeight, szuText, GetRValue(cr), GetGValue(cr), GetBValue(cr));
	hRsrc    = FindResource(IDX_GM_HTM_SCRIPT, RT_RCDATA);
	cbRsrc   = SizeofResource(hRsrc);
	CopyMemory(pcGmHtm + cbGmHtm, LockResource(LoadResource(hRsrc)), cbRsrc);
	cbGmHtm += cbRsrc;
	LoadString(e_hInstance, IDS_GM_MARKER_VAR, szMarkerRes, countof(szMarkerRes));
	lstrcpy(szFileName, e_pszProfile);
	PathRemoveFileSpec(szFileName);
	pcBS = szFileName;
	while(pcBS = StrChr(pcBS, CHR_BSLH))
	{
		*pcBS = CHR_SLH;
		pcBS++;
	}
#ifdef UNICODE
	cbGmHtm += WCStoMBS(CP_UTF8, szwText, wsprintf(szwText, szMarkerRes, szFileName),
		pcGmHtm + cbGmHtm, sizeof(szuText));
#else
	cbGmHtm += WCStoMBS(CP_UTF8, szwText,
		MBStoWCS(szuText, wsprintf(szuText, szMarkerRes, szFileName),
		szwText, countof(szwText)), pcGmHtm + cbGmHtm, sizeof(szuText));
#endif
	LoadStringA(e_hInstance, IDS_GM_LF_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nLngFirst == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_DMS_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nDmsFormat == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_COMPASS_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nCompassDir == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_METRIC_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nMetricUnit == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_SC_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm, (m_fst == FST_SC) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_RPF_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm, m_bRPF ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_COOKIE_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nGmUseCookie == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_MEMO_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nGmUpdateMemo == BST_CHECKED) ? szTrue : szFalse);
	LoadStringA(e_hInstance, IDS_GM_ROUTE_VAR, szGmHtm, sizeof(szGmHtm));
	cbGmHtm += wsprintfA(pcGmHtm + cbGmHtm, szGmHtm,
		(m_jOptionDlg.m_nGmFlashRoute == BST_CHECKED) ? szTrue : szFalse);
	cbGmHtm += LoadStringA(e_hInstance, IDS_GM_LOAD_MARKER, pcGmHtm + cbGmHtm, sizeof(szGmHtm));
	LoadString(e_hInstance, IDS_GM_ADD_MARKER, szMarkerRes, countof(szMarkerRes));
	while(bAllItem ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		lvi.iSubItem   = LVC_LNG;
		lvi.pszText    = szLng;
		lvi.cchTextMax = countof(szLng);
		ccLng          = m_jListView.GetItemText(iLvItem, &lvi);
		if(!ccLng)
			continue;
		lvi.iSubItem = LVC_LAT;
		lvi.pszText  = szLat;
		ccLat        = m_jListView.GetItemText(iLvItem, &lvi);
		if(!ccLat)
			continue;
		GetCoordDdString(szLng, ccLng);
		GetCoordDdString(szLat, ccLat);
		lvi.iSubItem   = LVC_NAME;
		lvi.pszText    = szTemp;
		lvi.cchTextMax = countof(szTemp);
		m_jListView.GetItemText(iLvItem, &lvi);
		for(i = 0, j = 0; szTemp[i] && (j < (countof(szName) - 2)); i++, j++)
		{
#ifndef UNICODE
			if(IsDBCSLeadByte(szTemp[i]))
				szName[j++] = szTemp[i++];
			else
#endif
			if((szTemp[i] == CHR_SQM) || (szTemp[i] == CHR_BSLH))
				szName[j++] = CHR_BSLH;
			szName[j] = szTemp[i];
		}
		szName[j]      = CHR_NULL;
		lvi.iSubItem   = LVC_TEL;
		lvi.pszText    = szTel;
		lvi.cchTextMax = countof(szTel);
		m_jListView.GetItemText(iLvItem, &lvi);
		lvi.iSubItem   = LVC_MEMO;
		lvi.pszText    = szTemp;
		lvi.cchTextMax = countof(szTemp);
		m_jListView.GetItemText(iLvItem, &lvi);
		for(i = 0, j = 0; szTemp[i] && (j < (countof(szMemo) - 2)); i++, j++)
		{
#ifndef UNICODE
			if(IsDBCSLeadByte(szTemp[i]))
				szMemo[j++] = szTemp[i++];
			else
#endif
			if((szTemp[i] == CHR_SQM) || (szTemp[i] == CHR_BSLH))
				szMemo[j++] = CHR_BSLH;
			szMemo[j] = szTemp[i];
		}
		szMemo[j] = CHR_NULL;
#ifdef UNICODE
		cbGmHtm  += WCStoMBS(CP_UTF8, szAddMarker,
			wsprintf(szAddMarker, szMarkerRes, szName, szLat, szLng, szTel, szMemo),
			pcGmHtm + cbGmHtm, countof(szAddMarker) * FT_UTF8S3);
#else
		WCHAR szwAddMarker[countof(szAddMarker)];
		cbGmHtm += WCStoMBS(CP_UTF8, szwAddMarker, MBStoWCS(szAddMarker,
			wsprintf(szAddMarker, szMarkerRes, szName, szLat, szLng, szTel, szMemo),
			szwAddMarker, countof(szwAddMarker)),
			pcGmHtm + cbGmHtm, countof(szwAddMarker) * FT_UTF8S3);
#endif
		if(iFirstItem == LVI_ERR)
			iFirstItem = iLvItem;
	}
	hRsrc  = FindResource(snGmFoot[m_jOptionDlg.m_iGmLang], RT_RCDATA);
	cbRsrc = SizeofResource(hRsrc);
	CopyMemory(pcGmHtm + cbGmHtm, LockResource(LoadResource(hRsrc)), cbRsrc);
	cbGmHtm += cbRsrc;

	int   iResId;
	HFILE hFile;
	iResId = IDS_DISK_ERROR;
	hFile  = CreateFile(m_pszGmTempFile, GENERIC_WRITE, CREATE_ALWAYS);
	if(hFile == INVALID_HANDLE)
	{
		if(GetLastError() == ERROR_SHARING_VIOLATION)
			iResId = MIN_INT;
	}
	else
	{
		if(WriteFile(hFile, pcGmHtm, cbGmHtm, &cbGmHtm))
			iResId = 0;
		CloseHandle(hFile);
	}
	free(pcGmHtm);
	if(iResId)
		return ((iResId == MIN_INT) ? 0 : iResId);

	SCOCHAR szFileUrl[] = { L'f', L'i', L'l', L'e', L':', L'/', L'/', L'/' };
	OCHAR   szGmUrl[MAX_PATH];
	BSTR    bszGmUrl;
	HRESULT hResult;
	if(m_pHD2)
	{
		m_pHD2->Release();
		m_pHD2 = NULL;
	}
	if(m_pDsp)
	{
		m_pDsp->Release();
		m_pDsp = NULL;
	}
	CopyMemory(szGmUrl, szFileUrl, sizeof(szFileUrl));
#ifdef UNICODE
	lstrcpyW(szGmUrl + countof(szFileUrl), m_pszGmTempFile);
#else
	MBStoWCS(m_pszGmTempFile, szGmUrl + countof(szFileUrl), countof(szGmUrl));
#endif
	bszGmUrl = SysAllocString(szGmUrl);
	hResult  = m_pWB2->Navigate(bszGmUrl, &m_vtEmpty, &m_vtEmpty, &m_vtEmpty, &m_vtEmpty);
	SysFreeString(bszGmUrl);
	if(FAILED(hResult))
		return -IDS_GOOGLE_MAP_ERROR;
	if(FAILED(m_pWB2->get_Document(&m_pDsp)))
		return -IDS_GOOGLE_MAP_ERROR;
	if(FAILED(m_pDsp->QueryInterface(iidof(IHTMLDocument2), (PPVOID)&m_pHD2)))
		return -IDS_GOOGLE_MAP_ERROR;
	if(iFirstItem != LVI_ERR)
		m_jListView.EnsureVisible(iFirstItem);
	m_bGmReady = FALSE;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sorts the list view items

void CTomPoiEdr::SortLvItems(int iLvCol, BOOL bAscend, ACTIONTYPE at)
{
	TRACE(_T("Entering CTomPoiEdr::SortLvItems\n"));

	TCHAR    szText[MAX_STR];
	int      i;
	UINT     nTbBtnId;
	HDITEM   hi;
	LVCOLUMN lvc;
	ASSERT((at == AT_NEW) || (at == AT_UNDO) || (at == AT_REDO));
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	nTbBtnId = bAscend ? ID_POS_SORT_ASCEND : ID_POS_SORT_DESCEND;
	m_jToolbar.PressButton(nTbBtnId);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDP_SORTING_POI, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	lvc.mask = LVCF_FMT;
	hi.mask  = HDI_FORMAT;
	i        = LVC_FIRST;
	do
	{
		if(i == LVC_DIST)
		{
			lvc.fmt = LVCFMT_RIGHT;
			hi.fmt  = HDF_RIGHT | HDF_STRING;
		}
		else
		{
			lvc.fmt = LVCFMT_LEFT;
			hi.fmt  = HDF_LEFT | HDF_STRING;
		}
		if((i == iLvCol) && (at != AT_UNDO))
		{
			if(bAscend)
			{
				lvc.fmt |= HDF_SORTUP;
				hi.fmt  |= HDF_SORTUP;
			}
			else
			{
				lvc.fmt |= HDF_SORTDOWN;
				hi.fmt  |= HDF_SORTDOWN;
			}
		}
		m_jListView.SetColumn(i, &lvc);
		m_jLvHeader.SetItem(i, &hi);
		i++;
	}
	while(i < LVC_COUNT);
	m_iLvCol      = iLvCol;
	m_bSortAscend = bAscend;
	if(at == AT_UNDO)
		m_jListView.SortItems(CLvCtrl::CompareLvIndex);
	else
	{
		if(at == AT_NEW)
			m_jListView.SetAction(AC_SORT, TRUE);
		if(iLvCol == LVC_DIST)
			m_jListView.SortItems(CompareLvParam);
		else
			m_jListView.SortItems(CompareLvItem);
		if(at == AT_NEW)
			m_jListView.SetAction(AC_SORT, FALSE);
	}
	if(m_iLvBaseItem != LVI_ERR)
	{
		m_iLvBaseItem = FindBasePoint();
		if(m_bAdjDistance)
			SetDistanceText(LVI_ALL, FALSE);
	}
	m_jToolbar.PressButton(nTbBtnId, FALSE);
	m_bFileDirty  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Swaps the coordinate columns

void CTomPoiEdr::SwapCoordCol()
{
	TRACE(_T("Entering CTomPoiEdr::SwapCoordCol\n"));

	int iLngOrder, iLatOrder;
	if(m_jOptionDlg.m_nLngFirst == BST_CHECKED)
	{
		iLngOrder = LVC_LNG;
		iLatOrder = LVC_LAT;
	}
	else
	{
		iLngOrder = LVC_LAT;
		iLatOrder = LVC_LNG;
	}
	e_iHdOrder[LVCR_LNG] = iLngOrder;
	e_iHdOrder[LVCR_LAT] = iLatOrder;
	m_jListView.SetColumnOrder(e_iHdOrder, countof(e_iHdOrder));
	m_jLvHeader.SetItemOrder(e_iHdOrder, countof(e_iHdOrder));
	m_jListView.InvalidateRect();
	AutoSizeLvCol(LVC_LNG, FALSE);
	AutoSizeLvCol(LVC_LAT, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Converts TomChar to WinChar

WCHAR CTomPoiEdr::TomChar2WinChar(WCHAR wTom)
{
	TRACE(_T("Entering CTomPoiEdr::TomChar2WinChar\n"));

	if((wTom >= m_eAsciiPage) && (wTom < (m_eAsciiPage + m_eccCharPage)))
		return (wTom - m_eAsciiPage);

	BYTE  byTom;
	WCHAR wWin;
	if(wTom < m_eLatinPage)
		wTom += m_eccCharPage;
	if(wTom < m_eCJKRPage)
		wTom += m_eccCharPage;
	byTom = LOBYTE(wTom);
	wWin  = HIBYTE(wTom) * m_ePageDelta - m_eMapDelta + byTom;
	if(byTom > m_eCharAdjH)
		wWin -= 2;
	else if(byTom > m_eCharAdjL)
		wWin--;
	if(wWin > (m_eCJKCPage - m_eCJKCDelta))
		wWin += m_eCJKCDelta;
	return wWin;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Converts TomFile to WinFile

int CTomPoiEdr::TomFile2WinFile(PCTSTR pszFileName, PTSTR pszBuffer)
{
	TRACE(_T("Entering CTomPoiEdr::TomFile2WinFile\n"));

	if(!pszFileName || !*pszFileName)
	{
		*pszBuffer = CHR_NULL;
		return 0;
	}

	WCHAR szwTomFile[MAX_PATH];
	PWSTR pszwWinFile;
	int   ccTomFile;
#ifdef UNICODE
	ccTomFile = lstrlenW(pszFileName);
	CopyMemory(szwTomFile, pszFileName, (ccTomFile + 1) * sizeof(WCHAR));
	pszwWinFile = pszBuffer;
#else
	WCHAR szwWinFile[countof(szwTomFile)];
	ccTomFile   = MBStoWCS(pszFileName, szwTomFile, countof(szwTomFile)) - 1;
	pszwWinFile = szwWinFile;
#endif

	PCWCHAR pwBSlash;
	BYTE    byTfnCode[3];
	int     ccPathName, ceRepeat;
	pwBSlash = StrRChrW(szwTomFile, NULL, CHR_BSLH);
	if(pwBSlash)
	{
		ccPathName = pwBSlash - szwTomFile + 1;
		CopyMemory(pszwWinFile, szwTomFile, ccPathName * sizeof(WCHAR));
	}
	else
		ccPathName = 0;
	ceRepeat = ccTomFile - sizeof(byTfnCode);
	for(int i = ccPathName, j = i, k; i < ceRepeat; i++, j++)
	{
		if(szwTomFile[i] != CHR_RCB)
		{
			pszwWinFile[j] = szwTomFile[i];
			continue;
		}
		k = 0;
		do
		{
			byTfnCode[k] = GetTomFileCode(szwTomFile[i + k + 1]);
			if(byTfnCode[k] == MAX_BYTE)
				break;
			k++;
		}
		while(k < sizeof(byTfnCode));
		if(k != sizeof(byTfnCode))
		{
			pszwWinFile[j] = szwTomFile[i];
			continue;
		}
		pszwWinFile[j] = TomChar2WinChar(byTfnCode[0] * sizeof(m_byTfnCode) *
			sizeof(m_byTfnCode) + byTfnCode[1] * sizeof(m_byTfnCode) + byTfnCode[2]);
		i += sizeof(byTfnCode);
	}
	do
	{
		pszwWinFile[j] = szwTomFile[i];
		j++;
		i++;
	}
	while(i <= ccTomFile);
#ifndef UNICODE
	j = WCStoMBS(pszwWinFile, j, pszBuffer, countof(szwTomFile));
#endif
	return (j - 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Updates coordinate data when format has changed

void CTomPoiEdr::UpdateCoordData(BOOL bDmsFormat)
{
	TRACE(_T("Entering CTomPoiEdr::UpdateCoordData\n"));

	PUINT pnOption;
	int   iPutCmd;
	UINT  nCheck;
	if(bDmsFormat)
	{
		pnOption = &m_jOptionDlg.m_nDmsFormat;
		if(*pnOption == BST_CHECKED)
		{
			nCheck  = BST_UNCHECKED;
			iPutCmd = m_eGmPutDFF;
		}
		else
		{
			nCheck  = BST_CHECKED;
			iPutCmd = m_eGmPutDFT;
		}
	}
	else
	{
		pnOption = &m_jOptionDlg.m_nCompassDir;
		if(*pnOption == BST_CHECKED)
		{
			nCheck  = BST_UNCHECKED;
			iPutCmd = m_eGmPutCDF;
		}
		else
		{
			nCheck  = BST_CHECKED;
			iPutCmd = m_eGmPutCDT;
		}
	}
	*pnOption = nCheck;
	PutGoogleMaps(iPutCmd, UNUSED);

	int ceLvItem;
	ceLvItem = m_jListView.GetItemCount();
	if(!ceLvItem)
		return;

	TCHAR  szCoord[32];
	int    iLvItem, iLvCol;
	LVITEM lvi;
	iLvItem        = 0;
	lvi.pszText    = szCoord;
	lvi.cchTextMax = countof(szCoord);
	do
	{
		iLvCol = LVC_LNG;
		do
		{
			lvi.iSubItem = iLvCol;
			if(m_jListView.GetItemText(iLvItem, &lvi))
			{
				GetCoordString(iLvCol, szCoord, szCoord);
				m_jListView.CListView::SetItemText(iLvItem, &lvi);
			}
			iLvCol++;
		}
		while(iLvCol <= LVC_LAT);
		iLvItem++;
	}
	while(iLvItem < ceLvItem);
	AutoSizeLvCol(LVC_LNG, FALSE);
	AutoSizeLvCol(LVC_LAT, FALSE);
	if(!m_jCheckDataDlg)
		return;

	ceLvItem = m_jCheckDataDlg.m_jListView.GetItemCount();
	if(!ceLvItem)
		return;

	int i;
	iLvItem = 0;
	do
	{
		lvi.mask     = LVIF_PARAM;
		lvi.iItem    = iLvItem;
		lvi.iSubItem = 0;
		m_jCheckDataDlg.m_jListView.GetItem(&lvi);
		i      = m_jListView.MapIdToIndex(((CCheckDataDlg::PLVITEMDATA)lvi.lParam)->nItemId);
		iLvCol = LVC_LNG;
		do
		{
			lvi.iSubItem = iLvCol;
			if(m_jListView.GetItemText(i, &lvi))
				m_jCheckDataDlg.m_jListView.SetItemText(iLvItem, &lvi);
			iLvCol++;
		}
		while(iLvCol <= LVC_LAT);
		iLvItem++;
	}
	while(iLvItem < ceLvItem);
	m_jCheckDataDlg.m_jListView.SetColumnWidth(LVC_LNG, LVSCW_AUTOSIZE_USEHEADER);
	m_jCheckDataDlg.m_jListView.SetColumnWidth(LVC_LAT, LVSCW_AUTOSIZE_USEHEADER);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Updates states of toolbar buttons

void CTomPoiEdr::UpdateTbButton()
{
	TRACE(_T("Entering CTomPoiEdr::UpdateTbButton\n"));

	BOOL bSelOne, bNextItem, bSortItem, bShowDist, bNotSelBP, bSaveFile, bCanQuery, bCanOrien;
	int  ceLvItem, ceSelItem, iLvItem;
	ceLvItem  = m_jListView.GetItemCount();
	ceSelItem = m_jListView.GetSelCount();
	iLvItem   = m_jListView.GetNextItem(LVNI_SELECTED);
	bSelOne   = ceSelItem == 1;
	bNextItem = *m_jFindReplDlg.m_pszFindText && ceLvItem;
	bSortItem = (ceLvItem >= 3) || ((ceLvItem == 2) && !m_bMyBP);
	bShowDist = m_jOptionDlg.m_nShowDist == BST_CHECKED;
	bNotSelBP = iLvItem || !m_bMyBP;
	bSaveFile = (ceLvItem >= 2) || (ceLvItem && !m_bMyBP);
	bCanQuery = (m_fst == FST_GENERAL) && !m_bRPF && ceLvItem;
	bCanOrien = FALSE;
	if(bSelOne)
	{
		TCHAR  szCoord[2];
		LVITEM lvi;
		lvi.iSubItem   = LVC_LNG;
		lvi.pszText    = szCoord;
		lvi.cchTextMax = countof(szCoord);
		if(m_jListView.GetItemText(iLvItem, &lvi))
		{
			lvi.iSubItem = LVC_LAT;
			if(m_jListView.GetItemText(iLvItem, &lvi))
				bCanOrien = TRUE;
		}
	}

	// File menu items
	m_jToolbar.EnableButton(ID_FILE_OPEN_DEVICE, m_bGpsDevice);
	m_jToolbar.EnableButton(ID_FILE_DUMP_DEVICE, m_gpsd != GPSD_NONE);
	m_jToolbar.EnableButton(ID_FILE_SAVE, m_bFileDirty && bSaveFile);
	m_jToolbar.EnableButton(ID_FILE_SAVE_AS, bSaveFile);
	m_jToolbar.EnableButton(ID_FILE_SAVE_DEVICE,
		m_bGpsDevice && (m_fst != FST_CPB) && bSaveFile);
	m_jToolbar.EnableButton(ID_FILE_EXPORT, ceSelItem);

	// POI menu items
	int iRedoLayer;
	iRedoLayer = m_jListView.m_iLayer + 1;
	m_jToolbar.EnableButton(ID_POI_UNDO, m_jListView.m_iLayer != INVALID_VALUE);
	m_jToolbar.EnableButton(ID_POI_REDO,
		(iRedoLayer < e_eceURLayer) && m_jListView.m_pAD[iRedoLayer]);
	m_jToolbar.EnableButton(ID_POI_INSERT, bSelOne);
	m_jToolbar.EnableButton(ID_POI_COMBINE_COL, (m_fst == FST_GENERAL) ? ceLvItem : FALSE);
	m_jToolbar.EnableButton(ID_POI_EDIT, bSelOne);
	m_jToolbar.EnableButton(ID_POI_DELETE, ceSelItem);
	m_jToolbar.EnableButton(ID_POI_CLEAR_COL, ceLvItem);
	m_jToolbar.EnableButton(ID_POI_CUT, ceSelItem);
	m_jToolbar.EnableButton(ID_POI_COPY, ceSelItem);
	m_jToolbar.EnableButton(ID_POI_PASTE,
		IsClipboardFormatAvailable((m_fst == FST_SC) ? m_nPoiScCbFmt : m_nPoiCbFmt) ||
		IsClipboardFormatAvailable(m_nItnCbFmt));
	m_jToolbar.EnableButton(ID_POI_QUERY_COORD, bCanQuery);
	m_jToolbar.EnableButton(ID_POI_QUERY_ADDR, bCanQuery);
	m_jToolbar.EnableButton(ID_POI_FIND, ceLvItem);
	m_jToolbar.EnableButton(ID_POI_FIND_NEXT, bNextItem);
	m_jToolbar.EnableButton(ID_POI_REPLACE, ceLvItem);
	m_jToolbar.EnableButton(ID_POI_REPLACE_NEXT, bNextItem);
	m_jToolbar.EnableButton(ID_POI_MESS_COORD, (m_fst != FST_CPB) && ceLvItem);
	m_jToolbar.EnableButton(ID_POI_CHECK, ceLvItem);
	m_jToolbar.CheckButton(ID_POI_CHECK, m_jCheckDataDlg != NULL);
	m_jToolbar.EnableButton(ID_POI_ADD_FAV, m_bTomTomHome && bCanOrien);
	m_jToolbar.EnableButton(ID_POI_ADD_ITN,
		ceSelItem && (m_fst == FST_GENERAL) && ::FindWindow(e_pszTieClassName));

	// Position menu items
	m_jToolbar.EnableButton(ID_POS_UP,
		bSelOne && ((iLvItem >= 2) || ((iLvItem == 1) && !m_bMyBP)));
	m_jToolbar.EnableButton(ID_POS_DOWN, bSelOne && bNotSelBP && (iLvItem < (ceLvItem - 1)));
	m_jToolbar.EnableButton(ID_POS_SET_BP, bShowDist && bNotSelBP && bCanOrien);
	m_jToolbar.EnableButton(ID_POS_MY_BP, bShowDist);
	m_jToolbar.EnableButton(ID_POS_SORT_ASCEND, bSortItem);
	m_jToolbar.EnableButton(ID_POS_SORT_DESCEND, bSortItem);
	m_jToolbar.EnableButton(ID_POS_GOOGLE_MAP, (m_fst != FST_CPB) && *m_pszGmTempFile);
	m_jToolbar.EnableButton(ID_POS_GOOGLE_EARTH, (m_fst != FST_CPB) && *m_pszGeTempFile);

	// Tools menu items
	m_jToolbar.CheckButton(ID_TOOLS_LVBKGND, m_bLvBkgnd);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Validates the lng and lat columns

void CTomPoiEdr::ValidateCoordCol(int iLvItem, int iLvCol, PTSTR pszText, BOOL bClearCol)
{
	TRACE(_T("Entering CTomPoiEdr::ValidateCoordCol\n"));

	PTSTR pszCoord;
	TCHAR szCoord[32];
	pszCoord = NULL;
	if(pszText && *pszText)
	{
		double dCoord;
		dCoord = _ttof(pszText);
		if(dCoord || IsValidFloat(pszText))
		{
			if((dCoord > -0.000005) && (dCoord <= 0))
				dCoord = 0;
			if(GetCoordString(iLvCol, dCoord, FALSE, szCoord) > 0)
				pszCoord = szCoord;
		}
	}

	if(!pszCoord && !bClearCol)
		return;

	LVITEM lvi;
	lvi.iSubItem = iLvCol;
	lvi.pszText  = pszCoord;
	m_jListView.SetItemText(iLvItem, &lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Validates the memo, camera and cpb columns

void CTomPoiEdr::ValidateMemoCol(int iLvItem, PTSTR pszText)
{
	TRACE(_T("Entering CTomPoiEdr::ValidateMemoCol\n"));

	if(pszText && *pszText)
	{
		PTCHAR pcErrToken;
		if(lstrlen(pszText) > (e_eccMemoCol - 1))
			pszText[e_eccMemoCol - 1] = CHR_NULL;
		pcErrToken = pszText;
		while(pcErrToken = StrPBrk(pcErrToken, m_szCtrlToken))
		{
			*pcErrToken = CHR_SP;
			pcErrToken++;
		}
		TrimString(pszText);
		if(m_fst != FST_GENERAL)
		{
			TCHAR szText[64];
			int   i, iLast;
			if(m_fst == FST_SC)
			{
				i     = IDS_SCM_FIRST;
				iLast = IDS_SCM_LAST;
			}
			else
			{
				ASSERT(m_fst == FST_CPB);
				i     = IDS_CPBM_FIRST;
				iLast = IDS_CPBM_LAST;
			}
			do
			{
				LoadString(e_hInstance, i, szText, countof(szText));
				if(!lstrcmp(pszText, szText))
					break;
				i++;
			}
			while(i <= iLast);
			if(i > iLast)
				pszText = NULL;
		}
	}

	LVITEM lvi;
	lvi.iSubItem = LVC_MEMO;
	lvi.pszText  = pszText;
	m_jListView.SetItemText(iLvItem, &lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Validates the name column

int CTomPoiEdr::ValidateNameCol(PTSTR pszText)
{
	TRACE(_T("Entering CTomPoiEdr::ValidateNameCol\n"));

	if(!*pszText)
		return 0;

	TCHAR  szErrToken[32];
	PTCHAR pcErrToken;
	if(lstrlen(pszText) > (e_eccNameCol - 1))
		pszText[e_eccNameCol - 1] = CHR_NULL;
	pcErrToken = pszText;
	lstrcpy(szErrToken, m_szErrToken);
	lstrcat(szErrToken, m_szCtrlToken);
	while(pcErrToken = StrPBrk(pcErrToken, szErrToken))
	{
		*pcErrToken = CHR_SP;
		pcErrToken++;
	}
	return TrimString(pszText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Validates the telephone and speed columns

void CTomPoiEdr::ValidateTelCol(int iLvItem, PTSTR pszText)
{
	TRACE(_T("Entering CTomPoiEdr::ValidateTelCol\n"));

	if(pszText && *pszText)
	{
		int i, j;
		i = 0;
		j = TrimString(pszText);
		if(m_fst == FST_SC)
		{
			do
			{
				if(!IsValidNumber(pszText[i]))
					break;
				i++;
			}
			while(pszText[i]);
			if(pszText[i])
				pszText = NULL;
			else
			{
				i = _ttoi(pszText);
				if(i != e_eNonSpeed)
					i = clamp(i, e_eMinSpeed, e_eMaxSpeed) / e_eDivSpeed * e_eDivSpeed;
				_ltot(i, pszText);
			}
		}
		else
		{
			if(j >= e_eccTelCol)
			{
				pszText[e_eccTelCol - 1] = CHR_NULL;
				TrimString(pszText);
			}
			j = 0;
			while(pszText[i])
			{
				if(!IsValidNumber(pszText[i]) && !IsValidTelSign(pszText[i]))
					pszText[i] = CHR_SP;
				if((pszText[i] != CHR_SP) || (i && (pszText[i - 1] != CHR_SP)))
				{
					pszText[j] = pszText[i];
					j++;
				}
				i++;
			}
			pszText[j] = CHR_NULL;
		}
	}

	LVITEM lvi;
	lvi.iSubItem = LVC_TEL;
	lvi.pszText  = pszText;
	m_jListView.SetItemText(iLvItem, &lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Converts WinChar to TomChar

WCHAR CTomPoiEdr::WinChar2TomChar(WCHAR wWin)
{
	TRACE(_T("Entering CTomPoiEdr::WinChar2TomChar\n"));

	if(wWin < m_eccCharPage)
		return (wWin + m_eAsciiPage);

	union
	{
		BYTE  byTom[2];
		WCHAR wTom;
	};
	if(wWin > m_eCJKCPage)
		wWin -= m_eCJKCDelta;
	wWin += m_eMapDelta;
	byTom[1] = wWin / m_ePageDelta;
	byTom[0] = wWin % m_ePageDelta;
	if(byTom[0] < m_eCharAdjD)
		wTom -= m_eDupDelta;
	else if(byTom[0] >= m_eCharAdjH)
		byTom[0] += 2;
	else if(byTom[0] > m_eCharAdjL)
		byTom[0]++;
	if(wTom < (m_eCJKRPage + m_eccCharPage))
		wTom -= m_eccCharPage;
	if(wTom < (m_eLatinPage + m_eccCharPage))
		wTom -= m_eccCharPage;
	return wTom;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Converts WinFile to TomFile

int CTomPoiEdr::WinFile2TomFile(PCTSTR pszFileName, PTSTR pszBuffer)
{
	TRACE(_T("Entering CTomPoiEdr::WinFile2TomFile\n"));

	if(!pszFileName || !*pszFileName)
	{
		*pszBuffer = CHR_NULL;
		return 0;
	}

	WCHAR szwWinFile[MAX_PATH];
	PWSTR pszwTomFile;
	int   ccWinFile;
#ifdef UNICODE
	ccWinFile = lstrlenW(pszFileName);
	CopyMemory(szwWinFile, pszFileName, (ccWinFile + 1) * sizeof(WCHAR));
	pszwTomFile = pszBuffer;
#else
	WCHAR szwTomFile[countof(szwWinFile)];
	ccWinFile   = MBStoWCS(pszFileName, szwWinFile, countof(szwWinFile)) - 1;
	pszwTomFile = szwTomFile;
#endif

	PCWCHAR pwBSlash;
	int     ccPathName;
	WCHAR   wTom;
	pwBSlash = StrRChrW(szwWinFile, NULL, CHR_BSLH);
	if(pwBSlash)
	{
		ccPathName = pwBSlash - szwWinFile + 1;
		CopyMemory(pszwTomFile, szwWinFile, ccPathName * sizeof(WCHAR));
	}
	else
		ccPathName = 0;
	for(int i = ccPathName, j = i; i <= ccWinFile; i++, j++)
	{
		if(szwWinFile[i] < m_eccStdAnsi)
		{
			pszwTomFile[j] = szwWinFile[i];
			continue;
		}
		wTom                = WinChar2TomChar(szwWinFile[i]);
		pszwTomFile[j + 3]  = wTom % sizeof(m_byTfnCode);
		pszwTomFile[j + 1]  = wTom / sizeof(m_byTfnCode);
		pszwTomFile[j + 2]  = pszwTomFile[j + 1] % sizeof(m_byTfnCode);
		pszwTomFile[j + 1] /= sizeof(m_byTfnCode);
		pszwTomFile[j]      = CHR_RCB;
		pszwTomFile[j + 1]  = m_byTfnCode[pszwTomFile[j + 1]];
		pszwTomFile[j + 2]  = m_byTfnCode[pszwTomFile[j + 2]];
		pszwTomFile[j + 3]  = m_byTfnCode[pszwTomFile[j + 3]];
		j += 3;
	}
#ifndef UNICODE
	j = WCStoMBS(pszwTomFile, j, pszBuffer, countof(szwWinFile));
#endif
	return (j - 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes records of cell phonebook

AFXINLINE UINT CTomPoiEdr::WriteCpbRecord(HFILE hFile)
{
	TRACE(_T("Entering CTomPoiEdr::WriteCpbRecord\n"));

	BOOL bWrite;
	int  ceLvItem, ceSelItem, iLvItem;
	UINT nResId;
	if(m_opf == OPF_WRITE)
	{
		bWrite    = TRUE;
		ceSelItem = m_jListView.GetItemCount();
	}
	else
	{
		bWrite    = FALSE;
		ceSelItem = m_jListView.GetSelCount();
	}
	iLvItem  = -1;
	ceLvItem = 0;
	while(bWrite ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		DisplayFileProgress(++ceLvItem, ceSelItem);
		nResId = WriteCpbRecord(hFile, iLvItem);
		if(nResId)
			return nResId;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes a record of cell phonebook

AFXINLINE UINT CTomPoiEdr::WriteCpbRecord(HFILE hFile, int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::WriteCpbRecord\n"));

	TCHAR  szTel[e_eccTelCol];
	int    ccTel;
	LVITEM lvi;
	lvi.iSubItem   = LVC_TEL;
	lvi.pszText    = szTel;
	lvi.cchTextMax = countof(szTel);
	ccTel          = m_jListView.GetItemText(iLvItem, &lvi);
	if(!ccTel)
		return 0;

	char   szCpbRd[(e_eccTextRd + 10) * sizeof(TCHAR)];
	TCHAR  szText[e_eccNameCol];
	PTCHAR pcDQM;
	int    i, j, ccText;
	*szCpbRd       = CHR_DQM;
	lvi.iSubItem   = LVC_NAME;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	ccText         = m_jListView.GetItemText(iLvItem, &lvi);
	pcDQM          = szText;
	while(pcDQM = StrChr(pcDQM, CHR_DQM))
	{
		*pcDQM = CHR_SQM;
		pcDQM++;
	}

#ifdef UNICODE
	for(i = 0; (i < ccText) && (szText[i] < m_eccStdAnsi); i++);
#else
	for(i = 0; (i < ccText) && !IsDBCSLeadByte(szText[i]); i++);
#endif
	if(i == ccText)
	{
#ifdef UNICODE
		for(j = 1; j <= ccText; j++)
			szCpbRd[j] = (char)szText[j - 1];
#else
		CopyMemory(szCpbRd + 1, szText, ccText);
		j = ccText + 1;
#endif
	}
	else
	{
		PWSTR pszwText;
#ifdef UNICODE
		pszwText = szText;
#else
		WCHAR szwText[countof(szText)];
		ccText   = MBStoWCS(szText, ccText, szwText, countof(szwText));
		pszwText = szwText;
#endif
		*(PWCHAR)(szCpbRd + 1) = FT_TOMTOM;
		for(i = 0, j = sizeof(WCHAR) + 1; i < ccText; i++, j += sizeof(WCHAR))
			*(PWCHAR)(szCpbRd + j) = WinChar2TomChar(pszwText[i]);
	}
	szCpbRd[j++] = CHR_DQM;
	szCpbRd[j++] = CHR_CA;
	szCpbRd[j++] = CHR_DQM;
#ifdef UNICODE
	for(i = 0; i < ccTel; i++, j++)
		szCpbRd[j] = (char)szTel[i];
#else
	CopyMemory(szCpbRd + j, szTel, ccTel);
	j           += ccTel;
#endif
	szCpbRd[j++] = CHR_DQM;
	szCpbRd[j++] = CHR_CA;
	szCpbRd[j++] = CHR_DQM;
	lvi.iSubItem = LVC_MEMO;
	if(m_jListView.GetItemText(iLvItem, &lvi))
	{
		TCHAR szCpbMode[32];
		i = IDS_CPBM_FIRST;
		do
		{
			LoadString(e_hInstance, i, szCpbMode, countof(szCpbMode));
			if(!lstrcmp(szText, szCpbMode))
				break;
			i++;
		}
		while(i <= IDS_CPBM_LAST);
		if(i > IDS_CPBM_LAST)
			i = IDS_CPBM_DEFAULT;
	}
	else
		i = IDS_CPBM_DEFAULT;
	j           += LoadStringA(e_hInstance, i - IDS_CPBM_FIRST + IDS_CPBT_FIRST,
		szCpbRd + j, e_eccMemoCol);
	szCpbRd[j++] = CHR_DQM;
	szCpbRd[j++] = CHR_CR;
	szCpbRd[j++] = CHR_LF;

	DWORD cbDone;
	return (WriteFile(hFile, szCpbRd, j, &cbDone) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes csv/asc records to POI file

UINT CTomPoiEdr::WriteCsvRecord(HFILE hFile)
{
	TRACE(_T("Entering CTomPoiEdr::WriteCsvRecord\n"));

	UINT nFilterIndex;
	nFilterIndex = (m_jFileDlg.nFilterIndex - FTW_CSVA) % (FTW_CSVU8 - FTW_CSVA + 1) + FTW_CSVA;
	if(nFilterIndex != FTW_CSVA)
	{
		PCBYTE pbyMark;
		DWORD  cbMark;
		if(nFilterIndex == FTW_CSVU)
		{
			pbyMark = m_byUnicodeMark;
			cbMark  = sizeof(m_byUnicodeMark);
		}
		else
		{
			pbyMark = m_byUtf8Mark;
			cbMark  = sizeof(m_byUtf8Mark);
		}
		if(!WriteFile(hFile, pbyMark, cbMark, &cbMark))
			return IDS_DISK_ERROR;
	}

	BOOL bWrite;
	int  ceLvItem, ceSelItem, iLvItem;
	UINT nResId;
	if(m_opf == OPF_WRITE)
	{
		bWrite    = TRUE;
		ceSelItem = m_jListView.GetItemCount();
		iLvItem   = m_bMyBP ? 0 : -1;
	}
	else
	{
		bWrite    = FALSE;
		ceSelItem = m_jListView.GetSelCount();
		iLvItem   = -1;
	}
	ceLvItem = 0;
	while(bWrite ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		DisplayFileProgress(++ceLvItem, ceSelItem);
		nResId = WriteCsvRecord(hFile, iLvItem);
		if(nResId)
			return nResId;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes a csv/asc record to POI file

AFXINLINE UINT CTomPoiEdr::WriteCsvRecord(HFILE hFile, int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::WriteCsvRecord\n"));

	SCINT  siColOrder[] = { LVC_LNG, LVC_LAT, LVC_NAME, LVC_MEMO, LVC_TEL };
	TCHAR  szText[e_eccAllCol * 2], szTemp[e_eccNameCol];
	PTCHAR pcTemp;
	BOOL   bAddDQM;
	int    i, j, ccText, iLvCol, iFileExt;
	UINT   nFilterIndex;
	LVITEM lvi;
	lvi.pszText    = szTemp;
	lvi.cchTextMax = countof(szTemp);
	ccText         = 0;
	i              = 0;
	nFilterIndex   = m_jFileDlg.nFilterIndex;
	if(nFilterIndex <= FTW_CSVU8)
		iFileExt = FTW_CSV;
	else if(nFilterIndex <= FTW_ASCU8)
	{
		iFileExt      = FTW_ASC;
		nFilterIndex -= FTW_ASCA - FTW_CSVA;
	}
	else
	{
		iFileExt      = FTW_POI;
		nFilterIndex -= FTW_POIA - FTW_CSVA;
	}
	do
	{
		iLvCol       = siColOrder[i];
		lvi.iSubItem = iLvCol;
		j            = m_jListView.GetItemText(iLvItem, &lvi);
		if(j)
		{
			if(GetCoordDdString(iLvCol, szTemp, j))
			{
				if((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT))
					bAddDQM = FALSE;
				else if(iFileExt == FTW_POI)
					bAddDQM = TRUE;
				else if(iLvCol == LVC_TEL)
					bAddDQM = FALSE;
				else if(iFileExt == FTW_ASC)
					bAddDQM = TRUE;
				else
					bAddDQM = (BOOL)StrPBrk(szTemp, _T(",\""));
				if(bAddDQM)
					szText[ccText++] = CHR_DQM;
				pcTemp = szTemp;
				while(*pcTemp)
				{
					if(*pcTemp == CHR_DQM)
						szText[ccText++] = CHR_DQM;
					szText[ccText++] = *pcTemp++;
				}
				if(bAddDQM)
					szText[ccText++] = CHR_DQM;
			}
		}
		szText[ccText++] = CHR_CA;
		i++;
	}
	while(i < countof(siColOrder));
	i = ccText;
	while(szText[ccText - 1] == CHR_CA)
		ccText--;
	if((i - ccText) == 2)
		ccText++;
	szText[ccText++] = CHR_CR;
	szText[ccText++] = CHR_LF;

	char  szData[countof(szText) * FT_UTF8S3];
#ifndef UNICODE
	WCHAR szwData[countof(szText)];
#endif
	PSTR  pszData;
	DWORD cbData;
	if(nFilterIndex == FTW_CSVA)
	{
#ifdef UNICODE
		pszData = szData;
		cbData  = WCStoMBS(szText, ccText, szData, sizeof(szData));
#else
		pszData = szText;
		cbData  = ccText;
#endif
	}
	else
	{
		if(nFilterIndex == FTW_CSVU)
		{
#ifdef UNICODE
			pszData = (PSTR)szText;
			cbData  = ccText * sizeof(WCHAR);
#else
			pszData = (PSTR)szwData;
			cbData  = MBStoWCS(szText, ccText, szwData, countof(szwData)) * sizeof(WCHAR);
#endif
		}
		else
		{
			pszData = szData;
#ifdef UNICODE
			cbData  = WCStoMBS(CP_UTF8, szText, ccText, szData, sizeof(szData));
#else
			cbData  = WCStoMBS(CP_UTF8, szwData,
				MBStoWCS(szText, ccText, szwData, countof(szwData)), szData, sizeof(szData));
#endif
		}
	}

	return (WriteFile(hFile, pszData, cbData, &cbData) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes gpx records to POI file

UINT CTomPoiEdr::WriteGpxRecord(HFILE hFile)
{
	TRACE(_T("Entering CTomPoiEdr::WriteGpxRecord\n"));

	char  szuText[e_eccNameCol * countof(m_szQuotEntity)];
	DWORD cbDone;
	if(!WriteFile(hFile, szuText, LoadStringA(e_hInstance, IDS_GPXF_HEADER,
		szuText, MAX_STR), &cbDone))
		return IDS_DISK_ERROR;

	char   szGpxWpt[64], szGpxName[32], szGpxCmt[32], szGpxDesc[32];
	char   szGpxWptEnd[32], szGpxEnd[32];
	char   szName[sizeof(szuText)], szLng[32], szLat[sizeof(szLng)];
	char   szTel[e_eccTelCol], szMemo[e_eccMemoCol * countof(m_szQuotEntity)];
	TCHAR  szText[sizeof(szuText)];
	BOOL   bWrite;
	int    ceLvItem, ceSelItem, iLvItem, ccText;
	LVITEM lvi;
#ifdef UNICODE
	int    i;
	lvi.pszText = szText;
#endif
	LoadStringA(e_hInstance, IDS_GPXF_WPT, szGpxWpt, sizeof(szGpxWpt));
	LoadStringA(e_hInstance, IDS_GPXF_NAME, szGpxName, sizeof(szGpxName));
	LoadStringA(e_hInstance, IDS_GPXF_CMT, szGpxCmt, sizeof(szGpxCmt));
	LoadStringA(e_hInstance, IDS_GPXF_DESC, szGpxDesc, sizeof(szGpxDesc));
	LoadStringA(e_hInstance, IDS_GPXF_WPT_END, szGpxWptEnd, sizeof(szGpxWptEnd));
	LoadStringA(e_hInstance, IDS_GPXF_END, szGpxEnd, sizeof(szGpxEnd));
	if(m_opf == OPF_WRITE)
	{
		bWrite    = TRUE;
		ceSelItem = m_jListView.GetItemCount();
		iLvItem   = m_bMyBP ? 0 : -1;
	}
	else
	{
		bWrite    = FALSE;
		ceSelItem = m_jListView.GetSelCount();
		iLvItem   = -1;
	}
	ceLvItem = 0;
	while(bWrite ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		DisplayFileProgress(++ceLvItem, ceSelItem);
		lvi.iSubItem   = LVC_LNG;
#ifndef UNICODE
		lvi.pszText    = szLng;
#endif
		lvi.cchTextMax = sizeof(szLng);
		ccText         = m_jListView.GetItemText(iLvItem, &lvi);
		if(!ccText)
			continue;
#ifdef UNICODE
		ccText = GetCoordDdString(szText, ccText);
		for(i = 0; i <= ccText; i++)
			szLng[i] = (char)szText[i];
#else
		GetCoordDdString(szLng, ccText);
#endif
		lvi.iSubItem = LVC_LAT;
#ifndef UNICODE
		lvi.pszText  = szLat;
#endif
		ccText       = m_jListView.GetItemText(iLvItem, &lvi);
		if(!ccText)
			continue;
#ifdef UNICODE
		ccText = GetCoordDdString(szText, ccText);
		for(i = 0; i <= ccText; i++)
			szLat[i] = (char)szText[i];
#else
		GetCoordDdString(szLat, ccText);
#endif
		lvi.iSubItem   = LVC_NAME;
#ifndef UNICODE
		lvi.pszText    = szText;
#endif
		lvi.cchTextMax = e_eccNameCol;
		m_jListView.GetItemText(iLvItem, &lvi);
		ccText = EncodeEntity(szText);
#ifdef UNICODE
		WCStoMBS(CP_UTF8, szText, ccText + 1, szName, sizeof(szName));
#else
		WCHAR szwText[countof(szText)];
		WCStoMBS(CP_UTF8, szwText, MBStoWCS(szText, ccText + 1,
			szwText, countof(szwText)), szName, sizeof(szName));
#endif
		lvi.iSubItem   = LVC_TEL;
#ifndef UNICODE
		lvi.pszText    = szTel;
#endif
		lvi.cchTextMax = sizeof(szTel);
		ccText         = m_jListView.GetItemText(iLvItem, &lvi);
#ifdef UNICODE
		for(i = 0; i <= ccText; i++)
			szTel[i] = (char)szText[i];
#endif
		lvi.iSubItem   = LVC_MEMO;
#ifndef UNICODE
		lvi.pszText    = szText;
#endif
		lvi.cchTextMax = e_eccMemoCol;
		m_jListView.GetItemText(iLvItem, &lvi);
		ccText = EncodeEntity(szText);
#ifdef UNICODE
		WCStoMBS(CP_UTF8, szText, ccText + 1, szMemo, sizeof(szMemo));
#else
		WCStoMBS(CP_UTF8, szwText, MBStoWCS(szText, ccText + 1,
			szwText, countof(szwText)), szMemo, sizeof(szMemo));
#endif
		if(!WriteFile(hFile, szuText, wsprintfA(szuText, szGpxWpt, szLat, szLng), &cbDone))
			return IDS_DISK_ERROR;
		if(!WriteFile(hFile, szuText, wsprintfA(szuText, szGpxName, szName), &cbDone))
			return IDS_DISK_ERROR;
		if(*szTel && !WriteFile(hFile, szuText, wsprintfA(szuText, szGpxCmt, szTel), &cbDone))
			return IDS_DISK_ERROR;
		if(*szMemo && !WriteFile(hFile, szuText,
			wsprintfA(szuText, szGpxDesc, szMemo), &cbDone))
			return IDS_DISK_ERROR;
		if(!WriteFile(hFile, szuText, wsprintfA(szuText, szGpxWptEnd), &cbDone))
			return IDS_DISK_ERROR;
	}
	return (WriteFile(hFile, szuText, wsprintfA(szuText, szGpxEnd), &cbDone) ?
		0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes kml records to POI file

UINT CTomPoiEdr::WriteKmlRecord(HFILE hFile, PINT pcePoint)
{
	TRACE(_T("Entering CTomPoiEdr::WriteKmlRecord\n"));

	char  szResText[MAX_STR];
	DWORD cbDone;
	if(!WriteFile(hFile, szResText, LoadStringA(e_hInstance, IDS_GOOGLE_EARTH_HEADER,
		szResText, sizeof(szResText)), &cbDone))
		return IDS_DISK_ERROR;

	char   szuText[e_eccAllCol * countof(m_szQuotEntity)];
	char   szName[e_eccNameCol * countof(m_szQuotEntity)], szLng[32], szLat[sizeof(szLng)];
	char   szTel[e_eccTelCol], szMemo[e_eccMemoCol * countof(m_szQuotEntity)];
	char   szDesc[32], szDescEnd[sizeof(szDesc)];
	TCHAR  szText[sizeof(szName)];
	PCSTR  pszDesc, pszDescEnd;
	BOOL   bSelItem;
	int    ceSelItem, ceWriteItem, ccText, iStopItem;
	LVITEM lvi;
#ifdef UNICODE
	int    i;
#endif
	ceSelItem = m_jListView.GetSelCount();
	bSelItem  = pcePoint ? ceSelItem : (m_opf == OPF_EXPORT);
	if(bSelItem)
	{
		lvi.iItem   = m_jListView.GetNextItem(LVNI_SELECTED);
		iStopItem   = RV_ERR;
		ceWriteItem = ceSelItem;
	}
	else
	{
		lvi.iItem   = m_bMyBP ? 1 : 0;
		iStopItem   = m_jListView.GetItemCount();
		ceWriteItem = iStopItem - lvi.iItem;
	}
#ifdef UNICODE
	lvi.pszText = szText;
#endif
	LoadStringA(e_hInstance, IDS_GOOGLE_EARTH_POINT, szResText, sizeof(szResText));
	LoadStringA(e_hInstance, IDS_KMLT_DESC, szDesc, sizeof(szDesc));
	LoadStringA(e_hInstance, IDS_KMLT_DESC_END, szDescEnd, sizeof(szDescEnd));
	for(int cePoint = 0; (lvi.iItem != iStopItem);
		DisplayFileProgress(cePoint, ceWriteItem), bSelItem ?
		(lvi.iItem = m_jListView.GetNextItem(LVNI_SELECTED, lvi.iItem)) : lvi.iItem++)
	{
		lvi.iSubItem   = LVC_LNG;
#ifndef UNICODE
		lvi.pszText    = szLng;
#endif
		lvi.cchTextMax = sizeof(szLng);
		ccText         = m_jListView.GetItemText(lvi.iItem, &lvi);
		if(!ccText)
		{
			ceWriteItem--;
			continue;
		}
#ifdef UNICODE
		ccText = GetCoordDdString(szText, ccText);
		for(i = 0; i <= ccText; i++)
			szLng[i] = (char)szText[i];
#else
		GetCoordDdString(szLng, ccText);
#endif
		lvi.iSubItem = LVC_LAT;
#ifndef UNICODE
		lvi.pszText  = szLat;
#endif
		ccText       = m_jListView.GetItemText(lvi.iItem, &lvi);
		if(!ccText)
		{
			ceWriteItem--;
			continue;
		}
#ifdef UNICODE
		ccText = GetCoordDdString(szText, ccText);
		for(i = 0; i <= ccText; i++)
			szLat[i] = (char)szText[i];
#else
		GetCoordDdString(szLat, ccText);
#endif
		lvi.iSubItem   = LVC_NAME;
#ifndef UNICODE
		lvi.pszText    = szText;
#endif
		lvi.cchTextMax = e_eccNameCol;
		m_jListView.GetItemText(lvi.iItem, &lvi);
		ccText = EncodeEntity(szText);
#ifdef UNICODE
		WCStoMBS(CP_UTF8, szText, ccText + 1, szName, sizeof(szName));
#else
		WCHAR szwText[countof(szText)];
		WCStoMBS(CP_UTF8, szwText, MBStoWCS(szText, ccText + 1,
			szwText, countof(szwText)), szName, sizeof(szName));
#endif
		lvi.iSubItem = LVC_MEMO;
		m_jListView.GetItemText(lvi.iItem, &lvi);
		ccText = EncodeEntity(szText);
#ifdef UNICODE
		WCStoMBS(CP_UTF8, szText, ccText + 1, szMemo, sizeof(szMemo));
#else
		WCStoMBS(CP_UTF8, szwText, MBStoWCS(szText, ccText + 1,
			szwText, countof(szwText)), szMemo, sizeof(szMemo));
#endif
		lvi.iSubItem   = LVC_TEL;
#ifndef UNICODE
		lvi.pszText    = szTel;
#endif
		lvi.cchTextMax = sizeof(szTel);
		ccText         = m_jListView.GetItemText(lvi.iItem, &lvi);
#ifdef UNICODE
		for(i = 0; i <= ccText; i++)
			szTel[i] = (char)szText[i];
#endif
		if(*szTel || *szMemo)
		{
			pszDesc    = szDesc;
			pszDescEnd = szDescEnd;
		}
		else
		{
			pszDesc    = (PCSTR)e_szNull;
			pszDescEnd = pszDesc;
		}
		if(!WriteFile(hFile, szuText,
			wsprintfA(szuText, szResText, szName, pszDesc, *szTel ? szTel : (PCSTR)e_szNull,
			*szMemo ? "\n" : (PCSTR)e_szNull, szMemo, pszDescEnd, szLng, szLat), &cbDone))
			return IDS_DISK_ERROR;
		cePoint++;
	}
	if(pcePoint)
		*pcePoint = cePoint;
	return (WriteFile(hFile, szResText, LoadStringA(e_hInstance, IDS_GOOGLE_EARTH_FOOTER,
		szResText, sizeof(szResText)), &cbDone) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes ov2 records to POI file

UINT CTomPoiEdr::WriteOv2Record(HFILE hFile)
{
	TRACE(_T("Entering CTomPoiEdr::WriteOv2Record\n"));

	SPOIRECORD64 spr64SM =
	{
		FRT_SIGNAL, sizeof(POIRECORD64),
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		m_eSmAddDelta,
		0x00,
		0x00,
		0x00, 0x00, 0x00, 0x00,
	};
	SPOIRECORD64 spr64SC =
	{
		FRT_SIGNAL, sizeof(POIRECORD64),
		0x56, 0x33, 0xA2, 0x0B, 0x01, 0x00, 0x00, 0x00, 0x01,
		0x00,
		0x00,
		0xB1,
		0x00, 0x00, 0x00, 0x00,
	};
	PPOIRECORD64 pPR64;
	UINT         (CTomPoiEdr::*WriteOv2Recordx)(PVOID, int, BOOL);
	int          i;
	DWORD        cbDone;
	if(m_fst == FST_SC)
	{
		pPR64           = CheckSignalMark(&m_pr64, m_eSmChkSumSC) ? &m_pr64 : &spr64SC;
		WriteOv2Recordx = &CTomPoiEdr::WriteOv2Record1E;
	}
	else
	{
		pPR64 = NULL;
		if(m_jOptionDlg.m_nSignalMark == BST_CHECKED)
		{
			if(CheckSignalMark(&m_pr64, m_eSmChkSum))
				pPR64 = &m_pr64;
			if(!pPR64)
			{
				UINT nSignalSum;
				srand(GetTickCount());
				pPR64 = &spr64SM;
				do
				{
					nSignalSum = m_eMaxSmSum;
					i          = 0;
					do
					{
						pPR64->bySignal[i] = rand(m_eMinSm, m_eMaxSm);
						nSignalSum        -= pPR64->bySignal[i];
						i++;
					}
					while(i < m_eceSmSum);
				}
				while(nSignalSum > (m_eMaxSmSubDelta - m_eMinSmSubDelta));
				pPR64->bySubDelta = m_eMaxSmSubDelta - nSignalSum;
				CopyMemory(&m_pr64, pPR64, sizeof(POIRECORD64));
			}
		}
		WriteOv2Recordx = m_bRPF ? &CTomPoiEdr::WriteOv2Record1D : &CTomPoiEdr::WriteOv2Record2;
	}
	if(pPR64 && !WriteFile(hFile, pPR64, sizeof(POIRECORD64), &cbDone))
		return IDS_DISK_ERROR;

	BOOL bWrite;
	int  ceSelItem, iLvItem;
	if(m_opf == OPF_WRITE)
	{
		bWrite    = TRUE;
		ceSelItem = m_jListView.GetItemCount();
		iLvItem   = m_bMyBP ? 0 : -1;
	}
	else
	{
		bWrite    = FALSE;
		ceSelItem = m_jListView.GetSelCount();
		iLvItem   = -1;
	}

	TCHAR  szCoord[32];
	int    ceLvItem;
	UINT   nResId;
	LVITEM lvi;
	ceLvItem = 0;
	if(m_jOptionDlg.m_nOptPoi == BST_CHECKED)
	{
		int iTemp;
		iTemp          = iLvItem;
		lvi.pszText    = szCoord;
		lvi.cchTextMax = 2;
		while(bWrite ? (++iTemp < ceSelItem) :
			((iTemp = m_jListView.GetNextItem(LVNI_SELECTED, iTemp)) != RV_ERR))
		{
			lvi.iSubItem = LVC_LNG;
			if(!m_jListView.GetItemText(iTemp, &lvi))
				break;
			lvi.iSubItem = LVC_LAT;
			if(!m_jListView.GetItemText(iTemp, &lvi))
				break;
		}
	}
	else
		*szCoord = CHR_NULL;
	if(!*szCoord)
	{
		while(bWrite ? (++iLvItem < ceSelItem) :
			((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
		{
			DisplayFileProgress(++ceLvItem, ceSelItem);
			nResId = (*this.*WriteOv2Recordx)(hFile, iLvItem, FALSE);
			if(nResId)
				return nResId;
		}
		return 0;
	}

	SCPOICOORDBLOCK spcb =
	{
		{ FRT_SKIPPER, sizeof(POIRECORD1), e_eMinLng, e_eMinLat, e_eMaxLng, e_eMaxLat },
		UNUSED, UNUSED, NULL, 0, FALSE,
	};
	PPOICOORDBLOCK pPCB;
	int            cePCB;
	LONG           lLng, lLat;
	pPCB           = NULL;
	cePCB          = 0;
	lvi.cchTextMax = countof(szCoord);
	while(bWrite ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		lvi.iSubItem = LVC_LNG;
		m_jListView.GetItemText(iLvItem, &lvi);
		lLng         = GetCoordValueL(szCoord);
		lvi.iSubItem = LVC_LAT;
		m_jListView.GetItemText(iLvItem, &lvi);
		lLat = GetCoordValueL(szCoord);
		for(i = 0; i < cePCB; i++)
			if((pPCB[i].lLng <= lLng) && (lLng < (pPCB[i].lLng + m_ecxCB)) &&
				(pPCB[i].lLat <= lLat) && (lLat < (pPCB[i].lLat + m_ecyCB)))
				break;
		if(i == cePCB)
		{
			if(!(cePCB % m_eceAllocCB))
				pPCB = (PPOICOORDBLOCK)realloc(pPCB,
					(cePCB + m_eceAllocCB) * sizeof(POICOORDBLOCK));
			CopyMemory(&pPCB[i], &spcb, sizeof(POICOORDBLOCK));
			pPCB[i].lLng = lLng / m_ecxCB * m_ecxCB;
			pPCB[i].lLat = lLat / m_ecyCB * m_ecyCB;
			cePCB++;
		}
		if(pPCB[i].pr1.lLngW < lLng)
			pPCB[i].pr1.lLngW = lLng;
		if(pPCB[i].pr1.lLatS < lLat)
			pPCB[i].pr1.lLatS = lLat;
		if(pPCB[i].pr1.lLngE > lLng)
			pPCB[i].pr1.lLngE = lLng;
		if(pPCB[i].pr1.lLatN > lLat)
			pPCB[i].pr1.lLatN = lLat;
		if(!(pPCB[i].ceLvItem % m_eceAllocBI))
			pPCB[i].piLvItem = (PINT)realloc(pPCB[i].piLvItem,
				(pPCB[i].ceLvItem + m_eceAllocBI) * sizeof(int));
		pPCB[i].piLvItem[pPCB[i].ceLvItem] = iLvItem;
		pPCB[i].ceLvItem++;
	}
	for(i = 0; i < cePCB; i++)
	{
		pPCB[i].pr1.lLngW++;
		pPCB[i].pr1.lLatS++;
		pPCB[i].pr1.lLngE--;
		pPCB[i].pr1.lLatN--;
	}

	PCOMBINECOORDBLOCK pCCB;
	BOOL               bInitCCB;
	int                ceCCB, ceLoop, j;
	RECT               rcDest, rcSrc1, rcSrc2;
	pCCB   = NULL;
	ceCCB  = 0;
	ceLoop = cePCB - 1;
	for(i = 0; i < ceLoop; i++)
	{
		if(pPCB[i].bCombined)
			continue;
		SetRect(&rcSrc1, pPCB[i].pr1.lLngE, pPCB[i].pr1.lLatN,
			pPCB[i].pr1.lLngW, pPCB[i].pr1.lLatS);
		InflateRect(&rcSrc1, m_ecxAB, m_ecyAB);
		bInitCCB = TRUE;
		for(j = i + 1; j < cePCB; j++)
		{
			if(pPCB[j].bCombined)
				continue;
			SetRect(&rcSrc2, pPCB[j].pr1.lLngE, pPCB[j].pr1.lLatN,
				pPCB[j].pr1.lLngW, pPCB[j].pr1.lLatS);
			InflateRect(&rcSrc2, m_ecxAB, m_ecyAB);
			if(!IntersectRect(&rcDest, &rcSrc1, &rcSrc2))
				continue;
			if(bInitCCB)
			{
				if(!(ceCCB % m_eceAllocCB))
					pCCB = (PCOMBINECOORDBLOCK)realloc(pCCB,
						(ceCCB + m_eceAllocCB) * sizeof(COMBINECOORDBLOCK));
				CopyMemory(&pCCB[ceCCB].pr1, &pPCB[i].pr1, sizeof(POIRECORD1));
				pCCB[ceCCB].piPCB = NULL;
				pCCB[ceCCB].cePCB = 0;
			}
			if(!(pCCB[ceCCB].cePCB % m_eceAllocBI))
				pCCB[ceCCB].piPCB = (PINT)realloc(pCCB[ceCCB].piPCB,
					(pCCB[ceCCB].cePCB + m_eceAllocBI) * sizeof(int));
			if(bInitCCB)
			{
				*pCCB[ceCCB].piPCB = i;
				pCCB[ceCCB].cePCB  = 1;
				pPCB[i].bCombined  = TRUE;
				bInitCCB           = FALSE;
			}
			CombineCoordBlock(&pCCB[ceCCB].pr1.lLngW, &pPCB[j].pr1.lLngW);
			pPCB[j].bCombined                    = TRUE;
			pCCB[ceCCB].piPCB[pCCB[ceCCB].cePCB] = j;
			pCCB[ceCCB].cePCB++;
			j = i;
		}
		if(!bInitCCB)
			ceCCB++;
	}

	PPOIRECORD1 pAllPR1, pCPR1, pPPR1;
	PPOIRECORD2 pPR2;
	int         ceBlockItem, k;
	DWORD       cbSize, cbRecord;
	pAllPR1 = (PPOIRECORD1)malloc((ceCCB + cePCB) * sizeof(POIRECORD1) +
		ceSelItem * e_eccTextRd * sizeof(WCHAR));
	CopyMemory(pAllPR1, &spcb.pr1, sizeof(POIRECORD1));
	pPR2 = (PPOIRECORD2)(pAllPR1 + 1);
	for(i = 0; i < ceCCB; i++)
	{
		if(pCCB[i].cePCB < m_eceMinAB)
		{
			ceBlockItem = 0;
			for(j = 0; j < pCCB[i].cePCB; j++)
				ceBlockItem += pPCB[pCCB[i].piPCB[j]].ceLvItem;
			if(ceBlockItem < m_eceMinBI)
			{
				for(j = 0; j < pCCB[i].cePCB; j++)
					pPCB[pCCB[i].piPCB[j]].bCombined = FALSE;
				continue;
			}
		}
		pCPR1 = (PPOIRECORD1)pPR2;
		pPR2  = (PPOIRECORD2)(pCPR1 + 1);
		for(j = 0; j < pCCB[i].cePCB; j++)
		{
			cbSize = 0;
			if(pPCB[pCCB[i].piPCB[j]].ceLvItem >= m_eceMinBI)
			{
				pPPR1 = (PPOIRECORD1)pPR2;
				pPR2  = (PPOIRECORD2)(pPPR1 + 1);
			}
			else
				pPPR1 = NULL;
			for(k = 0; k < pPCB[pCCB[i].piPCB[j]].ceLvItem; k++)
			{
				DisplayFileProgress(++ceLvItem, ceSelItem);
				cbRecord = (*this.*WriteOv2Recordx)
					(pPR2, pPCB[pCCB[i].piPCB[j]].piLvItem[k], TRUE);
				cbSize  += cbRecord;
				pPR2     = (PPOIRECORD2)((PBYTE)pPR2 + cbRecord);
			}
			if(pPPR1)
			{
				CopyMemory(pPPR1, &pPCB[pCCB[i].piPCB[j]].pr1, sizeof(POIRECORD1));
				pPPR1->cbSize      += cbSize;
				pCCB[i].pr1.cbSize += pPPR1->cbSize;
			}
			else
				pCCB[i].pr1.cbSize += cbSize;
		}
		CopyMemory(pCPR1, &pCCB[i].pr1, sizeof(POIRECORD1));
		CombineCoordBlock(&pAllPR1->lLngW, &pCPR1->lLngW);
		pAllPR1->cbSize += pCPR1->cbSize;
	}
	for(i = 0; i < cePCB; i++)
	{
		if(pPCB[i].bCombined)
			continue;
		cbSize = 0;
		if(pPCB[i].ceLvItem >= m_eceMinBI)
		{
			pPPR1 = (PPOIRECORD1)pPR2;
			pPR2  = (PPOIRECORD2)(pPPR1 + 1);
		}
		else
			pPPR1 = NULL;
		for(j = 0; j < pPCB[i].ceLvItem; j++)
		{
			DisplayFileProgress(++ceLvItem, ceSelItem);
			cbRecord = (*this.*WriteOv2Recordx)(pPR2, pPCB[i].piLvItem[j], TRUE);
			cbSize  += cbRecord;
			pPR2     = (PPOIRECORD2)((PBYTE)pPR2 + cbRecord);
		}
		if(pPPR1)
		{
			CopyMemory(pPPR1, &pPCB[i].pr1, sizeof(POIRECORD1));
			pPPR1->cbSize   += cbSize;
			pAllPR1->cbSize += pPPR1->cbSize;
		}
		else
			pAllPR1->cbSize += cbSize;
		CombineCoordBlock(&pAllPR1->lLngW, &pPCB[i].pr1.lLngW);
	}
	pPPR1 = pAllPR1 + 1;
	if((pPPR1->byType != FRT_SKIPPER) ||
		(pAllPR1->cbSize != (pPPR1->cbSize + sizeof(POIRECORD1))))
		pPPR1 = pAllPR1;
	nResId = WriteFile(hFile, pPPR1, pPPR1->cbSize, &cbDone) ? 0 : IDS_DISK_ERROR;
	free(pAllPR1);
	for(i = 0; i < ceCCB; i++)
		free(pCCB[i].piPCB);
	free(pCCB);
	for(i = 0; i < cePCB; i++)
		free(pPCB[i].piLvItem);
	free(pPCB);
	return nResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes an ov2 record (type 2) to POI file

UINT CTomPoiEdr::WriteOv2Record2(PVOID pvMemory, int iLvItem, BOOL bMemory)
{
	TRACE(_T("Entering CTomPoiEdr::WriteOv2Record2\n"));

	char        cPoiRd2[sizeof(POIRECORD2) + e_eccTextRd * sizeof(WCHAR)];
	PPOIRECORD2 pPR2;
	HFILE       hFile;
	if(bMemory)
		pPR2 = (PPOIRECORD2)pvMemory;
	else
	{
		hFile = (HFILE)pvMemory;
		pPR2  = (PPOIRECORD2)cPoiRd2;
	}

	TCHAR  szText[e_eccNameCol + e_eccTelCol];
	BOOL   bStdAnsi;
	int    i, ccText, ccTel;
	LVITEM lvi;
	pPR2->byType   = FRT_SIMPLE;
	lvi.iSubItem   = LVC_NAME;
	lvi.pszText    = szText;
	lvi.cchTextMax = e_eccNameCol;
	ccText         = m_jListView.GetItemText(iLvItem, &lvi);
#ifdef UNICODE
	for(i = 0; (i < ccText) && (szText[i] < m_eccStdAnsi); i++);
#else
	for(i = 0; (i < ccText) && !IsDBCSLeadByte(szText[i]); i++);
#endif
	bStdAnsi       = i == ccText;
	lvi.iSubItem   = LVC_TEL;
	lvi.pszText   += ccText;
	*lvi.pszText++ = FT_POI_SEP;
	lvi.cchTextMax = e_eccTelCol;
	ccTel          = m_jListView.GetItemText(iLvItem, &lvi);
	if(ccTel)
		ccText += ccTel + 1;
	if(bStdAnsi)
	{
#ifdef UNICODE
		for(i = 0; i < ccText; i++)
			pPR2->szName[i] = (char)szText[i];
#else
		CopyMemory(pPR2->szName, szText, ccText);
		i = ccText;
#endif
	}
	else
	{
		PWSTR pszwText, pszwName;
#ifdef UNICODE
		pszwText = szText;
#else
		WCHAR szwText[countof(szText)];
		ccText   = MBStoWCS(szText, ccText, szwText, countof(szwText));
		pszwText = szwText;
#endif
		pszwName  = (PWSTR)pPR2->szName;
		*pszwName = FT_TOMTOM;
		pszwName++;
		for(i = 0; i < ccText; i++)
			pszwName[i] = WinChar2TomChar(pszwText[i]);
		i = (i + 1) * sizeof(WCHAR);
	}
	pPR2->szName[i] = CHR_NULL;
	i++;

	TCHAR szCoord[32];
	lvi.iSubItem   = LVC_LNG;
	lvi.pszText    = szCoord;
	lvi.cchTextMax = countof(szCoord);
	pPR2->lLng     = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szCoord) : MIN_LONG;
	lvi.iSubItem   = LVC_LAT;
	pPR2->lLat     = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szCoord) : MIN_LONG;

	lvi.iSubItem   = LVC_MEMO;
#ifdef UNICODE
	lvi.pszText    = (PWSTR)(pPR2->szName + i);
#else
	lvi.pszText    = szText;
#endif
	lvi.cchTextMax = e_eccMemoCol;
	ccText         = m_jListView.GetItemText(iLvItem, &lvi);
	if(ccText)
	{
#ifdef UNICODE
		i += ccText * sizeof(WCHAR);
#else
		i += MBStoWCS(szText, ccText,
			(PWSTR)(pPR2->szName + i), e_eccMemoCol - 1) * sizeof(WCHAR);
#endif
	}
	pPR2->cbSize = i + (sizeof(POIRECORD2) - 1);
	if(bMemory)
		return pPR2->cbSize;

	DWORD cbDone;
	return (WriteFile(hFile, pPR2, pPR2->cbSize, &cbDone) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes an ov2 record (type 1D) to POI file

UINT CTomPoiEdr::WriteOv2Record1D(PVOID pvMemory, int iLvItem, BOOL bMemory)
{
	TRACE(_T("Entering CTomPoiEdr::WriteOv2Record1D\n"));

	char         cPoiRd1D[sizeof(POIRECORD1D) + (e_eccNameCol + e_eccTelCol) * sizeof(WCHAR)];
	PPOIRECORD1D pPR1D;
	HFILE        hFile;
	if(bMemory)
		pPR1D = (PPOIRECORD1D)pvMemory;
	else
	{
		hFile = (HFILE)pvMemory;
		pPR1D = (PPOIRECORD1D)cPoiRd1D;
	}

	PWSTR  pszwName;
	TCHAR  szText[e_eccNameCol + e_eccTelCol];
	int    ccText, ccTel, iInfoId;
	LVITEM lvi;
	pszwName       = (PWSTR)pPR1D->szName;
	lvi.iSubItem   = LVC_MEMO;
	lvi.pszText    = szText;
	lvi.cchTextMax = e_eccNameCol;
	if(m_jListView.GetItemText(iLvItem, &lvi))
	{
		pPR1D->byNET  = NET_TOMTOM_6;
		pPR1D->cbSize = m_eRpfNameDelta;
		iInfoId       = _ttoi(szText);
		ASSERT((0 <= iInfoId) && (iInfoId <= m_eceMaxRpfId));
		*(PWORD)pPR1D->byInfoId = LOWORD(iInfoId);
		pPR1D->byInfoId[2]      = LOBYTE(HIWORD(iInfoId));
	}
	else
	{
		pPR1D->byNET     = NET_TOMTOM_5;
		pPR1D->cbSize    = m_eRpfNameDelta - sizeof(WCHAR);
		*pPR1D->byInfoId = m_eRpfDefIconId;
		pszwName--;
	}
	pPR1D->byType  = FRT_RPF;
	lvi.iSubItem   = LVC_NAME;
	ccText         = m_jListView.GetItemText(iLvItem, &lvi);
	lvi.pszText   += ccText;
	*lvi.pszText++ = FT_POI_SEP;
	lvi.iSubItem   = LVC_TEL;
	lvi.cchTextMax = e_eccTelCol;
	ccTel          = m_jListView.GetItemText(iLvItem, &lvi);
	if(ccTel)
		ccText += ccTel + 1;

	PWSTR pszwText;
#ifdef UNICODE
	pszwText = szText;
#else
	WCHAR szwText[countof(szText)];
	ccText   = MBStoWCS(szText, ccText, szwText, countof(szwText));
	pszwText = szwText;
#endif
	if(ccText > (m_ecbMaxRpfName / sizeof(WCHAR)))
		ccText = m_ecbMaxRpfName / sizeof(WCHAR);
	for(int i = 0; i < ccText; i++)
		pszwName[i] = WinChar2TomChar(pszwText[i]);
	pPR1D->cbSize += ccText * sizeof(WCHAR);

	TCHAR szCoord[32];
	lvi.iSubItem   = LVC_LNG;
	lvi.pszText    = szCoord;
	lvi.cchTextMax = countof(szCoord);
	pPR1D->lLng    = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szCoord) : MIN_LONG;
	lvi.iSubItem   = LVC_LAT;
	pPR1D->lLat    = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szCoord) : MIN_LONG;

	DWORD cbRecord;
	cbRecord = pPR1D->cbSize + m_eRpfRdDelta;
	if(bMemory)
		return cbRecord;

	return (WriteFile(hFile, pPR1D, cbRecord, &cbRecord) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes an ov2 record (type 1E) to POI file

UINT CTomPoiEdr::WriteOv2Record1E(PVOID pvMemory, int iLvItem, BOOL bMemory)
{
	TRACE(_T("Entering CTomPoiEdr::WriteOv2Record1E\n"));

	TCHAR        szText[64], szCamera[countof(szText)];
	UINT         nCamera, nSpeed;
	PPOIRECORD1E pPR1E;
	LVITEM       lvi;
	POIRECORD1E  pr1e;
	HFILE        hFile;
	if(bMemory)
		pPR1E = (PPOIRECORD1E)pvMemory;
	else
	{
		hFile = (HFILE)pvMemory;
		pPR1E = &pr1e;
	}
	pPR1E->byType  = FRT_SC;
	lvi.iSubItem   = LVC_CAMERA;
	lvi.pszText    = szText;
	lvi.cchTextMax = countof(szText);
	if(m_jListView.GetItemText(iLvItem, &lvi))
	{
		nCamera = IDS_SCM_FIRST;
		do
		{
			LoadString(e_hInstance, nCamera, szCamera, countof(szCamera));
			if(!lstrcmp(szText, szCamera))
				break;
			nCamera++;
		}
		while(nCamera <= IDS_SCM_LAST);
		nCamera = (nCamera <= IDS_SCM_LAST) ?
			(nCamera - IDS_SCM_FIRST + FT_SCM_FIRST) : FT_SCM_OTHER;
	}
	else
		nCamera = FT_SCM_OTHER;
	pPR1E->byCamera = nCamera;
	lvi.iSubItem    = LVC_LNG;
	pPR1E->lLng     = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szText) : MIN_LONG;
	lvi.iSubItem    = LVC_LAT;
	pPR1E->lLat     = m_jListView.GetItemText(iLvItem, &lvi) ?
		GetCoordValueL(szText) : MIN_LONG;
	lvi.iSubItem    = LVC_SPEED;
	if(m_jListView.GetItemText(iLvItem, &lvi))
	{
		nSpeed = _ttoi(szText);
		if(nSpeed != e_eNonSpeed)
			nSpeed = clamp(nSpeed, e_eMinSpeed, e_eMaxSpeed) / e_eBaseSpeed;
	}
	else
		nSpeed = e_eDefSpeed / e_eBaseSpeed;
	pPR1E->bySpeed = nSpeed;
	if(bMemory)
		return sizeof(POIRECORD1E);

	DWORD cbDone;
	return (WriteFile(hFile, pPR1E, sizeof(pr1e), &cbDone) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes the POI file

AFXINLINE BOOL CTomPoiEdr::WritePoiFile()
{
	TRACE(_T("Entering CTomPoiEdr::WritePoiFile\n"));

	// Auto translates the file name
	TCHAR szSameFile[MAX_PATH];
	BOOL  bDiffName;
	if((m_jFileDlg.nFilterIndex == FTW_OV2) && (m_jOptionDlg.m_nTranName == BST_CHECKED))
	{
		TCHAR szFileName[MAX_PATH];
		int   ccFileName;
		ccFileName = WinFile2TomFile(m_jFileDlg.lpstrFile, szFileName);
		bDiffName  = lstrcmpi(m_jFileDlg.lpstrFile, szFileName);
		if(bDiffName)
		{
			lstrcpy(szSameFile, m_jFileDlg.lpstrFile);
			CopyMemory(m_jFileDlg.lpstrFile, szFileName, (ccFileName + 1) * sizeof(TCHAR));
		}
	}
	else
		bDiffName = FALSE;

	// Creates the file
	UINT  nResId;
	DWORD dwFileAttrs;
	HFILE hFile;
	nResId = IDS_CANNOT_WRITE;
	do
	{
		hFile = CreateFile(m_jFileDlg.lpstrFile, GENERIC_WRITE, CREATE_ALWAYS);
		if(hFile != INVALID_HANDLE)
		{
			nResId = 0;
			break;
		}

		dwFileAttrs = GetFileAttributes(m_jFileDlg.lpstrFile);
		if(!(dwFileAttrs & FILE_ATTRIBUTE_READONLY))
			break;

		if(DisplayMessage(IDS_WRITE_RETRY,
			MB_ICONWARNING | MB_OKCANCEL | MB_DEFBUTTON2) == ID_CANCEL)
			return FALSE;
	}
	while(SetFileAttributes(m_jFileDlg.lpstrFile, dwFileAttrs & ~FILE_ATTRIBUTE_READONLY));

	if(!nResId)
	{
		switch(m_jFileDlg.nFilterIndex)
		{
		case FTW_OV2:
			nResId = WriteOv2Record(hFile);
			break;
		case FTW_CPB:
			nResId = WriteCpbRecord(hFile);
			break;
		case FTW_KML:
			nResId = WriteKmlRecord(hFile, NULL);
			break;
		case FTW_GPX:
			nResId = WriteGpxRecord(hFile);
			break;
		case FTW_CSVA:
		case FTW_CSVU:
		case FTW_CSVU8:
		case FTW_ASCA:
		case FTW_ASCU:
		case FTW_ASCU8:
		case FTW_POIA:
		case FTW_POIU:
		case FTW_POIU8:
			nResId = WriteCsvRecord(hFile);
			break;
		default:
			nResId = WriteTxtRecord(hFile);
		}
		CloseHandle(hFile);
	}

	// Determines whether any error has occurred
	if(nResId)
	{
		DisplayMessage(nResId, MB_ICONWARNING | MB_OK);
		return FALSE;
	}

	DisplayFileProgress(1, 1);

	// Deletes the same file, if necessary
	if(bDiffName && IsValidFile(szSameFile))
	{
		dwFileAttrs = GetFileAttributes(szSameFile);
		if(dwFileAttrs & FILE_ATTRIBUTE_READONLY)
			SetFileAttributes(szSameFile, dwFileAttrs & ~FILE_ATTRIBUTE_READONLY);
		DeleteFile(szSameFile);
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes txt records to POI file

UINT CTomPoiEdr::WriteTxtRecord(HFILE hFile)
{
	TRACE(_T("Entering CTomPoiEdr::WriteTxtRecord\n"));

	if(m_jFileDlg.nFilterIndex != FTW_TXTA)
	{
		PCBYTE pbyMark;
		DWORD  cbMark;
		if(m_jFileDlg.nFilterIndex == FTW_TXTU)
		{
			pbyMark = m_byUnicodeMark;
			cbMark  = sizeof(m_byUnicodeMark);
		}
		else
		{
			pbyMark = m_byUtf8Mark;
			cbMark  = sizeof(m_byUtf8Mark);
		}
		if(!WriteFile(hFile, pbyMark, cbMark, &cbMark))
			return IDS_DISK_ERROR;
	}

	BOOL bWrite;
	int  ceLvItem, ceSelItem, iLvItem;
	UINT nResId;
	if(m_opf == OPF_WRITE)
	{
		bWrite    = TRUE;
		ceSelItem = m_jListView.GetItemCount();
		iLvItem   = m_bMyBP ? 0 : -1;
	}
	else
	{
		bWrite    = FALSE;
		ceSelItem = m_jListView.GetSelCount();
		iLvItem   = -1;
	}
	ceLvItem = 0;
	while(bWrite ? (++iLvItem < ceSelItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		DisplayFileProgress(++ceLvItem, ceSelItem);
		nResId = WriteTxtRecord(hFile, iLvItem);
		if(nResId)
			return nResId;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Writes a txt record to POI file

AFXINLINE UINT CTomPoiEdr::WriteTxtRecord(HFILE hFile, int iLvItem)
{
	TRACE(_T("Entering CTomPoiEdr::WriteTxtRecord\n"));

	TCHAR  szText[e_eccAllCol + 6];
	int    i, j;
	LVITEM lvi;
	lvi.cchTextMax = e_eccNameCol;
	i              = LVCR_NAME;
	j              = 0;
	do
	{
		lvi.iSubItem = e_iHdOrder[i];
		lvi.pszText  = szText + j;
		j           += GetCoordDdString(lvi.iSubItem, lvi.pszText,
			m_jListView.GetItemText(iLvItem, &lvi));
		szText[j]    = CHR_TAB;
		j++;
		i++;
	}
	while(i < LVCR_COUNT);
	while(szText[j - 1] == CHR_TAB)
		j--;
	szText[j++] = CHR_CR;
	szText[j++] = CHR_LF;

	char  szData[countof(szText) * FT_UTF8S3];
#ifndef UNICODE
	WCHAR szwData[countof(szText)];
#endif
	PSTR  pszData;
	DWORD cbData;
	if(m_jFileDlg.nFilterIndex == FTW_TXTA)
	{
#ifdef UNICODE
		pszData = szData;
		cbData  = WCStoMBS(szText, j, szData, sizeof(szData));
#else
		pszData = szText;
		cbData  = j;
#endif
	}
	else
	{
		if(m_jFileDlg.nFilterIndex == FTW_TXTU)
		{
#ifdef UNICODE
			pszData = (PSTR)szText;
			cbData  = j * sizeof(WCHAR);
#else
			pszData = (PSTR)szwData;
			cbData  = MBStoWCS(szText, j, szwData, countof(szwData)) * sizeof(WCHAR);
#endif
		}
		else
		{
			pszData = szData;
#ifdef UNICODE
			cbData  = WCStoMBS(CP_UTF8, szText, j, szData, sizeof(szData));
#else
			cbData  = WCStoMBS(CP_UTF8, szwData,
				MBStoWCS(szText, j, szwData, countof(szwData)), szData, sizeof(szData));
#endif
		}
	}

	return (WriteFile(hFile, pszData, cbData, &cbData) ? 0 : IDS_DISK_ERROR);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item strings

int CALLBACK CTomPoiEdr::CompareLvItem(int iItem1, int iItem2, LPARAM lSortParam)
{
	TRACE(_T("Entering CTomPoiEdr::CompareLvItem\n"));

	if(e_pTPE->m_bMyBP && (!iItem1 || !iItem2))
		return (iItem1 ? 1 : -1);

	TCHAR  szText1[e_eccNameCol], szText2[countof(szText1)];
	int    iLvCol, iResult;
	LVITEM lvi;
	iLvCol         = e_pTPE->m_iLvCol;
	lvi.iSubItem   = iLvCol;
	lvi.pszText    = szText1;
	lvi.cchTextMax = countof(szText1);
	e_pTPE->m_jListView.GetItemText(iItem1, &lvi);
	lvi.pszText = szText2;
	e_pTPE->m_jListView.GetItemText(iItem2, &lvi);
	if((iLvCol == LVC_LNG) || (iLvCol == LVC_LAT))
	{
		if(!*szText1)
			iResult = -1;
		else if(!*szText2)
			iResult = 1;
		else
		{
			double dTemp;
			dTemp   = GetCoordValueD(szText1) - GetCoordValueD(szText2);
			iResult = (dTemp < 0) ? -1 : ((dTemp > 0) ? 1 : 0);
		}
	}
	else
	{
#ifdef UNICODE
		iResult = StrCmpLogicalW(szText1, szText2);
		if(!iResult)
			iResult = lstrcmp(szText1, szText2);
#else
		iResult = lstrcmp(szText1, szText2);
#endif
	}
	return (e_pTPE->m_bSortAscend ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Compares the specified item parameters

int CALLBACK CTomPoiEdr::CompareLvParam(LPARAM lParam1, LPARAM lParam2, LPARAM lSortParam)
{
	TRACE(_T("Entering CTomPoiEdr::CompareLvParam\n"));

	if(e_pTPE->m_bMyBP && ((lParam1 == MIN_LPARAM) || (lParam2 == MIN_LPARAM)))
		return ((lParam1 == MIN_LPARAM) ? -1 : 1);

	if(lParam1 == lParam2)
		return 0;

	int iResult;
	iResult = (lParam1 > lParam2) ? 1 : -1;
	return (e_pTPE->m_bSortAscend ? iResult : -iResult);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A list view is about to be drawn

UINT CALLBACK CTomPoiEdr::CustomDrawListView(PNMLVCUSTOMDRAW pNMLVCD)
{
	TRACE(_T("Entering CTomPoiEdr::CustomDrawListView\n"));

	PCNMHDR       pNMH;
	PNMCUSTOMDRAW pNMCD;
	UINT          nLvCtrlId;
	pNMCD     = &pNMLVCD->nmcd;
	pNMH      = &pNMCD->hdr;
	nLvCtrlId = pNMH->idFrom;
	switch(nLvCtrlId)
	{
	case IDW_LIST_VIEW:
		if(!e_pTPE->m_jListView.m_bRedraw)
			return CDRF_SKIPDEFAULT;
		break;
	case IDC_CHKDATA_LV:
		if(!e_pTPE->m_jCheckDataDlg.m_jListView.m_bRedraw)
			return CDRF_SKIPDEFAULT;
		break;
	case IDC_BP_LV:
		CMyBPDlg* pMBPD;
		GetObjectPtr(::GetParent(pNMH->hwndFrom), &pMBPD);
		if(!pMBPD->m_jListView.m_bRedraw)
			return CDRF_SKIPDEFAULT;
#ifdef DEBUG
		break;
	default:
		ASSERT((nLvCtrlId == IDC_GPS_OPEN_LV) || (nLvCtrlId == IDC_GPS_DUMP_LV));
#endif
	}

	DWORD dwDrawStage;
	dwDrawStage = pNMCD->dwDrawStage;
	if(dwDrawStage == CDDS_PREPAINT)
		return CDRF_NOTIFYITEMDRAW;
	if(dwDrawStage == CDDS_ITEMPREPAINT)
	{
		pNMCD->uItemState &= ~CDIS_FOCUS;
		return CDRF_NOTIFYSUBITEMDRAW;
	}

	SBOOL sbShowDist;
	int   iLvCol;
	UINT  nItemRect;
	iLvCol = pNMLVCD->iSubItem;
	if(dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
	{
		if((iLvCol == LVC_DIST) && (nLvCtrlId != IDC_GPS_OPEN_LV))
		{
			ASSERT((nLvCtrlId == IDW_LIST_VIEW) || (nLvCtrlId == IDC_CHKDATA_LV));
			nItemRect  = LVIR_LABEL;
			sbShowDist = (nLvCtrlId == IDW_LIST_VIEW) ?
				(e_pTPE->m_jOptionDlg.m_nShowDist == BST_CHECKED) :
				(e_pTPE->m_jCheckDataDlg.m_iCheckType == CCheckDataDlg::m_eNearDist);
		}
		else
		{
			if(iLvCol != LVC_NAME)
				return CDRF_DODEFAULT;
			if(nLvCtrlId != IDC_GPS_OPEN_LV)
			{
				if(e_dwWinVer > WV_XP)
					return CDRF_DODEFAULT;
				if((nLvCtrlId != IDC_BP_LV) && (nLvCtrlId != IDC_GPS_DUMP_LV))
					return CDRF_NOTIFYPOSTPAINT;
			}
			sbShowDist = FALSE;
		}
	}
	else
	{
		ASSERT((dwDrawStage == (CDDS_ITEMPOSTPAINT | CDDS_SUBITEM)) && (iLvCol == LVC_NAME));
		nItemRect = LVIR_ICON;
	}

	TCHAR     szText[32];
	BOOL      bSelItem;
	int       iLvItem;
	LPARAM    lParam;
	COLORREF  crText, crBkgnd;
	LVITEM    lvi;
	CListView jListView;
	jListView   = pNMH->hwndFrom;
	iLvItem     = pNMCD->dwItemSpec;
	lParam      = pNMCD->lItemlParam;
	lvi.pszText = szText;
	bSelItem    = jListView.GetItemState(iLvItem, LVIS_SELECTED);
	if(nLvCtrlId == IDC_CHKDATA_LV)
		lParam = ((CCheckDataDlg::PLVITEMDATA)lParam)->lParam;
	if(bSelItem && (e_dwWinVer <= WV_XP))
	{
		int iTextColor, iBkgndColor;
		if(GetFocus() == jListView)
		{
			iTextColor  = COLOR_HIGHLIGHTTEXT;
			iBkgndColor = COLOR_HIGHLIGHT;
		}
		else
		{
			if((nLvCtrlId == IDW_LIST_VIEW) && e_pTPE->m_bLvBkgnd)
			{
				iTextColor  = COLOR_WINDOWTEXT;
				iBkgndColor = COLOR_3DFACE;
			}
			else
			{
				iTextColor  = COLOR_INACTIVECAPTIONTEXT;
				iBkgndColor = COLOR_INACTIVECAPTION;
			}
		}
		crText  = GetSysColor(iTextColor);
		crBkgnd = GetSysColor(iBkgndColor);
	}
	else
	{
		if((sbShowDist || (nLvCtrlId == IDC_GPS_OPEN_LV)) && (lParam == MIN_LPARAM))
			crText = RGB_BLUE;
		else
		{
			crText = jListView.GetTextColor();
			if(nLvCtrlId == IDW_LIST_VIEW)
			{
				lvi.iSubItem   = (e_pTPE->m_fst != FST_CPB) ? LVC_LNG : LVC_TEL;
				lvi.cchTextMax = 2;
				if(jListView.GetItemText(iLvItem, &lvi))
				{
					if(e_pTPE->m_fst != FST_CPB)
					{
						lvi.iSubItem = LVC_LAT;
						if(!jListView.GetItemText(iLvItem, &lvi))
							crText = RGB_RED;
					}
				}
				else
					crText = RGB_RED;
			}
		}
		if(e_dwWinVer <= WV_XP)
		{
			crBkgnd = jListView.GetTextBkColor();
			if(e_pTPE->m_jOptionDlg.m_nShadeTable == BST_CHECKED)
			{
				int iRValue, iGValue, iBValue;
				iRValue = GetRValue(crBkgnd);
				iGValue = GetGValue(crBkgnd);
				iBValue = GetBValue(crBkgnd);
				crBkgnd = (iLvItem % 2) ? RGB(iRValue, iGValue, max(0, iBValue - 8)) :
					RGB(max(0, iRValue - 8), max(0, iGValue - 8), iBValue);
			}
		}
	}
	pNMLVCD->clrText = crText;
	if(e_dwWinVer <= WV_XP)
		pNMLVCD->clrTextBk = crBkgnd;
	if(iLvCol == LVC_DIST)
	{
		if(!sbShowDist)
			return CDRF_SKIPDEFAULT;
	}
	else if(dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
	{
		ASSERT(((nLvCtrlId == IDC_BP_LV) || (nLvCtrlId == IDC_GPS_OPEN_LV) ||
			(nLvCtrlId == IDC_GPS_DUMP_LV)) && (iLvCol == LVC_NAME));
		return CDRF_NOTIFYPOSTPAINT;
	}

	RECT rc;
	HDC  hDC;
	hDC = pNMCD->hdc;
	jListView.GetItemRect(iLvItem, iLvCol, nItemRect, &rc);
	if((e_dwWinVer <= WV_XP) || (nLvCtrlId == IDC_GPS_OPEN_LV))
	{
		if((nLvCtrlId != IDW_LIST_VIEW) || !e_pTPE->m_bLvBkgnd || bSelItem)
		{
			RECT rcFill;
			CopyRect(&rcFill, &rc);
			if(e_dwWinVer <= WV_XP)
			{
				HBRUSH hBrush;
				hBrush = CreateSolidBrush(crBkgnd);
				if(iLvCol == LVC_NAME)
				{
					rcFill.left -= m_exLvNameAdj;
					if(sbShowDist)
						rcFill.left++;
				}
				rcFill.bottom--;
				FillRect(hDC, &rcFill, hBrush);
				DeleteBrush(hBrush);
			}
			if(nLvCtrlId == IDC_GPS_OPEN_LV)
			{
				COpenDeviceDlg* pODD;
				ASSERT(iLvCol == LVC_NAME);
				lvi.mask     = LVIF_IMAGE;
				lvi.iItem    = iLvItem;
				lvi.iSubItem = 0;
				jListView.GetItem(&lvi);
				GetObjectPtr(jListView.GetParent(), &pODD);
				pODD->m_jLvItemIL.Draw(lvi.iImage, hDC, rc.left + 1,
					(rcFill.top + rcFill.bottom - COpenDeviceDlg::m_ecyPoiImg) / 2,
					ILD_TRANSPARENT);
			}
		}
	}
	if(iLvCol == LVC_NAME)
		return CDRF_DODEFAULT;

	int  iLvDistAdjL, iLvDistAdjR, iBkMode;
	SIZE size;
	ASSERT(iLvCol == LVC_DIST);
	if(e_dwWinVer > WV_XP)
	{
		iLvDistAdjL = m_exLvDistAdjL;
		iLvDistAdjR = m_exLvDistAdjR;
	}
	else
	{
		iLvDistAdjL = m_exLvDistAdjLXP;
		iLvDistAdjR = m_exLvDistAdjRXP;
	}
	if(lParam == MIN_LPARAM)
	{
		TCHAR   cBasePoint;
		PCTSTR  pszFaceName;
		LOGFONT lf;
		HFONT   hFont;
		GetTextExtent(hDC, (e_pTPE->m_jOptionDlg.m_nMetricUnit == BST_CHECKED) ?
			e_pszDistKm : e_pszDistMi, 2, &size);
		rc.left = max(rc.left + iLvDistAdjL, rc.right - size.cx - iLvDistAdjR);
		GetObject(jListView.GetWindowFont(), &lf);
		if((nLvCtrlId == IDW_LIST_VIEW) && e_pTPE->m_bMyBP)
		{
			pszFaceName = _T("Wingdings");
			cBasePoint  = _T('\xAB');
			lf.lfHeight = (LONG)(lf.lfHeight * 1.2);
		}
		else
		{
			pszFaceName = _T("Symbol");
			cBasePoint  = _T('\xC5');
		}
		lstrcpy(lf.lfFaceName, pszFaceName);
		hFont   = SelectFont(hDC, CreateFont(&lf));
		crText  = SetTextColor(hDC, crText);
		iBkMode = SetBkMode(hDC, TRANSPARENT);
		DrawText(hDC, &cBasePoint, 1, &rc, DT_LEFT | DT_SINGLELINE | DT_VCENTER);
		DeleteFont(SelectFont(hDC, hFont));
	}
	else
	{
		int ccText;
		lvi.iSubItem   = LVC_DIST;
		lvi.cchTextMax = countof(szText);
		ccText         = jListView.GetItemText(iLvItem, &lvi);
		if(!ccText)
			return CDRF_SKIPDEFAULT;

		int cxDelta;
		if(StrChr(szText, CHR_DOT))
			cxDelta = 0;
		else
		{
			PCTSTR pszLargeUnit, pszSmallUnit;
			int    ccSmallUnit;
			if(e_pTPE->m_jOptionDlg.m_nMetricUnit == BST_CHECKED)
			{
				pszLargeUnit = e_pszDistKm;
				pszSmallUnit = e_pszDistM;
				ccSmallUnit  = 1;
			}
			else
			{
				pszLargeUnit = e_pszDistMi;
				pszSmallUnit = e_pszDistFt;
				ccSmallUnit  = 2;
			}
			GetTextExtent(hDC, pszLargeUnit, 2, &size);
			cxDelta = size.cx;
			GetTextExtent(hDC, pszSmallUnit, ccSmallUnit, &size);
			cxDelta -= size.cx;

			PCTCHAR pcSrcUnit;
			PTCHAR  pcDestUnit;
			pcDestUnit = StrChr(szText, CHR_SP) + 2;
			pcSrcUnit  = pcDestUnit;
			do
				pcSrcUnit++;
			while(*pcSrcUnit == CHR_SP);
			do
			{
				*pcDestUnit = *pcSrcUnit;
				pcDestUnit++;
				pcSrcUnit++;
			}
			while(*pcSrcUnit);
			*pcDestUnit = CHR_NULL;
			ccText      = pcDestUnit - szText;
		}

		int  cxDistRect;
		UINT nTextFmt;
		rc.left   += iLvDistAdjL;
		cxDistRect = GetRectWidth(&rc) - iLvDistAdjR;
		GetTextExtent(hDC, szText, ccText, &size);
		if(cxDistRect > (size.cx + cxDelta))
		{
			rc.right -= cxDelta + iLvDistAdjR;
			nTextFmt  = DT_RIGHT | DT_SINGLELINE | DT_VCENTER;
		}
		else
		{
			SCTCHAR szRetainText[] = { _T('7'), CHR_DOT, CHR_DOT, CHR_DOT };
			GetTextExtent(hDC, szRetainText, countof(szRetainText), &size);
			rc.right -= clamp(cxDistRect - size.cx, 0, iLvDistAdjR);
			nTextFmt  = DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER;
		}
		crText  = SetTextColor(hDC, crText);
		iBkMode = SetBkMode(hDC, TRANSPARENT);
		DrawText(hDC, szText, ccText, &rc, nTextFmt);
	}
	SetBkMode(hDC, iBkMode);
	SetTextColor(hDC, crText);

	// Result value of the message processed
	return CDRF_SKIPDEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sends DELETE requests to GPS device to delete screenshots

DWORD CALLBACK CTomPoiEdr::GpsDeleteDumpFile(PVOID pvParam)
{
	TRACE(_T("Entering CTomPoiEdr::GpsDeleteDumpFile\n"));

	PTSTR pszNextPos;
	TCHAR szText[MAX_STR];
	HHTTP hHttp;
	pszNextPos  = szText;
	pszNextPos += LoadString(e_hInstance, IDS_GPS_HTTP_PREFIX, pszNextPos, countof(szText));
	pszNextPos += LoadString(e_hInstance, IDS_GPS_SITE_IP, pszNextPos, countof(szText));
	pszNextPos += LoadString(e_hInstance, IDS_GPS_DUMP_PATH3, pszNextPos, countof(szText));
	*pszNextPos = CHR_SLH;
	pszNextPos++;
	LoadString(e_hInstance, IDS_GPS_DELETE_DUMP, pszNextPos, countof(szText));
	hHttp = InternetOpenUrl((HINTERNET)pvParam, szText, INTERNET_FLAG_DONT_CACHE);
	if(!hHttp)
		return IDS_GPS_CONNECT_ERROR;
	InternetCloseHandle(hHttp);
	Sleep(m_eGpsSleepIntl);
	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sends DELETE requests to GPS device to delete POI files

DWORD CALLBACK CTomPoiEdr::GpsDeletePoiFile(PVOID pvParam)
{
	TRACE(_T("Entering CTomPoiEdr::GpsDeletePoiFile\n"));

	PGPSDELETE pGPSD;
	TCHAR      szText[64];
	HHTTP      hHttp;
	pGPSD = (PGPSDELETE)pvParam;
	LoadString(e_hInstance, IDS_GPS_SITE_IP, szText , countof(szText));
	hHttp = InternetConnect(pGPSD->hInternet, szText,
		INTERNET_DEFAULT_HTTP_PORT, INTERNET_SERVICE_HTTP);
	if(!hHttp)
		return IDS_GPS_CONNECT_ERROR;

	PPGPSFILE ppGPSF;
	PCTSTR    pszName;
	PTSTR     pszFileExt;
	TCHAR     szPoiFile[MAX_STR + MAX_FNAME * FT_UTF8S3 * 3];
	int       i;
	DWORD     dwResId;
	HHTTP     hReqHttp;
	LoadString(e_hInstance, IDS_GPS_DELETE_VERB, szText, countof(szText));
	pszFileExt  = szPoiFile;
	pszFileExt += LoadString(e_hInstance, IDS_GPS_POI_PATH, szPoiFile, MAX_STR);
	*pszFileExt = CHR_SLH;
	pszFileExt++;
	pszName = pGPSD->pszName;
	lstrcpy(pszFileExt, pszName);
	pszFileExt += e_pTPE->EncodeUrl(pszFileExt);
	*pszFileExt = CHR_DOT;
	pszFileExt++;
	ppGPSF  = pGPSD->pGPSFL->pGPSF;
	i       = GPSF_FIRST;
	dwResId = UNUSED;
	do
	{
		if(GpsFindFile(ppGPSF[i], pszName))
		{
			CopyMemory(pszFileExt, m_pszGPSF[i], sizeof(m_szOv2Ext));
			hReqHttp = HttpOpenRequest(hHttp, szText, szPoiFile);
			if(!hReqHttp)
			{
				dwResId = IDS_GPS_CONNECT_ERROR;
				break;
			}
			if(!HttpSendRequest(hReqHttp, NULL, UNUSED, NULL, UNUSED))
				dwResId = IDS_GPS_DELETE_ERROR;
			InternetCloseHandle(hReqHttp);
			if(dwResId)
				break;
		}
		i++;
	}
	while(i < GPSF_COUNT);
	InternetCloseHandle(hHttp);
	return dwResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a GPS device has been added/removed to/from the system

DWORD CALLBACK CTomPoiEdr::GpsDeviceChange(PVOID pvParam)
{
	TRACE(_T("Entering CTomPoiEdr::GpsDeviceChange\n"));

	BOOL      bGpsDevice, bPrevGpsDevice;
	HINTERNET hInternet;
	bGpsDevice     = FALSE;
	bPrevGpsDevice = e_pTPE->m_bGpsDevice;
	hInternet      = GpsInternetOpen();
	if(hInternet)
	{
		PTSTR pszPath;
		TCHAR szText[MAX_STR];
		DWORD dwTimeout;
		HHTTP hHttp;
		dwTimeout = bPrevGpsDevice ? 0 : (GetTickCount() + m_eGpsLoopIntl);
		pszPath   = szText + LoadString(e_hInstance, IDS_GPS_HTTP_PREFIX,
			szText, countof(szText));
		pszPath  += LoadString(e_hInstance, IDS_GPS_SITE_IP, pszPath, countof(szText));
		LoadString(e_hInstance, IDS_GPS_POI_PATH, pszPath, countof(szText));
		do
		{
			hHttp = InternetOpenUrl(hInternet, szText, INTERNET_FLAG_DONT_CACHE);
			if(hHttp)
			{
				InternetCloseHandle(hHttp);
				bGpsDevice = TRUE;
				break;
			}
			Sleep(m_eGpsSleepIntl);
		}
		while(GetTickCount() < dwTimeout);
	}
	e_pTPE->m_bGpsDevice = bGpsDevice;
	if(bGpsDevice)
		e_pTPE->GpsVerifyDump(hInternet);
	if(hInternet)
		InternetCloseHandle(hInternet);
	if(e_pTPE->GpsCloseDialog(bPrevGpsDevice))
		e_pTPE->GpsBroadcastChange(bGpsDevice);
	CloseHandle(e_pTPE->m_hGpsDevThread);
	e_pTPE->m_hGpsDevThread = NULL;
	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Sends a POST request to GPS device to post POI files

DWORD CALLBACK CTomPoiEdr::GpsPostPoiFile(PVOID pvParam)
{
	TRACE(_T("Entering CTomPoiEdr::GpsPostPoiFile\n"));

	PGPSPOST pGPSP;
	TCHAR    szText[MAX_STR];
	HHTTP    hHttp;
	pGPSP = (PGPSPOST)pvParam;
	LoadString(e_hInstance, IDS_GPS_SITE_IP, szText, countof(szText));
	hHttp = InternetConnect(pGPSP->hInternet, szText,
		INTERNET_DEFAULT_HTTP_PORT, INTERNET_SERVICE_HTTP);
	if(!hHttp)
		return IDS_GPS_CONNECT_ERROR;

	TCHAR szPoiPath[64];
	DWORD dwResId;
	HHTTP hReqHttp;
	LoadString(e_hInstance, IDS_GPS_POST_VERB, szText, countof(szText));
	LoadString(e_hInstance, IDS_GPS_POI_PATH, szPoiPath, countof(szPoiPath));
	hReqHttp = HttpOpenRequest(hHttp, szText, szPoiPath);
	if(hReqHttp)
	{
		SCCHAR szGPSF[GPSF_COUNT][3] =
		{
			'o', 'v', '2',
			'b', 'm', 'p',
			'o', 'g', 'g',
			'r', 'p', 'f',
			'v', 'i', 'f',
		};
		PCHAR pcPayload, pcNextPos;
		PTSTR pszFileExt;
		char  szName[MAX_FNAME * FT_UTF8S3], szPayload1[MAX_STR];
		char  szPayload2[sizeof(szPayload1)], szPayload3[sizeof(szPayload1)];
		TCHAR szFileName[MAX_PATH];
		int   i, cbFile, cbName, cbPayload, cbPayload1, cbPayload2, cbPayload3;
		UINT  nSaveFlags;
		DWORD cbRead;
		HFILE hFile;
		lstrcpy(szFileName, pGPSP->pszFileName);
		pszFileExt = PathFindExtension(szFileName);
		if(!*pszFileExt)
			*pszFileExt = CHR_DOT;
		pszFileExt++;
#ifdef UNICODE
		cbName = WCStoMBS(CP_UTF8, pGPSP->pszName, szName, sizeof(szName));
#else
		WCHAR szwName[MAX_FNAME];
		cbName = WCStoMBS(CP_UTF8, szwName, MBStoWCS(pGPSP->pszName,
			szwName, countof(szwName)), szName, sizeof(szName));
#endif
		cbName--;
		cbPayload1   = LoadStringA(e_hInstance, IDS_GPS_POST_PAYLOAD1,
			szPayload1, sizeof(szPayload1));
		cbPayload2   = LoadStringA(e_hInstance, IDS_GPS_POST_PAYLOAD2,
			szPayload2, sizeof(szPayload2));
		cbPayload3   = LoadStringA(e_hInstance, IDS_GPS_POST_PAYLOAD3,
			szPayload3, sizeof(szPayload3));
		i          = GPSF_FIRST;
		nSaveFlags = pGPSP->nSaveFlags;
		pcPayload  = NULL;
		cbPayload  = 0;
		do
		{
			if(nSaveFlags & 1)
			{
				CopyMemory(pszFileExt, m_pszGPSF[i], sizeof(m_szOv2Ext));
				hFile = CreateFile(szFileName, GENERIC_READ, OPEN_EXISTING);
				if(hFile == INVALID_HANDLE)
					break;
				cbFile    = GetFileSize(hFile);
				pcPayload = (PSTR)realloc(pcPayload, cbPayload + m_ecbGpsPayload + cbFile);
				pcNextPos = pcPayload + cbPayload;
				if(cbPayload)
				{
					*pcNextPos = CHR_CR;
					pcNextPos++;
					*pcNextPos = CHR_LF;
					pcNextPos++;
				}
				CopyMemory(pcNextPos, szPayload1, cbPayload1);
				pcNextPos += cbPayload1;
				CopyMemory(pcNextPos, szName, cbName);
				pcNextPos += cbName;
				*pcNextPos = CHR_DOT;
				pcNextPos++;
				CopyMemory(pcNextPos, szGPSF[i], sizeof(szGPSF[0]));
				pcNextPos += sizeof(szGPSF[0]);
				CopyMemory(pcNextPos, szPayload2, cbPayload2);
				pcNextPos += cbPayload2;
				ReadFile(hFile, pcNextPos, cbFile, &cbRead);
				CloseHandle(hFile);
				if(cbFile != cbRead)
					break;
				pcNextPos += cbFile;
				CopyMemory(pcNextPos, szPayload3, cbPayload3);
				pcNextPos += cbPayload3;
				cbPayload  = pcNextPos - pcPayload;
			}
			nSaveFlags >>= 1;
			i++;
		}
		while(nSaveFlags);
		if(nSaveFlags)
			dwResId = IDS_GPS_FILE_ERROR;
		else
		{
			LoadString(e_hInstance, IDS_GPS_POST_HEADER, szText, countof(szText));
			cbPayload += LoadStringA(e_hInstance, IDS_GPS_POST_PAYLOAD4, pcNextPos, MAX_STR);
			if(HttpSendRequest(hReqHttp, szText, pcPayload, cbPayload))
			{
				char szResponse[32];
				dwResId = UNUSED;
				InternetReadFile(hReqHttp, szResponse, sizeof(szResponse) - 1, &cbRead);
				if(cbRead)
				{
					PCSTR pszRespCode;
					szResponse[cbRead] = CHR_NULL;
					pszRespCode        = StrChrA(szResponse, CHR_SP);
					if(pszRespCode)
					{
						i = atoi(pszRespCode);
						if(i == m_eGpsUnknownRC)
							dwResId = IDS_GPS_UNKNOWN_ERROR;
						else if(i == m_eGpsSpaceRC)
							dwResId = IDS_GPS_SPACE_ERROR;
#ifdef DEBUG
						else
							ASSERT((i == m_eGpsOKRC) || (i == m_eGpsCreatedRC));
#endif
					}
				}
			}
			else
				dwResId = IDS_GPS_SAVE_ERROR;
		}
		free(pcPayload);
		InternetCloseHandle(hReqHttp);
	}
	else
		dwResId = IDS_GPS_CONNECT_ERROR;
	InternetCloseHandle(hHttp);
	return dwResId;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Notifies whether the new version is available for download, if necessary

DWORD CALLBACK CTomPoiEdr::NotifyNewVersion(PVOID pvParam)
{
	TRACE(_T("Entering CTomPoiEdr::NotifyNewVersion\n"));

	int        iCurTime;
	FILETIME   ft;
	SYSTEMTIME st;
	GetSystemTime(&st);
	st.wYear -= 400;
	SystemTimeToFileTime(&st, &ft);
	iCurTime = (int)((((DLONG)ft.dwHighDateTime << 32) + ft.dwLowDateTime) / 10000000);
	if((iCurTime - GetPrivateProfileInt(e_szProfileSection, m_szLastNotifyEntry,
		0, e_pszProfile)) > m_eNewVerIntl)
	{
		HINTERNET hInternet;
		hInternet = InternetOpen(e_pszTpeAppName, INTERNET_OPEN_TYPE_PRECONFIG);
		if(hInternet)
		{
			TCHAR szText[MAX_STR];
			HHTTP hHttp;
			LoadString(e_hInstance, IDS_QUERY_VERSION_URL, szText, countof(szText));
			hHttp = InternetOpenUrl(hInternet, szText, INTERNET_FLAG_DONT_CACHE);
			if(hHttp)
			{
				PSTR           pszBuffer;
				DWORD          cbRead;
				NOTIFYICONDATA nid;
#ifdef UNICODE
				char           szBuffer[countof(nid.szInfo)];
				pszBuffer = szBuffer;
#else
				pszBuffer = nid.szInfo;
#endif
				InternetReadFile(hHttp, pszBuffer, countof(nid.szInfo), &cbRead);
				InternetCloseHandle(hHttp);
				if(cbRead && (*pszBuffer == 'v') && (cbRead < countof(nid.szInfo)))
				{
					PCTCHAR pcSpace;
					PTSTR   pszText;
					int     ccText;
#ifdef UNICODE
					ccText  = MBStoWCS(pszBuffer, cbRead, nid.szInfo, countof(nid.szInfo));
					pszText = nid.szInfo;
#else
					ccText  = cbRead;
					pszText = pszBuffer;
#endif
					pcSpace = StrChr(pszText, CHR_SP);
					if(pcSpace && (StrCmpN(e_pszTpeVersion, pszText, pcSpace - pszText) < 0))
					{
						TCHAR szCurTime[32];
						nid.cbSize           = sizeof(nid);
						nid.hWnd             = *e_pTPE;
						nid.uID              = IDR_MAIN;
						nid.uFlags           = NIF_ICON | NIF_INFO | NIF_MESSAGE;
						nid.uCallbackMessage = WM_TASKBAR_ICON;
						nid.hIcon            = (HICON)e_pTPE->GetClassLong(GCL_HICON);
						nid.uTimeout         = m_eNewVerTimeout;
						nid.dwInfoFlags      = NIIF_INFO;
						LoadString(e_hInstance, IDS_NEW_VERSION_AVL,
							nid.szInfoTitle, countof(nid.szInfoTitle));
						LoadString(e_hInstance, IDS_BROWSE_DL_WEBPAGE,
							pszText + ccText, countof(nid.szInfo));
						Shell_NotifyIcon(NIM_ADD, &nid);
						_ltot(iCurTime, szCurTime);
						WritePrivateProfileString(e_szProfileSection, m_szLastNotifyEntry,
							szCurTime, e_pszProfile);
					}
				}
			}
			InternetCloseHandle(hInternet);
		}
	}
	CloseHandle(e_pTPE->m_hNewVerThread);
	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Top-level window is being activated or deactivated

AFXINLINE void CTomPoiEdr::OnActivate(UINT nState, HWND hOtherWnd, BOOL bMinimized)
{
	TRACE(_T("Entering CTomPoiEdr::OnActivate\n"));

	if(!m_bCustTbBtn)
		return;

	ASSERT((nState == WA_INACTIVE) && hOtherWnd && (*this == ::GetParent(hOtherWnd)));
	m_bCustTbBtn = FALSE;
	m_jCustTbDlg.InitDialog(hOtherWnd);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/New

AFXINLINE void CTomPoiEdr::OnFileNew()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileNew\n"));

	// Prompts to save the file, if necessary
	SCINT scxLvCol[] =
	{
		m_ecxLvNameCol,
		m_ecxLvLngCol,
		m_ecxLvLatCol,
		m_ecxLvTelCol,
		m_ecxLvMemoCol,
		m_ecxLvDistCol,
	};
	ASSERT(countof(scxLvCol) == LVC_COUNT);
	m_jToolbar.PressButton(ID_FILE_NEW);
	m_jToolbar.UpdateWindow();
	if(!m_bFileDirty || PromptMessage())
	{
		TCHAR    szText[32];
		int      i, ceLvItem;
		HDITEM   hi;
		LVCOLUMN lvc;
		if(m_jCheckDataDlg)
			m_jCheckDataDlg.SendMessage(WM_COMMAND, MAKEWPARAM(ID_CANCEL, BN_CLICKED));
		ceLvItem = m_jListView.GetItemCount();
		if(ceLvItem)
			m_jListView.DeleteAllItems();
		if((e_dwWinVer > WV_XP) || !ceLvItem)
			m_jListView.Scroll(MIN_INT, 0);
		m_jListView.FreeStorage();
		lvc.mask = LVCF_FMT | LVCF_WIDTH;
		hi.mask  = HDI_FORMAT | HDI_WIDTH;
		i        = LVC_FIRST;
		do
		{
			lvc.cx = scxLvCol[i];
			if(i == LVC_DIST)
			{
				if(m_jOptionDlg.m_nShowDist == BST_UNCHECKED)
					lvc.cx = 0;
				lvc.fmt = LVCFMT_RIGHT;
				hi.fmt  = HDF_RIGHT | HDF_STRING;
				hi.cxy  = lvc.cx + ((e_dwWinVer > WV_XP) ? m_ecxHdColPad : m_ecxHdColPadXP);
			}
			else
			{
				lvc.fmt = LVCFMT_LEFT;
				hi.fmt  = HDF_LEFT | HDF_STRING;
				hi.cxy  = lvc.cx;
			}
			m_jListView.SetColumn(i, &lvc);
			m_jLvHeader.SetItem(i, &hi);
			i++;
		}
		while(i < LVC_COUNT);
		lvc.mask    = LVCF_TEXT;
		lvc.pszText = szText;
		hi.mask     = HDI_TEXT;
		hi.pszText  = szText;
		LoadString(e_hInstance, IDS_TEL_COL, szText, countof(szText));
		m_jListView.SetColumn(LVC_TEL, &lvc);
		m_jLvHeader.SetItem(LVC_TEL, &hi);
		LoadString(e_hInstance, IDS_MEMO_COL, szText, countof(szText));
		m_jListView.SetColumn(LVC_MEMO, &lvc);
		m_jLvHeader.SetItem(LVC_MEMO, &hi);
		LoadString(e_hInstance, IDS_UNTITLED_NAME, m_jFileDlg.lpstrFileTitle, MAX_FNAME);
		*m_jFileDlg.lpstrFile   = CHR_NULL;
		m_fst                   = FST_GENERAL;
		m_jFileDlg.nFilterIndex = FTW_OV2;
		m_pr64.byType           = UNUSED;
		m_iLvCol                = LVC_NAME;
		m_bFileDirty            = FALSE;
		m_bSetCaption           = TRUE;
		m_bSetBmp               = TRUE;
		m_bRPF                  = FALSE;
		m_iLvBaseItem           = LVI_ERR;
		m_bMyBP                 = FALSE;
		if(m_jIEPane.IsWindowVisible())
			PostMessage(WM_COMMAND, ID_POS_GOOGLE_MAP);
	}
	m_jToolbar.PressButton(ID_FILE_NEW, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Open

AFXINLINE void CTomPoiEdr::OnFileOpen()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileOpen\n"));

	// Prompts to save the file, if necessary
	// Displays the Open dialog box and operates the POI file
	m_jToolbar.PressButton(ID_FILE_OPEN);
	m_jToolbar.UpdateWindow();
	if(!m_bFileDirty || PromptMessage())
		OperateFileName(OPF_READ);
	m_jToolbar.PressButton(ID_FILE_OPEN, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Open Favorites

AFXINLINE void CTomPoiEdr::OnFileOpenFav()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileOpenFav\n"));

	// Prompts to save the file, if necessary
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_OPEN_FAV);
	m_jToolbar.UpdateWindow();
	if(!m_bFileDirty || PromptMessage())
	{
		TCHAR szDisplayName[MAX_PATH], szFileName[countof(szDisplayName)];
		TCHAR szTitle[64], szFavName[32];
		LoadString(e_hInstance, IDS_OFD_TITLE, szTitle, countof(szTitle));
		LoadString(e_hInstance, IDS_FAV_FILE_NAME, szFavName, countof(szFavName));
		m_jOpenFavDlg.hwndOwner      = *this;
		m_jOpenFavDlg.pszDisplayName = szDisplayName;
		m_jOpenFavDlg.lpszTitle      = szTitle;
		m_jOpenFavDlg.m_pszFavName   = szFavName;
		if(m_jOpenFavDlg.Browse(szFileName))
		{
			lstrcpy(m_jFileDlg.lpstrFile, szFileName);
			m_jFileDlg.nFilterIndex = FTR_CFG;
			if(OperatePoiFile(OPF_READ))
			{
				LoadString(e_hInstance, IDS_FAVORITES_NAME,
					m_jFileDlg.lpstrFileTitle, MAX_FNAME);
				*m_jFileDlg.lpstrFile = CHR_NULL;
				m_bSetBmp             = TRUE;
			}
		}
	}
	m_jToolbar.PressButton(ID_FILE_OPEN_FAV, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Open Device

AFXINLINE void CTomPoiEdr::OnFileOpenDevice()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileOpenDevice\n"));

	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_OPEN_DEVICE);
	m_jToolbar.UpdateWindow();
	if(!m_bFileDirty || PromptMessage())
	{
		DWORD     dwResId;
		HINTERNET hInternet;
		hInternet = GpsInternetOpen();
		if(hInternet)
		{
			TCHAR          szSelName[MAX_FNAME], szText[MAX_STR], szTempFile[MAX_PATH];
			int            iResult;
			GPSDELETE      gpsd;
			GPSFILELIST    gpsfl;
			HTHREAD        hThread;
			COpenDeviceDlg jODD(hInternet, &gpsfl, szSelName);
			do
			{
				SetCursor(LoadCursor(IDC_WAIT));
				ShowCursor();
				dwResId = GpsGetFileList(hInternet, TRUE, &gpsfl);
				if(!dwResId)
				{
					m_bSetReady = TRUE;
					iResult     = jODD.DialogBox(e_hInstance, IDD_OPEN_DEVICE, *this);
					switch(iResult)
					{
					case ID_OK:
						LoadString(e_hInstance, IDP_GPS_READING_FILE, szText, countof(szText));
						m_jStatusBar.SetWindowText(szText);
						dwResId = GpsGetFile(hInternet, TRUE, szSelName, TRUE, szTempFile);
						if(dwResId)
							break;
						m_jFileDlg.nFilterIndex = FTR_OV2;
						lstrcpy(m_jFileDlg.lpstrFile, szTempFile);
						lstrcpy(m_jFileDlg.lpstrFileTitle, szSelName);
						OperatePoiFile(OPF_READ);
						*m_jFileDlg.lpstrFile = CHR_NULL;
						m_bSetBmp             = TRUE;
						DeleteFile(szTempFile);
						if(m_fst != FST_GENERAL)
							break;
						LoadString(e_hInstance, IDS_FAVORITES_NAME, szText, countof(szText));
						if(lstrcmpi(szSelName, szText))
							GpsGetFile(hInternet, TRUE, szSelName, FALSE, szTempFile);
						break;
					case IDC_GPS_DELETE:
						SetCursor(LoadCursor(IDC_WAIT));
						GpsBroadcastChange(FALSE);
						GpsCreateMarquee(IDP_GPS_DELETING_FILE);
						gpsd.hInternet = hInternet;
						gpsd.pszName   = szSelName;
						gpsd.pGPSFL    = &gpsfl;
						hThread        = CreateThread(GpsDeletePoiFile, &gpsd, CT_IMMEDIATE);
						do
							PeekUpdateMessage(FALSE);
						while(WaitForSingleObject(hThread, m_ePbMqIntl) == WAIT_TIMEOUT);
						GetExitCodeThread(hThread, &dwResId);
						CloseHandle(hThread);
						m_jProgress.DestroyWindow();
						GpsBroadcastChange(TRUE);
						if(dwResId)
							break;
					case IDC_GPS_REFRESH:
						dwResId = MAX_DWORD;
#ifdef DEBUG
						break;
					default:
						ASSERT(iResult == ID_CANCEL);
#endif
					}
					GpsFreeFileList(&gpsfl);
				}
				HideCursor();
				SetCursor(LoadCursor(IDC_ARROW));
			}
			while(dwResId == MAX_DWORD);
			InternetCloseHandle(hInternet);
		}
		else
			dwResId = IDS_GPS_CONNECT_ERROR;
		m_bSetReady = TRUE;
		if(dwResId)
			DisplayMessage(dwResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	}
	m_jToolbar.PressButton(ID_FILE_OPEN_DEVICE, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Dump Device

AFXINLINE void CTomPoiEdr::OnFileDumpDevice()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileDumpDevice\n"));

	TCHAR     szImage[MAX_PATH];
	int       ceImage;
	DWORD     dwResId;
	HINTERNET hInternet;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_DUMP_DEVICE);
	m_jToolbar.UpdateWindow();
	hInternet = GpsInternetOpen();
	if(hInternet)
	{
		SCTCHAR  szPrefix[]     = _T("image/");
		SCTCHAR  szGifExt[]     = _T("gif");
		SCPCTSTR spszMimeType[] =
		{
			_T("jpeg"),
			szGifExt,
			m_szPngExt,
			_T("tiff"),
			m_szBmpExt,
		};
		SCPCTSTR spszImageExt[] =
		{
			_T("jpg"),
			szGifExt,
			m_szPngExt,
			_T("tif"),
			m_szBmpExt,
		};
		PCTSTR              pszMimeType, pszImageExt;
		PTSTR               pszName, pszAttr, pszFileExt, pszTS[5];
		PGPSFILE            pGPSF;
		PPGPSFILE           ppGPSF;
		PIMAGE              pImage, pThumbImage;
		PICI                pICI, pAllocICI;
		TCHAR               szFolder[MAX_PATH], szTempFile[countof(szFolder)];
		TCHAR               szSbText[countof(szFolder) + MAX_STR], szText[MAX_STR];
		BOOL                bTimestamp, bTransform;
		int                 i, iRotate, iFormat, iResult, ccFolder, ccName;
		UINT                nFEDSel, ceEncoder, cbEncoder;
		DWORD               dwFileAttr;
		ULONG_PTR           gdipt;
		GPSFILELIST         gpsfl;
		SIZE                size;
		Status              status;
		HTHREAD             hThread;
		CFileExistDlg       jFED;
		GdiplusStartupInput gdipsi;
		ppGPSF = &gpsfl.pGPSF[GPSF_BMP];
		CDumpDeviceDlg jDDD(hInternet, ppGPSF, szFolder,
			&bTimestamp, &size, &iRotate, &iFormat);
		do
		{
			SetCursor(LoadCursor(IDC_WAIT));
			ShowCursor();
			dwResId = GpsGetFileList(hInternet, FALSE, &gpsfl);
			if(!dwResId)
			{
				m_bSetReady = TRUE;
				iResult     = jDDD.DialogBox(e_hInstance, IDD_DUMP_DEVICE, *this);
				pGPSF       = *ppGPSF;
				switch(iResult)
				{
				case ID_OK:
					ccFolder = lstrlen(szFolder);
					CopyMemory(szImage, szFolder, ccFolder * sizeof(TCHAR));
					pszName  = szImage + ccFolder;
					*pszName = CHR_BSLH;
					pszName++;
					if(size.cx)
						bTransform = TRUE;
					else if(m_gpsd == GPSD_V3)
						bTransform = iFormat != CDumpDeviceDlg::IF_BMP;
					else
						bTransform = iFormat != CDumpDeviceDlg::IF_PNG;
					GdiplusStartup(&gdipt, &gdipsi, NULL);
					GetImageEncodersSize(&ceEncoder, &cbEncoder);
					pAllocICI = (PICI)malloc(cbEncoder);
					GetImageEncoders(ceEncoder, cbEncoder, pAllocICI);
					pICI        = pAllocICI - 1;
					pszMimeType = spszMimeType[iFormat];
					do
					{
						pICI++;
						ASSERT(EqualMemory(pICI->MimeType, szPrefix,
							sizeof(szPrefix) - sizeof(TCHAR)));
					}
					while(lstrcmp(pICI->MimeType + countof(szPrefix) - 1, pszMimeType));
					pszImageExt = spszImageExt[iFormat];
					LoadString(e_hInstance, IDP_CONVERTED_FORMAT, szText, countof(szText));
					for(nFEDSel = ID_NO, ceImage = 0; pGPSF; pGPSF = pGPSF->pNext)
					{
						if(PeekUpdateMessage(TRUE))
							break;
						ccName = lstrlen(pGPSF->pszName);
						CopyMemory(pszName, pGPSF->pszName, ccName * sizeof(TCHAR));
						pszFileExt = pszName + ccName;
						if((m_gpsd == GPSD_V4) && bTimestamp)
						{
							pszAttr = pGPSF->pszAttr;
							i       = 0;
							ZeroMemory(pszTS, sizeof(pszTS));
							while(*pszAttr && (i < countof(pszTS)))
							{
								if(IsValidNumber(*pszAttr))
								{
									if(!pszTS[i])
										pszTS[i] = pszAttr;
								}
								else
								{
									*pszAttr = CHR_NULL;
									i++;
								}
								pszAttr++;
							}
							if(pszTS[2])
							{
								*pszFileExt = CHR_DSH;
								pszFileExt++;
								i = lstrlen(pszTS[2]);
								CopyMemory(pszFileExt, pszTS[2], i * sizeof(TCHAR));
								pszFileExt += i;
								i           = lstrlen(pszTS[1]);
								CopyMemory(pszFileExt, pszTS[1], i * sizeof(TCHAR));
								pszFileExt += i;
								i           = lstrlen(pszTS[0]);
								CopyMemory(pszFileExt, pszTS[0], i * sizeof(TCHAR));
								pszFileExt += i;
								if(pszTS[4])
								{
									*pszFileExt = CHR_DSH;
									pszFileExt++;
									i = lstrlen(pszTS[3]);
									CopyMemory(pszFileExt, pszTS[3], i * sizeof(TCHAR));
									pszFileExt += i;
									i           = lstrlen(pszTS[4]);
									CopyMemory(pszFileExt, pszTS[4], i * sizeof(TCHAR));
									pszFileExt += i;
								}
							}
						}
						*pszFileExt = CHR_DOT;
						CopyMemory(pszFileExt + 1, pszImageExt, sizeof(m_szBmpExt));
						wsprintf(szSbText, szText, ceImage, pszName);
						m_jStatusBar.SetWindowText(szSbText);
						dwFileAttr = GetFileAttributes(szImage);
						if(dwFileAttr != INVALID_VALUE)
						{
							if(nFEDSel == IDC_NO_ALL)
								continue;
							if(nFEDSel != IDC_YES_ALL)
							{
								jFED.m_pszFileName = szImage;
								nFEDSel            = jFED.DialogBox(e_hInstance,
									IDD_FILE_EXIST, *this);
								SetCursor(LoadCursor(IDC_WAIT));
								if(nFEDSel == ID_CANCEL)
									break;
								if((nFEDSel == ID_NO) || (nFEDSel == IDC_NO_ALL))
									continue;
							}
							if(dwFileAttr & FILE_ATTRIBUTE_READONLY)
								SetFileAttributes(szImage,
									dwFileAttr & ~FILE_ATTRIBUTE_READONLY);
						}
						dwResId = GpsGetFile(hInternet, FALSE, pGPSF->pszName,
							FALSE, szTempFile);
						if(dwResId)
							break;
						if(bTransform)
						{
							pImage = Image::FromFile(szTempFile);
							if(size.cx)
							{
								pThumbImage = pImage->GetThumbnailImage(size.cx, size.cy,
									NULL, NULL);
								delete pImage;
								pImage = pThumbImage;
							}
							status = pImage->Save(szImage, &pICI->Clsid, NULL);
							delete pImage;
							DeleteFile(szTempFile);
						}
						else
						{
							if(IsValidFile(szImage))
								DeleteFile(szImage);
							status = MoveFile(szTempFile, szImage) ? Ok : AccessDenied;
						}
						if(status == Ok)
						{
							if(iRotate)
							{
								switch(iRotate)
								{
								case 1:
									iRotate = Rotate90FlipNone;
									break;
								case 2:
									iRotate = Rotate180FlipNone;
									break;
								default:
									iRotate = Rotate270FlipNone;
								}
								pImage = Image::FromFile(szImage);
								status = pImage->RotateFlip((RotateFlipType)iRotate);
								if(status == Ok)
									status = pImage->Save(szImage, &pICI->Clsid, NULL);
								delete pImage;
							}
						}
						if(status != Ok)
						{
							dwResId = IDS_CANNOT_WRITE;
							ceImage = (int)szImage;
							break;
						}
						ceImage++;
					}
					GdiplusShutdown(gdipt);
					free(pAllocICI);
					if(!dwResId && !pGPSF)
						dwResId = IDS_CONVERT_FORMAT_COMPLETED;
					break;
				case IDC_GPS_DELETE_ALL:
					SetCursor(LoadCursor(IDC_WAIT));
					GpsBroadcastChange(FALSE);
					GpsCreateMarquee(IDP_GPS_DELETING_FILE);
					hThread = CreateThread(GpsDeleteDumpFile, hInternet, CT_IMMEDIATE);
					do
						PeekUpdateMessage(FALSE);
					while(WaitForSingleObject(hThread, m_ePbMqIntl) == WAIT_TIMEOUT);
					GetExitCodeThread(hThread, &dwResId);
					CloseHandle(hThread);
					m_jProgress.DestroyWindow();
					GpsBroadcastChange(TRUE);
					if(dwResId)
						break;
				case IDC_GPS_REFRESH:
					dwResId = MAX_DWORD;
#ifdef DEBUG
					break;
				default:
					ASSERT(iResult == ID_CANCEL);
#endif
				}
				GpsFreeFileList(&gpsfl);
			}
			HideCursor();
			SetCursor(LoadCursor(IDC_ARROW));
		}
		while(dwResId == MAX_DWORD);
		InternetCloseHandle(hInternet);
	}
	else
		dwResId = IDS_GPS_CONNECT_ERROR;
	m_bSetReady = TRUE;
	if(dwResId)
		DisplayMessage(dwResId, MB_ICONINFORMATION | MB_OK, ceImage);
	m_jToolbar.PressButton(ID_FILE_DUMP_DEVICE, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Save

AFXINLINE void CTomPoiEdr::OnFileSave()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileSave\n"));

	// Operates the POI file directly
	if(*m_jFileDlg.lpstrFile && (m_jFileDlg.nFilterIndex != FTR_GPI))
	{
		m_jToolbar.PressButton(ID_FILE_SAVE);
		m_jToolbar.UpdateWindow();
		OperatePoiFile(OPF_WRITE);
		m_jToolbar.PressButton(ID_FILE_SAVE, FALSE);
		return;
	}

	// Skips the process when the file is backup
	if(m_bBackup)
		return;

	// Displays the Save As dialog box and operates the POI file
	SendMessage(WM_COMMAND, ID_FILE_SAVE_AS);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Save As

AFXINLINE void CTomPoiEdr::OnFileSaveAs()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileSaveAs\n"));

	// Displays the Save As dialog box and operates the POI file
	m_jToolbar.PressButton(ID_FILE_SAVE_AS);
	m_jToolbar.UpdateWindow();
	OperateFileName(OPF_WRITE);
	m_jToolbar.PressButton(ID_FILE_SAVE_AS, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Save Device

AFXINLINE void CTomPoiEdr::OnFileSaveDevice()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileSaveDevice\n"));

	TCHAR  szText[32];
	BOOL   bEmptyCoord;
	int    i, ceLvItem;
	LVITEM lvi;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_SAVE_DEVICE);
	m_jToolbar.UpdateWindow();
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	ceLvItem       = m_jListView.GetItemCount();
	i              = ceLvItem;
	lvi.pszText    = szText;
	lvi.cchTextMax = 2;
	lvi.state      = LVIS_FOCUSED | LVIS_SELECTED;
	bEmptyCoord    = FALSE;
	do
	{
		i--;
		lvi.iSubItem = LVC_LNG;
		if(m_jListView.GetItemText(i, &lvi))
		{
			lvi.iSubItem = LVC_LAT;
			if(m_jListView.GetItemText(i, &lvi))
				continue;
		}
		m_jListView.SetItemState(i, &lvi);
		bEmptyCoord = TRUE;
	}
	while(i);

	DWORD dwResId;
	if(bEmptyCoord)
	{
		EnsureLvItemVisible(m_jListView.GetNextItem(LVNI_SELECTED));
		dwResId = IDS_GPS_EMPTY_COORD;
	}
	else
	{
		PTSTR  pszFileName;
		PCTSTR pszName;
		BOOL   bValidPath;
		pszFileName = m_jFileDlg.lpstrFile;
		pszName     = m_jFileDlg.lpstrFileTitle;
		dwResId     = IDS_GPS_CONNECT_ERROR;
		if(*pszFileName)
			bValidPath = TRUE;
		else
		{
			bValidPath = FALSE;
			if(m_bFileDirty)
			{
				LoadString(e_hInstance, IDS_UNTITLED_NAME, szText, countof(szText));
				if(!lstrcmpi(szText, pszName))
				{
					SendMessage(WM_COMMAND, ID_FILE_SAVE_AS);
					if(m_bFileDirty)
						dwResId = UNUSED;
					else
						bValidPath = TRUE;
				}
			}
		}
		if(dwResId)
		{
			if(m_bMyBP)
				ceLvItem--;
			if(ceLvItem > m_eceMaxFavPoi)
			{
				LoadString(e_hInstance, IDS_FAVORITES_NAME, szText, countof(szText));
				if(!lstrcmpi(szText, pszName) && (DisplayMessage(IDS_GPS_MAX_FAVORITES,
					MB_ICONQUESTION | MB_OKCANCEL | MB_DEFBUTTON2,
					m_eceMaxFavPoi) == ID_CANCEL))
					dwResId = UNUSED;
			}
			if(dwResId)
			{
				UINT nSaveFlags;
				if(bValidPath)
				{
					CSaveDeviceDlg jSDD(pszFileName, &nSaveFlags);
					if(jSDD.DialogBox(e_hInstance, IDD_SAVE_DEVICE, *this) == ID_CANCEL)
						dwResId = UNUSED;
				}
				else
					nSaveFlags = GPSSF_OV2;
				if(dwResId)
				{
					HINTERNET hInternet;
					hInternet = GpsInternetOpen();
					if(hInternet)
					{
						GPSFILELIST gpsfl;
						SetCursor(LoadCursor(IDC_WAIT));
						ShowCursor();
						dwResId = GpsGetFileList(hInternet, TRUE, &gpsfl);
						if(!dwResId)
						{
							PTSTR     pszOwExt;
							PPGPSFILE ppGPSF;
							TCHAR     szOwExt[64];
							UINT      nFlags;
							m_bSetReady = TRUE;
							pszOwExt    = szOwExt;
							*pszOwExt   = CHR_NULL;
							i           = GPSF_FIRST;
							ppGPSF      = gpsfl.pGPSF;
							nFlags      = nSaveFlags;
							do
							{
								if((nFlags & 1) && GpsFindFile(ppGPSF[i], pszName))
								{
									if(*szOwExt)
									{
										CopyMemory(pszOwExt,
											m_pszGpsfSep, sizeof(m_pszGpsfSep));
										pszOwExt += countof(m_pszGpsfSep);
									}
									CopyMemory(pszOwExt, m_pszGPSF[i], sizeof(m_szOv2Ext));
									pszOwExt += countof(m_szOv2Ext) - 1;
								}
								i++;
								nFlags >>= 1;
							}
							while(nFlags);
							GpsFreeFileList(&gpsfl);
							if(!*szOwExt || (DisplayMessage(IDS_GPS_OVERWRITE_FILE,
								MB_ICONQUESTION | MB_OKCANCEL, (LPARAM)szOwExt) == ID_OK))
							{
								TCHAR   szFileName[MAX_PATH], szOv2Ext[8];
								BOOL    bSetBmp;
								GPSPOST gpsp;
								HTHREAD hThread;
								SetCursor(LoadCursor(IDC_WAIT));
								GpsBroadcastChange(FALSE);
								szOv2Ext[0] = CHR_DOT;
								CopyMemory(szOv2Ext + 1, m_szOv2Ext, sizeof(m_szOv2Ext));
								if(bValidPath)
								{
									if(m_bFileDirty && (m_jFileDlg.nFilterIndex != FTW_GPI))
										SendMessage(WM_COMMAND, ID_FILE_SAVE);
									if(m_jFileDlg.nFilterIndex == FTW_OV2)
										bSetBmp = FALSE;
									else
									{
										m_jFileDlg.nFilterIndex = FTW_OV2;
										PathRenameExtension(pszFileName, szOv2Ext);
										SendMessage(WM_COMMAND, ID_FILE_SAVE);
										bSetBmp = TRUE;
									}
								}
								else
								{
									HFILE hFile;
									lstrcpy(szFileName, m_pszGmTempFile);
									PathRenameExtension(szFileName, szOv2Ext);
									hFile = CreateFile(szFileName,
										GENERIC_WRITE, CREATE_ALWAYS);
									if(hFile != INVALID_HANDLE)
									{
										m_opf = OPF_WRITE;
										m_jProgress.CreateWindow(m_ePbStyle, 0, 0, 0, 0,
											m_jStatusBar, IDW_PROGRESS);
										dwResId = WriteOv2Record(hFile);
										m_jProgress.DestroyWindow();
										CloseHandle(hFile);
										pszFileName   = szFileName;
										m_bFileDirty  = FALSE;
										m_bSetCaption = TRUE;
									}
									else
										dwResId = IDS_GPS_FILE_ERROR;
								}
								if(!dwResId)
								{
									SetCursor(LoadCursor(IDC_WAIT));
									GpsCreateMarquee(IDP_GPS_SAVING_FILE);
									gpsp.hInternet   = hInternet;
									gpsp.pszName     = pszName;
									gpsp.pszFileName = pszFileName;
									gpsp.nSaveFlags  = nSaveFlags;
									hThread          = CreateThread(GpsPostPoiFile,
										&gpsp, CT_IMMEDIATE);
									do
										PeekUpdateMessage(FALSE);
									while(WaitForSingleObject(hThread,
										m_ePbMqIntl) == WAIT_TIMEOUT);
									GetExitCodeThread(hThread, &dwResId);
									CloseHandle(hThread);
									m_jProgress.DestroyWindow();
									if(bValidPath)
										m_bSetBmp = bSetBmp;
									else
										DeleteFile(pszFileName);
								}
								GpsBroadcastChange(TRUE);
							}
						}
						HideCursor();
						SetCursor(LoadCursor(IDC_ARROW));
						InternetCloseHandle(hInternet);
					}
				}
			}
		}
	}
	m_bSetReady = TRUE;
	if(dwResId)
		DisplayMessage(dwResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(ID_FILE_SAVE_DEVICE, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Import

AFXINLINE void CTomPoiEdr::OnFileImport()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileImport\n"));

	// Displays the Open dialog box and operates the POI file
	m_jToolbar.PressButton(ID_FILE_IMPORT);
	m_jToolbar.UpdateWindow();
	OperateFileName(OPF_IMPORT);
	m_jToolbar.PressButton(ID_FILE_IMPORT, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Export

AFXINLINE void CTomPoiEdr::OnFileExport()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileExport\n"));

	// Displays the Save As dialog box and operates the POI file
	m_jToolbar.PressButton(ID_FILE_EXPORT);
	m_jToolbar.UpdateWindow();
	OperateFileName(OPF_EXPORT);
	m_jToolbar.PressButton(ID_FILE_EXPORT, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item in the range from ID_RECENT_FILE1 to ID_RECENT_FILE9

AFXINLINE void CTomPoiEdr::OnRecentFile(UINT nItemId)
{
	TRACE(_T("Entering CTomPoiEdr::OnRecentFile\n"));

	OpenPoiFile(nItemId - ID_RECENT_FILE1);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Query Names

AFXINLINE void CTomPoiEdr::OnFileQuery()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileQuery\n"));

	CQueryNameDlg jQND;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_QUERY);
	m_jToolbar.UpdateWindow();
	jQND.DialogBox(e_hInstance, IDD_QUERY_NAME, *this);
	m_jToolbar.PressButton(ID_FILE_QUERY, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Translate Names

AFXINLINE void CTomPoiEdr::OnFileTranslate()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileTranslate\n"));

	TCHAR szFileNameF[MAX_PATH], szFileNameT[countof(szFileNameF)], szResText[MAX_STR];
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_TRANSLATE);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDS_TND_TITLE, szResText, countof(szResText));
	m_jTranNameDlg.hwndOwner      = *this;
	m_jTranNameDlg.pszDisplayName = szFileNameT;
	m_jTranNameDlg.lpszTitle      = szResText;
	if(m_jTranNameDlg.Browse(szFileNameF))
	{
		PTSTR           pszFileNameF;
		int             ccName, ceFile;
		UINT            nResId;
		WIN32_FIND_DATA wfd;
		HANDLE          hFind;
		SetCursor(LoadCursor(IDC_WAIT));
		ShowCursor();
		LoadString(e_hInstance, IDP_TRANSLATING_NAME, szResText, countof(szResText));
		m_jStatusBar.SetWindowText(szResText);
		ccName       = lstrlen(szFileNameF);
		pszFileNameF = szFileNameF + ccName;
		if(*(pszFileNameF - 1) != CHR_BSLH)
		{
			*pszFileNameF = CHR_BSLH;
			pszFileNameF++;
			ccName++;
		}
		*pszFileNameF       = CHR_AST;
		*(pszFileNameF + 1) = CHR_NULL;
		hFind               = FindFirstFile(szFileNameF, &wfd);
		if(hFind != INVALID_HANDLE)
		{
			SCTCHAR szPostfix[] = _T("_Images");
			TCHAR   szSbText[countof(szFileNameF) + MAX_STR];
			PCTSTR  pszFileExt;
			PTSTR   pszFileNameT;
			int     (CTomPoiEdr::*ConvertFileName)(PCTSTR, PTSTR);
			int     i;
			LoadString(e_hInstance, IDP_TRANSLATED_NAME, szResText, countof(szResText));
			CopyMemory(szFileNameT, szFileNameF, ccName * sizeof(TCHAR));
			pszFileNameT    = szFileNameT + ccName;
			ConvertFileName = (m_jTranNameDlg.m_nTomTom == BST_CHECKED) ?
				&CTomPoiEdr::WinFile2TomFile : &CTomPoiEdr::TomFile2WinFile;
			nResId          = 0;
			ceFile          = 0;
			do
			{
				if(PeekUpdateMessage(TRUE))
				{
					nResId = MAX_UINT;
					break;
				}
				if(wfd.cFileName[0] == CHR_DOT)
					continue;
				ccName = lstrlen(wfd.cFileName);
				if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if(ccName < countof(szPostfix))
						continue;
					if(lstrcmpi(wfd.cFileName + ccName - countof(szPostfix) + 1, szPostfix))
						continue;
				}
				else
				{
					pszFileExt = PathFindExtension(wfd.cFileName);
					if(!*pszFileExt)
						continue;
					pszFileExt++;
					i = 0;
					do
					{
						if(!lstrcmpi(pszFileExt, m_pszGPSF[i]))
							break;
						i++;
					}
					while(i < countof(m_pszGPSF));
					if(i == countof(m_pszGPSF))
						continue;
				}
				(*this.*ConvertFileName)(wfd.cFileName, pszFileNameT);
				if(!lstrcmpi(wfd.cFileName, pszFileNameT))
					continue;
				CopyMemory(pszFileNameF, wfd.cFileName, (ccName + 1) * sizeof(TCHAR));
				wsprintf(szSbText, szResText, ceFile, pszFileNameF);
				m_jStatusBar.SetWindowText(szSbText);
				if(GetFileAttributes(szFileNameT) != INVALID_VALUE)
				{
					nResId       = IDS_FILE_EXIST_ERROR;
					pszFileNameF = pszFileNameT;
					break;
				}
				if(wfd.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
					SetFileAttributes(szFileNameF,
						wfd.dwFileAttributes & ~FILE_ATTRIBUTE_READONLY);
				if(!MoveFile(szFileNameF, szFileNameT))
				{
					nResId = IDS_RENAME_FILE_ERROR;
					break;
				}
				if(wfd.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
					SetFileAttributes(szFileNameT, wfd.dwFileAttributes);
				ceFile++;
			}
			while(FindNextFile(hFind, &wfd));
			if(!nResId || (nResId == MAX_UINT))
			{
				if((nResId == MAX_UINT) || (GetLastError() == ERROR_NO_MORE_FILES))
				{
					nResId       = IDS_TRAN_NAME_COMPLETED;
					pszFileNameF = (PTSTR)ceFile;
				}
				else
				{
					nResId       = IDS_DISK_STRUCT_ERROR;
					pszFileNameF = NULL;
				}
			}
			FindClose(hFind);
		}
		else
		{
			nResId       = IDS_DISK_STRUCT_ERROR;
			pszFileNameF = NULL;
		}
		HideCursor();
		SetCursor(LoadCursor(IDC_ARROW));
		m_bSetReady = TRUE;
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, (LPARAM)pszFileNameF);
	}
	m_jToolbar.PressButton(ID_FILE_TRANSLATE, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Convert Formats

AFXINLINE void CTomPoiEdr::OnFileConvert()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileConvert\n"));

	TCHAR szFileNameF[MAX_PATH], szFileNameT[countof(szFileNameF)], szResText[MAX_STR];
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_FILE_CONVERT);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDS_CFD_TITLE, szResText, countof(szResText));
	m_jConvertFormatDlg.hwndOwner      = *this;
	m_jConvertFormatDlg.pszDisplayName = szFileNameT;
	m_jConvertFormatDlg.lpszTitle      = szResText;
	if(m_jConvertFormatDlg.Browse(szFileNameF))
	{
		PTSTR           pszFileNameF;
		int             ccName, ceFile;
		UINT            nResId;
		WIN32_FIND_DATA wfd;
		HANDLE          hFind;
		SetCursor(LoadCursor(IDC_WAIT));
		ShowCursor();
		LoadString(e_hInstance, IDP_CONVERTING_FORMAT, szResText, countof(szResText));
		m_jStatusBar.SetWindowText(szResText);
		ccName       = lstrlen(szFileNameF);
		pszFileNameF = szFileNameF + ccName;
		if(*(pszFileNameF - 1) != CHR_BSLH)
		{
			*pszFileNameF = CHR_BSLH;
			pszFileNameF++;
			ccName++;
		}
		*pszFileNameF = CHR_AST;
		if(m_jConvertFormatDlg.m_nOv2Btn == BST_CHECKED)
			*(pszFileNameF + 1) = CHR_NULL;
		else
		{
			*(pszFileNameF + 1) = CHR_DOT;
			CopyMemory(pszFileNameF + 2, m_szOv2Ext, sizeof(m_szOv2Ext));
		}
		hFind = FindFirstFile(szFileNameF, &wfd);
		if(hFind != INVALID_HANDLE)
		{
			SCPCTSTR spszFileExt[] =
			{
				m_szKmlExt,
				m_szGpxExt,
				m_szGpiExt,
				m_szCsvExt,
				m_szAscExt,
				m_szPoiExt,
				m_szTxtExt,
			};
			SCINT spszFTR[] =
			{
				FTR_KML,
				FTR_GPX,
				FTR_GPI,
				FTR_CSV,
				FTR_ASC,
				FTR_POI,
				FTR_TXT,
			};
			SCINT spszFTW[] =
			{
				FTW_KML,
				FTW_GPX,
				FTW_CSVA,
				FTW_CSVU,
				FTW_CSVU8,
				FTW_ASCA,
				FTW_ASCU,
				FTW_ASCU8,
				FTW_POIA,
				FTW_POIU,
				FTW_POIU8,
				FTW_TXTA,
				FTW_TXTU,
				FTW_TXTU8,
			};
			PVOID          pvFile;
			PCTSTR         pszFileExtF, pszFileExtT;
			PTSTR          pszFileNameT;
			PWCHAR         pwEnding;
			TCHAR          szLvCol[32], szSbText[countof(szFileNameF) + MAX_STR];
			BOOL           bBackup, bRPF, bMyBP;
			int            i, iFTR, iFTW;
			UINT           nDmsFormat, nFEDSel;
			DWORD          dwFilterIndex, dwFileAttr, cbAlloc, cbMemory, cbFile;
			FILESTRUCTTYPE fst;
			LVCOLUMN       lvc;
			POIRECORD64    pr64;
			HFILE          hFile;
			HWND           hListView;
			CFileExistDlg  jFED;
			CListView      jListView;
			bBackup                   = m_bBackup;
			m_bBackup                 = TRUE;
			fst                       = m_fst;
			dwFilterIndex             = m_jFileDlg.nFilterIndex;
			pr64                      = m_pr64;
			bRPF                      = m_bRPF;
			bMyBP                     = m_bMyBP;
			m_bMyBP                   = FALSE;
			nDmsFormat                = m_jOptionDlg.m_nDmsFormat;
			m_jOptionDlg.m_nDmsFormat = BST_UNCHECKED;
			hListView                 = m_jListView;
			jListView.CreateWindow(WS_CHILD | LVS_REPORT, 0, 0, 0, 0, *this, IDW_LIST_VIEW);
			lvc.mask    = LVCF_TEXT;
			lvc.pszText = szLvCol;
			i           = LVC_FIRST;
			do
			{
				LoadString(e_hInstance, IDS_FIRST_COL + i, szLvCol, countof(szLvCol));
				jListView.InsertColumn(i, &lvc);
				i++;
			}
			while(i < LVC_COUNT);
			jListView.SetColumnOrder(e_iHdOrder, countof(e_iHdOrder));
			LoadString(e_hInstance, IDP_CONVERTED_FORMAT, szResText, countof(szResText));
			if(m_jConvertFormatDlg.m_nOv2Btn == BST_CHECKED)
			{
				iFTW        = FTW_OV2;
				pszFileExtT = m_szOv2Ext;
			}
			else
			{
				iFTR = FTR_OV2;
				i    = m_jConvertFormatDlg.m_iSelExt;
				iFTW = spszFTW[i];
				if(i >= 2)
					i = (i - 2) / (FTW_CSVU8 - FTW_CSVA + 1) + 3;
				pszFileExtT = spszFileExt[i];
			}
			CopyMemory(szFileNameT, szFileNameF, ccName * sizeof(TCHAR));
			pszFileNameT = szFileNameT + ccName;
			nFEDSel      = ID_NO;
			m_nPoiDupSel = IDC_RETAIN_ALL;
			pvFile       = NULL;
			cbMemory     = 0;
			nResId       = 0;
			ceFile       = 0;
			do
			{
				if(PeekUpdateMessage(TRUE))
				{
					nResId = MAX_UINT;
					break;
				}
				if(wfd.cFileName[0] == CHR_DOT)
					continue;
				if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					continue;
				if(m_jConvertFormatDlg.m_nOv2Btn == BST_CHECKED)
				{
					pszFileExtF = PathFindExtension(wfd.cFileName);
					if(!*pszFileExtF)
						continue;
					pszFileExtF++;
					i = 0;
					do
					{
						if(!lstrcmpi(pszFileExtF, spszFileExt[i]))
							break;
						i++;
					}
					while(i < countof(spszFileExt));
					if(i == countof(spszFileExt))
						continue;
					iFTR = spszFTR[i];
				}
				ccName = lstrlen(wfd.cFileName);
				CopyMemory(pszFileNameF, wfd.cFileName, (ccName + 1) * sizeof(TCHAR));
				i = ccName - countof(m_szOv2Ext) + 1;
				CopyMemory(pszFileNameT, wfd.cFileName, i * sizeof(TCHAR));
				CopyMemory(pszFileNameT + i, pszFileExtT, sizeof(m_szOv2Ext));
				dwFileAttr = GetFileAttributes(szFileNameT);
				if(dwFileAttr != INVALID_VALUE)
				{
					if(nFEDSel == IDC_NO_ALL)
						continue;
					if(nFEDSel != IDC_YES_ALL)
					{
						jFED.m_pszFileName = pszFileNameT;
						nFEDSel            = jFED.DialogBox(e_hInstance, IDD_FILE_EXIST, *this);
						SetCursor(LoadCursor(IDC_WAIT));
						if(nFEDSel == ID_CANCEL)
						{
							nResId = MAX_UINT;
							break;
						}
						if((nFEDSel == ID_NO) || (nFEDSel == IDC_NO_ALL))
							continue;
					}
					if(dwFileAttr & FILE_ATTRIBUTE_READONLY)
						SetFileAttributes(szFileNameT, dwFileAttr & ~FILE_ATTRIBUTE_READONLY);
				}
				wsprintf(szSbText, szResText, ceFile, pszFileNameF);
				m_jStatusBar.SetWindowText(szSbText);
				hFile = CreateFile(szFileNameF, GENERIC_READ, OPEN_EXISTING);
				if(hFile == INVALID_HANDLE)
				{
					nResId = IDS_CANNOT_OPEN;
					break;
				}
				cbFile  = wfd.nFileSizeLow;
				cbAlloc = cbFile + sizeof(WCHAR);
				if(cbAlloc > cbMemory)
				{
					pvFile = realloc(pvFile, cbAlloc);
					if(!pvFile)
					{
						nResId = IDS_MEMORY_ERROR_FILE;
						break;
					}
					cbMemory = cbAlloc;
				}
				if(!ReadFile(hFile, pvFile, cbFile, &cbFile))
				{
					nResId = IDS_CANNOT_READ;
					break;
				}
				m_jListView.m_hWnd = jListView.m_hWnd;
				m_jListView.DeleteAllItems();
				m_jListView.SetAction(AT_CUT);
				m_opf                   = OPF_READ;
				m_fst                   = FST_GENERAL;
				m_pr64.byType           = UNUSED;
				m_bRPF                  = FALSE;
				pwEnding                = (PWCHAR)((PBYTE)pvFile + cbFile);
				m_jFileDlg.nFilterIndex = iFTR;
				switch(iFTR)
				{
				case FTR_OV2:
					nResId = ReadOv2Record(pvFile, cbFile);
					break;
				case FTR_KML:
					*pwEnding = CHR_NULL;
					nResId    = ReadKmlRecord(pvFile, cbFile);
					break;
				case FTR_GPX:
					*pwEnding = CHR_NULL;
					nResId    = ReadGpxRecord(pvFile, cbFile);
					break;
				case FTR_GPI:
					nResId = ReadGpiRecord(pvFile, cbFile);
					break;
				case FTR_CSV:
				case FTR_ASC:
				case FTR_POI:
					*pwEnding = CHR_LF;
					nResId    = ReadCsvRecordA(pvFile, cbFile);
					break;
				default:
					*pwEnding = CHR_CR;
					nResId    = ReadTxtRecordA(pvFile, cbFile);
				}
				CloseHandle(hFile);
				if(nResId)
					break;
				if(m_jListView.GetItemCount())
				{
					hFile = CreateFile(szFileNameT, GENERIC_WRITE, CREATE_ALWAYS);
					if(hFile == INVALID_HANDLE)
					{
						nResId       = IDS_CANNOT_WRITE;
						pszFileNameF = pszFileNameT;
						break;
					}
					m_opf                   = OPF_WRITE;
					m_jFileDlg.nFilterIndex = iFTW;
					switch(iFTW)
					{
					case FTW_OV2:
						nResId = WriteOv2Record(hFile);
						break;
					case FTW_KML:
						nResId = WriteKmlRecord(hFile, NULL);
						break;
					case FTW_GPX:
						nResId = WriteGpxRecord(hFile);
						break;
					case FTW_CSVA:
					case FTW_CSVU:
					case FTW_CSVU8:
					case FTW_ASCA:
					case FTW_ASCU:
					case FTW_ASCU8:
					case FTW_POIA:
					case FTW_POIU:
					case FTW_POIU8:
						nResId = WriteCsvRecord(hFile);
						break;
					default:
						nResId = WriteTxtRecord(hFile);
					}
					CloseHandle(hFile);
					if(nResId)
					{
						pszFileNameF = pszFileNameT;
						break;
					}
					ceFile++;
				}
				m_jListView.m_hWnd = hListView;
			}
			while(FindNextFile(hFind, &wfd));
			if(!nResId || (nResId == MAX_UINT))
			{
				if((nResId == MAX_UINT) || (GetLastError() == ERROR_NO_MORE_FILES))
				{
					nResId       = IDS_CONVERT_FORMAT_COMPLETED;
					pszFileNameF = (PTSTR)ceFile;
				}
				else
				{
					nResId       = IDS_DISK_STRUCT_ERROR;
					pszFileNameF = NULL;
				}
			}
			FindClose(hFind);
			free(pvFile);
			jListView.DestroyWindow();
			m_jListView.m_hWnd        = hListView;
			m_jOptionDlg.m_nDmsFormat = nDmsFormat;
			m_bMyBP                   = bMyBP;
			m_bRPF                    = bRPF;
			m_pr64                    = pr64;
			m_jFileDlg.nFilterIndex   = dwFilterIndex;
			m_fst                     = fst;
			m_bBackup                 = bBackup;
		}
		else
		{
			nResId       = IDS_DISK_STRUCT_ERROR;
			pszFileNameF = NULL;
		}
		HideCursor();
		SetCursor(LoadCursor(IDC_ARROW));
		m_bSetReady = TRUE;
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, (LPARAM)pszFileNameF);
	}
	m_jToolbar.PressButton(ID_FILE_CONVERT, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from File/Exit

AFXINLINE void CTomPoiEdr::OnFileExit()
{
	TRACE(_T("Entering CTomPoiEdr::OnFileExit\n"));

	PostMessage(WM_CLOSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Undo

AFXINLINE void CTomPoiEdr::OnPoiUndo()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiUndo\n"));

	DoAction(IDP_UNDOING_ACTION);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Redo

AFXINLINE void CTomPoiEdr::OnPoiRedo()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiRedo\n"));

	DoAction(IDP_REDOING_ACTION);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Add

AFXINLINE void CTomPoiEdr::OnPoiAdd()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiAdd\n"));

	AddPoiRecord(MAX_INT);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Insert

AFXINLINE void CTomPoiEdr::OnPoiInsert()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiInsert\n"));

	AddPoiRecord(m_jListView.GetNextItem(LVNI_SELECTED));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Combine Columns

AFXINLINE void CTomPoiEdr::OnPoiCombineCol()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiCombineCol\n"));

	TCHAR       szCombText[e_eccNameCol];
	CCombColDlg jCombColDlg(szCombText);
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_COMBINE_COL);
	m_jToolbar.UpdateWindow();
	if(jCombColDlg.DialogBox(e_hInstance, IDD_COMBINE_COL, *this) == ID_OK)
	{
		SCINT  siCbCol[] = { LVC_NAME, LVC_TEL, LVC_MEMO };
		TCHAR  szNewText[countof(szCombText)], szText[countof(szCombText)], szSerial[32];
		BOOL   bAllItem;
		int    iFCbCol, iBCbCol, iDestCol, ceLvItem, ceSelItem, iLvItem, iSerial;
		LVITEM lvi;
		LoadString(e_hInstance, IDP_COMBINING_COL, szText, countof(szText));
		m_jStatusBar.SetWindowText(szText);
		m_jListView.SetAction(AT_NEW);
		if((jCombColDlg.m_iFCbSel >= CCombColDlg::m_eFCbName) &&
			(jCombColDlg.m_iFCbSel <= CCombColDlg::m_eFCbMemo))
		{
			iFCbCol  = siCbCol[jCombColDlg.m_iFCbSel - CCombColDlg::m_eFCbName];
			iDestCol = iFCbCol;
		}
		else
			iDestCol = -1;
		if((jCombColDlg.m_iBCbSel >= CCombColDlg::m_eBCbName) &&
			(jCombColDlg.m_iBCbSel <= CCombColDlg::m_eBCbMemo))
		{
			iBCbCol = siCbCol[jCombColDlg.m_iBCbSel - CCombColDlg::m_eBCbName];
			if(iDestCol == -1)
				iDestCol = iBCbCol;
		}
		ceLvItem  = m_jListView.GetItemCount();
		ceSelItem = m_jListView.GetSelCount();
		if(ceSelItem && (ceSelItem != ceLvItem))
		{
			bAllItem = FALSE;
			iLvItem  = m_jListView.GetNextItem(LVNI_SELECTED);
		}
		else
		{
			bAllItem  = TRUE;
			ceSelItem = ceLvItem;
			iLvItem   = 0;
		}
		if(iLvItem || !m_bMyBP)
			iLvItem--;
		lvi.cchTextMax = countof(szText);
		if((jCombColDlg.m_iFCbSel == CCombColDlg::m_eFCbSerial) ||
			(jCombColDlg.m_iBCbSel == CCombColDlg::m_eBCbSerial))
		{
			int iQuotient, ceSerial;
			TrimString(szCombText);
			iSerial = _ttoi(szCombText);
			if((*szCombText == _T('0')) && (szCombText[1] != CHR_NULL))
			{
				iQuotient = ceSelItem + iSerial - 1;
				ceSerial  = 0;
				do
				{
					iQuotient /= 10;
					ceSerial++;
				}
				while(iQuotient);
			}
			else
				ceSerial = 1;
			wsprintf(szSerial, _T("%%.%dd"), ceSerial);
		}
		while(bAllItem ? (++iLvItem < ceSelItem) :
			((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
		{
			PutGoogleMaps(m_eGmPutDelete, iLvItem);
			if(jCombColDlg.m_iFCbSel == CCombColDlg::m_eFCbText)
				lstrcpy(szNewText, szCombText);
			else if(jCombColDlg.m_iFCbSel == CCombColDlg::m_eFCbSerial)
			{
				wsprintf(szNewText, szSerial, iSerial);
				iSerial++;
			}
			else
			{
				lvi.iSubItem = iFCbCol;
				lvi.pszText  = szNewText;
				m_jListView.GetItemText(iLvItem, &lvi);
			}
			if(jCombColDlg.m_iBCbSel == CCombColDlg::m_eBCbText)
				lvi.pszText = szCombText;
			else
			{
				lvi.pszText = szText;
				if(jCombColDlg.m_iBCbSel == CCombColDlg::m_eBCbSerial)
				{
					wsprintf(szText, szSerial, iSerial);
					iSerial++;
				}
				else
				{
					lvi.iSubItem = iBCbCol;
					m_jListView.GetItemText(iLvItem, &lvi);
				}
			}
			StrCatBuff(szNewText, lvi.pszText, countof(szNewText));
			if(iDestCol == LVC_NAME)
			{
				ValidateNameCol(szNewText);
				lvi.iSubItem = iDestCol;
				lvi.pszText  = szNewText;
				m_jListView.SetItemText(iLvItem, &lvi);
			}
			else if(iDestCol == LVC_TEL)
				ValidateTelCol(iLvItem, szNewText);
			else
				ValidateMemoCol(iLvItem, szNewText);
		}
		AutoSizeLvCol(iDestCol, FALSE);
		m_bGmSelItem  = TRUE;
		m_bFileDirty  = TRUE;
		m_bSetCaption = TRUE;
		m_bSetReady   = TRUE;
	}
	m_jToolbar.PressButton(ID_POI_COMBINE_COL, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Edit

AFXINLINE void CTomPoiEdr::OnPoiEdit()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiEdit\n"));

	BeginLvEdit(m_jListView.GetNextItem(LVNI_SELECTED));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Delete

AFXINLINE void CTomPoiEdr::OnPoiDelete()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiDelete\n"));

	m_jToolbar.PressButton(ID_POI_DELETE);
	m_jToolbar.UpdateWindow();
	DeletePoiRecord(IDP_DELETING_POI);
	m_jToolbar.PressButton(ID_POI_DELETE, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Clear Columns

AFXINLINE void CTomPoiEdr::OnPoiClearCol()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiClearCol\n"));

	CClearColDlg jClearColDlg;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_CLEAR_COL);
	m_jToolbar.UpdateWindow();
	if(jClearColDlg.DialogBox(e_hInstance, IDD_CLEAR_COL, *this) == ID_OK)
	{
		TCHAR  szText[e_eccNameCol];
		BOOL   bAllItem;
		int    i, ceLvItem, ceSelItem, iLvItem, ccText;
		LVITEM lvi;
		LoadString(e_hInstance, IDP_CLEARING_COL, szText, countof(szText));
		m_jStatusBar.SetWindowText(szText);
		m_jListView.SetAction(AT_NEW);
		ceLvItem  = m_jListView.GetItemCount();
		ceSelItem = m_jListView.GetSelCount();
		if(ceSelItem && (ceSelItem != ceLvItem))
		{
			bAllItem = FALSE;
			iLvItem  = m_jListView.GetNextItem(LVNI_SELECTED);
		}
		else
		{
			bAllItem  = TRUE;
			ceSelItem = ceLvItem;
			iLvItem   = 0;
		}
		if(iLvItem || !m_bMyBP)
			iLvItem--;
		while(bAllItem ? (++iLvItem < ceSelItem) :
			((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
		{
			for(i = 0; i < CClearColDlg::m_eceCol; i++)
			{
				if(jClearColDlg.m_nCol[i] == BST_UNCHECKED)
					continue;

				if((i == CClearColDlg::m_eNameLCol) || (i == CClearColDlg::m_eNameTCol))
				{
					lvi.iSubItem   = LVC_NAME;
					lvi.cchTextMax = countof(szText);
				}
				else
				{
					lvi.iSubItem   = i - CClearColDlg::m_eLngCol + LVC_LNG;
					lvi.cchTextMax = 2;
				}
				lvi.pszText = szText;
				ccText      = m_jListView.GetItemText(iLvItem, &lvi);
				if(!ccText)
					continue;

				if(i == CClearColDlg::m_eNameLCol)
				{
					lvi.pszText = szText;
					while(IsValidNumber(*lvi.pszText))
						lvi.pszText++;
					if((lvi.pszText == szText) || !*lvi.pszText)
						continue;
				}
				else if(i == CClearColDlg::m_eNameTCol)
				{
					lvi.pszText = szText + ccText;
					do
						lvi.pszText--;
					while((lvi.pszText >= szText) && IsValidNumber(*lvi.pszText));
					if(lvi.pszText < szText)
						continue;
					lvi.pszText++;
					if(!*lvi.pszText)
						continue;
					*lvi.pszText = CHR_NULL;
					lvi.pszText  = szText;
				}
				else
					lvi.pszText = NULL;
				if(lvi.pszText)
					TrimString(lvi.pszText);
				PutGoogleMaps(m_eGmPutDelete, iLvItem);
				m_jListView.SetItemText(iLvItem, &lvi);
				m_bFileDirty = TRUE;
			}
		}
		if((m_iLvBaseItem != LVI_ERR) &&
			((jClearColDlg.m_nCol[CClearColDlg::m_eLngCol] == BST_CHECKED) ||
			(jClearColDlg.m_nCol[CClearColDlg::m_eLatCol] == BST_CHECKED)))
			SetBasePoint(m_iLvBaseItem, FALSE);
		AutoSizeLvCol(LVC_ALL, FALSE);
		m_bGmSelItem  = TRUE;
		m_bSetCaption = TRUE;
		m_bSetReady   = TRUE;
	}
	m_jToolbar.PressButton(ID_POI_CLEAR_COL, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Cut

AFXINLINE void CTomPoiEdr::OnPoiCut()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiCut\n"));

	if(CopyPoiRecord())
		DeletePoiRecord(IDP_CUTTING_POI);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Copy

AFXINLINE void CTomPoiEdr::OnPoiCopy()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiCopy\n"));

	CopyPoiRecord();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Paste

AFXINLINE void CTomPoiEdr::OnPoiPaste()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiPaste\n"));

	UINT nResId;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_jToolbar.PressButton(ID_POI_PASTE);
	m_jToolbar.UpdateWindow();
	m_cePoiDupDel = 0;
	if(OpenClipboard())
	{
		TCHAR   szResText[MAX_STR];
		PCCHAR  pcMemory;
		int     cbMemory, ccData, cePastePoi, ceLvCol, ceLvItem, iFirstItem;
		UINT    nCbFmt;
		PCBD    pCBD;
		LVITEM  lvi;
		HMEMORY hMemory;
		m_jListView.UpdateWindow();
		m_jListView.SetWindowRedraw(FALSE);
		LoadString(e_hInstance, IDP_PASTING_POI, szResText, countof(szResText));
		m_jStatusBar.SetWindowText(szResText);
		m_jListView.SetAction(AT_NEW);
		nCbFmt = (m_fst == FST_SC) ? m_nPoiScCbFmt : m_nPoiCbFmt;
		if(IsClipboardFormatAvailable(nCbFmt))
			ceLvCol = LVC_COUNT - 1;
		else
		{
			nCbFmt  = m_nItnCbFmt;
			ceLvCol = LVCI_COUNT;
		}
		m_nPoiDupSel = IDC_RETAIN;
		hMemory      = GetClipboardData(nCbFmt);
		pcMemory     = (PCCHAR)LockMemory(hMemory);
		cbMemory     = sizeof(int);
		cePastePoi   = *(PCINT)pcMemory;
		ceLvItem     = m_jListView.GetItemCount();
		iFirstItem   = m_jListView.GetNextItem(LVNI_SELECTED);
		if(iFirstItem == RV_ERR)
			iFirstItem = ceLvItem;
		else if(!iFirstItem && m_bMyBP)
			iFirstItem = 1;
		lvi.iItem     = iFirstItem;
		lvi.mask      = LVIF_STATE | LVIF_TEXT;
		lvi.state     = UNUSED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(&lvi);
		lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
#ifndef UNICODE
		char szText[e_eccNameCol];
		lvi.pszText = szText;
#endif
		for(int i = 0, j; i < cePastePoi; i++)
		{
			pCBD = (PCBD)(pcMemory + cbMemory);
#ifdef UNICODE
			lvi.pszText = pCBD->szwData;
#else
			WCStoMBS(pCBD->szwData, pCBD->ccData[LVC_NAME], szText, sizeof(szText));
#endif
			if((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(lvi.pszText))
				for(j = LVC_FIRST, ccData = 0; j < ceLvCol; j++)
					ccData += pCBD->ccData[j];
			else
			{
				lvi.iSubItem = 0;
				lvi.iItem    = m_jListView.InsertItem(&lvi);
				ccData       = pCBD->ccData[LVC_NAME];
				j            = LVC_LNG;
				do
				{
#ifdef UNICODE
					lvi.pszText = pCBD->szwData + ccData;
#else
					WCStoMBS(pCBD->szwData + ccData, pCBD->ccData[j], szText, sizeof(szText));
#endif
					if((j == LVC_LNG) || (j == LVC_LAT))
					{
						if(m_fst != FST_CPB)
							ValidateCoordCol(lvi.iItem, j, lvi.pszText, FALSE);
					}
					else if(j == LVC_TEL)
						ValidateTelCol(lvi.iItem, lvi.pszText);
					else if(!m_bRPF)
						ValidateMemoCol(lvi.iItem, lvi.pszText);
					ccData += pCBD->ccData[j];
					j++;
				}
				while(j < ceLvCol);
				lvi.iItem++;
			}
			if(nCbFmt == m_nItnCbFmt)
				ccData++;
			cbMemory += ccData * sizeof(WCHAR) + m_ecbCBD;
		}
		UnlockMemory(hMemory);
		CloseClipboard();
		SetBasePoint(LVI_FIND, FALSE);
		AutoSizeLvCol(LVC_ALL, FALSE);
		m_jListView.SetWindowRedraw();
		if(ceLvItem != m_jListView.GetItemCount())
		{
			EnsureLvItemVisible(iFirstItem);
			m_bFileDirty  = TRUE;
			m_bSetCaption = TRUE;
		}
		nResId = m_cePoiDupDel ? IDS_POI_DUP_DEL : 0;
	}
	else
		nResId = IDS_OPEN_CB_ERROR;
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, m_cePoiDupDel);
	m_jToolbar.PressButton(ID_POI_PASTE, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Copy Coordinates

AFXINLINE void CTomPoiEdr::OnPoiCopyCoord()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiCopyCoord\n"));

	UINT nResId;
	if(OpenClipboard())
	{
		HMEMORY hCoord;
		hCoord = AllocMemory(m_eccAllocCC * sizeof(TCHAR));
		if(hCoord)
		{
			PTSTR  pszCoord;
			int    iLvItem, ccText;
			UINT   nCompassDir;
			LVITEM lvi;
			nCompassDir    = m_jOptionDlg.m_nCompassDir;
			pszCoord       = (PTSTR)LockMemory(hCoord);
			iLvItem        = m_jListView.GetNextItem(LVNI_SELECTED);
			lvi.iSubItem   = LVC_LAT;
			lvi.pszText    = pszCoord;
			lvi.cchTextMax = m_eccAllocCC / 2;
			ccText         = m_jListView.GetItemText(iLvItem, &lvi);
			if(nCompassDir == BST_UNCHECKED)
			{
				m_jOptionDlg.m_nCompassDir = BST_CHECKED;
				ccText = SetCoordCompassDir(LVC_LAT, pszCoord, ccText);
			}
			pszCoord += ccText;
			*pszCoord = CHR_CA;
			pszCoord++;
			*pszCoord = CHR_SP;
			pszCoord++;
			lvi.iSubItem = LVC_LNG;
			lvi.pszText  = pszCoord;
			ccText       = m_jListView.GetItemText(iLvItem, &lvi);
			if(nCompassDir == BST_UNCHECKED)
			{
				SetCoordCompassDir(LVC_LNG, pszCoord, ccText);
				m_jOptionDlg.m_nCompassDir = BST_UNCHECKED;
			}
			EmptyClipboard();
			UnlockMemory(hCoord);
			SetClipboardData(CF_TCHAR, hCoord);
			CloseClipboard();
			return;
		}
		nResId = IDS_MEMORY_ERROR_CB;
	}
	else
		nResId = IDS_OPEN_CB_ERROR;
	DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, UNUSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Paste Coordinates

AFXINLINE void CTomPoiEdr::OnPoiPasteCoord()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiPasteCoord\n"));

	if(!OpenClipboard())
	{
		DisplayMessage(IDS_OPEN_CB_ERROR, MB_ICONINFORMATION | MB_OK, UNUSED);
		return;
	}

	SCTCHAR szDir[] = _T("EWSN");
	TCHAR   szResText[MAX_STR], szCoord[64], szLng[32], szLat[countof(szLng)];
	int     ccText, iLng, iLat, iFirstItem, iLvItem;
	PTSTR   pszLng, pszLat;
	HMEMORY hCoord;
	m_jListView.UpdateWindow();
	LoadString(e_hInstance, IDP_PASTING_POI, szResText, countof(szResText));
	m_jStatusBar.SetWindowText(szResText);
	m_jListView.SetAction(AT_NEW);
	hCoord = GetClipboardData(CF_TCHAR);
	lstrcpyn(szCoord, (PTSTR)LockMemory(hCoord), countof(szCoord));
	CharUpper(szCoord);
	pszLat = StrChr(szCoord, CHR_TAB);
	if(pszLat)
		*pszLat = CHR_CA;
	ccText = TrimString(szCoord);
	pszLng = szCoord;
	pszLat = StrChr(szCoord, CHR_CA);
	if(!pszLat)
	{
		pszLat = StrPBrk(szCoord, szDir);
		if(pszLat)
		{
			if(pszLat == szCoord)
				pszLat = StrPBrk(pszLat + 1, szDir);
			else
				pszLat++;
			if(pszLat)
				MoveMemory(pszLat + 1, pszLat, (szCoord - pszLat + ccText + 1) * sizeof(TCHAR));
		}
		else
			pszLat = StrChr(szCoord, CHR_SP);
	}
	if(pszLat)
	{
		*pszLat = CHR_NULL;
		pszLat++;
	}
	if(pszLat && *pszLat)
	{
		PCTCHAR pcLngDir, pcLatDir;
		pcLngDir = StrPBrk(pszLng, szDir);
		pcLatDir = StrPBrk(pszLat, szDir);
		if(pcLngDir || pcLatDir)
		{
			if((pcLngDir && ((*pcLngDir == _T('S')) || (*pcLngDir == _T('N')))) ||
				(pcLatDir && ((*pcLatDir == _T('E')) || (*pcLatDir == _T('W')))))
			{
				iLng = LVC_LAT;
				iLat = LVC_LNG;
			}
			else
			{
				iLng = LVC_LNG;
				iLat = LVC_LAT;
			}
		}
		else
		{
			iLng = e_iHdOrder[LVCR_LNG];
			iLat = e_iHdOrder[LVCR_LAT];
		}
		pszLng = NormalizeCoord(LVC_LNG | LVC_LAT, pszLng, szLng) ? szLng : NULL;
		pszLat = NormalizeCoord(LVC_LNG | LVC_LAT, pszLat, szLat) ? szLat : NULL;
	}
	else
	{
		iLng   = LVC_LNG;
		pszLng = NULL;
		iLat   = LVC_LAT;
		pszLat = NULL;
	}
	iFirstItem = m_jListView.GetNextItem(LVNI_SELECTED);
	iLvItem    = iFirstItem;
	do
	{
		PutGoogleMaps(m_eGmPutDelete, iLvItem);
		ValidateCoordCol(iLvItem, iLng, pszLng, TRUE);
		ValidateCoordCol(iLvItem, iLat, pszLat, TRUE);
		if(iLvItem || !m_bMyBP)
			m_bFileDirty = TRUE;
		iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem);
	}
	while(iLvItem != RV_ERR);
	UnlockMemory(hCoord);
	CloseClipboard();
	if(m_iLvBaseItem != LVI_ERR)
		SetBasePoint(m_iLvBaseItem, TRUE);
	AutoSizeLvCol(LVC_LNG, FALSE);
	AutoSizeLvCol(LVC_LAT, FALSE);
	if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
		AutoSizeLvCol(LVC_DIST, FALSE);
	EnsureLvItemVisible(iFirstItem);
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Query Coordinates

AFXINLINE void CTomPoiEdr::OnPoiQueryCoord()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiQueryCoord\n"));

	UINT      nResId;
	HINTERNET hInternet;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_QUERY_COORD);
	m_jToolbar.UpdateWindow();
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_bSetReady = FALSE;
	hInternet   = InternetOpen(e_pszTpeAppName, INTERNET_OPEN_TYPE_PRECONFIG);
	if(hInternet)
	{
		TCHAR  szMemo[e_eccMemoCol], szMemoUtf8[countof(szMemo) * FT_UTF8S3 * 3];
		TCHAR  szQueryUrl[MAX_STR], szQueryData[MAX_STR];
		TCHAR  szBuffer[countof(szQueryUrl) + countof(szMemoUtf8)];
		TCHAR  szSuggest[MAX_STR], szIgrMark[MAX_STR], szSugAddr[countof(szMemo)], szDummy[2];
		TCHAR  szStatus[32], szStatusEnd[countof(szStatus)];
		TCHAR  szNoAnswer[countof(szStatus)], szOverLimit[countof(szStatus)];
		TCHAR  szReqDenied[countof(szStatus)];
		TCHAR  szAddr[countof(szStatus)], szAddrEnd[countof(szStatus)];
		TCHAR  szLoc[countof(szStatus)], szLocEnd[countof(szStatus)];
		TCHAR  szLng[countof(szStatus)], szLngEnd[countof(szStatus)];
		TCHAR  szLat[countof(szStatus)], szLatEnd[countof(szStatus)];
		PSTR   pszBuffer;
		PTCHAR pcIgrMark;
		PTSTR  pszXmlData, pszToken, pszAddr, pszLng, pszLat, pszTemp;
		PWSTR  pszwBuffer;
		BOOL   bAllItem, bSizeCoord, bSizeMemo;
		int    ceSelItem, iLvItem, ccStatus, ccAddr, ccAddrEnd, ccLoc, ccLng, ccLat;
		int    ccMemo, ccSugAddr;
		int    iRightPos[m_eceSbPart], cxCurSbText;
		DWORD  cbRead, cbDone, ceTick;
		SIZE   size;
		LVITEM lvi;
		HHTTP  hHttp;
		m_jListView.SetAction(AT_NEW);
		pszBuffer  = (PSTR)malloc(m_ecbGmBuffer);
		pszwBuffer = (PWSTR)malloc(m_ecbGmBuffer * sizeof(WCHAR));
		m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
		m_jProgress.CreateWindow(m_ePbMqStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
		m_jProgress.SetMarquee(m_ePbMqIntl);
		LoadString(e_hInstance, IDS_GOOGLE_QUERY_COORD_URL, szQueryUrl, countof(szQueryUrl));
		LoadString(e_hInstance, IDP_GOOGLE_QUERYING_DATA, szQueryData, countof(szQueryData));
		LoadString(e_hInstance, IDS_GOOGLE_QUERY_SUGGEST, szSuggest, countof(szSuggest));
		LoadString(e_hInstance, IDS_GOOGLE_ADDR_IGRMARK, szIgrMark, countof(szIgrMark));
		ccStatus = LoadString(e_hInstance, IDS_XMLT_STATUS, szStatus, countof(szStatus));
		LoadString(e_hInstance, IDS_XMLT_STATUS_END, szStatusEnd, countof(szStatusEnd));
		LoadString(e_hInstance, IDS_XMLT_NO_ANSWER, szNoAnswer, countof(szNoAnswer));
		LoadString(e_hInstance, IDS_XMLT_OVER_LIMIT, szOverLimit, countof(szOverLimit));
		LoadString(e_hInstance, IDS_XMLT_REQ_DENIED, szReqDenied, countof(szReqDenied));
		ccAddr    = LoadString(e_hInstance, IDS_XMLT_ADDR, szAddr, countof(szAddr));
		ccAddrEnd = LoadString(e_hInstance, IDS_XMLT_ADDR_END, szAddrEnd, countof(szAddrEnd));
		ccLoc     = LoadString(e_hInstance, IDS_XMLT_LOC, szLoc, countof(szLoc));
		LoadString(e_hInstance, IDS_XMLT_LOC_END, szLocEnd, countof(szLocEnd));
		ccLng = LoadString(e_hInstance, IDS_XMLT_LNG, szLng, countof(szLng));
		LoadString(e_hInstance, IDS_XMLT_LNG_END, szLngEnd, countof(szLngEnd));
		ccLat = LoadString(e_hInstance, IDS_XMLT_LAT, szLat, countof(szLat));
		LoadString(e_hInstance, IDS_XMLT_LAT_END, szLatEnd, countof(szLatEnd));
#if (defined(VER_EN_USA) || defined(VER_ESPANA) || defined(VER_FRANCE))
		TCHAR szCountry1[countof(szStatus)], szCountry2[countof(szCountry1)];
		int   ccCountry1, ccCountry2;
		szCountry1[0] = CHR_CA;
		szCountry1[1] = CHR_SP;
		ccCountry1    = LoadString(e_hInstance, IDS_COUNTRY_NAME1,
			szCountry1 + 2, countof(szCountry1) - 2) + 2;
		szCountry2[0] = CHR_CA;
		szCountry2[1] = CHR_SP;
		ccCountry2    = LoadString(e_hInstance, IDS_COUNTRY_NAME2,
			szCountry2 + 2, countof(szCountry2) - 2) + 2;
#endif
		ceSelItem = m_jListView.GetSelCount();
		if(ceSelItem)
			bAllItem = FALSE;
		else
		{
			bAllItem  = TRUE;
			ceSelItem = m_jListView.GetItemCount();
		}
		iLvItem     = -1;
		bSizeCoord  = ceSelItem != 1;
		bSizeMemo   = bSizeCoord;
		cxCurSbText = 0;
		nResId      = 0;
		for(int i = 0, j; bAllItem ? (++iLvItem < ceSelItem) :
			((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR); i++)
		{
			GetTextExtent(m_hSbDC, szBuffer,
				wsprintf(szBuffer, szQueryData, i, ceSelItem), &size);
			size.cx += m_ecxPbPadL;
			if(cxCurSbText != size.cx)
			{
				cxCurSbText = size.cx;
				m_jProgress.MoveWindow(cxCurSbText, m_ecyPbPadT,
					iRightPos[m_eInfoPart] - cxCurSbText - m_ecxPbPadR,
					m_cyStatusBar - m_ecyPbPadB);
			}
			m_jStatusBar.SetWindowText(szBuffer);
			if(PeekUpdateMessage(TRUE))
				break;
			if(bAllItem)
			{
				lvi.iSubItem   = LVC_LNG;
				lvi.pszText    = szDummy;
				lvi.cchTextMax = countof(szDummy);
				if(m_jListView.GetItemText(iLvItem, &lvi))
				{
					lvi.iSubItem = LVC_LAT;
					if(m_jListView.GetItemText(iLvItem, &lvi))
						continue;
				}
			}
			lvi.iSubItem   = LVC_MEMO;
			lvi.pszText    = szMemo;
			lvi.cchTextMax = countof(szMemo);
			ccMemo         = m_jListView.GetItemText(iLvItem, &lvi);
			pcIgrMark      = StrPBrk(szMemo, szIgrMark);
			if(pcIgrMark)
			{
				*pcIgrMark = CHR_NULL;
				ccMemo     = TrimString(szMemo);
			}
			pszLng = NULL;
			pszLat = NULL;
			if(ccMemo)
			{
				CopyMemory(szMemoUtf8, szMemo, (ccMemo + 1) * sizeof(TCHAR));
				EncodeUrl(szMemoUtf8);
				wsprintf(szBuffer, szQueryUrl, szMemoUtf8);
				hHttp = InternetOpenUrl(hInternet, szBuffer);
				if(!hHttp)
				{
					nResId = IDS_GOOGLE_CONNECT_ERROR;
					break;
				}
				cbRead = 0;
				cbDone = 0;
				do
				{
					cbDone += cbRead;
					InternetReadFile(hHttp, pszBuffer + cbDone,
						m_ecbGmBuffer - cbDone - 1, &cbRead);
				}
				while(cbRead);
				InternetCloseHandle(hHttp);
				if(!cbDone)
				{
					nResId = IDS_GOOGLE_QUERY_COORD_ERROR;
					break;
				}
				pszBuffer[cbDone] = CHR_NULL;
#ifdef UNICODE
				MBStoWCS(CP_UTF8, pszBuffer, cbDone + 1, pszwBuffer, m_ecbGmBuffer);
				pszXmlData = pszwBuffer;
#else
				WCStoMBS(pszwBuffer, MBStoWCS(CP_UTF8, pszBuffer, cbDone + 1,
					pszwBuffer, m_ecbGmBuffer), pszBuffer, m_ecbGmBuffer);
				pszXmlData = pszBuffer;
#endif
				pszAddr    = StrStrI(pszXmlData, szAddr);
				if(pszAddr)
				{
					pszAddr += ccAddr;
					pszToken = StrStrI(pszAddr, szAddrEnd);
					if(pszToken)
					{
						*pszToken = CHR_NULL;
						ccSugAddr = pszToken - pszAddr;
						if(ccSugAddr > (countof(szSugAddr) - 1))
							pszAddr[countof(szSugAddr) - 1] = CHR_NULL;
						ccSugAddr = TrimString(pszAddr);
						CopyMemory(szSugAddr, pszAddr, (ccSugAddr + 1) * sizeof(TCHAR));
						if(lstrcmpi(pszAddr, szMemo))
						{
#if (defined(VER_EN_USA) || defined(VER_ESPANA) || defined(VER_FRANCE))
							pszTemp = StrStrI(pszAddr, szCountry1);
							if(pszTemp && !pszTemp[ccCountry1])
							{
								*pszTemp = CHR_NULL;
								TrimString(pszAddr);
							}
							else
							{
								pszTemp = StrStrI(pszAddr, szCountry2);
								if(pszTemp && !pszTemp[ccCountry2])
								{
									*pszTemp = CHR_NULL;
									TrimString(pszAddr);
								}
							}
#endif
							for(j = 0; *pszAddr && szMemo[j]; pszAddr++)
								if((*pszAddr == szMemo[j]) ||
									(CharLower(*pszAddr) == CharLower(szMemo[j])))
									j++;
						}
						else
							pszAddr = NULL;
						pszToken = StrStrI(pszToken + ccAddrEnd, szLoc);
						if(pszToken)
						{
							pszToken += ccLoc;
							pszTemp   = StrStrI(pszToken, szLocEnd);
							if(pszTemp)
							{
								pszLng = StrStrI(pszToken, szLng);
								pszLat = StrStrI(pszToken, szLat);
								if(pszLng)
								{
									if(pszLng < pszTemp)
									{
										pszLng  += ccLng;
										pszToken = StrStrI(pszLng, szLngEnd);
										if(pszToken)
											*pszToken = CHR_NULL;
										else
											pszLng = NULL;
									}
									else
										pszLng = NULL;
								}
								if(pszLat)
								{
									if(pszLat < pszTemp)
									{
										pszLat  += ccLat;
										pszToken = StrStrI(pszLat, szLatEnd);
										if(pszToken)
											*pszToken = CHR_NULL;
										else
											pszLat = NULL;
									}
									else
										pszLat = NULL;
								}
							}
						}
					}
				}
				else
				{
					pszToken = StrStrI(pszXmlData, szStatus);
					if(!pszToken)
					{
						nResId = IDS_GOOGLE_QUERY_COORD_ERROR;
						break;
					}
					pszToken += ccStatus;
					pszTemp   = StrStrI(pszToken, szStatusEnd);
					if(!pszTemp)
					{
						nResId = IDS_GOOGLE_QUERY_COORD_ERROR;
						break;
					}
					*pszTemp = CHR_NULL;
					if(!lstrcmpi(pszToken, szOverLimit))
					{
						ceTick = GetTickCount();
						do
							PeekUpdateMessage(FALSE);
						while((GetTickCount() - ceTick) < m_eGmQuerySleep);
						iLvItem--;
						i--;
						continue;
					}
					if(!lstrcmpi(pszToken, szReqDenied))
					{
						nResId = IDS_GOOGLE_QUERY_ERROR;
						break;
					}
					if(lstrcmpi(pszToken, szNoAnswer))
					{
						nResId = IDS_GOOGLE_QUERY_COORD_ERROR;
						break;
					}
				}
			}
			PutGoogleMaps(m_eGmPutDelete, iLvItem);
			EnsureLvItemVisible(iLvItem);
			ValidateCoordCol(iLvItem, LVC_LNG, pszLng, TRUE);
			ValidateCoordCol(iLvItem, LVC_LAT, pszLat, TRUE);
			if(bSizeCoord)
			{
				AutoSizeLvCol(LVC_LNG, FALSE);
				AutoSizeLvCol(LVC_LAT, FALSE);
				bSizeCoord = FALSE;
			}
			if(pszLng && pszLat && pszAddr && (*pszAddr || szMemo[j]))
			{
				if(pcIgrMark)
					m_jListView.GetItemText(iLvItem, &lvi);
				wsprintf(szBuffer, szSuggest, szMemo, szSugAddr);
				ValidateMemoCol(iLvItem, szBuffer);
				if(bSizeMemo)
				{
					AutoSizeLvCol(LVC_MEMO, FALSE);
					bSizeMemo = FALSE;
				}
			}
			if(iLvItem || !m_bMyBP)
				m_bFileDirty = TRUE;
		}
		SetBasePoint(LVI_FIND, FALSE);
		AutoSizeLvCol(LVC_LNG, FALSE);
		AutoSizeLvCol(LVC_LAT, FALSE);
		AutoSizeLvCol(LVC_MEMO, FALSE);
		if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
			AutoSizeLvCol(LVC_DIST, FALSE);
		m_jProgress.DestroyWindow();
		InternetCloseHandle(hInternet);
		free(pszwBuffer);
		free(pszBuffer);
	}
	else
		nResId = IDS_GOOGLE_CONNECT_ERROR;
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(ID_POI_QUERY_COORD, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_ceDragDrop--;
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Query Addresses

AFXINLINE void CTomPoiEdr::OnPoiQueryAddr()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiQueryAddr\n"));

	UINT      nResId;
	HINTERNET hInternet;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_QUERY_ADDR);
	m_jToolbar.UpdateWindow();
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_bSetReady = FALSE;
	hInternet   = InternetOpen(e_pszTpeAppName, INTERNET_OPEN_TYPE_PRECONFIG);
	if(hInternet)
	{
		TCHAR  szQueryUrl[MAX_STR], szQueryData[MAX_STR], szBuffer[countof(szQueryUrl) * 2];
		TCHAR  szLng[32], szLat[countof(szLng)], szDummy[2];
		TCHAR  szStatus[32], szStatusEnd[countof(szStatus)];
		TCHAR  szNoAnswer[countof(szStatus)], szOverLimit[countof(szStatus)];
		TCHAR  szReqDenied[countof(szStatus)];
		TCHAR  szCountry1[countof(szStatus)], szCountry2[countof(szCountry1)];
		TCHAR  szAddr[countof(szStatus)], szAddrEnd[countof(szStatus)];
		TCHAR  szAddrNo[countof(szStatus)];
		PSTR   pszBuffer;
		PTSTR  pszXmlData, pszToken, pszAddr, pszTemp;
		PWSTR  pszwBuffer;
		BOOL   bAllItem, bSizeMemo;
		int    ceSelItem, iLvItem, ccStatus, ccAddr, ccCountry1, ccCountry2;
		int    ccLng, ccLat;
		int    iRightPos[m_eceSbPart], cxCurSbText;
		DWORD  cbRead, cbDone, ceTick;
		SIZE   size;
		LVITEM lvi;
		HHTTP  hHttp;
		m_jListView.SetAction(AT_NEW);
		pszBuffer  = (PSTR)malloc(m_ecbGmBuffer);
		pszwBuffer = (PWSTR)malloc(m_ecbGmBuffer * sizeof(WCHAR));
		m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
		m_jProgress.CreateWindow(m_ePbMqStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
		m_jProgress.SetMarquee(m_ePbMqIntl);
		LoadString(e_hInstance, IDS_GOOGLE_QUERY_ADDR_URL, szQueryUrl, countof(szQueryUrl));
		LoadString(e_hInstance, IDP_GOOGLE_QUERYING_DATA, szQueryData, countof(szQueryData));
		ccStatus = LoadString(e_hInstance, IDS_XMLT_STATUS, szStatus, countof(szStatus));
		LoadString(e_hInstance, IDS_XMLT_STATUS_END, szStatusEnd, countof(szStatusEnd));
		LoadString(e_hInstance, IDS_XMLT_NO_ANSWER, szNoAnswer, countof(szNoAnswer));
		LoadString(e_hInstance, IDS_XMLT_OVER_LIMIT, szOverLimit, countof(szOverLimit));
		LoadString(e_hInstance, IDS_XMLT_REQ_DENIED, szReqDenied, countof(szReqDenied));
		ccAddr = LoadString(e_hInstance, IDS_XMLT_ADDR, szAddr, countof(szAddr));
		LoadString(e_hInstance, IDS_XMLT_ADDR_NO, szAddrNo, countof(szAddrNo));
		LoadString(e_hInstance, IDS_XMLT_ADDR_END, szAddrEnd, countof(szAddrEnd));
#if (defined(VER_EN_USA) || defined(VER_ESPANA) || defined(VER_FRANCE))
		szCountry1[0] = CHR_CA;
		szCountry1[1] = CHR_SP;
		ccCountry1    = LoadString(e_hInstance, IDS_COUNTRY_NAME1,
			szCountry1 + 2, countof(szCountry1) - 2) + 2;
		szCountry2[0] = CHR_CA;
		szCountry2[1] = CHR_SP;
		ccCountry2    = LoadString(e_hInstance, IDS_COUNTRY_NAME2,
			szCountry2 + 2, countof(szCountry2) - 2) + 2;
#else
		ccCountry1    = LoadString(e_hInstance, IDS_COUNTRY_NAME1,
			szCountry1, countof(szCountry1));
		ccCountry2    = LoadString(e_hInstance, IDS_COUNTRY_NAME2,
			szCountry2, countof(szCountry2));
#endif
		ceSelItem = m_jListView.GetSelCount();
		if(ceSelItem)
			bAllItem = FALSE;
		else
		{
			bAllItem  = TRUE;
			ceSelItem = m_jListView.GetItemCount();
		}
		iLvItem     = -1;
		bSizeMemo   = ceSelItem != 1;
		cxCurSbText = 0;
		nResId      = 0;
		for(int i = 0; bAllItem ? (++iLvItem < ceSelItem) :
			((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR); i++)
		{
			GetTextExtent(m_hSbDC, szBuffer,
				wsprintf(szBuffer, szQueryData, i, ceSelItem), &size);
			size.cx += m_ecxPbPadL;
			if(cxCurSbText != size.cx)
			{
				cxCurSbText = size.cx;
				m_jProgress.MoveWindow(cxCurSbText, m_ecyPbPadT,
					iRightPos[m_eInfoPart] - cxCurSbText - m_ecxPbPadR,
					m_cyStatusBar - m_ecyPbPadB);
			}
			m_jStatusBar.SetWindowText(szBuffer);
			if(PeekUpdateMessage(TRUE))
				break;
			if(bAllItem)
			{
				lvi.iSubItem   = LVC_MEMO;
				lvi.pszText    = szDummy;
				lvi.cchTextMax = countof(szDummy);
				if(m_jListView.GetItemText(iLvItem, &lvi))
					continue;
			}
			lvi.iSubItem   = LVC_LNG;
			lvi.pszText    = szLng;
			lvi.cchTextMax = countof(szLng);
			ccLng          = m_jListView.GetItemText(iLvItem, &lvi);
			if(ccLng)
			{
				lvi.iSubItem = LVC_LAT;
				lvi.pszText  = szLat;
				ccLat        = m_jListView.GetItemText(iLvItem, &lvi);
				if(ccLat)
				{
					GetCoordDdString(szLng, ccLng);
					GetCoordDdString(szLat, ccLat);
					wsprintf(szBuffer, szQueryUrl, szLat, szLng);
					hHttp = InternetOpenUrl(hInternet, szBuffer);
					if(!hHttp)
					{
						nResId = IDS_GOOGLE_CONNECT_ERROR;
						break;
					}
					cbRead = 0;
					cbDone = 0;
					do
					{
						cbDone += cbRead;
						InternetReadFile(hHttp, pszBuffer + cbDone,
							m_ecbGmBuffer - cbDone - 1, &cbRead);
					}
					while(cbRead);
					InternetCloseHandle(hHttp);
					if(!cbDone)
					{
						nResId = IDS_GOOGLE_QUERY_ADDR_ERROR;
						break;
					}
					pszBuffer[cbDone] = CHR_NULL;
#ifdef UNICODE
					MBStoWCS(CP_UTF8, pszBuffer, cbDone + 1, pszwBuffer, m_ecbGmBuffer);
					pszXmlData = pszwBuffer;
#else
					WCStoMBS(pszwBuffer, MBStoWCS(CP_UTF8, pszBuffer, cbDone + 1,
						pszwBuffer, m_ecbGmBuffer), pszBuffer, m_ecbGmBuffer);
					pszXmlData  = pszBuffer;
#endif
					lvi.pszText = NULL;
					pszAddr     = StrStrI(pszXmlData, szAddr);
					if(pszAddr)
					{
						pszAddr += ccAddr;
						pszToken = StrStrI(pszAddr, szAddrEnd);
						if(pszToken)
						{
							*pszToken = CHR_NULL;
							TrimString(pszAddr);
							pszToken = StrStr(pszAddr, szAddrNo);
							if(!pszToken || (pszToken != pszAddr))
							{
#if (defined(VER_EN_USA) || defined(VER_ESPANA) || defined(VER_FRANCE))
								pszToken = StrStrI(pszAddr, szCountry1);
								if(pszToken && !pszToken[ccCountry1])
									*pszToken = CHR_NULL;
								else
								{
									pszToken = StrStrI(pszAddr, szCountry2);
									if(pszToken && !pszToken[ccCountry2])
										*pszToken = CHR_NULL;
								}
#else
								pszToken = StrStrI(pszAddr, szCountry1);
								if(pszToken && pszToken[ccCountry1])
									pszAddr = pszToken + ccCountry1;
								else
								{
									pszToken = StrStrI(pszAddr, szCountry2);
									if(pszToken && pszToken[ccCountry2])
										pszAddr = pszToken + ccCountry2;
								}
#endif
							}
							lvi.pszText = pszAddr;
						}
					}
					else
					{
						pszToken = StrStrI(pszXmlData, szStatus);
						if(!pszToken)
						{
							nResId = IDS_GOOGLE_QUERY_ADDR_ERROR;
							break;
						}
						pszToken += ccStatus;
						pszTemp   = StrStrI(pszToken, szStatusEnd);
						if(!pszTemp)
						{
							nResId = IDS_GOOGLE_QUERY_ADDR_ERROR;
							break;
						}
						*pszTemp = CHR_NULL;
						if(!lstrcmpi(pszToken, szOverLimit))
						{
							ceTick = GetTickCount();
							do
								PeekUpdateMessage(FALSE);
							while((GetTickCount() - ceTick) < m_eGmQuerySleep);
							iLvItem--;
							i--;
							continue;
						}
						if(!lstrcmpi(pszToken, szReqDenied))
						{
							nResId = IDS_GOOGLE_QUERY_ERROR;
							break;
						}
						if(lstrcmpi(pszToken, szNoAnswer))
						{
							nResId = IDS_GOOGLE_QUERY_ADDR_ERROR;
							break;
						}
					}
				}
			}
			PutGoogleMaps(m_eGmPutDelete, iLvItem);
			EnsureLvItemVisible(iLvItem);
			ValidateMemoCol(iLvItem, lvi.pszText);
			if(iLvItem || !m_bMyBP)
				m_bFileDirty = TRUE;
			if(bSizeMemo)
			{
				AutoSizeLvCol(LVC_MEMO, FALSE);
				bSizeMemo = FALSE;
			}
		}
		AutoSizeLvCol(LVC_MEMO, FALSE);
		m_jProgress.DestroyWindow();
		InternetCloseHandle(hInternet);
		free(pszwBuffer);
		free(pszBuffer);
	}
	else
		nResId = IDS_GOOGLE_CONNECT_ERROR;
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(ID_POI_QUERY_ADDR, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_ceDragDrop--;
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Find

AFXINLINE void CTomPoiEdr::OnPoiFind()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiFind\n"));

	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_FIND);
	m_jToolbar.UpdateWindow();
	if(m_jFindReplDlg.DisplayDialogBox(e_hInstance, IDD_FIND_POI, *this) == ID_OK)
	{
		FindPoiRecord(0, m_jFindReplDlg.m_nAllPoi == BST_CHECKED);
		EnsureLvItemVisible(m_jListView.GetNextItem(LVNI_SELECTED));
	}
	m_jToolbar.PressButton(ID_POI_FIND, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Find Next

AFXINLINE void CTomPoiEdr::OnPoiFindNext()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiFindNext\n"));

	m_jToolbar.PressButton(ID_POI_FIND_NEXT);
	m_jToolbar.UpdateWindow();
	EnsureLvItemVisible(FindPoiRecord((m_jListView.GetNextItem(LVNI_SELECTED) + 1) %
		m_jListView.GetItemCount(), FALSE));
	m_jToolbar.PressButton(ID_POI_FIND_NEXT, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Replace

AFXINLINE void CTomPoiEdr::OnPoiReplace()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiReplace\n"));

	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_REPLACE);
	m_jToolbar.UpdateWindow();
	if(m_jFindReplDlg.DisplayDialogBox(e_hInstance, IDD_REPLACE_POI, *this) == ID_OK)
	{
		if(m_jFindReplDlg.m_nAllPoi == BST_CHECKED)
		{
			TCHAR szText[MAX_STR];
			LoadString(e_hInstance, IDP_REPLACING_POI, szText, countof(szText));
			m_jStatusBar.SetWindowText(szText);
			if(ReplacePoiRecord(0, TRUE) && (m_jFindReplDlg.m_nOnlyName == BST_UNCHECKED) &&
				(m_iLvBaseItem != LVI_ERR))
				SetBasePoint(m_iLvBaseItem, TRUE);
			EnsureLvItemVisible(m_jListView.GetNextItem(LVNI_SELECTED));
			m_bSetReady = TRUE;
		}
		else
			PostMessage(WM_COMMAND, ID_POI_FIND_NEXT);
	}
	m_jToolbar.PressButton(ID_POI_REPLACE, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Replace Next

AFXINLINE void CTomPoiEdr::OnPoiReplaceNext()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiReplaceNext\n"));

	int iLvItem;
	m_jToolbar.PressButton(ID_POI_REPLACE_NEXT);
	m_jToolbar.UpdateWindow();
	iLvItem = m_jListView.GetNextItem(LVNI_SELECTED);
	if(iLvItem != RV_ERR)
	{
		int iHitLvItem;
		iHitLvItem = FindPoiRecord(iLvItem, FALSE);
		if(iHitLvItem == iLvItem)
		{
			if(ReplacePoiRecord(iLvItem, FALSE) &&
				(m_jFindReplDlg.m_nOnlyName == BST_UNCHECKED) && (m_iLvBaseItem != LVI_ERR))
				SetDistanceText2(iLvItem);
			iHitLvItem = FindPoiRecord((iLvItem + 1) % m_jListView.GetItemCount(), FALSE);
		}
		EnsureLvItemVisible(iHitLvItem);
	}
	else
		PostMessage(WM_COMMAND, ID_POI_FIND_NEXT);
	m_jToolbar.PressButton(ID_POI_REPLACE_NEXT, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Mess Up Coordinates

AFXINLINE void CTomPoiEdr::OnPoiMessCoord()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiMessCoord\n"));

	TCHAR  szText[MAX_STR], szCoord[2][32];
	int    i, j, k, ceLvItem, iLvItem;
	LVITEM lvi;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_jToolbar.PressButton(ID_POI_MESS_COORD);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDP_MESSING_COORD, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	m_jListView.UpdateWindow();
	m_jListView.SetAction(AT_NEW);
	srand(GetTickCount());
	ceLvItem       = m_jListView.GetSelCount() ? 0 : m_jListView.GetItemCount();
	iLvItem        = -1;
	lvi.cchTextMax = countof(szCoord[0]);
	while(ceLvItem ? (++iLvItem < ceLvItem) :
		((iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem)) != RV_ERR))
	{
		lvi.iSubItem = LVC_LNG;
		lvi.pszText  = szCoord[0];
		if(!m_jListView.GetItemText(iLvItem, &lvi))
			continue;
		lvi.iSubItem = LVC_LAT;
		lvi.pszText  = szCoord[1];
		if(!m_jListView.GetItemText(iLvItem, &lvi))
			continue;
		PutGoogleMaps(m_eGmPutDelete, iLvItem);
		i = LVC_LNG;
		j = 0;
		k = CR_LNG_FROM;
		do
		{
			GetCoordString(i, (double)clamp(GetCoordValueL(szCoord[j]) +
				rand(m_eMinMess, m_eMaxMess),
				e_lCoordLimit[k], e_lCoordLimit[k + 1]) / e_eCoordPump, szCoord[j]);
			lvi.iSubItem = i;
			lvi.pszText  = szCoord[j];
			m_jListView.SetItemText(iLvItem, &lvi);
			i++;
			j++;
			k += 2;
		}
		while(i <= LVC_LAT);
		if(iLvItem || !m_bMyBP)
			m_bFileDirty = TRUE;
	}
	if(m_iLvBaseItem != LVI_ERR)
		SetBasePoint(m_iLvBaseItem, TRUE);
	m_jToolbar.PressButton(ID_POI_MESS_COORD, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Check

AFXINLINE void CTomPoiEdr::OnPoiCheck()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiCheck\n"));

	if(m_jCheckDataDlg)
	{
		m_jCheckDataDlg.SetActiveWindow();
		return;
	}

	int iResult;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POI_CHECK);
	m_jToolbar.UpdateWindow();
	iResult = m_jCheckPoiDlg.DialogBox(e_hInstance, IDD_CHECK_POI, *this);
	m_jToolbar.PressButton(ID_POI_CHECK, FALSE);
	if(iResult == ID_CANCEL)
	{
		m_ceDragDrop--;
		return;
	}

	m_bSetReady = FALSE;
	m_jCheckDataDlg.CreateDialog(e_hInstance, IDD_CHECK_DATA, *this);
	if(e_dwWinVer > WV_XP)
	{
		CImageList jLvColPadIL;
		jLvColPadIL.LoadImage(e_hInstance, IDB_COLUMN_PAD, m_ecxLvColPadIL, 0, CLR_NONE,
			IMAGE_BITMAP, LR_CREATEDIBSECTION);
		m_jCheckDataDlg.m_jListView.SetImageList(jLvColPadIL, LVSIL_SMALL);
	}
	m_jCheckDataDlg.CheckEmptyCoord();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Add Favorite

AFXINLINE void CTomPoiEdr::OnPoiAddFav()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiAddFav\n"));

	PTCHAR pcErrToken;
	TCHAR  szText[MAX_STR];
	TCHAR  szName[e_eccNameCol * FT_UTF8S3 * 3], szLng[32], szLat[countof(szLng)];
	TCHAR  szCmdLine[countof(szText) + countof(szName) + countof(szLng) + countof(szLat)];
	LVITEM lvi;
	m_jToolbar.PressButton(ID_POI_ADD_FAV);
	m_jToolbar.UpdateWindow();
	lvi.iItem      = m_jListView.GetNextItem(LVNI_SELECTED);
	lvi.iSubItem   = LVC_NAME;
	lvi.pszText    = szName;
	lvi.cchTextMax = e_eccNameCol;
	m_jListView.GetItemText(lvi.iItem, &lvi);
	while(pcErrToken = StrPBrk(szName, _T("%&")))
	{
		*pcErrToken = CHR_SP;
		pcErrToken++;
	}
	EncodeUrl(szName);
	lvi.iSubItem   = LVC_LNG;
	lvi.pszText    = szLng;
	lvi.cchTextMax = countof(szLng);
	GetCoordDdString(szLng, m_jListView.GetItemText(lvi.iItem, &lvi));
	lvi.iSubItem = LVC_LAT;
	lvi.pszText  = szLat;
	GetCoordDdString(szLat, m_jListView.GetItemText(lvi.iItem, &lvi));
	LoadString(e_hInstance, IDS_TOMTOM_ADD_FAVORITE, szText, countof(szText));
	wsprintf(szCmdLine, szText, szLat, szLng, szName);
	ShellExecute(NULL, NULL, szCmdLine, NULL, NULL, SW_SHOW);
	m_jToolbar.PressButton(ID_POI_ADD_FAV, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Add ITNs

AFXINLINE void CTomPoiEdr::OnPoiAddItn()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiAddItn\n"));

	TCHAR szText[e_eccNameCol];
	PWSTR pszPoiData;
	UINT  nResId;
	HFILE hFile;
	m_ceDragDrop++;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_jToolbar.PressButton(ID_POI_ADD_ITN);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDP_SENDING_POI, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	nResId = IDS_MEMORY_ERROR_SEND_POI;
	hFile  = CreateFileMapping(PAGE_READWRITE, m_jListView.GetSelCount() *
		((e_eccNameCol + e_eccLngCol + e_eccLatCol) * sizeof(WCHAR)), e_pszTpeAppName);
	if(hFile)
	{
		pszPoiData = (PWSTR)MapViewOfFile(hFile, FILE_MAP_WRITE);
		if(pszPoiData)
		{
			PWSTR  pszDest;
			int    i, iLvItem, ccText;
			DWORD  dwResult;
			LVITEM lvi;
			CWnd   jTieWnd;
			lvi.cchTextMax = countof(szText);
			pszDest        = pszPoiData;
			iLvItem        = -1;
			while(iLvItem = m_jListView.GetNextItem(LVNI_SELECTED, iLvItem), iLvItem != RV_ERR)
			{
				i = LVC_NAME;
				do
				{
					lvi.iSubItem = i;
#ifdef UNICODE
					lvi.pszText  = pszDest;
#else
					lvi.pszText  = szText;
#endif
					ccText       = GetCoordDdString(i, lvi.pszText,
						m_jListView.GetItemText(iLvItem, &lvi));
#ifdef UNICODE
					pszDest     += ccText + 1;
#else
					pszDest     += MBStoWCS(szText, ccText + 1, pszDest, sizeof(szText));
#endif
					i++;
				}
				while(i <= LVC_LAT);
			}
			*pszDest = CHR_NULL;
			UnmapViewOfFile(pszPoiData);
			jTieWnd.FindWindow(e_pszTieClassName);
			if(jTieWnd.SendMessageTimeout(WM_SEND_POI, TRUE, UNUSED,
				SMTO_BLOCK, 500, &dwResult) && (dwResult == m_eRecvPoiOK))
			{
				jTieWnd.SendMessage(WM_SEND_POI);
				nResId = 0;
			}
			else
				nResId = IDS_BUSY_SEND_POI;
		}
		CloseHandle(hFile);
	}
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(ID_POI_ADD_ITN, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_ceDragDrop--;
	m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Select All

AFXINLINE void CTomPoiEdr::OnPoiSelAll()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiSelAll\n"));

	LVITEM lvi;
	lvi.state     = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from POI/Toggle Select

AFXINLINE void CTomPoiEdr::OnPoiToggleSel()
{
	TRACE(_T("Entering CTomPoiEdr::OnPoiToggleSel\n"));

	LVITEM lvi;
	lvi.iItem     = m_jListView.GetItemCount() - 1;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	do
	{
		lvi.state = m_jListView.GetItemState(lvi.iItem, LVIS_SELECTED) ? UNUSED : LVIS_SELECTED;
		m_jListView.SetItemState(lvi.iItem, &lvi);
		lvi.iItem--;
	}
	while(lvi.iItem >= 0);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Up

AFXINLINE void CTomPoiEdr::OnPosUp()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosUp\n"));

	MoveLvItem(TRUE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Down

AFXINLINE void CTomPoiEdr::OnPosDown()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosDown\n"));

	MoveLvItem(FALSE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Set Base Point

AFXINLINE void CTomPoiEdr::OnPosSetBP()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosSetBP\n"));

	int iItem;
	m_jToolbar.PressButton(ID_POS_SET_BP);
	m_jToolbar.UpdateWindow();
	m_jListView.SetAction(AT_NEW);
	if(m_bMyBP)
	{
		PutGoogleMaps(m_eGmPutDelete, 0);
		m_jListView.DeleteItem(0);
		m_bMyBP = FALSE;
	}
	iItem = m_jListView.GetNextItem(LVNI_SELECTED);
	m_jListView.SetAction(AC_BASEPT, iItem);
	SetBasePoint(iItem, FALSE);
	AutoSizeLvCol(LVC_ALL, TRUE);
	m_jToolbar.PressButton(ID_POS_SET_BP, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/My Base Point

AFXINLINE void CTomPoiEdr::OnPosMyBP()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosMyBP\n"));

	CMyBPDlg jMyBPDlg;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_POS_MY_BP);
	m_jToolbar.UpdateWindow();
	if(jMyBPDlg.DialogBox(e_hInstance, IDD_MY_BP, *this) == ID_OK)
	{
		LVITEM lvi;
		lvi.state     = UNUSED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(&lvi);
		lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(0, &lvi);
		SetBasePoint(0, FALSE);
		AutoSizeLvCol(LVC_ALL, TRUE);
	}
	m_jToolbar.PressButton(ID_POS_MY_BP, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Sort Ascending

AFXINLINE void CTomPoiEdr::OnPosSortAscend()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosSortAscend\n"));

	SortLvItems(LVC_NAME, TRUE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Sort Descending

AFXINLINE void CTomPoiEdr::OnPosSortDescend()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosSortDescend\n"));

	SortLvItems(LVC_NAME, FALSE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Google Maps

AFXINLINE void CTomPoiEdr::OnPosGoogleMap()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosGoogleMap\n"));

	TCHAR szText[MAX_STR];
	int   iResId;
	m_ceDragDrop++;
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	m_jToolbar.PressButton(ID_POS_GOOGLE_MAP);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, IDP_SHOWING_GOOGLE_MAP, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	iResId = ShowIEPane();
	if(iResId)
	{
		if(iResId < 0)
		{
			m_jIEPane.HideWindow();
			ResizeWindow(FALSE);
			ReleaseIEObject();
			iResId = -iResId;
		}
		DisplayMessage(iResId, MB_ICONINFORMATION | MB_OK, UNUSED);
		m_bSetReady = TRUE;
	}
	m_jToolbar.PressButton(ID_POS_GOOGLE_MAP, FALSE);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Position/Google Earth

AFXINLINE void CTomPoiEdr::OnPosGoogleEarth()
{
	TRACE(_T("Entering CTomPoiEdr::OnPosGoogleEarth\n"));

	int   cePoint;
	UINT  nResId;
	HFILE hFile;
	m_jToolbar.PressButton(ID_POS_GOOGLE_EARTH);
	m_jToolbar.UpdateWindow();
	hFile = CreateFile(m_pszGeTempFile, GENERIC_WRITE, CREATE_ALWAYS);
	if(hFile != INVALID_HANDLE)
	{
		m_jProgress.CreateWindow(m_ePbStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
		m_opf  = OPF_EXPORT;
		nResId = WriteKmlRecord(hFile, &cePoint);
		m_jProgress.DestroyWindow();
		CloseHandle(hFile);
		m_bSetReady = TRUE;
	}
	else
		nResId = IDS_CANNOT_WRITE;
	if(nResId)
		DisplayMessage(nResId, MB_ICONWARNING | MB_OK);
	else
	{
		TCHAR szGoogle[MAX_STR];
		DWORD ccGoogle;
		CWnd  jGeWnd;
		ccGoogle = countof(szGoogle);
		UrlCreateFromPath(m_pszGeTempFile, szGoogle, &ccGoogle, UNUSED);
		ShellExecute(NULL, NULL, szGoogle, NULL, NULL, SW_SHOW);
		LoadString(e_hInstance, IDS_GOOGLE_EARTH_WNDNAME, szGoogle, countof(szGoogle));
		if(jGeWnd.FindWindow(NULL, szGoogle))
			jGeWnd.SetForegroundWindow();
	}
	m_jToolbar.PressButton(ID_POS_GOOGLE_EARTH, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/QuickGPSfix

AFXINLINE void CTomPoiEdr::OnToolsQGPSfix()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsQGPSfix\n"));

	TCHAR       szPathName[MAX_PATH], szFileName[MAX_PATH], szTempFile[MAX_PATH];
	TCHAR       szText[MAX_STR];
	int         iRightPos[m_eceSbPart];
	SIZE        size;
	CQGPSfixDlg jQGPSfixDlg(szPathName, szFileName, szTempFile);
	m_bSetReady = FALSE;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_TOOLS_QGPSFIX);
	m_jToolbar.UpdateWindow();
	m_jProgress.CreateWindow(m_ePbMqStyle, 0, 0, 0, 0, m_jStatusBar, IDW_PROGRESS);
	m_jProgress.SetMarquee(m_ePbMqIntl);
	GetTextExtent(m_hSbDC, szText, LoadString(e_hInstance, IDP_UPDATING_EPHEM,
		szText, countof(szText)), &size);
	m_jStatusBar.SetWindowText(szText);
	size.cx += m_ecxPbPadL;
	m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
	m_jProgress.MoveWindow(size.cx, m_ecyPbPadT,
		iRightPos[m_eInfoPart] - size.cx - m_ecxPbPadR, m_cyStatusBar - m_ecyPbPadB);
	jQGPSfixDlg.DialogBox(e_hInstance, IDD_QGPSFIX, *this);
	m_jProgress.DestroyWindow();
	m_jToolbar.PressButton(ID_TOOLS_QGPSFIX, FALSE);
	m_ceDragDrop--;
	m_bSetReady = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Show Menu Bar

AFXINLINE void CTomPoiEdr::OnToolsShowMb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsShowMb\n"));

	HMENU hMenuBar;
	if(m_jOptionDlg.m_nShowMb == BST_CHECKED)
	{
		m_jOptionDlg.m_nShowMb = BST_UNCHECKED;
		hMenuBar               = NULL;
	}
	else
	{
		m_jOptionDlg.m_nShowMb = BST_CHECKED;
		hMenuBar               = m_hMainMenu;
	}
	SetMenu(hMenuBar);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Show Toolbar

AFXINLINE void CTomPoiEdr::OnToolsShowTb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsShowTb\n"));

	int iShowCmd;
	if(m_jOptionDlg.m_nShowTb == BST_CHECKED)
	{
		m_jOptionDlg.m_nShowTb = BST_UNCHECKED;
		iShowCmd               = SW_HIDE;
	}
	else
	{
		m_jOptionDlg.m_nShowTb = BST_CHECKED;
		iShowCmd               = SW_SHOW;
	}
	m_jToolbar.ShowWindow(iShowCmd);
	ResizeWindow(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Large Toolbar

AFXINLINE void CTomPoiEdr::OnToolsLargeTb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsLargeTb\n"));

	m_jOptionDlg.m_nLargeTb = (m_jOptionDlg.m_nLargeTb == BST_CHECKED) ?
		BST_UNCHECKED : BST_CHECKED;
	CreateTbCtrl();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Auto Wrap Toolbar

AFXINLINE void CTomPoiEdr::OnToolsWrapTb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsWrapTb\n"));

	DWORD dwTbStyle;
	dwTbStyle = m_jToolbar.GetStyle();
	if(m_jOptionDlg.m_nWrapTb == BST_CHECKED)
	{
		m_jOptionDlg.m_nWrapTb = BST_UNCHECKED;
		dwTbStyle             &= ~TBSTYLE_WRAPABLE;
	}
	else
	{
		m_jOptionDlg.m_nWrapTb = BST_CHECKED;
		dwTbStyle             |= TBSTYLE_WRAPABLE;
	}
	m_jToolbar.SetStyle(dwTbStyle);
	ResizeWindow(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Toolbar Background

AFXINLINE void CTomPoiEdr::OnToolsTbBkgnd()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsTbBkgnd\n"));

	PTSTR    pszRecord;
	TCHAR    szEntry[64];
	int      i;
	COLORREF crCustom[16];
	CTbBkDlg jTbBkDlg;
	m_ceDragDrop++;
	i = lstrlen(m_szTbBkgndEntry);
	CopyMemory(szEntry, m_szTbBkgndEntry, i * sizeof(TCHAR));
	pszRecord  = szEntry + i;
	*pszRecord = CHR_SP;
	pszRecord++;
	i = 0;
	do
	{
		_ltot(i, pszRecord);
		crCustom[i] = GetPrivateProfileInt(m_szOptionSection, szEntry,
			(e_dwWinVer <= WV_XP) ? m_ecrTbBkgndXP : m_ecrTbBkgnd, e_pszProfile);
		i++;
	}
	while(i < countof(crCustom));
	jTbBkDlg.lStructSize  = sizeof(CHOOSECOLOR);
	jTbBkDlg.hwndOwner    = *this;
	jTbBkDlg.rgbResult    = m_crTbBkgnd;
	jTbBkDlg.lpCustColors = crCustom;
	jTbBkDlg.Flags        = CC_ANYCOLOR | CC_ENABLEHOOK | CC_FULLOPEN | CC_RGBINIT;
	jTbBkDlg.lpfnHook     = GetCommonDialogProc(CTbBkDlg);
	if(jTbBkDlg.ChooseColor())
	{
		TCHAR szValue[32];
		if(jTbBkDlg.rgbResult != m_crTbBkgnd)
		{
			m_crTbBkgnd = jTbBkDlg.rgbResult;
			m_jToolbar.SendMessage(WM_NCPAINT);
			m_jToolbar.InvalidateRect();
			_ltot(m_crTbBkgnd, szValue);
			WritePrivateProfileString(m_szOptionSection, m_szTbBkgndEntry,
				szValue, e_pszProfile);
		}
		i = 0;
		do
		{
			_ltot(i, pszRecord);
			_ltot(crCustom[i], szValue);
			WritePrivateProfileString(m_szOptionSection, szEntry, szValue, e_pszProfile);
			i++;
		}
		while(i < countof(crCustom));
	}
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Customize Toolbar Buttons

AFXINLINE void CTomPoiEdr::OnToolsCustTb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsCustTb\n"));

	m_jToolbar.Customize();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Display Font

AFXINLINE void CTomPoiEdr::OnToolsLvFont()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsLvFont\n"));

	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_TOOLS_LVFONT);
	m_jToolbar.UpdateWindow();
	if(m_jFontDlg.ChooseFont())
		SetLvFont();
	else
		GetObject(m_hLvFont, &m_jFontDlg);
	m_jToolbar.PressButton(ID_TOOLS_LVFONT, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Background Image

AFXINLINE void CTomPoiEdr::OnToolsLvBkgnd()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsLvBkgnd\n"));

	DWORD    dwLvStyle;
	COLORREF crLvBkgnd;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_TOOLS_LVBKGND);
	m_jToolbar.UpdateWindow();
	if(m_bLvBkgnd)
	{
		LVBKIMAGE lvbki;
		m_bLvBkgnd    = FALSE;
		lvbki.ulFlags = LVBKIF_SOURCE_NONE;
		m_jListView.SetBkImage(&lvbki);
	}
	else
	{
		TCHAR szFileName[MAX_PATH], szFileTitle[MAX_FNAME];
		TCHAR szInitDir[MAX_PATH], szTitle[MAX_STR];
		PTSTR pszFileName, pszFileTitle;
		LoadString(e_hInstance, IDS_TITLE_LVBKGND, szTitle, countof(szTitle));
		pszFileName               = m_jLvBkDlg.lpstrFile;
		pszFileTitle              = m_jLvBkDlg.lpstrFileTitle;
		m_jLvBkDlg.lpstrFile      = szFileName;
		m_jLvBkDlg.lpstrFileTitle = szFileTitle;
		m_jLvBkDlg.lpstrTitle     = szTitle;
		lstrcpy(szFileName, pszFileTitle);
		if(IsValidFile(pszFileName))
		{
			lstrcpy(szInitDir, pszFileName);
			PathRemoveFileSpec(szInitDir);
			m_jLvBkDlg.lpstrInitialDir = szInitDir;
		}
		else
			m_jLvBkDlg.lpstrInitialDir = NULL;
		if(m_jLvBkDlg.GetOpenFileName() && IsValidFile(szFileName))
		{
			LVBKIMAGE lvbki;
			lvbki.ulFlags  = LVBKIF_SOURCE_URL | LVBKIF_STYLE_TILE;
			lvbki.pszImage = szFileName;
			m_bLvBkgnd     = m_jListView.SetBkImage(&lvbki);
			if(m_bLvBkgnd)
			{
				lstrcpy(pszFileName, szFileName);
				lstrcpy(pszFileTitle, szFileTitle);
			}
		}
		m_jLvBkDlg.lpstrFile      = pszFileName;
		m_jLvBkDlg.lpstrFileTitle = pszFileTitle;
	}
	dwLvStyle = m_jListView.GetWindowStyle();
	if(m_bLvBkgnd)
	{
		dwLvStyle |= LVS_SHOWSELALWAYS;
		crLvBkgnd  = CLR_NONE;
	}
	else
	{
		if(e_dwWinVer > WV_XP)
			dwLvStyle |= LVS_SHOWSELALWAYS;
		else
			dwLvStyle &= ~LVS_SHOWSELALWAYS;
		crLvBkgnd = GetSysColor(COLOR_WINDOW);
	}
	m_jListView.SetWindowStyle(dwLvStyle);
	m_jListView.SetTextBkColor(crLvBkgnd);
	m_jToolbar.PressButton(ID_TOOLS_LVBKGND, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Show Status Bar

AFXINLINE void CTomPoiEdr::OnToolsShowSb()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsShowSb\n"));

	int iShowCmd;
	if(m_jOptionDlg.m_nShowSb == BST_CHECKED)
	{
		m_jOptionDlg.m_nShowSb = BST_UNCHECKED;
		iShowCmd               = SW_HIDE;
	}
	else
	{
		m_jOptionDlg.m_nShowSb = BST_CHECKED;
		iShowCmd               = SW_SHOW;
	}
	m_jStatusBar.ShowWindow(iShowCmd);
	ResizeWindow(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Tools/Options

AFXINLINE void CTomPoiEdr::OnToolsOption()
{
	TRACE(_T("Entering CTomPoiEdr::OnToolsOption\n"));

	int  ceRecentFile, ceBackupIntl, iGmLang;
	UINT nShowMb, nShowTb, nLargeTb, nWrapTb, nLngFirst, nDmsFormat, nCompassDir, nAutoColumn;
	UINT nShowDist, nMetricUnit, nRecentFile, nShadeTable, nShowSb, nBackupFile;
	UINT nGmUseCookie, nGmUpdateMemo, nGmFlashRoute;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_TOOLS_OPTION);
	m_jToolbar.UpdateWindow();
	nShowMb       = m_jOptionDlg.m_nShowMb;
	nShowTb       = m_jOptionDlg.m_nShowTb;
	nLargeTb      = m_jOptionDlg.m_nLargeTb;
	nWrapTb       = m_jOptionDlg.m_nWrapTb;
	nLngFirst     = m_jOptionDlg.m_nLngFirst;
	nDmsFormat    = m_jOptionDlg.m_nDmsFormat;
	nCompassDir   = m_jOptionDlg.m_nCompassDir;
	nAutoColumn   = m_jOptionDlg.m_nAutoColumn;
	nShowDist     = m_jOptionDlg.m_nShowDist;
	nMetricUnit   = m_jOptionDlg.m_nMetricUnit;
	nRecentFile   = m_jOptionDlg.m_nRecentFile;
	ceRecentFile  = m_jOptionDlg.m_ceRecentFile;
	nShadeTable   = m_jOptionDlg.m_nShadeTable;
	nShowSb       = m_jOptionDlg.m_nShowSb;
	nBackupFile   = m_jOptionDlg.m_nBackupFile;
	ceBackupIntl  = m_jOptionDlg.m_ceBackupIntl;
	iGmLang       = m_jOptionDlg.m_iGmLang;
	nGmUseCookie  = m_jOptionDlg.m_nGmUseCookie;
	nGmUpdateMemo = m_jOptionDlg.m_nGmUpdateMemo;
	nGmFlashRoute = m_jOptionDlg.m_nGmFlashRoute;
	if(m_jOptionDlg.DialogBox(e_hInstance, IDD_OPTION, *this) == ID_OK)
	{
		// Shows window menu bar
		if(nShowMb != m_jOptionDlg.m_nShowMb)
		{
			m_jOptionDlg.m_nShowMb = nShowMb;
			PostMessage(WM_COMMAND, ID_TOOLS_SHOWMB);
		}

		// Shows/large/auto wrap toolbar control
		if(nShowTb != m_jOptionDlg.m_nShowTb)
		{
			m_jOptionDlg.m_nShowTb = nShowTb;
			PostMessage(WM_COMMAND, ID_TOOLS_SHOWTB);
		}
		if(nLargeTb != m_jOptionDlg.m_nLargeTb)
		{
			m_jOptionDlg.m_nLargeTb = nLargeTb;
			PostMessage(WM_COMMAND, ID_TOOLS_LARGETB);
		}
		if(nWrapTb != m_jOptionDlg.m_nWrapTb)
		{
			m_jOptionDlg.m_nWrapTb = nWrapTb;
			PostMessage(WM_COMMAND, ID_TOOLS_WRAPTB);
		}

		// Reorders coordinate columns
		if(nLngFirst != m_jOptionDlg.m_nLngFirst)
		{
			m_jOptionDlg.m_nLngFirst = nLngFirst;
			PostMessage(WM_COMMAND, ID_LNG_FIRST);
		}

		// Shows coordinates in DMS format
		if(nDmsFormat != m_jOptionDlg.m_nDmsFormat)
		{
			m_jOptionDlg.m_nDmsFormat = nDmsFormat;
			PostMessage(WM_COMMAND, ID_DMS_FORMAT);
		}

		// Shows coordinates with compass direction
		if(nCompassDir != m_jOptionDlg.m_nCompassDir)
		{
			m_jOptionDlg.m_nCompassDir = nCompassDir;
			PostMessage(WM_COMMAND, ID_COMPASS_DIR);
		}

		// Automatic sizes column widths
		if((nAutoColumn != m_jOptionDlg.m_nAutoColumn) &&
			(m_jOptionDlg.m_nAutoColumn == BST_CHECKED))
			PostMessage(WM_COMMAND, ID_AUTOSIZE_COL);

		// Shows Distance column
		if(nShowDist != m_jOptionDlg.m_nShowDist)
		{
			m_jOptionDlg.m_nShowDist = nShowDist;
			PostMessage(WM_COMMAND, ID_SHOW_DISTANCE);
		}

		// Uses metric units for distance
		if(nMetricUnit != m_jOptionDlg.m_nMetricUnit)
		{
			m_jOptionDlg.m_nMetricUnit = nMetricUnit;
			PostMessage(WM_COMMAND, ID_METRIC_UNIT);
		}

		// Recent file list
		if((nRecentFile != m_jOptionDlg.m_nRecentFile) ||
			(ceRecentFile != m_jOptionDlg.m_ceRecentFile))
		{
			if(m_jOptionDlg.m_nRecentFile == BST_CHECKED)
			{
				if(!m_pszRecentFile)
					m_pszRecentFile = (PTSTR)malloc(m_eceMaxRecentFile *
						MAX_PATH * sizeof(TCHAR));
				for(int i = m_jOptionDlg.m_ceRecentFile; i < m_ceRecentFile; i++)
					RemoveMenu(m_hMainMenu, ID_RECENT_FILE1 + i, MF_BYCOMMAND);
				if(m_ceRecentFile > m_jOptionDlg.m_ceRecentFile)
					m_ceRecentFile = m_jOptionDlg.m_ceRecentFile;
			}
			else
			{
				if(m_ceRecentFile)
				{
					m_ceRecentFile++;
					do
					{
						RemoveMenu(m_hFileMenu, m_eRecentInsPos, MF_BYPOSITION);
						m_ceRecentFile--;
					}
					while(m_ceRecentFile);
				}
			}
		}

		// Shades table colors
		if(nShadeTable != m_jOptionDlg.m_nShadeTable)
		{
			if(!m_bLvBkgnd)
				m_jListView.InvalidateRect();
			if(m_jCheckDataDlg)
				m_jCheckDataDlg.m_jListView.InvalidateRect();
		}

		// Shows status bar control
		if(nShowSb != m_jOptionDlg.m_nShowSb)
		{
			m_jOptionDlg.m_nShowSb = nShowSb;
			PostMessage(WM_COMMAND, ID_TOOLS_SHOWSB);
		}

		// Auto backup
		if((nBackupFile != m_jOptionDlg.m_nBackupFile) ||
			(ceBackupIntl != m_jOptionDlg.m_ceBackupIntl))
		{
			if(m_jOptionDlg.m_nBackupFile == BST_CHECKED)
				SetTimer(m_eBackupTimerId, m_jOptionDlg.m_ceBackupIntl * m_eMinuteIntl);
			else
				KillTimer(m_eBackupTimerId);
		}

		// Google Maps language, Memo info and IQ Routes
		if(m_jIEPane.IsWindowVisible())
		{
			if(iGmLang != m_jOptionDlg.m_iGmLang)
				PostMessage(WM_COMMAND, ID_POS_GOOGLE_MAP);
			else
			{
				if(nGmUseCookie != m_jOptionDlg.m_nGmUseCookie)
					PutGoogleMaps((m_jOptionDlg.m_nGmUseCookie == BST_CHECKED) ?
						m_eGmPutUCT : m_eGmPutUCF, UNUSED);
				if(nGmUpdateMemo != m_jOptionDlg.m_nGmUpdateMemo)
					PutGoogleMaps((m_jOptionDlg.m_nGmUpdateMemo == BST_CHECKED) ?
						m_eGmPutUMT : m_eGmPutUMF, UNUSED);
				if(nGmFlashRoute != m_jOptionDlg.m_nGmFlashRoute)
					PutGoogleMaps((m_jOptionDlg.m_nGmFlashRoute == BST_CHECKED) ?
						m_eGmPutFRT : m_eGmPutFRF, UNUSED);
			}
		}
	}
	m_jToolbar.PressButton(ID_TOOLS_OPTION, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item in the range from Help/TomTom to Help/Mailto Programer

AFXINLINE void CTomPoiEdr::OnHelpMisc(UINT nItemId)
{
	TRACE(_T("Entering CTomPoiEdr::OnHelpMisc\n"));

	TCHAR szCmdLine[MAX_STR];
	m_jToolbar.PressButton(nItemId);
	m_jToolbar.UpdateWindow();
	LoadString(e_hInstance, nItemId - ID_HELP_TOM_HOME + IDS_TOM_HOME,
		szCmdLine, countof(szCmdLine));
	if(((int)ShellExecute(NULL, NULL, szCmdLine, NULL, NULL, SW_NORMAL) <= 32) &&
		(nItemId == ID_HELP_MAILTO))
		DisplayMessage(IDS_MAIL_ERROR, MB_ICONINFORMATION | MB_OK, UNUSED);
	m_jToolbar.PressButton(nItemId, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Help/About

AFXINLINE void CTomPoiEdr::OnHelpAbout()
{
	TRACE(_T("Entering CTomPoiEdr::OnHelpAbout\n"));

	CAboutDlg jAboutDlg;
	m_ceDragDrop++;
	m_jToolbar.PressButton(ID_HELP_ABOUT);
	m_jToolbar.UpdateWindow();
	jAboutDlg.DialogBox(e_hInstance, IDD_ABOUT, *this);
	m_jToolbar.PressButton(ID_HELP_ABOUT, FALSE);
	m_ceDragDrop--;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Open Command Line

AFXINLINE void CTomPoiEdr::OnOpenCmdLine()
{
	TRACE(_T("Entering CTomPoiEdr::OnOpenCmdLine\n"));

	OpenPoiFile((UINT)m_pszCmdLine);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Resize Window

AFXINLINE void CTomPoiEdr::OnResizeWindow(LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnResizeWindow\n"));

	ResizeWindow(lParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Select List View Item

AFXINLINE void CTomPoiEdr::OnSelectLvItem(LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnSelectLvItem\n"));

	if(m_jLvEdit.IsWindowVisible() || m_jListView.GetSelCount())
		return;

	int ceLvItem;
	ceLvItem = m_jListView.GetItemCount();
	if(!ceLvItem)
		return;

	int    iLvItem;
	LVITEM lvi;
	iLvItem       = min(lParam, ceLvItem - 1);
	lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(iLvItem, &lvi);
	m_jListView.EnsureVisible(iLvItem);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Longitude In Front

AFXINLINE void CTomPoiEdr::OnLngFirst()
{
	TRACE(_T("Entering CTomPoiEdr::OnLngFirst\n"));

	UINT nCheck;
	int  iPutCmd;
	if(m_jOptionDlg.m_nLngFirst == BST_CHECKED)
	{
		nCheck  = BST_UNCHECKED;
		iPutCmd = m_eGmPutLFF;
	}
	else
	{
		nCheck  = BST_CHECKED;
		iPutCmd = m_eGmPutLFT;
	}
	m_jOptionDlg.m_nLngFirst = nCheck;
	SwapCoordCol();
	PutGoogleMaps(iPutCmd, UNUSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Coordinates In DMS Format

AFXINLINE void CTomPoiEdr::OnDmsFormat()
{
	TRACE(_T("Entering CTomPoiEdr::OnDmsFormat\n"));

	UpdateCoordData(TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Coordinates With Compass Direction

AFXINLINE void CTomPoiEdr::OnCompassDir()
{
	TRACE(_T("Entering CTomPoiEdr::OnCompassDir\n"));

	UpdateCoordData(FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Adjust Widths

AFXINLINE void CTomPoiEdr::OnAutoSizeCol()
{
	TRACE(_T("Entering CTomPoiEdr::OnAutoSizeCol\n"));

	AutoSizeLvCol(LVCA_ALL, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Clear Column

AFXINLINE void CTomPoiEdr::OnClearCol()
{
	TRACE(_T("Entering CTomPoiEdr::OnClearCol\n"));

	TCHAR  szText[MAX_STR];
	int    ceLvItem, iLvItem;
	LVITEM lvi;
	m_jToolbar.PressButton(ID_POI_CLEAR_COL);
	m_jToolbar.UpdateWindow();
	m_jListView.SetAction(AT_NEW);
	LoadString(e_hInstance, IDP_CLEARING_COL, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	ceLvItem       = m_jListView.GetItemCount();
	iLvItem        = 0;
	lvi.iSubItem   = m_iLvCol;
	lvi.pszText    = szText;
	lvi.cchTextMax = 2;
	do
	{
		if(!iLvItem && m_bMyBP)
			continue;
		if(!m_jListView.GetItemText(iLvItem, &lvi))
			continue;
		PutGoogleMaps(m_eGmPutDelete, iLvItem);
		lvi.pszText = NULL;
		m_jListView.SetItemText(iLvItem, &lvi);
		lvi.pszText  = szText;
		m_bFileDirty = TRUE;
	}
	while(++iLvItem < ceLvItem);
	if((m_iLvBaseItem != LVI_ERR) && ((m_iLvCol == LVC_LNG) || (m_iLvCol == LVC_LAT)))
		SetBasePoint(m_iLvBaseItem, TRUE);
	AutoSizeLvCol(m_iLvCol, FALSE);
	m_jToolbar.PressButton(ID_POI_CLEAR_COL, FALSE);
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Clear Address

AFXINLINE void CTomPoiEdr::OnClearAddr()
{
	TRACE(_T("Entering CTomPoiEdr::OnClearAddr\n"));

	TCHAR  szText[MAX_STR], szMemo[e_eccMemoCol], szSuggest[32];
	PTSTR  pszSuggest;
	BOOL   bClear;
	int    ceLvItem, iLvItem;
	LVITEM lvi;
	m_jToolbar.PressButton(ID_POI_CLEAR_COL);
	m_jToolbar.UpdateWindow();
	m_jListView.SetAction(AT_NEW);
	LoadString(e_hInstance, IDP_CLEARING_COL, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	LoadString(e_hInstance, IDS_ADDRESS_SUGGEST, szSuggest, countof(szSuggest));
	ceLvItem       = m_jListView.GetItemCount();
	iLvItem        = 0;
	lvi.iSubItem   = LVC_MEMO;
	lvi.pszText    = szMemo;
	lvi.cchTextMax = countof(szMemo);
	bClear         = FALSE;
	do
	{
		if(!m_jListView.GetItemText(iLvItem, &lvi))
			continue;
		pszSuggest = StrStr(szMemo, szSuggest);
		if(!pszSuggest)
			continue;
		PutGoogleMaps(m_eGmPutDelete, iLvItem);
		*pszSuggest = CHR_NULL;
		TrimString(szMemo);
		m_jListView.SetItemText(iLvItem, &lvi);
		if(iLvItem || !m_bMyBP)
			m_bFileDirty = TRUE;
		bClear = TRUE;
	}
	while(++iLvItem < ceLvItem);
	if(bClear)
		AutoSizeLvCol(LVC_MEMO, FALSE);
	m_jToolbar.PressButton(ID_POI_CLEAR_COL, FALSE);
	m_bGmSelItem  = TRUE;
	m_bSetCaption = TRUE;
	m_bSetReady   = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Show Distance

AFXINLINE void CTomPoiEdr::OnShowDistance()
{
	TRACE(_T("Entering CTomPoiEdr::OnShowDistance\n"));

	int iBaseItem;
	m_jListView.SetAction(AT_NEW);
	iBaseItem = m_iLvBaseItem;
	if(m_jOptionDlg.m_nShowDist == BST_CHECKED)
	{
		m_jOptionDlg.m_nShowDist = BST_UNCHECKED;
		if(m_bMyBP)
		{
			PutGoogleMaps(m_eGmPutDelete, 0);
			m_jListView.DeleteItem(0);
			m_bMyBP = FALSE;
		}
		m_iLvBaseItem = LVI_ERR;
		if(!m_jListView.GetSelCount() && m_jListView.GetItemCount())
		{
			LVITEM lvi;
			lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			m_jListView.SetItemState(0, &lvi);
		}
		if(!m_bLvBkgnd)
			m_jListView.InvalidateRect();
	}
	else
	{
		m_jOptionDlg.m_nShowDist = BST_CHECKED;
		SetBasePoint(0, FALSE);
	}
	m_jListView.SetAction(AC_SHOWDIST, iBaseItem);
	AutoSizeLvCol(LVC_DIST + LVCA_FIRST, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Metric For Distances

AFXINLINE void CTomPoiEdr::OnMetricUnit()
{
	TRACE(_T("Entering CTomPoiEdr::OnMetricUnit\n"));

	UINT nCheck;
	int  iPutCmd;
	if(m_jOptionDlg.m_nMetricUnit == BST_CHECKED)
	{
		nCheck  = BST_UNCHECKED;
		iPutCmd = m_eGmPutMUF;
	}
	else
	{
		nCheck  = BST_CHECKED;
		iPutCmd = m_eGmPutMUT;
	}
	m_jOptionDlg.m_nMetricUnit = nCheck;
	SetBasePoint(LVI_FIND, TRUE);
	PutGoogleMaps(iPutCmd, UNUSED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Adjacent Distances

AFXINLINE void CTomPoiEdr::OnAdjDistance()
{
	TRACE(_T("Entering CTomPoiEdr::OnAdjDistance\n"));

	if(m_bAdjDistance)
		return;

	CheckMenuRadioItem(m_hHdPopMenu, ID_ADJ_DISTANCE, ID_BASE_DISTANCE,
		ID_ADJ_DISTANCE, MF_BYCOMMAND);
	m_bAdjDistance = TRUE;
	if(m_iLvBaseItem != LVI_ERR)
		SetDistanceText(LVI_ALL, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Base Point Distances

AFXINLINE void CTomPoiEdr::OnBaseDistance()
{
	TRACE(_T("Entering CTomPoiEdr::OnBaseDistance\n"));

	if(!m_bAdjDistance)
		return;

	CheckMenuRadioItem(m_hHdPopMenu, ID_ADJ_DISTANCE, ID_BASE_DISTANCE,
		ID_BASE_DISTANCE, MF_BYCOMMAND);
	m_bAdjDistance = FALSE;
	if(m_iLvBaseItem != LVI_ERR)
		SetDistanceText(LVI_ALL, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Sort Ascending

AFXINLINE void CTomPoiEdr::OnSortAscend()
{
	TRACE(_T("Entering CTomPoiEdr::OnSortAscend\n"));

	SortLvItems(m_iLvCol, TRUE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Sort Descending

AFXINLINE void CTomPoiEdr::OnSortDescend()
{
	TRACE(_T("Entering CTomPoiEdr::OnSortDescend\n"));

	SortLvItems(m_iLvCol, FALSE, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from Application key is down

AFXINLINE void CTomPoiEdr::OnAppKeyDown()
{
	TRACE(_T("Entering CTomPoiEdr::OnAppKeyDown\n"));

	POINT pt;
	HWND  hHitWnd;
	GetCursorPos(&pt);
	hHitWnd = ::WindowFromPoint(pt);
	if(!hHitWnd)
		return;
	if((hHitWnd != m_jListView) && (hHitWnd != m_jLvHeader) && (hHitWnd != m_jToolbar))
		return;

	NMITEMACTIVATE nmia;
	nmia.hdr.hwndFrom = hHitWnd;
	nmia.hdr.code     = NM_RCLICK;
	if(hHitWnd == m_jListView)
	{
		LVHITTESTINFO lvhti;
		m_jListView.ScreenToClient(&pt);
		lvhti.pt   = pt;
		nmia.iItem = m_jListView.HitTest(&lvhti);
		if(nmia.iItem != RV_ERR)
		{
			LVITEM lvi;
			if(!m_jListView.GetItemState(nmia.iItem, LVIS_SELECTED))
			{
				lvi.state     = UNUSED;
				lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
				m_jListView.SetItemState(&lvi);
				lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
				m_jListView.SetItemState(nmia.iItem, &lvi);
			}
		}
	}
	SendMessage(WM_NOTIFY, UNUSED, (LPARAM)&nmia);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window is being closed

AFXINLINE BOOL CTomPoiEdr::OnClose()
{
	TRACE(_T("Entering CTomPoiEdr::OnClose\n"));

	if(m_jLvEdit.IsWindowVisible())
	{
		m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
		if(!m_jLvEdit.m_bTextOK)
			return FALSE;
	}

	if(m_bFileDirty && !PromptMessage())
		return FALSE;

	if(!m_jIEPane.IsWindowVisible())
		return TRUE;

	HideWindow();
	CloseIEPane();
	SetTimer(m_eExitTimerId, m_eExitTimerIntl);

	// Accepts (TRUE) or rejects (FALSE) the window to close
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window is being created

AFXINLINE int CTomPoiEdr::OnCreate(PCCREATESTRUCT pCS)
{
	TRACE(_T("Entering CTomPoiEdr::OnCreate\n"));

	// Initializes variables
	m_jMenuItemIL    = NULL;
	m_hMainMenu      = NULL;
	m_hPopMenu       = NULL;
	m_jToolbarIL     = NULL;
	m_jToolbarHotIL  = NULL;
	m_jTbChevronIL   = NULL;
	m_hSbDC          = NULL;
	m_hMenuBarBkgnd  = NULL;
	m_jPoiBmp        = NULL;
	m_hPoiBmp        = NULL;
	m_iLvCol         = LVC_NAME;
	m_ceDragDrop     = 0;
	m_bFileDirty     = FALSE;
	m_bSetReady      = TRUE;
	m_bAdjIEPane     = FALSE;
	m_bCustTbBtn     = FALSE;
	m_bMarquee       = FALSE;
	m_bGmReady       = FALSE;
	m_bGmSelTimer    = FALSE;
	m_bGmSelItem     = FALSE;
	m_bMBtnDown      = FALSE;
	m_bLRKeyDown     = FALSE;
	m_bBackup        = FALSE;
	m_pszRecentFile  = NULL;
	*m_pszGmTempFile = CHR_NULL;
	*m_pszGeTempFile = CHR_NULL;
	m_pOO            = NULL;
	m_pWB2           = NULL;
	m_pOIPAO         = NULL;
	m_pDsp           = NULL;
	m_pHD2           = NULL;

	// Initializes the OLE library
	OleInitialize();

	// Initializes options
	int i;
	ASSERT(countof(m_pszOptionEntry) == COptionDlg::m_eceCheckBox);
	ASSERT(countof(m_pszOptionEntry) == countof(m_nOptionEntry));
	i = 0;
	do
	{
		*m_jOptionDlg.m_pnCheckBox[i] =
			GetPrivateProfileInt(m_szOptionSection, m_pszOptionEntry[i],
			m_nOptionEntry[i], e_pszProfile);
		if(*m_jOptionDlg.m_pnCheckBox[i] != BST_UNCHECKED)
			*m_jOptionDlg.m_pnCheckBox[i] = BST_CHECKED;
		i++;
	}
	while(i < countof(m_pszOptionEntry));
	m_jOptionDlg.m_ceRecentFile = GetPrivateProfileInt(m_szOptionSection, m_szFileCountEntry,
		m_eceDefRecentFile, e_pszProfile);
	if((m_jOptionDlg.m_ceRecentFile < m_eceMinRecentFile) ||
		(m_jOptionDlg.m_ceRecentFile > m_eceMaxRecentFile))
		m_jOptionDlg.m_ceRecentFile = m_eceDefRecentFile;
	m_jOptionDlg.m_ceBackupIntl = GetPrivateProfileInt(m_szOptionSection, m_szBackupIntlEntry,
		m_eceDefBackupIntl, e_pszProfile);
	if((m_jOptionDlg.m_ceBackupIntl < m_eceMinBackupIntl) ||
		(m_jOptionDlg.m_ceBackupIntl > m_eceMaxBackupIntl))
		m_jOptionDlg.m_ceBackupIntl = m_eceDefBackupIntl;
	m_jOptionDlg.m_iGmLang = clamp(GetPrivateProfileInt(m_szOptionSection, m_szGmLangEntry,
		m_eDefGmLang, e_pszProfile), m_eMinGmLang, m_eMaxGmLang);
	m_crTbBkgnd            = GetPrivateProfileInt(m_szOptionSection, m_szTbBkgndEntry,
		(e_dwWinVer <= WV_XP) ? m_ecrTbBkgndXP : m_ecrTbBkgnd, e_pszProfile);
	m_iInsertBP            = clamp(GetPrivateProfileInt(CMyBPDlg::m_szSection,
		CMyBPDlg::m_szInsertEntry, 1, e_pszProfile), 1, CMyBPDlg::m_eceMaxLvItem) - 1;

	// Creates a list view, header, edit and IE pane controls
	m_jListView.CreateWindow(m_eLvStyle, 0, 0, 0, 0, *this, IDW_LIST_VIEW);
	if(!m_jListView)
		return -1;

	DWORD    dwLvStyle;
	COLORREF crLvBkgnd;
	if(e_dwWinVer > WV_XP)
	{
		CImageList jLvColPadIL;
		jLvColPadIL.LoadImage(e_hInstance, IDB_COLUMN_PAD, m_ecxLvColPadIL, 0, CLR_NONE,
			IMAGE_BITMAP, LR_CREATEDIBSECTION);
		m_jListView.SetImageList(jLvColPadIL, LVSIL_SMALL);
		dwLvStyle = m_eLvExStyle;
	}
	else
		dwLvStyle = m_eLvExStyleXP;
	m_jListView.SetExStyle(dwLvStyle, dwLvStyle);
	if(GetPrivateProfileString(m_szFontSection, m_szFontBkgndEntry, e_szNull,
		m_jLvBkDlg.lpstrFile, MAX_PATH, e_pszProfile) && IsValidFile(m_jLvBkDlg.lpstrFile))
	{
		lstrcpy(m_jLvBkDlg.lpstrFileTitle, m_jLvBkDlg.lpstrFile);
		PathStripPath(m_jLvBkDlg.lpstrFileTitle);
		m_bLvBkgnd = GetPrivateProfileInt(m_szFontSection, m_szFontUseBkEntry,
			FALSE, e_pszProfile);
		if(m_bLvBkgnd)
		{
			LVBKIMAGE lvbki;
			lvbki.ulFlags  = LVBKIF_SOURCE_URL | LVBKIF_STYLE_TILE;
			lvbki.pszImage = m_jLvBkDlg.lpstrFile;
			m_bLvBkgnd     = m_jListView.SetBkImage(&lvbki);
		}
	}
	else
	{
		*m_jLvBkDlg.lpstrFile      = CHR_NULL;
		*m_jLvBkDlg.lpstrFileTitle = CHR_NULL;
		m_bLvBkgnd                 = FALSE;
	}
	dwLvStyle = m_jListView.GetWindowStyle();
	if(m_bLvBkgnd)
	{
		dwLvStyle |= LVS_SHOWSELALWAYS;
		crLvBkgnd  = CLR_NONE;
	}
	else
	{
		if(e_dwWinVer > WV_XP)
			dwLvStyle |= LVS_SHOWSELALWAYS;
		else
			dwLvStyle &= ~LVS_SHOWSELALWAYS;
		crLvBkgnd = GetSysColor(COLOR_WINDOW);
	}
	m_jListView.SetWindowStyle(dwLvStyle);
	m_jListView.SetWindowTheme(e_szAppTheme);
	m_jListView.SetTextBkColor(crLvBkgnd);
	m_jListView.SubclassWindow();
	if(!m_jLvHeader.CreateWindow(m_eHdStyle, 0, 0, 0, 0, *this, IDW_LV_HEADER))
		return -1;
	m_jLvHeader.SubclassWindow();
	if(!m_jLvCombo.CreateWindow(m_eCbStyle, 0, 0, 0, 0, m_jListView, IDW_LV_COMBO))
		return -1;
	m_jLvCombo.SubclassWindow();
	if(!m_jLvEdit.CreateWindow(NULL, m_eEdStyle, 0, 0, 0, 0, m_jListView, IDW_LV_EDIT))
		return -1;
	m_jLvEdit.SubclassWindow();
	if(!m_jIEPane.CreateWindow(CIEPane::m_szClassName, NULL, CIEPane::m_ePaneStyle,
		0, 0, 0, 0, *this, IDW_IE_PANE))
		return -1;

	TCHAR    szLvCol[32];
	HDITEM   hi;
	LVCOLUMN lvc;
	lvc.mask    = LVCF_TEXT;
	lvc.pszText = szLvCol;
	hi.mask     = HDI_TEXT;
	hi.pszText  = szLvCol;
	i           = LVC_FIRST;
	do
	{
		LoadString(e_hInstance, IDS_FIRST_COL + i, szLvCol, countof(szLvCol));
		m_jListView.InsertColumn(i, &lvc);
		m_jLvHeader.AppendItem(&hi);
		i++;
	}
	while(i < LVC_COUNT);
	SwapCoordCol();

	COMBOBOXINFO cbi;
	TEXTMETRIC   tm;
	HFONT        hLvOldFont, hLvBuddyFont, hGuiFont;
	HDC          hDC;
	hLvOldFont = m_jListView.GetWindowFont();
	hGuiFont   = GetStockFont(DEFAULT_GUI_FONT);
	if(GetPrivateProfileString(m_szFontSection, m_szFontNameEntry, e_szNull,
		m_jFontDlg.lfFaceName, LF_FACESIZE, e_pszProfile))
	{
		m_jFontDlg.lfHeight         = GetPrivateProfileInt(m_szFontSection, e_szHeight,
			0, e_pszProfile);
		m_jFontDlg.lfWidth          = 0;
		m_jFontDlg.lfEscapement     = 0;
		m_jFontDlg.lfOrientation    = 0;
		m_jFontDlg.lfWeight         = GetPrivateProfileInt(m_szFontSection, m_szFontWeightEntry,
			FW_NORMAL, e_pszProfile);
		m_jFontDlg.lfItalic         = GetPrivateProfileInt(m_szFontSection, m_szFontItalicEntry,
			FALSE, e_pszProfile);
		m_jFontDlg.lfUnderline      = FALSE;
		m_jFontDlg.lfStrikeOut      = FALSE;
		m_jFontDlg.lfOutPrecision   = OUT_DEFAULT_PRECIS;
		m_jFontDlg.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
		m_jFontDlg.lfQuality        = DEFAULT_QUALITY;
		m_jFontDlg.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
		m_jFontDlg.rgbColors        = GetPrivateProfileInt(m_szFontSection, m_szFontColorEntry,
			RGB_BLACK, e_pszProfile);
	}
	else
	{
		m_hLvFont = hLvOldFont ? hLvOldFont : hGuiFont;
		GetObject(m_hLvFont, &m_jFontDlg);
		m_jFontDlg.rgbColors = RGB_BLACK;
	}
	m_jFontDlg.lfCharSet = DEFAULT_CHARSET;
	m_hLvFont            = CreateFont(&m_jFontDlg);
	m_jListView.SetWindowFont(m_hLvFont);
	m_jListView.SetTextColor(m_jFontDlg.rgbColors);
#ifdef NDEBUG
	if(hLvOldFont)
		DeleteFont(hLvOldFont);
#endif
	m_jFontDlg.lStructSize = sizeof(CHOOSEFONT);
	m_jFontDlg.hwndOwner   = *this;
	m_jFontDlg.lpLogFont   = &m_jFontDlg;
	m_jFontDlg.Flags       = CF_APPLY | CF_EFFECTS | CF_ENABLEHOOK | CF_INITTOLOGFONTSTRUCT |
		CF_NOSCRIPTSEL | CF_SCREENFONTS | CF_SELECTSCRIPT;
	m_jFontDlg.lpfnHook    = GetCommonDialogProc(CFontDlg);
	hLvBuddyFont           = m_jLvHeader.GetWindowFont();
	m_jLvHeader.SetWindowFont(m_hLvFont);
	hDC = m_jLvHeader.GetDC();
	SelectFont(hDC, m_hLvFont);
	GetTextMetrics(hDC, &tm);
	m_cyLvHeader = tm.tmHeight + GetSystemMetrics(SM_CYBORDER) + m_ecyHdPad;
	m_cyLvEdit   = tm.tmHeight + (GetSystemMetrics(SM_CYBORDER) + 1) * 2;
	m_jLvHeader.ReleaseDC(hDC);
	if(hLvBuddyFont)
		DeleteFont(hLvBuddyFont);
	hLvBuddyFont = m_jLvEdit.GetWindowFont();
	m_jLvEdit.SetWindowFont(m_hLvFont);
	m_jLvEdit.MoveWindow(0, 0, 0, m_cyLvEdit);
	m_jLvCombo.SetWindowFont(m_hLvFont);
	cbi.cbSize = sizeof(cbi);
	m_jLvCombo.GetInfo(&cbi);
	m_jLvCombo.m_cxDropArrow = GetRectWidth(&cbi.rcButton) + CLvCombo::m_ecxStaticPadding;
	if(hLvBuddyFont)
		DeleteFont(hLvBuddyFont);

	// Creates a status bar control
	if(!m_jStatusBar.CreateWindow(m_eSbStyle, NULL, *this, IDW_STATUS_BAR))
		return -1;
	m_hSbDC = m_jStatusBar.GetDC();
	if(!m_hSbDC)
		return -1;
	if(m_jOptionDlg.m_nShowSb == BST_UNCHECKED)
		m_jStatusBar.HideWindow();

	int   iPartBorder[3];
	HFONT hSbFont;
	m_jStatusBar.GetBorders(iPartBorder);
	m_cxyPartBorder = iPartBorder[2];
	m_cxBmpPart     = 0;
	m_cxPoiPart     = 0;
	m_cxTimePart    = 0;
	hSbFont         = m_jStatusBar.GetWindowFont();
	if(!hSbFont)
		hSbFont = hGuiFont;
	SelectFont(m_hSbDC, hSbFont);
	m_jStatusBar.SubclassWindow();

	// Creates a toolbar control
	m_jToolbar.m_hWnd        = NULL;
	m_jCustTbDlg.m_cxSpacing = clamp(GetPrivateProfileInt(m_szOptionSection, m_szTbSpacingEntry,
		CCustTbDlg::m_eDefSpacing, e_pszProfile),
		CCustTbDlg::m_eMinSpacing, CCustTbDlg::m_eMaxSpacing);
	CreateTbCtrl();
	m_jTbChevronIL.LoadImage(e_hInstance, IDB_TOOLBAR_CHEVRON, m_ecxTbChevronImage, 0, CLR_NONE,
		IMAGE_BITMAP, LR_CREATEDIBSECTION);
	if(m_jOptionDlg.m_nShowTb == BST_UNCHECKED)
		m_jToolbar.HideWindow();

	// Menu help descriptions
	m_hMainMenu     = LoadMenu(e_hInstance, IDR_MAIN);
	m_mh[0].hMenu   = m_hMainMenu;
	m_mh[0].nBaseId = IDP_MAIN_MENU0;
	m_mh[1].hMenu   = NULL;
	m_hPopMenu      = LoadMenu(e_hInstance, IDM_POPUP);
	m_hTbPopMenu    = GetSubMenu(m_hPopMenu, m_eTbPopMenu);
	m_hHdPopMenu    = GetSubMenu(m_hPopMenu, m_eHdPopMenu);
	m_hLvPopMenu    = GetSubMenu(m_hPopMenu, m_eLvPopMenu);
	m_bAdjDistance  = GetPrivateProfileInt(m_szOptionSection, m_szAdjDistEntry,
		TRUE, e_pszProfile);
	CheckMenuRadioItem(m_hHdPopMenu, ID_ADJ_DISTANCE, ID_BASE_DISTANCE,
		m_bAdjDistance ? ID_ADJ_DISTANCE : ID_BASE_DISTANCE, MF_BYCOMMAND);

	// Menu information
	PTBBUTTON    pTBB;
	UINT         nItemId;
	MENUITEMINFO mii;
	m_jMenuItemIL.LoadImage(e_hInstance, IDB_TOOLBAR_SMALL, m_ecxTbImageS, 0, CLR_NONE,
		IMAGE_BITMAP, LR_CREATEDIBSECTION);
	mii.cbSize = sizeof(mii);
	mii.fMask  = MIIM_BITMAP | MIIM_DATA;
	pTBB       = m_tbb + countof(m_tbb) - 1;
	if(e_dwWinVer > WV_XP)
	{
		// Menu item bitmaps
		PVOID      pvBitmap;
		PBYTE      pbyImage;
		int        cbImage;
		DIBSECTION dibs;
		IMAGEINFO  ii;
		m_jMenuItemIL.GetImageInfo(0, &ii);
		GetObject(ii.hbmImage, &dibs);
		dibs.dsBmih.biHeight    = m_ecyTbImageS;
		dibs.dsBmih.biSizeImage = 0;
		cbImage                 = dibs.dsBm.bmWidthBytes * m_ecyTbImageS;
		pbyImage                = (PBYTE)dibs.dsBm.bmBits +
			((dibs.dsBm.bmHeight / m_ecyTbImageS) - m_eceTbImage) * cbImage;
		do
		{
			nItemId = pTBB->idCommand;
			if(nItemId)
			{
				mii.hbmpItem = CreateDIBSection((PCBITMAPINFO)&dibs.dsBmih,
					DIB_RGB_COLORS, &pvBitmap);
				CopyMemory(pvBitmap, pbyImage, cbImage);
				mii.dwItemData = pTBB->iBitmap;
				SetMenuItemInfo(m_hMainMenu, nItemId, FALSE, &mii);
				SetMenuItemInfo(m_hLvPopMenu, nItemId, FALSE, &mii);
				pbyImage += cbImage;
			}
			pTBB--;
		}
		while(pTBB >= m_tbb);
	}
	else
	{
		// No bitmap spacing for menu items
		MENUINFO mi;
		HMENU    hPopMenu;
		mi.cbSize  = sizeof(mi);
		mi.fMask   = MIM_STYLE;
		mi.dwStyle = MNS_NOCHECK;
		i          = 0;
		do
		{
			if(i != m_eToolsMenuPos)
			{
				hPopMenu = GetSubMenu(m_hMainMenu, i);
				SetMenuInfo(hPopMenu, &mi);
			}
			i++;
		}
		while(i <= m_eLastMenuPos);
		SetMenuInfo(m_hLvPopMenu, &mi);

		// Menu bar color
		mi.fMask   = MIM_BACKGROUND;
		mi.hbrBack = CreateSolidBrush(m_ecrMenuBkgnd);
		SetMenuInfo(m_hMainMenu, &mi);
		m_hMenuBarBkgnd = mi.hbrBack;

		// Menu item bitmaps
		mii.hbmpItem = HBMMENU_CALLBACK;
		do
		{
			nItemId = pTBB->idCommand;
			if(nItemId)
			{
				mii.dwItemData = pTBB->iBitmap;
				SetMenuItemInfo(m_hMainMenu, nItemId, FALSE, &mii);
				SetMenuItemInfo(m_hLvPopMenu, nItemId, FALSE, &mii);
			}
			pTBB--;
		}
		while(pTBB >= m_tbb);

		// No bitmaps for menu items
		SCUINT snItemId[] =
		{
			ID_POI_COPY_COORD,
			ID_POI_PASTE_COORD,
			ID_FILE_EXIT,
			ID_POI_SEL_ALL,
			ID_POI_TOGGLE_SEL,
			ID_TOOLS_SHOWMB,
			ID_TOOLS_SHOWTB,
			ID_TOOLS_LARGETB,
			ID_TOOLS_WRAPTB,
			ID_TOOLS_TBBKGND,
			ID_TOOLS_CUSTTB,
			ID_TOOLS_SHOWSB,
		};
		mii.dwItemData = MAX_DWORD;
		i              = 0;
		do
		{
			SetMenuItemInfo(m_hMainMenu, snItemId[i], FALSE, &mii);
			if(i < 2)
				SetMenuItemInfo(m_hLvPopMenu, snItemId[i], FALSE, &mii);
			i++;
		}
		while(i < countof(snItemId));
	}

	// Recent file list
	m_hFileMenu    = GetSubMenu(m_hMainMenu, m_eFileMenuPos);
	m_ceRecentFile = 0;
	if(m_jOptionDlg.m_nRecentFile == BST_CHECKED)
	{
		TCHAR szEntry[MAX_STR];
		PTSTR pszRecentFile;
		m_pszRecentFile = (PTSTR)malloc(m_eceMaxRecentFile * MAX_PATH * sizeof(TCHAR));
		do
		{
			pszRecentFile = m_pszRecentFile + m_ceRecentFile * MAX_PATH;
			wsprintf(szEntry, m_szRecentListEntry, m_ceRecentFile + 1);
			if(!GetPrivateProfileString(m_szOptionSection, szEntry, e_szNull,
				pszRecentFile, MAX_PATH, e_pszProfile))
				break;
			InsertRecentFile(pszRecentFile, m_ceRecentFile);
			m_ceRecentFile++;
		}
		while(m_ceRecentFile < m_jOptionDlg.m_ceRecentFile);
		if(m_ceRecentFile)
			InsertMenu(m_hFileMenu, m_eRecentInsPos + m_ceRecentFile,
				MF_BYPOSITION | MF_SEPARATOR);
	}

	// System menu
	SCUINT snBarId[] = { ID_TOOLS_SHOWMB, ID_TOOLS_SHOWTB, ID_TOOLS_SHOWSB };
	TCHAR  szText[MAX_STR];
	m_hSysMenu = GetSystemMenu();
	i          = 0;
	do
	{
		nItemId = snBarId[i];
		GetMenuString(m_hMainMenu, nItemId, szText, countof(szText), MF_BYCOMMAND);
		RemoveMenuItemAmp(szText);
		InsertMenu(m_hSysMenu, i, MF_STRING | MF_BYPOSITION, nItemId, szText);
		i++;
	}
	while(i < countof(snBarId));
	InsertMenu(m_hSysMenu, i, MF_SEPARATOR | MF_BYPOSITION);

	// Shows window menu bar
	if(m_jOptionDlg.m_nShowMb == BST_CHECKED)
		SetMenu(m_hMainMenu);

	// Initializes the structure for the Open and Save As dialog boxes
	SCTCHAR  szFileFilterOv2[] = _T(" (*.ov2)\0*.ov2");
	SCTCHAR  szFileFilterTxt[] = _T(" (*.txt)\0*.txt");
	SCTCHAR  szFileFilterKml[] = _T(" (*.kml)\0*.kml");
	SCTCHAR  szFileFilterGpx[] = _T(" (*.gpx)\0*.gpx");
	SCTCHAR  szFileFilterGpi[] = _T(" (*.gpi)\0*.gpi");
	SCTCHAR  szFileFilterCsv[] = _T(" (*.csv)\0*.csv");
	SCTCHAR  szFileFilterAsc[] = _T(" (*.asc)\0*.asc");
	SCTCHAR  szFileFilterPoi[] = _T(" (*.poi)\0*.poi");
	SCPCTSTR spszFilterR[] =
	{
		szFileFilterOv2,
		szFileFilterTxt,
		szFileFilterKml,
		szFileFilterGpx,
		szFileFilterGpi,
		szFileFilterCsv,
		szFileFilterAsc,
		szFileFilterPoi,
		szFileFilterTxt,
	};
	SCINT siResIdR[] =
	{
		IDS_FILE_FILTER_OV2,
		IDS_FILE_FILTER_CPB,
		IDS_FILE_FILTER_KML,
		IDS_FILE_FILTER_GPX,
		IDS_FILE_FILTER_GPI,
		IDS_FILE_FILTER_CSV,
		IDS_FILE_FILTER_ASC,
		IDS_FILE_FILTER_POI,
		IDS_FILE_FILTER_TXT,
	};
	SCPCTSTR spszFilterW[] =
	{
		szFileFilterOv2,
		szFileFilterTxt,
		szFileFilterKml,
		szFileFilterGpx,
		szFileFilterCsv,
		szFileFilterCsv,
		szFileFilterCsv,
		szFileFilterAsc,
		szFileFilterAsc,
		szFileFilterAsc,
		szFileFilterPoi,
		szFileFilterPoi,
		szFileFilterPoi,
		szFileFilterTxt,
		szFileFilterTxt,
		szFileFilterTxt,
	};
	SCINT siResIdW[] =
	{
		IDS_FILE_FILTER_OV2,
		IDS_FILE_FILTER_CPB,
		IDS_FILE_FILTER_KML,
		IDS_FILE_FILTER_GPX,
		IDS_FILE_FILTER_CSVA,
		IDS_FILE_FILTER_CSVU,
		IDS_FILE_FILTER_CSVU8,
		IDS_FILE_FILTER_ASCA,
		IDS_FILE_FILTER_ASCU,
		IDS_FILE_FILTER_ASCU8,
		IDS_FILE_FILTER_POIA,
		IDS_FILE_FILTER_POIU,
		IDS_FILE_FILTER_POIU8,
		IDS_FILE_FILTER_TXTA,
		IDS_FILE_FILTER_TXTU,
		IDS_FILE_FILTER_TXTU8,
	};
	int iNextPos;
	iNextPos = 0;
	i        = 0;
	do
	{
		iNextPos += LoadString(e_hInstance, siResIdR[i], m_pszFileFilterR + iNextPos, MAX_STR);
		CopyMemory(m_pszFileFilterR + iNextPos, spszFilterR[i], sizeof(szFileFilterOv2));
		iNextPos += countof(szFileFilterOv2);
		i++;
	}
	while(i < countof(spszFilterR));
	m_pszFileFilterR[iNextPos] = CHR_NULL;
	iNextPos                   = 0;
	i                          = 0;
	do
	{
		iNextPos += LoadString(e_hInstance, siResIdW[i], m_pszFileFilterW + iNextPos, MAX_STR);
		CopyMemory(m_pszFileFilterW + iNextPos, spszFilterW[i], sizeof(szFileFilterOv2));
		iNextPos += countof(szFileFilterOv2);
		i++;
	}
	while(i < countof(spszFilterW));
	m_pszFileFilterW[iNextPos]   = CHR_NULL;
	m_jFileDlg.lStructSize       = sizeof(OPENFILENAME);
	m_jFileDlg.hwndOwner         = *this;
	m_jFileDlg.lpstrCustomFilter = NULL;
	m_jFileDlg.nMaxFile          = MAX_PATH;
	m_jFileDlg.nMaxFileTitle     = MAX_FNAME;
	m_jFileDlg.lpstrDefExt       = m_szOv2Ext;
	m_jFileDlg.lpfnHook          = GetCommonDialogProc(CFileDlg);
	m_jFileDlg.FlagsEx           = UNUSED;

	// Initializes the structure for the List View Background dialog box
	SCTCHAR szFileFilterAll[] = _T(" (*.*)\0*.*\0");
	CopyMemory(m_pszFileFilterB + LoadString(e_hInstance, IDS_FILE_FILTER_IMAGE,
		m_pszFileFilterB, MAX_STR), szFileFilterAll, sizeof(szFileFilterAll));
	m_jLvBkDlg.lStructSize       = sizeof(OPENFILENAME);
	m_jLvBkDlg.hwndOwner         = *this;
	m_jLvBkDlg.lpstrFilter       = m_pszFileFilterB;
	m_jLvBkDlg.lpstrCustomFilter = NULL;
	m_jLvBkDlg.nFilterIndex      = 1;
	m_jLvBkDlg.nMaxFile          = MAX_PATH;
	m_jLvBkDlg.nMaxFileTitle     = MAX_FNAME;
	m_jLvBkDlg.Flags             = OFN_ENABLEHOOK | OFN_ENABLESIZING | OFN_EXPLORER |
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
	m_jLvBkDlg.lpstrDefExt       = NULL;
	m_jLvBkDlg.lpfnHook          = GetCommonDialogProc(CLvBkDlg);
	m_jLvBkDlg.FlagsEx           = UNUSED;

	// Creates temporary files used by Google Maps and Earth
	int  ccTempPath;
	LONG cbValue;
	ccTempPath = GetTempPath(MAX_PATH, m_pszGeTempFile);
	GetTempFileName(m_pszGeTempFile, _T("TPE"), 0, m_pszGmTempFile);
	DeleteFile(m_pszGmTempFile);
	PathRenameExtension(m_pszGmTempFile, _T(".htm"));
	LoadString(e_hInstance, IDS_GOOGLE_EARTH_APPID, szText, countof(szText));
	cbValue = sizeof(szText);
	if(RegQueryValue(HKEY_LOCAL_MACHINE, szText, &cbValue) == REG_OK)
		LoadString(e_hInstance, IDS_GOOGLE_EARTH_TMPFILE,
			m_pszGeTempFile + ccTempPath, MAX_PATH);
	else
		*m_pszGeTempFile = CHR_NULL;

	// Checks whether the TomTom HOME services are available
	LoadString(e_hInstance, IDS_TOMTOM_HOME_SERVICE, szText, countof(szText));
	cbValue       = sizeof(szText);
	m_bTomTomHome = RegQueryValue(HKEY_LOCAL_MACHINE, szText, &cbValue) == REG_OK;

	// Sets the required timers
	SetTimer(m_eMainTimerId, m_eMainTimerIntl);
	if(m_jOptionDlg.m_nBackupFile == BST_CHECKED)
		SetTimer(m_eBackupTimerId, m_jOptionDlg.m_ceBackupIntl * m_eMinuteIntl);

	// Registers clipboard formats
	SCTCHAR szSC[] = _T(" (SC)");
	m_nPoiCbFmt = RegisterClipboardFormat(e_pszTpeAppName);
	lstrcpy(szText, e_pszTpeAppName);
	lstrcat(szText, szSC);
	m_nPoiScCbFmt = RegisterClipboardFormat(szText);
	m_nItnCbFmt   = RegisterClipboardFormat(e_pszTieAppName);

	// Registers main window to accept dropped files
	DragAcceptFiles(TRUE);

	// Creates a new file
	SendMessage(WM_COMMAND, ID_FILE_NEW);

	// Determines whether a GPS device has been added to the system
	m_bGpsDevice    = FALSE;
	m_gpsd          = GPSD_NONE;
	m_hGpsDevThread = CreateThread(GpsDeviceChange, CT_IMMEDIATE);

	// Notifies whether the new version is available
	m_hNewVerThread = CreateThread(NotifyNewVersion, CT_IMMEDIATE);

	// Loads the file specified in the command line
	if(IsValidFile(m_pszCmdLine))
	{
		PostMessage(WM_COMMAND, ID_OPEN_CMDLINE);
		return 0;
	}

	// Reloads the last file, if necessary
	if((m_jOptionDlg.m_nOpenLast == BST_CHECKED) &&
		m_ceRecentFile && IsValidFile(m_pszRecentFile))
		PostMessage(WM_COMMAND, ID_RECENT_FILE1);

	// Continues creation (0) or destroys (-1) the window
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window is being destroyed

AFXINLINE void CTomPoiEdr::OnDestroy()
{
	TRACE(_T("Entering CTomPoiEdr::OnDestroy\n"));

	TCHAR szValue[MAX_STR];
	int   i;
	szValue[1] = CHR_NULL;
	i          = 0;
	do
	{
		szValue[0] = *m_jOptionDlg.m_pnCheckBox[i] + _T('0');
		WritePrivateProfileString(m_szOptionSection, m_pszOptionEntry[i],
			szValue, e_pszProfile);
		i++;
	}
	while(i < countof(m_pszOptionEntry));

	szValue[0] = m_bAdjDistance + _T('0');
	WritePrivateProfileString(m_szOptionSection, m_szAdjDistEntry, szValue, e_pszProfile);

	if(m_jOptionDlg.m_nRecentFile == BST_CHECKED)
	{
		TCHAR szEntry[MAX_STR];
		szValue[0] = m_jOptionDlg.m_ceRecentFile + _T('0');
		WritePrivateProfileString(m_szOptionSection, m_szFileCountEntry, szValue, e_pszProfile);
		for(i = 0; i < m_ceRecentFile; i++)
		{
			wsprintf(szEntry, m_szRecentListEntry, i + 1);
			WritePrivateProfileString(m_szOptionSection, szEntry,
				m_pszRecentFile + i * MAX_PATH, e_pszProfile);
		}
	}

	if(m_jOptionDlg.m_nBackupFile == BST_CHECKED)
	{
		_ltot(m_jOptionDlg.m_ceBackupIntl, szValue);
		WritePrivateProfileString(m_szOptionSection, m_szBackupIntlEntry,
			szValue, e_pszProfile);
	}

	SCPCTSTR        spszWndEntry[] = { e_szLeft, e_szTop, e_szWidth, e_szHeight };
	int             cyIEPane;
	RECT            rcIEPane;
	WINDOWPLACEMENT wp;
	GetWindowPlacement(&wp);
	wp.rcNormalPosition.right  -= wp.rcNormalPosition.left;
	wp.rcNormalPosition.bottom -= wp.rcNormalPosition.top;
	i                           = 0;
	do
	{
		_ltot(((PCLONG)&wp.rcNormalPosition)[i], szValue);
		WritePrivateProfileString(e_szWndSection, spszWndEntry[i], szValue, e_pszProfile);
		i++;
	}
	while(i < countof(spszWndEntry));
	szValue[0] = wp.showCmd + _T('0');
	szValue[1] = CHR_NULL;
	WritePrivateProfileString(e_szWndSection, e_szWndShowEntry, szValue, e_pszProfile);
	m_jIEPane.GetWindowRect(&rcIEPane);
	cyIEPane = GetRectWidth(&rcIEPane);
	if(cyIEPane)
	{
		_ltot(cyIEPane, szValue);
		WritePrivateProfileString(e_szWndSection, m_szIEPaneEntry, szValue, e_pszProfile);
	}

	SCPCTSTR spszFontEntry[] = { e_szHeight, m_szFontWeightEntry, m_szFontColorEntry };
	int      iFontValue[countof(spszFontEntry)];
	LOGFONT  lf;
	GetObject(m_hLvFont, &lf);
	iFontValue[0] = m_jFontDlg.lfHeight;
	iFontValue[1] = m_jFontDlg.lfWeight;
	iFontValue[2] = m_jListView.GetTextColor();
	WritePrivateProfileString(m_szFontSection, m_szFontNameEntry, lf.lfFaceName, e_pszProfile);
	i = 0;
	do
	{
		_ltot(iFontValue[i], szValue);
		WritePrivateProfileString(m_szFontSection, spszFontEntry[i], szValue, e_pszProfile);
		i++;
	}
	while(i < countof(spszFontEntry));
	szValue[0] = m_jFontDlg.lfItalic + _T('0');
	szValue[1] = CHR_NULL;
	WritePrivateProfileString(m_szFontSection, m_szFontItalicEntry, szValue, e_pszProfile);
	szValue[0] = m_bLvBkgnd + _T('0');
	WritePrivateProfileString(m_szFontSection, m_szFontUseBkEntry, szValue, e_pszProfile);
	if(IsValidFile(m_jLvBkDlg.lpstrFile))
		WritePrivateProfileString(m_szFontSection, m_szFontBkgndEntry,
			m_jLvBkDlg.lpstrFile, e_pszProfile);

	_ltot(m_jOptionDlg.m_iGmLang, szValue);
	WritePrivateProfileString(m_szOptionSection, m_szGmLangEntry, szValue, e_pszProfile);

	szValue[0] = m_jCustTbDlg.m_cxSpacing + _T('0');
	szValue[1] = CHR_NULL;
	WritePrivateProfileString(m_szOptionSection, m_szTbSpacingEntry, szValue, e_pszProfile);

	SendMessage(WM_TASKBAR_ICON, IDR_MAIN, NIN_BALLOONTIMEOUT);

	if(*m_pszGeTempFile)
		DeleteFile(m_pszGeTempFile);
	if(*m_pszGmTempFile)
		DeleteFile(m_pszGmTempFile);
	free(m_pszRecentFile);
	ASSERT(!m_ceDragDrop);
	if(m_hPoiBmp)
		DeleteBitmap(m_hPoiBmp);
	if(m_hMenuBarBkgnd)
		DeleteBrush(m_hMenuBarBkgnd);
	if(m_hSbDC)
		m_jStatusBar.ReleaseDC(m_hSbDC);
	if(m_jTbChevronIL)
		m_jTbChevronIL.Destroy();
	if(m_jToolbarHotIL)
		m_jToolbarHotIL.Destroy();
	if(m_jToolbarIL)
		m_jToolbarIL.Destroy();
	if(m_hPopMenu)
		DestroyMenu(m_hPopMenu);
	if(m_hMainMenu)
	{
		if(e_dwWinVer > WV_XP)
		{
			PTBBUTTON    pTBB;
			UINT         nItemId;
			MENUITEMINFO mii;
			mii.cbSize = sizeof(mii);
			mii.fMask  = MIIM_BITMAP;
			pTBB       = m_tbb + countof(m_tbb) - 1;
			do
			{
				nItemId = pTBB->idCommand;
				if(nItemId)
				{
					GetMenuItemInfo(m_hMainMenu, nItemId, FALSE, &mii);
					if(mii.hbmpItem)
						DeleteBitmap(mii.hbmpItem);
				}
				pTBB--;
			}
			while(pTBB >= m_tbb);
		}
		if(m_jOptionDlg.m_nShowMb == BST_UNCHECKED)
			DestroyMenu(m_hMainMenu);
	}
	if(m_jMenuItemIL)
		m_jMenuItemIL.Destroy();
	ReleaseIEObject();
	OleUninitialize();
	PostQuitMessage(EC_SUCCESS);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A change to the hardware configuration of a device or the computer

AFXINLINE UINT CTomPoiEdr::OnDeviceChange(UINT nEvent, PCVOID pvStruct)
{
	TRACE(_T("Entering CTomPoiEdr::OnDeviceChange\n"));

	if(nEvent == DBT_DEVNODES_CHANGED)
	{
		if(!m_hGpsDevThread && (::FindWindow(e_pszTpeClassName) == *this))
			m_hGpsDevThread = CreateThread(GpsDeviceChange, CT_IMMEDIATE);
		return TRUE;
	}

	if((nEvent != DBT_DEVICEARRIVAL) && (nEvent != DBT_DEVICEREMOVECOMPLETE))
		return TRUE;

	HWND hPopup;
	hPopup = GetLastActivePopup();
	if(hPopup == *this)
		return TRUE;

	SCUINT snCmdId[] = { ID_FILE_OPEN_FAV, ID_FILE_TRANSLATE, ID_FILE_CONVERT };
	int    i;
	UINT   nCmdId;
	i = 0;
	do
	{
		nCmdId = snCmdId[i];
		if(m_jToolbar.IsButtonPressed(nCmdId))
			break;
		i++;
	}
	while(i < countof(snCmdId));
	if(i == countof(snCmdId))
		return TRUE;

	HWND hDialog[] = { m_jOpenFavDlg, m_jTranNameDlg, m_jConvertFormatDlg };
	ASSERT(hPopup != m_jCheckDataDlg);
	if(hPopup != hDialog[i])
		return TRUE;

	::PostMessage(hPopup, WM_COMMAND, MAKEWPARAM(ID_CANCEL, BN_CLICKED));
	PostMessage(WM_COMMAND, nCmdId);

	// Accepts (TRUE) or rejects (BROADCAST_QUERY_DENY) the request
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A visual aspect of an owner-drawn control or menu item has changed

AFXINLINE void CTomPoiEdr::OnDrawItem(PCDRAWITEMSTRUCT pDIS)
{
	TRACE(_T("Entering CTomPoiEdr::OnDrawItem\n"));

	ASSERT(pDIS->CtlType == ODT_MENU);
	if(pDIS->itemData == MAX_DWORD)
		return;

	SIMAGELISTDRAWPARAMS sildp =
	{
		sizeof(sildp),
		NULL,
		UNUSED,
		UNUSED,
		UNUSED,
		UNUSED,
		0,
		0,
		0,
		0,
		CLR_NONE,
		CLR_NONE,
		ILD_TRANSPARENT,
		UNUSED,
		UNUSED,
		UNUSED,
		UNUSED,
	};
	sildp.himl   = m_jMenuItemIL;
	sildp.i      = pDIS->itemData;
	sildp.hdcDst = pDIS->hDC;
	sildp.x      = pDIS->rcItem.left + m_ecxMenuBmpPadL;
	sildp.y      = pDIS->rcItem.top + m_ecyMenuBmpPadT;
	sildp.fState = (pDIS->itemState & ODS_GRAYED) ? ILS_SATURATE : ILS_NORMAL;
	DrawImage(&sildp);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Drops files in a window under the cursor

AFXINLINE void CTomPoiEdr::OnDropFiles(HDROP hDropFile)
{
	TRACE(_T("Entering CTomPoiEdr::OnDropFiles\n"));

	TCHAR szDropFile[MAX_PATH];
	int   ceDropFile;
	POINT pt;
	ceDropFile = DragQueryFile(hDropFile);
	DragQueryFile(hDropFile, 0, szDropFile, countof(szDropFile));
	DragQueryPoint(hDropFile, &pt);
	DragFinish(hDropFile);
	if(m_jIEPane.IsWindowVisible())
	{
		RECT rc;
		m_jIEPane.GetWindowRect(&rc);
		::MapWindowPoints(*this, &rc);
		if(PtInRect(&rc, pt))
			return;
	}

	UINT   nResId;
	LPARAM lParam;
	if(ceDropFile == 1)
	{
		PCTSTR pszFileExt;
		pszFileExt = PathFindExtension(szDropFile);
		if(*pszFileExt)
		{
			SCPCTSTR spszFTR[] =
			{
				m_szOv2Ext,
				m_szKmlExt,
				m_szGpxExt,
				m_szGpiExt,
				m_szCsvExt,
				m_szAscExt,
				m_szPoiExt,
				m_szTxtExt,
			};
			pszFileExt++;
			for(int i = 0; (i < countof(spszFTR)) && lstrcmpi(pszFileExt, spszFTR[i]); i++);
			if(i < countof(spszFTR))
			{
				OpenPoiFile((UINT)szDropFile);
				return;
			}
		}
		nResId = IDS_FILE_EXT_ERROR;
		lParam = (LPARAM)PathFindFileName(szDropFile);
	}
	else
	{
		nResId = IDS_ONLY_DROP_ONE;
		lParam = UNUSED;
	}
	DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, lParam);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size or position is about to change

AFXINLINE void CTomPoiEdr::OnGetMinMaxInfo(PMINMAXINFO pMMI)
{
	TRACE(_T("Entering CTomPoiEdr::OnGetMinMaxInfo\n"));

	pMMI->ptMinTrackSize.x = max(pMMI->ptMinTrackSize.x, e_ecxMinWnd);
	pMMI->ptMinTrackSize.y = max(pMMI->ptMinTrackSize.y, e_ecyMinWnd);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A menu is about to become active

AFXINLINE void CTomPoiEdr::OnInitMenu(HMENU hMenu)
{
	TRACE(_T("Entering CTomPoiEdr::OnInitMenu\n"));

	if(m_jLvEdit.IsWindowVisible())
	{
		m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
		if(!m_jLvEdit.m_bTextOK)
			return;
	}

	int  ceLvItem;
	UINT nLvItem;
	ceLvItem = m_jListView.GetItemCount();
	nLvItem  = ceLvItem ? MF_ENABLED : MF_GRAYED;
	if((hMenu == m_hMainMenu) || (hMenu == m_hTbPopMenu))
	{
		// Only in main menu and toolbar popup menu
		UINT nEnable;
		nEnable = m_cyToolbar ? MF_ENABLED : MF_GRAYED;
		CheckMenuItem(hMenu, ID_TOOLS_LARGETB,
			(m_jOptionDlg.m_nLargeTb == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_TOOLS_LARGETB, nEnable);
		CheckMenuItem(hMenu, ID_TOOLS_WRAPTB,
			(m_jOptionDlg.m_nWrapTb == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_TOOLS_WRAPTB, nEnable);
		EnableMenuItem(hMenu, ID_TOOLS_TBBKGND, nEnable);
		EnableMenuItem(hMenu, ID_TOOLS_CUSTTB, nEnable);
		if(hMenu == m_hTbPopMenu)
			return;
	}
	else if(hMenu == m_hHdPopMenu)
	{
		// Only in header popup menu
		UINT nSort, nShowDist, nCPB;
		if(m_bMyBP)
			ceLvItem--;
		nSort     = ((ceLvItem > 1) && ((m_iLvCol != LVC_DIST) || (m_iLvBaseItem != LVI_ERR)) &&
			(m_iLvCol != INVALID_VALUE)) ? MF_ENABLED : MF_GRAYED;
		nShowDist = (m_jOptionDlg.m_nShowDist == BST_CHECKED) ? MF_ENABLED : MF_GRAYED;
		nCPB      = (m_fst != FST_CPB) ? MF_ENABLED : MF_GRAYED;
		EnableMenuItem(hMenu, ID_LNG_FIRST, nCPB);
		CheckMenuItem(hMenu, ID_LNG_FIRST,
			(m_jOptionDlg.m_nLngFirst == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_DMS_FORMAT, nCPB);
		CheckMenuItem(hMenu, ID_DMS_FORMAT,
			(m_jOptionDlg.m_nDmsFormat == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		CheckMenuItem(hMenu, ID_COMPASS_DIR,
			(m_jOptionDlg.m_nCompassDir == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_CLEAR_COL, ((m_iLvCol != LVC_NAME) && (m_iLvCol != LVC_DIST) &&
			(m_iLvCol != INVALID_VALUE)) ? nLvItem : MF_GRAYED);
		EnableMenuItem(hMenu, ID_CLEAR_ADDR, ((m_iLvCol == LVC_MEMO) &&
			(m_fst == FST_GENERAL) && !m_bRPF) ? nLvItem : MF_GRAYED);
		EnableMenuItem(hMenu, ID_SORT_ASCEND, nSort);
		EnableMenuItem(hMenu, ID_SORT_DESCEND, nSort);
		EnableMenuItem(hMenu, ID_SHOW_DISTANCE, nCPB);
		CheckMenuItem(hMenu, ID_SHOW_DISTANCE,
			(m_jOptionDlg.m_nShowDist == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_METRIC_UNIT, nCPB);
		CheckMenuItem(hMenu, ID_METRIC_UNIT,
			(m_jOptionDlg.m_nMetricUnit == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED);
		EnableMenuItem(hMenu, ID_ADJ_DISTANCE, nShowDist);
		EnableMenuItem(hMenu, ID_BASE_DISTANCE, nShowDist);
		return;
	}

	// Only in main menu and list view popup menu
	if((hMenu == m_hMainMenu) || (hMenu == m_hLvPopMenu))
	{
		PTBBUTTON pTBB;
		int       ceSelItem;
		UINT      nItemId;
		UpdateTbButton();
		pTBB = m_tbb + countof(m_tbb) - 1;
		do
		{
			nItemId = pTBB->idCommand;
			if(nItemId)
				EnableMenuItem(hMenu, nItemId,
					m_jToolbar.IsButtonEnabled(nItemId) ? MF_ENABLED : MF_GRAYED);
			pTBB--;
		}
		while(pTBB >= m_tbb);
		ceSelItem = m_jListView.GetSelCount();
		EnableMenuItem(hMenu, ID_POI_COPY_COORD, (m_fst != FST_CPB) && (ceSelItem == 1) ?
			MF_ENABLED : MF_GRAYED);
		EnableMenuItem(hMenu, ID_POI_PASTE_COORD,
			((m_fst == FST_CPB) ||
			IsClipboardFormatAvailable(m_nPoiCbFmt) ||
			IsClipboardFormatAvailable(m_nPoiScCbFmt) ||
			IsClipboardFormatAvailable(m_nItnCbFmt)) ?
			MF_GRAYED : ((IsClipboardFormatAvailable(CF_TEXT) && ceSelItem) ?
			MF_ENABLED : MF_GRAYED));
		EnableMenuItem(hMenu, ID_POI_SEL_ALL, nLvItem);
		EnableMenuItem(hMenu, ID_POI_TOGGLE_SEL, nLvItem);
		if(hMenu == m_hLvPopMenu)
			return;
	}

	// Only in main menu and system menu
	UINT nCheck;
	nCheck = (m_jOptionDlg.m_nShowMb == BST_CHECKED) ? MF_CHECKED : MF_UNCHECKED;
	CheckMenuItem(hMenu, ID_TOOLS_SHOWMB, nCheck);
	CheckMenuItem(m_hSysMenu, ID_TOOLS_SHOWMB, nCheck);
	nCheck = m_cyToolbar ? MF_CHECKED : MF_UNCHECKED;
	CheckMenuItem(hMenu, ID_TOOLS_SHOWTB, nCheck);
	CheckMenuItem(m_hSysMenu, ID_TOOLS_SHOWTB, nCheck);
	CheckMenuItem(hMenu, ID_TOOLS_LVBKGND, m_bLvBkgnd ? MF_CHECKED : MF_UNCHECKED);
	nCheck = m_cyStatusBar ? MF_CHECKED : MF_UNCHECKED;
	CheckMenuItem(hMenu, ID_TOOLS_SHOWSB, nCheck);
	CheckMenuItem(m_hSysMenu, ID_TOOLS_SHOWSB, nCheck);
	if(hMenu != m_hMainMenu)
		return;

	// Recent file list
	int i;
	for(i = 0; i < m_ceRecentFile; i++)
		EnableMenuItem(hMenu, ID_RECENT_FILE1 + i,
			IsValidFile(m_pszRecentFile + i * MAX_PATH) ? MF_ENABLED : MF_GRAYED);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has double-clicked

AFXINLINE void CTomPoiEdr::OnLButtonDblClk(UINT nFlags, int xCursor, int yCursor)
{
	TRACE(_T("Entering CTomPoiEdr::OnLButtonDblClk\n"));

	if(PtInIEPaneSep(xCursor, yCursor))
		CloseIEPane();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has pressed

AFXINLINE void CTomPoiEdr::OnLButtonDown(UINT nFlags, int xCursor, int yCursor)
{
	TRACE(_T("Entering CTomPoiEdr::OnLButtonDown\n"));

	if(!PtInIEPaneSep(xCursor, yCursor))
		return;

	SetCapture();
	KillTimer(m_eIEPaneTimerId);
	m_bAdjIEPane = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Left mouse button has released

AFXINLINE void CTomPoiEdr::OnLButtonUp(UINT nFlags, int xCursor, int yCursor)
{
	TRACE(_T("Entering CTomPoiEdr::OnLButtonUp\n"));

	if(!m_bAdjIEPane)
		return;

	ReleaseCapture();
	SetTimer(m_eIEPaneTimerId, m_eIEPaneTimerIntl);
	m_bAdjIEPane = FALSE;
	m_bSetReady  = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// An owner-drawn control or menu item is created

AFXINLINE void CTomPoiEdr::OnMeasureItem(PMEASUREITEMSTRUCT pMIS)
{
	TRACE(_T("Entering CTomPoiEdr::OnMeasureItem\n"));

	ASSERT(pMIS->CtlType == ODT_MENU);
	pMIS->itemWidth  = m_ecxTbImageS + m_ecxMenuBmpPadL + m_ecxMenuBmpPadR + m_ecyMenuBmpPadG;
	pMIS->itemHeight = m_ecyTbImageS + m_ecyMenuBmpPadT + m_ecyMenuBmpPadB;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A menu item is selected

AFXINLINE void CTomPoiEdr::OnMenuSelect(UINT nItemId, UINT nFlags, HMENU hMenu)
{
	TRACE(_T("Entering CTomPoiEdr::OnMenuSelect\n"));

	if((nItemId < ID_RECENT_FILE1) || (nItemId > ID_RECENT_FILE9))
	{
		// Handles non-system pop-up menu descriptions
		int    i;
		LPARAM lParam;
		lParam = (LPARAM)hMenu;
		if((nFlags & MF_POPUP) && !(nFlags & MF_SYSMENU))
		{
			for(i = 0; hMenu != m_mh[i].hMenu; i++)
				ASSERT(i < countof(m_mh) - 2);
		}
		else
		{
			hMenu = NULL;
			i     = countof(m_mh) - 1;
		}

		// Displays helpful text in the status bar control
		m_jStatusBar.MenuHelp(MAKEWPARAM(nItemId, nFlags),
			lParam, hMenu, e_hInstance, m_mh + i);
		return;
	}

	TCHAR szText[MAX_STR + MAX_PATH];
	PTSTR pszFileName;
	int   ccText, cxFileName;
	SIZE  size;
	RECT  rc;
	m_jStatusBar.Simple();
	m_jStatusBar.GetClientRect(&rc);
	ccText         = LoadString(e_hInstance, IDP_RECENT_FILE, szText, MAX_STR);
	szText[ccText] = CHR_SP;
	ccText++;
	GetTextExtent(m_hSbDC, szText, ccText, &size);
	pszFileName = szText + ccText;
	cxFileName  = rc.right - size.cx - m_ecxPbPadR;
	GetTextExtent(m_hSbDC, pszFileName,
		TomFile2WinFile(m_pszRecentFile + (nItemId - ID_RECENT_FILE1) * MAX_PATH,
		pszFileName), &size);
	if(size.cx > cxFileName)
		PathCompactPath(m_hSbDC, pszFileName, cxFileName);
	m_jStatusBar.SetText(szText);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse cursor moves in the client area of a window

AFXINLINE void CTomPoiEdr::OnMouseMove(UINT nFlags, int xCursor, int yCursor)
{
	TRACE(_T("Entering CTomPoiEdr::OnMouseMove\n"));

	if(m_bAdjIEPane)
	{
		RECT rc;
		SetCursor(LoadCursor(e_hInstance, IDC_VERT_SPLIT));
		m_jIEPane.GetWindowRect(&rc);
		::MapWindowPoints(*this, &rc);
		xCursor = clamp(xCursor + m_ecxIEPaneSep / 2,
			m_ecxMinIEPane, m_cxClient - m_ecxMinIEPane);
		if(xCursor == rc.left)
			return;
		m_jIEPane.MoveWindow(xCursor, rc.top, m_cxClient - xCursor, GetRectHeight(&rc));
		ResizeWindow(FALSE);
		return;
	}

	if(!PtInIEPaneSep(xCursor, yCursor))
		return;

	InvalidateRect();
	SetTimer(m_eIEPaneTimerId, m_eIEPaneTimerIntl);
	if(m_jLvEdit.IsWindowVisible())
		return;

	TCHAR szText[MAX_STR], szPrompt[countof(szText)];
	m_jStatusBar.GetWindowText(szText, countof(szText));
	TrimString(szText);
	LoadString(e_hInstance, IDP_CLOSE_IE_PANE, szPrompt, countof(szPrompt));
	if(!lstrcmp(szText, szPrompt))
		return;
	m_jStatusBar.SetWindowText(szPrompt);
	m_bSetReady = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user double-clicked the divider area of a header

AFXINLINE void CTomPoiEdr::OnHdnDividerDblClick(PCNMHEADER pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnHdnDividerDblClick\n"));

	AutoSizeLvCol(pNMH->iItem + LVCA_FIRST, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has ended dragging a divider in a header

AFXINLINE void CTomPoiEdr::OnHdnEndTrack(PCNMHEADER pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnHdnEndTrack\n"));

	if(pNMH->iItem != LVC_DIST)
		return;

	int cxHdColPad;
	cxHdColPad = (e_dwWinVer > WV_XP) ? m_ecxHdColPad : m_ecxHdColPadXP;
	if(pNMH->pitem->cxy >= cxHdColPad)
		return;

	static HDITEM shi = { HDI_WIDTH, cxHdColPad };
	m_jLvHeader.PostMessage(HDM_SETITEM, LVC_DIST, (LPARAM)&shi);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user clicked an item in a header

AFXINLINE void CTomPoiEdr::OnHdnItemClick(PCNMHEADER pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnHdnItemClick\n"));

	BOOL bAscend;
	int  iLvCol;
	if(m_iLvCol == pNMH->iItem)
	{
		iLvCol  = m_iLvCol;
		bAscend = !m_bSortAscend;
	}
	else
	{
		iLvCol  = pNMH->iItem;
		bAscend = m_bSortAscend;
	}
	SortLvItems(iLvCol, bAscend, AT_NEW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user is dragging a divider in a header

AFXINLINE BOOL CTomPoiEdr::OnHdnTrack(PCNMHEADER pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnHdnTrack\n"));

	int    cxLvCol;
	HDITEM hi;
	if(pNMH->iItem == LVC_DIST)
	{
		int cxHdColPad;
		cxHdColPad = (e_dwWinVer > WV_XP) ? m_ecxHdColPad : m_ecxHdColPadXP;
		hi.cxy     = max(cxHdColPad, pNMH->pitem->cxy);
		cxLvCol    = hi.cxy - cxHdColPad;
	}
	else
	{
		hi.cxy  = pNMH->pitem->cxy;
		cxLvCol = hi.cxy;
	}
	hi.mask = HDI_WIDTH;
	m_jListView.SetColumnWidth(pNMH->iItem, cxLvCol);
	m_jLvHeader.SetItem(pNMH->iItem, &hi);
	ResizeWindow(FALSE);
	m_jListView.InvalidateRect();
	m_jListView.UpdateWindow();
	m_jLvHeader.InvalidateRect();
	m_jLvHeader.UpdateWindow();

	// Continues (FALSE) tracking the divider or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has begun dragging by left mouse button in a list view

AFXINLINE void CTomPoiEdr::OnLvnBeginDrag(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CTomPoiEdr::OnLvnBeginDrag\n"));

	if(GetKeyState(VK_MBUTTON) & KS_DOWN)
		return;

	LVITEM lvi;
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED;
	m_jListView.SetItemState(&lvi);
	m_jListView.SetFocus();
	m_jListView.UpdateWindow();
	m_jListView.SetWindowRedraw(FALSE);
	m_jListView.SetCapture();
	m_jListView.m_hDC       = m_jListView.GetDC();
	m_jListView.m_iDropItem = -1;
	m_jListView.m_bDragOper = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Ends a scrolling operation in a list view

AFXINLINE void CTomPoiEdr::OnLvnEndScroll(PCNMLVSCROLL pNMLVS)
{
	TRACE(_T("Entering CTomPoiEdr::OnLvnEndScroll\n"));

	if(!pNMLVS->dx)
	{
		if(!pNMLVS->dy)
			return;

		if(!m_jLvEdit.IsWindowVisible())
			return;
	}
	PostMessage(WM_COMMAND, ID_RESIZE_WINDOW, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes of an item in a list view have changed

AFXINLINE void CTomPoiEdr::OnLvnItemChanged(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CTomPoiEdr::OnLvnItemChanged\n"));

	if(!(pNMLV->uNewState & LVIS_SELECTED))
		return;

	if(m_bMBtnDown)
	{
		m_bMBtnDown = FALSE;
		return;
	}

	if(m_bLRKeyDown)
	{
		m_bLRKeyDown = FALSE;
		return;
	}

	if(m_bMarquee)
		return;

	SetTimer(m_eGmSelTimerId, (GetKeyState(VK_LBUTTON) & KS_DOWN) ? 0 : m_eGmSelTimerIntl);
	m_bGmSelTimer = TRUE;
	m_bGmSelItem  = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Bounding box selection in a list view has begun

AFXINLINE BOOL CTomPoiEdr::OnLvnMarqueeBegin(PCNMLISTVIEW pNMLV)
{
	TRACE(_T("Entering CTomPoiEdr::OnLvnMarqueeBegin\n"));

	if(m_jLvEdit.IsWindowVisible())
	{
		m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
		if(!m_jLvEdit.m_bTextOK)
			return TRUE;
	}

	if(m_bMBtnDown)
		return TRUE;

	LVITEM lvi;
	lvi.state     = UNUSED;
	lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	m_jListView.SetItemState(&lvi);
	m_bMarquee = TRUE;

	// Accepts (FALSE) the bounding box selection or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has clicked (an item with) the left mouse button within a control

AFXINLINE BOOL CTomPoiEdr::OnNmClick(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnNmClick\n"));

	if(pNMH->hwndFrom != m_jListView)
	{
		if(pNMH->hwndFrom != m_jStatusBar)
			return FALSE;
		if(!m_jLvEdit.IsWindowVisible())
			return FALSE;
		m_jLvEdit.SendMessage(WM_CHAR, CHR_CR);
		return !m_jLvEdit.m_bTextOK;
	}

	if(m_bMarquee)
	{
		m_bMarquee = FALSE;
		return FALSE;
	}

	PCNMITEMACTIVATE pNMIA;
	pNMIA = (PCNMITEMACTIVATE)pNMH;
	if(pNMIA->iItem == -1)
	{
		LVITEM lvi;
		lvi.state     = UNUSED;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		m_jListView.SetItemState(&lvi);
		m_bMBtnDown = FALSE;
	}

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A control is about to be drawn

AFXINLINE UINT CTomPoiEdr::OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD)
{
	TRACE(_T("Entering CTomPoiEdr::OnNmCustomDraw\n"));

	switch(pNMCD->hdr.idFrom)
	{
	case IDW_LIST_VIEW:
		return CustomDrawListView((PNMLVCUSTOMDRAW)pNMCD);
	case IDW_TOOLBAR:
		return CustomDrawToolbar(pNMCD);
	}

	// Result value of the message processed
	return CDRF_DODEFAULT;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has double-clicked (an item with) the left mouse button within a control

AFXINLINE BOOL CTomPoiEdr::OnNmDblClk(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnNmDblClk\n"));

	if(pNMH->hwndFrom != m_jListView)
		return FALSE;

	int              iLvItem;
	PCNMITEMACTIVATE pNMIA;
	pNMIA   = (PCNMITEMACTIVATE)pNMH;
	iLvItem = pNMIA->iItem;
	if(iLvItem == -1)
		return FALSE;

	int iLvCol;
	iLvCol = pNMIA->iSubItem;
	if(iLvCol == LVC_DIST)
		return FALSE;

	if(m_bRPF && (iLvCol == LVC_MEMO))
		return FALSE;

	m_iLvCol = iLvCol;
	BeginLvEdit(iLvItem);

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has clicked (an item with) the right mouse button within a control

AFXINLINE BOOL CTomPoiEdr::OnNmRClick(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnNmRClick\n"));

	BOOL  bDelPopMenu;
	POINT pt;
	HMENU hPopMenu;
	GetCursorPos(&pt);
	bDelPopMenu = FALSE;
	if(pNMH->hwndFrom == m_jListView)
	{
		if(m_bMarquee)
			m_bMarquee = FALSE;
		else
		{
			PCNMITEMACTIVATE pNMIA;
			pNMIA = (PCNMITEMACTIVATE)pNMH;
			if(pNMIA->iItem == -1)
			{
				LVITEM lvi;
				lvi.state     = UNUSED;
				lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
				m_jListView.SetItemState(&lvi);
			}
		}
		hPopMenu = m_hLvPopMenu;
	}
	else if(pNMH->hwndFrom == m_jLvHeader)
	{
		HDHITTESTINFO hhti;
		SetPoint(&hhti.pt, pt.x, pt.y);
		m_jLvHeader.ScreenToClient(&hhti.pt);
		m_jLvHeader.HitTest(&hhti);
		m_iLvCol = hhti.iItem;
		hPopMenu = m_hHdPopMenu;
	}
	else if(pNMH->hwndFrom == m_jToolbar)
	{
		hPopMenu = m_hTbPopMenu;
		if(m_jOptionDlg.m_nWrapTb == BST_UNCHECKED)
		{
			int  cxToolbar;
			RECT rcChevron;
			m_jToolbar.GetWindowRect(&rcChevron);
			cxToolbar      = GetRectWidth(&rcChevron);
			rcChevron.left = rcChevron.right - m_ecxTbChevron;
			if(PtInRect(&rcChevron, pt))
			{
				int      iButton, iFirstBtn, iLastBtn;
				RECT     rcButton;
				TBBUTTON tbb;
				iLastBtn = MAX_INT;
				iButton  = m_jToolbar.ButtonCount();
				while(iButton)
				{
					iButton--;
					m_jToolbar.GetButton(iButton, &tbb);
					if(!tbb.idCommand)
						continue;
					m_jToolbar.GetItemRect(iButton, &rcButton);
					if(rcButton.right <= cxToolbar)
						break;
					if(iLastBtn == MAX_INT)
						iLastBtn = iButton;
					iFirstBtn = iButton;
				}
				if(iLastBtn != MAX_INT)
				{
					TCHAR        szText[64];
					BOOL         bSeparator;
					UINT         nCmdId;
					MENUITEMINFO mii;
					hPopMenu    = CreatePopupMenu();
					bDelPopMenu = TRUE;
					mii.cbSize  = sizeof(mii);
					if(e_dwWinVer > WV_XP)
						mii.fMask = MIIM_BITMAP;
					else
					{
						MENUINFO mi;
						mii.fMask  = MIIM_BITMAP | MIIM_DATA;
						mi.cbSize  = sizeof(mi);
						mi.fMask   = MIM_STYLE;
						mi.dwStyle = MNS_NOCHECK;
						SetMenuInfo(hPopMenu, &mi);
					}
					iButton = iFirstBtn;
					do
					{
						m_jToolbar.GetButton(iButton, &tbb);
						nCmdId = tbb.idCommand;
						if(nCmdId)
						{
							GetMenuString(m_hMainMenu, nCmdId,
								szText, countof(szText), MF_BYCOMMAND);
							RemoveMenuItemAmp(szText);
							AppendMenu(hPopMenu, (tbb.fsState & TBSTATE_ENABLED) ?
								MF_ENABLED : MF_GRAYED, nCmdId, szText);
							GetMenuItemInfo(m_hMainMenu, nCmdId, FALSE, &mii);
							SetMenuItemInfo(hPopMenu, nCmdId, FALSE, &mii);
							bSeparator = FALSE;
						}
						else if(!bSeparator)
						{
							AppendMenu(hPopMenu, MF_SEPARATOR);
							bSeparator = TRUE;
						}
						iButton++;
					}
					while(iButton <= iLastBtn);
				}
			}
		}
	}
	else
		return FALSE;

	TrackPopupMenu(hPopMenu, pt.x, pt.y, TPM_RIGHTBUTTON);
	if(bDelPopMenu)
		DestroyMenu(hPopMenu);

	// Allows (FALSE) the default processing or not (TRUE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Display mode of a status bar is changed

AFXINLINE void CTomPoiEdr::OnSbnSimpleModeChange(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnSbnSimpleModeChange\n"));

	if(m_jPoiBmp)
		m_jPoiBmp.ShowWindow(m_jStatusBar.IsSimple() ? SW_HIDE : SW_SHOW);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has begun customizing a toolbar

AFXINLINE void CTomPoiEdr::OnTbnBeginAdjust(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnBeginAdjust\n"));

	m_bCustTbBtn = TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A button in a toolbar is about to be deleted

AFXINLINE void CTomPoiEdr::OnTbnDeletingButton(PCNMTOOLBAR pNMTB)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnDeletingButton\n"));

	UINT nCmdId;
	nCmdId = pNMTB->iItem;
	if(nCmdId)
		m_tbb[m_jToolbar.FindButton(nCmdId)].fsState = m_jToolbar.GetState(nCmdId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Retrieves customization information about buttons in a toolbar

AFXINLINE BOOL CTomPoiEdr::OnTbnGetButtonInfo(PNMTOOLBAR pNMTB)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnGetButtonInfo\n"));

	int iButton;
	iButton = pNMTB->iItem;
	if(iButton >= m_eceTbImage)
		return FALSE;

	PTBBUTTON pTBB;
	pTBB = m_tbb;
	while(!pTBB->idCommand || (pTBB->iBitmap != iButton))
		pTBB++;
	pNMTB->tbButton = *pTBB;
	LoadString(e_hInstance, pTBB->idCommand - ID_FIRST_MENU_ITEM + IDS_INFOTIP_BASE,
		pNMTB->pszText, pNMTB->cchText);

	// Required information was copied into the structure (TRUE) or not (FALSE)
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Requests additional text information to display a tooltip in a toolbar

AFXINLINE void CTomPoiEdr::OnTbnGetInfoTip(PNMTBGETINFOTIP pNMTBGIT)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnGetInfoTip\n"));

	LoadString(e_hInstance, pNMTBGIT->iItem - ID_FIRST_MENU_ITEM + IDS_INFOTIP_BASE,
		pNMTBGIT->pszText, pNMTBGIT->cchTextMax);
	if(m_jLvEdit.IsWindowVisible())
		return;

	TCHAR szResText[MAX_STR];
	LoadString(e_hInstance, pNMTBGIT->iItem, szResText, countof(szResText));
	m_jStatusBar.SetWindowText(szResText);
	m_bSetReady = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// When customizing a toolbar has started

AFXINLINE int CTomPoiEdr::OnTbnInitCustomize(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnInitCustomize\n"));

	// Suppresses (TBNRF_HIDEHELP) the help button or not (0)
	return TBNRF_HIDEHELP;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Whether a button may be deleted while the user is customizing a toolbar

AFXINLINE BOOL CTomPoiEdr::OnTbnQueryDelete(PCNMTOOLBAR pNMTB)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnQueryDelete\n"));

	// Allows (TRUE) the button to be deleted or not (FALSE)
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Whether a button may be inserted while the user is customizing a toolbar

AFXINLINE BOOL CTomPoiEdr::OnTbnQueryInsert(PCNMTOOLBAR pNMTB)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnQueryInsert\n"));

	// Allows (TRUE) the button to be inserted or not (FALSE)
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has chosen the Reset button in the Customize Toolbar dialog box

AFXINLINE void CTomPoiEdr::OnTbnReset(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnReset\n"));

	int iButton;
	m_jToolbar.HideWindow();
	iButton = m_jToolbar.ButtonCount();
	while(iButton)
	{
		iButton--;
		m_jToolbar.DeleteButton(iButton);
	}
	m_jToolbar.AddButtons(m_tbb, countof(m_tbb));
	ResizeWindow(TRUE);
	m_jToolbar.ShowWindow();
	m_jCustTbDlg.m_jCloseBtn.EnableWindow();
	m_jCustTbDlg.m_jResetBtn.DisableWindow();
	WritePrivateProfileString(m_szOptionSection, m_szTbButtonEntry, NULL, e_pszProfile);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// The user has customized a toolbar

AFXINLINE void CTomPoiEdr::OnTbnToolbarChange(PCNMHDR pNMH)
{
	TRACE(_T("Entering CTomPoiEdr::OnTbnToolbarChange\n"));

	int ceButton;
	ceButton = m_jToolbar.ButtonCount();
	if(ceButton)
	{
		PTCHAR   pcNext, pcLast;
		TCHAR    szValue[MAX_STR];
		int      iButton;
		TBBUTTON tbb;
		iButton = 0;
		pcNext  = szValue;
		pcLast  = szValue + countof(szValue) - 3;
		do
		{
			m_jToolbar.GetButton(iButton, &tbb);
			pcNext += wsprintf(pcNext, _T("%d "), tbb.idCommand ? tbb.iBitmap : m_eceTbImage);
			if(pcNext >= pcLast)
				break;
			iButton++;
		}
		while(iButton < ceButton);
		*(pcNext - 1) = CHR_NULL;
		WritePrivateProfileString(m_szOptionSection, m_szTbButtonEntry, szValue, e_pszProfile);
	}
	ResizeWindow(TRUE);
	m_jCustTbDlg.EnableCloseButton();
	m_jCustTbDlg.m_jResetBtn.EnableWindow();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Paints the client area of a window

AFXINLINE void CTomPoiEdr::OnPaint()
{
	TRACE(_T("Entering CTomPoiEdr::OnPaint\n"));

	int         iTemp, cxListView;
	COLORREF    crTbBorder;
	RECT        rc;
	PAINTSTRUCT ps;
	HPEN        hPen;
	HDC         hDC;
	iTemp = m_cyToolbar;
	if(e_dwWinVer > WV_XP)
	{
		iTemp     += m_eyTbTop;
		crTbBorder = m_ecrTbBorder;
	}
	else
	{
		iTemp     += m_eyTbTopXP;
		crTbBorder = m_ecrTbBorderXP;
	}
	hDC  = BeginPaint(&ps);
	hPen = CreatePen(PS_SOLID, 1, crTbBorder);
	SelectPen(hDC, hPen);
	MoveTo(hDC, 0, iTemp);
	LineTo(hDC, m_cxClient, iTemp);
	SelectPen(hDC, GetStockPen(WHITE_PEN));
	if(m_jIEPane.IsWindowVisible())
	{
		m_jIEPane.GetWindowRect(&rc);
		::MapWindowPoints(*this, &rc);
		cxListView = rc.left - m_ecxIEPaneSep;
	}
	else
		cxListView = m_cxClient;
	iTemp++;
	MoveTo(hDC, 0, iTemp);
	LineTo(hDC, cxListView, iTemp);
	if(m_jIEPane.IsWindowVisible())
	{
		HBRUSH hBrush;
		DeletePen(hPen);
		hPen = CreatePen(PS_SOLID, 1, m_ecrIEPaneSepBorder);
		SelectPen(hDC, hPen);
		rc.left--;
		MoveTo(hDC, rc.left, rc.top);
		LineTo(hDC, rc.left, rc.bottom);
		iTemp = rc.left - m_ecxIEPaneSep + 1;
		MoveTo(hDC, iTemp, rc.top);
		LineTo(hDC, iTemp, rc.bottom);
		SetRect(&rc, iTemp + 1, rc.top, rc.left, rc.bottom);
		hBrush = CreateSolidBrush((m_bAdjIEPane || PtInIEPaneSep()) ?
			m_ecrIEPaneSepHilite : m_ecrIEPaneSep);
		FillRect(hDC, &rc, hBrush);
		DeleteBrush(hBrush);
	}
	EndPaint(&ps);
	DeletePen(hPen);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Mouse cursor shape should be set

AFXINLINE BOOL CTomPoiEdr::OnSetCursor(HWND hCursorWnd, int iHitTest, UINT nMsg)
{
	TRACE(_T("Entering CTomPoiEdr::OnSetCursor\n"));

	if(PtInIEPaneSep())
	{
		SetCursor(LoadCursor(e_hInstance, IDC_VERT_SPLIT));
		return TRUE;
	}

	// Cursor shape is set (TRUE) or not (FALSE)
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window has gained the keyboard focus

AFXINLINE void CTomPoiEdr::OnSetFocus(HWND hOldWnd)
{
	TRACE(_T("Entering CTomPoiEdr::OnSetFocus\n"));

	CWnd jFocusWnd;
	if(m_jLvEdit.IsWindowVisible())
	{
		if(m_jLvCombo.IsWindowVisible())
			jFocusWnd = m_jLvCombo;
		else
			jFocusWnd = m_jLvEdit;
	}
	else
		jFocusWnd = m_jListView;
	jFocusWnd.SetFocus();
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Window size has changed

AFXINLINE void CTomPoiEdr::OnSize(UINT nType, int cxClient, int cyClient)
{
	TRACE(_T("Entering CTomPoiEdr::OnSize\n"));

	if(nType == SIZE_MINIMIZED)
		return;

	// Client area of window
	m_cxClient = cxClient;
	m_cyClient = cyClient;

	// Resizes child windows
	ResizeWindow(TRUE);

	// Updates the contents
	m_bMoveBmp = TRUE;
	PostMessage(WM_TIMER, m_eMainTimerId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A system color is changed

AFXINLINE void CTomPoiEdr::OnSysColorChange()
{
	TRACE(_T("Entering CTomPoiEdr::OnSysColorChange\n"));

	m_jListView.PostMessage(WM_SYSCOLORCHANGE);
	m_jListView.SetTextBkColor(m_bLvBkgnd ? CLR_NONE : GetSysColor(COLOR_WINDOW));
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A command item from the system menu

AFXINLINE void CTomPoiEdr::OnSysCommand(UINT nItemId, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnSysCommand\n"));

	if(nItemId < SC_SIZE)
		PostMessage(WM_COMMAND, nItemId);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// A timer interval expires

AFXINLINE void CTomPoiEdr::OnTimer(UINT nTimerId)
{
	TRACE(_T("Entering CTomPoiEdr::OnTimer\n"));

	// Exits the application when IE pane is closed completely
	if(nTimerId == m_eExitTimerId)
	{
		KillTimer(m_eExitTimerId);
		DestroyWindow();
		return;
	}

	// Paints the IE pane separator if mouse moves out of the client area
	if(nTimerId == m_eIEPaneTimerId)
	{
		if(PtInIEPaneSep())
			return;
		KillTimer(m_eIEPaneTimerId);
		InvalidateRect();
		m_bSetReady = TRUE;
	}

	// Sets the window caption
	TCHAR szText[MAX_FNAME + MAX_STR];
	if(m_bSetCaption)
	{
		SCTCHAR szItemSep[] = { CHR_SP, CHR_DSH, CHR_SP };
		PTCHAR  pcCaption;
		TCHAR   szCaption[countof(szText)];
		GetWindowText(szText, countof(szText));
		pcCaption = szCaption + TomFile2WinFile(m_pszFileTitle, szCaption);
		if(m_bFileDirty)
		{
			*pcCaption = CHR_AST;
			pcCaption++;
		}
		CopyMemory(pcCaption, szItemSep, sizeof(szItemSep));
		lstrcpy(pcCaption + countof(szItemSep), e_pszTpeAppName);
		if(lstrcmp(szText, szCaption))
			SetWindowText(szCaption);
		m_bSetCaption = FALSE;
	}

	// Backups the file
	if((nTimerId == m_eBackupTimerId) && !m_bBackup)
	{
		if(m_bFileDirty && m_jListView.GetItemCount())
		{
			m_bBackup = TRUE;
			SendMessage(WM_COMMAND, ID_FILE_SAVE);
			m_bBackup = FALSE;
		}
	}

	// Gets/puts command from/into Google Maps
	int i;
	GetGoogleMaps();
	if(nTimerId == m_eGmSelTimerId)
	{
		KillTimer(m_eGmSelTimerId);
		m_bGmSelTimer = FALSE;
	}
	if(m_bGmSelItem && !m_bGmSelTimer)
	{
		i = m_jListView.GetNextItem(LVNI_SELECTED);
		if(i != RV_ERR)
			PutGoogleMaps(m_eGmPutSelect, i);
		m_bGmSelItem = FALSE;
	}

	// Never update parts of status bar when it's in simple mode
	if(m_jStatusBar.IsSimple())
		return;

	// Updates states of toolbar buttons
	if(m_cyToolbar)
		UpdateTbButton();

	// Never update parts of status bar when it's hidden
	if(!m_cyStatusBar)
		return;

	// Determines whether the edit control is visible
	int iCurPos;
	if(m_jLvEdit.IsWindowVisible())
	{
		// Editing item
		iCurPos = m_jLvEdit.m_iLvItem;
	}
	else
	{
		// Updates the info part to Ready
		if(m_bSetReady)
		{
			TCHAR szReady[MAX_STR];
			m_jStatusBar.GetWindowText(szText, countof(szText));
			TrimString(szText);
			LoadString(e_hInstance, IDP_INFO_PART, szReady, countof(szReady));
			if(lstrcmp(szText, szReady))
				m_jStatusBar.SetWindowText(szReady);
			m_bSetReady = FALSE;
		}

		// First selected item
		iCurPos = m_jListView.GetNextItem(LVNI_SELECTED);
	}

	// Updates the POI count
	SINT  siCurPos = -1, sceSelItem = -1, sceLvItem = -1;
	TCHAR szPart[64];
	int   ceSelItem, ceLvItem, ccPart, iRightPos[m_eceSbPart];
	SIZE  size;
	ceSelItem = m_jListView.GetSelCount();
	ceLvItem  = m_jListView.GetItemCount();
	if((siCurPos != iCurPos) || (sceSelItem != ceSelItem) || (sceLvItem != ceLvItem))
	{
		SINT  scxPoiPart = MAX_INT;
		TCHAR szInfoPart[64];
		siCurPos   = iCurPos;
		sceSelItem = ceSelItem;
		sceLvItem  = ceLvItem;
		LoadString(e_hInstance, IDS_POI_PART, szInfoPart, countof(szInfoPart));
		ccPart = wsprintf(szPart, _T("  %s  %d - %d / %d  "),
			szInfoPart, iCurPos + 1, ceSelItem, ceLvItem);
		m_jStatusBar.SetText(m_ePoiPart, szPart);
		GetTextExtent(m_hSbDC, szPart, ccPart, &size);
		m_cxPoiPart = size.cx + m_cxyPartBorder * 5;
		if(e_dwWinVer > WV_XP)
			m_cxPoiPart--;
		if(scxPoiPart != m_cxPoiPart)
		{
			scxPoiPart             = m_cxPoiPart;
			iRightPos[m_eTimePart] = -1;
			iRightPos[m_ePoiPart]  = m_cxClient - m_cxTimePart;
			iRightPos[m_eBmpPart]  = iRightPos[m_ePoiPart] - m_cxPoiPart;
			iRightPos[m_eInfoPart] = iRightPos[m_eBmpPart] - m_cxBmpPart;
			m_jStatusBar.SetParts(iRightPos, countof(iRightPos));
			m_bMoveBmp = TRUE;
		}
	}

	// Updates the current time
	SWORD      swHour = MAX_WORD;
	SINT       scxColon, scxSpace;
	TCHAR      cAmPm;
	SYSTEMTIME st;
	GetLocalTime(&st);
	if(st.wHour >= 12)
	{
		st.wHour -= 12;
		cAmPm     = _T('P');
	}
	else
		cAmPm = _T('A');
	if(!st.wHour)
		st.wHour = 12;
	if(!scxColon)
	{
		TCHAR cTimeSep;
		cTimeSep = CHR_CLN;
		GetTextExtent(m_hSbDC, &cTimeSep, 1, &size);
		scxColon = size.cx;
		cTimeSep = CHR_SP;
		GetTextExtent(m_hSbDC, &cTimeSep, 1, &size);
		scxSpace = size.cx;
	}
	ccPart = wsprintf(szPart, _T("  %cM %u%c%.2u  "), cAmPm, st.wHour,
		((st.wMilliseconds < 500) && (scxColon == scxSpace)) ? CHR_SP : CHR_CLN, st.wMinute);
	m_jStatusBar.SetText(m_eTimePart, szPart);
	if(swHour != st.wHour)
	{
		SINT scxTimePart = MAX_INT;
		swHour = st.wHour;
		GetTextExtent(m_hSbDC, szPart, ccPart, &size);
		m_cxTimePart = size.cx + m_cxyPartBorder * 5;
		if(e_dwWinVer > WV_XP)
			m_cxTimePart--;
		if(scxTimePart != m_cxTimePart)
		{
			scxTimePart            = m_cxTimePart;
			iRightPos[m_eTimePart] = -1;
			iRightPos[m_ePoiPart]  = m_cxClient - m_cxTimePart;
			iRightPos[m_eBmpPart]  = iRightPos[m_ePoiPart] - m_cxPoiPart;
			iRightPos[m_eInfoPart] = iRightPos[m_eBmpPart] - m_cxBmpPart;
			m_jStatusBar.SetParts(iRightPos, countof(iRightPos));
			m_bMoveBmp = TRUE;
		}
	}

	// Updates the POI bitmap
	SINT scxPoiBmp, scyPoiBmp;
	if(m_bSetBmp)
	{
		int  cyMinBmp;
		UINT nPoiBmpId;
		m_bSetBmp = FALSE;
		nPoiBmpId = UNUSED;
		if(m_hPoiBmp)
		{
			DeleteBitmap(m_hPoiBmp);
			m_hPoiBmp = NULL;
		}
		if(*m_jFileDlg.lpstrFile)
		{
			if(m_fst == FST_GENERAL)
			{
				PCTSTR pszFileExt;
				pszFileExt = PathFindExtension(m_jFileDlg.lpstrFile);
				if(*pszFileExt)
				{
					pszFileExt++;
					if(!lstrcmpi(pszFileExt, m_szOv2Ext))
					{
						i = pszFileExt - m_jFileDlg.lpstrFile;
						CopyMemory(szText, m_jFileDlg.lpstrFile, i * sizeof(TCHAR));
						CopyMemory(szText + i, m_szBmpExt, sizeof(m_szBmpExt));
						if(IsValidFile(szText))
							m_hPoiBmp = LoadBitmap(szText);
						if(!m_hPoiBmp)
						{
							LoadString(e_hInstance, IDS_FAVORITES_NAME, szText, MAX_STR);
							nPoiBmpId = lstrcmpi(szText, m_jFileDlg.lpstrFileTitle) ?
								IDB_NO_POI_ICON : IDB_FAVORITES;
						}
					}
				}
			}
			else if(m_fst == FST_SC)
				nPoiBmpId = IDB_SAFETY_CAMERA;
			else
			{
				ASSERT(m_fst == FST_CPB);
				nPoiBmpId = IDB_PHONEBOOK;
			}
		}
		else
		{
			LoadString(e_hInstance, IDS_UNTITLED_NAME, szText, MAX_STR);
			if(lstrcmpi(szText, m_jFileDlg.lpstrFileTitle))
			{
				LoadString(e_hInstance, IDS_FAVORITES_NAME, szText, MAX_STR);
				if(lstrcmpi(szText, m_jFileDlg.lpstrFileTitle))
				{
					if(m_fst == FST_GENERAL)
					{
						TCHAR szFileExt[8];
						lstrcpy(szText, m_pszGmTempFile);
						szFileExt[0] = CHR_DOT;
						CopyMemory(szFileExt + 1, m_szBmpExt, sizeof(m_szBmpExt));
						PathRenameExtension(szText, szFileExt);
						if(IsValidFile(szText))
						{
							m_hPoiBmp = LoadBitmap(szText);
							DeleteFile(szText);
						}
						if(!m_hPoiBmp)
							nPoiBmpId = IDB_NO_POI_ICON;
					}
					else
					{
						ASSERT(m_fst == FST_SC);
						nPoiBmpId = IDB_SAFETY_CAMERA;
					}
				}
				else
					nPoiBmpId = IDB_FAVORITES;
			}
		}
		if(nPoiBmpId)
			m_hPoiBmp = (HBITMAP)LoadImage(e_hInstance, MAKERESPTR(nPoiBmpId),
				IMAGE_BITMAP, LR_CREATEDIBSECTION);
		if(m_hPoiBmp)
		{
			BITMAP bm;
			if(m_jPoiBmp)
				m_jPoiBmp.HideWindow();
			else
				m_jPoiBmp.CreateWindow(WS_CHILD | SS_BITMAP | SS_REALSIZECONTROL,
					0, 0, 0, 0, m_jStatusBar, IDW_POI_BITMAP);
			m_jPoiBmp.SetBitmap(m_hPoiBmp);
			GetObject(m_hPoiBmp, &bm);
			scyPoiBmp   = min(bm.bmHeight, m_ecyMaxBmp);
			scxPoiBmp   = bm.bmWidth * scyPoiBmp / bm.bmHeight;
			m_cxBmpPart = scxPoiBmp + m_cxyPartBorder * 5;
			if(e_dwWinVer > WV_XP)
				m_cxBmpPart--;
			cyMinBmp   = scyPoiBmp + m_cxyPartBorder * 2;
			m_bMoveBmp = TRUE;
		}
		else
		{
			if(m_jPoiBmp)
			{
				m_jPoiBmp.DestroyWindow();
				m_jPoiBmp = NULL;
			}
			m_cxBmpPart = 0;
			cyMinBmp    = 0;
		}
		m_jStatusBar.SetMinHeight(cyMinBmp);
		PostMessage(WM_COMMAND, ID_RESIZE_WINDOW, TRUE);
		return;
	}

	if(!m_bMoveBmp)
		return;

	if(m_jPoiBmp)
	{
		m_jStatusBar.GetParts(iRightPos, countof(iRightPos));
		m_jPoiBmp.MoveWindow(iRightPos[m_eInfoPart] + m_cxyPartBorder * 2,
			(m_cyStatusBar - scyPoiBmp) / 2 + 1, scxPoiBmp, scyPoiBmp);
		m_jPoiBmp.ShowWindow();
	}
	m_bMoveBmp = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Taskbar icon notification

AFXINLINE LRESULT CTomPoiEdr::OnTaskbarIcon(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnTaskbarIcon\n"));

	if((lParam != NIN_BALLOONUSERCLICK) && (lParam != NIN_BALLOONTIMEOUT))
		return UNUSED;

	NOTIFYICONDATA nid;
	nid.cbSize = sizeof(nid);
	nid.hWnd   = *this;
	nid.uID    = IDR_MAIN;
	Shell_NotifyIcon(NIM_DELETE, &nid);
	if(lParam == NIN_BALLOONUSERCLICK)
		PostMessage(WM_COMMAND, ID_HELP_TPE_HOME);
	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Queries to receive POIs

AFXINLINE LRESULT CTomPoiEdr::OnReceivePoi(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnReceivePoi\n"));

	if(wParam)
		return (m_ceDragDrop ? m_eRecvPoiBusy :
			((m_fst == FST_GENERAL) ? m_eRecvPoiOK : m_eRecvPoiError));

	TCHAR  szText[e_eccNameCol];
	PCWSTR pszPoiData;
	UINT   nResId;
	HFILE  hFile;
	SetForegroundWindow();
	SetCursor(LoadCursor(IDC_WAIT));
	ShowCursor();
	LoadString(e_hInstance, IDP_RECEIVING_POI, szText, countof(szText));
	m_jStatusBar.SetWindowText(szText);
	nResId        = IDS_MEMORY_ERROR_RECEIVE_POI;
	m_cePoiDupDel = 0;
	hFile         = OpenFileMapping(FILE_MAP_READ, FALSE, e_pszTieAppName);
	if(hFile)
	{
		pszPoiData = (PCWSTR)MapViewOfFile(hFile, FILE_MAP_READ);
		if(pszPoiData)
		{
			PCWSTR pszSrc;
			int    i, ccText, iFirstItem, iLvItem, ceLvItem;
			LVITEM lvi;
			m_jListView.SetWindowRedraw(FALSE);
			m_jListView.SetAction(AT_NEW);
			m_nPoiDupSel = IDC_RETAIN;
			ceLvItem     = m_jListView.GetItemCount();
			iFirstItem   = m_jListView.GetNextItem(LVNI_SELECTED);
			if(iFirstItem == RV_ERR)
				iFirstItem = ceLvItem;
			else if(!iFirstItem && m_bMyBP)
				iFirstItem = 1;
			iLvItem       = iFirstItem;
			lvi.mask      = LVIF_STATE | LVIF_TEXT;
			lvi.state     = UNUSED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			m_jListView.SetItemState(&lvi);
			lvi.state = LVIS_FOCUSED | LVIS_SELECTED;
			pszSrc    = pszPoiData;
			do
			{
				ccText = lstrlenW(pszSrc) + 1;
#ifdef UNICODE
				CopyMemory(szText, pszSrc, ccText * sizeof(WCHAR));
#else
				WCStoMBS(pszSrc, ccText, szText, sizeof(szText));
#endif
				i = LVC_LNG;
				if((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(szText))
				{
					pszSrc += ccText;
					do
					{
						while(*pszSrc)
							pszSrc++;
						pszSrc++;
						i++;
					}
					while(i <= LVC_LAT);
					continue;
				}
				lvi.iItem    = iLvItem;
				lvi.iSubItem = 0;
				lvi.pszText  = szText;
				iLvItem      = m_jListView.InsertItem(&lvi);
				do
				{
					pszSrc += ccText;
					ccText  = lstrlenW(pszSrc) + 1;
					if(ccText > 1)
					{
#ifdef UNICODE
						CopyMemory(szText, pszSrc, ccText * sizeof(WCHAR));
#else
						WCStoMBS(pszSrc, ccText, szText, sizeof(szText));
#endif
						ValidateCoordCol(iLvItem, i, szText, FALSE);
					}
					i++;
				}
				while(i <= LVC_LAT);
				iLvItem++;
				pszSrc += ccText;
			}
			while(*pszSrc);
			SetBasePoint(LVI_FIND, FALSE);
			AutoSizeLvCol(LVC_ALL, FALSE);
			m_jListView.SetWindowRedraw();
			if(ceLvItem != m_jListView.GetItemCount())
			{
				EnsureLvItemVisible(iFirstItem);
				m_bFileDirty  = TRUE;
				m_bSetCaption = TRUE;
			}
			nResId = m_cePoiDupDel ? IDS_POI_DUP_DEL : 0;
			UnmapViewOfFile(pszPoiData);
		}
		CloseHandle(hFile);
	}
	if(nResId)
		DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, m_cePoiDupDel);
	HideCursor();
	SetCursor(LoadCursor(IDC_ARROW));
	m_bSetReady = TRUE;
	return m_eRecvPoiUnused;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Queries to drag and drop POI data

AFXINLINE LRESULT CTomPoiEdr::OnDragDrop(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnDragDrop\n"));

	// Drags over
	if(wParam == CLvCtrl::DS_OVER)
	{
		if(m_ceDragDrop || (m_fst != FST_GENERAL))
			return CLvCtrl::DE_REJECT;

		if(!m_jListView.m_bDragOper)
		{
			CWnd jDragWnd;
			jDragWnd.GetForegroundWindow();
			if(jDragWnd)
				SetWindowPos(jDragWnd, UNUSED, UNUSED, UNUSED, UNUSED,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);

			MSG msg;
			if(m_jLvEdit.IsWindowVisible())
				m_jLvEdit.SendMessage(WM_CHAR, CHR_ESC);
			PeekMessage(&msg, WM_COMMAND, PM_REMOVE);

			LVITEM lvi;
			lvi.state     = UNUSED;
			lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
			m_jListView.SetItemState(&lvi);
			m_jListView.UpdateWindow();
			m_jListView.SetWindowRedraw(FALSE);
			m_jListView.m_hDC       = m_jListView.GetDC();
			m_jListView.m_iDropItem = -1;
			m_jListView.m_bDragOper = TRUE;
		}

		POINT pt;
		GetCursorPos(&pt);
		m_jListView.ScreenToClient(&pt);
		m_jListView.SendMessage(WM_MOUSEMOVE, MK_LBUTTON, PACKPOINT(pt));
		return ((m_jListView.m_do != CLvCtrl::DO_NO) ? CLvCtrl::DE_ACCEPT : CLvCtrl::DE_REJECT);
	}

	// Drops data
	if(wParam == CLvCtrl::DS_DROP)
	{
		TCHAR  szText[e_eccNameCol];
		PCWSTR pszPoiData;
		UINT   nResId;
		HFILE  hFile;
		SetForegroundWindow();
		SetCursor(LoadCursor(IDC_WAIT));
		m_jListView.PaintDropMark(m_jListView.m_iDropItem);
		LoadString(e_hInstance, IDP_RECEIVING_POI, szText, countof(szText));
		m_jStatusBar.SetWindowText(szText);
		nResId        = IDS_MEMORY_ERROR_RECEIVE_POI;
		m_cePoiDupDel = 0;
		hFile         = OpenFileMapping(FILE_MAP_READ, FALSE,
			lParam ? e_pszTpeAppName : e_pszTieAppName);
		if(hFile)
		{
			pszPoiData = (PCWSTR)MapViewOfFile(hFile, FILE_MAP_READ);
			if(pszPoiData)
			{
				PCWSTR pszSrc;
				int    i, ccText, iFirstItem, iLvItem, ceLvItem, iLastCol;
				LVITEM lvi;
				m_jListView.SetAction(AT_NEW);
				m_nPoiDupSel  = IDC_RETAIN;
				ceLvItem      = m_jListView.GetItemCount();
				iFirstItem    = max(0, m_jListView.m_iDropItem);
				iLvItem       = iFirstItem;
				lvi.mask      = LVIF_STATE | LVIF_TEXT;
				lvi.state     = LVIS_FOCUSED | LVIS_SELECTED;
				lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
				iLastCol      = lParam ? LVC_MEMO : LVC_LAT;
				pszSrc        = pszPoiData;
				do
				{
					if(!lParam)
						pszSrc++;
					ccText = lstrlenW(pszSrc) + 1;
#ifdef UNICODE
					CopyMemory(szText, pszSrc, ccText * sizeof(WCHAR));
#else
					WCStoMBS(pszSrc, ccText, szText, sizeof(szText));
#endif
					i = LVC_LNG;
					if((m_jOptionDlg.m_nCheckName == BST_CHECKED) && FindPoiName(szText))
					{
						pszSrc += ccText;
						do
						{
							while(*pszSrc)
								pszSrc++;
							pszSrc++;
							i++;
						}
						while(i <= iLastCol);
						continue;
					}
					lvi.iItem    = iLvItem;
					lvi.iSubItem = 0;
					lvi.pszText  = szText;
					iLvItem      = m_jListView.InsertItem(&lvi);
					do
					{
						pszSrc += ccText;
						ccText  = lstrlenW(pszSrc) + 1;
						if(ccText > 1)
						{
#ifdef UNICODE
							CopyMemory(szText, pszSrc, ccText * sizeof(WCHAR));
#else
							WCStoMBS(pszSrc, ccText, szText, sizeof(szText));
#endif
							if((i == LVC_LNG) || (i == LVC_LAT))
								ValidateCoordCol(iLvItem, i, szText, FALSE);
							else if(i == LVC_TEL)
								ValidateTelCol(iLvItem, szText);
							else if(!m_bRPF)
								ValidateMemoCol(iLvItem, szText);
						}
						i++;
					}
					while(i <= iLastCol);
					iLvItem++;
					pszSrc += ccText;
				}
				while(*pszSrc);
				SetBasePoint(LVI_FIND, FALSE);
				AutoSizeLvCol(LVC_ALL, FALSE);
				if(ceLvItem != m_jListView.GetItemCount())
				{
					EnsureLvItemVisible(iFirstItem);
					m_bFileDirty  = TRUE;
					m_bSetCaption = TRUE;
				}
				nResId = m_cePoiDupDel ? IDS_POI_DUP_DEL : 0;
				UnmapViewOfFile(pszPoiData);
			}
			CloseHandle(hFile);
		}
		if(nResId)
			DisplayMessage(nResId, MB_ICONINFORMATION | MB_OK, m_cePoiDupDel);
		m_jListView.m_iDropItem = -1;
		m_bSetReady             = TRUE;
		return CLvCtrl::DE_UNUSED;
	}

	// Cancels operation
	ASSERT(wParam == CLvCtrl::DS_OUT);
	m_jListView.m_do = CLvCtrl::DO_NO;
	m_jListView.PostMessage(WM_LBUTTONUP);
	return CLvCtrl::DE_UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// GPS device has been changed

AFXINLINE LRESULT CTomPoiEdr::OnGpsChange(WPARAM wParam, LPARAM lParam)
{
	TRACE(_T("Entering CTomPoiEdr::OnGpsChange\n"));

	BOOL bPrevGpsDevice;
	bPrevGpsDevice = m_bGpsDevice;
	m_bGpsDevice   = wParam;
	if(wParam)
		GpsVerifyDump(NULL);
	GpsCloseDialog(bPrevGpsDevice);

	return UNUSED;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Message maps for CTomPoiEdr

BEGIN_MESSAGE_MAP(CTomPoiEdr)
	ON_WM_ACTIVATE(OnActivate)
	BEGIN_COMMAND_MAP()
		ON_COMMAND(ID_FILE_NEW, OnFileNew)
		ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
		ON_COMMAND(ID_FILE_OPEN_FAV, OnFileOpenFav)
		ON_COMMAND(ID_FILE_OPEN_DEVICE, OnFileOpenDevice)
		ON_COMMAND(ID_FILE_DUMP_DEVICE, OnFileDumpDevice)
		ON_COMMAND(ID_FILE_SAVE, OnFileSave)
		ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
		ON_COMMAND(ID_FILE_SAVE_DEVICE, OnFileSaveDevice)
		ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
		ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
		ON_COMMAND(ID_FILE_QUERY, OnFileQuery)
		ON_COMMAND(ID_FILE_TRANSLATE, OnFileTranslate)
		ON_COMMAND(ID_FILE_CONVERT, OnFileConvert)
		ON_COMMAND(ID_FILE_EXIT, OnFileExit)
		ON_COMMAND(ID_POI_UNDO, OnPoiUndo)
		ON_COMMAND(ID_POI_REDO, OnPoiRedo)
		ON_COMMAND(ID_POI_ADD, OnPoiAdd)
		ON_COMMAND(ID_POI_INSERT, OnPoiInsert)
		ON_COMMAND(ID_POI_COMBINE_COL, OnPoiCombineCol)
		ON_COMMAND(ID_POI_EDIT, OnPoiEdit)
		ON_COMMAND(ID_POI_DELETE, OnPoiDelete)
		ON_COMMAND(ID_POI_CLEAR_COL, OnPoiClearCol)
		ON_COMMAND(ID_POI_CUT, OnPoiCut)
		ON_COMMAND(ID_POI_COPY, OnPoiCopy)
		ON_COMMAND(ID_POI_PASTE, OnPoiPaste)
		ON_COMMAND(ID_POI_COPY_COORD, OnPoiCopyCoord)
		ON_COMMAND(ID_POI_PASTE_COORD, OnPoiPasteCoord)
		ON_COMMAND(ID_POI_QUERY_COORD, OnPoiQueryCoord)
		ON_COMMAND(ID_POI_QUERY_ADDR, OnPoiQueryAddr)
		ON_COMMAND(ID_POI_FIND, OnPoiFind)
		ON_COMMAND(ID_POI_FIND_NEXT, OnPoiFindNext)
		ON_COMMAND(ID_POI_REPLACE, OnPoiReplace)
		ON_COMMAND(ID_POI_REPLACE_NEXT, OnPoiReplaceNext)
		ON_COMMAND(ID_POI_MESS_COORD, OnPoiMessCoord)
		ON_COMMAND(ID_POI_CHECK, OnPoiCheck)
		ON_COMMAND(ID_POI_ADD_FAV, OnPoiAddFav)
		ON_COMMAND(ID_POI_ADD_ITN, OnPoiAddItn)
		ON_COMMAND(ID_POI_SEL_ALL, OnPoiSelAll)
		ON_COMMAND(ID_POI_TOGGLE_SEL, OnPoiToggleSel)
		ON_COMMAND(ID_POS_UP, OnPosUp)
		ON_COMMAND(ID_POS_DOWN, OnPosDown)
		ON_COMMAND(ID_POS_SET_BP, OnPosSetBP)
		ON_COMMAND(ID_POS_MY_BP, OnPosMyBP)
		ON_COMMAND(ID_POS_SORT_ASCEND, OnPosSortAscend)
		ON_COMMAND(ID_POS_SORT_DESCEND, OnPosSortDescend)
		ON_COMMAND(ID_POS_GOOGLE_MAP, OnPosGoogleMap)
		ON_COMMAND(ID_POS_GOOGLE_EARTH, OnPosGoogleEarth)
		ON_COMMAND(ID_TOOLS_QGPSFIX, OnToolsQGPSfix)
		ON_COMMAND(ID_TOOLS_SHOWMB, OnToolsShowMb)
		ON_COMMAND(ID_TOOLS_SHOWTB, OnToolsShowTb)
		ON_COMMAND(ID_TOOLS_LARGETB, OnToolsLargeTb)
		ON_COMMAND(ID_TOOLS_WRAPTB, OnToolsWrapTb)
		ON_COMMAND(ID_TOOLS_TBBKGND, OnToolsTbBkgnd)
		ON_COMMAND(ID_TOOLS_CUSTTB, OnToolsCustTb)
		ON_COMMAND(ID_TOOLS_LVFONT, OnToolsLvFont)
		ON_COMMAND(ID_TOOLS_LVBKGND, OnToolsLvBkgnd)
		ON_COMMAND(ID_TOOLS_SHOWSB, OnToolsShowSb)
		ON_COMMAND(ID_TOOLS_OPTION, OnToolsOption)
		ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
		ON_COMMAND(ID_OPEN_CMDLINE, OnOpenCmdLine)
		ON_COMMAND_PARAM(ID_RESIZE_WINDOW, OnResizeWindow)
		ON_COMMAND_PARAM(ID_SELECT_LVITEM, OnSelectLvItem)
		ON_COMMAND(ID_LNG_FIRST, OnLngFirst)
		ON_COMMAND(ID_DMS_FORMAT, OnDmsFormat)
		ON_COMMAND(ID_COMPASS_DIR, OnCompassDir)
		ON_COMMAND(ID_AUTOSIZE_COL, OnAutoSizeCol)
		ON_COMMAND(ID_CLEAR_COL, OnClearCol)
		ON_COMMAND(ID_CLEAR_ADDR, OnClearAddr)
		ON_COMMAND(ID_SHOW_DISTANCE, OnShowDistance)
		ON_COMMAND(ID_METRIC_UNIT, OnMetricUnit)
		ON_COMMAND(ID_ADJ_DISTANCE, OnAdjDistance)
		ON_COMMAND(ID_BASE_DISTANCE, OnBaseDistance)
		ON_COMMAND(ID_SORT_ASCEND, OnSortAscend)
		ON_COMMAND(ID_SORT_DESCEND, OnSortDescend)
		ON_COMMAND(ID_APP_KEY_DOWN, OnAppKeyDown)
		ON_COMMAND_RANGE(ID_HELP_TOM_HOME, ID_HELP_MAILTO, OnHelpMisc)
		ON_COMMAND_RANGE(ID_RECENT_FILE1, ID_RECENT_FILE9, OnRecentFile)
	END_COMMAND_MAP()
	ON_WM_CLOSE(OnClose)
	ON_WM_CREATE(OnCreate)
	ON_WM_DESTROY(OnDestroy)
	ON_WM_DEVICECHANGE(OnDeviceChange)
	ON_WM_DRAWITEM(OnDrawItem)
	ON_WM_DROPFILES(OnDropFiles)
	ON_WM_GETMINMAXINFO(OnGetMinMaxInfo)
	ON_WM_INITMENU(OnInitMenu)
	ON_WM_LBUTTONDBLCLK(OnLButtonDblClk)
	ON_WM_LBUTTONDOWN(OnLButtonDown)
	ON_WM_LBUTTONUP(OnLButtonUp)
	ON_WM_MEASUREITEM(OnMeasureItem)
	ON_WM_MENUSELECT(OnMenuSelect)
	ON_WM_MOUSEMOVE(OnMouseMove)
	BEGIN_NOTIFY_MAP()
		ON_HDN_DIVIDERDBLCLICK(OnHdnDividerDblClick)
		ON_HDN_ENDTRACK(OnHdnEndTrack)
		ON_HDN_ITEMCLICK(OnHdnItemClick)
		ON_HDN_TRACK(OnHdnTrack)
		ON_LVN_BEGINDRAG(OnLvnBeginDrag)
		ON_LVN_ENDSCROLL(OnLvnEndScroll)
		ON_LVN_ITEMCHANGED(OnLvnItemChanged)
		ON_LVN_MARQUEEBEGIN(OnLvnMarqueeBegin)
		ON_NM_CLICK(OnNmClick)
		ON_NM_CUSTOMDRAW(OnNmCustomDraw)
		ON_NM_DBLCLK(OnNmDblClk)
		ON_NM_RCLICK(OnNmRClick)
		ON_SBN_SIMPLEMODECHANGE(OnSbnSimpleModeChange)
		ON_TBN_BEGINADJUST(OnTbnBeginAdjust)
		ON_TBN_DELETINGBUTTON(OnTbnDeletingButton)
		ON_TBN_GETBUTTONINFO(OnTbnGetButtonInfo)
		ON_TBN_GETINFOTIP(OnTbnGetInfoTip)
		ON_TBN_INITCUSTOMIZE(OnTbnInitCustomize)
		ON_TBN_QUERYDELETE(OnTbnQueryDelete)
		ON_TBN_QUERYINSERT(OnTbnQueryInsert)
		ON_TBN_RESET(OnTbnReset)
		ON_TBN_TOOLBARCHANGE(OnTbnToolbarChange)
	END_NOTIFY_MAP()
	ON_WM_PAINT(OnPaint)
	ON_WM_SETCURSOR(OnSetCursor)
	ON_WM_SETFOCUS(OnSetFocus)
	ON_WM_SIZE(OnSize)
	ON_WM_SYSCOLORCHANGE(OnSysColorChange)
	ON_WM_SYSCOMMAND(OnSysCommand)
	ON_WM_TIMER(OnTimer)
	ON_WM_USER(WM_TASKBAR_ICON, OnTaskbarIcon)
	ON_WM_APP(WM_RECEIVE_POI, OnReceivePoi)
	ON_WM_APP(WM_DRAG_DROP, OnDragDrop)
	ON_WM_APP(WM_GPS_CHANGE, OnGpsChange)
END_MESSAGE_MAP()
